package com.reyun.aop;

import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.util.Constant;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
/**
 * 通过AOP的方式，实现某些api的国际化
 * @author zhangsy
 */

@Aspect
public class LocaleAop {
	protected Logger logger = LoggerFactory.getLogger(LocaleAop.class);
	
	@Pointcut("execution(* com.reyun.controller..*.*(..))")
    private void controllerAspect(){}
	
	@Around("controllerAspect()")
	public Object around(ProceedingJoinPoint pjp) throws Throwable {
		Object object = pjp.proceed();
		if(object instanceof ResultModel) {
			RequestAttributes ra = RequestContextHolder.getRequestAttributes();
			ServletRequestAttributes sra = (ServletRequestAttributes) ra;
			HttpServletRequest request = sra.getRequest();
			String localeStr = request.getHeader(Constant.LOCALE_KEY);
			String uri = request.getRequestURI();
			
			LanguageTypeEnum locale = LanguageTypeEnum.getVal(localeStr);
			
			if(logger.isDebugEnabled()) {
				logger.debug("convert uri=" + uri);
			}
			if(locale == LanguageTypeEnum.ENGLISH) {
				if(uri.startsWith("/api/login/loginweb")) {
					Object content = ((ResultModel)object).getContent();
					if(content instanceof Map && !((Map) content).get("code").toString().equals("200")) {
						//do some thing
					}
				}else if(uri.startsWith("/api/report")) {//替换报表返回数据中，包含的汇总，自然量字样
					Object content = ((ResultModel)object).getContent();
					if(content != null) {
						long start = System.currentTimeMillis();
						String conStr = JSON.toJSONString(content);
						
						/*=======替换掉val中的中文============*/
						conStr = conStr.replaceAll("(:\\s*\")(汇总)\"","$1total\"").replaceAll("(:\\s*?\")(自然量)\"","$1organic\"");
						conStr = conStr.replaceAll("(:\\s*\")(今天)\"","$1today\"").replaceAll("(:\\s*?\")(昨天)\"","$1yesterday\"");
						
						/*=======替换掉name中的中文===========*/
						try {
							JSONObject conObj = JSON.parseObject(conStr);
							
							JSONArray nameList = conObj.getJSONArray("name");
							if(nameList != null) {
								JSONArray newNameList = new JSONArray();
								Iterator<Object> it = nameList.iterator();
								while(it.hasNext()) {
									Object next = it.next();
									String item = next == null ? "" : next.toString();
									if(item.endsWith("数")) {//处理掉自定义事件名称最后的'数'
										newNameList.add(item.replaceAll("数($)","$1"));
									}else {//处理掉指标中的代码硬编码进去的中文
										item = item.replaceAll("^子账号$","sub-account");
										item = item.replaceAll("^推广活动组$","campaign-group");
										newNameList.add(item);
									}
								}
								conObj.put("name",newNameList);
							}
							
							((ResultModel)object).setContent(conObj);
						}catch(ClassCastException e) {
							//如果content不是json，这里就暂时不处理
							logger.info("not json::convert uri=" + uri);
						}
						
						
						long end = System.currentTimeMillis();
						if(logger.isDebugEnabled()) {
							logger.debug("report convert cost" + (end -start) + " ms");
						}
					}
				}
				//对标准的错误提示，通过aop的方式，转化为英文描述
				if(((ResultModel)object).getCode() != ResultStatus.SUCCESS.getCode()){
					((ResultModel)object).convertEn();
				}
			}
		}
		return object;
	}
}
