package com.reyun.controller;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.Account;
import com.reyun.model.Campaign;
import com.reyun.model.Campaign4Web;
import com.reyun.model.Channel;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.CampaignRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.repository.PackageSubCampaignRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AuthService;
import com.reyun.service.CampaignService;
import com.reyun.service.IViewColumnService;
import com.reyun.util.Constant;
import com.reyun.util.ResultModel;
import com.reyun.util.StringUtil;
import net.sf.json.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * Created by nolan on 26/12/2016.
 * description:
 */
@Controller
@RequestMapping("/{appid}/campaign")
public class CampaignController {

    Logger logger = LoggerFactory.getLogger(CampaignController.class);

    @Autowired
    private CampaignService campaignService;
    @Autowired
    private IViewColumnService iViewColumnService;
    @Autowired
    private AuthService authService;
    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private PackageSubCampaignRepository packageSubCampaignRepository;
    @Autowired
    private ChannelRepository channelRepository;

    @RequestMapping(value = "find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel find(@PathVariable Long appid, @CurrentAccount Account loginAccount) {
            return ResultModel.OK(authService.getAuthCampaignListByAccount(loginAccount.getId(), appid));
        }


    @RequestMapping(value = "findall", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAll(@PathVariable Long appid, @CurrentAccount Account loginAccount) {

        Iterable<Campaign4Web> result = campaignService.listAllAndDisable(appid, loginAccount, null);

        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
        if (!loginAccount.getIsSuperUser() && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
            result = Iterables.filter(result, new Predicate<Campaign4Web>() {
                @Override
                public boolean apply(Campaign4Web campaign4Web) {
                    return authCampaignList.contains(campaign4Web.getId());
                }
            });
        }

        return ResultModel.OK(Lists.newArrayList(result));
    }


//    public void getNotSuperUserData(Iterable<Campaign4Web> result,Account loginAccount,Long appid){
//        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
//        if (!loginAccount.getIsSuperUser() && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
//                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
//
//            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
//            result = Iterables.filter(result, new Predicate<Campaign4Web>() {
//                @Override
//                public boolean apply(Campaign4Web campaign4Web) {
//                    return authCampaignList.contains(campaign4Web.getId());
//                }
//            });
//        }
//    }

    @RequestMapping(value = "dropdown", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel dropDown(@PathVariable Long appid, @CurrentAccount Account loginAccount,String name) {

        Iterable<Campaign> result;
        Map jsond = new HashMap();

//        if(StringUtils.isEmpty(name)){
//            result = campaignService.listAllAndDisable(appid, loginAccount, null);
//            List<Campaign4Web> campaign4Webs  =Lists.newArrayList(result);
//            jsond.put("total", campaign4Webs.size());
//        }else{
//            result = campaignService.listAllAndDisableLimit(appid, loginAccount,name,jsond, null);
//        }




        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
//        if (!loginAccount.getIsSuperUser()
//                && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
//                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
//
//            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
//            result = Iterables.filter(result, new Predicate<Campaign4Web>() {
//                @Override
//                public boolean apply(Campaign4Web campaign4Web) {
//                    return authCampaignList.contains(campaign4Web.getId());
//                }
//            });
//        }

        long st = System.currentTimeMillis();
        if (!loginAccount.getIsSuperUser()
                && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            result = authService.getAuthCampaignListLimit(loginAccount,name, appid,jsond,500);

            long timeAuth = System.currentTimeMillis()-st;

            if(timeAuth>1000){
                logger.info("获取权限用户[{}]推广活动下拉数耗时[{}]",loginAccount.getEmail(),timeAuth);
            }

        }else{
            result = campaignService.listAllAndDisableLimit(appid, loginAccount,name,jsond, null);

            long time = System.currentTimeMillis()-st;

            if(time >1000){
                logger.info("获取主账号管理员[{}]推广活动下拉数耗时[{}]",loginAccount.getEmail(),time);
            }
        }


       Iterator<Campaign> iterable =  result.iterator();
        List datas = new ArrayList();
        while (iterable.hasNext()){
            Campaign  web = iterable.next();

            if(web!=null){
                Map mapval = new HashMap();
                mapval.put("id", web.getId());
                mapval.put("campaignid", web.getCampaignid());
                mapval.put("name", web.getName());
                mapval.put("subscribes", web.getSubscribes());
                datas.add(mapval);
            }

        }
        jsond.put("data", datas);
        return ResultModel.OK(jsond);

//
//        List<Campaign4Web> campaign4Webs = Lists.newArrayList(result);
//
//        int arrLenth = campaign4Webs.size() >= 500 ? 500 : campaign4Webs.size();
//
//        for (int i = 0; i < arrLenth; i++) {
//            Campaign4Web web = campaign4Webs.get(i);
//            Map mapval = new HashMap();
//            mapval.put("id", web.getId());
//            mapval.put("campaignid", web.getCampaignid());
//            mapval.put("name", web.getName());
//            datas.add(mapval);
//        }
//
//        jsond.put("data", datas);
//
//        return ResultModel.OK(jsond);

    }
    @RequestMapping(value = "remarketing/dropdown", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel remarketingDropDown(@PathVariable Long appid, @CurrentAccount Account loginAccount,String name) {
        Iterable<Campaign> result;
        Map jsond = new HashMap();
        long st = System.currentTimeMillis();
        if (!loginAccount.getIsSuperUser()
                && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            result = authService.getAuthCampaignListLimit(loginAccount,name, appid,jsond,500);

            long timeAuth = System.currentTimeMillis()-st;

            if(timeAuth>1000){
                logger.info("获取权限用户[{}]推广活动下拉数耗时[{}]",loginAccount.getEmail(),timeAuth);
            }

        }else{
            result = campaignService.listAllAndDisableLimit(appid, loginAccount,name,jsond, null);

            long time = System.currentTimeMillis()-st;

            if(time >1000){
                logger.info("获取主账号管理员[{}]推广活动下拉数耗时[{}]",loginAccount.getEmail(),time);
            }
        }
        Iterator<Campaign> iterable =  result.iterator();
        List datas = new ArrayList();
        while (iterable.hasNext()){
            Campaign  web = iterable.next();

            if(web!=null){
                if(web.getUrl().contains(Constant.deeplinkHost) || web.getDeeplink()) {
                    Map mapval = new HashMap();
                    mapval.put("id", web.getId());
                    mapval.put("campaignid", web.getCampaignid());
                    mapval.put("name", web.getName());
                    datas.add(mapval);
                }
            }
        }
        jsond.put("data",datas );
        jsond.put("total",datas.size());
        return ResultModel.OK(jsond);
    }


    @RequestMapping(value = "findbychannel", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findByChannel(@PathVariable Long appid, @RequestParam Long channelid, @CurrentAccount Account loginAccount) {

        List<Campaign> result = campaignService.listCampaignByAppAndChannel(appid, channelid);

        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
        if (!loginAccount.getIsSuperUser() && loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
            result = Lists.newArrayList(Iterables.filter(result, new Predicate<Campaign>() {
                @Override
                public boolean apply(Campaign campaign4Web) {
                    return authCampaignList.contains(campaign4Web.getId());
                }
            }));
        }

        return ResultModel.OK(result);
    }

    @RequestMapping(value = "findbychannel/all", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findByChannelAll(@PathVariable Long appid, @RequestParam Long channelid, @CurrentAccount Account loginAccount) {

//        List<Campaign> result = campaignService.listCampaignByAppAndChannel(appid, channelid);
//
//        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
//        if (!loginAccount.getIsSuperUser() && loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
//                && loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
//
//            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
//            result = Lists.newArrayList(Iterables.filter(result, new Predicate<Campaign>() {
//                @Override
//                public boolean apply(Campaign campaign4Web) {
//                    return authCampaignList.contains(campaign4Web.getId());
//                }
//            }));
//        }

        List<Campaign> result = campaignService.listCampaignByAppAndChannelAll(appid, channelid,loginAccount);
        Channel one = channelRepository.findOne(channelid);
        if("APPMARKET".equals(one.getCategory())){
            for (Campaign campaign : result) {
                campaign.setPkgSubCampaignList(packageSubCampaignRepository.listSubCampaign(campaign.getId()));
            }
        }
        return ResultModel.OK(result);
    }


    /**
     * @param account
     * @param appid
     * @param menu
     * @param lanType(语言类型：zh(中文)/en（英文）)
     * @return 获取对应的编辑指标
     */
    @RequestMapping(value = "/viewcolumn/findall/{menu}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findViewColumns(@CurrentAccount Account account, @PathVariable Long appid,
                                       @PathVariable String menu, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
        return ResultModel.OK(this.iViewColumnService.findUserViewColumns(account, appid, menu, lanType));
    }

    @RequestMapping(value = "/viewcolumn/{menu}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel saveViewColumns(@RequestParam String viewcolumn, @CurrentAccount Account account, @PathVariable Long appid, @PathVariable String menu) {
        this.iViewColumnService.saveUserViewColumn(account.getId(), appid, viewcolumn, menu);
        return ResultModel.OK();
    }

    @RequestMapping(value = "findallthird/{channel}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAllThirdAccount(@PathVariable Long appid, @PathVariable Long channel,  @CurrentAccount Account loginAccount) {
        List<Object> result = new ArrayList<>();
        if (appid.equals(new Long(0))) {
            Long rootParent = loginAccount.getRootParent();
            //id=rootParent or parent=rootParent
            List<Long> accounts = accountRepository.findAllAccountList(rootParent);
            result = Lists.newArrayList(campaignRepository.findThirdAccountByAccountChannel(accounts, channel));
        } else {
            result = Lists.newArrayList(campaignRepository.findThirdAccountByAppChannel(appid, channel));
        }

        List<Map<String, String>> mapList = new ArrayList<>();
        List<String> keys = new ArrayList<>();
        for (Object ob : result) {
            JSONArray jsonA = JSONArray.fromObject(ob);
            String key = jsonA.getString(0);
            String value = jsonA.getString(1);
            if (StringUtil.isEmpty(value) || value=="null") {
                value = key;
            }
            if (!keys.contains(key) || !key.equals(value))
            {
                Map<String, String> resultMap = new HashMap<>();
                resultMap.put("id", key);
                resultMap.put("name", value);
                mapList.add(resultMap);
                keys.add(key);
            }
        }

        return ResultModel.OK(mapList);
    }


    @RequestMapping(value = "findthirdback/{channel}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findThirdBackAccount(@PathVariable Long appid, @PathVariable Long channel, @CurrentAccount Account loginAccount) {

        List<Object[]> result = new ArrayList<>();
        List<Object[]> thirdApps = new ArrayList<>();
        if (appid.equals(new Long(0))) {
            Long rootParent = loginAccount.getRootParent();
            List<Long> accounts = accountRepository.findAllAccountList(rootParent);
            result = campaignRepository.findThirdBackAccountByAccountChannel(accounts, channel);
            thirdApps = campaignRepository.findThirdBackAccountApp(accounts, channel);
        } else {
            result = campaignRepository.findThirdBackAccountByAppChannel(appid, channel);
        }

        Map resultMap = new HashMap();
        resultMap.put("accounts", getRList(result,1, "id", "name", "appcount"));
        resultMap.put("apps", getRList(thirdApps,0, "id", "name"));

        return ResultModel.OK(resultMap);
    }

    @RequestMapping(value = "thirdback/applist/{thirdAccount}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findThirdBackAccount(@PathVariable Long appid,@PathVariable String thirdAccount) {

        List<Object[]> apps = campaignRepository.findAppByThirdAccountBack(thirdAccount);
        return ResultModel.OK(getRList(apps,0, "id", "name"));

    }

    private List<Map> getRList(List<Object[]> dataList,int index, String... names) {
        List<Map> dataResult = new ArrayList<>();
        for (Object[] data : dataList) {
            Map dataLine = new HashMap();
            for (int i = 0, a = names.length; i < a; i++) {

                if (index > 0 && index == i) {
                    dataLine.put(names[i], data[index] == null ? data[index-1] : data[index]);
                } else {
                    dataLine.put(names[i], data[i]);
                }
            }

            dataResult.add(dataLine);
        }
        return dataResult;
    }







//
//=======
//package com.reyun.controller;
//
//import com.google.common.base.Predicate;
//import com.google.common.collect.Iterables;
//import com.google.common.collect.Lists;
//import com.reyun.dic.RoleEnumType;
//import com.reyun.model.Account;
//import com.reyun.model.Campaign;
//import com.reyun.model.Campaign4Web;
//import com.reyun.repository.AccountRepository;
//import com.reyun.repository.CampaignRepository;
//import com.reyun.security.annotation.CurrentAccount;
//import com.reyun.service.AuthService;
//import com.reyun.service.CampaignService;
//import com.reyun.service.IViewColumnService;
//import com.reyun.util.ResultModel;
//import com.reyun.util.StringUtil;
//import net.sf.json.JSONArray;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.stereotype.Controller;
//import org.springframework.web.bind.annotation.*;
//
//import java.util.ArrayList;
//import java.util.HashMap;
//import java.util.List;
//import java.util.Map;
//
///**
// * Created by nolan on 26/12/2016.
// * description:
// */
//@Controller
//@RequestMapping("/{appid}/campaign")
//public class CampaignController {
//    @Autowired
//    private CampaignService campaignService;
//    @Autowired
//    private IViewColumnService iViewColumnService;
//    @Autowired
//    private AuthService authService;
//    @Autowired
//    private CampaignRepository campaignRepository;
//
//    @Autowired
//    private AccountRepository accountRepository;
//
//    @RequestMapping(value = "find", method = RequestMethod.GET)
//    @ResponseBody
//    public ResultModel find(@PathVariable Long appid, @CurrentAccount Account loginAccount) {
//            return ResultModel.OK(authService.getAuthCampaignListByAccount(loginAccount.getId(), appid));
//        }
//
//
//    @RequestMapping(value = "findall", method = RequestMethod.GET)
//    @ResponseBody
//    public ResultModel findAll(@PathVariable Long appid, @CurrentAccount Account loginAccount) {
//
//        Iterable<Campaign4Web> result = campaignService.listAllAndDisable(appid, loginAccount, null);
//
//        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
//        if (!loginAccount.getIsSuperUser() && !loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
//                && !loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
//
//            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
//            result = Iterables.filter(result, new Predicate<Campaign4Web>() {
//                @Override
//                public boolean apply(Campaign4Web campaign4Web) {
//                    return authCampaignList.contains(campaign4Web.getId());
//                }
//            });
//        }
//
//        return ResultModel.OK(Lists.newArrayList(result));
//    }
//
//
//    @RequestMapping(value = "findbychannel", method = RequestMethod.GET)
//    @ResponseBody
//    public ResultModel findByChannel(@PathVariable Long appid, @RequestParam Long channelid, @CurrentAccount Account loginAccount) {
//
//        List<Campaign> result = campaignService.listCampaignByAppAndChannel(appid, channelid);
//
//        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
//        if (!loginAccount.getIsSuperUser() && loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
//                && loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
//
//            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
//            result = Lists.newArrayList(Iterables.filter(result, new Predicate<Campaign>() {
//                @Override
//                public boolean apply(Campaign campaign4Web) {
//                    return authCampaignList.contains(campaign4Web.getId());
//                }
//            }));
//        }
//
//        return ResultModel.OK(result);
//    }
//
//    /**
//     * @param account
//     * @param appid
//     * @param menu
//     * @param lanType(语言类型：zh(中文)/en（英文）)
//     * @return 获取对应的编辑指标
//     */
//    @RequestMapping(value = "/viewcolumn/findall/{menu}", method = RequestMethod.GET)
//    @ResponseBody
//    public ResultModel findViewColumns(@CurrentAccount Account account, @PathVariable Long appid,
//                                       @PathVariable String menu, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
//        return ResultModel.OK(this.iViewColumnService.findUserViewColumns(account.getId(), appid, menu, lanType));
//    }
//
//    @RequestMapping(value = "/viewcolumn/{menu}", method = RequestMethod.POST)
//    @ResponseBody
//    public ResultModel saveViewColumns(@RequestParam String viewcolumn, @CurrentAccount Account account, @PathVariable Long appid, @PathVariable String menu) {
//        this.iViewColumnService.saveUserViewColumn(account.getId(), appid, viewcolumn, menu);
//        return ResultModel.OK();
//    }
//
//    @RequestMapping(value = "findallthird/{channel}", method = RequestMethod.GET)
//    @ResponseBody
//    public ResultModel findAllThirdAccount(@PathVariable Long appid, @PathVariable Long channel,  @CurrentAccount Account loginAccount) {
//        List<Object> result = new ArrayList<>();
//        if (appid.equals(new Long(0))) {
//            Long rootParent = loginAccount.getRootParent();
//            //id=rootParent or parent=rootParent
//            List<Long> accounts = accountRepository.findAllAccountList(rootParent);
//            result = Lists.newArrayList(campaignRepository.findThirdAccountByAccountChannel(accounts, channel));
//        } else {
//            result = Lists.newArrayList(campaignRepository.findThirdAccountByAppChannel(appid, channel));
//        }
//
//        List<Map<String, String>> mapList = new ArrayList<>();
//        List<String> keys = new ArrayList<>();
//        for (Object ob : result) {
//            JSONArray jsonA = JSONArray.fromObject(ob);
//            String key = jsonA.getString(0);
//            String value = jsonA.getString(1);
//            if (StringUtil.isEmpty(value) || value=="null") {
//                value = key;
//            }
//            if (!keys.contains(key) || !key.equals(value))
//            {
//                Map<String, String> resultMap = new HashMap<>();
//                resultMap.put("id", key);
//                resultMap.put("name", value);
//                mapList.add(resultMap);
//                keys.add(key);
//            }
//        }
//
//        return ResultModel.OK(mapList);
//    }
//>>>>>>> feature_v_tran
}