package com.reyun.controller;

import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.Crowd;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AppService;
import com.reyun.service.CrowdService;
import com.reyun.util.ResultModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("crowd")
public class CrowdController {

    @Autowired
    private CrowdService crowdService;

    @Autowired
    private AppService appService;

    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel create(@CurrentAccount Account account, @RequestBody Crowd source) {
        Crowd crowd = crowdService.create(account, source);
        return ResultModel.OK(crowd);
    }

    @RequestMapping(value = "findApps", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel create(@CurrentAccount Account account) {
        List<App> apps = appService.listAppByAccount(account.getId());
        ArrayList<App> objects = new ArrayList<>();
        for(App app:apps){
            if(!app.getDebug()){
                objects.add(app);
            }
        }
        return ResultModel.OK(objects);
    }

    @RequestMapping(value = "/findDetail/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findCrowdDetail(@PathVariable Long id) {
        return ResultModel.OK(crowdService.findDetail(id));
    }

    @RequestMapping(value = "/findOrders", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findCrowdDetail(@CurrentAccount Account account) {
        return ResultModel.OK(crowdService.findOrders(account.getRootParent()));
    }


    @RequestMapping(value = "download/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getDownloadResponseEntity(@PathVariable Long id) {
        ResponseEntity<byte[]> download = null;
        try {
            download = crowdService.download(id);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return download;
    }

    @RequestMapping(value = "/findBlackData", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findBlackData(@CurrentAccount Account account) {
        Map<String, Object> total = crowdService.findBlackData(account);
        return ResultModel.OK(total);
    }

    @RequestMapping(value = "/findArouseData", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findArouseData(@CurrentAccount Account account) {
        /*先判断content是否为null：未申请
            如果不为null：判断字段status为0：已申请
            如果status为：2  判断 pastdate 是否过期
            其余正常显示过期时间*/
        Map<String, Object> total = crowdService.findArouseData(account);
        return ResultModel.OK(total);
    }
}
