package com.reyun.controller;

import com.google.common.base.Strings;
import com.google.common.collect.Maps;
import com.reyun.model.Account;
import com.reyun.model.DemoAppointment;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.BussinessManRepository;
import com.reyun.repository.EditionPricingLevelRepository;
import com.reyun.security.RedisLoginStatusManager;
import com.reyun.security.TokenManager;
import com.reyun.service.*;
import com.reyun.util.*;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;

@Controller
@RequestMapping("demo")
public class DemoController {

	protected Logger logger = LoggerFactory.getLogger(DemoController.class);

	@Autowired
	private DemoAppointmentService demoAppointmentService;

	@Autowired
	LoginService loginService;

	@Autowired
	TokenManager tokenManager;

	@Autowired
	AccountRepository accountRepository;

	@Autowired
	UserLogService userLogService;

    @Autowired
    ConfigParamService configParamService;

    @Autowired
    RedisLoginStatusManager redisLoginStatusManager;
    
    @Autowired
    BussinessManRepository bussinessManRepository;
    
    @Autowired
    AuthService authService;
    
    @Autowired
    EditionPricingLevelRepository levelRepository;

    @Autowired
	MessageCodeService messageCodeService;

	@RequestMapping(value = "/{source}",method = RequestMethod.GET)
	@ResponseBody
	public ResultModel demologin(HttpServletRequest request,HttpServletResponse response,@PathVariable String source) {
		String email = "demo@reyun.com";
		String password = "111111";

		if("2".equals(source)){
			email = "demo@chinacfsc.com";
		}

		Account account = loginService.login(email, password ,request.getRemoteAddr());

		Map<String, Object> rtnMap = Maps.newHashMap();
		if (account != null) {

            //存储token
			String token = tokenManager.createToken(String.valueOf(account.getId()));
            //存储登陆密码
            redisLoginStatusManager.createLoginStatus(token, account.getIsMasterLogin() ? CipherUtil.generatePassword(configParamService.generateMasterPwd()) : account.getPassword());

			// 母账号禁用
			if (account.getIsSuperUser() && account.getStatus() == 0) {
				rtnMap.put("msg", "母账号禁用");
				rtnMap.put("code", 208);
				return ResultModel.OK(rtnMap);
			} else if (account.getIsSuperUser() && account.getStatus() == -1) {
				//还没通过审核
				rtnMap.put("msg", "还没通过审核");
				rtnMap.put("code", 210);
				return ResultModel.OK(rtnMap);
			}
			// 子账号禁用
			if (account.getStatus() == 0) {
				rtnMap.put("msg", "子账号禁用");
				rtnMap.put("code", 203);
				return ResultModel.OK(rtnMap);
			}
			// 子账号所在母账号被禁用
			if (!account.getIsSuperUser() && accountRepository.findOne(account.getParent()).getStatus() == 0) {
				rtnMap.put("msg", "子账号所在母账号被禁用");
				rtnMap.put("code", 209);
				return ResultModel.OK(rtnMap);
			}

			String nowdate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);

			if (!account.getIsSuperUser()) {
				Account parent = accountRepository.findOne(account.getParent());
				if (!StringUtil.isEmpty(parent.getPastDate())
						&& DateUtil.compare_date(nowdate, parent.getPastDate()) == 1) {
					rtnMap.put("msg", "子账号所在母账号过期");
					rtnMap.put("code", 207);
					return ResultModel.OK(rtnMap);
				}
			} else {
				if (!StringUtil.isEmpty(account.getPastDate()) && DateUtil.compare_date(nowdate, account.getPastDate()) == 1) {
					rtnMap.put("msg", "母账号过期");
					rtnMap.put("code", 206);
					return ResultModel.OK(rtnMap);
				}
			}

            // 没有控制权限
            if (!account.getIsSuperUser() || (account.getIsSuperUser() && account.getIsChannelPerson())) {
                if (!ValidateUtil.isValid(account.getAuthStr()) || !ValidateUtil.isValid(JSONObject.fromObject(account.getAuthStr()))) {
                    rtnMap.put("msg", "没有控制权限");
                    rtnMap.put("code", 202);
                    return ResultModel.OK(rtnMap);
                }
            }


            rtnMap.put("code", 200);
            rtnMap.put("token", token);
            rtnMap.put("account", account);
            return ResultModel.OK(rtnMap);

        } else {
            // 没有此账号
            rtnMap.put("code", 205);
            rtnMap.put("msg", "用户名或密码不正确");
            return ResultModel.OK(rtnMap);
        }

    }

    @RequestMapping(value = "/freetrial", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel freetrial(@RequestParam("code") String code, @RequestBody DemoAppointment demoAppointment, HttpServletRequest httpServletRequest) {
        checkArgument(!Strings.isNullOrEmpty(code), "验证码不允许为空");
        checkArgument(!Strings.isNullOrEmpty(demoAppointment.getName()), "姓名不允许为空");
        checkArgument(!Strings.isNullOrEmpty(demoAppointment.getPhone()), "手机号不允许为空");


        String vefifuResult = messageCodeService.varifyMessagecode(demoAppointment.getPhone(), code);

        if ("code error".equals(vefifuResult))
        {
            return ResultModel.ERROR(ResultStatus.MSG_SMS_WRONG);
        }
        else if ("code out of time".equals(vefifuResult))
        {
            return ResultModel.ERROR(ResultStatus.MSG_SMS_OUTOFTIME);
        }
        demoAppointment.setStatus(1);
        demoAppointment.setCreateTime(new Date());
        demoAppointment.setIpAddr(IPAddrUtil.getIpAddr(httpServletRequest));
        demoAppointment.setArea(IPAddrUtil.getAllLocationFromIpAddr(demoAppointment.getIpAddr()));

		demoAppointment = demoAppointmentService.freetrial(demoAppointment);

		final Long genId = demoAppointment.getId();
		final String htmlFreeTrialUrl = Constant.adminUrl + "/api/demo/freetrial/mail?id=" + genId + "&phe=" + demoAppointment.getPhone();
		logger.info("发送预约申请邮件至运营支持人员. id:{}. //Begin", genId);
		String result = HttpClientUtil.doHttpGetRequest(htmlFreeTrialUrl, "trackingio-appointment");
		logger.info("发送预约申请邮件至运营支持人员. id:{}, httpResult:{}. //End", genId, result);
		return ResultModel.OK();
	}

	@RequestMapping(value = "/freetrial/checkexist", method = RequestMethod.GET)
	@ResponseBody
	public ResultModel checkPhoneExist(@RequestParam("email") String email) {
//		return ResultModel.OK(demoAppointmentService.checkExist(phone));
		return ResultModel.OK(demoAppointmentService.checkExistByEmail(email));
	}


    /**
     * 发送 验证码到用户邮箱
     * @param email
     * @param request
     * @return
     */
    @RequestMapping(value = "/freetrial/code2email", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel code2Email(String email, HttpServletRequest request) {

        boolean reslut = demoAppointmentService.sendCode2Email(email, request);

        return ResultModel.OK(reslut?"success":"fail");
    }


    @RequestMapping(value = "/freetrial/code2email/ckeck", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel code2EmailCheck(String code, HttpServletRequest request) {

        return demoAppointmentService.code2EmailCheck(code, request);

    }



    @RequestMapping(value = "/freetrial/new", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel freetrialNew(@RequestParam("code") String code, @RequestBody DemoAppointment demoAppointment, HttpServletRequest httpServletRequest) {

        ResultModel model =   demoAppointmentService.code2EmailCheck(code, httpServletRequest);

        if(model!=null){
            return model;
        }


        if(demoAppointmentService.checkExistByEmail(demoAppointment.getEmail())){
            return ResultModel.ERROR(ResultStatus.DEMO_EMAIL_APPLY_EXIST);
        }
        
        demoAppointment.setStatus(1);
        demoAppointment.setCreateTime(new Date());
        demoAppointment.setIpAddr(IPAddrUtil.getIpAddr(httpServletRequest));
        demoAppointment.setArea(IPAddrUtil.getAllLocationFromIpAddr(demoAppointment.getIpAddr()));
        demoAppointment = demoAppointmentService.freetrial(demoAppointment);

        return ResultModel.OK(demoAppointment);
    }


    public static void main(String[] args) {
        StringBuffer codeBuffer = new StringBuffer();
        for (int i = 0; i < 6; i++) {
            codeBuffer.append(RandomUtils.nextInt(0, 9));
        }

        System.out.println(codeBuffer.toString());

    }


}
