package com.reyun.controller;

import com.google.common.collect.ImmutableList;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.App;
import com.reyun.model.ExternalDownload;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ExternalDownloadRepository;
import com.reyun.service.AuthService;
import com.reyun.service.UserLogService;
import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.StringUtil;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by sunhao on 17/4/6.
 * description:外部下载接口
 */
@Controller
@RequestMapping("external")
public class ExternalInterfaceController {
    private Logger logger = LoggerFactory.getLogger(ExternalInterfaceController.class);
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ExternalDownloadRepository externalDownloadRepository;
    @Autowired
    private AuthService authService;
    @Autowired
    UserLogService userLogService;

    private final static String DOWNLOAD_TYPE_INSTALL = "install";
    private final static String DOWNLOAD_TYPE_REGISTER = "reged";
    private final static String DOWNLOAD_TYPE_CLICK = "click";
    private final static String DOWNLOAD_TYPE_BLACKLIST = "blacklist";
    private final static String DOWNLOAD_TYPE_FAKE_INSTALL = "fake";
    private final static String DOWNLOAD_TYPE_PAYMENT = "pay";
    private final static String DOWNLOAD_TYPE_DEVICELOGIN = "loggedin";
    private final static String ORDER = "order";
    private final String file_template = "%s_%s_%s.tar.gz";
    private final String file_templateTemp = "%s_%s_%s.txt.gz";
    private final String file_template_en = "%s_%s_%s_%s.tar.gz";
    private final String file_template_old = "%s_%s_%s.txt";
    private final String file_template_old_en = "%s_%s_%s_%s.txt";

    /**
     * 下载前一天生成的激活数据文件
     */
    @RequestMapping(value = "download/inst", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppInstallFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_INSTALL);
    }

    /**
     * 下载前一天生成的注册数据文件
     */
    @RequestMapping(value = "download/reg", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppRegisterFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_REGISTER);
    }

    /**
     * 下载前一天生成的订单数据文件
     */
    @RequestMapping(value = "download/order", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppOrderFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, ORDER);
    }

    /**
     * 下载前一天生成的黑名单数据文件
     */
    @RequestMapping(value = "download/blacklist", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppBlacklistFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_BLACKLIST);
    }
    /**
     * 下载前一天生成的防作弊激活数据文件
     */
    @RequestMapping(value = "download/blacklistinstall", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppBlacklistInstallFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_FAKE_INSTALL);
    }
    /**
     * 下载前一天生成的付费数据文件
     */
    @RequestMapping(value = "download/pay", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppPaymentListFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_PAYMENT);
    }

    /**
     * 下载前一天生成的设备登录数据文件
     */
    @RequestMapping(value = "download/loggedin", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getDeviceLoginListFile(HttpServletRequest request, @RequestParam String token) {
        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, DOWNLOAD_TYPE_DEVICELOGIN);
    }



    /**
     * 下载数据
     */
    @RequestMapping(value = "export", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getAppFile(HttpServletRequest request, @RequestParam String downloadType, @RequestParam String token) {
        if ("click".equals(downloadType)) {
            throw new TipException("暂不支持点击文件下载.");
        }

        App app = appRepository.findOneByAppToken(token);
        checkAuth(app);
        return getFileResponse(request, app, downloadType);
    }

    private void checkAuth(App app) {
        if (app == null) {
            throw new TipException("应用不存在");
        }

        boolean isAccountPast = authService.isOriginAccountPast(app.getAccount());
        if (isAccountPast) {
            throw new TipException("母账号已过期 ：" + app.getAccount());
        }
    }

    /**
     * 获取文件流
     */
    private ResponseEntity<byte[]> getFileResponse(HttpServletRequest request, App app, String downloadType) {
        try {
            userLogService.insertLog(request, LogEnumType.DOWNLOAD.getCode(),
                    LogEnumType.DOWNLOAD.getName() + downloadType + "日志地址 ", app.getId(), OperateObjectEnumType.LOGADRESS.getCode());
        } catch (Exception e) {
            logger.error("fail to parse ip," + e.getMessage());
        }

        String downloadIP = request.getHeader("x-forwarded-for") == null ? request.getRemoteAddr() : request.getHeader("x-forwarded-for");

        //日期yyyyMMdd和文件名
        final String yesterday = new DateTime(DateUtil.getBeforeDays(1)).toString("yyyy-MM-dd");

        String langType = request.getHeader("lang");

        if(StringUtils.isEmpty(langType)){
            //获取cookie中 zh/en
            Map<String, Cookie> cookieMap = new HashMap<String, Cookie>();
            Cookie[] cookies = request.getCookies();
            if (null != cookies) {
                for (Cookie cookie : cookies) {
                    cookieMap.put(cookie.getName(), cookie);
                }
            }

            Cookie languageType = cookieMap.get("lang");

            if(languageType!=null){
                langType = languageType.getValue();
            }

        }

        String fileName = null;
        String fileName_old = null;
        String fileName_temp = null;



        //检验常规 下载记录
        fileName = String.format(file_template, downloadType, app.getAppkey(), yesterday);
        fileName_old = String.format(file_template_old, downloadType, app.getAppkey(), yesterday);
        fileName_temp = String.format(file_templateTemp, downloadType, app.getAppkey(), yesterday);
        //查询下载记录
        ExternalDownload externalDownload = externalDownloadRepository.findDownloadInfoByFileName(app.getAppkey(), ImmutableList.of(fileName, fileName_old));

        // 如果为英文 则切换到英文文件名 下载相应文件
        if(langType!=null && StringUtils.equals(LanguageTypeEnum.ENGLISH.getCode(),langType)){
            fileName = String.format(file_template_en, downloadType, app.getAppkey(), yesterday,LanguageTypeEnum.ENGLISH.getCode());
            fileName_old = String.format(file_template_old_en, downloadType, app.getAppkey(), yesterday,LanguageTypeEnum.ENGLISH.getCode());
        }
        if (null != externalDownload) {
            InputStream in = null;
            byte[] content = new byte[0];
            String fileType = "";
            try {
                try {
                    in = ObjectStorageUtil.getObjectContentStream(Constant.bucket,
                            "trackingio/" + downloadType + "/" + fileName);
                } catch (Exception e) {
                    try {
                        in = ObjectStorageUtil.getObjectContentStream(Constant.bucket,
                                "trackingio/" + downloadType + "/" + fileName_old);
                        fileName = fileName_old;
                        fileType = "txt";
                    } catch (Exception e1) {
                        in = ObjectStorageUtil.getObjectContentStream(Constant.bucket,
                                "trackingio/" + downloadType + "/" + fileName_temp);
                        fileName = fileName_temp;
                    }
                }
                fileType = URLConnection.guessContentTypeFromName(fileName);
                content = IOUtils.toByteArray(in);
            } catch (IOException e) {
                throw new TipException("文件不存在");
            } finally {
                IOUtils.closeQuietly(in);
            }

            HttpHeaders headers = new HttpHeaders();
            headers.set("filename", fileName);
            headers.add("content-disposition", "attachment; filename=" + fileName);
            headers.add("Content-Type", fileType);
            //更新下载时间
            externalDownloadRepository.updateDownloadInfo(downloadIP, new Date(), externalDownload.getId());

            //返回文件字符串
            return new ResponseEntity<byte[]>(content, headers, HttpStatus.CREATED);
        } else {
            throw new TipException("文件不存在");
        }
    }


    public static void main(String[] args) {

         final String file_template = "%s_%s_%s.tar.gz";

       String  fileName = String.format(file_template, "reged", "debea1d81f49ded066581eb01b3c79aa", "2019-04-21");

        System.out.println(fileName);
    }
}
