package com.reyun.controller;

import com.alibaba.fastjson.JSON;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.repository.CampaignRepository;
import com.reyun.repository.TranslationRepository;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.impl.ConfigParamServiceImpl;
import com.reyun.util.*;
import org.joda.time.DateTime;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import sun.java2d.pipe.AAShapePipe;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;

@Controller
@RequestMapping(value = "/logtool/{appid}")
public class LogController {
    protected static Logger logger = LoggerFactory.getLogger(LogController.class);

    private static final Map<String, String> mockLogContent = new HashMap<>();

    private static final String[] LogWaht = {"order", "startup", "install"};

    @Autowired
    AppRepository apprepository;

    @Autowired
    AccountRepository accountrepository;

    @Autowired
    TranslationRepository translationRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    ConfigParamServiceImpl configParamService;

    @RequestMapping(method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("logtool")
    public List getlogs(HttpServletRequest request, @PathVariable Long appid, @CurrentAccount Account loginAccount) {
        App app = apprepository.findOne(appid);
        Account parent = accountrepository.findOne(app.getAccount());
        Account account = loginAccount;
        List result = null;

        List mockresult = this.getDemoMockLog(account, app);

        if (mockresult != null) {
            return mockresult;
        }

        try {
            result = getLogs(app.getAppkey());
        } catch (IOException e) {
            e.printStackTrace();
        }

        if (result == null) {
            result = new ArrayList();
        }
        return result;
    }

    public List getLogs(String appId) throws IOException {
        List<JSONObject> obList = new ArrayList<JSONObject>();

        String[] hostArray = Constant.receiverHost.split(",");
        for (String host : hostArray) {
            String url = "http://" + host.trim() + ":" + Constant.receiverPort + "/receive/newlylogs/" + appId;
            try {
                String httpresult = HttpClientUtil.doHttpGetRequest(url, host);
                if (!StringUtil.isEmpty(httpresult)) {
                    try {
                        JSONObject httpresultObject = new JSONObject(httpresult);
                        if (httpresultObject.has("result")) {
                            JSONArray resultArray = httpresultObject.getJSONArray("result");

                            for (int i = 0; i < resultArray.length(); i++) {
                                JSONObject ob = resultArray.getJSONObject(i);
                                obList.add(ob);
                            }
                        }
                    } catch (JSONException e) {

                        e.printStackTrace();
                    }
                }
            } catch (Exception e) {
                logger.error("url: {}", url, e);
                try {
                    //MailUtils.sendSimpleEmail("日志流服务", "日志流服务暂不可用, url: " + url + ", message:" + e.getMessage(), Constant.mlist);
                    SendCommonPostMail.sendMailReuse(Constant.mlist, "日志流服务", "日志流服务暂不可用, url: " + url + ", message:" + e.getMessage(), null, null);
                } catch (Exception e1) {
                    logger.error("邮件服务不可用 getLogs()");
                }
                continue;
            }
        }

        Collections.sort(obList, new Comparator<JSONObject>() {

            public int compare(JSONObject arg0, JSONObject arg1) {

                try {
                    return (arg1.getString("when")).compareTo((String) arg0.getString("when"));
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                return 0;
            }

        });
        List<String> resultList = new ArrayList<String>();
		/*
		if (obList.size() >= 40) {
			for (int i=0; i<40; i++) {
				resultList.add(obList.get(i).toString());
			}
		} else {
			for (int i=0; i<obList.size(); i++) {
				resultList.add(obList.get(i).toString());
			}
		}*/
        for (JSONObject ob : obList) {
            if (ob != null) {
            	String obj_str = ob.toString();
                if(obj_str.length() > 3000){
					resultList.add(obj_str.substring(0,1500));
				}else{
					resultList.add(obj_str);
				}
            }
           // resultList.add(ob.toString());
        }
        return resultList;
    }


    private List getDemoMockLog(Account account, App app) {

        String demoApps = configParamService.getConfigParamByKey("demo_appkey");

        if (demoApps == null) {
            return null;
        }
        List<String> demoAppList = Arrays.asList(demoApps.split(","));

        if (demoAppList != null && demoAppList.contains(app.getAppkey())) {

            if (mockLogContent.size() == 0) {
                mockLogContent.put("order", "{\"_currencyamount\":\"45\",\"_ry_origin_when\":\"2019-06-26 17:23:36\",\"_deviceid\":\"746559290881\",\"_currencytype\":\"CNY\",\"_ry_ts\":\"1561540889760\",\"_tz\":\"+8\",\"_ip\":\"215.82.236.98\",\"_paymenttype\":\"alipay\",\"_imei\":\"746559290881\",\"_timestamp\":\"1561541016099\",\"_androidid\":\"746559290881\",\"_transactionid\":\"0561997559249649681\"}");
                mockLogContent.put("startup", "{\"_ry_origin_when\":\"2019-06-26 17:23:15\",\"_deviceid\":\"718725999225\",\"_ry_ts\":\"1561540869491\",\"_tz\":\"+8\",\"_ip\":\"10.0.55.32\",\"_imei\":\"718725999225\",\"_timestamp\":\"1561540995855\",\"_androidid\":\"718725999225\"}");
                mockLogContent.put("install", "{\"_op\":\"中国移动\",\"_cmpaignid\":\"default\",\"_ry_ts\":\"1561540840192\",\"_rydevicetype\":\"vivox9\",\"_tz\":\"+8\",\"_ip\":\"3.209.205.159\",\"_imei\":\"941890820248\",\"_manufacturer\":\"vivo\",\"_ry_origin_when\":\"2019-06-26 17:22:46\",\"_deviceid\":\"941890820248\",\"_resolution\":\"1344*750\",\"_ryos\":\"android\",\"_timestamp\":\"1561540966506\",\"_androidid\":\"941890820248\",\"_network\":\"wifi\"}");
            }

            List<String> result = new ArrayList<>();

            for (int i = 0; i < 20; i++) {
                Map mockMap = new HashMap();

                String what = LogWaht[RandomUtils.nextInt(0, 3)];

                mockMap.put("what", what);
                mockMap.put("appType", "");
                mockMap.put("appid", app.getAppkey());
                mockMap.put("context", JSON.parseObject(mockLogContent.get(what)));
                mockMap.put("where", "event");
                mockMap.put("when", new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
                mockMap.put("ds", new DateTime().toString("yyyy-MM-dd"));
                mockMap.put("who", "lxxcvs");

                result.add(JSON.toJSONString(mockMap));

            }
            return result;
        }

        return null;
    }


}
