package com.reyun.controller;


import com.google.common.collect.Maps;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.security.RedisLoginStatusManager;
import com.reyun.security.RedisLoginStatusManagerImpl;
import com.reyun.security.TokenManager;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.security.interceptor.AuthorizationInterceptor;
import com.reyun.service.*;
import com.reyun.taskexecute.LoginThread;
import com.reyun.util.*;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Controller
@RequestMapping("login")
public class LoginController {

    protected Logger logger = LoggerFactory.getLogger(LoginController.class);

    @Autowired
    LoginService loginService;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    UserLogService userLogService;

    @Autowired
    ConfigParamService configParamService;

    @Autowired
    RedisLoginStatusManager redisLoginStatusManager;

    @Autowired
    BussinessManRepository bussinessManRepository;

    @Autowired
    AuthService authService;

    @Autowired
    EditionPricingLevelRepository levelRepository;

    @Autowired
    AbTestRepository abTestRepository;

    @Autowired
    IsNotifiedRepository isNotifiedRepository;

    @Autowired
    PackageTypeRepository packageTypeRepository;

    @Autowired
    AccountFlowRestrictService accountFlowRestrictService;

    @Autowired
    BlacklistService blacklistService;

    @Autowired
    AccountService accountService;

    @Autowired
    AutoTodayFlowService autoTodayFlowService;

    @RequestMapping(value = "login", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel loginH5(HttpServletRequest request, HttpServletResponse response, @RequestParam String email,
                               @RequestParam String password, @RequestParam Boolean isrember, @RequestParam Boolean autologin) {
        Map<String, Object> rtnMap = Maps.newHashMap();

        Account account = loginService.login(email, password, request.getRemoteAddr());
        if (account != null) {

            //存储token
            String token = tokenManager.createToken(String.valueOf(account.getId()));
            //存储登陆密码
            redisLoginStatusManager.createLoginStatus(token, account.getIsMasterLogin() ? CipherUtil.generatePassword(configParamService.generateMasterPwd()) : account.getPassword());

            String nowDate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);

            if (account.getIsSuperUser()) {

                //母账号
                if (account.getStatus() == 0) {
                    //主账号禁用
                    rtnMap.put("msg", "主账号禁用");
                    rtnMap.put("code", 208);
                    return ResultModel.OK(rtnMap);

                } else if (account.getStatus() == -1) {
                    //主账号还没通过审核
                    rtnMap.put("msg", "还没通过审核");
                    rtnMap.put("code", 210);
                    return ResultModel.OK(rtnMap);
                }

                //主账号过期
                if (!StringUtil.isEmpty(account.getPastDate()) && DateUtil.compare_date(nowDate, account.getPastDate()) == 1) {
                    rtnMap.put("msg", "主账号过期");
                    rtnMap.put("code", 206);
                    return ResultModel.OK(rtnMap);
                }

            } else {

                //子账号
                if (account.getStatus() == 0) {
                    // 子账号禁用
                    rtnMap.put("msg", "子账号禁用");
                    rtnMap.put("code", 203);
                    return ResultModel.OK(rtnMap);

                } else {

                    //获取根节点母账号
                    Account parent = authService.findRootParentAccount(account.getId());

                    if (parent.getStatus() == 0) {
                        //子账号所在主账号被禁用
                        rtnMap.put("msg", "子账号所在主账号被禁用");
                        rtnMap.put("code", 209);
                        return ResultModel.OK(rtnMap);

                    } else if (!StringUtil.isEmpty(parent.getPastDate()) && DateUtil.compare_date(nowDate, parent.getPastDate()) == 1) {
                        //子账号所在主账号过期
                        rtnMap.put("msg", "子账号所在主账号过期");
                        rtnMap.put("code", 207);
                        return ResultModel.OK(rtnMap);
                    }
                }

                //没有控制权限AuthStr没有授权
                if (!account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {

                    if (!ValidateUtil.isValid(account.getAuthStr()) || !ValidateUtil.isValid(JSONObject.fromObject(account.getAuthStr()))) {

                        rtnMap.put("msg", "没有产品权限");
                        rtnMap.put("code", 202);
                        return ResultModel.OK(rtnMap);

                    }
                }
            }

            String domain = accountService.getCookieData(request.getCookies(), "sys_domain");

            if (StringUtils.isEmpty(domain)) {
                domain = Constant.getDomainStr();
            }

            if (isrember) {
                Cookie cookie = new Cookie("ryioUpass", password);
                cookie.setDomain(domain);
                cookie.setPath("/");
                cookie.setMaxAge(60 * 60 * 24 * 7);
                response.addCookie(cookie);

            } else {

                Cookie cookie = new Cookie("ryioUpass", "");
                cookie.setDomain(domain);
                cookie.setPath("/");
                cookie.setMaxAge(0);
                response.addCookie(cookie);
            }

            //自动登录
            Cookie cookie = new Cookie("TOKEN", token);
//            cookie.setDomain("trackingio.com");
            cookie.setDomain(domain);
            cookie.setPath("/");
            cookie.setMaxAge(autologin ? 60 * 60 * 24 * 7 : 60 * 30);
            response.addCookie(cookie);

            rtnMap.put("code", 200);
            rtnMap.put("token", token);
            rtnMap.put("account", account);

            try {
                userLogService.insertLog(account, LogEnumType.LOGIN.getCode(), LogEnumType.LOGIN.getName() + account.getEmail(), account, null, null, request);
            } catch (Exception e) {
                logger.error("fail to parse ip," + e.getMessage());
            }
            return ResultModel.OK(rtnMap);

        } else {
            // 没有此账号
            rtnMap.put("code", 205);
            rtnMap.put("msg", "用户名或密码不正确");
            return ResultModel.OK(rtnMap);
        }

    }

    @RequestMapping(value = "loginweb", method = RequestMethod.POST)
    @ResponseBody
//    public ResultModel login(HttpServletRequest request, HttpServletResponse response, @RequestParam String email,
//                             @RequestParam String password, @RequestParam Boolean isrember, @RequestParam Boolean autologin,
//                             String cip) {
    public ResultModel login(@RequestBody LoginDomain domain, HttpServletRequest request, HttpServletResponse response) {

        Map<String, Object> rtnMap = Maps.newHashMap();
        Account account = loginService.login(domain.getEmail(), domain.getPassword(), domain.getCip());

        if (account != null) {
            if (account.getLoginStatus() == 1) {
                /**
                 * 判断用户行为分析和广告检测产品是否到期
                 */
                Long b = account.getRoleCategory();

                if (!account.getRoleCategory().equals(5L)) {
                    Boolean trackExhaust = accountFlowRestrictService.isTrackExhaust(account.getId());
                    Long rootParent = accountRepository.findRootParentByAccountId(account.getId()).longValue();
                    Account rootAccount = accountRepository.findOne(rootParent);
                    if (trackExhaust) {
                        rtnMap.put("msg", "广告效果监控流量已经用尽！");
                        rtnMap.put("code", 211);
                        return ResultModel.OK(rtnMap);
                    }
                }

                String nowDate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);

                if (account.getIsSuperUser()) {

                    //母账号
                    if (account.getStatus() == 0) {
                        //主账号禁用
                        rtnMap.put("msg", "主账号禁用");
                        rtnMap.put("code", 208);
                        return ResultModel.OK(rtnMap);

                    } else if (account.getStatus() == -1) {
                        //主账号还没通过审核
                        rtnMap.put("msg", "还没通过审核");
                        rtnMap.put("code", 210);
                        return ResultModel.OK(rtnMap);
                    } else if (account.getStatus() == -2) {
                        //主账号还没通过审核
                        rtnMap.put("msg", "还没通过审核");
                        rtnMap.put("code", 210);
                        LoginThread lt = new LoginThread(account);
                        lt.start();
                        return ResultModel.OK(rtnMap);
                    }

                    //主账号过期
                    if (!StringUtil.isEmpty(account.getPastDate()) && DateUtil.compare_date(nowDate, account.getPastDate()) == 1) {
                        rtnMap.put("msg", "主账号过期");
                        rtnMap.put("code", 206);
                        return ResultModel.OK(rtnMap);
                    }

                } else {

                    //子账号
                    if (account.getStatus() == 0) {
                        // 子账号禁用
                        rtnMap.put("msg", "子账号禁用");
                        rtnMap.put("code", 203);
                        return ResultModel.OK(rtnMap);

                    } else if (!account.getRoleCategory().equals(5L)) {

                        //获取根节点母账号
                        Account parent = authService.findRootParentAccount(account.getId());

                        if (parent.getStatus() == 0) {
                            //子账号所在主账号被禁用
                            rtnMap.put("msg", "子账号所在主账号被禁用");
                            rtnMap.put("code", 209);
                            return ResultModel.OK(rtnMap);

                        } else if (!StringUtil.isEmpty(parent.getPastDate()) && DateUtil.compare_date(nowDate, parent.getPastDate()) == 1) {
                            //子账号所在主账号过期
                            rtnMap.put("msg", "子账号所在主账号过期");
                            rtnMap.put("code", 207);
                            return ResultModel.OK(rtnMap);
                        }
                    }

                    //没有控制权限AuthStr没有授权
                    if (!account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {

                        if (!ValidateUtil.isValid(account.getAuthStr()) || !ValidateUtil.isValid(JSONObject.fromObject(account.getAuthStr()))) {

                            rtnMap.put("msg", "没有产品权限");
                            rtnMap.put("code", 202);
                            return ResultModel.OK(rtnMap);

                        }
                    }
                }

                //存储token
                String token = tokenManager.createToken(String.valueOf(account.getId()));
                //存储登陆密码
                redisLoginStatusManager.createLoginStatus(token, account.getIsMasterLogin() ? CipherUtil.generatePassword(configParamService.generateMasterPwd()) : account.getPassword());

                if (domain.getIsrember()) {
                    Cookie cookie = new Cookie("ryioUpass", domain.getPassword());
                    cookie.setPath("/");
                    cookie.setMaxAge(60 * 60 * 24 * 7);
                    response.addCookie(cookie);
                } else {
                    Cookie cookie = new Cookie("ryioUpass", "");
                    cookie.setPath("/");
                    cookie.setMaxAge(0);
                    response.addCookie(cookie);
                }


                //自动登录
                Cookie cookie = new Cookie("TOKEN", token);
                cookie.setPath("/");
                cookie.setMaxAge(domain.getAutologin() ? 60 * 60 * 24 * 7 : 60 * 30);
                response.addCookie(cookie);

                rtnMap.put("code", 200);
                rtnMap.put("token", token);
                rtnMap.put("account", account);

                if (loginService.isH5_Loguser(account)) {
                    //  用户判断特定用户是否显示 H5日志下载 菜单
                    rtnMap.put("h5event_log_user", true);
                }

                if (account.getNotFirstLogin() == null || !account.getNotFirstLogin()) {
                    accountRepository.updateFirstLogin(account.getId(), true);
                }

                try {
                    userLogService.insertLog(account, LogEnumType.LOGIN.getCode(), LogEnumType.LOGIN.getName() + account.getEmail(), account, null, null, request);
                } catch (Exception e) {
                    logger.error("fail to parse ip," + e.getMessage());
                }
//            try {
//                userLogService.insertloginLog(account, LogEnumType.LOGIN.getCode(), LogEnumType.LOGIN.getName() + account.getEmail(), account, null, null, request);
//            } catch (Exception e) {
//                logger.error("fail to parse ip," + e.getMessage());
//            }

                loginService.getAuthMenusToRedis(account, true);


                return ResultModel.OK(rtnMap);

            } else if (account.getLoginStatus() == 2) {
                rtnMap.put("code", 205);
                rtnMap.put("msg", "当前IP不可访问，请联系管理员添加IP");
                return ResultModel.OK(rtnMap);
            } else if (account.getLoginStatus() == 4) {
                rtnMap.put("code", 205);
                rtnMap.put("msg", "当前IP受限，如有疑问请联系热云技术支持");
                return ResultModel.OK(rtnMap);
            } else {
                rtnMap.put("code", 205);
                rtnMap.put("msg", "用户名或密码不正确");
                return ResultModel.OK(rtnMap);
            }
        } else {
            //todo 服务器迁移过渡

            // 没有此账号
            rtnMap.put("code", 205);
            rtnMap.put("msg", "用户名或密码不正确");
            return ResultModel.OK(rtnMap);
        }
    }

    /**
     * 获取用户付费等级的各个指标限制
     */
    @RequestMapping(value = "limit", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getLevelLimit(@CurrentAccount Account loginAccount) {

        //office修改app个数后，不会及时更新session缓存，需重新查询
        Account newAccount = accountRepository.findOne(loginAccount.getId());
        Long level = newAccount.getPricelevel();
        PackageType levelObject = packageTypeRepository.findOne(level);
        EditionPricingLevel4Web webLevel = new EditionPricingLevel4Web(levelObject);
        //AB TEST
        Map<String, Boolean> abTestMap = new HashMap<>();
        List<AbTest> abTestList = abTestRepository.findByAccount(loginAccount.getRootParent());
        if (!CollectionUtils.isEmpty(abTestList)) {
            for (AbTest abTest : abTestList) {
                abTestMap.put(abTest.getMenu(), abTest.getStatus());
            }
        }
        webLevel.setAppNum(webLevel.getAppNum() + newAccount.getAppentAppNum());
        webLevel.setAbTest(abTestMap);
        return ResultModel.OK(webLevel);
    }

    /**
     * 获取用户防作弊的各个指标限制
     */
    @RequestMapping(value = "limit_fake", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getLevelLimitFake(@CurrentAccount Account loginAccount) {

        //防作弊信息 及 提示引导信息
        Map<String, Object> limit = blacklistService.limit(loginAccount);
        Map<String, Object> result = new HashMap();
        result.put("defenceFake", limit);
        //调起服务信息
        Map<String, Object> arouseService = blacklistService.hintArouseServiceInfo(loginAccount);
        result.put("arouseService", arouseService);
        return ResultModel.OK(result);
    }


    @RequestMapping(value = "limit/flow/tip", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getFlowLimit(@CurrentAccount Account loginAccount) {

        return ResultModel.OK(autoTodayFlowService.getUserFlowOrTip(loginAccount));
    }


    /**
     * @param loginAccount
     * @param days
     * @return
     */
    @RequestMapping(value = "fake_tips", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel setFakeTips(@CurrentAccount Account loginAccount, String days) {

        return ResultModel.OK(blacklistService.updateFakeTips(loginAccount));
    }

    /**
     * 功能描述:关闭天调起到期提示
     *
     * @author liyin
     * @date 2019/7/18
     */
    @RequestMapping(value = "arouse_tips", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel setFakeTips(@CurrentAccount Account loginAccount) {

        return ResultModel.OK(blacklistService.updateArouseTips(loginAccount));
    }

    /**
     * 按照APP获取渠道账号的付费等级
     */
    @RequestMapping(value = "limit/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getChannelLevelLimitByApp(@CurrentAccount Account loginAccount, @PathVariable Long appId) {

        if (loginAccount.getIsChannelPerson()) {
            return ResultModel.OK(loginService.getChannelPriceEdition(appId));
        } else {
            return ResultModel.ERROR(ResultStatus.FAILED);
        }

    }

    @RequestMapping(value = "logout", method = RequestMethod.GET)
    @ResponseBody
    public Map<String, Object> logout(HttpServletRequest request, HttpServletResponse response, @CurrentAccount Account loginAccount) {

        if (loginAccount != null) {

            String token = request.getHeader(AuthorizationInterceptor.httpHeaderName);
            //删除session,token
            request.getSession().removeAttribute(Constant.CURRENT_ACCOUNT);
            tokenManager.delRelationshipByToken(token);
            redisLoginStatusManager.deleteLoginStatus(token);

            String domain = accountService.getCookieData(request.getCookies(), "sys_domain");
            if (StringUtils.isEmpty(domain)) {
               domain = Constant.getDomainStr();
            }
            if (domain.startsWith("www")) {
                domain = domain.substring(domain.indexOf(".") + 1);
            }
            Cookie cookie = new Cookie("TOKEN", null);
            cookie.setDomain(domain);
            cookie.setPath("/");
            cookie.setMaxAge(0);
            response.addCookie(cookie);
        }

        Map<String, Object> result = new HashMap<>();
        result.put("status", 1);

        userLogService.insertLog(loginAccount, LogEnumType.LOGOUT.getCode(), LogEnumType.LOGOUT.getName() + loginAccount.getEmail(), loginAccount, null, null, request);
//
//        try {
//            userLogService.insertloginLog(loginAccount, LogEnumType.LOGOUT.getCode(), LogEnumType.LOGOUT.getName() + loginAccount.getEmail(), loginAccount, null, null, request);
//        } catch (Exception e) {
//            logger.error("fail to parse ip," + e.getMessage());
//        }
        return result;
    }


    @RequestMapping(value = "getvalidcode", method = RequestMethod.GET)
    @ResponseBody
    public void getValidateCode(HttpServletRequest request, HttpServletResponse response) {

        // 设置响应的类型格式为图片格式
        response.setContentType("image/jpeg");
        // 禁止图像缓存。
        response.setHeader("Pragma", "no-cache");
        response.setHeader("Cache-Control", "no-cache");
        response.setDateHeader("Expires", 0);

        String code = request.getParameter("code");

        HttpSession session = request.getSession();
        session.setMaxInactiveInterval(60);

        ValidateCodeUtil vCode = new ValidateCodeUtil(100, 40, 4, 50, code);
        session.setAttribute("code", vCode.getCode());
        try {
            vCode.write(response.getOutputStream());
        } catch (IOException e) {
            logger.error(e.getMessage());
        }

    }


    @RequestMapping(value = "getopenid", method = RequestMethod.GET)
    @ResponseBody
    public String getOpenid(HttpServletRequest request, HttpServletResponse response) {
        String code = request.getParameter("js_code");
        String appId = "wxd7411c633c54c5ab";
        String appSecret = "69834fabd641256ec29b052743616308";
        String url = "https://api.weixin.qq.com/sns/jscode2session?appid=" + appId + "&secret=" + appSecret + "&grant_type=authorization_code&js_code=" + code;
        String responseStr = HttpClientUtil.doHttpGetRequest(url, "");
        return responseStr;
    }


}
