package com.reyun.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.reyun.model.*;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.repository.OpenApiRepository;
import com.reyun.service.OpenApiWebService;
import com.reyun.service.ReportService;
import com.reyun.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.*;

@Controller
@RequestMapping("/tkio/openapi")
public class OpenApiController {
    protected Logger logger = LoggerFactory.getLogger(OpenApiController.class);

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private OpenApiRepository openApiRepository;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ReportService reportService;
    @Autowired
    private OpenApiWebService openApiWebService;

    private static Map<String, String> reportNameMap = new HashMap<>(3);
    static {
        reportNameMap.put("campaign","campaigninfo_bycampgroup");
        reportNameMap.put("channel","campaigninfo_bychannel");
        reportNameMap.put("date","campaigninfo_byds");
    }
    private static Map<String, String> raplaceMap = new HashMap<>(3);
    static {
        raplaceMap.put("campaign","campaignid");
        raplaceMap.put("channel","channelname");
        raplaceMap.put("date","ds");
    }

    private static final String AUTHORIZATION_TOKEN_SUFFIX = "_OPENAPI_AUTHORIZATION_TOKEN";
    private static final String REFRESH_TOKEN_SUFFIX = "_OPENAPI_REFRESH_TOKEN";

    private static final String REPORT_REQUEST_SUFFIX = "_REPORT_REQUEST";

    //每分钟最大访问量为5次
    private static final long REPORT_MAX_REQUEST_TIME = 60;
    private static final long REPORT_MAX_REQUEST_NUM = 5;

    //每小时
    private static final long MAX_REQUEST_TIME = 3600;
    //访问6次
    private static final long MAX_REQUEST_NUM  = 6;




    @RequestMapping(value = "authorization_token", method = RequestMethod.POST ,produces = "application/json;charset=UTF-8")
    @ResponseBody
    public ResultModel authorization_token(String appkey,String appsecret) {
        int intercept = RedisUtilNew.getInstance().intercept(appkey + AUTHORIZATION_TOKEN_SUFFIX, MAX_REQUEST_TIME);
        if(intercept>MAX_REQUEST_NUM){
            return ResultModel.ERROR(ResultStatus.REPORT_TOKEN_OUT);
        }
        App app = appRepository.getByAppidAndToken(appkey, appsecret);
        if(app==null){
            return ResultModel.ERROR(ResultStatus.REPORT_TOKEN_ERROR);
        }
        OpenApi openApi = openApiWebService.create(app);
        Map<String, Object> rs = new HashMap<>();
        rs.put("access_token",openApi.getAccessToken());
        rs.put("refresh_token",openApi.getRefreshToken());
        rs.put("access_token_expires_in",openApiWebService.ACCESS_TOKEN_EXPIRES_IN);
        rs.put("refresh_token_expires_in",openApiWebService.REFRESH_TOKEN_EXPIRES_IN);
        return ResultModel.OK(rs);
    }

    @RequestMapping(value = "refresh_token", method = RequestMethod.POST,produces = "application/json;charset=UTF-8")
    @ResponseBody
    public ResultModel refresh_token(String appkey,String refresh_token) {

        int intercept = RedisUtilNew.getInstance().intercept(appkey + REFRESH_TOKEN_SUFFIX, MAX_REQUEST_TIME);
        //System.out.println("intercept = " + intercept);
        if(intercept>MAX_REQUEST_NUM){
            return ResultModel.ERROR(ResultStatus.REPORT_TOKEN_OUT);
        }
        OpenApi rt = openApiWebService.getByAppidAndRefreshToken(appkey,refresh_token);
        if(rt==null){
            return ResultModel.ERROR(ResultStatus.REPORT_TOKEN_ERROR);
        }
        if(new Date().getTime()>rt.getRefreshTokenExpires().getTime()){
            return ResultModel.ERROR(ResultStatus.REPORT_TOKEN_FAILURE);
        }
        OpenApi openApi = openApiWebService.refresh(rt);
        Map<String, Object> rs = new HashMap<>();
        rs.put("access_token",openApi.getAccessToken());
        rs.put("refresh_token",openApi.getRefreshToken());
        rs.put("access_token_expires_in",openApiWebService.ACCESS_TOKEN_EXPIRES_IN);
        rs.put("refresh_token_expires_in",openApiWebService.REFRESH_TOKEN_EXPIRES_IN);
        return ResultModel.OK(rs);
    }


    @PostMapping(value ="/report",produces = "application/json;charset=UTF-8")
    @ResponseBody
    public ResultModel webReport(@RequestBody OpenApi4web openApi4web) {
        if(openApi4web.isEmpty()){
            return new ResultModel(220,"请求参数不全");
        }
        try {
            if(DateUtil.daysBetween(openApi4web.getStartDate(),openApi4web.getEndDate())>31){
                return new ResultModel(230,"查询时间跨度超31天");
            }
        } catch (ParseException e) {
            e.printStackTrace();
            return new ResultModel(230,"日期格式必须为 yyyy-MM-dd");
        }
        if(!reportNameMap.containsKey(openApi4web.getType())){
            return new ResultModel(220,"'type' value not exist");
        }
        int intercept = RedisUtilNew.getInstance().intercept(openApi4web.getAppkey() + "_"+openApi4web.getType()+REPORT_REQUEST_SUFFIX, REPORT_MAX_REQUEST_TIME);
        //System.out.println("intercept = " + intercept);
        if(intercept>REPORT_MAX_REQUEST_NUM){
            return ResultModel.ERROR(ResultStatus.REPORT_REQUEST_OUT);
        }
        return this.report(openApi4web);
    }
    /*@PostMapping("/campaign")
    @ResponseBody
    public ResultModel campaign(String startDate,) {
        return this.report("campaigninfo_bycampgroup",openApiVisit,"campaign");
    }
    @PostMapping("/campgroup")
    @ResponseBody
    public ResultModel campgroup(OpenApiVisit openApiVisit) {
        return this.report("campaigninfo_bycampgroup",openApiVisit,"campgroup");
    }
    @PostMapping("/channel")
    @ResponseBody
    public ResultModel channel(OpenApiVisit openApiVisit) {
        return this.report("campaigninfo_bychannel",openApiVisit,"channel");
    }
    @PostMapping("/date")
    @ResponseBody
    public ResultModel date(OpenApiVisit openApiVisit) {

        return this.report("campaigninfo_byds",openApiVisit,"date");
    }*/

    private ResultModel report(OpenApi4web openApi4web){
        OpenApi openApi = openApiRepository.findappkeyAndaccessToken(openApi4web.getAppkey(),openApi4web.getToken(),new Date());
        if(openApi==null) return new ResultModel(210,"token无效");
        String reportName = reportNameMap.get(openApi4web.getType());
        App app = appRepository.findByAppkey(openApi4web.getAppkey());
        String viewcolumn = "rate_retentiond1,rate_retentiond3,rate_retentiond7,rate_retentiond14,rate_retentiond30";

        Map<String, String> conditions = new HashMap<>();
        conditions.put("viewtype",openApi4web.getType());
        conditions.put("func","campaigninfo");
        conditions.put("datatype","list");
        conditions.put("lang","zh");
        conditions.put("startdate",openApi4web.getStartDate());
        conditions.put("enddate",openApi4web.getEndDate());
        conditions.put("viewcolumn",viewcolumn);
        Account account = accountRepository.findOne(app.getRootParent());
        JSONArray parse=null;
        String keyName = raplaceMap.get(openApi4web.getType());
        try {
            ReportListResult reportListResult = (ReportListResult)reportService.report(app.getId(),reportName,account,conditions);
            if(raplaceMap.containsKey(openApi4web.getType())){
                 reportListResult.getVal().iterator();
                List<Map<String, Object>> val = reportListResult.getVal();
                for (Map<String, Object> valMap : val) {
                    //Map<String, Object> next = iterator.next();
                    valMap.put("name",valMap.get(keyName));
                    valMap.put("num_d1_install",valMap.get("num_install"));
                    Iterator<Map.Entry<String, Object>> iterator = valMap.entrySet().iterator();
                    while(iterator.hasNext()){
                        //只保留api可以提供的数据
                        Map.Entry<String, Object> next = iterator.next();
                        if(next.getKey().startsWith("num_d")||next.getKey().startsWith("rate_retentiond")||next.getKey().equals("name"))continue;
                        iterator.remove();
                    }
                }
            }
             return ResultModel.OK(reportListResult.getVal());

        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultModel.OK(parse);
    }

    private JSONArray parse(List<Map<String,Object>> val,OpenApi4web openApi4web) {
        String s=JSON.toJSONString(val).replace(raplaceMap.get(openApi4web.getType()),"name");
        return JSON.parseArray(s);
    }

    public static void main(String[] args) {
        System.out.println("/api/tkio/campaign".startsWith("/api/tkio/*"));
    }

}
