package com.reyun.controller;

import com.google.common.base.Function;
import com.google.common.collect.Maps;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.Account;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.ReportService;
import com.reyun.util.Constant;
import com.reyun.util.ResultModel;
import com.reyun.util.StringUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

@Controller
@RequestMapping("/report/{appid}")
public class ReportController {

    Logger logger = LoggerFactory.getLogger(ReportController.class);
    @Autowired
    private ReportService reportService;

    @RequestMapping(value = "{name}/number", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel reportNumber(final HttpServletRequest request, HttpServletResponse response, @PathVariable Long appid,
                                    @PathVariable("name") String reportName, @CurrentAccount Account loginAccount)
            throws IOException {
        Map<String, String> conditions = preCondition(request);

        return ResultModel.OK(reportService.reportNumber(appid, reportName, loginAccount, conditions));
    }

    @RequestMapping(value = "{name}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel report(final HttpServletRequest request, HttpServletResponse response, @PathVariable Long appid, @PathVariable("name") String reportName,
                              @CurrentAccount Account loginAccount, @RequestHeader(name = "region", required = false) String region,
                              @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
        Map<String, String> conditions = preCondition(request);

        //屏蔽前端错误的排序参数
        ignoreErroOrderBy(conditions, "campaignname");

        try {
            return ResultModel.OK(reportService.report(appid, reportName, loginAccount, conditions));
        } catch (Exception e) {
            String email = loginAccount != null ? loginAccount.getEmail() : "";
            logger.error("报表错误,email =" + email + " appid=" + appid
                    + " reportname =" + reportName +" 参数"+ conditions.get("startdate")+" - "+ conditions.get("enddate"), e);
            return new ResultModel(-999, e.getMessage());
        }
    }

    /*@RequestMapping(value = "/campaigninfo_byds", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel reportCampaignDate(final HttpServletRequest request, @PathVariable Long appid, @PathVariable("name") String reportName,
                              @CurrentAccount Account loginAccount)

    {
        Map<String, String> conditions = preCondition(request);

        //屏蔽前端错误的排序参数
        ignoreErroOrderBy(conditions,"campaignname");

        return ResultModel.OK(reportService.reportCampaignDate(appid, "campaigninfo_byds", loginAccount, conditions));
    }*/


    @RequestMapping(value = "/home", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel report(final HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        Map<String, String> conditions = preCondition(request);

        return ResultModel.OK(reportService.homeReport(conditions));
    }

    private Map<String, String> preCondition(final HttpServletRequest request) {
        Map<String, String> conditions = Maps.newHashMap();
        conditions.putAll(Maps.toMap(Collections.list(request.getParameterNames()), new Function<String, String>() {
            @Override
            public String apply(String s) {
                return request.getParameter(s);
            }
        }));

        //添加国际化的处理
        String lang = request.getHeader(Constant.LOCALE_KEY);
        LanguageTypeEnum locale = LanguageTypeEnum.getVal(lang);
        conditions.put(Constant.LOCALE_KEY, locale.getCode());

        //添加地域支持
        String region = request.getHeader(Constant.REGION_KEY);

        if (StringUtils.isNotEmpty(region) && !"ALL".equals(region)) {//如果是all，不需要筛选地区，直接统计全部
            String[] cancledRegion = region.split(",");
            StringBuffer sb = new StringBuffer();
            for (String str : cancledRegion) {
                sb.append("'").append(str).append("',");
            }
            region = sb.toString().substring(0, sb.toString().length() - 1);
            conditions.put("region", region);
        }
        //end
        return conditions;
    }

    private void ignoreErroOrderBy(Map<String, String> conditions, String... names) {

        String orderbyName = conditions.get("orderby");
        for (String name : names) {
            if (StringUtils.equals(orderbyName, name)) {
                conditions.remove("orderby");
                break;
            }
        }
    }
}
