package com.reyun.controller;

import com.reyun.model.App;
import com.reyun.model.ReportToken;
import com.reyun.service.AppService;
import com.reyun.service.ReportTokenService;
import com.reyun.util.RedisUtilNew;
import com.reyun.util.ReportTokenModel;
import com.reyun.util.ResultStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by nolan on 22/12/2016.
 * description:
 */
@Controller
@RequestMapping("tkio")
public class ReportTokenController {

    @Autowired
    private AppService appService;
    @Autowired
    private ReportTokenService reportTokenService;

    private static final String AUTHORIZATION_TOKEN_SUFFIX = "_AUTHORIZATION_TOKEN";
    private static final String REFRESH_TOKEN_SUFFIX = "_REFRESH_TOKEN";

    //每小时
    private static final long MAX_REQUEST_TIME = 3600;
    //访问6次
    private static final long MAX_REQUEST_NUM  = 6;




    @RequestMapping(value = "authorization_token", method = RequestMethod.POST)
    @ResponseBody
    public ReportTokenModel authorization_token(String appid,String appsecret) {
        int intercept = RedisUtilNew.getInstance().intercept(appid + AUTHORIZATION_TOKEN_SUFFIX, MAX_REQUEST_TIME);
        //System.out.println("intercept = " + intercept);
        if(intercept>MAX_REQUEST_NUM){
            return ReportTokenModel.ERROR(ResultStatus.REPORT_TOKEN_OUT);
        }
        App app = appService.getByAppidAndToken(appid, appsecret);
        if(app==null){
            return ReportTokenModel.ERROR(ResultStatus.REPORT_TOKEN_ERROR);
        }
        ReportToken reportToken = reportTokenService.create(app);
        Map<String, Object> rs = new HashMap<>();
        rs.put("access_token",reportToken.getAccessToken());
        rs.put("refresh_token",reportToken.getRefreshToken());
        rs.put("access_token_expires_in",reportTokenService.ACCESS_TOKEN_EXPIRES_IN);
        rs.put("refresh_token_expires_in",reportTokenService.REFRESH_TOKEN_EXPIRES_IN);
        return ReportTokenModel.OK(rs);
    }

    @RequestMapping(value = "refresh_token", method = RequestMethod.POST)
    @ResponseBody
    public ReportTokenModel refresh_token(String appid,String refresh_token) {

        int intercept = RedisUtilNew.getInstance().intercept(appid + REFRESH_TOKEN_SUFFIX, MAX_REQUEST_TIME);
        //System.out.println("intercept = " + intercept);
        if(intercept>MAX_REQUEST_NUM){
            return ReportTokenModel.ERROR(ResultStatus.REPORT_TOKEN_OUT);
        }
        ReportToken rt = reportTokenService.getByAppidAndRefreshToken(appid,refresh_token);
        if(rt==null){
            return ReportTokenModel.ERROR(ResultStatus.REPORT_TOKEN_ERROR);
        }
        if(new Date().getTime()>rt.getRefreshTokenExpires().getTime()){
            return ReportTokenModel.ERROR(ResultStatus.REPORT_TOKEN_FAILURE);
        }
        ReportToken reportToken = reportTokenService.refresh(rt);
        Map<String, Object> rs = new HashMap<>();
        rs.put("access_token",reportToken.getAccessToken());
        rs.put("refresh_token",reportToken.getRefreshToken());
        rs.put("access_token_expires_in",reportTokenService.ACCESS_TOKEN_EXPIRES_IN);
        rs.put("refresh_token_expires_in",reportTokenService.REFRESH_TOKEN_EXPIRES_IN);
        return ReportTokenModel.OK(rs);
    }

}
