package com.reyun.controller;

import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.model.KeywordsCreative;
import com.reyun.model.SemRename;
import com.reyun.repository.*;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.SemRenameService;
import com.reyun.service.SmSearchService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

@Controller
@RequestMapping("/sem")
public class SemRenameController {

    @Autowired
    private SemRenameService semRenameService;

    @Autowired
    private SemRenameRepository semRenameRepository;

    @Autowired
    private KeywordsCreativeRepository keywordsCreativeRepository;

    @Autowired
    private DataAuthRepository dataAuthRepository;

    @Autowired
    private SmSearchService smSearchService;

    @Autowired
    private ConfigParamRepository configParamRepository;

    @Autowired
    private ChannelRepository channelRepository;

    @RequestMapping(value = "/upload/{cid}/{appId}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel Upload(@CurrentAccount Account loginAccount, @RequestParam("file") MultipartFile file, @PathVariable("cid") Long cid, @PathVariable("appId") Long appId) {
        if (file.getOriginalFilename().lastIndexOf(".xls") != -1 || file.getOriginalFilename().lastIndexOf(".xlsx") != -1) {

            ResultModel statusModel = semRenameService.checkSemTemplateFormat(file);
            if (statusModel != null) {
                //格式校验未通过
                return statusModel;
            }

            try {
                return ResultModel.OK(semRenameService.semRenameTransfer(file, cid, appId, loginAccount.getId()));
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {

            return ResultModel.ERROR(ResultStatus.FILE_TYPE_ERROR);
        }
        return null;
    }

    @RequestMapping(value = "/taskList/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel taskList(@PathVariable("appId") Long appid) {
        List<SemRename> all = semRenameRepository.findByappid(appid);
        return ResultModel.OK(all);
    }

    @RequestMapping(value = "/keywordsList", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel keywordsList(@CurrentAccount Account loginAccount) {
        String[] sem_channels = configParamRepository.findParamsByKey("sem_channel").split(",");
        ArrayList<Integer> ids = new ArrayList<>();
        for (String s : sem_channels) {
            ids.add(Integer.parseInt(s));
        }
        List<KeywordsCreative> all = keywordsCreativeRepository.findBaiDuAndSouGou(ids, loginAccount.getRootParent());
        return ResultModel.OK(all);
    }

    @RequestMapping(value = "/keywordsDelete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel keywordsListDelete(@PathVariable("id") Long id) {
        keywordsCreativeRepository.delete(id);
        return ResultModel.OK();
    }

    @RequestMapping(value = "/semtaskDelete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel semtaskDelete(@PathVariable("id") Long id) {
        semRenameRepository.delete(id);
        return ResultModel.OK();
    }

    @RequestMapping(value = "/findAuth", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAuth(@CurrentAccount Account loginAccount) {
        HashMap<Long, Boolean> hashMap = new HashMap<>();

        String[] semChannels = configParamRepository.findParamsByKey("sem_channel").split(",");
        for (String s : semChannels) {
            long channelId = Long.parseLong(s);
            hashMap.put(channelId, false);
            List<Boolean> dataAuthByAccount = dataAuthRepository.findDataAuthByAccountAndCid(loginAccount.getId(), channelId);
            if (dataAuthByAccount != null && dataAuthByAccount.size() == 1 && dataAuthByAccount.get(0)) {
                hashMap.put(channelId, true);
            }
        }
        return ResultModel.OK(hashMap);
    }

    @RequestMapping(value = "/smkeypatch", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel smkeypatch(@RequestParam String fileName, @RequestParam String userName, @CurrentAccount Account loginAccount) {
        smSearchService.patch(fileName, userName, loginAccount.getId());
        return ResultModel.OK();
    }

    @RequestMapping(value = "/findChannel", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findChannel(@CurrentAccount Account loginAccount) {
        String[] sem_channels = configParamRepository.findParamsByKey("sem_channel").split(",");
        ArrayList<Long> ids = new ArrayList<>();
        for (String s : sem_channels) {
            ids.add(Long.parseLong(s));
        }
        List<Channel> semChannels = channelRepository.findById(ids);
        return ResultModel.OK(semChannels);
    }
}
