package com.reyun.controller;
import com.reyun.model.Channel;
import com.reyun.model.SubChannelRename;
import com.reyun.repository.ChannelRepository;
import com.reyun.repository.SubChannelRenameRepository;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.service.SubChannelService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by song on 2018/1/23.
 */

@Controller
@RequestMapping("/subchannel")
public class SubChannelController {

    @Autowired
    SubChannelService subChannelService;

    @Autowired
    SubChannelRenameRepository subChannelRenameRepository;

    @Autowired
    ChannelRepository channelRepository;


    @RequestMapping(value = "findchannel/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findChannel(@PathVariable Long appId) {

        return ResultModel.OK(subChannelService.listChannelIncludeSubchannel(appId));

    }


    @RequestMapping(value = "findSubchannelList/{appId}/{cid}", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel findSubchannelList(HttpServletRequest request, @PathVariable Long appId, @PathVariable Long cid) {
        String searchtext = request.getParameter("search");
        return ResultModel.OK(subChannelService.findSubchannelList(appId,cid,searchtext));
    }

    @RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel deleteSubChannel(@PathVariable Long id) {
        return ResultModel.OK(subChannelRenameRepository.deleteSubChannel(id));
    }

    @RequestMapping(value = "update/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public Object updateName(@PathVariable Long id, @RequestParam String name,@RequestParam Long cid, @RequestParam Long appId) {
        if(subChannelRenameRepository.findByCidAndSubChannelChName(name,cid,appId)!=null && !subChannelRenameRepository.findByCidAndSubChannelChName(name, cid,appId).getId().equals(id)){
            return false;
        }else {
            subChannelService.updateName(id, name);
            return true;
        }
    }

    /**
     * 在推广活动详情里面修改子渠道名字
     * @param origname
     * @param rename
     * @param cid
     * @param appId
     * @return
     */
    @RequestMapping(value = "updateInfo", method = RequestMethod.PUT)
    @ResponseBody
    public Object updateInfoName(@RequestParam String origname,@RequestParam String rename,@RequestParam Long cid, @RequestParam Long appId) {
        if (subChannelRenameRepository.findByCidAndSubChannelChName(origname, cid, appId) != null &&
                (subChannelRenameRepository.findByCidAndSubChannelChName(rename, cid, appId) != null &&
                        !subChannelRenameRepository.findByCidAndSubChannelChName(rename, cid, appId).getSubChannelName().equals(origname))) {
            return false;
        } else if (subChannelRenameRepository.findByCidAndSubChannelChName(origname, cid, appId) != null &&
                subChannelRenameRepository.findByCidAndSubChannelChName(rename, cid, appId) == null) {
            subChannelService.updateName(subChannelRenameRepository.findByCidAndSubChannelChName(origname, cid, appId).getId(), rename);
            return true;
        } else if (subChannelRenameRepository.findByCidAndSubChannelChName(origname, cid, appId) == null) {
            Channel channel = channelRepository.findById(cid);
            SubChannelRename sub = new SubChannelRename();
            if (channel != null) {
                sub.setSubChannel(origname);
                sub.setSubChannelName(rename);
                sub.setCid(cid);
                sub.setChannelName(channel.getName());
                sub.setDelFlag(false);
                sub.setAppid(appId);
                subChannelRenameRepository.save(sub);
            }
        }
        return true;
    }

    @RequestMapping(value = "/upload/{cid}/{appId}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel Upload(@RequestParam("file") MultipartFile file, @PathVariable("cid") Long cid,@PathVariable("appId") Long appId)
    {
        if (file.getOriginalFilename().lastIndexOf(".xls") != -1 || file.getOriginalFilename().lastIndexOf(".xlsx") != -1) {

                return ResultModel.OK(subChannelService.subChannelTransfer(file,cid,appId));

        } else {
            return ResultModel.ERROR(ResultStatus.FILE_TYPE_ERROR);
        }

    }

}
