package com.reyun.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AdsAccountService;
import com.reyun.service.CampaignService;
import com.reyun.service.UserLogService;
import com.reyun.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;


@RestController
@RequestMapping("/adsAccount")
public class AdsAccountController {

    private static final String URL = "http://pre-api.adsdesk.cn/adsdesk/api/open_api/tkio/tkioAppId";
    private static final String APPLICATION_JSON = "application/json";
    private static final String CONTENT_TYPE_TEXT_JSON = "application/json;charset=UTF-8";
    private static final Logger logger = LoggerFactory.getLogger(AdsAccountController.class);
    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private AdsAccountRepository adsAccountRepository;
    @Autowired
    private CampaignService campaignService;
    @Autowired
    private CampaignRepository campaignRepository;
    @Autowired
    private AdsAccountService adsAccountService;
    @Autowired
    private DataAuthRepository dataAuthRepository;
    @Autowired
    private ChannelRepository channelRepository;
    @Autowired
    UserLogService userLogService;

    private static final String FLAG = "tkio-ads";

    /*@PostMapping(value = "/bind")
    public ResultModel bind(String email, String password, int type) {
        Account account = accountRepository.findAccountByEmail(email);
        if (account == null || !CipherUtil.generatePassword(account.getPassword()+FLAG).equals(password)) {
            return new ResultModel(-1000, "账号或密码错误");
        }
        AdsAccount adsAccount = adsAccountRepository.findByAccount(account.getId());
        if (type == 1) {
            //1解绑
            if (adsAccount != null) {
                adsAccountRepository.deleteById(adsAccount.getId());
                return new ResultModel(200, "解绑成功");
            }
            return new ResultModel(-1001, "解绑失败,该用户尚未绑定");
        }
        //0绑定
        String s = this.adsAccountService.checkAccount(account);
        if (!"true".equals(s)) return new ResultModel(-1001, s);
        AdsAccount ads = new AdsAccount();
        if (adsAccount != null) {
            //return new ResultModel(-1001, "请勿重复绑定");
            ads = adsAccount;
            ads.setUpdateTime(new Date());
        }
        ads.setAccount(account.getId());
        ads.setEmail(account.getEmail());
        ads.setPermission(0);
        AdsAccount save = adsAccountRepository.save(ads);
        try {
            adsAccountService.send(save,false);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ResultModel(200, "绑定成功");
    }
*/

    @PostMapping(value = "/bindUser")
    public ResultModel bindSuperUser(String email, String password, int type) {
        logger.info("ads bindUser...");
        Account account = accountRepository.findAccountByEmail(email);
        if (account == null || !CipherUtil.generatePassword(account.getPassword() + FLAG).equals(password)) {
            return new ResultModel(-1000, "账号或密码错误");
        }
        /*if(!account.getIsSuperUser()){
            return new ResultModel(-1010, "只有主账号可以绑定");
        }*/
        AdsAccount adsAccount = adsAccountRepository.findByAccount(account.getId());
        if (type == 1) {
            //1解绑
            if (adsAccount != null) {
                adsAccountRepository.delete(adsAccount);
                return ResultModel.OK("解绑成功");
            }
            return new ResultModel(-1001, "解绑失败,该用户尚未绑定");
        }
        String token = UUID.randomUUID().toString().replace("-", "");
        //0绑定
        String s = this.adsAccountService.checkAccount(account);
        if (!"true".equals(s)) return new ResultModel(-1001, s);
        AdsAccount aas = new AdsAccount();
        if (adsAccount != null) {
            //return new ResultModel(-1001, "请勿重复绑定");
            aas = adsAccount;
            aas.setUpdateTime(new Date());
        }
        aas.setIsSuperUser(account.getIsSuperUser());
        aas.setToken(token);
        aas.setAccount(account.getId());
        aas.setEmail(account.getEmail());
        aas.setPermission(0);
        AdsAccount save = adsAccountRepository.save(aas);
        JSONObject jsonobj = new JSONObject(4);
        jsonobj.put("email", save.getEmail());
        jsonobj.put("isSuperUser", save.getIsSuperUser());
        jsonobj.put("account", save.getAccount());
        jsonobj.put("token", save.getToken());
        return ResultModel.OK(jsonobj);
    }

    @PostMapping(value = "/findApp")
    public ResultModel findApp(String token) {
        logger.info("ads findApp...");
        AdsAccount adsAccount = adsAccountRepository.getByToken(token);
        if (adsAccount == null) {
            return new ResultModel(1020,"token不存在");
        } else if (adsAccount.getPermission() != 0) {
            return new ResultModel(999, "账号失效请重新绑定");
        }
        return ResultModel.OK(adsAccountService.findApp(adsAccount));
    }

    /**
     * 创建推广活动,将单个创建和批量创建合在一起
     */
    @RequestMapping(value = "/addCampaign", method = RequestMethod.POST)
    public ResultModel createOneAndBatch(@RequestBody Campaign resource, String token) {
        logger.info("ads addCampaign...");
        AdsAccount adsAccount = adsAccountRepository.getByToken(token);
        if (adsAccount == null) {
            return new ResultModel(1020,"token不存在");
        } else if (adsAccount.getPermission() != 0) {
            return new ResultModel(999, "账号失效请重新绑定");
        }
        Account one = accountRepository.findOne(adsAccount.getAccount());
        if (one.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())
                || one.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())
                || one.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

            DataAuth dataAuth = dataAuthRepository.getByAppAndChannelAndAccountAndPermit(resource.getApp(), resource.getChannel(), one.getId());
            if(dataAuth ==null){
                return new ResultModel(-999, "没有创建权限");
            }
        }
        Campaign campaign = this.campaignRepository.findByNameAndAppIncludeDel(resource.getName(), resource.getApp());
        if(campaign!=null){
            return new ResultModel(999, "活动名称重复");
        }
        List<Campaign> result = new ArrayList<>();
        resource.setAccount(adsAccount.getAccount());
        resource.setCreateAccount(adsAccount.getAccount());
        resource.setCreateTime(new Date());
        List<String> shortUrlList = campaignService.makesurls(resource.getApp(), one, resource.getBatchNumber());

        if (!CollectionUtils.isEmpty(shortUrlList) && resource.getBatchNumber() == 1) {
            //单个创建推广活动
            resource.setSurl(shortUrlList.get(0));
            result.add(campaignService.create(resource.getApp(), resource));
        } else if (!CollectionUtils.isEmpty(shortUrlList) && resource.getBatchNumber() > 1) {
            //批量创建推广活动
            ArrayList<Campaign> campaigns = new ArrayList<>();
            ArrayList<String> campaignsName = new ArrayList<>();
            int startNumber = resource.getStartNumber();
            int batchNumber = resource.getBatchNumber();
            //String campaignName = resource.getCampaignName();
            String campaignName = resource.getName();
            int i = startNumber;
            while (i < batchNumber + startNumber) {
                String name = campaignName + "_" + i;
                Campaign cam = campaignRepository.findByNameAndAppIncludeDel(name, resource.getApp());
                if (ValidateUtil.isValid(cam)) {
                    campaigns.add(cam);
                    campaignsName.add(cam.getName());
                }
                i++;
            }
            if (campaigns.size() > 0) {
                return new ResultModel(ResultStatus.CAMPAIGN_REPEAT, campaignsName);
            } else {
                result = campaignService.createBatch(resource.getApp(), shortUrlList, resource);
            }
        } else {
            //失败
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
        List<JSONObject> rs = new ArrayList<>();
        for (Campaign cam : result) {
            String surl = cam.getSurl();
            Channel channel = channelRepository.findOne(cam.getChannel());
            String appPlatform = cam.getOs();
            String iosParam = channel.getIosparam();
            String androidParam = channel.getAndroidparam();
            String impUrl = Constant.trackurl + "/" + surl + "?impression=true";
            boolean isTrackUrlSpecial = channel.isIstrackurlspecial();
            boolean isTrackHttps = false;

            boolean isImpHttps = false;


            if (channel.isIshttps()) {
                isTrackHttps = true;
            }
            if (channel.isImpIsHttps()) {
                isImpHttps = true;
            }

            String trackUrl = Constant.trackurl + "/" + cam.getSurl();

            if (isTrackUrlSpecial && !StringUtil.isEmpty(androidParam) && AppPlatformEnum.ANDROID.getKey().equalsIgnoreCase(appPlatform)) {

                androidParam = androidParam.replace("$surl", surl);
                trackUrl = trackUrl + "?" + androidParam;

            } else if (isTrackUrlSpecial && !StringUtil.isEmpty(iosParam) && AppPlatformEnum.IOS.getKey().equalsIgnoreCase(appPlatform)) {

                iosParam = iosParam.replace("$surl", surl);
                trackUrl = trackUrl + "?" + iosParam;
            }

            if (isTrackHttps) {
                trackUrl = trackUrl.replace("http:", "https:");
            }

            if (isImpHttps) {
                impUrl=impUrl.replace("http:", "https:");
            }
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("app",cam.getApp());
            jsonObject.put("channel",cam.getChannel());
            jsonObject.put("campaignid",cam.getCampaignid());
            jsonObject.put("name",cam.getName());
            jsonObject.put("os",cam.getOs());
            jsonObject.put("url",cam.getUrl());
            jsonObject.put("surl",cam.getSurl());
            jsonObject.put("createTime",cam.getCreateTime());
            jsonObject.put("trackUrl",trackUrl);
            jsonObject.put("impUrl",impUrl);
            rs.add(jsonObject);
        }
        return ResultModel.OK(rs);
    }
    //复用推广活动管理的接口
    @RequestMapping(value = "/findCampaign", method = RequestMethod.POST)
    public ResultModel findCampaign(Long app,String token,boolean delFlag,String name){
        logger.info("ads findCampaign...");
        AdsAccount adsAccount = adsAccountRepository.getByToken(token);
        if (adsAccount == null) {
            return new ResultModel(1020,"token不存在");
        } else if (adsAccount.getPermission() != 0) {
            return new ResultModel(999, "账号失效请重新绑定");
        }
        Account account = accountRepository.findOne(adsAccount.getAccount());
        Map data = new HashMap();
        Campaign campaign = new Campaign();
        campaign.setDelFlag(delFlag);
        campaign.setName(name);
        campaign.setType("ry_coop");
        campaign.setNoLimit(true);
        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(account, app, "ry_coop",campaign,data);
        //List<Campaign4Web> res = campaignService.listCampaignByLink(result);
        data.put("data",result);
        return ResultModel.OK(data);
    }

    //复用推广活动管理的启用停用修改接口
    @RequestMapping(value = "/updateCampaign", method = RequestMethod.POST)
    public ResultModel updateCampaign(HttpServletRequest request, Long app, String token, Boolean delFlag, String name, String ids){
        logger.info("ads updateCampaign...");
        AdsAccount adsAccount = adsAccountRepository.getByToken(token);
        if (adsAccount == null) {
            return new ResultModel(1020,"token不存在");
        } else if (adsAccount.getPermission() != 0) {
            return new ResultModel(999, "账号失效请重新绑定");
        }
        Account account = accountRepository.findOne(adsAccount.getAccount());
        if(StringUtil.isEmpty(ids))return new ResultModel(999, "ids为空");
        List<Long> longs = StringUtil.transString2LongList(ids);
        String ads = "ads项目";
        if(!StringUtil.isEmpty(name)&&longs.size()==1){
            String oldName = campaignService.findById(longs.get(0)).getName();
            boolean result = this.campaignService.updateCampaignName(app, longs.get(0), name);
            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), ads+LogEnumType.UPDATE.getName() + "推广活动名称", String.format("%s改为%s", oldName, name), app, OperateObjectEnumType.CAMPAIGN.getCode(), request);
            if(result)return ResultModel.OK(result);
        }else if(delFlag!=null){
            for (Long id : longs) {
                if(delFlag){
                    Campaign result = this.campaignService.delete(app, id, account.getId());
                    userLogService.insertLog(account, LogEnumType.DISABLE.getCode(), ads+LogEnumType.DISABLE.getName() + "推广活动" + result.getName(), result, app, OperateObjectEnumType.CAMPAIGN.getCode(), request);
                }else{
                    Campaign result = this.campaignService.enableCampaign(app, id, account.getId());
                    userLogService.insertLog(account, LogEnumType.ENABLE.getCode(), ads+LogEnumType.ENABLE.getName() + "推广活动" + result.getName(), result, app, OperateObjectEnumType.CAMPAIGN.getCode(), request);
                }
            }
            return ResultModel.OK(true);
        }

        return new ResultModel(999, "false");
    }

    public static void main(String[] args) {
        String pw = CipherUtil.generatePassword("111111l");
        System.out.println(CipherUtil.generatePassword(pw + FLAG));
    }
}
