package com.reyun.controller.manage;

import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.Account;
import com.reyun.model.AccountRestrict4Web;
import com.reyun.model.PackageType;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AccountRestrict4WebRepository;
import com.reyun.repository.PackageTypeRepository;
import com.reyun.service.AccountFlowRestrictService;
import com.reyun.service.AutoTodayFlowService;
import com.reyun.task.AutoCheckBlackListAppTask;
import com.reyun.util.ResultModel;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigInteger;


/**
 * Created by song on 2017/10/19.
 */

@Controller
@RequestMapping("/mng/accountRestrict")
public class MngAccountFlowController {

    Logger logger = LoggerFactory.getLogger(MngAccountFlowController.class);

    @Autowired
    private AccountRestrict4WebRepository accountRestrict4WebRepository;
    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private AccountFlowRestrictService accountFlowRestrictService;
    @Autowired
    AutoCheckBlackListAppTask autoCheckBlackListAppTask;

    @Autowired
    AutoTodayFlowService autoTodayFlowService;

    @Autowired
    PackageTypeRepository packageTypeRepository;

    @RequestMapping(value = "find/{accountId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel list(@PathVariable Long accountId, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

//        long superAccountId = accountRepository.findRootParentByAccountId(accountId).longValue();

        Account rootAccount = accountRepository.findRootAccountById(accountId);
        AccountRestrict4Web byAccountId = accountRestrict4WebRepository.findByAccountId(rootAccount.getId());

        if (byAccountId != null && (byAccountId.getPackageFlowId() == null && rootAccount.getPackageFlowId() != null)) {
            //重新初始化用户流量
            accountRestrict4WebRepository.delete(byAccountId);
            byAccountId = null;
        }

        if (byAccountId == null) {

            Account account = new Account();
            account.setId(accountId);
            byAccountId = accountFlowRestrictService.RestrictFlowByAccountTask(account, lanType);

            logger.info("初始化用户流量,时间 {},账号id {}", new DateTime().toString("yyyy-MM-dd HH:mm:ss"), rootAccount.getId());
        }

        if (byAccountId != null) {
            //设置为 office 中设置的流量提醒开启关闭值
            byAccountId.setFlowRestrict(rootAccount.getFlowRestrict());
        }

        Account account = accountRepository.findOne(accountId);
        boolean isSuperUser = account.getIsSuperUser();
        if (!isSuperUser) {
            byAccountId.setIOPastTimeNotified(false);
            byAccountId.setTrackPastTimeNotified(false);
            byAccountId.setTrackFlowNotified(false);
            byAccountId.setIOFlowNotified(false);
        }
        //国际化处理
        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            byAccountId = accountFlowRestrictService.toEnglish(byAccountId);
        }

//        if(byAccountId.getIncrementFlow()!=null){
        return ResultModel.OK(accountFlowRestrictService.getAccountUsedFlows(byAccountId, rootAccount));
    }


    @RequestMapping(value = "confirm", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel confirm(@RequestBody AccountRestrict4Web resource) {
        long superAccountId = accountRepository.findRootParentByAccountId(resource.getAccountId()).longValue();
        AccountRestrict4Web byAccountId1 = accountRestrict4WebRepository.findByAccountId(superAccountId);
        if (resource.getTrackPastTime() != null && resource.getTrackPastTime()) {
            byAccountId1.setTrackPastTimeNotified(false);
        }
        if (resource.getIoPastTime() != null && resource.getIoPastTime()) {
            byAccountId1.setIOPastTimeNotified(false);
        }
        if (resource.getTrackFlow() != null && resource.getTrackFlow()) {
            byAccountId1.setTrackFlowNotified(false);
        }
        if (resource.getIoFlow() != null && resource.getIoFlow()) {
            byAccountId1.setIOFlowNotified(false);
        }
        accountRestrict4WebRepository.save(byAccountId1);
        return ResultModel.OK();
    }

    @RequestMapping(value = "apiflow", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel confirm(String email) {

        if (StringUtils.isEmpty(email)) {
            return new ResultModel(207, "用户账号不能为空", "");
        }

        return accountFlowRestrictService.findRestrictByAccountWithoutEmail(email);

    }


    @RequestMapping(value = "/mail/test", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel mailTest(Account account) {

        //测试流量提醒邮件
        accountFlowRestrictService.RestrictFlowByAccountTask(account, "cn");
        autoCheckBlackListAppTask.run();
        return ResultModel.OK();

    }
}
