package com.reyun.controller.manage;

import com.reyun.model.Account;
import com.reyun.model.AdNameSetting;
import com.reyun.model.AdvertisingBinduser;
import com.reyun.repository.AdvertisingBinduserRepository;
import com.reyun.repository.AdvertisingPlatformRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AdvertisingService;
import com.reyun.task.AdvertisingApiTask;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/mng/advertising")
public class MngAdvertisingController {
    protected Logger logger = LoggerFactory.getLogger(MngAdvertisingController.class);

    @Autowired
    private AdvertisingPlatformRepository advertisingPlatformRepository;
    @Autowired
    private AdvertisingService advertisingService;
    @Autowired
    private AdvertisingBinduserRepository advertisingBinduserRepository;
    @Autowired
    private AdvertisingApiTask advertisingApiTask;


    @RequestMapping(value = "platform", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel platform() {

        List<Map<String, String>> data = advertisingPlatformRepository.findNameAndType().stream().map(v -> {
            Map<String, String> item = new HashMap(2);
            item.put("name", v[0].toString());
            item.put("id", v[1].toString());
            return item;
        }).collect(Collectors.toList());

        return ResultModel.OK(data);
    }

    @RequestMapping(value = "upload/{id}/{appid}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel Upload(@CurrentAccount Account loginAccount,
                              @RequestParam("file") MultipartFile file,
                              @PathVariable("id") String id, @PathVariable("appid") Long appid) {


        if (file == null) {
            return ResultModel.ERROR(ResultStatus.FILE_EMPFFTY);
        }

        String originalName = file.getOriginalFilename();

        if (originalName.lastIndexOf(".xls") == -1 && originalName.lastIndexOf(".xlsx") == -1 && originalName.lastIndexOf(".csv") == -1) {
            return ResultModel.ERROR(ResultStatus.FILE_TYPE_ERROR);
        }

        ResultStatus status = null;

        if (originalName.toLowerCase().endsWith(".csv")) {

            status = advertisingService.importDataWithCSV(id, appid, file);

        } else {

            if ("mintegral".equals(id)) {
                status = advertisingService.importDataOnCSV(id, appid, file);
            } else {
                status = advertisingService.importDataWithExcel(id, appid, file);

            }

        }

        if (status == null) {
            return ResultModel.OK();
        }

        return ResultModel.ERROR(status);

    }


    @RequestMapping(value = "/adname", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel adNames(String appid, String adType, String searchName, String cid) {

        List result = advertisingService.getAdNamesList(appid, adType, searchName, cid);
        return ResultModel.OK(result);
    }

    @RequestMapping(value = "/adchannel/settings", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel adChannels() {

        List result = advertisingService.adChannels();
        return ResultModel.OK(result);
    }

    @RequestMapping(value = "/binduser/count", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getBinduserCount(@CurrentAccount Account loginAccount, Long app) {

        return ResultModel.OK(advertisingBinduserRepository.getCount(loginAccount.getRootParent(), app));
    }

    @RequestMapping(value = "/binduser/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel saveBinduser(@CurrentAccount Account loginAccount, @RequestBody AdvertisingBinduser resourse) {
        AdvertisingBinduser save = null;
        //修改
        if (resourse.getId() != null) {
            AdvertisingBinduser one = advertisingBinduserRepository.findOne(resourse.getId());
            long id = one.getId().longValue();
            if ("csj".equals(resourse.getTypeName())) {
                AdvertisingBinduser byUserId = advertisingBinduserRepository.getByUserId(resourse.getApp(), resourse.getUserId());
                if (byUserId != null && id != byUserId.getId().longValue()) {
                    return ResultModel.ERROR("该user_id已经绑定，请检查");
                }
            } else if ("topon".equals(resourse.getTypeName())) {
                AdvertisingBinduser bySecure_key = advertisingBinduserRepository.getBySecure_key(resourse.getApp(), resourse.getSecureKey());
                if (bySecure_key != null && id != bySecure_key.getId().longValue()) {
                    return ResultModel.ERROR("该publisher_key已经绑定，请检查");
                }
            } else if ("adtiming".equals(resourse.getTypeName())) {
                AdvertisingBinduser bySecure_key = advertisingBinduserRepository.getBySecure_key(resourse.getApp(), resourse.getSecureKey());
                if (bySecure_key != null) {
                    return ResultModel.ERROR("该api_key已经绑定，请检查");
                }
            }
            one.setUpdate_time(new Date());
            one.setSecureKey(resourse.getSecureKey());
            one.setUserId(resourse.getUserId());
            save = advertisingBinduserRepository.save(one);
        } else {
            //新建
            if ("csj".equals(resourse.getTypeName())) {
                AdvertisingBinduser byUserId = advertisingBinduserRepository.getByUserId(resourse.getApp(), resourse.getUserId());
                if (byUserId != null) {
                    return ResultModel.ERROR("该user_id已经绑定，请检查");
                }
            } else if ("topon".equals(resourse.getTypeName())) {
                AdvertisingBinduser bySecure_key = advertisingBinduserRepository.getBySecure_key(resourse.getApp(), resourse.getSecureKey());
                if (bySecure_key != null) {
                    return ResultModel.ERROR("该publisher_key已经绑定，请检查");
                }
            } else if ("adtiming".equals(resourse.getTypeName())) {
                AdvertisingBinduser bySecure_key = advertisingBinduserRepository.getBySecure_key(resourse.getApp(), resourse.getSecureKey());
                if (bySecure_key != null) {
                    return ResultModel.ERROR("该api_key已经绑定，请检查");
                }
            }

            resourse.setRootAccount(loginAccount.getRootParent());
            resourse.setStatus(true);
            save = advertisingBinduserRepository.save(resourse);
        }
        return ResultModel.OK(save);
    }

    @RequestMapping(value = "/binduser/openandclose", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel openAndClose(@CurrentAccount Account loginAccount, @RequestBody AdvertisingBinduser resourse) {
        AdvertisingBinduser one = advertisingBinduserRepository.findOne(resourse.getId());
        one.setUpdate_time(new Date());
        one.setStatus(resourse.getStatus());
        return ResultModel.OK(advertisingBinduserRepository.save(one));
    }

    @RequestMapping(value = "/binduser/list", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel list(@CurrentAccount Account loginAccount, Long app) {
        return ResultModel.OK(advertisingBinduserRepository.list(loginAccount.getRootParent(), app));
    }

    @RequestMapping(value = "/binduser/delete", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel delete(@CurrentAccount Account loginAccount, Long id) {
        AdvertisingBinduser one = advertisingBinduserRepository.findOne(id);
        advertisingBinduserRepository.delete(one);
        return ResultModel.OK(true);
    }

    @RequestMapping(value = "/adrename/list", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel adRenamelist(@CurrentAccount Account loginAccount, Long app) {

        List<String> names = Arrays.asList("Banner", "信息流", "激励视频", "开屏", "插屏", "全屏视频", "短视频信息流", "大横幅", "视频贴片");
        List<Map> result = new ArrayList<>();
        for (int i = 0; i < names.size(); i++) {
            String nameOne = names.get(i);
            Map data = new HashMap();
            data.put("id", i + 1);
            data.put("name", nameOne);
            result.add(data);
        }
        return ResultModel.OK(result);
    }

    @RequestMapping(value = "/adrename/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel saveBinduser(@CurrentAccount Account loginAccount, @RequestBody AdNameSetting adName) {

        if (loginAccount != null && loginAccount.getEmail().equals("demo@reyun.com")) {
            return ResultModel.OK();
        }

        advertisingService.saveAdRenameData(adName);
        return ResultModel.OK();

    }

    @RequestMapping(value = "/syncdata", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel syncdata(Long id, String startDate, String endDate) {
        AdvertisingBinduser one = advertisingBinduserRepository.findOne(id);
        //auto 为falase时使用自定义时间,ture时一律是昨日
        one.setAuto(false);
        one.setStartDate(startDate);
        one.setEndDate(endDate);
        String typeName = one.getTypeName();
        if ("csj".equals(typeName)) {
            advertisingApiTask.run(one);
            return ResultModel.OK();
        } else if ("topon".equals(typeName) || "adtiming".equals(typeName)) {
            advertisingApiTask.run4Topon(one);
            return ResultModel.OK();
        }
        return ResultModel.ERROR("不支持该平台");
    }


    @RequestMapping(value = "/advertiserid/list", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel advertiseridList(@CurrentAccount Account account, HttpServletRequest request) {

        return ResultModel.OK(advertisingService.advertiseridByReport(request, account));

    }

}
