package com.reyun.controller.manage;

import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.repository.AppRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AppService;
import com.reyun.service.SdkDebugService;
import com.reyun.service.UserLogService;
import com.reyun.service.impl.AppServiceImpl;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping("/mng/app")
public class MngAppController {
    protected Logger logger = LoggerFactory.getLogger(MngAppController.class);

    @Autowired
    private AppService appService;

    @Autowired
    private UserLogService userLogService;

    @Autowired
    private SdkDebugService sdkDebugService;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private AppServiceImpl appServiceImpl;


    @RequestMapping(value = "find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel list(HttpServletRequest request, @CurrentAccount Account loginAccount) {
//		userLogService.insertLog(loginAccount.getId(), LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "应用列表", null);
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "应用列表", null, null, OperateObjectEnumType.APP.getCode());
        return ResultModel.OK(appService.listAppByAccount(loginAccount.getId()));
    }

    @RequestMapping(value = "find/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listone(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        App app = appService.findById(id);
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "应用" + app.getName(), app, id, OperateObjectEnumType.APP.getCode());
        return ResultModel.OK(app);
    }

    @RequestMapping(value = "find/getcategorys", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listCategory(HttpServletRequest request, @CurrentAccount Account loginAccount) {
        return ResultModel.OK(appService.listCategory());
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseStatus(HttpStatus.CREATED)
    @ResponseBody
    public ResultModel create(HttpServletRequest request, @RequestBody App resource, @CurrentAccount Account account) {

        boolean  appIsWxOrWg =resource != null &&(AppPlatformEnum.WX.getKey().equals(resource.getPlatform()) || AppPlatformEnum.WG.getKey().equals(resource.getPlatform()));

        boolean wxAppIdNoExist = resource != null &&  StringUtil.isEmpty(resource.getMipAppid());
        boolean wxAppSecretNoExist = resource != null && StringUtil.isEmpty(resource.getMipSecret());

        if(appIsWxOrWg && (wxAppIdNoExist || wxAppSecretNoExist)){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
        App app = appService.create(resource, account.getId());
        userLogService.insertLog(account, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "应用" + app.getName(), app, app.getId(), OperateObjectEnumType.APP.getCode());

        if(app.getErroMessage()!=null){
            return new ResultModel(-1001,app.getErroMessage());
        }

        return ResultModel.OK(app);
    }

    @RequestMapping(value = "update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel update(HttpServletRequest request, HttpServletResponse response,
                              @RequestBody App resource, @CurrentAccount Account account) {
        boolean  appIsWxOrWg =resource != null &&(AppPlatformEnum.WX.getKey().equals(resource.getPlatform()) || AppPlatformEnum.WG.getKey().equals(resource.getPlatform()));

        boolean wxAppIdNoExist = resource != null &&  StringUtil.isEmpty(resource.getMipAppid());
        boolean wxAppSecretNoExist = resource != null && StringUtil.isEmpty(resource.getMipSecret());

        if(appIsWxOrWg && (wxAppIdNoExist || wxAppSecretNoExist)){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
        App app = appService.update(resource, account.getId());
        
        if(app.getErroMessage()!=null){
            return new ResultModel(-1001,app.getErroMessage());
        }

        userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "应用" + app.getName(), app, app.getId(), OperateObjectEnumType.APP.getCode());
        return ResultModel.OK(app);
    }

    @RequestMapping(value = "enable/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enable(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account account) {

        try {
            App app = appService.enable(id, account.getId());
            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), "启用应用" + app.getName(), app, app.getId(), OperateObjectEnumType.APP.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "forbidden/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel forbidden(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account account) {

        try {
            App app = appService.forbidden(id, account.getId());
            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), "停用应用" + app.getName(), app, app.getId(), OperateObjectEnumType.APP.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "valid", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@RequestParam String name, @CurrentAccount Account account) {
        Boolean flag = appService.validAppleId(account.getId(), name);
        if (!flag) {
            return ResultModel.OK(false);
        } else {
            return ResultModel.OK(true);
        }
    }

    @RequestMapping(value = "validdel/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel validDel(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        App app = appService.findById(id);
        if (app.getDelFlag()) {
            return ResultModel.OK(true);
        } else {
            return ResultModel.OK(false);
        }
    }

    @RequestMapping(value = "validname", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkName(@RequestParam String name, @CurrentAccount Account account) {
        Boolean flag = appService.validName(account.getId(), name);
        if (!flag) {
            return ResultModel.OK(false);
        } else {
            return ResultModel.OK(true);
        }
    }

    @RequestMapping(value = "delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResultModel delete(HttpServletRequest request, HttpServletResponse response, @PathVariable Long id, @CurrentAccount Account account) {
        try {
            App app = appService.delete(id, account.getId());
            String cookieId = getCookieToken(request.getCookies(), "ryioLastApp");
            if (!StringUtil.isEmpty(cookieId) && id.toString().equals(cookieId)) {
                Cookie cookie = new Cookie("ryioLastApp", null);
                cookie.setPath("/");
                cookie.setMaxAge(0);
                response.addCookie(cookie);
            }
            userLogService.insertLog(account, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "应用" + app.getName(), app, app.getId(), OperateObjectEnumType.APP.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    private String getCookieToken(Cookie[] cookies, String name) {

        String token = null;

        if (null != cookies) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals(name)) {
                    token = cookie.getValue();
                }
            }
        }

        return token;
    }

    /**
     * 查询APP的所有事件
     *
     * @param appId LONG
     * @return ResultModel
     */
    @RequestMapping(value = "debug/event/list/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllDebugEvent(@PathVariable Long appId) {
        return ResultModel.OK(sdkDebugService.getAllEventsByApp(appId));
    }

    @RequestMapping(value = "debug/log/list/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllDebugEventLog(@PathVariable Long appId) {
        return ResultModel.OK(sdkDebugService.getAllEventLogByApp(appId));
    }

    @RequestMapping(value = "debug/device/list/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getEventTotalInfo(@PathVariable Long appId) {
        return ResultModel.OK(sdkDebugService.getSdkDebugDeviceCount(appId));
    }

    @RequestMapping(value = "debug/find/common", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getCommonEventAttr() {
        return ResultModel.OK(sdkDebugService.getCommonEventAttr());
    }

    @RequestMapping(value = "debug/complete/{appId}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel completeAppSdkDebug(@CurrentAccount Account loginAccount, @PathVariable Long appId) {
        return ResultModel.OK(sdkDebugService.appSdkDebugComplete(loginAccount, appId));
    }

    @RequestMapping(value = "update/sort/{appidBefore}/{appidAfter}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel updateSort(@CurrentAccount Account loginAccount, @PathVariable Long appidBefore, @PathVariable Long appidAfter) {
        appService.updateSort(loginAccount, appidBefore, appidAfter);
        return ResultModel.OK();
    }

    @RequestMapping(value = "update/sort/{isAsc}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel updateSort(@CurrentAccount Account loginAccount, @PathVariable Integer isAsc) {
        appService.sort(loginAccount, isAsc);
        return ResultModel.OK();
    }

    /**
     * 功能描述:app维度的横幅提醒
     * @author liyin
     * @date 2020/7/6
     */
    @RequestMapping(value = "tipRemind/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel tipRemind(@PathVariable Long id,@CurrentAccount Account loginAccount) {
        Map<String, Object> result = new HashMap();
        Map<String, Object> appOnlineService = appService.tipRemind(id,loginAccount);
        result.put("appOnlineService", appOnlineService);
        return ResultModel.OK(result);
    }
    /**
     * 功能描述:
     * @author liyin
     * @date 2020/7/6
     */
    /**
     * 功能描述:关闭天调起到期提示
     *
     * @author liyin
     * @date 2019/7/18
     */
    @RequestMapping(value = "todayCloseTip/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel todayCloseTip( @PathVariable Long id) {

        return ResultModel.OK(appService.todayCloseTip(id));
    }
    /**
     * @param request
     * @param response
     * @param appkey
     * @return 根据appkey获取微信的appid与appSecet
     * (备注：此处只会传入小游戏与小程序的appkey)
     *     只有存在数据的时候code=200
     */
//    @RequestMapping(value = "findWxMes")
//    @ResponseBody
//    public ResultModel findWxMes(HttpServletRequest request, HttpServletResponse response,
//                             String appkey) {
//        App app = appRepository.findByAppkey(appkey);
//        Map<String, String> result = new HashMap<>();
//        boolean wxAppidExist = app != null && !StringUtil.isEmpty(app.getMipAppid());
//        boolean wxSecretExist = app != null && !StringUtil.isEmpty(app.getMipSecret());
//
//        if(wxAppidExist && wxSecretExist){
//            result.put("appid", appkey);
//            result.put("wx_appid", app.getMipAppid());
//            result.put("wx_secret", app.getMipSecret());
//            //同步数据到ddb
//            appServiceImpl.synWxApp(app);
//            return ResultModel.OK(result);
//        }else{
//            return ResultModel.ERROR(ResultStatus.FAILED);
//        }
//    }
}
