package com.reyun.controller.manage;

import com.amazonaws.services.dynamodbv2.xspec.S;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.Account;
import com.reyun.model.AccountFunctionAuth;
import com.reyun.model.RoleAuth;
import com.reyun.model.ThirdAccountAuth;
import com.reyun.repository.AccountFunctionAuthRepository;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AuthRepository;
import com.reyun.repository.ThirdAccountAuthRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AuthService;
import com.reyun.service.UserLogService;
import com.reyun.util.*;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/mng/auth")
public class MngAuthController {

    protected Logger logger = LoggerFactory.getLogger(MngAuthController.class);

    private static final int CUSTOM_ROLE_NAME_LENGTH = 8;

    private static final int STATUS_INACTIVATE = -3;

    @Autowired
    private AuthService authService;

    @Autowired
    private UserLogService userLogService;

    @Autowired
    private AuthRepository authRepository;

    @Autowired
    private ThirdAccountAuthRepository thirdAccountAuthRepository;
    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private AccountFunctionAuthRepository accountFunctionAuthRepository;

    @RequestMapping(value = "getAuth", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAuth(Long account) {
        HashMap<String, Boolean> rs = new HashMap<>();
        List<AccountFunctionAuth> accountFunctionAuths = this.accountFunctionAuthRepository.getByAccount(accountRepository.findOne(account).getRootParent());
        for (AccountFunctionAuth accountFunctionAuth : accountFunctionAuths) {
            rs.put(accountFunctionAuth.getMenu(),true);
        }
        return ResultModel.OK(rs);
    }

    /**
     * 查询所有的非管理员和子应用管理员的子账号和渠道账号。
     */
    @RequestMapping(value = "find/{appId}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllSubAccountAndChannel(@CurrentAccount Account loginAccount, @PathVariable Long appId) {
        return ResultModel.OK(authService.getAuthorizedSubAndChannelAccount(loginAccount, appId));
    }

    @RequestMapping(value = "findchannel/{appId}/{channel}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllCampaignAccountByAppChannel(@PathVariable Long appId, @PathVariable Long channel) {
        return ResultModel.OK(authRepository.findAllcampaignAccountByChanelApp(appId, channel));
    }

    /**
     * 新建渠道账号时查询账号信息
     */
    @RequestMapping(value = "channel/find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAccountInfo(@CurrentAccount Account loginAccount, @RequestParam String email) {

        Account channelAccount = authService.getChannelAccountInfoByEmail(loginAccount, email);
        String nowDate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);

        if (null != channelAccount && channelAccount.getIsChannelPerson()) {

            if (channelAccount.getStatus() == STATUS_INACTIVATE) {
                //渠道账号未激活
                return ResultModel.ERROR(ResultStatus.ACCOUNT_NOT_ACTIVATE);
            } else if (DateUtil.compare_date(nowDate, channelAccount.getPastDate()) == 1) {
                //渠道账号已过期
                return ResultModel.ERROR(ResultStatus.ACCOUNT_BUSINESS_EXPIRE);
            } else if (!CollectionUtils.isEmpty(channelAccount.getAuthList())) {
                //已经授权过
                return ResultModel.ERROR(ResultStatus.RECORD_EXIST);
            }

        } else if (null != channelAccount && !channelAccount.getIsChannelPerson()) {
            //非渠道账号
            return ResultModel.ERROR(ResultStatus.FAILED);
        }

        return ResultModel.OK(channelAccount);
    }

    /**
     * 根据角色查询菜单权限
     * roleId 1:管理员,2:子应用管理员,N:自定义角色,5:渠道人员
     */
    @RequestMapping(value = "{roleId}/findMenu", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllMenuByRole(@CurrentAccount Account loginAccount, @PathVariable Long roleId ,@RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        return ResultModel.OK(authService.getMenuAthByRole(loginAccount, roleId, lanType));
    }

    /**
     * 线上权限菜单处理
     *
     */
    @RequestMapping(value = "dealauth", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllAuth() {
        authService.dealWithOldRoleDetail();
        return ResultModel.OK();
    }

    /**
     * 查询所有的自定义角色
     */
    @RequestMapping(value = "role/find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllCustomRoleList(@CurrentAccount Account loginAccount) {
        return ResultModel.OK(authService.getAllCustomRoleList(loginAccount));
    }

    /**
     * 查询单个子账号信息
     * created by sunhao 20170605
     */
    @RequestMapping(value = "{subAccountId}/find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getSubAccountById(@CurrentAccount Account loginAccount, @PathVariable Long subAccountId) {
        return ResultModel.OK(authService.getSubAccountById(loginAccount, subAccountId));
    }

    /**
     * 查询所有启用子账户信息
     * created by sunhao 20170605
     */
    @RequestMapping(value = "list/enable", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllActiveSubAccount(@CurrentAccount Account loginAccount) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "启用状态子用户列表", null,
//                null, OperateObjectEnumType.ACCOUNT.getCode());
        List<Account> subAccounts = authService.getAllActiveSubAccount(loginAccount);
        subAccounts.addAll(authService.getAllActiveChannelAccount(loginAccount));


        return ResultModel.OK(subAccounts);

    }

    /**
     * 查询所有停用子账户信息
     * created by sunhao 20170605
     */
    @RequestMapping(value = "list/disable", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllDisableSubAccount(@CurrentAccount Account loginAccount) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "停用状态子用户列表", null,
//                null, OperateObjectEnumType.ACCOUNT.getCode());

        List<Account> subAccounts = authService.getAllDisableSubAccount(loginAccount);
        subAccounts.addAll(authService.getAllDisableChannelAccount(loginAccount));
        return ResultModel.OK(subAccounts);
    }

    /**
     * 启用子账号
     * created by sunhao 20170605
     */
    @RequestMapping(value = "{subAccountId}/enable", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enableSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long subAccountId) {

        Account subAccount = authService.getSubAccountInfo(subAccountId);

        if (RoleEnumType.CHANNEL_PERSON.getKey().equals(subAccount.getRoleCategory())) {
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "启用渠道账号", subAccount,
                    null, OperateObjectEnumType.ACCOUNT.getCode(), request);

            return ResultModel.OK(authService.enableChannelAccountById(loginAccount, subAccountId));
        } else {
            //启用管理员账号时判断已开启的是否超过3个
            if (subAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) && authService.checkSubMangerNumber(loginAccount)) {
                return ResultModel.ERROR(ResultStatus.NUMBER_LIMIT);
            }
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "启用子账号", subAccount,
                    null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            return ResultModel.OK(authService.enableSubAccountById(loginAccount, subAccountId));
        }


    }

    /**
     * 禁用子账号
     * created by sunhao 20170605
     */
    @RequestMapping(value = "{subAccountId}/disable", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel disableSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long subAccountId) {

        Account subAccount = authService.getSubAccountInfo(subAccountId);
        if (RoleEnumType.CHANNEL_PERSON.getKey().equals(subAccount.getRoleCategory())) {
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "禁用渠道账号", subAccount,
                    null, OperateObjectEnumType.ACCOUNT.getCode(), request);

            return ResultModel.OK(authService.disableChannelAccountById(loginAccount, subAccountId));
        } else {
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "禁用子账号", subAccount,
                    null, OperateObjectEnumType.ACCOUNT.getCode(), request);

            return ResultModel.OK(authService.disableSubAccountById(loginAccount, subAccountId));
        }


    }

    /**
     * 删除未激活的子账号
     * created by sunhao 20170609
     */
    @RequestMapping(value = "{subAccountId}/delete", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long subAccountId) {

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.DELETE.getName() + "子账号", authService.getSubAccountInfo(subAccountId),
                null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.deleteInactivateSubAccount(subAccountId));
    }

    /**
     * 创建子账号
     * created by sunhao 20170606
     */
    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ResponseBody

    public ResultModel createSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount,
                                        @RequestBody Account subAccount,
                                        @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        //校验参数
        if (StringUtil.isEmpty(subAccount.getEmail()) || StringUtil.isEmpty(subAccount.getName()) || null == subAccount.getRoleCategory()) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验角色
        RoleAuth roleAuth = authService.getRoleAuthById(subAccount.getRoleCategory());

        if (null == roleAuth) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //是否是渠道账号
        if (RoleEnumType.CHANNEL_PERSON.getKey().equals(subAccount.getRoleCategory())) {
            //校验邮箱是否存在

            if (!authService.checkChannelEmailExists(subAccount.getEmail())) {
                return ResultModel.ERROR(ResultStatus.CHANNELNAME_EXIST);
            }
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道账号:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            return ResultModel.OK(authService.createChannelAccount(loginAccount, subAccount, lanType));
        } else {
            //校验邮箱是否存在
            if (authService.checkEmailExists(subAccount.getEmail())) {
                return ResultModel.ERROR(ResultStatus.NAME_EXIST);
            }

            //校验管理员数量是否超量     update by zhangyao  2018-11-19
            if (subAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) && authService.checkSubMangerNumber(loginAccount)) {
                return ResultModel.ERROR(ResultStatus.NUMBER_LIMIT);
            }
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "子账号:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            return ResultModel.OK(authService.createSubAccount(loginAccount, subAccount, roleAuth, lanType));
        }


    }

    /**
     * 配置自定义权限子账号权限
     * created by zxy 18-03-01
     */
    @RequestMapping(value = "createauth", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createSubAccountAuth(HttpServletRequest request, @CurrentAccount Account loginAccount, @RequestBody Account subAccount) {

        //校验角色
        RoleAuth roleAuth = authService.getRoleAuthById(subAccount.getRoleCategory());

        if (null == roleAuth) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }


        boolean result = false;

        if (RoleEnumType.CHANNEL_PERSON.getKey().equals(subAccount.getRoleCategory())) {
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道账号权限:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            result = authService.saveChannelAuthAndDataAuth(subAccount.getAuthList(), loginAccount, subAccount);
        } else {
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "子账号权限:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            result = authService.modifySubAccountAuth(loginAccount, subAccount, roleAuth);
        }

        List<ThirdAccountAuth> thirdAccountAuth = subAccount.getThirdAccountAuth();
        int i = this.thirdAccountAuthRepository.deleteByAccount(subAccount.getId());
        if(!CollectionUtils.isEmpty(thirdAccountAuth)){
            this.thirdAccountAuthRepository.save(thirdAccountAuth);
        }

        return ResultModel.OK(result);
    }



    /**
     * 修改子账号
     * created by sunhao 20170606
     */
    @RequestMapping(value = "{subAccountId}/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel modifySubAccount(HttpServletRequest request,@CurrentAccount Account loginAccount, @PathVariable Long subAccountId, @RequestBody Account subAccount) {

        RoleAuth roleAuth = authService.getRoleAuthById(subAccount.getRoleCategory());

        //参数校验name
        if (StringUtils.isEmpty(subAccount.getName()) || null == roleAuth) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验子账号管理员数量是否超过
        if (authService.checkUpdateSubMangerNumber(loginAccount, subAccount)) {
            return ResultModel.ERROR(ResultStatus.NUMBER_LIMIT);
        }

        subAccount.setId(subAccountId);
        if (RoleEnumType.CHANNEL_PERSON.getKey().equals(subAccount.getRoleCategory())) {
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            return ResultModel.OK(authService.modifyChannelAccount(loginAccount, subAccount));
        } else {
            //记录日志
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "子账号:" + subAccount.getEmail(),
                    subAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);
            return ResultModel.OK(authService.modifySubAccount(loginAccount, subAccount, roleAuth));
        }


    }




    /**
     * 创建自定义角色
     * created by sunhao 20170606
     */
    @RequestMapping(value = "role/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createCustomRole(HttpServletRequest request, @CurrentAccount Account loginAccount, @RequestBody RoleAuth roleAuth) {

        //校验参数
        if (StringUtils.isEmpty(roleAuth.getRoleName()) || roleAuth.getRoleName().length() > CUSTOM_ROLE_NAME_LENGTH
                || CollectionUtils.isEmpty(roleAuth.getRoleAuthDetails())) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验名字是否重复
        if (authService.checkCustomRoleName(loginAccount, roleAuth.getRoleName(), null)) {
            return ResultModel.ERROR(ResultStatus.NAME_EXIST);
        }

        //校验角色数量是否超过
        if (authService.checkCustomRoleNumber(loginAccount)) {
            return ResultModel.ERROR(ResultStatus.NUMBER_LIMIT);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "定义角色:" + roleAuth.getRoleName(),
                roleAuth, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.createCustomRole(loginAccount, roleAuth));
    }

    /**
     * 修改自定义角色
     * created by sunhao 20170606
     */
    @RequestMapping(value = "role/{roleId}/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel modifyCustomRole(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long roleId, @RequestBody RoleAuth roleAuth) {

        roleAuth.setId(roleId);

        //校验是否修改名字,是否名字重复
        if (authService.checkCustomRoleName(loginAccount, roleAuth.getRoleName(), roleId)) {
            return ResultModel.ERROR(ResultStatus.NAME_EXIST);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "定义角色:" + roleAuth.getRoleName(),
                roleAuth, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.modifyCustomRole(loginAccount, roleAuth));
    }

    /**
     * 删除自定义角色
     * created by sunhao 20170607
     */
    @RequestMapping(value = "role/{roleId}/delete", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteCustomRole(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long roleId) {

        //校验此角色是否有新建的子账号,有则无法删除
        List<String> emails = authService.checkCustomRoleAccount(roleId);
        if (emails.size() > 0) {
            Map<String, Object> result = new HashMap<>();
            result.put("status", ResultStatus.PARAM_INVALID);
            result.put("emails", String.join(",", emails));
            return ResultModel.OK(result);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "定义角色:" + roleId,
                roleId, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.deleteCustomRole(loginAccount, roleId));
    }


    /**
     * 查询所有启用渠道账户信息
     * created by sunhao 20170607
     */
    @RequestMapping(value = "channel/list/enable", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllActiveChannelSubAccount(@CurrentAccount Account loginAccount) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "启用状态渠道用户列表", null,
//                null, OperateObjectEnumType.ACCOUNT.getCode());

        return ResultModel.OK(authService.getAllActiveChannelAccount(loginAccount));
    }

    /**
     * 查询所有停用渠道账户信息
     * created by sunhao 20170607
     */
    @RequestMapping(value = "channel/list/disable", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getAllDisableChannelSubAccount(@CurrentAccount Account loginAccount) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "停用状态渠道用户列表", null,
//                null, OperateObjectEnumType.ACCOUNT.getCode());

        return ResultModel.OK(authService.getAllDisableChannelAccount(loginAccount));
    }

    /**
     * 启用渠道账号
     * created by sunhao 20170612
     */
    @RequestMapping(value = "channel/{channelAccountId}/enable", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enableChannelSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long channelAccountId) {

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "启用渠道账号", authService.getSubAccountInfo(channelAccountId),
                null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.enableChannelAccountById(loginAccount, channelAccountId));
    }

    /**
     * 禁用渠道账号
     * created by sunhao 20170612
     */
    @RequestMapping(value = "channel/{channelAccountId}/disable", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel disableChannelSubAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long channelAccountId) {

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "禁用渠道账号", authService.getSubAccountInfo(channelAccountId),
                null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.disableChannelAccountById(loginAccount, channelAccountId));
    }

    /**
     * 创建渠道账号
     * created by sunhao 20170612
     */
    @RequestMapping(value = "channel/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createChannelAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @RequestBody Account channelAccount
            ,@RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        //校验参数
        if (StringUtil.isEmpty(channelAccount.getEmail()) || StringUtil.isEmpty(channelAccount.getName()) || null == channelAccount.getRoleCategory()) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验角色
        RoleAuth roleAuth = authService.getRoleAuthById(channelAccount.getRoleCategory());

        if (null == roleAuth || !RoleEnumType.CHANNEL_PERSON.getKey().equals(channelAccount.getRoleCategory())) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道账号:" + channelAccount.getEmail(),
                channelAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.createChannelAccount(loginAccount, channelAccount, lanType));
    }

    /**
     * 创建渠道账号
     * created by sunhao 20170612
     */
    @RequestMapping(value = "channel/createauth", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createChannelAccountAuth(HttpServletRequest request, @CurrentAccount Account loginAccount, @RequestBody Account channelAccount) {

        //校验参数
        if (StringUtil.isEmpty(channelAccount.getEmail()) || StringUtil.isEmpty(channelAccount.getName()) || null == channelAccount.getRoleCategory()) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验角色
        RoleAuth roleAuth = authService.getRoleAuthById(channelAccount.getRoleCategory());

        if (null == roleAuth || !RoleEnumType.CHANNEL_PERSON.getKey().equals(channelAccount.getRoleCategory())) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道账号权限:" + channelAccount.getEmail(),
                channelAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        return ResultModel.OK(authService.saveChannelAuthAndDataAuth(channelAccount.getAuthList(), loginAccount, channelAccount));
    }

    /**
     * 修改渠道账号信息
     * created by sunhao 20170612
     */
    @RequestMapping(value = "channel/{channelAccountId}/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel updateChannelAccount(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long channelAccountId, @RequestBody Account channelAccount) {

        //校验参数
        if (StringUtil.isEmpty(channelAccount.getEmail()) || StringUtil.isEmpty(channelAccount.getName()) || null == channelAccount.getRoleCategory()) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //校验角色
        RoleAuth roleAuth = authService.getRoleAuthById(channelAccount.getRoleCategory());

        if (null == roleAuth) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //记录日志
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号:" + channelAccount.getEmail(),
                channelAccount, null, OperateObjectEnumType.ACCOUNT.getCode(), request);

        channelAccount.setId(channelAccountId);
        return ResultModel.OK(authService.modifyChannelAccount(loginAccount, channelAccount));
    }

    /**
     * 修改渠道账号信息
     * created by sunhao 20170612
     */
    @RequestMapping(value = "mail/{subAccountId}/resend", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel resendSubAccountActiveEmail(@CurrentAccount Account loginAccount, @PathVariable Long subAccountId,
                                                   @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        Account subAccount = authService.getSubAccountInfo(subAccountId);

        if (null != subAccount) {
            return ResultModel.OK(authService.sendSubAccountActivateEmail(loginAccount, subAccount, lanType));
        } else {
            return ResultModel.OK(false);
        }
    }


    @RequestMapping(value = "/test/deal", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel dealAuth() {
        return ResultModel.OK(authService.dealWithOldAuth());
    }
}
