package com.reyun.controller.manage;

import com.google.common.base.Function;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.ChannelCategoryEnumType;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.*;
import com.reyun.service.impl.OnelinkService;
import com.reyun.taskexecute.UserLogThread;
import com.reyun.util.*;
import com.sun.jersey.core.util.Base64;
import org.apache.poi.hssf.record.aggregates.CFRecordsAggregate;
import org.mortbay.util.UrlEncoded;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;

@Controller
@RequestMapping("/mng/campaign")
public class MngCampaignController {
    private Logger logger = LoggerFactory.getLogger(MngCampaignController.class);

    @Autowired
    private CampaignService campaignService;

    @Autowired
    private SubCampaignService subCampaignService;
    @Autowired
    private SubCampaignRepository subCampaignRepository;

    @Autowired
    private OnelinkService onelinkService;
    @Autowired
    private OnelinkRepository onelinkRepository;

    @Autowired
    UserLogService userLogService;

    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private PackageSubCampaignService packageSubCampaignService;

    @Autowired
    private PackageSubCampaignRepository packageSubCampaignRepository;

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private ChannelAccountRepository channelAccountRepository;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private GdtPlanRepository gdtPlanRepository;

    @Autowired
    private ExportCampaignConfigRepository exportCampaignConfigRepository;

    @Autowired
    private AppDeepLinkRepository appDeepLinkRepository;

    @Autowired
    private AppExtendService appExtendServiceImpl;

    /**
     * 获取分包活动列表，母账号获取该App下的所有的活动，子账号获取与该账户关联的并且status是1的活动列表.此列表中包含账户自己创建的活动.
     * 每个活动，都处理了追踪短链.
     */
    @RequestMapping(value = "{appid}/find/package", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel findAllByPackage(@PathVariable Long appid, @CurrentAccount Account loginAccount, @RequestParam(value = "type", required = false) String type) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动列表",
//                null,appid,OperateObjectEnumType.CAMPAIGN.getCode());

        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appid, type, false);
        List<Campaign4Web> res = campaignService.listCampaignByPackage(result);

        return ResultModel.OK(res);
    }

    /**
     * 获取分包已停用的活动列表
     * 需求修改后:只要有推广活动管理权限,就能看到此APP下所有推广活动。
     * created by mxq 20170815
     */
    @RequestMapping(value = "{appId}/findDisable/package", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAllDisableByPackage(@PathVariable Long appId, @CurrentAccount Account loginAccount, @RequestParam(value = "type", required = false) String type) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动列表",
//                null, appId, OperateObjectEnumType.CAMPAIGN.getCode());


        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appId, type, true);
        List<Campaign4Web> res = campaignService.listCampaignByPackage(result);

        return ResultModel.OK(res);
    }

    /**
     * 获取分链接活动列表，母账号获取该App下的所有的活动，子账号获取与该账户关联的并且status是1的活动列表.此列表中包含账户自己创建的活动.
     * 每个活动，都处理了追踪短链.
     */
    @RequestMapping(value = "{appid}/find/link", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel findAllByLink(@PathVariable Long appid, @CurrentAccount Account loginAccount,
                                     @RequestBody Campaign campaign,
                                     @RequestParam(value = "type", required = false) String type) {
        Map data = new HashMap();
        campaign.setDelFlag(false);
        // 未删除
//        type = campaign.getType();
//        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount,appid,type,false);
        List<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appid, type, campaign, data);
//        List<Campaign4Web> res = campaignService.listCampaignByLink(result);
        data.put("data", result);
        //网眼定制需求 2116
        if(27289L==loginAccount.getRootParent()||38L==loginAccount.getRootParent()) {
            List<Campaign> byWangyanApp = campaignRepository.findByWangyanApp(appid);
            List<Long> ids = new ArrayList<>(byWangyanApp.size());
            for (Campaign cams : byWangyanApp) {
                ids.add(cams.getId());
            }
            for (Campaign4Web campaign4Web : result) {
                if(ids.contains(campaign4Web.getId())){
                    campaign4Web.setImpTrackurl("请以您系统生成的监测链接为准");
                    campaign4Web.setTrackurl("请以您系统生成的监测链接为准");
                    campaign4Web.setWangyan(true);
                }
            }
        }
            return ResultModel.OK(data);
    }

    /**
     * 获取渠道下拉列表
     *
     * @param appid
     * @param loginAccount
     * @param type
     * @return
     */
    @RequestMapping(value = "{appid}/find/linkchannels", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel findAllByLink(@PathVariable Long appid, @CurrentAccount Account loginAccount,
                                     @RequestParam(value = "type", required = false) String type) {

        List<Map> channels = campaignService.getChannelsNameAndId(appid, loginAccount, type);
        return ResultModel.OK(channels);
    }

    /**
     * 获取分链接已停用的活动列表
     * 需求修改后:只要有推广活动管理权限,就能看到此APP下所有推广活动。
     * created by sunhao 20170602
     */
    @RequestMapping(value = "{appId}/findDisable/link", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel findAllDisableByLink(@PathVariable Long appId, @CurrentAccount Account loginAccount,
                                            @RequestBody Campaign campaign,
                                            @RequestParam(value = "type", required = false) String type) {

        Map data = new HashMap();
        campaign.setDelFlag(true);
        List<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appId, type, campaign, data);
        data.put("data", result);
        return ResultModel.OK(data);
    }

    @RequestMapping(value = "/{appid}/find/bychannel/{channelid}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listCampaignByChannel(@PathVariable Long appid, @PathVariable Long channelid, @CurrentAccount Account loginAccount) {
        //userLogService.insertLog(loginAccount.getId(), LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动列表", null);
        return ResultModel.OK(campaignService.listCampaignByAppAndChannel(appid, channelid));
    }

    /**
     * 创建推广活动,将单个创建和批量创建合在一起
     * created by sunhao 20170519
     */
    @RequestMapping(value = "/{appId}/{isAutoExport}/create", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel createOneAndBatch(HttpServletRequest request, @PathVariable Long appId, @PathVariable Integer isAutoExport, @RequestBody Campaign resource, @CurrentAccount Account loginAccount) {
        List<Campaign> result = new ArrayList<>();
        resource.setAccount(loginAccount.getId());
        resource.setCreateAccount(loginAccount.getId());
        resource.setCreateTime(new Date());

        // vivo 渠道时 推广活动不保存 SpecialKeys
        resource.setVivo2DDb(false);
//        if(resource.getChannel().equals(767L) || resource.getChannel().equals(13296L)){
        if (resource.getChannel().equals(13296L)) {
            resource.setVivo2DDb(true);
        }

        //校验分包渠道相关参数
        if (!campaignService.checkPackChannle(resource, request)) {
            return ResultModel.ERROR("缺少渠道ID参数");
        }

        if (!StringUtils.isEmpty(resource.getSurl())) {
            //前端生成的short url
            result.add(campaignService.create(appId, resource));
            campaignService.createVivoParam(result, resource);
        } else {

            //后端生成的short url
            List<String> shortUrlList = campaignService.makesurls(appId, loginAccount, resource.getBatchNumber());

            if (!CollectionUtils.isEmpty(shortUrlList) && resource.getBatchNumber() == 1) {
                //单个创建推广活动
                resource.setSurl(shortUrlList.get(0));
                result.add(campaignService.create(appId, resource));
                campaignService.createVivoParam(result, resource);
            } else if (!CollectionUtils.isEmpty(shortUrlList) && resource.getBatchNumber() > 1) {
                //批量创建推广活动
                ArrayList<Campaign> campaigns = new ArrayList<>();
                ArrayList<String> campaignsName = new ArrayList<>();
                int startNumber = resource.getStartNumber();
                int batchNumber = resource.getBatchNumber();
                //String campaignName = resource.getCampaignName();
                String campaignName = resource.getName();
                int i = startNumber;
                while (i < batchNumber + startNumber) {
                    String name = campaignName + "_" + i;
                    Campaign cam = campaignRepository.findByNameAndAppIncludeDel(name, appId);
                    String s;
                    if (ValidateUtil.isValid(cam)) {
                        campaigns.add(cam);
                        campaignsName.add(cam.getName());
                    }
                    i++;
                }
                if (campaigns.size() > 0) {
                    return new ResultModel(ResultStatus.CAMPAIGN_REPEAT, campaignsName);
                } else {
                    result = campaignService.createBatch(appId, shortUrlList, resource);
                    campaignService.createVivoParam(result, resource);
                }
            } else {
                //失败
                return ResultModel.ERROR(ResultStatus.FAILED);
            }
        }
        if (loginAccount.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey()) || loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
            //是否自动导出Excel
            AppExtend appExtend = appExtendServiceImpl.findOneByAppid(appId);
            if (appExtend == null) {
                appExtend = new AppExtend();
            }
            appExtend.setAppid(appId);
            appExtend.setIsAutoExport(isAutoExport);
            appExtendServiceImpl.save(appExtend);
        }
        //日志
        UserLogThread userLogThread = new UserLogThread(loginAccount.getEmail(), loginAccount.getIsMasterLogin(), LogEnumType.CREATE.getCode(), String.format("%s%s%s%s", result.size() > 1 ? "批量" : "", LogEnumType.CREATE.getName(), "推广活动", resource.getName()), result, request, appId, OperateObjectEnumType.CAMPAIGN.getCode());
        userLogThread.start();
        return ResultModel.OK(result);
    }

    /**
     * 目前没有在使用
     *
     * @param appid
     * @param resource
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/{appid}/createbatch", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createBatch(@PathVariable Long appid, @RequestBody Campaign resource, @CurrentAccount Account loginAccount) {
        /*注: 需逗号分割surl字段, 获取多个短链*/
        logger.info("createBatch({}, {})", appid, resource);
        resource.setAccount(loginAccount.getId());
        resource.setCreateAccount(loginAccount.getId());
        resource.setCreateTime(new Date());

        Iterable<String> surlIter = Splitter.on(",").omitEmptyStrings().trimResults().split(resource.getSurl());
        return ResultModel.OK(this.campaignService.createBatch(appid, surlIter, resource));
    }

    @RequestMapping(value = "/{appid}/{isAutoExport}/{exportid}/update", method = RequestMethod.PUT)
    @ResponseBody
    public Object update(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long exportid, @PathVariable Integer isAutoExport, @RequestBody Campaign resource, @CurrentAccount Account loginAccount) {

        resource.setAccount(loginAccount.getId());
        resource.setModifyAccount(loginAccount.getId());
        resource.setModifyTime(new Date());

        if (Strings.isNullOrEmpty(resource.getName())) {
            return ResultModel.ERROR(ResultStatus.NAME_NULL);
        }
        if (resource.getName().contains("\t")) {
            return ResultModel.ERROR(ResultStatus.NAME_INVAILD);
        }
        if (!resource.getApp().equals(appid)) {
            return ResultModel.ERROR(ResultStatus.CAMPAIGN_NOT_MATCH_APP);
        }
        if (Strings.isNullOrEmpty(resource.getSurl())) {
            return ResultModel.ERROR(ResultStatus.SURL_NULL);
        }

        Campaign retrievedResource = this.campaignRepository.findOne(resource.getId());
        if (retrievedResource == null) {
            return ResultModel.ERROR(ResultStatus.CAMPAIGN_NULL);
        }

        if (!retrievedResource.getName().equalsIgnoreCase(resource.getName().trim())) {

            List<Campaign> retrievedResources = this.campaignRepository.findCampaignByAppAndName(appid, resource.getName().trim());
            if (retrievedResources != null && retrievedResources.size() > 0) {
                return ResultModel.ERROR(ResultStatus.NAME_EXIST);
            }
        }
        if (loginAccount.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey()) || loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
            //是否自动导出Excel
            AppExtend appExtend = appExtendServiceImpl.finOne(exportid);
            if (appExtend == null) {
                appExtend = new AppExtend();
            }
            appExtend.setAppid(appid);
            appExtend.setIsAutoExport(isAutoExport);
            appExtendServiceImpl.save(appExtend);
        }
        Campaign result = this.campaignService.update(loginAccount, appid, resource);
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广活动" + result.getName(), result, appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);
        return result;
    }

    @RequestMapping(value = "/{appid}/update/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public Object updateName(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long id,
                             @RequestParam String name, @CurrentAccount Account loginAccount) {
        String oldName = campaignService.findById(id).getName();
        boolean result = this.campaignService.updateCampaignName(appid, id, name);

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广活动名称", String.format("%s改为%s", oldName, name), appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);
        return result;
    }

    /**
     * 停用推广活动
     *
     * @param request
     * @param appid
     * @param id
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/{appid}/delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public Object delete(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        //onelink校验
        if (onelinkService.findOnelinkNumber(id) > 0) {
            return ResultModel.ERROR(ResultStatus.ONELINK_OCCUPY);
        }

        Campaign result = this.campaignService.delete(appid, id, loginAccount.getId());
        userLogService.insertLog(loginAccount, LogEnumType.DISABLE.getCode(), LogEnumType.DISABLE.getName() + "推广活动" + result.getName(), result, appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);

        return ResultModel.OK(result);
    }

    @RequestMapping(value = "/{appid}/enable/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enable(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long id, @CurrentAccount Account loginAccount) {

        Campaign campaign = this.campaignRepository.findOne(id);

        if (campaign == null) {
            return ResultModel.ERROR(ResultStatus.CAMPAIGN_NULL);
        }

        //启用分包的时候校验渠道是否已经存在
        if (ChannelCategoryEnumType.APPMARKET.getKey().equals(campaign.getChannelCategory())
                && null != campaignService.findPackChannelid(campaign.getPack_channelid(), appid)) {
            return ResultModel.ERROR(ResultStatus.CAMPAIGN_EXISTS);
        }

        Campaign result = this.campaignService.enableCampaign(appid, campaign.getId(), loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.ENABLE.getCode(), LogEnumType.ENABLE.getName() + "推广活动" + result.getName(), result, appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);

        return ResultModel.OK(result);
    }

    /**
     * 批量停用推广活动
     *
     * @param appid
     * @param ids
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/{appid}/batchdelete", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteBatch(HttpServletRequest request, @PathVariable Long appid, @RequestParam String ids, @CurrentAccount Account loginAccount) {
        Iterable<String> iterids = Splitter.on(",").trimResults().omitEmptyStrings().split(ids);
        Iterable<Long> campids = Iterables.transform(iterids,
                new Function<String, Long>() {
                    @Override
                    public Long apply(String s) {
                        return Long.parseLong(s);
                    }
                });

        //子活动校验
        Iterator<Long> iterator = campids.iterator();
        while (iterator.hasNext()) {
            if (subCampaignService.findSubCampaignNumber(iterator.next()) > 0) {
                return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
            }
        }

        String[] split = ids.split(",");
        List<Long> caids = new ArrayList<>(split.length);
        for (String s : split) {
            caids.add(Long.valueOf(s));
        }
        //子活动停用
        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.subCampaignInParent(caids);
        String pkgCampaignIds = "";
        if (!CollectionUtils.isEmpty(packageSubCampaigns)) {
            for (PackageSubCampaign packageSubCampaign : packageSubCampaigns) {
                pkgCampaignIds = pkgCampaignIds + packageSubCampaign.getId() + ",";
            }
            packageSubCampaignService.forbiden(pkgCampaignIds.substring(0, pkgCampaignIds.length() - 1), loginAccount.getId());
        }

        //onelink校验
        Iterator<Long> iterator1 = campids.iterator();
        while (iterator1.hasNext()) {
            if (onelinkService.findOnelinkNumber(iterator1.next()) > 0) {
                return ResultModel.ERROR(ResultStatus.ONELINK_OCCUPY);
            }
        }
        userLogService.insertLog(loginAccount, LogEnumType.DISABLE.getCode(), LogEnumType.DISABLE.getName() + "批量停用推广活动", ids, appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);
        return ResultModel.OK(this.campaignService.batchdelete(appid, campids, loginAccount.getId()));

    }

    @RequestMapping(value = "/{appid}/batchenable", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enableBatch(HttpServletRequest request, @PathVariable Long appid, @RequestParam String ids, @CurrentAccount Account loginAccount) {

        Iterable<String> iteratorIds = Splitter.on(",").trimResults().omitEmptyStrings().split(ids);
        Iterable<Long> campaignIds = Iterables.transform(iteratorIds, new Function<String, Long>() {
            @Override
            public Long apply(String s) {
                return Long.parseLong(s);
            }
        });

        userLogService.insertLog(loginAccount, LogEnumType.ENABLE.getCode(), LogEnumType.ENABLE.getName() + "批量启用推广活动", ids, appid, OperateObjectEnumType.CAMPAIGN.getCode(), request);

        return ResultModel.OK(this.campaignService.batchEnable(appid, campaignIds, loginAccount.getId()));
    }

    @RequestMapping(value = "/{appid}/makesurl", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel makesurl(@PathVariable Long appid, @CurrentAccount Account loginAccount) {
        return makesurls(appid, 1, loginAccount);
    }

    @RequestMapping(value = "/{appid}/makesurls", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel makesurls(@PathVariable Long appid, @RequestParam Integer gennum, @CurrentAccount Account loginAccount) {
        List<Map<String, String>> rtnList = Lists.newArrayList();

        List<String> makesurls = this.campaignService.makesurls(appid, loginAccount, gennum);
        for (String surl : makesurls) {
            rtnList.add(ImmutableMap.of("surl", surl, "url", Constant.trackurl + "/" + surl));
        }
        return ResultModel.OK(rtnList);
    }

    @RequestMapping(value = "/{appid}/exportsurls", method = RequestMethod.POST)
    @ResponseBody
    public void exportsurls(HttpServletRequest request, HttpServletResponse httpServletResponse, @PathVariable Long appid,
                            @RequestBody SubCampaignExport subCampaignExport, @CurrentAccount Account loginAccount,
                            @RequestParam(name = "lang", defaultValue = "zh", required = false) String lanType,
                            @RequestParam String showurl) {
        String type = request.getParameter("type");
        Iterable<Long> campids = subCampaignExport.getIds();
        //短链文本处理
        StringBuffer exportbuffer = this.campaignService.exportsurls(appid, campids, loginAccount.getId(), type, lanType, showurl);

        //文件下载处理
        String filename = "活动短链列表";

        String userAgent = request.getHeader("User-Agent");
        byte[] bytes = new byte[0];
        try {
            bytes = userAgent.contains("MSIE") ? filename.getBytes() : filename.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            filename = new String(bytes, "ISO-8859-1");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        // 各浏览器基本都支持ISO编码
        //httpServletResponse.setHeader("Content-disposition", String.format("attachment; filename=\"%s\"", name)); // 文件名外的双引号处理firefox的空格截断问题


        // try {
//			if (request.getHeader("User-Agent").toUpperCase().indexOf("MSIE") > 0) {
        //filename = URLEncoder.encode(filename, "UTF-8");
//			} else {
//				filename = new String(filename.getBytes("UTF-8"), "ISO-8859-1");
//			}
        //  }
       /* catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }*/
        httpServletResponse.setHeader("Content-Disposition", "attachment;filename=" + filename + ".txt");
        httpServletResponse.setContentType("APPLICATION/OCTET-STREAM");
        ServletOutputStream outputStream = null;
        try {
            outputStream = httpServletResponse.getOutputStream();
            ((ServletOutputStream) outputStream).write(exportbuffer.toString().getBytes());
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    @RequestMapping(value = "/find/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listone(@PathVariable Long id, @CurrentAccount Account loginAccount) {
        Campaign4Web result = campaignService.findById(id);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动" + result.getName(), result, null, OperateObjectEnumType.CAMPAIGN.getCode());
        //网眼定制
        if(27289L==loginAccount.getRootParent()||38L==loginAccount.getRootParent()) {
            List<Campaign> byWangyanApp = campaignRepository.findByWangyanApp(result.getApp());
            List<Long> ids = new ArrayList<>(byWangyanApp.size());
            for (Campaign cams : byWangyanApp) {
                ids.add(cams.getId());
            }
                if(ids.contains(result.getId())){
                    result.setImpTrackurl("请以您系统生成的监测链接为准");
                    result.setTrackurl("请以您系统生成的监测链接为准");
                }
        }
        return ResultModel.OK(result);
    }

    /**
     * 获取活动列表，母账号获取该App下的所有的活动，子账号获取与该账户关联的并且status是1的活动列表.此列表中包含账户自己创建的活动.
     * 每个活动，都处理了追踪短链.
     */
    @RequestMapping(value = "{appid}/find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAll(@PathVariable Long appid, @CurrentAccount Account loginAccount, @RequestParam(value = "type", required = false) String type) {

//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动列表",
//                null,appid,OperateObjectEnumType.CAMPAIGN.getCode());

        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appid, type, false);

        return ResultModel.OK(Lists.newArrayList(result));
    }

    /**
     * 获取已停用的活动列表
     * 需求修改后:只要有推广活动管理权限,就能看到此APP下所有推广活动。
     * created by sunhao 20170602
     */
    @RequestMapping(value = "{appId}/findDisable", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findAllDisable(@PathVariable Long appId, @CurrentAccount Account loginAccount, @RequestParam(value = "type", required = false) String type) {

//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广活动列表",
//                null, appId, OperateObjectEnumType.CAMPAIGN.getCode());

        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appId, type, true);

        return ResultModel.OK(Lists.newArrayList(result));
    }

    @RequestMapping(value = "valid/{appid}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@RequestParam String name, @PathVariable Long appid) {
        Campaign campaign = this.campaignRepository.findByNameAndAppIncludeDel(name, appid);
        if (ValidateUtil.isValid(campaign)) {
            return ResultModel.OK(true);
        } else {
            return ResultModel.OK(false);
        }
    }

    @RequestMapping(value = "batchValid/{appId}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createOneAndBatch(HttpServletRequest request, @PathVariable Long appId) {
        ArrayList<Campaign> campaigns = new ArrayList<>();
        ArrayList<String> campaignsName = new ArrayList<>();
        int startNumber = Integer.parseInt(request.getParameter("startNumber"));
        int batchNumber = Integer.parseInt(request.getParameter("batchNumber"));
        String campaignName = request.getParameter("campaignName");
        int i = startNumber;
        while (i < batchNumber + startNumber) {
            String name = campaignName + "_" + i;
            Campaign cam = campaignRepository.findByNameAndAppIncludeDel(name, appId);
            if (ValidateUtil.isValid(cam)) {
                campaigns.add(cam);
                campaignsName.add(cam.getName());
            }
            i++;
        }
        if (campaigns.size() > 0) {
            return ResultModel.OK(campaignsName.toArray());
        } else {
            return ResultModel.OK(false);
        }
    }

    @RequestMapping(value = "packcid/valid/{appid}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkChannel(@RequestParam String packchannelid, @PathVariable Long appid) {
        Campaign campaign = this.campaignService.findPackChannelidExist(packchannelid, appid);
        if (ValidateUtil.isValid(campaign)) {
            return ResultModel.OK(true);
        } else {
            return ResultModel.OK(false);
        }
    }

    /*
     * 下载二位码
     */

    @RequestMapping(value = "export/img", method = RequestMethod.POST)
    @ResponseBody
    public String exportImg(HttpServletRequest request,
                            HttpServletResponse response)
            throws Exception {
        request.setCharacterEncoding("UTF-8");
        String data = request.getParameter("data");
        String surl = request.getParameter("surl");

        response.setHeader("Content-Disposition", "attachment;filename=" + surl
                + ".png");
        response.setContentType("APPLICATION/OCTET-STREAM");
        if (data == null) {
            return "nodata";
        }
        // 对字节数组字符串进行Base64解码
        try {
            // Base64解码
            byte[] b = Base64.decode(data.substring(data.indexOf("base64,") + 7));
            for (int i = 0; i < b.length; ++i) {
                if (b[i] < 0) {// 调整异常数据
                    b[i] += 256;
                }
            }
            response.getOutputStream().write(b);
            response.getOutputStream().flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                response.getOutputStream().close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return "none";
    }

    @RequestMapping(value = "downloadurl/find/{appid}", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel findUrl(@CurrentAccount Account loginAccount, @PathVariable Long appid) {
        List<CampaignUrl4Web> list = campaignService.findUrlList(appid);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "下载地址列表",
//                null, appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode());
        return ResultModel.OK(list);
    }

    @RequestMapping(value = "downloadurl/modifyname", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel modifyUrlName(@CurrentAccount Account loginAccount, @RequestBody CampaignUrl4Web web) {
        int line = campaignService.modifyUrlName(web.getCampaignId(), web.getUrlName(), web.getTabType());

        return ResultModel.OK(line);
    }

    @RequestMapping(value = "downloadurl/update/{appid}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel updateUrl(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long appid, @RequestBody CampaignUrl4Web web) {
        campaignService.updateUrlList(loginAccount.getId(), appid, web.getUrl(), web.getNewUrl());
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "下载地址 " + web.getUrl() + " 修改为 " + web.getNewUrl(),
                "", appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode(), request);
        return ResultModel.OK();
    }


    /**
     * 根据id批量更新活动url
     *
     * @param request
     * @param loginAccount
     * @param appid
     * @param web
     * @return
     */
    @RequestMapping(value = "downloadurl/update/{appid}/campainids", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel updateUrlByCampin(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long appid, @RequestBody CampaignUrl4Web web) {
        campaignService.updateUrlListByCampin(loginAccount.getId(), appid, web);
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "下载地址 " + web.getUrl() + " 修改为 " + web.getNewUrl(),
                "修改数量：" + web.getCampaings().size(), appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode(), request);
        return ResultModel.OK();
    }


    @RequestMapping(value = "downloadurl/find/campaign/{appid}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel findCampaignByUrl(@CurrentAccount Account loginAccount, @PathVariable Long appid, @RequestBody String url) {
        List<CampaignUrl4Web> list = campaignService.findCampaignByUrl(appid, url);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "下载地址"  + url + "查看推广活动列表",
//                null, appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode());
        return ResultModel.OK(list);
    }

    @RequestMapping(value = "autodownloadsurls/{appid}", method = RequestMethod.GET)
    public ResultModel autodownloadsurls(HttpServletRequest request, HttpServletResponse httpServletResponse, @PathVariable Long appid, @RequestParam String ids,
                                         @RequestParam String name, @CurrentAccount Account loginAccount, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
        Iterable<String> iterids = Splitter.on(",").trimResults().omitEmptyStrings().split(ids);
        if (iterids == null) {
            return ResultModel.OK();
        }
        Iterable<Long> transids = Iterables.transform(iterids, new Function<String, Long>() {
            @Override
            public Long apply(String s) {
                return Long.parseLong(s);
            }
        });
        StringBuffer exportbuffer = this.campaignService.exportsurls2column(appid, transids, loginAccount.getId(), lanType);

        String filename = name + ".csv";

        //为了解决中文名称乱码问题
        try {
            filename = new String(filename.getBytes("UTF-8"));
            filename = URLEncoder.encode(filename, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
//        try {
//            if (request.getHeader("User-Agent").toUpperCase().indexOf("MSIE") > 0) {
//                filename = URLEncoder.encode(filename, "UTF-8");
//            } else {
//                filename = new String(filename.getBytes("UTF-8"), "ISO8859-1");
//            }
//        } catch (UnsupportedEncodingException e) {
//            e.printStackTrace();
//        }
        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.setHeader("filename", filename);
        httpServletResponse.setHeader("Content-Disposition", "attachment;filename=" + filename);
        httpServletResponse.setHeader("Content-Type", "text/csv");
        httpServletResponse.setContentType("APPLICATION/OCTET-STREAM");
        ServletOutputStream outputStream = null;
        try {
            outputStream = httpServletResponse.getOutputStream();
            ((ServletOutputStream) outputStream).write(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF});
            ((ServletOutputStream) outputStream).write(exportbuffer.toString().getBytes());
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (outputStream != null)
                    outputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "/{channel}/alteralias", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel alterThirdAccountAlias(HttpServletRequest request, @PathVariable Long channel, @RequestParam String thirdAccount, @RequestParam String thirdAccountAlias, @CurrentAccount Account loginAccount) {

        Long rootParent = loginAccount.getRootParent();
        //id=rootParent or parent=rootParent
        List<Long> accounts = accountRepository.findAllAccountList(rootParent);

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "第三方账号别名", thirdAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);

        return ResultModel.OK(this.campaignRepository.updateThirdAccountAlias(channel, thirdAccount, thirdAccountAlias, accounts));
    }


    @RequestMapping(value = "findbygdtchannel", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findByGDTChannel(@CurrentAccount Account loginAccount) {

        Long rootAccount = loginAccount.getRootParent();

        List<Account> accountList = accountRepository.findAccountByRootParent(rootAccount);

        List<Long> accountidList = new ArrayList<>();
        for (Account account : accountList) {
            accountidList.add(account.getId());
        }

        List<Long> appidList = campaignRepository.findByAccount(accountidList);

        List<App> appList = appRepository.findByIds(appidList);


        return ResultModel.OK(appList);
    }


    /**
     * @param request
     * @param response
     * @param channelid
     * @param appid
     * @return 判断产品下是否有活跃的推广活动
     */
    @RequestMapping(value = "/checkCamValid")
    @ResponseBody
    public ResultModel checkCamValid(HttpServletRequest request, HttpServletResponse response,
                                     @RequestParam(value = "channelid") Long channelid, @RequestParam(value = "appid") Long appid) {

//       Boolean hasActiveCampaign = campaignService.checkActiveCampaign(appid, channelid);
        Boolean hasActiveCampaign = campaignService.checkActiveCampaignNew(appid, channelid);// add by zhangsy 20180822
        HashMap<String, Object> result = new HashMap<>();
        result.put("activeCampaign", hasActiveCampaign);
        return ResultModel.OK(result);
    }


    @RequestMapping(value = "/{appId}/creategdt")
    //@RequestMapping(value = "/{appId}/createGdt", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createGdt(HttpServletRequest request, @PathVariable Long appId, @RequestBody Campaign resource, @CurrentAccount Account loginAccount) {
        List<Campaign> result = new ArrayList<>();
        resource.setAccount(loginAccount.getId());
        resource.setCreateAccount(loginAccount.getId());
        resource.setCreateTime(new Date());
        resource.setChannel(5305L);
        resource.setName("广点通点击数据");
        //后端生成的short url
        List<String> shortUrlList = campaignService.makesurls(appId, loginAccount, 1);

        if (!CollectionUtils.isEmpty(shortUrlList)) {
            //单个创建推广活动
            resource.setSurl(shortUrlList.get(0));
            Campaign campaign = campaignService.create(appId, resource);
            result.add(campaign);
            GdtPlan gdtPlan = new GdtPlan();
            gdtPlan.setFeedbackURL(campaign.getSurl());
            GdtPlan gdtlanByRootAccount = this.gdtPlanRepository.getGdtlanByRootAccount(loginAccount.getRootParent());
            gdtPlan.setPlan(gdtlanByRootAccount == null ? 2 : gdtlanByRootAccount.getPlan());
            gdtPlan.setAppkey(this.appRepository.findAppkeyById(appId));
            gdtPlan.setRootAccount(loginAccount.getRootParent());
            this.gdtPlanRepository.save(gdtPlan);
        } else {
            //失败
            return ResultModel.ERROR(ResultStatus.FAILED);
        }


        //日志
        UserLogThread userLogThread = new UserLogThread(loginAccount.getEmail(), loginAccount.getIsMasterLogin(), LogEnumType.CREATE.getCode(), String.format("%s%s%s%s", result.size() > 1 ? "批量" : "", LogEnumType.CREATE.getName(), "推广活动", resource.getName()), result, request, appId, OperateObjectEnumType.CAMPAIGN.getCode());
        userLogThread.start();
        return ResultModel.OK(result);
    }

    @RequestMapping(value = "/{appid}/isAutoExport", method = RequestMethod.GET)
    @ResponseBody
    public Object isAutoExport(HttpServletRequest request, @PathVariable Long appid, @CurrentAccount Account loginAccount) {
        AppExtend appExtend = appExtendServiceImpl.findOneByAppid(appid);
        if (appExtend == null) {
            appExtend = new AppExtend();
            appExtend.setId(-1L);
            appExtend.setIsAutoExport(1);
        }
        return ResultModel.OK(appExtend);
    }

    @RequestMapping(value = "/{type}/{appid}/checkDeepLink", method = RequestMethod.GET)
    @ResponseBody
    public Object checkDeepLink(HttpServletRequest request, @RequestParam("name") String name, @PathVariable Long appid, @PathVariable Integer type, @CurrentAccount Account loginAccount) {
        //因为这是创建活动时候校验app级别的deeplink配置，所以查询时候要排除本appid的scheme和子域
        AppDeepLink appDeepLink = null;
        if (type != null && type == 0) {//scheme
            appDeepLink = appDeepLinkRepository.findDeepLinkNotOwnAppByLinkId(appid, name);
        }
        if (appDeepLink == null) {
            return ResultModel.OK(true);
        } else {
            return ResultModel.OK(false);
        }
    }

    @RequestMapping(value = "/{appid}/{campaignid}/echoDeepLink", method = RequestMethod.GET)
    @ResponseBody
    public Object echoDeepLink(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long campaignid, @CurrentAccount Account loginAccount) {
        AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(appid);
        AppDeepLink appDeepLink1 = new AppDeepLink();
        appDeepLink1 = appDeepLink;
        Campaign oneById = campaignService.findOneById(campaignid);
        String url = null;
        ResultModel resultModel = null;
        if (oneById != null) {
            url = oneById.getUrl();
        }
        String containsUrl = Constant.deeplinkHost;
        if (url != null && url.contains(containsUrl)) {
            String substring = url.substring(url.indexOf("download=") + "download=".length());
            if (appDeepLink1 == null) {
                appDeepLink1 = new AppDeepLink();
                appDeepLink1.setCallType(-1);
                appDeepLink1.setLinkId(url.substring(url.indexOf("https://") + "https://".length(), url.indexOf("." + Constant.deeplinkHost)));
            }
            appDeepLink1.setAppDownload(UrlEncoded.decodeString(substring, 0, substring.length(), "UTF-8"));
            if (url.contains("ry/")) {
                if ((appDeepLink1.getPackageName() == null && oneById.getOs().equals("Android") || (appDeepLink1.getAppIdPrefix() == null && oneById.getOs().equals("iOS")))) {
                    appDeepLink1.setPaths(null);
                } else {
                    appDeepLink1.setPaths(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                }
            } else {
                appDeepLink1.setPaths(null);
            }
            if (url.contains("ry_dp=")) {
                String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                appDeepLink1.setUrlScheme(UrlEncoded.decodeString(substring2, 0, substring2.length(), "UTF-8"));
            } else if (url.contains("ry_sc=") && appDeepLink1.getCallType() == null) {
                String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                appDeepLink1.setUrlScheme(UrlEncoded.decodeString(substring2, 0, substring2.length(), "UTF-8"));
            }
            resultModel = new ResultModel(201, "old", appDeepLink1);
        } else {
            if (appDeepLink1 != null) {
                appDeepLink1.setPaths(null);
                appDeepLink1.setUrlScheme(null);
                if (oneById != null)
                    appDeepLink1.setAppDownload(oneById.getUrl());
            }
            resultModel = new ResultModel(201, "new", appDeepLink1);
        }
        //old:根据deeplink的值判断是否开启
        //new：默认关闭，如果content不为null，则照常回显
        return resultModel;
    }

    @RequestMapping(value = "/{appid}/getExportCampaignConfig", method = RequestMethod.GET)
    @ResponseBody
    public Object getExportCampaignConfig(@PathVariable Long appid) {
        return ResultModel.OK(exportCampaignConfigRepository.findOne(appid) != null);
    }

    @RequestMapping(value = "/{appid}/{status}/controlExportCampaignConfig", method = RequestMethod.GET)
    @ResponseBody
    public Object controlExportCampaignConfig(@PathVariable Long appid, @PathVariable int status, @CurrentAccount Account loginAccount) {
        ExportCampaignConfig one = exportCampaignConfigRepository.findOne(appid);
        if (one == null && status == 1) {
            ExportCampaignConfig exportCampaignConfig = new ExportCampaignConfig();
            exportCampaignConfig.setApp(appid);
            exportCampaignConfig.setAppkey(appRepository.findOne(appid).getAppkey());
            exportCampaignConfig.setAccount(loginAccount.getId());
            exportCampaignConfig.setEmail(loginAccount.getEmail());
            exportCampaignConfigRepository.save(exportCampaignConfig);
        } else if (one != null && status != 1) {
            exportCampaignConfigRepository.delete(one);
        }
        return ResultModel.OK(true);
    }

    @GetMapping("/{appid}/type/dropdownlist")
    @ResponseBody
    public Object dropDownList(@PathVariable Long appid){
        Map<String,Boolean> map = campaignService.dropDownList(appid);
        return ResultModel.OK(map);
    }


}
