package com.reyun.controller.manage;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.reyun.security.annotation.AuthorizationMenu;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.CampaignGroup;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.CampaignGroupService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.ValidateUtil;

@Controller
@RequestMapping(value = "/mng/campgroup")
public class MngCampaignGroupController {
	protected static Logger logger = LoggerFactory
			.getLogger(MngCampaignGroupController.class);

	@Autowired
	CampaignGroupService campaignGroupService;

	@Autowired
	UserLogService userLogService;

	/**
	 * 创建活动组
	 * 
	 * @param request
	 * @param resource
	 * @return
	 */
	@RequestMapping(value = "/create", method = RequestMethod.POST)
	@ResponseStatus(HttpStatus.CREATED)
	@ResponseBody
	@AuthorizationMenu("campgroup")
	public ResultModel create(HttpServletRequest request,
			@RequestBody CampaignGroup resource,
			@CurrentAccount Account loginAccount) {

		boolean flag = this.campaignGroupService.valiad(resource);
		if(!flag){
			return ResultModel.ERROR(ResultStatus.NAME_EXIST);
		}

		String campaigns = resource.getCampaigns();
		CampaignGroup result = this.campaignGroupService.create(
				resource.getApp(), resource, loginAccount.getId(), campaigns);
		userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "推广活动组" + result.getName(), result,result.getApp(),OperateObjectEnumType.CAMPAIGN_GROUP.getCode());
		return ResultModel.OK(result);
	}

	@RequestMapping(value = "/nameValid", method = RequestMethod.POST)
	@ResponseBody
	public ResultModel nameValid(HttpServletRequest request,
							  @RequestBody CampaignGroup resource,
							  @CurrentAccount Account loginAccount) {
		boolean flag = this.campaignGroupService.nameValiad(resource,loginAccount);
		if(!flag){
			return  new ResultModel(ResultStatus.NAME_EXIST);
		}
		return ResultModel.OK();
	}

	/**
	 * 删除活动组
	 * 
	 * @param request
	 * @param id
	 */
	@RequestMapping(value = "delete/{id}", method = RequestMethod.DELETE)
	@ResponseBody
	@ResponseStatus(HttpStatus.NO_CONTENT)
	@AuthorizationMenu("campgroup")
	public ResultModel delete(HttpServletRequest request,
			@PathVariable Long id, @CurrentAccount Account loginAccount) {

		CampaignGroup campGroup = this.campaignGroupService.delete(id,
				loginAccount.getId());
		if (ValidateUtil.isValid(campGroup)) {
			userLogService.insertLog(loginAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "推广活动组" + campGroup.getName(), campGroup,campGroup.getApp(),OperateObjectEnumType.CAMPAIGN_GROUP.getCode());
			return ResultModel.OK(campGroup);
		} else {
			return ResultModel.ERROR(ResultStatus.ID_NOT_EXIST);
		}
	}

	/**
	 * 修改活动组
	 * 
	 * @param request
	 * @param response
	 * @param resource
	 * @param loginAccount
	 * @return
	 */
	@RequestMapping(value = "/update", method = RequestMethod.PUT)
	@ResponseBody
	@AuthorizationMenu("campgroup")
	public ResultModel update(HttpServletRequest request,
			HttpServletResponse response, @RequestBody CampaignGroup resource,
			@CurrentAccount Account loginAccount) {

		boolean flag = this.campaignGroupService.nameValiad(resource,loginAccount);
		if(!flag){
			//return ResultModel.ERROR(ResultStatus.NAME_EXIST);
            return  new ResultModel(ResultStatus.NAME_EXIST);
		}

		String campaigns = resource.getCampaigns();
		Long id = resource.getId();
		CampaignGroup retrievedResource = this.campaignGroupService
				.findOneById(id);
		if (retrievedResource == null) {
			response.setStatus(205);
			return ResultModel.ERROR(ResultStatus.ID_NOT_EXIST);
		}
		retrievedResource.setModifyAccount(loginAccount.getId());
		retrievedResource.setApp(resource.getApp());
		retrievedResource.setName(resource.getName());
		retrievedResource.setDeeplink(resource.isDeeplink());
		CampaignGroup result = this.campaignGroupService.update(campaigns,
				retrievedResource);
		userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广活动组" + result.getName(), result,result.getApp(),OperateObjectEnumType.CAMPAIGN_GROUP.getCode());
		return ResultModel.OK();
	}

	@RequestMapping(value = "/{appid}/find", method = RequestMethod.GET)
	@ResponseBody
	public ResultModel listCamgroupByAccount(HttpServletRequest request, @PathVariable Long appid, @CurrentAccount Account loginAccount) {

        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "活动组列表",
                null,appid,OperateObjectEnumType.CAMPAIGN_GROUP.getCode());

        return ResultModel.OK(this.campaignGroupService.listCamgroupByAccountAndApp(appid, loginAccount.getId()));
	}
//
//	@RequestMapping(value = "/valid", method = RequestMethod.GET)
//	@ResponseBody
//	public ResultModel check(@RequestParam String name) {
//		CampaignGroup campaignGroup = this.campaignGroupService
//				.findByName(name);
//		if (null != campaignGroup) {
//			return ResultModel.OK(false);
//		} else {
//			return ResultModel.OK(true);
//		}
//
//	}

}
