package com.reyun.controller.manage;

import com.google.common.base.Strings;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.model.ChannelAccount;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.ChannelAccountService;
import com.reyun.service.ChannelService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import java.util.List;

/**
 * description:
 *
 * @author nolan
 * @date 11/09/2017
 */
@Controller
@RequestMapping(value = "/mng/channel/account")
public class MngChannelAccountCommonController
{
    @Autowired
    private ChannelAccountService channelAccountService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    UserLogService userLogService;

    /**
     * 创建渠道帐号
     *
     * @param resource
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel create(HttpServletRequest request, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount)
    {
        Channel channel = channelService.findoneById(resource.getChannel());
        if (channel == null) {
            throw new TipException("渠道不存在");
        }
        if (Strings.isNullOrEmpty(channel.getType())) {
            throw new TipException("该渠道为非合作渠道");
        }

        if("adwords".equals(channel.getUniqueName())) {
            //1.渠道账号别名：主账号下不可重复，重复提示“该名称系统已存在，请重新输入。
            if (!channelAccountService.validAlias(resource.getAlias(), loginAccount.getId())) {
                return ResultModel.ERROR(ResultStatus.CHANNEL_ALIASNAME_EXIST);
            }
            //2.渠道账号id：主账号下不可重复，重复提示“该账号ID已创建投放账号，不可重复创建。
            List<Long> accountIds = channelAccountService.getLinkAccounts(loginAccount.getId());
            List<ChannelAccount> channelAccounts = channelAccountService.listByAccounts(accountIds);
            if (channelAccounts.stream()
                    .filter(channelAccount -> !com.google.api.client.repackaged.com.google.common.base.Strings.isNullOrEmpty(resource.getThirdAccountCode()) && resource.getThirdAccountCode().equals(channelAccount.getThirdAccountCode()))
                    .count() > 0) {
                return ResultModel.ERROR(ResultStatus.CHANNELACCOUNT_EXIST);
            }
        }else if("gdt".equals(channel.getUniqueName())){
        }else if("smsearch".equals(channel.getUniqueName())){
        }


        resource.setUniqueName(channel.getUniqueName());
        ChannelAccount channelAccount = channelAccountService.create(resource, loginAccount.getId());
        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }

    /**
     * 更新渠道帐号
     *
     * @param resource
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel update(HttpServletRequest request, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount)
    {
        ChannelAccount channelAccount = channelAccountService.update(resource, loginAccount.getId());
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }

    /**
     * 检查渠道帐号状态
     *
     * @param id 渠道帐号id
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/checkstatus", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkStatus(@RequestParam Long id, @CurrentAccount Account loginAccount)
    {
        ChannelAccount channelAccount = channelAccountService.findById(id);
        if (channelAccount != null) {
            return ResultModel.OK(channelAccount.getPastStatus() != null && !channelAccount.getPastStatus());
        }
        else {
            return ResultModel.ERROR(ResultStatus.CHANNELACCOUNT_NOEXIST);
        }
    }

    /**
     * 神马回调地址,更新用户token信息
     *
     * @return
     */
    @RequestMapping(value = "/cback", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel serverCallback()
    {
        return ResultModel.OK();
    }
}
