package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.ChannelAccount;
import com.reyun.model.ChannelAccountLinkApp;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.CampaignService;
import com.reyun.service.ChannelAccountService;
import com.reyun.service.ChannnelAccountLinkAppService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.StringUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "/mng/channelaccount")
public class MngChannelAccountController {
    private static Logger logger = LoggerFactory.getLogger(MngChannelAccountController.class);
    @Autowired
    ChannelAccountService channelAccountService;

    @Autowired
    UserLogService userLogService;

    @Autowired
    ChannnelAccountLinkAppService channnelAccountLinkAppService;

    @Autowired
    CampaignService campaignService;


    //region 广点通专属接口定义
    /**
     * #广点通
     * 删除渠道帐号
     */
    @RequestMapping(value = "/delete/authorize/{flag}/{qq}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteAuthorize(@PathVariable int flag, @PathVariable String qq, @CurrentAccount Account loginAccount) {
        if(flag == 1){
            channelAccountService.deleteAuthorize(qq);
        }
        return ResultModel.OK();
    }

    /**
     * 验证qq号是否重复
     */
    @RequestMapping(value = "/valid/qq", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel validQQ(@RequestParam String qq, @CurrentAccount Account loginAccount){
        boolean flag = channelAccountService.validQQ(qq, loginAccount.getId());
        return ResultModel.OK(flag);
    }


    /**
     * 验证渠道帐号第三方accountid
     */
    @RequestMapping(value = "/valid/accountid", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel validAccountId(@RequestParam String accountid, @CurrentAccount Account loginAccount){
        boolean flag = channelAccountService.validAccountId(accountid, loginAccount.getId());
        return ResultModel.OK(flag);
    }

    /**
     * 获取GDT回调token
     */
    @RequestMapping(value = "/code", method = RequestMethod.GET)
    public void getCode(HttpServletRequest request, @RequestParam(required = false) String authorization_code, @RequestParam(required = false) String state,
                        @CurrentAccount Account loginAccount, HttpServletResponse response) throws Exception {
        if(null != authorization_code || null != state){
            ChannelAccount channelAccountSave = channelAccountService.getGDTTokenAndSave(request, authorization_code, state, loginAccount);
        }
//        if (loginAccount.getId() == 38 || "trackingio@reyun.com".equals(loginAccount.getEmail())) {
//            response.sendRedirect("/gdtAuthorize.html");
//        } else {
//            response.sendRedirect("/index.html");
//        }
        response.sendRedirect("/gdtAuthorize.html");

    }


    /**
     *
     */
    @RequestMapping(value = "/authorize", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel authorize(HttpServletRequest request, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {
        /*if (StringUtil.isEmpty(resource.getThirdAccountCode())) {
            return ResultModel.ERROR(ResultStatus.FAILED);
        }*/
        logger.info("ThirdAccountCode before deal:" + resource);
        ChannelAccount channelAccount = channelAccountService.authorize(resource, loginAccount.getId());
        channelAccount.setRefreshToken(null);
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "" + channelAccount.getName() + "的ThirdAccountCode", channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }

    /**
     * 保存别名
     */
    @RequestMapping(value = "/alias", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel alias(HttpServletRequest request, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {
        ChannelAccount channelAccount = channelAccountService.alias(resource, loginAccount.getId());
        channelAccount.setRefreshToken(null);
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "" + channelAccount.getName() + "的别名", channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }

    /**
     * 创建用户行为源
     */
    @RequestMapping(value = "/userid/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel useridCreate(HttpServletRequest request, @RequestBody ChannelAccountLinkApp resource,
                                    @CurrentAccount Account loginAccount,
                                    @RequestParam(value = "channelid",defaultValue = "0", required = false) Long channelid) throws JSONException, IOException {

        //cereate by 20180723(针对渠道广点通，需要校验是否该产品有活跃的推广活动--原则上是必须不存在)
        Long appid = Long.valueOf(resource.getAppName());
        boolean hasActiveCampain = false;//默认没有活跃的渠道推广活动
        if(channelid > 0){
            hasActiveCampain = campaignService.checkActiveCampaign(appid, channelid);
        }

        if(hasActiveCampain){
            return ResultModel.ERROR(ResultStatus.CHANNEL_CAMPAIGN_NOTDEL);
        }
        //---------------------end

        Map<String, String> map = channnelAccountLinkAppService.createUserId(resource);
        return ResultModel.OK(map);
    }
    //endregion



    //region 渠道帐号通用接口定义
    /**
     * 创建渠道帐号
     */
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel create(HttpServletRequest request, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {
        boolean flag = channelAccountService.simulationLogin(resource);

        if (!flag) {
            return ResultModel.OK(false);
        }

        ChannelAccount channelAccount = channelAccountService.create(resource, loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }
    /**
     * 绑定百度原生ocpc渠道帐号
     */
    @RequestMapping(value = "/createByBaidu", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createByBaidu(@RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {

        boolean flag = channelAccountService.createByBaidu(resource, loginAccount.getId());
        if(flag){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }
    /**
     * 绑定快手渠道帐号
     */
    @RequestMapping(value = "/createByKuaishou", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel createByKuaishou(@RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {

        boolean flag = channelAccountService.createByKuaishou(resource, loginAccount.getId());
        if(flag){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }
    /**
     * 绑定微信mp渠道帐号
     */
    @RequestMapping(value = "/createByWeixinMP", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel createByWeixinMP(String state , String authorization_code,@CurrentAccount Account loginAccount) {

        boolean flag = channelAccountService.createByWeixinMP(state, authorization_code,loginAccount);
        if(flag){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }
    /**
     * 确认微信mp是否授权成功
     */
    @RequestMapping(value = "/checkWeixinMP", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkWeixinMP(String alias ) {

        ChannelAccount channelAccount = channelAccountService.getByAliasAndChannelTrue(alias,5306L);
        if(channelAccount!=null){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }
    /**
     * 绑定快手渠道帐号
     */
    @RequestMapping(value = "/updateByKuaishou", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel updateByKuaishou(@RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {

        boolean flag = channelAccountService.updateByKuaishou(resource, loginAccount.getId());
        if(flag){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }
    /**
     * 修改百度原生ocpc渠道帐号
     */
    @RequestMapping(value = "/updateByBaidu", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel updateByBaidu(@RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {

        boolean flag = channelAccountService.updateByBaidu(resource, loginAccount.getId());
        if(flag){
            return ResultModel.OK(ResultStatus.SUCCESS);
        }
        return ResultModel.ERROR(ResultStatus.FAILED);
    }

    /**
     *
     */
    @RequestMapping(value = "/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel update(HttpServletRequest request,
                              HttpServletResponse response, @RequestBody ChannelAccount resource, @CurrentAccount Account loginAccount) {
        boolean flag = channelAccountService.simulationLogin(resource);

        if (!flag) {
            return ResultModel.OK(false);
        }

        ChannelAccount channelAccount = channelAccountService.update(resource, loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK(channelAccount);
    }

    @RequestMapping(value = "/forbidden/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel delete(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        ChannelAccount channelAccount = channelAccountService.delete(id, loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.DISABLE.getCode(), LogEnumType.DISABLE.getName() + "渠道账号" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);

        return ResultModel.OK(channelAccount);
    }

    @RequestMapping(value = "/enable/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enable(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        ChannelAccount channelAccount = channelAccountService.enable(id, loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.ENABLE.getCode(), LogEnumType.ENABLE.getName() + "渠道账号" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);

        return ResultModel.OK(channelAccount);
    }

    @RequestMapping(value = "valid", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@RequestParam String name, @CurrentAccount Account loginAccount) {
        return ResultModel.OK(channelAccountService.validAlias(name, loginAccount.getId()));
    }

    @RequestMapping(value = "/find/{cid}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listOwnChannelByCreateAccount(@PathVariable long cid, @CurrentAccount Account loginAccount) {
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "渠道账号列表", null, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode());
        return ResultModel.OK(channelAccountService.find(loginAccount, cid));
    }
    @RequestMapping(value = "/getToutiaoAccessToken", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel getToutiaoAccessToken(Long appId, String secret , String grantType , String authCode ,String alias,@CurrentAccount Account loginAccount) {
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), "获取今日头条access_token", null, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode());
        return ResultModel.OK(channelAccountService.getToutiaoAccessToken(appId, secret,grantType,authCode,alias,loginAccount));
    }
    @RequestMapping(value = "/checkToutiaoNum", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkToutiaoNum(@CurrentAccount Account loginAccount) {
        return ResultModel.OK(channelAccountService.checkToutiaoNum(loginAccount));
    }
    @RequestMapping(value = "/channellist", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel channellist( @CurrentAccount Account loginAccount) {
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "渠道账号列表", null, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode());
        return ResultModel.OK(channelAccountService.channellist(loginAccount));
    }

    @RequestMapping(value = "/findcount", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel listOwnChannelCount(@CurrentAccount Account loginAccount,@RequestBody Map<String,String> channleids) {

        return ResultModel.OK(channelAccountService.findChannelAccountCount(loginAccount, channleids));
    }

    /**
     * 通用接口
     * <br>
     * 删除产品配置
     *
     * @param id
     * @param loginAccount
     * @return
     */
    @RequestMapping(value = "/delete/app/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteApp(HttpServletRequest request, @PathVariable Long id, @CurrentAccount Account loginAccount)
    {
        channnelAccountLinkAppService.delete(id);
        userLogService.insertLog(loginAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "渠道账号产品" + id, "" + id, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        return ResultModel.OK();
    }

    /**
     * 创建产品channellinkapp
     */
    @RequestMapping(value = "/app/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel appCreate(HttpServletRequest request, @RequestBody ChannelAccountLinkApp resource, @CurrentAccount Account loginAccount) throws JSONException, IOException {
        Map<String, String> valid = channnelAccountLinkAppService.valid(resource);
        if("0".equals(valid.get("code"))){
            resource.setTestSuccess(true);
            ChannelAccountLinkApp channnelAccountLinkApp = channnelAccountLinkAppService.save(resource, loginAccount.getId());
            userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道账号产品" + channnelAccountLinkApp.getAppName(), channnelAccountLinkApp, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        }
        return ResultModel.OK(valid);
    }

    /**
     * 查看渠道帐号的app列表
     */
    @RequestMapping(value = "/app/list", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel appList(HttpServletRequest request, @RequestParam Long linkuserid, @RequestParam String unique_name, @CurrentAccount Account loginAccount) throws JSONException, IOException {
        List<App> appList = channnelAccountLinkAppService.listApp2Manager(loginAccount, linkuserid, unique_name,true);
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), String.format("%s linkuserid:%d,uniquename:%s 的渠道账号产品列表", LogEnumType.FIND.getName(), linkuserid, unique_name)+ "渠道账号产品列表", null, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode());
        return ResultModel.OK(appList);
    }

    /**
     * 查看渠道帐号下已授权app
     */
    @RequestMapping(value = "/app/find/{channelAccount}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel appFind(HttpServletRequest request, @PathVariable Long channelAccount, @CurrentAccount Account loginAccount) {
        List<ChannelAccountLinkApp> list = channnelAccountLinkAppService.findByChannelAccount(loginAccount, channelAccount);
        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), String.format("%s channelAccount:%d 的渠道账号产品列表", LogEnumType.FIND.getName(), channelAccount)+ "渠道账号产品列表", null, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode());
        return ResultModel.OK(list);
    }
    /**
     * 验证别名是否重复
     * create by zhangyao 2018-11-19
     */
    @RequestMapping(value = "/valid/checkAlias", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkAlias(@RequestParam String alias){
        boolean flag = channelAccountService.checkAlias(alias);
        return ResultModel.OK(flag);
    }
    @RequestMapping(value = "/valid/getByAlias", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel getByAlias(@RequestParam String alias){

        return ResultModel.OK(channelAccountService.getByAlias(alias));
    }

    /**
     * 绑定产品时候的第一步 - 查询已经绑定的商品
     * create by zhangyao 2018-11-23
     */
    @RequestMapping(value = "/findLinkedApp/{cid}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findLinkedApp(@PathVariable long cid, @CurrentAccount Account loginAccount){
        return ResultModel.OK(channelAccountService.findLinkedApp(loginAccount,cid));
    }

    /**
     * 修改广点通方案
     * create by zhangyao 2019-1-17
     */
    @RequestMapping(value = "/updatePlan/{plan}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findLinkedApp(@PathVariable int plan, @CurrentAccount Account loginAccount ,  String cip, String shortUrl){
        return ResultModel.OK(channelAccountService.updatePlan(loginAccount,plan,cip,shortUrl));
    }

    /**
     * 需求变更，该方法已过期
     * 查看当前账号的广点通方案
     * create by zhangyao 2019-1-17
     */
    @RequestMapping(value = "/checkPlan", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkPlan( @CurrentAccount Account loginAccount){
        return ResultModel.OK(channelAccountService.checkPlan(loginAccount));
    }

    /**
     * 功能描述:更新广点通deeplink开启关闭更新，仅限方案1
     * @author liyin
     * @date 2020/4/21
     */
    @RequestMapping(value = "/renewalDeepLink", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel renewalDeepLink(@CurrentAccount Account loginAccount ,@RequestBody Map<String,String> map){
        String feedbackurl = map.get("feedbackurl");
        String urlScheme = map.get("urlScheme");
        String state = map.get("state");
        if(StringUtil.isEmpty(state)){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
        if( StringUtil.isEmpty(feedbackurl) || StringUtil.isEmpty(urlScheme)){

        }
        boolean b = channelAccountService.renewalDeepLink(loginAccount,feedbackurl,urlScheme,state);
        if(!b){
            return ResultModel.ERROR(ResultStatus.NETWORK_ERROR);
        }
        return ResultModel.OK();
    }

    /**
     * 功能描述:更新广点通预约监测开启关闭更新，仅限方案1
     * @author liyin
     * @date 2020/6/29
     */
    @RequestMapping(value = "/renewalSubscribes", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel renewalSubscribes(@CurrentAccount Account loginAccount ,@RequestBody Map<String,String> map){
        String feedbackurl = map.get("feedbackurl");
        String state = map.get("state");
        if(StringUtil.isEmpty(state)){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
        boolean b = channelAccountService.renewalSubscribes(loginAccount,feedbackurl,state);
        if(!b){
            return ResultModel.ERROR(ResultStatus.NETWORK_ERROR);
        }
        return ResultModel.OK();
    }
}
