package com.reyun.controller.manage;

import com.reyun.dic.ChannelEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.model.ChannelAccount;
import com.reyun.model.ChannelAccountLinkApp;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AdwordsService;
import com.reyun.service.ChannelAccountService;
import com.reyun.service.ChannelService;
import com.reyun.service.ChannnelAccountLinkAppService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.json.JSONException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.IOException;
import java.util.Map;

@Controller
    @RequestMapping(value = "/mng/channelaccount/{linkUserId}/app")
public class MngChannelAccountLinkAppController
{
    @Autowired
    private ChannelAccountService channelAccountService;
    @Autowired
    private ChannnelAccountLinkAppService channnelAccountLinkAppService;
    @Autowired
    private AdwordsService adwordsService;
    @Autowired
    private ChannelService channelService;

    /**
     * 创建产品配置
     * @param linkUserId 渠道帐号id
     * @param resource  关联产品应用
     * @param loginAccount 登录用户
     * @return
     * @throws JSONException
     * @throws IOException
     */
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel appCreate(@PathVariable Long linkUserId, @RequestBody ChannelAccountLinkApp resource, @CurrentAccount Account loginAccount) throws JSONException, IOException {
        ChannelAccount channelAccount = this.channelAccountService.findById(linkUserId);
        if (channelAccount == null) {
            throw new TipException("渠道帐号不存在");
        }

        boolean isAllow = true;
        if("gdt".equals(channelAccount.getUniqueName())){
            Map<String, String> valid = channnelAccountLinkAppService.valid(resource);
            isAllow = "0".equals(valid.get("code"));
        }else if("adwords".equals(channelAccount.getUniqueName())){

            int linkCount = adwordsService.findLinkIdCount(resource.getUserActionSetId());

            if (linkCount > 0) {
                return ResultModel.ERROR(ResultStatus.CHANNEL_LINKID_EXIST_WITHAPP);
            }

            isAllow = adwordsService.isValid(resource.getUserActionSetId(),resource.getAppType());
            if (!isAllow) {
                return ResultModel.ERROR(ResultStatus.CHANNEL_LINKID_NOEXIST);
            }
        }
        resource.setTestSuccess(isAllow);
        return ResultModel.OK(isAllow ? channnelAccountLinkAppService.save(resource, loginAccount.getId()) : null);
    }

    @RequestMapping(value = "/checkname", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkName(@PathVariable Long linkUserId, @RequestParam("channel") Long channelid, @RequestParam("fieldkey") String fieldKey, @RequestParam("fieldvalue") String fieldValue, @CurrentAccount Account loginAccount)
    {
        Channel channel = this.channelService.findoneById(channelid);
        if (channel == null) {
            throw new TipException("渠道不存在");
        }

        final ChannelEnumType enumType = ChannelEnumType.getEnumType(channel.getUniqueName());
        return ResultModel.OK(this.channnelAccountLinkAppService.checkName(enumType, fieldKey, fieldValue, linkUserId, loginAccount.getId()));
    }



    /**
     * 删除指渠道帐号下关联产品
     * @param linkUserId 渠道帐号id
     * @param id 关联应用id
     * @param loginAccount 登录帐号
     * @return
     */
    @RequestMapping(value = "/delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel deleteApp(@PathVariable Long linkUserId, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        channnelAccountLinkAppService.delete(id);
        return ResultModel.OK();
    }

    /**
     * 查询渠道帐号下产品列表
     * @param
     * @param loginAccount
     * @return
     * @throws JSONException
     * @throws IOException
     */
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel appList(@RequestParam Long linkuserid, @RequestParam String unique_name, @CurrentAccount Account loginAccount) throws JSONException, IOException {
        return ResultModel.OK(channnelAccountLinkAppService.listApp(loginAccount, linkuserid, unique_name));
    }

    /**
     * 查询渠道帐号下已授权产品列表
     */
    @RequestMapping(value = "/find/{channelAccount}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel appFind(@PathVariable Long channelAccount, @CurrentAccount Account loginAccount) {
        return ResultModel.OK(channnelAccountLinkAppService.findByChannelAccount(loginAccount, channelAccount));
    }

}
