package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.repository.ChannelAdvertisingRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.ChannelService;
import com.reyun.service.UserLogService;
import com.reyun.util.PinYinUtil;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.ValidateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Controller
@RequestMapping(value = "/mng/channel")
public class MngChannelController {
    private static final Logger logger = LoggerFactory.getLogger(MngChannelController.class);

    @Autowired
    private ChannelService channelService;
    @Autowired
    private ChannelAdvertisingRepository channelAdvertisingRepository;

    @Autowired
    UserLogService userLogService;

    /**
     * 创建渠道及授权相关
     */
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel create(@RequestBody Channel resource, @CurrentAccount Account loginAccount
                              //  @RequestHeader(name = "lang", defaultValue = "zh") String lanType
    ) {

        //自定义渠道时 校验渠道名称
        List<Channel> existChannel = channelService.findByChannelName(resource.getName(), loginAccount);
        if (existChannel != null && existChannel.size() > 0) {
            return ResultModel.ERROR(ResultStatus.CHANNEL_NAME_EXIST);
        }

        // 设置汉字拼音首声母
        resource.setFirstchar(PinYinUtil.getFirstCharSpell(resource.getName().trim()).toLowerCase());
        // 设置创建用户的id
        resource.setCreateAccount(loginAccount.getId());
        resource.setAccount(loginAccount.getId());
        // 默认假删除标记为F
        resource.setDelFlag(false);

        // 1. 创建渠道
        Channel rtnChannel = this.channelService.save(resource);
		/*
		  功能描述:发送邮件给运营团队，暂时注释
		 try{
			SendEmailThread t = new SendEmailThread(rtnChannel, loginAccount);
			t.start();
		} catch (Exception e){
			logger.error(e.getMessage(), e);
		}*/
        userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "渠道列表" + rtnChannel.getName(), rtnChannel, null, OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(rtnChannel);
    }

    /**
     * 修改渠道信息及权限相关
     */
    @RequestMapping(value = "/update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel update(@RequestBody Channel resource, @CurrentAccount Account loginAccount) {

        Assert.notNull(resource.getId(), "id cannot be null");
        Channel retrievedResource = this.channelService.findoneById(resource.getId());
        if (retrievedResource == null) {
            throw new TipException("该渠道不存在");
        }

        if (!retrievedResource.getName().equals(resource.getName())) {
            //自定义渠道时 校验渠道名称
            List<Channel> existChannel = channelService.findByChannelName(resource.getName(), loginAccount);
            if (existChannel != null && existChannel.size() > 0) {
                return ResultModel.ERROR(ResultStatus.CHANNEL_NAME_EXIST);
            }
        }

        // 1. 更新渠道
        resource.setCreateAccount(loginAccount.getId());
        resource.setFirstchar(PinYinUtil.getFirstCharSpell(resource.getName()));
        resource.setModifyAccount(loginAccount.getId());
        resource.setModifyTime(new Date());
        Channel c = this.channelService.update(resource, loginAccount.getId());

        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道" + c.getName(), c, null, OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(c);
    }

    @RequestMapping(value = "/channelname/check", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel checkName(@RequestBody Channel resource, @CurrentAccount Account loginAccount) {
        List<Channel> existChannel = channelService.findByChannelName(resource.getName(), loginAccount);
        if (existChannel != null && existChannel.size() > 0) {

            final boolean[] checked = {false};
            existChannel.forEach(v -> {
                if (!v.getId().equals(resource.getId())) {
                    checked[0] = true;
                    return;
                }
            });

            if (checked[0]) {
                return ResultModel.ERROR(ResultStatus.CHANNEL_NAME_EXIST);
            }
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "/delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel delete(@PathVariable Long id, @CurrentAccount Account loginAccount) {

        Channel channel = this.channelService.findoneById(id);

        if (channel.getCreateAccount().equals(loginAccount.getId())) {

            // 删除渠道--假删除
            this.channelService.delete(channel, loginAccount.getId());
            // 删除权限--删除所有channelid的auth记录
            // this.channelService.delChannelAuth(id);
            userLogService.insertLog(loginAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "渠道" + channel.getName(), channel, null, OperateObjectEnumType.CHANNEL.getCode());
        }
        return ResultModel.OK();
    }


    @RequestMapping(value = "/delete/valid/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel deleteValid(@PathVariable Long id) {
        //验证是否可以删除
        int num = channelService.deleteValid(id);

        return ResultModel.OK(num);
    }

    @RequestMapping(value = "/enable/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enable(@PathVariable Long id, @CurrentAccount Account loginAccount) {
        Channel channel = this.channelService.enable(id, loginAccount.getId());

        try {
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道启用" + channel.getName(), channel, null, OperateObjectEnumType.CHANNEL.getCode());
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }

        return ResultModel.OK();
    }


    @RequestMapping(value = "/find/{appId}/byaccount", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listCampaignByChannel(@CurrentAccount Account loginAccount, @PathVariable Long appId) {
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "渠道列表", null, null, OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(channelService.listChannel4CreateCampaign(loginAccount.getId(), appId));
    }

    /**
     * 去掉合并的渠道
     *
     * @param loginAccount
     * @param appId
     * @param lanType(语言类型：zh(中文)/en（英文）)
     * @return
     */
    @RequestMapping(value = "/find/{appId}/byaccount/show", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listCampaignByChannelShow(@CurrentAccount Account loginAccount, @PathVariable Long appId,
                                                 @RequestHeader(name = "lang", defaultValue = "zh") String lanType,
                                                 String showDel, String accountAll,Boolean filter) {
//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "渠道列表", null, null, OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(channelService.listChannel4CreateCampaignShow(loginAccount, appId, lanType, showDel, accountAll,filter));
    }


    @RequestMapping(value = "/find", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listOwnChannelByCreateAccount(HttpServletRequest request, @CurrentAccount Account loginAccount) {
//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "自定义渠道列表", null,null,OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(channelService.listOwnChannelByCreateAccount(loginAccount.getId()));
    }

	@RequestMapping(value = "/find/unown", method = RequestMethod.GET)
	@ResponseBody
	public ResultModel listNotOwnChannelByCreateAccount(@CurrentAccount Account loginAccount,String delflg) {
//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "非自定义渠道列表", null,null,OperateObjectEnumType.CHANNEL.getCode());
        return ResultModel.OK(channelService.listUnOwnChannel(loginAccount, delflg));
    }

    @RequestMapping(value = "valid", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@RequestParam String name) {
        Channel channel = this.channelService.valid(name);
        if (ValidateUtil.isValid(channel)) {
            return ResultModel.OK(true);
        } else {
            return ResultModel.OK(false);
        }

    }
    @RequestMapping(value = "findChannelAdvertising", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findChannelAdvertising() {

        return ResultModel.OK(channelAdvertisingRepository.listAll());
    }

}
