package com.reyun.controller.manage;


import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.ChannelParam;
import com.reyun.model.CommonIP;
import com.reyun.model.PreorderParam;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AuthService;
import com.reyun.service.ChannelParamService;
import com.reyun.service.CommonIPService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;



/**
 * Created by mxq on 2017/9/11.
 */
@Controller
@RequestMapping("/mng/channelparam")
public class MngChannelParamController {

    @Autowired
    ChannelParamService channelParamService;

    private static final Logger logger = LoggerFactory.getLogger(MngChannelParamController.class);

    @Autowired
    private CommonIPService commonIPService;

    @Autowired
    UserLogService userLogService;

    @Autowired
    private AuthService authService;


    @RequestMapping(value = "update/valid", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel createValid(@RequestParam(defaultValue = "",required = false) String validFunction,@RequestParam boolean isdefault, @RequestParam boolean isall,
                                   @RequestBody ChannelParam resourse, @CurrentAccount Account loginAccount) {
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道参数激活匹配点击有效期   " + (isall ? "全渠道" : "单个渠道"), resourse,null, OperateObjectEnumType.CHANNELPARAM.getCode());
        return ResultModel.OK(channelParamService.update(isdefault, isall,resourse, loginAccount.getId(), "valid",validFunction));
    }


    @RequestMapping(value = "update/black", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel createBlack(@RequestParam boolean isdefault, @RequestParam boolean isall,
                                   @RequestBody ChannelParam resourse, @CurrentAccount Account loginAccount, HttpServletRequest request) {
        userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道参数黑名单   " + (isall ? "全渠道" : "单个渠道"), resourse,null, OperateObjectEnumType.CHANNELPARAM.getCode());


		LogEnumType logEnumType = null  ;

		if (resourse != null) {

			if (!resourse.getBlacklistStatus() && !resourse.getDeviceBlackStatus()) {
				logEnumType = LogEnumType.BLACK_CLOSE_ALL;
			} else if (!resourse.getBlacklistStatus()) {
				logEnumType = LogEnumType.BLACK_LIST_CLOSE;
			} else if (!resourse.getBlacklistStatus()) {
				logEnumType = LogEnumType.BLACK_DEVICE_CLOSE;
			} else {
				userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道参数黑名单   " + (isall ? "全渠道" : "单个渠道"), resourse, null, OperateObjectEnumType.CHANNELPARAM.getCode());
			}
		}

		if (logEnumType != null) {
			userLogService.insertLog(loginAccount, logEnumType.getCode(), logEnumType.getName() + loginAccount.getEmail(), loginAccount, null, null, request);
		}


        return ResultModel.OK(channelParamService.update(isdefault, isall,resourse, loginAccount.getId(), "black",null));
    }


	//预约归因设置
	@RequestMapping(value = "update/preorder", method = RequestMethod.PUT)
	@ResponseBody
	public ResultModel createPreorder(@RequestBody PreorderParam resourse,String appId, @CurrentAccount Account loginAccount) {

    	userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "预约点击参数", resourse,null, OperateObjectEnumType.APP.getCode());
		return ResultModel.OK(channelParamService.updatePreorder(resourse, loginAccount));
	}


	@RequestMapping(value = "update/impression", method = RequestMethod.PUT)
	@ResponseBody
	public ResultModel createImpression(@RequestParam boolean isdefault, @RequestParam boolean isall,
								   @RequestBody ChannelParam resourse, @CurrentAccount Account loginAccount) {
		return ResultModel.OK(channelParamService.updateImpression(resourse, loginAccount.getId()));
	}

	//回流归因回溯期设置
	@RequestMapping(value = "update/backflowAttrNum", method = RequestMethod.POST)
	@ResponseBody
	public ResultModel backflowAttrNum(@RequestBody ChannelParam resourse, @CurrentAccount Account loginAccount) {
		return ResultModel.OK(channelParamService.backflowAttrNum(resourse.getApp(),resourse.getBackflowAttrNum(), loginAccount.getId()));
	}
	//回流归因回溯期回显
	@RequestMapping(value = "getBackflowAttrNum", method = RequestMethod.GET)
	@ResponseBody
	public ResultModel getBackflowAttrNum(Long app) {
		return ResultModel.OK(channelParamService.getBackflowAttrNum(app));
	}




    @RequestMapping(value = "find/black/{app}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel find(@CurrentAccount Account loginAccount,@PathVariable Long app) {
        ChannelParam channelParam = channelParamService.find(loginAccount.getId(),app);
        return ResultModel.OK(channelParam);
    }
    @RequestMapping(value = "find/{cid}/{app}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findOne(@PathVariable Long cid,@PathVariable Long app, @CurrentAccount Account loginAccount) {
        ChannelParam one = channelParamService.findOne(cid, loginAccount.getId(),app);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "渠道参数", one,null,OperateObjectEnumType.CHANNELPARAM.getCode());
        return ResultModel.OK(one);
    }

    @RequestMapping(value = "find/default", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findDefault() {
        ChannelParam aDefault = channelParamService.findDefault();
        return ResultModel.OK(aDefault);
    }

	/**
	 * 创建某渠道公共ip名单
	 */

	@RequestMapping(value = "/create", method = RequestMethod.POST)
	@ResponseBody
    public ResultModel create(@RequestBody CommonIP resource, @CurrentAccount Account loginAccount) {
        //参数校验
        if (null == resource || StringUtils.isEmpty(resource.getIp())) {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

		//查询其母账户
		Account parentAccount = authService.findRootParentAccount(loginAccount.getId());

        if(commonIPService.checkCommonIp(parentAccount.getId(), resource.getIp())) {//验证新建公共ip，是否存在
        	return ResultModel.ERROR(ResultStatus.COMMON_IP_EXISTS);
        }

		// 创建公共ip
		CommonIP commonIP = commonIPService.save(parentAccount.getId(), resource.getIp(), loginAccount.getId());
		userLogService.insertLog(loginAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "某账号公共ip名单", commonIP);
		return ResultModel.OK(commonIP);
	}

	/**
	 * 修改某渠道公共ip
	 * @param request
	 * @param response
	 * @param resource
	 * @param account
	 * @return
	 */
	 @RequestMapping(value = "update", method = RequestMethod.PUT)
	 @ResponseBody
     public ResultModel update(HttpServletRequest request, HttpServletResponse response,
                              @RequestBody CommonIP resource, @CurrentAccount Account account) {
	    if(commonIPService.checkCommonIp(resource.getAccount(), resource.getIp())) {//验证编辑公共ip，是否存在
        	return ResultModel.ERROR(ResultStatus.COMMON_IP_EXISTS);
        }

	    CommonIP commonIP = commonIPService.update(resource.getIp(), account.getId(), resource.getId());
        userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "修改某渠道公共ip" + resource.getIp(), resource);
        return ResultModel.OK(commonIP);
     }

	 /**
	  * 删除某渠道公共ip
	  * @param id
	  * @param loginAccount
	  * @return
	  */
	 @RequestMapping(value = "/delete/{id}", method = RequestMethod.DELETE)
	 @ResponseBody
	 public ResultModel delete(@PathVariable String id,@CurrentAccount Account loginAccount) {

		if(org.apache.commons.lang.StringUtils.isNotBlank(id)) {
			String[] str = id.split(",");//批量删,id逗号分隔
			if(null!=str && str.length>0) {
				for(int i = 0 ;i<str.length; i++) {
					long commonIPId = Long.parseLong(str[i]);
					CommonIP commonIP = commonIPService.findoneById(commonIPId);

					if (null!= commonIP) {
						// 删除某渠道公共ip
						commonIPService.delete(loginAccount.getId(), commonIPId);
						userLogService.insertLog(loginAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "删除某渠道公共ip"+commonIP.getIp(), commonIP,null,OperateObjectEnumType.CHANNEL.getCode());
					}
				}
			}
		}

		return ResultModel.OK();
	 }

	 /**
	  * 查询某渠道下公共ip列表
	  * @param request
	  * @param loginAccount
	  * @return
	  */
	 @RequestMapping(value = "find", method = RequestMethod.GET)
	 @ResponseBody
	 public ResultModel list(HttpServletRequest request, @CurrentAccount Account loginAccount) {
		    //查询其母账户
	        Account parentAccount = authService.findRootParentAccount(loginAccount.getId());

	        List<CommonIP> list = commonIPService.listCommonIPByAccount(parentAccount.getId());
//	        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "查询某渠道下公共ip列表", null, null, OperateObjectEnumType.APP.getCode());
	        return ResultModel.OK(list);
	 }

}
