package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.dic.WhatEventEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AdwordsService;
import com.reyun.service.AuthService;
import com.reyun.service.DataCallBackService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.StringUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Controller
@RequestMapping("/mng/datacb")
public class MngDataCallBackController {
    protected Logger logger = LoggerFactory
            .getLogger(MngDataCallBackController.class);

    @Autowired
    DataCallBackService dataCallBackService;

    @Autowired
    UserLogService userLogService;

    @Autowired
    AuthService authService;

    @Autowired
    CallbackCustomizeRepository callbackCustomizeRepository;


    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private ChannnelAccountLinkAppRepository channnelAccountLinkAppRepository;

    @Autowired
    private AdwordsService adwordsService;

    @Autowired
    private CallbackNextdayRepository callbackNextdayRepository;

    @Autowired
    private CallbackFirstdayPaymentRepository callbackFirstdayPaymentRepository;


    @RequestMapping(value = "find/params", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listparams(HttpServletRequest request) {
        return ResultModel.OK(dataCallBackService.listParams());
    }


    @RequestMapping(value = "find/events/{channel}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listevents(HttpServletRequest request, @RequestHeader(name = "lang", defaultValue = "zh") String lanType,
                                  @PathVariable Long channel, @CurrentAccount Account account) {
        return ResultModel.OK(dataCallBackService.listEvents(channel, lanType, account.getId()));
    }

    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("datacb")
    public ResultModel create(HttpServletRequest request, @RequestBody CallbackCustomize resource, @RequestParam(required = false) Boolean iscustom, @CurrentAccount Account currentAccount) {
        boolean isFirstdayPayment = false;
        if (WhatEventEnumType.NEXTDAY.getId().equals(resource.getWhat())) {
            List<CallbackNextday> nextdays = callbackNextdayRepository.findByAppChannel(resource.getApp(), resource.getChannel(), resource.getWhat());
            if (nextdays.size() > 0) {
                return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
            }
        } else {
            if ("firstdayPayment".equals(resource.getWhat())) {
                resource.setWhat("payment");
                isFirstdayPayment = true;
            }
            List<CallbackCustomize> cbList = callbackCustomizeRepository.findByChannelAndEvent(resource.getChannel(), resource.getWhat(), resource.getApp());
            if (cbList.size() > 0) {
                return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
            }
        }

        resource.setActive(true);

        //adwords渠道试回调
        if (resource.getChannel().equals(new Long(5303))) {
            List<ChannelAccountLinkApp> linkList = channnelAccountLinkAppRepository.findByAppkey(appRepository.findAppkeyById(resource.getApp()));
            if (linkList.size() == 0) {
                ResultStatus status = ResultStatus.ADWORDS_LINKID_FAIL;
                status.setMessage("Linkid is null.");
                return ResultModel.ERROR(status);
            }
//			boolean tryResult = true;
            for (ChannelAccountLinkApp linkapp : linkList) {
                String errors = adwordsService.isValid(linkapp.getUserActionSetId(), resource.getWhat(), linkapp.getAppType());
                if (errors.length() > 2) {
                    ResultStatus status = ResultStatus.ADWORDS_LINKID_FAIL;
                    status.setMessage("Failed to send event to adwords with linkid " + linkapp.getUserActionSetId() + " and " + resource.getWhat() + ".\n" + errors);
                    return ResultModel.ERROR(status);
                }
            }
        }

        CallbackCustomize callbackCustomize = dataCallBackService.save(resource, currentAccount.getId(), iscustom);
        if(!StringUtils.isEmpty(callbackCustomize.getErroMessage())){
            return ResultModel.ERROR(callbackCustomize.getErroMessage());
        }

        if (callbackCustomize != null && isFirstdayPayment) {
            CallbackFirstdayPayment callbackFirstdayPayment = new CallbackFirstdayPayment();
            callbackFirstdayPayment.setAppkey(appRepository.findOne(callbackCustomize.getApp()).getAppkey());
            callbackFirstdayPayment.setCid(callbackCustomize.getChannel());
            callbackFirstdayPayment.setCreateAccount(currentAccount.getId());
            callbackFirstdayPayment.setCallbackCustomizeId(callbackCustomize.getId());
            callbackFirstdayPaymentRepository.save(callbackFirstdayPayment);
        }
        userLogService.insertLog(currentAccount, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "推广回调" + callbackCustomize.getName(), callbackCustomize, callbackCustomize.getApp(), OperateObjectEnumType.CALLBACK.getCode());
        return ResultModel.OK(callbackCustomize);
    }

    @RequestMapping(value = "delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    @ResponseStatus(HttpStatus.NO_CONTENT)
    @AuthorizationMenu("datacb")
    public ResultModel delete(HttpServletRequest request, String what,
                              @PathVariable Long id, @CurrentAccount Account currentAccount) {
        try {

            if (WhatEventEnumType.NEXTDAY.getId().equals(what)) {
                // 0 关闭 1 开启 2 删除
                dataCallBackService.setNextActiveData(id, 2);
                return ResultModel.OK();
            }

            CallbackCustomize callbackCustomize = dataCallBackService.delete(id, currentAccount.getId());
            if ("firstdayPayment".equals(what) && callbackCustomize != null && callbackCustomize.getChannel().longValue() == 514L) {
                CallbackFirstdayPayment callbackFirstdayPayment = callbackFirstdayPaymentRepository.findOne(callbackCustomize.getId());
                if (callbackFirstdayPayment != null) {
                    callbackFirstdayPaymentRepository.delete(callbackFirstdayPayment);
                }
            }
            userLogService.insertLog(currentAccount, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "推广回调" + callbackCustomize.getName(), callbackCustomize, callbackCustomize.getApp(), OperateObjectEnumType.CALLBACK.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "update", method = RequestMethod.PUT)
    @ResponseBody
    @AuthorizationMenu("datacb")
    public ResultModel update(HttpServletRequest request, HttpServletResponse response, @RequestBody CallbackCustomize resource, @RequestParam(required = false) Boolean iscustom, @CurrentAccount Account currentAccount) {
        boolean isFirstdayPayment = false;
        if (resource.getChannel().longValue() == 514L && "firstdayPayment".equals(resource.getWhat())) {
            resource.setWhat("payment");
            isFirstdayPayment = true;
        }
        if (WhatEventEnumType.NEXTDAY.getId().equals(resource.getWhat())) {
            List<CallbackNextday> nextdays = callbackNextdayRepository.findByAppChannel(resource.getApp(), resource.getChannel(), resource.getWhat());
            if (nextdays != null && nextdays.size() > 0) {
                if (nextdays.size() == 1) {
                    CallbackNextday customize = nextdays.get(0);
                    if (customize.getId() != resource.getId()) {
                        return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
                    }
                } else {
                    return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
                }
            }
        } else {
//			List<CallbackCustomize> cbList = callbackCustomizeRepository.findByChannelAndEvent(resource.getChannel(), resource.getWhat(), resource.getApp());
//			if (cbList!=null && cbList.size() > 0) {
//				if (cbList.size() == 1) {
//					CallbackCustomize customize = cbList.get(0);
//					if (customize.getId() != resource.getId()) {
//						return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
//					}
//				} else {
//					return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_EXIST);
//				}
//			}

            CallbackCustomize customizeEx = callbackCustomizeRepository.findOne(resource.getId());

            if (customizeEx == null) {
                return new ResultModel(ResultStatus.CUSTOMIZE_CALLBACK_NOTEXIST);
            } else {
                if (customizeEx.getWhat().equals("payment") && !isFirstdayPayment) {
                    CallbackFirstdayPayment one = callbackFirstdayPaymentRepository.findOne(customizeEx.getId());
                    if (one != null) {
                        callbackFirstdayPaymentRepository.delete(one);
                    }
                }
            }
        }

        //adwords渠道试回调
        if (resource.getChannel().equals(new Long(5303))) {
            List<ChannelAccountLinkApp> linkList = channnelAccountLinkAppRepository.findByAppkey(appRepository.findAppkeyById(resource.getApp()));
//			boolean tryResult = true;
            for (ChannelAccountLinkApp linkapp : linkList) {
                String errors = adwordsService.isValid(linkapp.getUserActionSetId(), resource.getWhat(), linkapp.getAppType());
                if (errors.length() > 2) {
                    ResultStatus status = ResultStatus.ADWORDS_LINKID_FAIL;
                    status.setMessage("Failed to send event to adwords with linkid " + linkapp.getUserActionSetId() + " and " + resource.getWhat() + ".\n" + errors);
                    return ResultModel.ERROR(status);
                }
            }
        }
        CallbackCustomize callbackCustomize = dataCallBackService.update(resource, currentAccount.getId(), iscustom);
        if(!StringUtils.isEmpty(callbackCustomize.getErroMessage())){
            return ResultModel.ERROR(callbackCustomize.getErroMessage());
        }

        if (isFirstdayPayment) {
            CallbackFirstdayPayment one = callbackFirstdayPaymentRepository.findOne(callbackCustomize.getId());
            if (one == null) {
                CallbackFirstdayPayment callbackFirstdayPayment = new CallbackFirstdayPayment();
                callbackFirstdayPayment.setCreateAccount(currentAccount.getId());
                callbackFirstdayPayment.setCid(callbackCustomize.getChannel());
                callbackFirstdayPayment.setCallbackCustomizeId(callbackCustomize.getId());
                callbackFirstdayPayment.setAppkey(appRepository.findOne(callbackCustomize.getApp()).getAppkey());
                callbackFirstdayPaymentRepository.save(callbackFirstdayPayment);
            }
        }
        userLogService.insertLog(currentAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广回调" + callbackCustomize.getName(), callbackCustomize, callbackCustomize.getApp(), OperateObjectEnumType.CALLBACK.getCode());
        return ResultModel.OK(callbackCustomize);
    }

    @RequestMapping(value = "find/{appid}", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("datacb")
    public ResultModel list(@CurrentAccount Account account, @PathVariable Long appid) {
        //logger.info(account.toString());
        List<CallbackCustomize> list = new ArrayList<>();
        if (account.getIsSuperUser()
                || RoleEnumType.MANAGER.getKey().equals(account.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(account.getRoleCategory())) {
            list = dataCallBackService.listAll(appid);
        } else {
            list = dataCallBackService.list(account.getId(), appid);
        }

        //查询次留信息
        List<CallbackCustomize> nextdata = dataCallBackService.nextCallApp(appid);
        if (nextdata != null) {
            list.addAll(nextdata);
        }

        //排序
        Collections.sort(list, new Comparator<CallbackCustomize>() {
            @Override
            public int compare(CallbackCustomize o1, CallbackCustomize o2) {

                return o1.getCreateTime().compareTo(o2.getCreateTime()) * (-1);
            }
        });

        List<Long> channels = new ArrayList<>(list.size());
        for (CallbackCustomize call : list) {
            channels.add(call.getChannel());
        }

        List<Channel> channelList = new ArrayList<>();
        if (channels.size() > 0) {
            channelList = channelRepository.findById(channels);
        }


        Map<Long, String> channelMap = new HashMap<>(channelList.size());
        for (Channel channel : channelList) {
            channelMap.put(channel.getId(), channel.getName());
        }

        for (CallbackCustomize customize : list) {
            if (customize.getChannel().longValue() == 514L && "payment".equals(customize.getWhat()) && callbackFirstdayPaymentRepository.findOne(customize.getId()) != null) {
                customize.setWhat("firstdayPayment");
            }

            if (customize.getHasPaymentAmount() == null) {
                customize.setHasPaymentAmount(0);
            }

            customize.setChannelName(channelMap.get(customize.getChannel()));
        }


        userLogService.insertLog(account, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广回调列表", null, appid, OperateObjectEnumType.CALLBACK.getCode());
        return ResultModel.OK(list);
    }

    @RequestMapping(value = "enable/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel enable(HttpServletRequest request, @RequestBody CallbackCustomize customize,
                              @PathVariable Long id, @CurrentAccount Account currentAccount) {

        try {

            if (customize != null && WhatEventEnumType.NEXTDAY.getId().equals(customize.getWhat())) {
                // 0 关闭 1 开启 2 删除
                dataCallBackService.setNextActiveData(id, 1);
                return ResultModel.OK();
            }

            CallbackCustomize callbackCustomize = dataCallBackService.enable(id, currentAccount.getId());
            if (customize.getWhat().equals("firstdayPayment")) {
                CallbackFirstdayPayment one = callbackFirstdayPaymentRepository.findOne(callbackCustomize.getId());
                if (one == null) {
                    CallbackFirstdayPayment callbackFirstdayPayment = new CallbackFirstdayPayment();
                    callbackFirstdayPayment.setCreateAccount(currentAccount.getId());
                    callbackFirstdayPayment.setCid(callbackCustomize.getChannel());
                    callbackFirstdayPayment.setCallbackCustomizeId(callbackCustomize.getId());
                    callbackFirstdayPayment.setAppkey(appRepository.findOne(callbackCustomize.getApp()).getAppkey());
                    callbackFirstdayPaymentRepository.save(callbackFirstdayPayment);
                }
            }
            userLogService.insertLog(currentAccount, LogEnumType.UPDATE.getCode(), "启用推广回调" + callbackCustomize.getName(), callbackCustomize, callbackCustomize.getApp(), OperateObjectEnumType.CALLBACK.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "forbidden/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel forbidden(HttpServletRequest request, @RequestBody CallbackCustomize customize,
                                 @PathVariable Long id, @CurrentAccount Account currentAccount) {

        try {

            if (customize != null && WhatEventEnumType.NEXTDAY.getId().equals(customize.getWhat())) {
                // 0 关闭 1 开启 2 删除
                dataCallBackService.setNextActiveData(id, 0);
                return ResultModel.OK();
            }

            CallbackCustomize callbackCustomize = dataCallBackService.forbidden(id, currentAccount.getId());
			/*if(customize.getWhat().equals("firstdayPayment")){
				callbackFirstdayPaymentRepository.delete(callbackCustomize.getId());
			}*/
            userLogService.insertLog(currentAccount, LogEnumType.UPDATE.getCode(), "停用推广回调" + callbackCustomize.getName(), callbackCustomize, callbackCustomize.getApp(), OperateObjectEnumType.CALLBACK.getCode());
        } catch (Exception e) {
            logger.error("授权失败", e);
            throw new TipException("授权失败");
        }
        return ResultModel.OK();
    }

    @RequestMapping(value = "valid/{app}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@RequestParam String name, @PathVariable Long app, @CurrentAccount Account account) {
        Boolean flag = dataCallBackService.valid(account.getId(), app, name);
        if (!flag) {
            return ResultModel.OK(false);
        } else {
            return ResultModel.OK(true);
        }

    }

    /**
     * 该app下有没有关联的linkid
     */
    @RequestMapping(value = "/validlinkid/{app}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel findByApp(@PathVariable Long app) {
        List<ChannelAccountLinkApp> list = new ArrayList<>();
        list = channnelAccountLinkAppRepository.findByAppkey(appRepository.findOne(app).getAppkey());
        return ResultModel.OK(list.size());
    }
}
