package com.reyun.controller.manage;

import com.google.common.base.Preconditions;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.CallbackDisable;
import com.reyun.repository.CallbackDisableRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.DataCallbackSettingService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by nolan on 14/02/2017.
 * description:
 */
@Controller
@RequestMapping("/mng/datacb/install/{appid}")
public class MngDataCallbackSettingController
{
    @Autowired
    UserLogService userLogService;
    @Autowired
    private CallbackDisableRepository callbackDisableRepository;
    @Autowired
    private DataCallbackSettingService dataCallbackSettingService;

    @RequestMapping(method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listChannel(@PathVariable Long appid, @CurrentAccount Account currentAccount ,@RequestHeader(name = "lang", defaultValue = "zh") String lanType)
    {
        return ResultModel.OK(dataCallbackSettingService.findAll(appid, currentAccount,lanType));
    }

    @RequestMapping(value = "/setting", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel opt(@PathVariable Long appid, @RequestParam Long channelid, @RequestParam Boolean button, @CurrentAccount Account loginAccount)
    {
        Preconditions.checkArgument(appid != null && channelid != null, "缺少参数(appid, channelid)");

        if (button) { //打开激活回调
            this.callbackDisableRepository.delete(appid, channelid);
            userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), "打开激活回调", null, appid, OperateObjectEnumType.CALLBACK.getCode());
        }
        else { //关闭激活回调
            List<CallbackDisable> itemlist = this.callbackDisableRepository.find(appid, channelid);
            if (itemlist == null || itemlist.size() == 0) {
                CallbackDisable callbackDisable = new CallbackDisable();
                callbackDisable.setAccount(loginAccount.getId());
                callbackDisable.setApp(appid);
                callbackDisable.setChannel(channelid);
                this.callbackDisableRepository.saveAndFlush(callbackDisable);

                userLogService.insertLog(loginAccount, LogEnumType.UPDATE.getCode(), "关闭激活回调", null, appid, OperateObjectEnumType.CALLBACK.getCode());
            }
        }
        return ResultModel.OK();
    }
}
