package com.reyun.controller.manage;

import com.amazonaws.HttpMethod;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.GeneratePresignedUrlRequest;
import com.google.common.collect.Lists;
import com.reyun.dic.*;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.AppRepository;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.*;
import com.reyun.service.impl.DownloadSizeService;
import com.reyun.util.*;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

/**
 * Created by nolan on 03/01/2017.
 * description:
 */
@Controller
@RequestMapping("/mng/export/{app}")
public class MngDownloadController {
    private final String tpl_file = "%s_%s_%s_%s.tar.gz";
    private final String tpl_file_language = "%s_%s_%s_.tar.gz";
    private final String tpl_file_extent = ".tar.gz";
    @Autowired
    private IDownloadService iDownloadService;
    @Autowired
    private AppService appService;
    @Autowired
    private CampaignService campaignService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    private CampaignGroupService campaignGroupService;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ConfigParamService configparamService;
    @Autowired
    UserLogService userLogService;
    @Autowired
    BlacklistService blacklistService;

    @Autowired
    DownloadSizeService downloadSizeService;

    private static final Map<String, String> map = new HashMap();

    static {
        map.put("install", "激活");
        map.put("reged", "注册");
        map.put("pay", "付费");
        map.put("backflow", "回流");
        map.put("order", "订单");
        map.put("fake", "防作弊");
        map.put("usergroup", "用户群");
        map.put("launch", "打开小程序");
        map.put("register", "小程序注册");
        map.put("payment", "小程序付费");
        map.put("loggedin", "登录");
        map.put("h5_event", "H5自定义事件");
        map.put("hijack", "点击劫持分析");
        map.put("hijackpack", "分包劫持分析");
        map.put("corpse", "僵尸用户分析");
        map.put("silent", "沉默");
        map.put("loss", "流失");
        map.put("backflow_analysis", "回流分析-回流");
        map.put("backflow_channel", "渠道概览-回流");
        //再营销模块
        map.put("dp_install", "调起");
        map.put("dp_reg", "注册");
        map.put("dp_lognin", "登录");
        map.put("dp_pay", "付费");
        map.put("duration", "单次使用时长");

        //推广数据、防作弊卫士、用户生命周期、投放广告详情
        map.put("campaigninfo", "推广数据");
        map.put("advertisinginfo", "投放广告详情");
        map.put("menufake", "防作弊卫士");
        map.put("userLife", "用户生命周期");
        map.put("userengagement", "用户参与度");
        map.put("remarketing", "再营销");

    }

    @RequestMapping(value = "geturl", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("logdump")
    public ResultModel getUrl(HttpServletRequest request, @PathVariable Long app, @CurrentAccount Account loginAccount,
                              @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        App appObject = appRepository.findOne(app);
        String token = appObject.getToken();
        List<String> name = new ArrayList<>();
        name.add(isEnLaguage(lanType) ? "Type" : "类型");
        name.add(isEnLaguage(lanType) ? "Pull up log address (click to copy)" : "拉取日志地址（点击即可复制）");
        List<String> columnKey = new ArrayList<>();
        columnKey.add("type");
        columnKey.add("url");

        List<Map<String, Object>> values = new ArrayList<>();

        //H5只有click
        List<ConfigParam> configParamList;
        //判断 点击是由安卓还是ios或者微信发起的
        if (AppPlatformEnum.H5.getKey().equals(appObject.getPlatform())) {
            configParamList = configparamService.getConfigParamByKeys(ConfigEnumType.SERVER_CLICK_DOWNLOAD_URL.getKey());
        } else {
            List<String> downloadType = Lists.newArrayList();
            downloadType.add(ConfigEnumType.SERVER_INSTALL_DOWNLOAD_URL.getKey());
            downloadType.add(ConfigEnumType.SERVER_REGED_DOWNLOAD_URL.getKey());
            // downloadType.add(ConfigEnumType.SERVER_CLICK_DOWNLOAD_URL.getKey());
            downloadType.add(ConfigEnumType.SERVER_PAY_DOWNLOAD_URL.getKey());
            downloadType.add(ConfigEnumType.SERVER_DEVICELOGIN_DOWNLOAD_URL.getKey());
            //当前用户等级是否为管理员或者母账号
            if (loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || loginAccount.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey())) {
                //母账号与管理员账号其一有防作弊功能就显示该功能
                BlacklistApplication blacklistApplication = blacklistService.findOneUsed(loginAccount);
                if (blacklistApplication == null) {
                    blacklistApplication = blacklistService.findOneByParentId(loginAccount.getParent());
                }
                if (null != blacklistApplication && blacklistApplication.getPastDate() != null && DateUtil.compareDate(DateUtil.getCurrentDateStr(DateUtil.C_DATE_PATTON_DEFAULT), DateUtil.addOneDay(blacklistApplication.getPastDate(), DateUtil.C_DATE_PATTON_DEFAULT)) != 1) {
                    downloadType.add(ConfigEnumType.SERVER_BLACKLIST_DOWNLOAD_URL.getKey());
                    downloadType.add(ConfigEnumType.SERVER_FAKE_INSTALL_DOWNLOAD_URL.getKey());
                }
            }
            if (appObject.getIndustory().equals("ec")) {
                downloadType.add(ConfigEnumType.ORDER_DOWNLOAD_URL.getKey());
            }
            String[] strings = downloadType.toArray(new String[downloadType.size()]);
            //根据install_download_url,reged_download_url,click_download_url等字段去config_param查对应的信息
            configParamList = configparamService.getConfigParamByKeys(downloadType.toArray(new String[downloadType.size()]));
        }
        for (ConfigParam config : configParamList) {

            Map<String, Object> map = new HashMap<>();

            String type = "";
            if (config.getKeyParam().equals(ConfigEnumType.SERVER_INSTALL_DOWNLOAD_URL.getKey())) {
                //激活
//                type = ConfigEnumType.SERVER_INSTALL_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_INSTALL_DOWNLOAD_URL, lanType);

                if ("selfBuilt".equals(appObject.getPlatform())) type = "表单";

            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_REGED_DOWNLOAD_URL.getKey())) {
                //注册
//                type = ConfigEnumType.SERVER_REGED_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_REGED_DOWNLOAD_URL, lanType);

            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_CLICK_DOWNLOAD_URL.getKey())) {
                //点击
//                type = ConfigEnumType.SERVER_CLICK_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_CLICK_DOWNLOAD_URL, lanType);
            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_BLACKLIST_DOWNLOAD_URL.getKey())) {
                //黑名单
//                type = ConfigEnumType.SERVER_BLACKLIST_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_BLACKLIST_DOWNLOAD_URL, lanType);
            } else if (config.getKeyParam().equals(ConfigEnumType.ORDER_DOWNLOAD_URL.getKey())) {
                //订单
//                type = ConfigEnumType.ORDER_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.ORDER_DOWNLOAD_URL, lanType);
            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_PAY_DOWNLOAD_URL.getKey())) {
                //付费
//                type = ConfigEnumType.SERVER_PAY_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_PAY_DOWNLOAD_URL, lanType);
            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_DEVICELOGIN_DOWNLOAD_URL.getKey())) {
                //设备登录日志
//                type = ConfigEnumType.SERVER_DEVICELOGIN_DOWNLOAD_URL.getValue();
                type = getURLType(ConfigEnumType.SERVER_DEVICELOGIN_DOWNLOAD_URL, lanType);
            } else if (config.getKeyParam().equals(ConfigEnumType.SERVER_FAKE_INSTALL_DOWNLOAD_URL.getKey())) {
                //防作弊拦截激活
                type = getURLType(ConfigEnumType.SERVER_FAKE_INSTALL_DOWNLOAD_URL, lanType);
            }

            map.put("type", type);
            if (StringUtil.isDemo(loginAccount.getEmail())) {
                map.put("url", "http://"+Constant.getDomainStr()+"/apiurl?token=***********");
            } else {
                map.put("url", config.getValueParam() + "?token=" + token);
            }
            values.add(map);
        }

        Map<String, List> result = new HashMap<>();
        result.put("name", name);
        result.put("columnkey", columnKey);
        result.put("val", values);
        return ResultModel.OK(result);

    }

    private String getURLType(ConfigEnumType configEnumType, String lanType) {
        if (StringUtils.equals(LanguageTypeEnum.ENGLISH.getCode(), lanType)) {
            return configEnumType.getEn_value();
        }
        return configEnumType.getValue();
    }

    @RequestMapping(value = "findall", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("logexport")
    public ResultModel findAll(@PathVariable Long app, @CurrentAccount Account loginAccount) {
        //userLogService.insertLog(loginAccount.getId(), LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "下载列表", null,null,OperateObjectEnumType.PROFILE.getCode());
        return ResultModel.OK(iDownloadService.find(app, loginAccount));
    }

    @RequestMapping(value = "/delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public ResultModel delete(@PathVariable Long id, @CurrentAccount Account loginAccount) {
        DownloadInfo result = this.iDownloadService.delete(id);
        //userLogService.insertLog(loginAccount.getId(), LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "下载" + result.getFilename(), result);
        return ResultModel.OK();
    }

    @RequestMapping(value = "generatefile", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("logexport")
    public ResultModel addInstallIdfaTask(@PathVariable("app") Long app, @RequestBody DownloadInfo resource,
                                          @CurrentAccount Account loginAccount, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
        App appEntity = appService.findById(app);
        if (appEntity == null) {
            throw new TipException("应用不存在");
        }

        final String downloadtype = resource.getDownloadtype();
        final String divtype = resource.getDivtype();
        final Long busiid = resource.getBusiid();

        DownloadInfo downloadInfo = buildDownloadInfo(app, downloadtype, resource.getDivtype(), busiid,
                resource.getStartdate(), resource.getEnddate(), loginAccount, appEntity, resource.getFakeRange());
        if (DownloadEnumType.INSTALL.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.REGED.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.PAY.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.BACKFLOW.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.ORDER.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.LAUNCH.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.REGISTER.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.PAYMENT.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.FAKE.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.LOGINEDIN.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.HIJACK.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.HIJACKPACK.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.CORPSE.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.BACKFLOW_ANALYSIS.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.BACKFLOW_CHANNEL.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.SILENT.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.LOSS.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.LOSS.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.DP_INSTALL.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.DP_LOGNIN.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.DP_PAY.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.DP_REG.getCode().equalsIgnoreCase(downloadtype)) {
        } else if (DownloadEnumType.DURATION.getCode().equalsIgnoreCase(downloadtype)) {

        } else if (DownloadEnumType.H5EVENTS.getCode().equalsIgnoreCase(downloadtype)) {
            resource.setDivtype(DivEnumType.ALL.getCode());

        } else {
            throw new TipException(downloadtype + ",暂未支持此业务数据下载");
        }

        String all_defualt = isEnLaguage(lanType) ? "All" : "全部";
        String oragino_defualt = isEnLaguage(lanType) ? "Organic" : "自然量";

        String businame = "";
        if (DivEnumType.ALL.getCode().equals(resource.getDivtype())) {
            businame = all_defualt;
        } else if (DivEnumType.DEFAULT.getCode().equals(resource.getDivtype())) {
            businame = oragino_defualt;
        } else if (DivEnumType.CAMPAIGN.getCode().equals(resource.getDivtype())) {
            Campaign campaign = this.campaignService.findOneById(busiid);
            String name = null;
            if (campaign != null && campaign.getName() != null) {
                name = campaign.getName().replace("、", "");
            }
            businame = name != null ? name : all_defualt;
        } else if (DivEnumType.CAMPAIGN_GROUP.getCode().equals(resource.getDivtype())) {
            CampaignGroup campaignGroup = this.campaignGroupService.findOneById(busiid);
            businame = campaignGroup != null ? campaignGroup.getName() : all_defualt;
        } else if (DivEnumType.CHANNEL.getCode().equals(divtype)) {
            Channel channel = this.channelService.findoneById(busiid);
            businame = channel != null ? channel.getName() : all_defualt;
        } else {
            throw new TipException(divtype + "此类型不存在");
        }

        String dataRange = null;
        if (downloadInfo.getFakeRange() != null) {
            dataRange = DownloadEnumType.HIJACK.getCode().equalsIgnoreCase(downloadtype)
                    ? downloadInfo.getFakeRange() + "s" : "c" + downloadInfo.getFakeRange();
        }

        String filename = String.format(tpl_file, appEntity.getAppkey(),
                dataRange == null ? businame : businame + "," + dataRange,
                downloadInfo.getStartdate(), downloadInfo.getEnddate());
        if (map.containsKey(downloadtype)) {
            filename = map.get(resource.getMenuType()) + "_" + map.get(downloadtype) + "_" + filename;
        }
        downloadInfo.setFilename(filename);
        downloadInfo.setBusiname(businame);

        //语种类型保存
        downloadInfo.setLanguageType(lanType);
        downloadInfo.setMenuType(resource.getMenuType());
        //end
        if (iDownloadService.exist(loginAccount.getId(), filename, downloadInfo)) {
            return ResultModel.ERROR(ResultStatus.EXPORT_DATA_EXIST);
        } else if (downloadSizeService.checkByteSize(downloadInfo)) {
            return ResultModel.ERROR(ResultStatus.EXPORT_DATA_BYTESIZE_LARGE);
        } else {
            return ResultModel.OK(iDownloadService.save(downloadInfo));
        }
    }

    private boolean isEnLaguage(String type) {
        return StringUtils.equals(LanguageTypeEnum.ENGLISH.getCode(), type);
    }

    @RequestMapping(value = "refresh/{id}", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel refreshDownload(@PathVariable("app") Long app, @PathVariable Long id, @CurrentAccount Account loginAccount) throws IOException {
        DownloadInfo downloadInfo = this.iDownloadService.findById(id);
        if (downloadInfo == null)
            throw new TipException("下载记录不存在");

        iDownloadService.updateStatus(id, DownloadStatusEnumType.INIT);
        downloadInfo.setStatus(DownloadStatusEnumType.INIT.getCode());

        return ResultModel.OK(downloadInfo);
    }

    @RequestMapping(value = "download/{id}", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("logexport")
    public ResultModel getDownloadResponseEntity(HttpServletRequest request, HttpServletResponse response,
                                                 @PathVariable("app") Long app, @PathVariable Long id, @CurrentAccount Account loginAccount) {
        DownloadInfo downloadInfo = this.iDownloadService.findById(id);

        userLogService.insertLog(loginAccount, LogEnumType.DOWNLOAD.getCode(), LogEnumType.DOWNLOAD.getName() + "日志导出 ",
                downloadInfo, app, OperateObjectEnumType.LOGEXPORT.getCode(), request);

        if (downloadInfo == null) {
            throw new TipException("下载记录不存在");
        }
        String objectKey = "trackingio/" + downloadInfo.getDownloadtype().toLowerCase() + "/"
                + downloadInfo.getFilename();
        // 检查文件是否存在
        if (!ObjectStorageUtil.doesObjectExist(Constant.bucket, objectKey)) {
            return ResultModel.OK(false);
        }
        return ResultModel.OK(ObjectStorageUtil.generatePresignedUrl(Constant.bucket, objectKey, 1000 * 60 * 2));
    }

    //该检查方法弃用，会占用内存
    @RequestMapping(value = "checkexist/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel checkFileExist(@PathVariable("app") Long app, @PathVariable Long id, @CurrentAccount Account loginAccount) throws MalformedURLException {
        DownloadInfo downloadInfo = this.iDownloadService.findById(id);
        if (downloadInfo == null)
            throw new TipException("下载记录不存在");

        String filename = downloadInfo.getFilename();
        if (StringUtils.equals(downloadInfo.getLanguageType(), LanguageTypeEnum.ENGLISH.getCode()) && filename != null && filename.indexOf(".") > 0) {
            String noExtent = filename.substring(0, filename.indexOf("."));
            String extent = filename.substring(filename.indexOf("."));
            downloadInfo.setFilename(noExtent + "_" + LanguageTypeEnum.ENGLISH.getCode() + extent);
        }
        InputStream in = null;
        try {
            in = ObjectStorageUtil.getObjectContentStream(Constant.bucket,
                    "trackingio/" + downloadInfo.getDownloadtype().toLowerCase() + "/" + downloadInfo.getFilename());
        } catch (Exception e) {
            return ResultModel.OK(false);
        } finally {
            if (in != null)
                IOUtils.closeQuietly(in);
        }
        return ResultModel.OK(true);
    }

    private DownloadInfo buildDownloadInfo(Long app, String downloadtype, String divtype, Long busiid, String startdate, String enddate, Account loginAccount, App appEntity, Integer fakeRange) {
        DownloadInfo downloadInfo = new DownloadInfo();
        downloadInfo.setApp(app);
        downloadInfo.setDownloadtype(downloadtype);
        downloadInfo.setDivtype(divtype);
        downloadInfo.setBusiid(busiid);
        downloadInfo.setAccount(loginAccount.getId());
        downloadInfo.setAppkey(appEntity.getAppkey());
        downloadInfo.setDs(DateTime.now().toString("yyyy-MM-dd"));
        if (!DownloadEnumType.USERGROUP.getCode().equalsIgnoreCase(downloadtype)) {
            downloadInfo.setStartdate(DateTime.parse(startdate).toString("yyyy-MM-dd"));
            downloadInfo.setEnddate(DateTime.parse(enddate).toString("yyyy-MM-dd"));
        }
        downloadInfo.setStatus(DownloadStatusEnumType.INIT.getCode());

        downloadInfo.setCreateAccount(loginAccount.getId());
        downloadInfo.setCreateTime(new Date());
        downloadInfo.setFakeRange(fakeRange);
        //userLogService.insertLog(loginAccount.getId(), LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "下载" + downloadInfo.getFilename(), downloadInfo);
        return downloadInfo;
    }

    public static void main(String[] args) {

        String filename = "abc.tar.gz";

        String noExtent = filename.substring(0, filename.indexOf("."));
        String extent = filename.substring(filename.indexOf(".") + 1);

        System.out.println(noExtent);
        System.out.println(extent);

    }
}


