package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.*;
import com.reyun.repository.ChannelRepository;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AppService;
import com.reyun.service.CampaignService;
import com.reyun.service.UserLogService;
import com.reyun.service.impl.OnelinkService;
import com.reyun.util.Constant;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.*;

@Controller
@RequestMapping("/mng/onelink/{appid}/")
public class MngOnelinkController {
    protected Logger logger = LoggerFactory.getLogger(MngOnelinkController.class);

    @Autowired
    private OnelinkService onelinkService;

    @Autowired
    private CampaignService campaignService;
    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    UserLogService userLogService;

    @Autowired
    AppService appService;

    @RequestMapping(value = "valid", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@PathVariable Long appid, @CurrentAccount Account loginAccount) {
        int app = onelinkService.validAppleId(loginAccount.getId(), appid);

        return ResultModel.OK(app);

    }

    @RequestMapping(value = "valid/name", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel check(@PathVariable Long appid, @RequestParam String name, @RequestParam(required = false) Long id) {
        Boolean app = onelinkService.validName(appid, name, id);

        return ResultModel.OK(app);

    }

    @RequestMapping(value = "find/apps", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listApps(HttpServletRequest request, @CurrentAccount Account loginAccount,
                                @RequestParam String platform, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        List<App> apps = onelinkService.listApps(loginAccount.getId(), platform, lanType);
        //userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "onelink中app列表", null, null, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(apps);
    }

    @RequestMapping(value = "find/channels", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listChannels(HttpServletRequest request, @CurrentAccount Account loginAccount,
                                    @PathVariable Long appid, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        List<Channel> channels = onelinkService.listChannels(appid, lanType);
        //添加onlink时 Android选择: 中自定义渠道需要展示分包下已创建过活动的自定义渠道
        Iterable<Campaign4Web> custom = campaignService.listAuthCampaignByType(loginAccount, appid, "custom", false);
        List<Long> cids = new ArrayList<>();
        for (Campaign4Web campaign4Web : custom) {
            if (cids.contains(campaign4Web.getChannel())) continue;
            if ("APPMARKET".equals(campaign4Web.getChannelCategory()) && campaign4Web.getSubCampaignNumber() > 0) {
                cids.add(campaign4Web.getChannel());
            }
        }
        if (!CollectionUtils.isEmpty(cids)) {
            List<Channel> pkgchannels = channelRepository.findById(cids);
            channels.addAll(pkgchannels);
        }
//		logger.info(channels.toString());
        //userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "onelink列表", null, null, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(channels);
    }

    @RequestMapping(value = "find/campaigns/{channel}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listCampaigns(HttpServletRequest request, @CurrentAccount Account loginAccount,
                                     @PathVariable Long appid, @PathVariable Long channel, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        List<Campaign4Web> campaigns = onelinkService.listCampaigns(appid, channel, lanType);
        //userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "onelink列表", null, null, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(campaigns);
    }

    @RequestMapping(value = "find", method = RequestMethod.GET)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel list(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long appid) {
//		userLogService.insertLog(loginAccount.getId(), LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "应用列表", null);
//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "onelink列表", null, null, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(onelinkService.find(loginAccount.getId(), appid));
    }

    @RequestMapping(value = "find/{id}", method = RequestMethod.GET)
    @ResponseBody
    public ResultModel listOne(HttpServletRequest request, @CurrentAccount Account loginAccount, @PathVariable Long id) {
//		userLogService.insertLog(loginAccount.getId(), LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "应用列表", null);
//		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "onelink列表", null, null, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(onelinkService.findone(id));
    }


    @RequestMapping(value = "create", method = RequestMethod.POST)
    @ResponseStatus(HttpStatus.CREATED)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel create(HttpServletRequest request, @RequestBody Onelink resource,
                              @CurrentAccount Account account, @PathVariable Long appid) {
        //生成短链
        List<String> shortUrlList = new ArrayList<>();
        shortUrlList = campaignService.makesurls(appid, account, 1);

        if (!CollectionUtils.isEmpty(shortUrlList)) {
            resource.setSurl(shortUrlList.get(0));
            Onelink onelink = onelinkService.create(account.getId(), resource);
            userLogService.insertLog(account, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "onelink" + onelink.getName(), onelink, appid, OperateObjectEnumType.ONELINK.getCode());
            return ResultModel.OK(onelink);

        } else {
            //短链生成失败
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
    }

    @RequestMapping(value = "update", method = RequestMethod.PUT)
    @ResponseBody
    public ResultModel update(HttpServletRequest request, HttpServletResponse response,
                              @RequestBody Onelink resource, @CurrentAccount Account account, @PathVariable Long appid) {
        Onelink update = onelinkService.update(account.getId(), resource);
        userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "onelink" + resource.getName(), resource, appid, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(update);
    }

    @RequestMapping(value = "delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    @AuthorizationMenu("campaign")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResultModel delete(HttpServletRequest request, HttpServletResponse response, @PathVariable Long id,
                              @CurrentAccount Account account, @PathVariable Long appid) {
        Onelink delete = onelinkService.delete(account.getId(), id);
        userLogService.insertLog(account, LogEnumType.DELETE.getCode(), LogEnumType.DELETE.getName() + "onelink" + delete.getName(), delete, appid, OperateObjectEnumType.ONELINK.getCode());
        return ResultModel.OK(delete);
    }

    @RequestMapping(value = "download/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> getDownloadResponseEntity(@PathVariable Long id) {
        return onelinkService.downloadQrCode(id);
    }

    @RequestMapping(value = "logoQcode/{surl}", method = RequestMethod.POST)
    @ResponseBody
    @AuthorizationMenu("campaign")
    public ResultModel createLogoQcode(@CurrentAccount Account account,
                                       @PathVariable("appid") Long appid,
                                       @PathVariable(value = "surl", required = false) String surl,
                                       @RequestParam("file") MultipartFile file) {

        return onelinkService.codeLogoMake(file, appid, account, surl);


    }

}
