package com.reyun.controller.manage;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.reyun.model.AppDeepLink;
import org.mortbay.util.UrlEncoded;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.reyun.model.Account;
import com.reyun.model.PackageSubCampaign;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.PackageSubCampaignService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;

/**
 * Created by sunhao on 17/5/19.
 * desc:子活动controller
 */
@Controller
@RequestMapping("/mng/campaign/packagesubcampaign")
public class MngPackageSubCampaignController {

    Logger logger = LoggerFactory.getLogger(MngPackageSubCampaignController.class);

    @Autowired
    PackageSubCampaignService packageSubCampaignService;

    @Autowired
    UserLogService userLogService;



    /**
     * 列出活动下的所有子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{campaignId}/list", method = RequestMethod.GET)
    public ResultModel listSubCampaign(@PathVariable Long campaignId) {
        return ResultModel.OK(packageSubCampaignService.listSubCampaign(campaignId));
    }

    /**
     * 列出活动下的所有子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{campaignId}/listDisable", method = RequestMethod.GET)
    public ResultModel listDisableSubCampaign(@PathVariable Long campaignId) {
        return ResultModel.OK(packageSubCampaignService.listDisableCampaign(campaignId));
    }

    /**
     * 查询单个子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{subCampaignId}/find", method = RequestMethod.GET)
    public ResultModel findSubCampaign(@PathVariable Long subCampaignId) {
        return ResultModel.OK(packageSubCampaignService.findSubCampaign(subCampaignId));
    }

    /**
     * 单个创建子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "create", method = RequestMethod.POST)
    public ResultModel createSubCampaign(@CurrentAccount Account account, @RequestBody PackageSubCampaign resource) {
//            return ResultModel.OK(packageSubCampaignService.save(subCampaignSource, account));
    	
    	if(resource.getBatchNumber() == -1 || resource.getBatchNumber() == 1) {//如果没有传这个参数或者个数为1的，走单个创建
    		return ResultModel.OK(packageSubCampaignService.save(resource, account));
    	}else {
    		List<PackageSubCampaign> list = new ArrayList<>();
    		
    		String  campaignName=resource.getName();
    		int startNumber = resource.getStartNumber();
    		int endNumber = resource.getEndNumber();
    		for(int i=startNumber;i<=endNumber;i++) {
    			PackageSubCampaign item = resource.clone();
                item.setAppDeepLink(resource.getAppDeepLink());
    			String newName = campaignName + "_" + i;
    			item.setName(newName);
    			list.add(item);
    		}
    		return packageSubCampaignService.batchSave(list, account);
    	}

    }
    
//    @ResponseBody
//    @RequestMapping(value = "batch/create", method = RequestMethod.POST)
//    public ResultModel createSubCampaignBatch(@CurrentAccount Account account, @RequestBody PackageSubCampaign resource) {
//    	
//    }

    /**
     * 修改单个子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "update", method = RequestMethod.PUT)
    public ResultModel modifySubCampaign(@CurrentAccount Account account, @RequestBody PackageSubCampaign subCampaign) {

        return ResultModel.OK(packageSubCampaignService.update(subCampaign, account.getId()));
    }


    /**
     * 停用子活动(单个停用和批量停用)
     * created by sunhao 20170523
     */
    @ResponseBody
    @RequestMapping(value = "del", method = RequestMethod.DELETE)
    public ResultModel deleteSubCampaign(@CurrentAccount Account account, @RequestParam String ids) {

        if (!StringUtils.isEmpty(ids)) {
            packageSubCampaignService.forbiden(ids, account.getId());
            return ResultModel.OK();

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 启用子活动(单个启用和批量启用)
     * created by sunhao 20170523
     */
    @ResponseBody
    @RequestMapping(value = "enable", method = RequestMethod.PUT)
    public ResultModel enableSubCampaign(@CurrentAccount Account account, @RequestParam String ids) {

        if (!StringUtils.isEmpty(ids)) {
            packageSubCampaignService.enable(ids, account.getId());
            return ResultModel.OK();

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }
    /**
     * 名称验证，基于推广活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{campaignId}/valid", method = RequestMethod.GET)
    public ResultModel valid(@PathVariable Long campaignId, @RequestParam String name) {
        return ResultModel.OK(packageSubCampaignService.valid(campaignId, name));
    }

    /**
     * 导出子活动数据
     * created by sunhao 20170519
     */
    @RequestMapping(value = "export/{campaignId}", method = RequestMethod.GET)
    @ResponseBody
    public void exportSubCampaign(@CurrentAccount Account account, @PathVariable Long campaignId,HttpServletRequest request,
                                  @RequestParam String ids,HttpServletResponse response){
        StringBuffer stringBuffer = null;
        String filename ="CampaignList.csv";
        try {
            stringBuffer = packageSubCampaignService.exportSubCampaign(account, campaignId, ids);
            filename = new String(filename.getBytes("UTF-8"));
            filename = URLEncoder.encode(filename, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setCharacterEncoding("UTF-8");
        response.setHeader("filename", filename);
        response.setHeader("Content-Disposition", "attachment;filename=" + filename);
        response.setHeader("Content-Type","text/csv");
        response.setContentType("APPLICATION/OCTET-STREAM");
        ServletOutputStream outputStream = null;
        try {
            outputStream = response.getOutputStream();
            ((ServletOutputStream) outputStream).write(new byte[] { (byte) 0xEF, (byte) 0xBB,(byte) 0xBF });
            ((ServletOutputStream) outputStream).write(stringBuffer.toString().getBytes());
            outputStream.flush();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 查询当前URL下的子活动
     * created by mxq 20170524
     */
//    @ResponseBody
//    @RequestMapping(value = "downloadurl/find/{appid}", method = RequestMethod.GET)
//    public ResultModel downloadSubCampaign(@CurrentAccount Account loginAccount, @PathVariable Long appid, @RequestParam String url){
//        List<CampaignUrl4Web> list = subCampaignService.findSubCampaignByUrl(appid, url);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "下载地址"  + url + "查看推广子活动列表",
//                null, appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode());
//        return ResultModel.OK(list);
//    }

    @RequestMapping(value = "/{appid}/{campaignid}/echoDeepLink", method = RequestMethod.GET)
    @ResponseBody
    public Object echoDeepLink(HttpServletRequest request, @PathVariable Long appid, @PathVariable Long campaignid, @CurrentAccount Account loginAccount) {
        return packageSubCampaignService.echoDeepLink(appid,campaignid);
    }
}
