package com.reyun.controller.manage;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.reyun.repository.AdvancedFunctionApplyRepository;
import com.reyun.util.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import com.reyun.dic.RecycleChannelEnum;
import com.reyun.model.Account;
import com.reyun.model.AdvancedFunctionApply;
import com.reyun.model.AdvancedFunctionRenew;
import com.reyun.model.RecycleChannel;
import com.reyun.model.RecycleChannelAccount;
import com.reyun.repository.RecycleChannelAccountRepository;
import com.reyun.repository.RecycleChannelRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.AdvancedFunctionApplyService;
import com.reyun.service.ChannelAccountService;
import com.reyun.service.RecycleAnalysisService;
import com.reyun.service.RecycleHandleService;
import com.reyun.service.impl.RecycleCommonServiceImpl;

/**
 * Created by sunhao on 18/5/16.
 * 回本分析
 */
@Controller
@RequestMapping("/mng/recycle")
public class MngRecycleAnalysisController {

    @Autowired
    private RecycleAnalysisService recycleAnalysisService;

    @Autowired
    private AdvancedFunctionApplyService advancedFunctionApplyService;

    @Autowired
    ChannelAccountService channelAccountService;


    @Autowired
    private RecycleChannelRepository recycleChannelRepository;
    @Autowired
    private RecycleChannelAccountRepository recycleChannelAccountRepository;
    @Autowired
    private RecycleCommonServiceImpl recycleCommonService;
    @Autowired
    private AdvancedFunctionApplyRepository advancedFunctionApplyRepository;

    /**
     * 申请回本分析功能
     */
    @ResponseBody
    @RequestMapping(value = "/apply", method = RequestMethod.POST)
    public ResultModel apply4RecycleFunction(@CurrentAccount Account account, @RequestBody AdvancedFunctionApply advancedFunctionApply) {

        if (ValidateUtil.isValid(advancedFunctionApply.getFunctionMenu()) && ValidateUtil.isValid(advancedFunctionApply.getApplierContact())
                && ValidateUtil.isValid(advancedFunctionApply.getApplierName())) {

            AdvancedFunctionApply result = advancedFunctionApplyService.addApply(account, advancedFunctionApply);

            return null != result ? ResultModel.OK(result) : ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_ALREADY_OPEN);
        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 查询所有渠道账号（没有权限展示数据结够用）
     */
    @ResponseBody
    @RequestMapping(value = "/account/list", method = RequestMethod.GET)
    public ResultModel listChannelAccountByApp(@CurrentAccount Account account) {

        if (null != account) {
            return ResultModel.OK(recycleAnalysisService.listAccountByAccount(account));
        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 查询广点通/神马已经绑定的渠道账号
     */
    @ResponseBody
    @RequestMapping(value = "/thirdaccount/list/{cid}", method = RequestMethod.GET)
    public ResultModel listThirdChannelAccount(@CurrentAccount Account account, @PathVariable Long cid) {
        return ResultModel.OK(recycleAnalysisService.listApplyChannelAccount(account, cid));
    }


    /**
     * 校验是否受渠道个数限制
     */
    @ResponseBody
    @RequestMapping(value = "/channel/limit", method = RequestMethod.GET)
    public ResultModel checkChannelNumLimit(@CurrentAccount Account account) {
        return ResultModel.OK(recycleAnalysisService.checkChannelNumberLimit(account));
    }


    /**
     * 添加渠道账号
     */
    @ResponseBody
    @RequestMapping(value = "/account/add/{app}", method = RequestMethod.POST)
    public ResultModel addChannelAccount(@CurrentAccount Account account, @PathVariable Long app, @RequestBody RecycleChannelAccount recycleChannelAccount) {
        if (ValidateUtil.isValid(recycleChannelAccount.getUniqueName()) && ValidateUtil.isValid(recycleChannelAccount.getUserName())
                && ValidateUtil.isValid(recycleChannelAccount.getChannelAccountId()) && ValidateUtil.isValid(recycleChannelAccount.getCid())) {

            if (!recycleAnalysisService.checkChannelNumberLimit(account)) {

                RecycleChannelAccount result = recycleAnalysisService.addRecycleChannelAccount(account, app, recycleChannelAccount);
                return null != result ? ResultModel.OK(result) : ResultModel.ERROR(ResultStatus.FAILED);

            } else {
                return ResultModel.ERROR(ResultStatus.CHANNEL_NUMBER_LIMIT);
            }

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 应用神马以及广点通渠道
     */
    @ResponseBody
    @RequestMapping(value = "/account/add/gdt", method = RequestMethod.POST)
    public ResultModel addChannelAccountGdt(@CurrentAccount Account account, @RequestBody RecycleChannelAccount recycleChannelAccount) {

        if (ValidateUtil.isValid(recycleChannelAccount.getChannelAccountId()) && ValidateUtil.isValid(recycleChannelAccount.getUserName())) {

            if (!recycleAnalysisService.checkChannelNumberLimit(account)) {

                RecycleChannelAccount result = recycleAnalysisService.addGdtChannelAccount(account, recycleChannelAccount);
                return null != result ? ResultModel.OK(result) : ResultModel.ERROR(ResultStatus.FAILED);

            } else {
                return ResultModel.ERROR(ResultStatus.CHANNEL_NUMBER_LIMIT);
            }

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 修改渠道账号
     */
    @ResponseBody
    @RequestMapping(value = "/account/modify", method = RequestMethod.PUT)
    public ResultModel modifyChannelAccount(@RequestBody RecycleChannelAccount recycleChannelAccount) {
        if (ValidateUtil.isValid(recycleChannelAccount.getUserName()) && ValidateUtil.isValid(recycleChannelAccount.getChannelAccountId())
                && ValidateUtil.isValid(recycleChannelAccount.getCid())) {

            RecycleChannelAccount result = recycleAnalysisService.modifyRecycleChannelAccount(recycleChannelAccount);

            return null != result ? ResultModel.OK(result) : ResultModel.ERROR(ResultStatus.FAILED);

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }


//    /**
//     * 查询已开回本功能的通渠道(被下方的替换)
//     */
//    @ResponseBody
//    @RequestMapping(value = "/channel/list", method = RequestMethod.GET)
//    public ResultModel listRecycleChannel() {
//        return ResultModel.OK(recycleAnalysisService.listEnableChannel());
//    }


    /**
     * 查询是否开通开回本功能
     */
    @ResponseBody
    @RequestMapping(value = "/status", method = RequestMethod.GET)
    public ResultModel findRecycleFunctionStatus(@CurrentAccount Account account) {
        return ResultModel.OK(recycleAnalysisService.hasRecycleFunctionOpen(account));
    }

    /**
     * 续订回本功能（点击服务续订）
     */
    @ResponseBody
    @RequestMapping(value = "/apply/renew", method = RequestMethod.POST)
    public ResultModel addApplyRenew(@CurrentAccount Account account, @RequestBody AdvancedFunctionRenew advancedFunctionRenew) {

        if (ValidateUtil.isValid(advancedFunctionRenew.getPhone()) && ValidateUtil.isValid(advancedFunctionRenew.getContact())) {

            AdvancedFunctionRenew result = advancedFunctionApplyService.addRenewApply(account, advancedFunctionRenew);

            return null != result ? ResultModel.OK(result) : ResultModel.ERROR(ResultStatus.FAILED);

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }


    /**
     * 验证头条是否授权成功
     */
    @ResponseBody
    @RequestMapping(value = "/jrtt/auth/test/{accountid}", method = RequestMethod.GET)
    public ResultModel findJRTTToken(@CurrentAccount Account account,@PathVariable String accountid){
        RecycleChannelAccount recycleChannelAccount =  recycleAnalysisService.findJRTTToken(account.getRootParent(),514L,accountid);
        return null != recycleChannelAccount ? ResultModel.OK(recycleChannelAccount) : ResultModel.ERROR(ResultStatus.FAILED);
    }

    /**
     *获取今日头条回调token
     */
    @RequestMapping(value = "/jrtt/cb", method = RequestMethod.GET)
    public void getTouTiaoCode(HttpServletRequest request, @RequestParam(required = false) String auth_code, @RequestParam(required = false) String state,
                        HttpServletResponse response) throws Exception {
        if(null != auth_code && null != state){
        	//state的格式是：[tkio登录账号id]###[渠道别称]
        	String[] arr = state.split("###");
        	String loginAccountid = arr[0];
        	String alias = arr[1];
            channelAccountService.getJRTTTokenAndSave(request, auth_code, loginAccountid,alias);
        }
        response.sendRedirect("/index.html");
    }
    
    /**
     * 今日头条连接测试
     */
    @ResponseBody
    @RequestMapping(value = "/jrtt/token/test/{accountid}", method = RequestMethod.GET)
    public ResultModel testJRTTToken(@CurrentAccount Account loginAccount,@PathVariable String accountid){
        boolean result = recycleAnalysisService.testJRTTToken(loginAccount.getRootParent(),514L,accountid);
        if(result){
             return ResultModel.OK();
        }else{
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
    }

    /**
     * 今日头条授权成功
     */
    @ResponseBody
    @RequestMapping(value = "/jrtt/success/{accountid}/{channelalias}", method = RequestMethod.GET)
    public ResultModel successJRTTOauth(@CurrentAccount Account loginAccount,@PathVariable String accountid,@PathVariable String channelalias){
        boolean result = recycleAnalysisService.successJRTTAouth(loginAccount.getRootParent(),514L,accountid,channelalias);
        if(result){
            return ResultModel.OK();
        }else{
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
    }

    /**
     * 爬虫相关授权保存账号密码
     */
    @ResponseBody
    @RequestMapping(value = "/spider/save/{accountid}/{channelalias}/{password}/{mccid}/{mcuniquename}/{type}", method = RequestMethod.GET)
    public ResultModel saveSpiderAccountAndPassword(@CurrentAccount Account account,@PathVariable String accountid,@PathVariable String channelalias,@PathVariable String password,@PathVariable String mccid,@PathVariable String mcuniquename,@PathVariable String type){
        boolean result = recycleAnalysisService.saveSpiderAccountAndPassword(accountid,channelalias,password,mccid,mcuniquename,type,account);
        if(result){
            return ResultModel.OK();
        }else{
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
    }

    /**
     * 爬虫相关授权账号唯一校验
     */
    @ResponseBody
    @RequestMapping(value = "/spider/check/{accountid}", method = RequestMethod.GET)
    public ResultModel checkSpiderAccount(@CurrentAccount Account account,@PathVariable String accountid){
        long result = recycleAnalysisService.checkSpiderAccount(accountid,account);
        if(result == 0){
            return ResultModel.OK();
        }else{
            return ResultModel.ERROR(ResultStatus.FAILED);
        }
    }
    /**
     * 渠道账号授权并添加
     * 备注：channel可以在RecycleChannelEnum找到
     */
    @ResponseBody
    @RequestMapping(value = "/auth/{channel}", method = RequestMethod.GET)
    public ResultModel authChannel(@CurrentAccount Account account,@ModelAttribute RecycleChannelAccount recycleChannelAccount,
                                   @PathVariable String  channel){
        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }


        //校验账号与账号别名唯一性
        ResultModel resultModel = recycleCommonService.usernameAndchannelAliasLimitOne(account,recycleChannelAccount);
        if(resultModel != null){
            return  resultModel;
        }

        //查询渠道是否开启回本
        RecycleChannel recycleChannel = recycleChannelRepository.findAllEnableByChannel(recycleChannelAccount.getCid());
        if(recycleChannel == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUNT_NOT_OPEN);
        }

        //校验是否已经达到渠道的限制（免费与付费过期用户只能绑定三个账号）
        if(recycleCommonService.checkChannelNumberLimit(account, recycleChannelAccount)){
            return ResultModel.ERROR(ResultStatus.CHANNEL_NUMBER_LIMIT);
        }

        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);
        boolean paramIsAll =  recycleHandleService.checkAuthParam(recycleChannelAccount);
        if(!paramIsAll){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
        return recycleHandleService.authChannel(recycleChannelAccount, account, recycleChannel);

    }

    /**
     * @param account
//     * @param username 渠道账号
//     * @param channelAlias 渠道别名
//     * @param cid 渠道id
     * @return  账号与别名的唯一性校验（主账号下只有一个）
     */
    @ResponseBody
    @RequestMapping(value = "/unique/check", method = RequestMethod.GET)
    public ResultModel addloginlicense(@CurrentAccount Account account,
                                       @ModelAttribute RecycleChannelAccount recycleChannelAccount){

        ResultModel resultModel = recycleCommonService.usernameAndchannelAliasLimitOne(account,recycleChannelAccount);
        if(resultModel != null){
            return  resultModel;
        }
        return ResultModel.OK();

    }

    /**
     * @param recycleChannelAccount
     * @param channel
     * @return 添加登陆凭证
     */
    @ResponseBody
    @RequestMapping(value = "/addloginlicense/{channel}", method = RequestMethod.GET)
    public ResultModel addloginlicense(@ModelAttribute RecycleChannelAccount recycleChannelAccount, @PathVariable String channel){
        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }
        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);
        return recycleHandleService.addLoginLicense(recycleChannelAccount);
    }


    /**
     * @param recycleChannelAccount
     * @param channel
     * @return 获取登陆凭证
     */
    @ResponseBody
    @RequestMapping(value = "/getloginlicense/{channel}", method = RequestMethod.GET)
    public ResultModel getloginlicense(@ModelAttribute RecycleChannelAccount recycleChannelAccount, @PathVariable String channel){
        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }
        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);
        return recycleHandleService.getLoginLicense(recycleChannelAccount);
    }

    /**
     * @param recycleChannelAccount
     * @return 改变渠道状态为不可用（当爬虫发现保存的登陆信息无法登陆时，标记该账号是无效）
     */
    @ResponseBody
    @RequestMapping(value = "/disableaccount", method = RequestMethod.GET)
    public ResultModel disableaccount(@ModelAttribute RecycleChannelAccount recycleChannelAccount){
        boolean uniqueAccountExist = !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
        boolean cidExist = recycleChannelAccount.getCid() > 0;
        boolean rootAccountExist = recycleChannelAccount.getRootAccount() > 0;
        boolean loginFailMesExist = !StringUtil.isEmpty(recycleChannelAccount.getLoginFailMes());
        if(uniqueAccountExist && cidExist && rootAccountExist && loginFailMesExist){
            RecycleChannelAccount channelAccount = recycleChannelAccountRepository.
                    findByRootAccountIdAndUniqueAccountAndCid(recycleChannelAccount.getRootAccount(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());
            if(channelAccount != null ){
                if(0 != channelAccount.getValidFlag()){//防止已经为0还再次发邮件的问题(也防止某一部分渠道为插入默认值)
                    channelAccount.setUtime(new Date());
                    channelAccount.setValidFlag(RecycleChannelAccount.VALID_FLAG_INVALID);
                    channelAccount.setLoginFailMes(recycleChannelAccount.getLoginFailMes());
                    recycleChannelAccountRepository.save(channelAccount);
                    recycleCommonService.sendDisableAcountEmail(channelAccount);
                    return ResultModel.OK();
                }
            }else{
                return  ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
            }
        }
        return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);

    }

    /**
     * @param channel recycle_channel的unique_name
     * @return 查询渠道下可用的所有账号信息(status = 1: 启用中 and validFlag = 1 可以登陆有效的)
     */
    @ResponseBody
    @RequestMapping(value = "/accountlist/{channel}", method = RequestMethod.GET)
    public ResultModel accountlist(@PathVariable String channel){

        List<RecycleChannelAccount> channelAccounts = new ArrayList<>();
        if("all".equals(channel)){
         channelAccounts = recycleChannelAccountRepository.findEnableAccount();
        }else{
          channelAccounts = recycleChannelAccountRepository.findByUniqueNameAndEnable(channel);
        }

        List<RecycleChannel> recycleChannels = recycleChannelRepository.findAllEnableChannel();

        List<RecycleChannelAccount> enableChannelAccounts = new ArrayList<>();//存储所有可用的账号（如果渠道整体关闭，数据不再查询）

        for(RecycleChannelAccount recycleChannelAccount : channelAccounts){
            for(RecycleChannel recycleChannel : recycleChannels){
                long cid1 = recycleChannel.getCid();
                long cid2 = recycleChannelAccount.getCid();
                boolean cidisOne = cid1 == cid2;
                if(cidisOne){
                    recycleChannelAccount.setMcCid(recycleChannel.getMcCid());
                    recycleChannelAccount.setMcUniqueName(recycleChannel.getMcUniqueName());
                    enableChannelAccounts.add(recycleChannelAccount);
                    continue;
                }
            }
        }
        return  ResultModel.OK(enableChannelAccounts);

    }


    /**
     * @param channel
     * @param recycleChannelAccount
     * @return 发送手机验证码信息接口
     */
    @ResponseBody
    @RequestMapping(value = "/sendvcode/{channel}", method = RequestMethod.POST)
    public ResultModel sendvcode(@PathVariable String channel,@RequestBody RecycleChannelAccount recycleChannelAccount){

        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }
        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);

        return  recycleHandleService.sendVcode(recycleChannelAccount);

    }

    /**
     * @param channel
     * @param recycleChannelAccount
     * @return  校验手机验证码登陆接口
     */
    @ResponseBody
    @RequestMapping(value = "/phoneverify/{channel}",  method = RequestMethod.POST)
    public ResultModel phoneverify(@CurrentAccount Account account,@PathVariable String channel,@RequestBody RecycleChannelAccount recycleChannelAccount){

        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }
        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);

        //校验参数
        boolean paramIsAll =  recycleHandleService.checkAuthParam(recycleChannelAccount);
        if(!paramIsAll){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //查询渠道是否开启回本
        RecycleChannel recycleChannel = recycleChannelRepository.findAllEnableByChannel(recycleChannelAccount.getCid());
        if(recycleChannel == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUNT_NOT_OPEN);
        }


        return  recycleHandleService.phoneVerify(recycleChannelAccount, account, recycleChannel);

    }


    /**
     * @param account
     * @param recycleChannelAccount
     * @return 获取验证信息
     */
    @ResponseBody
    @RequestMapping(value = "/getlogincheck", method = RequestMethod.GET)
    public ResultModel getlogincheck(@CurrentAccount Account account,@ModelAttribute RecycleChannelAccount recycleChannelAccount) {

        boolean uniqueAccountExist = recycleChannelAccount != null && !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
        boolean cidExist = recycleChannelAccount != null && recycleChannelAccount.getCid() > 0;
        if(uniqueAccountExist && cidExist){
            RecycleChannelAccount channelAccount =  recycleChannelAccountRepository.
                    findByRootAccountIdAndUniqueAccountAndCid(account.getRootParent(),recycleChannelAccount.getUniqueAccount(),recycleChannelAccount.getCid());
            if(channelAccount != null){
                Map<String, String> result = new HashMap<>();
                result.put("loginCkeck",channelAccount.getLoginCheck());
                return ResultModel.OK(result);
            }else{
                return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUN_DATA_NOT_EXIST);
            }
        }
        return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }

        /**
         * @param account
         * @param channel
         * @param recycleChannelAccount
         * @return 问题验证接口（需要前端提供唯一值来回填问题数据：rootAccountId+username+cid）
         */
    @ResponseBody
    @RequestMapping(value = "/questionverify/{channel}", method = RequestMethod.POST)
    public ResultModel questionverify(@CurrentAccount Account account,@PathVariable String channel,@RequestBody RecycleChannelAccount recycleChannelAccount){

        RecycleChannelEnum channelEnum =  RecycleChannelEnum.getRecycleChannelEnumByKey(channel);
        if(channelEnum == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_CHANNEL_NOT_EXIST);
        }
        RecycleHandleService recycleHandleService = recycleCommonService.getRealRecycleService(channelEnum);
        recycleChannelAccount.setRootAccount(account.getRootParent());

        //校验参数
        boolean paramIsAll =  recycleHandleService.checkAuthParam(recycleChannelAccount);
        if(!paramIsAll){
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

        //查询渠道是否开启回本
        RecycleChannel recycleChannel = recycleChannelRepository.findAllEnableByChannel(recycleChannelAccount.getCid());
        if(recycleChannel == null){
            return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUNT_NOT_OPEN);
        }

        return  recycleHandleService.questionVerify(recycleChannelAccount, account, recycleChannel);
    }


    /**
     * web端展示（启用/停用）渠道账号信息（关联推广数据）
     * status = enable : 启用
     * status = disable ：停用
     */
    @ResponseBody
    @RequestMapping(value = "/account/info/{status}", method = RequestMethod.GET)
    public ResultModel listEnbaleChannelAccountInfo(@CurrentAccount Account account, @PathVariable String status) {
        boolean accountStatus = true;
        if("disable".equals(status)){
            accountStatus = false;
        }
        long appid = 1218;//复用report的传输接口会校验appid,不允许为空，此处写死（app需要存在）
        return ResultModel.OK(recycleAnalysisService.listChannelAccountInfo(appid, account, accountStatus));
    }


    /**
     * 停用渠道账号
     */
    @ResponseBody
    @RequestMapping(value = "/account/disable/{cid}", method = RequestMethod.GET)
    public ResultModel disableChannelAccount(@CurrentAccount Account account,@PathVariable Long cid, @ModelAttribute RecycleChannelAccount recycleChannelAccount) {

        if (ValidateUtil.isValid(recycleChannelAccount.getUniqueAccount())) {

            int result =  recycleChannelAccountRepository.disableRecycleChannelAccount(account.getRootParent(), recycleChannelAccount.getUniqueAccount(), cid);

            return result == 1 ? ResultModel.OK() : ResultModel.ERROR(ResultStatus.FAILED);

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 启用渠道账号(校验启用的渠道个数)
     */
    @ResponseBody
    @RequestMapping(value = "/account/enable/{cid}", method = RequestMethod.GET)
    public ResultModel enableChannelAccount(@CurrentAccount Account account, @PathVariable Long cid, @ModelAttribute RecycleChannelAccount recycleChannelAccount) {

        if (ValidateUtil.isValid(recycleChannelAccount.getUniqueAccount())) {

            if (!recycleCommonService.checkChannelNumberLimit(account, recycleChannelAccount)) {
               int result =  recycleChannelAccountRepository.enableRecycleChannelAccount(account.getRootParent(), recycleChannelAccount.getUniqueAccount(), cid);
               return result == 1 ? ResultModel.OK() : ResultModel.ERROR(ResultStatus.FAILED);

            } else {
                return ResultModel.ERROR(ResultStatus.CHANNEL_NUMBER_LIMIT);
            }

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }


    /**
     * @param account
     * @return 校验付费账户是否到期（如果到期，通知前端提醒）
     */
    @ResponseBody
    @RequestMapping(value = "/checkadvancepast", method = RequestMethod.GET)
    public ResultModel checkadvancepast(@CurrentAccount Account account) {
        int warnTime = 3;//提醒的次数
        int deadline = 7;//7天的时候进行提醒
        AdvancedFunctionApply advancedFunctionApply = advancedFunctionApplyRepository.findByAccount(account.getId(), Constant.ADVANCED_FUNCTION_MENU_RECYCLE);
        if(advancedFunctionApply != null){
           boolean  advanceCanCheck =  advancedFunctionApply.getStatus() != 0;
           Date deadlineDate = DateUtil.addDays(new Date(), deadline);
           boolean advanceExpire = advanceCanCheck && deadlineDate.after(advancedFunctionApply.getPastDate());
           boolean warnTimeNoOverTop = advanceExpire && advancedFunctionApply.getWarnTime() <= warnTime;
           if(warnTimeNoOverTop){
               //此处防止多台电脑同时操作(都算作提醒过)
               int addWarnTimeSuccess = 0;
               addWarnTimeSuccess   = advancedFunctionApplyRepository.addWarnTime(advancedFunctionApply.getId(), warnTime);
               if(addWarnTimeSuccess == 1){
                   return  ResultModel.OK();
               }
           }
        }
        return ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_NO_NEED_WARN);
    }


    /**
     * 查询已开回本功能的渠道（type 区分是否要将mc_cid分离出来展示--百度原生存在俩个子渠道）
     *
     */
    @ResponseBody
    @RequestMapping(value = "/channel/list", method = RequestMethod.GET)
    public ResultModel listTypeRecycleChannel(@RequestParam(value = "type",required = false, defaultValue = "0") int type ,@RequestHeader(name = "lang", defaultValue = "zh") String lanType) {
        return ResultModel.OK(recycleAnalysisService.listEnableChannel(type, lanType));
    }



}
