package com.reyun.controller.manage;

import com.google.common.base.Function;
import com.google.common.base.Splitter;
import com.google.common.collect.Iterables;
import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.Campaign;
import com.reyun.model.CampaignUrl4Web;
import com.reyun.model.SubCampaign;
import com.reyun.model.SubCampaignExport;
import com.reyun.repository.CampaignRepository;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.SubCampaignService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

/**
 * Created by sunhao on 17/5/19.
 * desc:子活动controller
 */
@Controller
@RequestMapping("/mng/campaign/subcampaign")
public class MngSubCampaignController {

    Logger logger = LoggerFactory.getLogger(MngSubCampaignController.class);

    @Autowired
    SubCampaignService subCampaignService;

    @Autowired
    UserLogService userLogService;

    @Autowired CampaignRepository campaignRepository;

    /**
     * 列出活动下的所有子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{campaignId}/list", method = RequestMethod.GET)
    public ResultModel listSubCampaign(@PathVariable Long campaignId) {
        return ResultModel.OK(subCampaignService.listSubCampaign(campaignId));
    }

    /**
     * 列出活动下的所有子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{campaignId}/listDisable", method = RequestMethod.GET)
    public ResultModel listDisableSubCampaign(@PathVariable Long campaignId) {
        return ResultModel.OK(subCampaignService.listDisableCampaign(campaignId));
    }

    /**
     * 查询单个子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "{subCampaignId}/find", method = RequestMethod.GET)
    public ResultModel findSubCampaign(@PathVariable Long subCampaignId) {
        return ResultModel.OK(subCampaignService.findSubCampaign(subCampaignId));
    }

    /**
     * 手动创建子活动，根据创意的输入，可一个可多个
     * created by sunhao 20170519
     * return 失败列表和成功列表
     */
    @ResponseBody
    @RequestMapping(value = "create", method = RequestMethod.POST)
    public ResultModel createSubCampaign(@CurrentAccount Account account, @RequestBody SubCampaign subCampaignSource) {

        if (null != subCampaignSource.getChannelId() && null != subCampaignSource.getCampaignId()
                && !StringUtils.isEmpty(subCampaignSource.getKeyword()) && !StringUtils.isEmpty(subCampaignSource.getUrl())) {

            return ResultModel.OK(subCampaignService.saveSubCampaign(account, subCampaignSource));

        } else {

            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 修改单个子活动
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "update", method = RequestMethod.PUT)
    public ResultModel modifySubCampaign(@CurrentAccount Account account, @RequestBody SubCampaign subCampaign) {

        if (null != subCampaign.getId() && !StringUtils.isEmpty(subCampaign.getUrl())) {

            return ResultModel.OK(subCampaignService.modifySubCampaignUrl(subCampaign.getId(), subCampaign.getUrl(), account));
        } else {

            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }


    /**
     * 停用子活动(单个停用和批量停用)
     * created by sunhao 20170523
     */
    @ResponseBody
    @RequestMapping(value = "del", method = RequestMethod.DELETE)
    public ResultModel deleteSubCampaign(@CurrentAccount Account account, @RequestParam String ids) {

        if (!StringUtils.isEmpty(ids)) {

            return ResultModel.OK(subCampaignService.deleteSubCampaign(account, ids));

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }

    /**
     * 启用子活动(单个启用和批量启用)
     * created by sunhao 20170523
     */
    @ResponseBody
    @RequestMapping(value = "enable", method = RequestMethod.PUT)
    public ResultModel enableSubCampaign(@CurrentAccount Account account, @RequestParam String ids) {

        if (!StringUtils.isEmpty(ids)) {

            return ResultModel.OK(subCampaignService.enableSubCampaign(account, ids));

        } else {
            return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }
    }



    /**
     * 上传子活动模板文件
     * created by sunhao 20170519
     */
    @RequestMapping(value = "{campaignId}/upload", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel uploadFile(@RequestParam("file") MultipartFile file, @PathVariable Long campaignId, @CurrentAccount Account account) {


        if (file.getOriginalFilename().lastIndexOf(".xls") != -1 || file.getOriginalFilename().lastIndexOf(".xlsx") != -1) {

            return ResultModel.OK(subCampaignService.readKeywordFile(file, account, campaignId));

        } else {

            return ResultModel.ERROR(ResultStatus.FILE_TYPE_ERROR);
        }
    }

    /**
     * 保存上传的子活动数据
     * created by sunhao 20170519
     */
    @RequestMapping(value = "{campaignId}/{uploadHistoryId}/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel saveUploadSubCampaign(@CurrentAccount Account account, @PathVariable Long campaignId, @PathVariable Long uploadHistoryId) {
        return ResultModel.OK(subCampaignService.saveKeywordInfo(uploadHistoryId, account, campaignId));
    }


    @ResponseBody
    @RequestMapping(value = "export/list/{appId}", method = RequestMethod.GET)
    @AuthorizationMenu("subcampaign")
    public ResultModel listSubCampaignExport(@CurrentAccount Account account, @PathVariable Long appId) {
        return ResultModel.OK(subCampaignService.listSubCampaignExport(account, appId));
    }

    @ResponseBody
    @RequestMapping(value = "export/init/{exportId}", method = RequestMethod.PUT)
    public ResultModel initSubCampaignExport(@PathVariable Long exportId){
        return ResultModel.OK(subCampaignService.updateSubCampaignExportStatus(exportId, DownloadStatusEnumType.INIT.getCode()));
    }

    /**
     * 导出子活动数据
     * created by sunhao 20170519
     */
    @ResponseBody
    @RequestMapping(value = "export/{campaignId}", method = RequestMethod.POST)
    public ResultModel exportSubCampaign(@CurrentAccount Account account, @PathVariable Long campaignId,
                                         @RequestBody SubCampaignExport subCampaignExport, @RequestHeader(name = "lang", defaultValue = "zh") String lanType) {

        return ResultModel.OK(subCampaignService.exportSubCampaign(account, campaignId, subCampaignExport.getIds(), lanType));
    }


    /**
     * 批量导出子活动数据
     * created by song 20170519
     */
    @ResponseBody
    @RequestMapping(value = "batchExportSubCampaign/{appId}", method = RequestMethod.POST)
    public ResultModel exportBatchSubCampaign(@CurrentAccount Account account,@PathVariable Long appId,@RequestBody SubCampaignExport subCampaignExport) {
        return ResultModel.OK(subCampaignService.exportBatchSubCampaign(account,appId,subCampaignExport.getIds()));
    }


    /**
     * 下载导出的子活动
     * created by sunhao 20170524
     */
    @RequestMapping(value = "{exportId}/download", method = RequestMethod.GET)
    @AuthorizationMenu("subcampaign")
    public void downloadSubCampaign(HttpServletRequest request, HttpServletResponse response, @CurrentAccount Account account, @PathVariable Long exportId) throws IOException {
        subCampaignService.downloadSubCampaign(request, response, account, exportId);

        userLogService.insertLog(account, LogEnumType.DOWNLOAD.getCode(),
                LogEnumType.DOWNLOAD.getName() + "子活动", exportId,null, OperateObjectEnumType.SUBCAMPAIGN.getCode());
    }

    /**
     * 删除下载导出的子活动
     * created by sunhao 20170523
     */
    @ResponseBody
    @RequestMapping(value = "download/delete/{exportId}", method = RequestMethod.DELETE)
    public ResultModel deleteDownloadSubCampaign(@CurrentAccount Account account, @PathVariable Long exportId) {
        subCampaignService.deleteDownloadSubCampaign(exportId);

        return ResultModel.OK();
    }

    /**
     * 查询当前URL下的子活动
     * created by mxq 20170524
     */
    @ResponseBody
    @RequestMapping(value = "downloadurl/find/{appid}", method = RequestMethod.POST)
    public ResultModel downloadSubCampaign(@CurrentAccount Account loginAccount, @PathVariable Long appid, @RequestBody String url){
        List<CampaignUrl4Web> list = subCampaignService.findSubCampaignByUrl(appid, url);
//        userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "下载地址"  + url + "查看推广子活动列表",
//                null, appid, OperateObjectEnumType.CAMPAIGN_DOWNLOAD_URL.getCode());
        return ResultModel.OK(list);
    }


    /**
     * 实时导出子活动数据
     * created by zxy 20180223
     */
    @RequestMapping(value = "export/rt", method = RequestMethod.GET)
    public ResultModel exportSubCampaignRealtime(HttpServletResponse httpServletResponse, @RequestParam String ids) {

        Iterable<String> iterids = Splitter.on(",").trimResults().omitEmptyStrings().split(ids);
        if (iterids == null) {
            return ResultModel.OK();
        }
        Iterable<Long> transids = Iterables.transform(iterids, new Function<String, Long>() {
            @Override
            public Long apply(String s) {
                return Long.parseLong(s);
            }
        });
        String fileName = "子活动短链.csv";
        StringBuffer exportbuffer = this.subCampaignService.exportsurls2column(transids);

        //为了解决中文名称乱码问题
        try {
            fileName = new String(fileName.getBytes("UTF-8"));
            fileName = URLEncoder.encode(fileName, "UTF-8");
        }
        catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.setHeader("filename", fileName);
        httpServletResponse.setHeader("Content-Disposition", "attachment;filename="+fileName);
        httpServletResponse.setHeader("Content-Type","text/csv");
        httpServletResponse.setContentType("APPLICATION/OCTET-STREAM");
        ServletOutputStream outputStream = null;
        try {
            outputStream = httpServletResponse.getOutputStream();
            ((ServletOutputStream) outputStream).write(new byte[] { (byte) 0xEF, (byte) 0xBB,(byte) 0xBF });
            ((ServletOutputStream) outputStream).write(exportbuffer.toString().getBytes());
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (outputStream != null)
                    outputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ResultModel.OK();
    }
}
