package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.model.LifeCycleApps;
import com.reyun.model.SystemParam;
import com.reyun.repository.SystemParamRepository;
import com.reyun.security.annotation.CurrentAccount;
import com.reyun.service.ChannelParamService;
import com.reyun.service.ChannelService;
import com.reyun.service.SystemParamService;
import com.reyun.service.UserLogService;
import com.reyun.util.ResultModel;
import com.reyun.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
@RequestMapping("/mng/systemparam")
public class MngSystemParamController {
	protected Logger logger = LoggerFactory.getLogger(MngSystemParamController.class);
	
	@Autowired
	SystemParamService systemParameterService;

	@Autowired
	UserLogService userLogService;

	@Autowired
	SystemParamRepository systemParamRepository;


	@Autowired
	ChannelParamService channelParamService;

	@Autowired
	ChannelService channelService;

	
	@RequestMapping(value = "update",method = RequestMethod.PUT)
	@ResponseBody
	public ResultModel update(HttpServletRequest request,
			@RequestBody SystemParam resource,
			@CurrentAccount Account currentAccount) {
		resource.setModifyAccount(currentAccount.getId());
		Long account = currentAccount.getId();
		if (!currentAccount.getIsSuperUser()) {
			account = currentAccount.getRootParent();
		}
		SystemParam systemParam = systemParameterService.update(resource,account);
		userLogService.insertLog(currentAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广参数" , systemParam,null,OperateObjectEnumType.SYSTEM_PARAM.getCode(), request);

		return ResultModel.OK();
	}

	@RequestMapping(value = "updateimp",method = RequestMethod.PUT)
	@ResponseBody
	public ResultModel updateImp(HttpServletRequest request,
							  @CurrentAccount Account currentAccount,
								 @RequestParam String status,Long app) {
		Long account = currentAccount.getId();
		if (!currentAccount.getIsSuperUser()) {
			account = currentAccount.getRootParent();
		}
		String statusStr = request.getParameter("status");
		Boolean statusB = StringUtil.isEmpty(status)?false:Boolean.valueOf(statusStr);
		systemParamRepository.updateImpAttributeStatus(statusB, app);

		List<Channel> channelList = channelService.findCoopAndOwnChannel(account, null);
		for (Channel channel : channelList) {
			channelParamService.updateImp(channel.getId(), statusB, account,app);
		}

		userLogService.insertLog(currentAccount, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "推广参数-展示参与归因" , status,null,OperateObjectEnumType.SYSTEM_PARAM.getCode(), request);

		return ResultModel.OK();
	}
	
	@RequestMapping(value = "{app}/find",method = RequestMethod.GET)
	@ResponseBody
	public ResultModel find(@PathVariable Long app, @CurrentAccount Account loginAccount) {
		Long account = loginAccount.getId();
		if (!loginAccount.getIsSuperUser()) {
			account = loginAccount.getRootParent();
		}
		SystemParam find = systemParameterService.find(account,app);
		userLogService.insertLog(loginAccount, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName() + "推广参数" , find,null,OperateObjectEnumType.SYSTEM_PARAM.getCode());
		return ResultModel.OK(find);
	}

	@RequestMapping(value = "/lifecycle/setting",method = RequestMethod.POST)
	@ResponseBody
	public ResultModel setLifeCycle(@RequestBody LifeCycleApps lifeCycleApps, @CurrentAccount Account loginAccount) {
		LifeCycleApps apps = systemParameterService.setLifeCycle(lifeCycleApps,loginAccount);
		return ResultModel.OK(apps);
	}

	@RequestMapping(value = "/lifecycle/find/{app}",method = RequestMethod.GET)
	@ResponseBody
	public ResultModel setLifeCycle(@PathVariable String app, @CurrentAccount Account loginAccount) {
		LifeCycleApps apps = systemParameterService.findLifeCycleByApp(app,loginAccount);
		return ResultModel.OK(apps);
	}

}
