package com.reyun.controller.manage;

import com.reyun.dic.LogEnumType;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.BussinessManRepository;
import com.reyun.repository.SystemParamRepository;
import com.reyun.service.*;
import com.reyun.util.*;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;

import java.io.IOException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Controller
@RequestMapping("reged")
public class RegedController {

	protected Logger logger = LoggerFactory.getLogger(RegedController.class);

	@Autowired
	AppService appService;

	@Autowired
	RegedService regedService;

	@Autowired
	AccountRepository accountRepository;

	@Autowired
	UserLogService userLogService;

    @Autowired
    BussinessManRepository bussinessManRepository;
    
    @Autowired
    AuthService authService;

    @Autowired
	BussinessManRepository bussinessRepository;
    
    @Autowired
    SystemParamRepository systemParamRepository;

    @Autowired
    AccountService accountService;

    @Autowired
    TipService tipService;

    private static final String VERIFY_EMAIL_SEND_URL = Constant.adminUrl + "/api/tool/account/sendemail/";

//    private static final String ACTIVE_EMAIL_SEND_URL = Constant.accountCheckUrl + "/api/reged/active/";

//    private static final String ACTIVE_EMAIL_SEND_URL_XH = Constant.accountXhmpCheckUrl + "/api/reged/active/";


	private static final int VALID_EMAIL_EXPIRE_TIME = 5;

	private static Map<String,String> regex = new HashMap(){
		{
			put("phone","^((13[0-9])|(14[5,7,9])|(15([0-3]|[5-9]))|(166)|(17[0,1,3,5,6,7,8])|(18[0-9])|(19[8|9]))\\d{8}$");
			put("name","^[a-zA-Z0-9_\\s\\-\\u4E00-\\u9FA5\\(\\)\\（\\）]+$");
			put("wechat","^[a-zA-Z0-9]{1}[a-zA-Z\\d_\\d-]{5,19}$");
			put("email","^([a-zA-Z0-9]+[_|\\_|\\.|\\-]?)*[a-zA-Z0-9]+@([a-zA-Z0-9]+[_|\\_|\\.|\\-]?)*[a-zA-Z0-9]+\\.[a-zA-Z]{1,63}$");
			put("qq","^[1-9]\\d{4,9}$");
			put("company","^[a-zA-Z0-9_\\s\\-\\u4E00-\\u9FA5\\(\\)\\（\\）]+$");
		}
	};

    /**
	 * 网站首页注册帐号并发送激活邮件
	 * @param request
	 * @param resource
	 * @return
	 */
	@RequestMapping(value = "regedaccount",method = RequestMethod.POST)
	@ResponseBody
	public ResultModel createOrigiWeb(HttpServletRequest request, @RequestBody Account resource, @CookieValue(value = "lang",required = false, defaultValue = "zh") String lanType) {
		return createOrigi(request,resource,lanType);
	}


	@RequestMapping(value = "apiaccount",method = RequestMethod.POST)
	@ResponseBody
	public ResultModel createApiAccount(HttpServletRequest request, Account resource) {

		ResultModel check = this.checkParam(request,resource,"email","password","company","name","phone","wechat","qq");
		if(check!=null){
			return check;
		}
		resource.setRegSource(9);
		return createOrigi(request,resource,"zh");
	}


	private ResultModel checkParam(HttpServletRequest request, Account resource, String... keys){
		Map<String, String[]> param = request.getParameterMap();

		for (String key : keys) {
			if (param.get(key) == null || StringUtils.isEmpty(param.get(key)[0])) {
				return new ResultModel(204, "参数不能为空", key + " 值不能为空");
			}
//			"email","password","company","name","phone","wechat","qq"
			if ("email".equals(key)) {
				Pattern p = Pattern.compile(regex.get("email"));
				Matcher m = p.matcher(resource.getEmail());
				if (!m.matches() || resource.getEmail().length() > 64) {
					return new ResultModel(224, "请输入正确的邮箱格式", "");
				}

			} else if ("password".equals(key)) {
				if (resource.getPassword().length() != 32) {
					return new ResultModel(224, "需MD5加密后上报，需为32位", "");
				}
			} else if ("phone".equals(key)) {
				String phone = resource.getPhone();
				if (phone.length() != 11) {
					return new ResultModel(224, "手机号应为11位数", "");
				} else {
					Pattern p = Pattern.compile(regex.get("phone"));
					Matcher m = p.matcher(phone);
					boolean isMatch = m.matches();
					if (!isMatch) {
						return new ResultModel(224, "请填入正确的手机号", "");
					}
				}
			} else if ("wechat".equals(key)) {
				Pattern p = Pattern.compile(regex.get("wechat"));
				Matcher m = p.matcher(resource.getWechat());
				if (!m.matches()) {
					return new ResultModel(224, "微信号必须为 6-20位数字，字母，下划线或中划线，须以数字或字母开头", "wechat");
				}
			} else if ("name".equals(key)) {
				Pattern p = Pattern.compile(regex.get("name"));
				Matcher m = p.matcher(resource.getName());
				if (!m.matches() || resource.getName().length() > 32) {
					return new ResultModel(224, "您的姓名，支持16位内汉字或32位内字母数字", "name");
				}
			} else if ("qq".equals(key)) {
				Pattern p = Pattern.compile(regex.get("qq"));
				Matcher m = p.matcher(resource.getQq());
				if (!m.matches()) {
					return new ResultModel(224, "请输入正确的QQ号码", "qq");
				}
			} else if ("company".equals(key)) {
				Pattern p = Pattern.compile(regex.get("company"));
				Matcher m = p.matcher(resource.getCompany());
				if (!m.matches() || resource.getCompany().length() > 96) {
					return new ResultModel(224, "公司名称，支持48位内汉字或96位内字母数字", "company");
				}
			}

		}
		return null;
	}

	public ResultModel createOrigi(HttpServletRequest request, Account resource,String lanType) {
		Account accountDB = authService.check(resource.getEmail());
		if (accountDB != null) {
			if(resource.getRegSource()== 9){
				// 9 代表为api 注册账号
				return new ResultModel(-1,"该邮箱已注册，请更换邮箱", "该邮箱已注册，请更换邮箱");
			}
			return new ResultModel(-1,"该邮箱已存在，请更换邮箱", "该邮箱已存在，请更换邮箱");
		}

		if(resource.getRegSource()== 9){
			// 9 代表为api 注册账
			// api 注册账号  已MD5加密 直接保存
		}else{
			//1. 创建帐号
			resource.setPassword(CipherUtil.generatePassword(resource.getPassword()));
		}

		//国际化
		resource.setLanguageType(lanType);
		//end

		resource.setStatus(-3);	//表示未激活
		resource.setIsSuperUser(true);
		resource.setIsChannelPerson(false);
		resource.setActiveTime(System.currentTimeMillis());
		resource.setActiveStatus(false);
        resource.setRoleCategory(0L);
        resource.setDelFlag(false);

		try {
			resource.setRegIpAddr(IPAddrUtil.getIpAddr(request));
			resource.setRegIpLocation(IPAddrUtil.getLocationFromIpAddr(resource.getRegIpAddr()));
		} catch (Exception e1) {
			logger.error("fail to get region by ip," + e1.getMessage());
		}
		Account account = accountRepository.save(resource);

        //更新rootParent
        account.setRootParent(account.getId());
        accountRepository.save(account);

		String token = HashAlgorithms.MD5("superuser_" + account.getId() + account.getActiveTime());

		//2.发送邮件，激活
		Map<String,String> logo_txt = tipService.getLogoData(resource,"logo_res_jh","logo_res_zhuce","active_email_url",SendCommonPostMail.EMAIL_HOST,SendCommonPostMail.EMAIL_USERNAME,SendCommonPostMail.EMAIL_PASSWORD);
		String activeInterfaceUrl =logo_txt.get("active_email_url")+"/api/reged/active/" + 0 + token + resource.getId();

//	    String cpNameEn =	accountService.getAccountSoureceName(account,account,1);
//	    String cpName =	accountService.getAccountSoureceName(account,account,0);

		String subject =logo_txt.get("logo_res_jh");

		StringBuilder content = new StringBuilder();
		content.append("<!doctype html> <html><head></head><body> ")
				.append("亲爱的用户：</br>")
				.append("您好，您于").append(DateUtil.format(new Date(account.getActiveTime()), DateUtil.C_TIME_PATTON_DEFAULT))
				//.append(account.getRegSource() == 0 ? "注册" + cpName + "账号" : "注册" + cpName + "账号").append(account.getEmail())
				.append(logo_txt.get("logo_res_zhuce")).append(account.getEmail())
				.append("，以下链接激活账号</br>")
				.append("<a href=\"" + activeInterfaceUrl + "\">").append(activeInterfaceUrl).append("</a>")
				.append("</body></html>");

		String contents = content.toString();
		List<String> list = new ArrayList<>();

		list.add(account.getEmail());

		try {
			Long beforeTime=new Date().getTime();

			Map<String,String> mailConfig = null;
			if(account.getRegSource()!=0){
				mailConfig = logo_txt;
			}
			//MailUtils.sendHtmlEmail(subject, contents, list, mailConfig);
			SendCommonPostMail.sendMailReuse(list, subject,contents,null,null,mailConfig);
//			MailUtils.sendHtmlEmail(subject,contents,list);
			Long afterTime=new Date().getTime();
			long useTime = afterTime - beforeTime;
			logger.info("发送激活邮件到[{}]耗时{}毫秒",account.getEmail(),useTime);
		} catch (Exception e) {
			logger.error(e.getMessage(),e);
			return new ResultModel(250,"邮件发送失败", "邮件发送失败");
		}

		if(!ValidateUtil.isValid(account)){
			return new ResultModel(201,"注册失败", "注册失败");
		}
		userLogService.insertLog(account, LogEnumType.REGED.getCode(), LogEnumType.REGED.getName() + account.getEmail(), account, null, null, request);

		if(account.getRegSource()==2){
			return new ResultModel(200, "注册成功", "激活邮件已发送至您的邮箱：" +account.getEmail()+
					",请前往您的邮箱激活，完成账号注册");
		}

		return ResultModel.OK(account);
	}

	/**
	 * 激活账号，同时发送验证邮件
	 * @param httpServletRequest
	 * @param message flag（0,1）+token（32位）+ id
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "active/{message}" ,method = RequestMethod.GET)
	@ResponseBody
    public ModelAndView active(HttpServletRequest httpServletRequest, @PathVariable String message)  {
        String flagstr = message.substring(0, 1);
        String tokenstr = message.substring(1, 33);
        String idstr = message.substring(33, message.length());
        //验证url（token合法性，日期的时效性,url是否重复提交）

        logger.info(idstr);
        logger.info(tokenstr);

        String activeResult = this.regedService.active(tokenstr, Long.parseLong(idstr));

        Account account = this.accountRepository.findOne(Long.parseLong(idstr));

        if(account!=null){
            logger.info("账号[{}]激活状态[{}] ",account.getEmail(),activeResult);
        }


        if ("success".equals(activeResult)) {
//            Account one = this.accountRepository.findOne(Long.parseLong(idstr));
            //flag=1表示老用户激活
            //flag=0表示新用户激活  防止二次点击修改status
            if (flagstr.equals("0") && account.getStatus() != -1) {
                account.setStatus(-2);
                logger.info("防止重复，更新状态为-2[{}]",account.getEmail());
            }
            account.setActiveStatus(true);
            account.setFlowRestrict(true);
            Account result = this.accountRepository.save(account);


            //加上判断,防止多次点击  和 老用户不用审核
            if (account.getStatus() != -1 && "0".equals(flagstr)) {
                /**
                 * 激活成功发送审核请求
                 */
                String ip = "unknown";
                try {
                    ip = IPAddrUtil.getIpAddr(httpServletRequest);
                } catch (Exception e) {
                    logger.error("fail to parse ip");
                }
                if (RedisUtilNew.getInstance().tryLock(account.getEmail() + account.getId(), VALID_EMAIL_EXPIRE_TIME) != null) {
                    try {
                        logger.info("发送审核邮件[{}]",account.getEmail());
                        this.regedService.verifyAccount(account, ip);
                    } finally {
                        RedisUtilNew.getInstance().releaseLock(account.getEmail() + account.getId());
                    }
                } else {
                    logger.info("重复的用户注册审核请求[{}]", account.getEmail());
                }
            }

            return new ModelAndView("redirect:/success.html");
        } else if ("repeat url".equals(activeResult)) {
            logger.info("账号[{}]激活状态[{}] ",account.getEmail(),"重复激活");
            return new ModelAndView("redirect:/success.html");
        } else {
            //url验证失败，返回失败界面，并带上参数
            logger.info("账号[{}]激活状态[{}] ",account.getEmail(),"激活链接错误");
            return new ModelAndView("redirect:/failure.html#?message=" + flagstr + account.getId());
        }
    }

	/**
	 * 激活后审核接口
	 * @param httpServletRequest
	 * @param id
	 * @return
	 */
	@RequestMapping(value = "verify/{id}" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel verify(HttpServletRequest httpServletRequest,@PathVariable Long id)
	{
		Account account = this.accountRepository.findOne(id);

		//状态改为审核中
		account.setStatus(-1);
		account.setVerifyEmail(false);
		Account result = this.accountRepository.save(account);

		String token = HashAlgorithms.MD5("superuser_" + id);
		String ipAddr = IPAddrUtil.getIpAddr(httpServletRequest);



		//审核
		logger.info("调用office系统接口, 发送审核邮件. accountid: {}", account.getId());
		final String sendEmailUrl = VERIFY_EMAIL_SEND_URL + account.getId() + "/" + token + "?ip=" + (StringUtil.isEmpty(ipAddr) ? "" : ipAddr);
		String reqResult = HttpClientUtil.doHttpGetRequest(sendEmailUrl, "trackingio");
		logger.info("调用office系统接口, 发送审核邮件. accountid: {}, result:{}", account.getId(), reqResult);

		return ResultModel.OK(result);
	}

	/**
	 * 修改用户email
	 * @param id
	 * @param oldemail
	 * @param newemail
	 * @return
	 */
	@RequestMapping(value = "modifyemail/{id}" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel modifyemail(@PathVariable Long id ,@RequestParam String oldemail,@RequestParam String newemail,@RequestHeader(name = "lang", defaultValue = "zh") String lanType)
	{
		Account one = this.accountRepository.findOne(id);

		//验证新邮箱唯一性
		Account accountDB = authService.check(newemail);
		if (accountDB != null) {
			return new ResultModel(-1,"该邮箱已存在，请更换邮箱", "该邮箱已存在，请更换邮箱");
		}

		//验证用户是否已激活
		if(one.getStatus() >= -2 )
		{
			return new ResultModel(251,"邮箱已激活", "邮箱已激活");
		}

		/**
		 * 重新发送激活邮件
		 */

		//重置状态
		one.setActiveTime(new Date().getTime());
		one.setActiveStatus(false);

		Account account = this.accountRepository.save(one);

		String token = HashAlgorithms.MD5("superuser_" + account.getId()+account.getActiveTime());


		//判断新老用户
		int flag = 0;
		if(one.getStatus()==1)
		{
			flag = 1;
		}
		//2.发送邮件，激活
		Map<String,String> logo_txt = tipService.getLogoData(account,"logo_res_jh","logo_res_zhuce","active_email_url",SendCommonPostMail.EMAIL_HOST,SendCommonPostMail.EMAIL_USERNAME,SendCommonPostMail.EMAIL_PASSWORD);
		String activeInterfaceUrl =logo_txt.get("active_email_url")+"/api/reged/active/" + flag + token + account.getId();

		//		String activeInterfaceUrl = ACTIVE_EMAIL_SEND_URL+flag+token+account.getId();

		Boolean emialResult = this.regedService.sendactivewhenUpdateEmail(activeInterfaceUrl,newemail,one, lanType);

		//验证激活邮件是否成功
		if(emialResult == false)
		{
			return new ResultModel(250,"邮件发送失败", "邮件发送失败");
		}
		Account result = this.regedService.modifyEmail(id,oldemail,newemail);
		return ResultModel.OK(result);
	}

	/**
	 * 发送激活邮件
	 * @param id
	 * @param flag 0：新用户  1;老用户
	 * @return
	 */
	@RequestMapping(value = "sendeactivemail/{id}/{flag}" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel sendemail(@PathVariable Long id,@PathVariable Integer flag,@RequestHeader(name = "lang", defaultValue = "zh") String lanType)
	{

		String result = "failure";
		Account account = this.accountRepository.findOne(id);

		Account root = accountService.getRootAccountData(account,null);

        //判断用户是否已经激活
        if (account.getStatus() > -2 && flag == 0) {

            result = "actived";

        } else if (account.getIsSuperUser()) {
            //更新激活时间
            account.setActiveTime(new Date().getTime());
            account.setActiveStatus(false);
            this.accountRepository.save(account);

            String token = HashAlgorithms.MD5("superuser_" + id + account.getActiveTime());

			//String activeInterfaceUrl =( root.getRegSource() == 0 ? ACTIVE_EMAIL_SEND_URL : ACTIVE_EMAIL_SEND_URL_XH )+ flag + token + id;

			Map<String,String> logo_txt = tipService.getLogoData(account,"logo_res_jh","logo_res_zhuce","active_email_url",SendCommonPostMail.EMAIL_HOST,SendCommonPostMail.EMAIL_USERNAME,SendCommonPostMail.EMAIL_PASSWORD);
			String activeInterfaceUrl =logo_txt.get("active_email_url")+"/api/reged/active/" + flag + token + account.getId();

			Boolean emailResult = this.regedService.sendActiceEmail(activeInterfaceUrl, account.getId(),lanType);

            if (emailResult) {
                result = "success";
            } else {
                return new ResultModel(250, "邮件发送失败", "邮件发送失败");
            }

        }

		return ResultModel.OK(result);
	}

	/**
	 * 用户忘记密码邮件请求
	 * @param email
	 * @return
	 */
	@RequestMapping(value = "sendforgetpwdemail" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel sendForgetPwdEmail(@RequestParam String email,@RequestHeader(name = "lang", defaultValue = "zh") String lanType)
	{
		Map<String,Object> result = this.regedService.sendForgetPwdEmail(email, lanType);
		if("send email error".equals(result.get("message")))
		{
			return new ResultModel(250,"邮件发送失败", "邮件发送失败");
		}
		return ResultModel.OK(result);
	}

	/**
	 * 邮件确认后修改密码
	 * @param id
	 * @param pwd
	 * @return
	 */
	@RequestMapping(value = "updatepwd/{id}" ,method = RequestMethod.POST)
	@ResponseBody
	public ResultModel updatePwd(@PathVariable Long id,@RequestBody Map<String,String> pwd)
	{
		String result = "";
		Account account = this.accountRepository.findOne(id);
		if(ValidateUtil.isValid(account))
		{
		    if("admin@reyun.com".equals(account.getEmail())||"demo@reyun.com".equals(account.getEmail()))
            {
                result = "该账号密码不能修改";
            }
            else
            {
                result = this.regedService.updatepwd(id,pwd.get("newpwd"),pwd.get("token"));
            }

		}
		else
		{
			result = "invalid request";
		}
		return ResultModel.OK(result);
	}

	@RequestMapping(value = "validatepass" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel validatePass(@RequestParam String message)
	{
		String result = this.regedService.validatepass(message);
		return ResultModel.OK(result);
	}

	@RequestMapping(value = "find/{id}" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel validatePass(@PathVariable Long id)
	{
		Account result = this.accountRepository.findOne(id);
		return ResultModel.OK(result);
	}

	@RequestMapping(value = "findbyemail" ,method = RequestMethod.GET)
	@ResponseBody
	public ResultModel findbyemail(@RequestParam String email)
	{
		Account result = this.accountRepository.findAccountByEmail(email);
		if (result != null) {
			List<App> appList = appService.listAppByAccount(result.getId());
			result.setAppSize(appList.size());
		}
		return ResultModel.OK(result);
	}

	public static void main(String[] args) {
		String resultStr = HttpClientUtil.doHttpGetRequest("http://localhost:8091/api/server/check/account/email?email=41374718@qq.com", "");
		JSONObject fromObject = JSONObject.fromObject(resultStr);
		Object accountJSON = fromObject.get("content");
		if (!accountJSON.equals("null")) {
			System.out.println("不为空");
		}else{
			System.out.println("空");
		}
	}
}
