package com.reyun.controller.manage;

import com.reyun.model.Region;
import com.reyun.repository.RegionRepository;
import com.reyun.util.ResultModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Controller
@RequestMapping("region")
public class RegionController {

    @Autowired
    RegionRepository regionRepository;


    /**
     * @param response
     * @param level 地区等级（
     *                 all 获取全部层级区域{包括子区域}
     *                 root 获取所有第一层级区域
     *                 child 获取子节点（如果为子节点，需要提供对应的父节点）
     *                      ）
     * @param response rootNode : 父节点（非必需）
     * @return
     */
    @RequestMapping(value = "/find/{level}", method = RequestMethod.POST)
    @ResponseBody
    public ResultModel findRegion(HttpServletResponse response,@PathVariable String level,
                                  @RequestParam(value = "root_node",defaultValue = "1", required = false) Long rootNode) {
        List<Region>  rootRegions = regionRepository.getAllRootRegion();
        if(rootRegions == null){
            rootRegions = new ArrayList<>();
        }

        if("all".equals(level)){
            List<Region> regions = operateRegion(rootRegions);
            return ResultModel.OK(regions);
        }else if("root".equals(level)){
            return ResultModel.OK(rootRegions);
        }else if("child".equals(level)){
            List<Region>  childRegions = regionRepository.getChildRegion(rootNode);
            return ResultModel.OK(childRegions);
        }else{
            return ResultModel.OK();
        }
    }


    public List<Region> operateRegion(List<Region> rootRegions){
        if(rootRegions == null || rootRegions.size() == 0){
            return new ArrayList<>();
        }
        List<Region> allRegions = regionRepository.findAll();

        Stack<List<Region>> regionStack = new Stack<>();
        regionStack.push(rootRegions);

        Stack<List<Region>> regionStackList = handleChildRegion(rootRegions, allRegions, regionStack);
        List<Region> childRegions = regionStackList.pop();
        List<Region> regionList =  handelAllRegion(regionStackList, childRegions);

        return regionList;

    }

    /**
     * @param rootRegions
     * @param allRegions
     * @return 将树按层级放入栈（子节点后入）
     */
    private Stack<List<Region>> handleChildRegion(List<Region> rootRegions, List<Region> allRegions, Stack<List<Region>> regionStack){


        List<Region> childRegionList = new ArrayList<>();
        for (Region rootRegion : rootRegions) {
            for (Region region : allRegions) {
                if (region.getRootNode().equals(rootRegion.getId()) && !region.getId().equals(rootRegion.getId())) {
                    childRegionList.add(region);
                }
            }
        }

        regionStack.push(childRegionList);

        if(childRegionList.size() == 0){
            return  regionStack;
        }
        Stack<List<Region>>  regionStackList = handleChildRegion(childRegionList, allRegions, regionStack);
        return regionStackList;
    }

    /**
     *
     * @param regionStackList
     * @return 获取组装好的树结构区域
     */
    private List<Region> handelAllRegion(Stack<List<Region>> regionStackList, List<Region> childRegions){

        List<Region> rootRegions = regionStackList.pop();


        for(Region rootRegion : rootRegions){
            List<Region> childRegionList = new ArrayList<>();
            for(Region childRegion : childRegions){
                if(childRegion.getRootNode().equals(rootRegion.getId())){
                    childRegionList.add(childRegion);
                }
            }
            rootRegion.setChildRegions(childRegionList);
        }

        if(regionStackList.empty()){
            return rootRegions;
        }
        List<Region> regionList = handelAllRegion(regionStackList, rootRegions);
        return regionList;
    }




}
