package com.reyun.report;

import com.amazonaws.services.dynamodbv2.xspec.S;
import com.reyun.context.AppUtils;
import com.reyun.dic.ReportDealEnumType;
import com.reyun.model.Account;
import com.reyun.model.Channel;
import com.reyun.model.ReportChartResult;
import com.reyun.report.parse.ChartAvgParse;
import com.reyun.report.parse.ChartSumParse;
import com.reyun.report.parse.ChartTransNameParse;
import com.reyun.service.ChannelService;
import org.apache.poi.ss.formula.functions.T;
import org.jsoup.helper.StringUtil;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * Created by nolan on 19/12/2016.
 * description:
 */
public class ChartReportResultBuilder {

	protected ChannelService channelService = AppUtils.getApplicationContext().getBean(ChannelService.class);
	/**
	 * chart报表处理
	 *
	 * @return
	 * @throws IOException
	 */
	public ReportChartResult doChartReport(ReportChartResult reportChartResult, Long app, Account loginAcount, String reportName) throws IOException {

		return doChartReport(reportChartResult,app,loginAcount,reportName,null);
	}


	public ReportChartResult doChartReport(ReportChartResult reportChartResult, Long app, Account loginAcount, String reportName, Map<String,String> conditions) throws IOException {
		final String dealType = reportChartResult.getDealtype() != null && reportChartResult.getDealtype().size() > 0 ? reportChartResult.getDealtype().get(0) : "";
		final String isRate = reportChartResult.getIsrate() != null && reportChartResult.getIsrate().size() > 0 ? reportChartResult.getIsrate().get(0) : "";

		if (ReportDealEnumType.AVG.getCode().equals(dealType) && "true".equalsIgnoreCase(isRate)) {
			return new ChartAvgParse().parse(reportChartResult, app, loginAcount.getId(), reportName);
		} else if (ReportDealEnumType.SUM.getCode().equals(dealType)) {
			return new ChartSumParse().parse(reportChartResult, app, loginAcount.getId(), reportName);
		}

		ReportChartResult result = new ChartTransNameParse().parse(reportChartResult, app, loginAcount.getId(), reportName);

		this.reservMTTIreportData(result,reportName,conditions);
		this.reservUserEngagementreportData(result,reportName,conditions);
		this.reservTerminalreportData(result,reportName,conditions);
		return result;
	}



	private void reservMTTIreportData(ReportChartResult result,String reportName, Map<String,String> conditions){

		//MTTI趋势洞察  折线图 处理 （将 原 x,y 轴数据调换）

		if(!reportName.startsWith("black_top5_mtti_")){
			return;
		}

		final List<String>[] names = new List[]{new ArrayList<>(),new ArrayList()};

		Map<String,List<String>> mttiNames = new HashMap<>();

		List<String> nameReprot = result.getName();

		if (nameReprot != null && nameReprot.size() > 0) {

			nameReprot.forEach(v -> {
				if (v.endsWith("d")) {
					names[1].add(v);
				} else {
					names[0].add(v);
				}
			});
			names[0].remove(names[0].size() - 1);
			mttiNames.put("all", names[1]);
			mttiNames.put("day", names[0]);

		}

		//翻转数据
		// datas 的size与 sql查询到数据 表头的列数对应
		// Object[] 与 sql查询到数据 列数对应
		// 该数据格式为 report api 处理的格式
		List<Object[]> datas = result.getVal();
		int keyLength = result.getKey().size();

		List<Object[]> vals = new ArrayList();

		for(int i=0;i< keyLength;i++){

			int w = datas.size() - 1;

			Object[] row = new Object[w];

			for (int j = 0; j < w; j++) {

				Object orgw = datas.get(j)[i];
				//计算百分比
				Object install_num = datas.get(w)[i];

				if (orgw != null && install_num != null) {
					long l_num_install = Long.parseLong(install_num==null?"0":install_num.toString());

					if(l_num_install== 0){
						row[j] = new Double("0.00");
					}else {

						double rateVale = Long.parseLong(orgw==null?"0":orgw.toString())*100 /l_num_install;
						BigDecimal b = new BigDecimal(rateVale);
						row[j] = b.setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
					}

				} else {
					row[j] = new Double("0.00");
				}
			}

			vals.add(row);

		}

		result.setVal(vals);


		//翻转 key name
		result.setName(result.getKey());
		result.setKey(mttiNames.get("day".equals(conditions.get("mtti_all"))?"day":"all"));
		result.setColumn(new ArrayList() {
			{
				add("时间范围");
			}
		});

	}
	private void reservUserEngagementreportData(ReportChartResult result,String reportName, Map<String,String> conditions){

		//柱状图 处理 （将 原 x,y 轴数据调换）

		if(!reportName.startsWith("userengagement_")){
			return;
		}

		final List<String>[] names = new List[]{new ArrayList<>(),new ArrayList()};

		Map<String,List<String>> mttiNames = new HashMap<>();

		List<String> nameReprot = result.getName();

		if (nameReprot != null && nameReprot.size() > 0) {

			nameReprot.forEach(v -> {
				if (v.endsWith("d")) {
					names[1].add(v);
				} else {
					names[0].add(v);
				}
			});
			mttiNames.put("all", names[1]);
			mttiNames.put("day", names[0]);

		}
		if(reportName.endsWith("_bydate")) {
			// todo 日期顺序 和下面的0值生成的顺序
			//日期的key合并
				if (!result.getKey().contains(conditions.get("startdate"))) {
					result.getKey().add(conditions.get("startdate"));
				}
				if (conditions.containsKey("duration_dates") && !StringUtil.isBlank(conditions.get("duration_dates"))) {
					String[] duration_dates = conditions.get("duration_dates").split(",");
					for (String duration_date : duration_dates) {
						if (!result.getKey().contains(duration_date)) {
							result.getKey().add(duration_date);
						}
					}

				}
		}else if(reportName.endsWith("_bychannel")){
			if(conditions.containsKey("duration_cid") &&!StringUtil.isBlank(conditions.get("duration_cid"))){
				List<Long> cids = new ArrayList<>();
				String[] duration_cids = conditions.get("duration_cid").split(",");
				List<String> cidsStr = Arrays.asList(duration_cids);
				for (String cid : cidsStr) {
					cids.add(Long.parseLong(cid));
				}
				final List<Channel> channels = channelService.listChannelByIds(cids);
				if(cids.contains(-1L)){
					Channel channel = new Channel();
					channel.setId(-1L);
					channel.setName("自然量");
					channels.add(channel);
				}
				for (Channel channel : channels) {
					if(!result.getKey().contains(channel.getName())) {
						result.getKey().add(channel.getName());
					}
				}
			}

		}

		//翻转数据
		// datas 的size与 sql查询到数据 表头的列数对应
		// Object[] 与 sql查询到数据 列数对应
		// 该数据格式为 report api 处理的格式
		List<Object[]> datas = result.getVal();
		int keyLength = result.getKey().size();

		List<Object[]> vals = new ArrayList();

		for(int i=0;i< keyLength;i++){

			int w = datas.size();

			Object[] row = new Object[w];

			for (int j = 0; j < w; j++) {
				Object orgw;
				if(datas.get(j).length>i){
					orgw = datas.get(j)[i];
				}else{
					orgw = 0;
				}
				if (orgw != null) {
					row[j] = orgw;
				} else {
					row[j] = 0;
				}
			}
			vals.add(row);

		}

		result.setVal(vals);


		//翻转 key name
		result.setName(result.getKey());
		result.setKey(mttiNames.get("day"));
		/*result.setColumn(new ArrayList() {
			{
				add("时间范围");
			}
		});*/

	}
	private void reservTerminalreportData(ReportChartResult result, String reportName, Map<String,String> conditions) {
		if(!reportName.startsWith("terminalattributes_")){
			return;
		}
		//处理删除柱状图的占比字段
		List<String> name = result.getName();
		name.remove(1);
		result.setName(name);
		List<Object[]> val = result.getVal();
		if(val.size()<2)return;//返回最少两个字段，所以最少两个val
		val.remove(1);

		result.setVal(val);
		//处理省份的unknown
		if("terminalattributes_province_bydate".equals(reportName)){
			List<String> key = result.getKey();
			Object[] val_0 = result.getVal().get(0);
			List<String> key_new = new ArrayList<>();
			List<Object> val_new = new ArrayList<>();

			for (int i = 0; i < key.size(); i++) {
				if(!"unknown".equals(key.get(i))){
					key_new.add(key.get(i));
					val_new.add(val_0[i]);
				}
			}
			Object[] objects = val_new.toArray();
			result.setKey(key_new);
			result.getVal().set(0,objects);
		}
	}
}
