package com.reyun.report;

import com.reyun.context.AppUtils;
import com.reyun.dic.DataViewEnumType;
import com.reyun.model.*;
import com.reyun.repository.SdkEventRepository;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

public class FunnelReportBuilder extends AbstractReportParse {


    protected SdkEventRepository sdkEventRepository = AppUtils.getApplicationContext().getBean(SdkEventRepository.class);

    private static final Map<String, String> EVENT_TYPE_NAME = new HashMap() {{
        put("loggedin", "登录");
        put("reg", "注册");
        put("order", "订单");
        put("pay", "付费");
        put("install", "激活");
    }};


    public ReportListResult doListReportExport(ReportListResult reportListResult, DataViewEnumType dataViewEnumType, Long appid, Account account, ArrayList<String> con, String reportName, List<String> fillCampids, boolean searchtxt, List<String> searchIds, Map<String, String> conditions) {
        List<Map<String, Object>> dataValues = reportListResult.getVal();
        Map<String, Campaign> dicCampian = getDicCampaignMap(appid, account.getId());
        Map<String, Channel> dicChannel = getDicChannelMap(appid);

        for (int i = 0; i < dataValues.size(); i++) {
            Map<String, Object> line = dataValues.get(i);
            String cid = line.get("cid")+"";
            Channel channel = dicChannel.get(cid);
            Campaign campaign = dicCampian.get(line.get("channelid")+"");
            line.put("cid", channel == null ? "" : channel.getName());
            line.put("channleId", campaign == null ? "" : campaign.getName());
            line.put("date_range", conditions.get("start") + "-" + conditions.get("end"));
            line.put("ins_date_range", conditions.get("ins_start") + "-" + conditions.get("ins_end"));
            if ("-1".equals(cid)) {
                line.put("usergroup", "自然量");
            } else {
                line.put("usergroup", "推广量");
            }
            line.put("mode", "1".equals(conditions.get("user_funnel")) ? "用户转化漏斗" : "事件转化漏斗");
        }

        List<SdkEvent> sdkEvents = sdkEventRepository.findAllByAppid(appid);
        Map<String, String> sdkevnetName = sdkEvents.stream().collect(Collectors.toMap(SdkEvent::getEventNameSubstring, SdkEvent::getEventAlias, (v1, v2) -> v1));
        String[] viewColums = conditions.get("viewcolumn").split(",");

        Map<String, Object> nameMap = new HashMap<>();
        List<String> columKeyPart = new ArrayList<>();
        List<String> columNamePart = new ArrayList<>();
        for (int i = 0; i < viewColums.length; i++) {
            String clname = getLastClName(viewColums[i], nameMap, sdkevnetName);
            columKeyPart.add(clname);
            columNamePart.add(nameMap.get("name").toString());
        }

        reportListResult.setColumnkey(new ArrayList<>(Arrays.asList("mode", "usergroup", "ins_date_range", "date_range", "cid", "channleId")));
        reportListResult.getColumnkey().addAll(columKeyPart);
        reportListResult.setName(new ArrayList<>(Arrays.asList("漏斗模型", "用户群组", "用户群组日期", "事件范围日期", "渠道", "活动")));
        reportListResult.getName().addAll(columNamePart);
        return reportListResult;
    }


    public ReportListResult doListReport(ReportListResult reportListResult, DataViewEnumType dataViewEnumType, Long appid, Account account, ArrayList<String> con, String reportName, List<String> fillCampids, boolean searchtxt, List<String> searchIds, Map<String, String> conditions) {

        List<SdkEvent> sdkEvents = sdkEventRepository.findAllByAppid(appid);
        Map<String, String> sdkevnetName = sdkEvents.stream().collect(Collectors.toMap(SdkEvent::getEventNameSubstring, SdkEvent::getEventAlias, (v1, v2) -> v1));

        String[] viewColums = conditions.get("viewcolumn").split(",");

        Map<String, Object> valueData = reportListResult.getVal().get(0);
        Long firstValue = (Long) valueData.get(getLastClName(viewColums[0], null, sdkevnetName));
        List<Map<String, Object>> result = new ArrayList<>();
        for (int i = 0, j = viewColums.length; i < j; i++) {

            Map<String, Object> data = new HashMap<>();
            String clName = getLastClName(viewColums[i], data, sdkevnetName);

            Long value = (Long) valueData.get(clName);
            data.put("value", value);

            if (i == 0) {
                data.put("rate", firstValue == 0 ? 0 : 100);
                data.put("rate_remain", firstValue == 0 ? 100 : 0);
            } else {
                double ra_value = devideValue(value, firstValue);
                ra_value = ra_value > 100 ? 100.00 : ra_value;
                data.put("rate", ra_value);
                data.put("rate_remain", new BigDecimal(100 - ra_value).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            }

            if (i + 1 < j) {
                Long next_value = (Long) valueData.get(getLastClName(viewColums[i + 1], null, sdkevnetName));
                Double valueRate = devideValue(next_value, value);
                valueRate = valueRate > 100 ? 100.0 : valueRate;
                data.put("section_rate", valueRate);
            } else {
                data.put("section_rate", -1);
            }
            result.add(data);
        }

        reportListResult.setVal(result);
        return reportListResult;

    }

    private double devideValue(double num_a, double num_b) {

        if (num_b == 0) {
            return 0.00d;
        }
        return new BigDecimal(num_a / num_b * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    private String getLastClName(String clName, Map<String, Object> data, Map<String, String> sdkevnetName) {
        if (clName.startsWith("event")) {
            if (data != null) {
                data.put("name", sdkevnetName.containsKey(clName) ? sdkevnetName.get(clName) : clName);
            }
        } else {

            if (data != null) {
                data.put("name", EVENT_TYPE_NAME.get(clName));
            }

            if ("install".equals(clName)) {
                clName = "num_ins";
            } else if ("loggedin".equals(clName)) {
                clName = "loggedin_num";
            } else {
                clName = "num_" + clName;
            }
        }
        return clName;
    }

    public static void main(String[] args) {
        System.out.println(100 - 82.4);
    }
}
