package com.reyun.report;

import com.google.common.base.Function;
import com.google.common.collect.*;
import com.reyun.model.CamCamgroupMap;
import com.reyun.model.Campaign;
import com.reyun.model.CampaignGroup;
import com.reyun.model.ReportListResult;
import com.reyun.report.parse.ListCampaignGroupParse;
import com.reyun.util.DateUtil;
import com.reyun.util.ViewColumnDependUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * @author liyin
 * @description
 * @date
 */
public class ListCampGroupAndDateParse extends AbstractListReportParse
{
    Logger logger = LoggerFactory.getLogger(ListCampaignGroupParse.class);

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {
        List<CamCamgroupMap> querylist = Lists.newArrayList();
        ImmutableListMultimap<Long, CamCamgroupMap> dicCampaignGroupByCampId = ImmutableListMultimap.of();
        Map<String, Campaign> dicCampaignByCampaignid = Maps.newHashMap();

        List<CampaignGroup> itemlist = campaignGroupService.listCamgroupByAccountAndApp(app, account);
        Map<Long, CampaignGroup> dicCampaignGroupByGroupId = Maps.uniqueIndex(itemlist, new Function<CampaignGroup, Long>()
        {
            @Override
            public Long apply(CampaignGroup campaignGroup)
            {
                return campaignGroup.getId();
            }
        });
        if (dicCampaignGroupByGroupId.size() > 0) {
            querylist = this.camCamGroupMapRepository.listCamCampMapByIds(dicCampaignGroupByGroupId.keySet());
        }
        if (querylist.size() > 0) {
            dicCampaignGroupByCampId = Multimaps.index(querylist, new Function<CamCamgroupMap, Long>()
            {
                @Override
                public Long apply(CamCamgroupMap camCamgroupMap)
                {
                    return camCamgroupMap.getCampaign();
                }
            });
        }
        if (dicCampaignGroupByCampId.size() > 0) {
            dicCampaignByCampaignid = Maps.uniqueIndex(this.campaignService.findAllById(dicCampaignGroupByCampId.keySet()), new Function<Campaign, String>()
            {
                @Override
                public String apply(Campaign campaign)
                {
                    return campaign.getCampaignid();
                }
            });
        }

        //1. 按活动组分类推广活动
        final List<Map<String, Object>> val = reportResult.getVal();
        ListMultimap<String, Map> groupMultiMap = ArrayListMultimap.create();
        for (int i = 0, n = val != null ? val.size() : 0; i < n; i++) {
            Map rowItem = val.get(i);
            Campaign campaign = dicCampaignByCampaignid.get(rowItem.get("campaignid"));
            if (campaign == null) {
                continue;
            }

            List<CamCamgroupMap> refCamCamgroup = dicCampaignGroupByCampId.get(campaign.getId());
            if (refCamCamgroup == null || refCamCamgroup.size() == 0) {
                continue;
            }

            for (CamCamgroupMap camgroupMap : refCamCamgroup) {
                //关键字查看报表
                if(isSearchText&&!searchIds.contains(camgroupMap.getCampGroup().toString())){
                    continue;
                }
//                groupMultiMap.put(camgroupMap.getCampGroup(), rowItem);
                groupMultiMap.put(camgroupMap.getCampGroup()+"@"+rowItem.get("ds"), rowItem);
            }
        }

        //2. 按活动组统计推广活动汇总值
        Iterator<String> iter = groupMultiMap.keySet().iterator();
        List<Map<String, Object>> translist = Lists.newArrayList();
        while (iter.hasNext()) {
            String groupidAnds = iter.next();

            Long groupid = Long.parseLong(groupidAnds.split("@")[0]);

            List<Map> camplist = groupMultiMap.get(groupidAnds);


            ListMultimap<String, Object> attributemaplist = ArrayListMultimap.create();
            for (Map<String, Object> row : camplist) {
                for (Map.Entry<String, Object> entry : row.entrySet()) {
                    attributemaplist.put(entry.getKey(), entry.getValue());
                }
            }

            Map valuemap = Maps.newHashMap();
            final CampaignGroup campaignGroup = dicCampaignGroupByGroupId.get(groupid);
            valuemap.put("campaignid", campaignGroup != null ? campaignGroup.getName() : "");
            valuemap.put("ds",groupidAnds.split("@")[1]);
            for (String viewColumn : viewColumns) {
                final String fieldcode = viewColumn;

                List<Object> listbyAttribute = attributemaplist.get(fieldcode);
                if (listbyAttribute != null && listbyAttribute.size() > 0) {
                    Object[] obj = listbyAttribute.toArray();
                    if (obj[0] instanceof Long) {
                        Long[] f1 = Arrays.copyOf(obj, obj.length, Long[].class);
                        valuemap.put(fieldcode, plus(f1));
                    }
                    else if (obj[0] instanceof Double) {
                        Double[] f1 = Arrays.copyOf(obj, obj.length, Double[].class);
                        valuemap.put(fieldcode, plus(f1));
                    }
                }
            }

            boolean isall0 = true;
            for (String viewColumn : viewColumns) {
                if (viewColumn.toLowerCase().equals("duration")) {
                    valuemap.put(viewColumn, DateUtil.formatCaseTime(Long.valueOf(valuemap.get(viewColumn)==null?"0":valuemap.get(viewColumn).toString())));
                }
                Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewColumn);
                if (linkfields != null && linkfields.size() > 0) {
                    Object[] objects = linkfields.toArray();
                    String[] linkfield = Arrays.copyOf(objects, objects.length, String[].class);
                    final Object f1 = valuemap.get(linkfield[0]);

                    if (f1.getClass().isAssignableFrom(Long.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Long) f1) / 100, (Long) valuemap.get(linkfield[1])));
                        } else if(viewColumn.toLowerCase().startsWith("avg_time")){
                            valuemap.put(viewColumn, divideAvgTime(Long.valueOf(String.valueOf(f1.toString())), Long.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        }
                        else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }

//                            valuemap.put(viewColumn, divide((Long) f1, (Long) valuemap.get(linkfield[1])));
                            valuemap.put(viewColumn, divide((Long) f1, Long.valueOf(dataVal.toString())));
                        }
                    }
                    else if (f1.getClass().isAssignableFrom(Double.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        }
                        else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }
                            valuemap.put(viewColumn, divide((Double) f1, Double.valueOf(dataVal.toString())));
                        }
                    }
                }
                if (!"campaignid".equals(viewColumn) && isall0) {
                    Object tmpval = valuemap.get(viewColumn);
                    if (tmpval != null && (tmpval instanceof Double && Double.valueOf(String.valueOf(tmpval)) > 0) ||(tmpval instanceof Long && Long.valueOf(String.valueOf(tmpval)) > 0) || (tmpval instanceof String && (tmpval.toString().contains("秒")||tmpval.toString().contains("分")||tmpval.toString().contains("时")||tmpval.toString().contains("天"))&&!tmpval.equals("0秒"))) {
                        isall0 = false;
                    }
                }
            }
            valuemap.put("isall0", isall0);

            translist.add(valuemap);
        }

        //添加全0数据 add by sunhao
//        if(!isSearchText){
//            for (Long groupId : dicCampaignGroupByGroupId.keySet()) {
//                if (groupMultiMap.containsKey(groupId)) {
//                    continue;
//                }
//
//                Map<String, Object> map = getAllZeroValue(viewColumns);
//                map.put("campaignid", dicCampaignGroupByGroupId.get(groupId).getName());
//                translist.add(map);
//            }
//        }else{
//            for (String groupId : searchIds) {
//                if (groupMultiMap.containsKey(Long.parseLong(groupId))) {
//                    continue;
//                }
//
//                Map<String, Object> map = getAllZeroValue(viewColumns);
//                map.put("campaignid", dicCampaignGroupByGroupId.get(Long.parseLong(groupId)).getName());
//                map.put("isall0", true);
//                translist.add(map);
//            }
//        }

        reportResult.getName().remove(1);
        reportResult.getName().add(1, "推广活动组");
        reportResult.setVal(translist);

        //计算汇总行
//        List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);
//        reportResult.setVal(super.transCampaignGroupName(reportResult, app, account));
//        reportResult.getColumnkey().add(1,"campaignid");

        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId()
    {
        return "campaignid";
    }
}
