package com.reyun.report;

import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.ReportListResult;
import com.reyun.report.parse.ListChannelByCampaignParse;
import com.reyun.report.parse.ListChannelByChannelParse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class UserLifeCycleResultBuilder {
    private static final Logger logger = LoggerFactory.getLogger(UserLifeCycleResultBuilder.class);

    public Object doListReport(ReportListResult reportListResult, App app, Account account, String reportName, Map<String, String> conditions) {

        String startdate = conditions.get("startdate");
        String enddate = conditions.get("enddate");

        Map<String, Map<String, Object>> data = new HashMap<>();

        if (reportListResult.getVal() != null) {
            for (Map<String, Object> v : reportListResult.getVal()) {
                data.put(v.get("ds").toString(), v);
            }
        }

        List column = reportListResult.getColumn();
        column.remove("ds");

        this.getRatioData(data.get(enddate), data.get(startdate), column);

        List result = new ArrayList();
        result.add(data.get(enddate));
        reportListResult.setVal(result);
        return reportListResult;
    }

    public ReportListResult doChannelListReport(ReportListResult reportResult, App app, Account account, String reportName, Map<String, String> conditions, List<String> fillchannels) {
        return new ListChannelByChannelParse().parse(reportResult, app.getId(), account.getId(), Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(conditions.get("viewcolumn"))), reportName, !Strings.isNullOrEmpty(conditions.get("searchtxt")), fillchannels);
    }

    public Object doCampaignListReport(ReportListResult reportResult, App app, Account account, String reportName, Map<String, String> conditions, List<String> fillCampids) {
        return new ListChannelByCampaignParse().parse(reportResult, app.getId(), account.getId(), Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(conditions.get("viewcolumn"))), reportName, !Strings.isNullOrEmpty(conditions.get("searchtxt")), fillCampids);
    }

    private void getRatioData(Map<String, Object> data_end, Map<String, Object> data_start, List<String> keys) {
        for (String key : keys) {
            if (data_start == null || data_end == null || data_end.get(key) == null || data_start.get(key) == null) {
                data_end = new HashMap<>();
                data_end.put(key, 0);
                data_end.put(key + "_ratio", 0);
            } else {
                Double thisData = Double.valueOf(data_end.get(key).toString());
                Double preData = Double.valueOf(data_start.get(key).toString());

                if (preData == 0) {
                    data_end.put(key + "_ratio", 0.0);
                } else {
                    data_end.put(key + "_ratio", thisData == 0 ? 0.0 : new BigDecimal((thisData - preData) / preData * 100).setScale(2, BigDecimal.ROUND_HALF_UP));
                }
            }
        }
    }


}
