package com.reyun.report.parse;

import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.ReportChartResult;
import com.reyun.util.RandomUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

public class DemoMockCharParse {


    private static final Logger logger = LoggerFactory.getLogger(DemoMockCharParse.class);

    private ReportChartResult reportChartResult;
    private App app;
    private Account account;
    private String reportName;
    private Map<String, String> conditions;
    private boolean isRate;

    private static Map<String, Map> BLACK_TOP5_ORDER_TYPE = new HashMap();

    public ReportChartResult parse(ReportChartResult reportChartResult, App app, Account account, String reportName, Map<String, String> conditions) {


        this.reportChartResult = reportChartResult;
        this.app = app;
        this.account = account;
        this.reportName = reportName;
        this.conditions = conditions;

        return reportChartResult;
    }

    public ReportChartResult mockDatas(ReportChartResult reportChartResult, Map<String, String> conditions, String reportName) {

        DateTime start = new DateTime(conditions.get("startdate"));
        DateTime end = new DateTime(conditions.get("enddate"));
        int days = Days.daysBetween(start, end).getDays() + 1;

        this.reportChartResult = reportChartResult;
        this.conditions = conditions;
        this.reportName = reportName;

        this.isRate = reportChartResult.getIsrate() != null
                && reportChartResult.getIsrate().size() > 0
                && "true".equals(reportChartResult.getIsrate().get(0)) ? true : false;

        List<String> keys = new ArrayList<>();


        if (reportName.endsWith("_byhour")) {

            for (int i = 0; i < 24; i++) {
                keys.add(i + "");
            }
        } else if (reportName.endsWith("_byds") || reportName.endsWith("_ds_v20")) {
            for (int i = 0; i < days; i++) {
                keys.add(start.plusDays(i).toString("yyyy-MM-dd"));
            }
        } else if (reportName.startsWith("black_top5_") && reportName.endsWith("_ds")) {

            if (reportName.startsWith("black_top5_mtti_")) {

                days = days > 5 ? 6 : days;

                for (int i = 0; i < days; i++) {
                    keys.add(start.plusDays(i).toString("yyyy-MM-dd"));
                }

            } else {
                for (int i = 0; i < days; i++) {
                    keys.add(start.plusDays(i).toString("yyyy-MM-dd"));
                }
            }


        } else {

            keys = reportChartResult.getKey();

        }


        if (reportName.equals("black_top5_mtti_ds")) {
            reportChartResult.setName(keys);
        } else {
            reportChartResult.setKey(keys);
        }


        List<Object[]> charValue = new ArrayList<>();

        List<Object> totaldata = new ArrayList<>();

        Object[] alldata = new Object[1];

        if (reportName.startsWith("black_top5_mtti_")) {
            // MTTI 曲线处理

            Object[] baseValue = reportChartResult.getVal().get(0);

            int valueLenth = keys.size();


            for (int l = 0; l < valueLenth; l++) {

                Double baseData = 0d;

                for (int j = 0, nameLenght = reportChartResult.getName().size(); j < nameLenght; j++) {

                    if (baseData == 0) {
                        baseData = (Double) baseValue[l] + RandomUtils.nextDouble(0.0, 6.2);
                    } else {
                        baseData = baseData + RandomUtils.nextDouble(0.0, l == 0 ? 1.1 : 6.2);
                    }

                    if (baseData > 100) {
                        baseData = RandomUtils.nextDouble(99.9, 100.0);
                    }

                    if (l == 0) {

                        Object[] dataline = new Object[valueLenth];
                        dataline[l] = baseData;
                        charValue.add(dataline);

                    } else {
                        charValue.get(j)[l] = baseData;
                    }
                }
            }

        } else if (reportName.startsWith("userengagement")) {

            if ("chart".equals(conditions.get("viewtype")) && reportName.endsWith("_bychannel")) {
                this.reservKeyName(reportChartResult);
            }

            if (reportName.startsWith("userengagement") && reportName.endsWith("_bydate")) {
                // 用户参与度 渠道维度 处理后的结果再次 key name 互换
                if ("list".equals(conditions.get("viewtype"))) {
                    this.reservKeyName(reportChartResult);
                }
            }

            //用户参与度
            List<String> names = reportChartResult.getName();
            for (int i = 0; i < names.size(); i++) {
                commonChartLine(reportChartResult, reportChartResult.getKey(), alldata, charValue, totaldata);
            }

        } else {

            commonChartLine(reportChartResult, keys, alldata, charValue, totaldata);
        }

        reportChartResult.setVal(charValue);

        reportChartResult.setSubtotalval(alldata[0]);

        reportChartResult.setSubval(totaldata);

        // 防作弊2.0 demo 数据排序
        orderResult(reportChartResult, reportName, conditions);

        //终端分析 列表模式数据处理
        terminalattributesListDeal(reportChartResult, reportName, conditions);

        return reportChartResult;
    }

    private ReportChartResult terminalattributesListDeal(ReportChartResult reportChartResult, String reportName, Map<String, String> conditions) {
        if (!reportName.startsWith("terminalattributes")) {
            return reportChartResult;
        }

        if (!"list".equals(conditions.get("viewtype"))) {
            return reportChartResult;
        }


        // 处理占比数据
        reportChartResult.getName().add(1, reportChartResult.getName().get(0) + "占比");
        reportChartResult.getKey().add(0, "汇总");

        Object[] dataBase = reportChartResult.getVal().get(0);
        Object[] linePercent = new Object[dataBase.length + 1];
        linePercent[0] = 100.00;

        Long total = (Long) reportChartResult.getSubval().get(0);
        for (int i = 1; i < dataBase.length; i++) {
            linePercent[i] = new BigDecimal((long) dataBase[i] * 1.0 / total * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        }

        List<Object> oneLine = new ArrayList<>();
        oneLine.add(reportChartResult.getSubval().get(0));
        oneLine.addAll(Arrays.asList(reportChartResult.getVal().get(0)));
        reportChartResult.getVal().clear();
        reportChartResult.getVal().add(oneLine.toArray());
        reportChartResult.getVal().add(linePercent);
        return reportChartResult;

    }

    public void reservKeyName(ReportChartResult report) {
        List name = reportChartResult.getName();
        reportChartResult.setName(reportChartResult.getKey());
        reportChartResult.setKey(name);
    }

    public void commonChartLine(ReportChartResult reportChartResult,
                                List<String> keys,
                                Object[] alldata,
                                List<Object[]> charValue,
                                List<Object> totaldata) {
        int valueSize = reportChartResult.getVal().size();
        int valueLenth = keys.size();

        for (int i = 0; i < valueSize; i++) {

            Object[] dataline = new Object[valueLenth];

            Object[] lineTotal = new Object[2];

            if (reportName.startsWith("black_top5_list_")) {
                lineTotal[1] = null;
            } else {
                lineTotal[1] = alldata[0];
            }

            Object baseValue = reportChartResult.getVal().get(i)[0];

            int isdoubleType = 0;

            if (baseValue.getClass().isAssignableFrom(Double.class)) {
                isdoubleType = 1;
            } else if (isRate) {
                //计算比率
                isdoubleType = 3;

            } else {
                isdoubleType = 2;
            }

            if (isdoubleType == 1 && (reportName.startsWith("black_top5_mtti_") || reportName.startsWith("black_top5_list_mtti_"))) {
                //计算比率
                isRate = true;
            }

            for (int l = 0; l < valueLenth; l++) {

                boolean reservPoint = RandomUtils.nextBoolean();

                int rateValue = RandomUtils.nextInt(6, 15);

                if (isRate) {
                    //计算比率
                    if (l > RandomUtils.nextInt(3, 22)) {
                        // 下降
                        pubdataUpDown(0.0, RandomUtils.nextDouble(0.0, (double) baseValue), dataline, l, false, lineTotal);

                    } else if (l > 2 && reservPoint) {


                        if ((Double) dataline[l - 1] > (Double) dataline[l - 2]) {
                            //之前上升，现在下降
                            pubdataUpDown(0.0, RandomUtils.nextDouble(0.0, (double) baseValue), dataline, l, false, lineTotal);

                        } else {

                            //之前下降，现在上升
                            pubdataUpDown(0.0, RandomUtils.nextDouble((double) baseValue, (double) baseValue > 1 ? 99.1 : 0.98), dataline, l, true, lineTotal);
                        }
                    } else {

                        Double preValue = l == 0 ? 0.1 : (Double) (l == 0 ? baseValue : dataline[l - 1]);

                        pubdataUpDown(preValue, RandomUtils.nextDouble(0.0, preValue == 0 ? 1.2 : (Double) baseValue), dataline, l, true, lineTotal);

                    }

                } else if (isdoubleType == 1) {

                    Double preValue = (Double) (l == 0 ? baseValue : dataline[l - 1]);

                    if (preValue <= 0) {
                        preValue = RandomUtils.nextDouble(0.0, 60.99);
                    }

                    double itme = RandomUtils.nextDouble((double) baseValue, rateValue * (double) baseValue);

                    if (l > RandomUtils.nextInt(3, 22)) {

                        itme = RandomUtils.nextDouble((double) baseValue * 5, 5 * rateValue * (double) baseValue);

                        // 下降
                        pubdataUpDown(preValue, itme, dataline, l, false, lineTotal);

                    } else if (l > 3 && reservPoint) {


                        if ((Double) dataline[l - 1] > (Double) dataline[l - 2]) {
                            //之前上升，现在下降
                            pubdataUpDown(preValue, itme, dataline, l, false, lineTotal);

                        } else {
                            //之前下降，现在上升
                            pubdataUpDown(preValue, itme, dataline, l, true, lineTotal);

                        }
                    } else {
                        pubdataUpDown(preValue, itme, dataline, l, true, lineTotal);
                    }

                } else {

                    Long preValue = 0L;

                    Long baseValueL = Long.parseLong(baseValue.toString());

                    if (baseValueL <= 0) {
                        baseValueL = RandomUtils.nextLong(0, 266);
                    }

                    if (l == 0) {
                        preValue = baseValueL;
                    } else {
                        preValue = Long.parseLong(dataline[l - 1].toString());
                    }

                    Long itme = RandomUtils.nextLong(baseValueL, rateValue * baseValueL);

                    dataline[l] = preValue + itme;


                    if (l > RandomUtils.nextInt(3, 23)) {

                        itme = RandomUtils.nextLong(baseValueL * 3, rateValue * baseValueL * 3);

                        pubdataUpDown(preValue, itme, dataline, l, false, lineTotal);

                    } else if (l > 3 && reservPoint) {

                        if ((Long) dataline[l - 1] > (Long) dataline[l - 2]) {
                            //之前上升，现在下降

                            pubdataUpDown(preValue, itme, dataline, l, false, lineTotal);

                        } else {
                            //之前下降，现在上升
                            pubdataUpDown(preValue, itme, dataline, l, true, lineTotal);
                        }

                    } else {

                        pubdataUpDown(preValue, itme, dataline, l, true, lineTotal);

                    }
                }

            }

            // 该行汇总数据


            charValue.add(dataline);


            if (isRate) {
                int index = RandomUtils.nextInt(0, dataline.length);
                totaldata.add(dataline[index]);
                alldata[0] = dataline[index];

            } else {
                totaldata.add(lineTotal[0]);
                alldata[0] = lineTotal[1];
            }

        }
    }

    public void orderResult(ReportChartResult reportChartResult, String reportName, Map<String, String> conditions) {

        if (!reportName.startsWith("black_top5_")) {
            return;
        }

        if (!conditions.containsKey("orderby")) {
            return;
        }

        String orderType = conditions.get("orderby");

        String[] shortNameArr = reportName.split("_");

//        int shortPoint = shortNameArr.length == 4 ? 2 : 3;

        Integer typeIndex = getOrderTypeByMap(orderType, shortNameArr[shortNameArr.length - 2]);

        if (typeIndex == null) {
            return;
        }

        if (typeIndex < 200) {

            sortLines(reportChartResult, typeIndex);

        } else if (typeIndex == 200) {
            int sizeAll = reportChartResult.getVal().size();
            int indexAll[] = new int[sizeAll];
            for (int i = 0; i < sizeAll; i++) {
                indexAll[i] = i;
            }
            sortLines(reportChartResult, indexAll);
        }


    }

    private void sortLines(ReportChartResult reportChartResult, int... index) {
        List<Object[]> value = reportChartResult.getVal();
        for (int i = 0; i < index.length; i++) {
            Object[] data;
            if (value.get(i)[0].getClass().isAssignableFrom(Double.class)) {
                data = Arrays.stream(value.get(index[i])).sorted((v1, v2) ->
                        (double) v1 - (double) v2 >= 0 ? -1 : 1).toArray();

//                System.out.println(index[i]+ "[--] "+ Arrays.asList(data).toString());

            } else {

                data = Arrays.stream(value.get(index[i])).sorted((v1, v2) ->
                        (long) v1 - (long) v2 >= 0 ? -1 : 1).toArray();
//                System.out.println(index[i]+ "[--] "+ Arrays.asList(data).toString());
            }

            value.set(index[i], data);

        }
    }

    private Integer getOrderTypeByMap(String type, String shorName) {

        Map<String, Integer> mapIndex = BLACK_TOP5_ORDER_TYPE.get(shorName);
        if (mapIndex == null) {

            // click
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("click", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("fakeTotal", 200);
            mapIndex.put("channelBrush", 1);
            mapIndex.put("appBrush", 2);
            mapIndex.put("bigClick", 3);

            //black_top5_device_cid
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("device", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("fake", 200);
            mapIndex.put("Simulator", 1);
            mapIndex.put("Brush", 2);

            //black_top5_hijack_cid
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("hijack", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("clickJack", 1);

            //black_top5_hijackpack_cid
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("hijackpack", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("clickJack", 1);


            //black_top5_analysis_cid
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("analysis", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("precision", 0);
            mapIndex.put("blurry", 1);

            //black_top5_corpse_cid
            mapIndex = new HashMap<>();
            BLACK_TOP5_ORDER_TYPE.put("corpse", mapIndex);
            mapIndex.put("installTotal", 200);
            mapIndex.put("zombie", 0);
            mapIndex.put("active", 1);

        }

        mapIndex = BLACK_TOP5_ORDER_TYPE.get(shorName);

        if (mapIndex == null) {
            return null;
        }

        return mapIndex.get(type);
    }

    private boolean checkBlackTopName(String reportName, String shortName) {

        return reportName.startsWith("black_top5_" + shortName) || reportName.startsWith("black_top5_list_" + shortName);

    }

    public ReportChartResult dealRateData(ReportChartResult reportChartResult) {


        if (reportChartResult.getIsrate() != null && reportChartResult.getIsrate().size() > 0 && "true".equals(reportChartResult.getIsrate().get(0))) {

            List<Object[]> datas = reportChartResult.getVal();

            for (Object[] obarr : datas) {

                for (int i = 0; i < obarr.length; i++) {

                    if (obarr[i].getClass().isAssignableFrom(Double.class)) {

                        double d = (double) obarr[i];

                        if (d >= 100) {

                            BigDecimal b = new BigDecimal(RandomUtils.nextDouble(0.1, 99.2));
                            obarr[i] = b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                        }

                    }

                }

            }

        }

        return reportChartResult;

    }

    private void checkZero(Long temp, Double tempd) {

        if (temp.intValue() < 0 || tempd.doubleValue() < 0) {
            logger.debug("Long:[{}],Double:[{}]", temp, tempd);
        }

    }

    private void pubdataUpDown(Double preValue, Double itme, Object[] dataline, int l, boolean up, Object[] total) {

        double temp = up ? preValue + itme : itme - preValue;

        temp = temp < 0 ? -1 * temp : temp;

        if (isRate && temp >= 100) {
            temp = RandomUtils.nextDouble(0.0, 99.9);
        }

        checkZero(1L, temp);

        BigDecimal bg = new BigDecimal(temp).setScale(2, RoundingMode.UP);

        temp = bg.doubleValue();

        dataline[l] = temp;

        double old = total[0] == null ? 0 : (double) total[0];

        total[0] = new BigDecimal(temp + old).setScale(2, RoundingMode.HALF_UP).doubleValue();


        if (total[1] != null && total[1].getClass().isAssignableFrom(Long.class)) {
            total[1] = Double.valueOf(total[1].toString());
        }

        double all = total[1] == null ? 0 : (double) total[1];

        total[1] = new BigDecimal(temp + all).setScale(2, RoundingMode.HALF_UP).doubleValue();

    }

    private void pubdataUpDown(Long preValue, Long itme, Object[] dataline, int l, boolean up, Object[] total) {

        Long temp = up ? preValue + itme : itme - preValue;

        temp = temp < 0 ? -1 * temp : temp;

        checkZero(temp, 1.0);

        dataline[l] = temp;

        Long old = total[0] == null ? 0 : (long) total[0];

        total[0] = temp + old;

        Long all = total[1] == null ? 0 : (long) total[1];

        total[1] = temp + all;

    }
}
