package com.reyun.report.parse;

import com.reyun.context.AppUtils;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractListReportParse;
import com.reyun.repository.ChannelAdvertisingRepository;
import com.reyun.repository.ViewColumnRepository;
import com.reyun.service.impl.ReportServiceImp;
import com.reyun.util.RandomUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by sunhao on 17/7/6.
 * desc:按照推广活动和日期group出的结果处理,无需要汇总。
 */
public class DemoMockDateParse extends AbstractListReportParse {

    private static final Logger logger = LoggerFactory.getLogger(DemoMockDateParse.class);

    private ViewColumnRepository viewColumnRepository = AppUtils.getApplicationContext().getBean(ViewColumnRepository.class);
    private ChannelAdvertisingRepository channelAdvertisingRepository = AppUtils.getApplicationContext().getBean(ChannelAdvertisingRepository.class);


    private List<Object> columnsKeys;
    private List<Object> columns;
    private Map<String, Object> firstLine;
    private List<String> keys;
    private List<Map<String, Object>> valueList;
    private String viewType;
    private String reportName;
    private Map<String, String> conditions;
    private List<String> names;
    private List<String> orignalColums;

    private static final List<String> emailKeys = new ArrayList() {{
        add("glorsia@zen-game.cn");
        add("chansgxingfen@qxhd.com");
        add("fanjiayi@hetaolinmedia.com");
        add("lincaishong@pancou.com");
        add("phoesna.pang@centrixlink.com");
        add("magsgie.guo@mobcomedia.com");
        add("6162169984@qq.com");
        add("wenksy@zen-game.cn");
        add("4130229764@qq.com");
    }};


    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) throws JSONException {
        return null;
    }

    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Map<String, String> conditions) {

        valueList = reportResult.getVal();

        firstLine = valueList.get(0);

        columnsKeys = reportResult.getColumnkey();

        columns = reportResult.getColumnkey();

        keys = reportResult.getKey();

        this.reportName = reportName;

        this.conditions = conditions;

        names = reportResult.getName();

        if ("bd_bycampaign".equals(reportName)) {
            orignalColums = new ArrayList<>();
            orignalColums.add("campaignid");
        }

        if (reportName.equals("funnel_user_event_list")) {
            funnelMock(reportResult);
            return reportResult;
        }


        if ("roi_byds".equals(reportName)) {

            keys = this.getKeysByds(conditions.get("install_startdate"), conditions.get("install_enddate"), -1);

            //整理动态列名
            List<String> columds = this.getKeysByds(conditions.get("pay_startdate"), conditions.get("pay_enddate"), -1);
            names = names.subList(0, 4);
            columnsKeys = columnsKeys.subList(0, 4);

            for (String ds : columds) {
                names.add(ds + "_新增用户付费金额");
                columnsKeys.add(ds + "_amt_pay");
            }

            reportResult.setName(names);
            reportResult.setColumnkey(columnsKeys);
        } else if ("roi_byinterval".equals(reportName)) {


            //激活/付费详情
            int daysRangePay = Days.daysBetween(new DateTime(conditions.get("pay_startdate")), new DateTime(conditions.get("pay_enddate"))).getDays() + 1;

            names = new ArrayList<>();
            names.add("付费激活周期");

            columnsKeys = new ArrayList<>();
            columnsKeys.add("key");

            int pay_interval = Integer.parseInt(conditions.get("pay_interval"));
            daysRangePay = daysRangePay % pay_interval != 0 ? daysRangePay / pay_interval + 1 : daysRangePay / pay_interval;

            for (int i = 1; i <= daysRangePay; i++) {
                names.add("第" + i + "付费周期");
                columnsKeys.add((i - 1) + "");

            }

            int daysRangeIns = Days.daysBetween(new DateTime(conditions.get("install_startdate")), new DateTime(conditions.get("install_enddate"))).getDays() + 1;
            int install_interval = Integer.parseInt(conditions.get("install_interval"));

            int daysRangeInsGroup = daysRangeIns % install_interval != 0 ? daysRangeIns / install_interval + 1 : daysRangeIns / install_interval;

            DateTime start = new DateTime(conditions.get("install_startdate"));

            keys = new ArrayList<>();

            for (int i = 0; i < daysRangeInsGroup; i++) {

                StringBuffer line = new StringBuffer();
                line.append(start.toString("yyyy-MM-dd")).append("~");
                start = start.plusDays(install_interval - 1);

                if ((i + 1) * install_interval > daysRangeIns) {
                    line.append(new DateTime(conditions.get("install_enddate")).toString("yyyy-MM-dd"));
                } else {
                    line.append(start.toString("yyyy-MM-dd"));
                }

                start = start.plusDays(1);

                keys.add(line.toString());

            }

            reportResult.setName(names);
            reportResult.setKey(keys);
            reportResult.setColumnkey(columnsKeys);


        } else {


            //动态显示列数
            this.getKeysForResultList(reportResult, viewColumns);
        }

        viewType = conditions.get("viewtype");


        if (reportName.equals("campaigninfo_bypackagecampaign")
                || reportName.equals("campaigninfo_bysubpackagecampaign")) {
            viewType = "campaign";
        }

        if ("black_click_ds".equals(reportName) || reportName.endsWith("_ds_v20")) {
            viewType = "date";
        }


        int line_show_count = 10; // 默认mock 10 行数据

        if (reportResult.getShowLine() > 0) {

            line_show_count = reportResult.getShowLine();
        }


        if (reportName.startsWith("roi_")) {

            line_show_count = keys.size();

        } else if ("date".equals(viewType) && !reportName.equals("adsenseinfo_byadid") && !reportName.equals("adsenseinfo_byadtype")) {

            keys = this.getKeysByds(conditions.get("startdate"), conditions.get("enddate"), 1);

            line_show_count = keys.size();

        } else if ("subaccount".equals(viewType)) {

            keys = emailKeys;
            line_show_count = getLineCountByDays();

        } else {
            line_show_count = getLineCountByDays();
        }


        long s = System.currentTimeMillis();
        this.mockResultDatas(reportResult, line_show_count);

        if (reportName.startsWith("advertisinginfo_") || reportName.startsWith("adsenseinfo_")) {
            reportResult.getName().add(1, "渠道");
            reportResult.getColumnkey().add(1, "cid");

            if (reportName.equals("advertisinginfo_cash_day")) {
                reportResult.getName().add(1, "广告位ID");
                reportResult.getColumnkey().add(1, "adv_id");
                reportResult.getName().add(2, "变现平台");
                reportResult.getColumnkey().add(2, "adplatform");
                reportResult.getName().add(3, "广告位名称");
                reportResult.getColumnkey().add(3, "adv_name");
            }

            Map<String, String> channelsName = channelAdvertisingRepository.listAll().stream().collect(
                    Collectors.toMap(v -> v.getCid() + "", v -> v.getName(), (v1, v2) -> v1));

            String cid = conditions.get("cid");
            Object[] dataName = channelsName.values().toArray();

            List<Object> ckeys = reportResult.getColumnkey();
            String advname = null;
            if (ckeys.contains("adv_name")) {
                advname = "adv_name";
            } else if (ckeys.contains("ad_name")) {
                advname = "ad_name";
            }
            String platform = null;
            if (ckeys.contains("adplatform")) {
                platform = "adplatform";
            } else if (ckeys.contains("adplatform_name")) {
                platform = "adplatform_name";
            }

            String[] advnameDic = new String[]{"新素材", "付费", "IOS", "多素材", "工具"};
            String[] platformDic = new String[]{"穿山甲", "优量汇", "百青藤", "TopOn", "Sigmob", "快手", "Mintegral"};
            String[] advIdDic = null;
            String[] advTypeDic = null;
            if (reportName.equals("advertisinginfo_adname_rename")) {
                advIdDic = new String[]{"839296815", "839233815", "34229296815", "45454696815", "1222296815", "453296815"};
                advTypeDic = new String[]{"信息流", "激励视频", "开屏", "插屏"};
            }


            for (Map data : reportResult.getVal()) {

                if ("汇总".equals(data.get("name"))) {
                    continue;
                }
                if (cid != null) {
                    String cidArr[] = cid.split(",");
                    if (cidArr.length > 1) {
                        int index_cid = (int) (Math.random() * (cidArr.length));
                        data.put("cid", channelsName.get(cidArr[index_cid]));
                    } else {
                        data.put("cid", channelsName.get(cid));
                    }

                } else {
                    int index = (int) (Math.random() * (channelsName.size()));
                    data.put("cid", dataName[index]);
                }

                if (advname != null) {
                    RandomUtils.nextInt(0, advnameDic.length);
                    StringBuffer name = new StringBuffer();
                    for (int i = 0, j = RandomUtils.nextInt(3, advnameDic.length); i < j; i++) {
                        name.append(advnameDic[RandomUtils.nextInt(0, j)]).append("-");
                    }
                    data.put(advname, name.toString());
                }

                if (platform != null) {
                    data.put(platform, platformDic[RandomUtils.nextInt(0, platformDic.length)]);
                }

                if (advIdDic != null) {
                    data.put("adid", advIdDic[RandomUtils.nextInt(0, advIdDic.length)]);
                }

                if (advTypeDic != null) {
                    int type = RandomUtils.nextInt(0, advTypeDic.length);
                    data.put("ad_type_name", advTypeDic[type]);
                    data.put("ad_type", type);
                }
            }
        }

        logger.info("demo[{}] mock 数据用时{}", reportName, System.currentTimeMillis() - s);

        return reportResult;
    }


    private int getLineCountByDays() {
        int days = Days.daysBetween(
                new DateTime(conditions.get("startdate"))
                , new DateTime(conditions.get("enddate"))).getDays() + 1;

        if (days == 1) {
            return 5;
        } else if (days == 7) {
            return 10;
        } else {
            return days;
        }

    }


    private List<String> getKeysByds(String start, String end, int sort) {
        DateTime startDate = new DateTime(start);
        DateTime endDate = new DateTime(end);
        int line_show_count = Days.daysBetween(startDate, endDate).getDays() + 1;
        List<String> dateKeys = new ArrayList<>();

        for (int i = 0; i < line_show_count; i++) {
            if (sort > 0) {

                dateKeys.add(endDate.plusDays(-i).toString("yyyy-MM-dd"));
            } else {

                dateKeys.add(startDate.plusDays(i).toString("yyyy-MM-dd"));
            }

        }
        return dateKeys;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }


    private void getKeysForResultList(ReportListResult reportResult, List<String> viewColumns) {

        if (viewColumns == null || viewColumns.size() == 0) {
            return;
        }

        List<String> names = reportResult.getName();

        Map<String, String> viewCmaps = new HashMap<>();
        for (String clm : viewColumns) {
            viewCmaps.put(clm, "1");
        }

        List<Object> columnsKeys = reportResult.getColumnkey();
        List<String> nameList = new ArrayList<>();
        List<Object> clmkeys = new ArrayList<>();
        nameList.add(names.get(0));
        clmkeys.add(columnsKeys.get(0));

        for (int i = 1; i < columnsKeys.size(); i++) {

            String key = columnsKeys.get(i).toString();
            if (viewCmaps.containsKey(key)) {
                nameList.add(names.get(i));
                clmkeys.add(key);
            }
        }

        reportResult.setName(nameList);
        reportResult.setColumnkey(clmkeys);
    }

    /**
     * @param data
     * @return
     */
    private String getKeyType(Map<String, Object> data) {

        if (data.get("cid") != null) {
            return "cid";
        } else if (data.get("campaignid") != null) {
            return "campaignid";
        } else if (data.get("creativename") != null) {
            return "creativename";
        } else if (data.get("subchannel") != null) {
            return "subchannel";
        } else if (data.get("keywordname") != null) {
            return "keywordname";
        }
        return "campaignid";

    }

    /**
     * 根据模板数据 mock
     *
     * @param reportResult
     */
    private void mockResultDatas(ReportListResult reportResult, int lines) {

        List<Map<String, Object>> vals = new ArrayList<>();

        // 汇总数据
        Map totalLine = new HashMap();

        for (int i = 0; i < lines; i++) {
            // 首列数据
            Map data = new HashMap();
            this.putDatas(0, data, totalLine, false, i);// 第一个参数表示列下标
            vals.add(data);
        }

        if (!"date".equals(viewType)
                && !reportName.startsWith("roi_")
                && !reportName.equals("channelinfo_bychannel")
                && !reportName.equals("h5_channel_detail_list")
                && !reportName.startsWith("black_click_")) {
            for (int i = 1; i <= 5; i++) {
                Map data = new HashMap();
                //放入全零数据
                this.putDatas(0, data, totalLine, true, i + lines);
                vals.add(data);
            }
        }


        for (int i = 1; i < columnsKeys.size(); i++) {

            for (Map data : vals) {
                this.putDatas(i, data, totalLine, (Boolean) data.get("isall0"), i);
            }
        }


        List<Map<String, Object>> last = new ArrayList();

        if (!reportName.startsWith("roi_")) {
            last.add(totalLine);
        }

        last.addAll(vals);
        reportResult.setVal(last);

    }

    public void putDatas(int index, Map data, Map totalLine, Boolean isall0, int lineNum) {

        Object key = columnsKeys.get(index);


        if (key.equals("num_newuser")) {
            logger.debug("num_newuser");
        }

        Object datatemp = firstLine.get(key);
        data.put("isall0", isall0);

        if (!columns.get(0).equals(columnsKeys.get(0))) {
            // 填充 campainid cid channel 等
            data.put(columns.get(0), 0);
        }


        if (reportName.equals("subchannel_bychannel") && index == 1) {

            data.put("cid", -1);
        }

        if (index == 0 && orignalColums != null) {

            for (String cl : orignalColums) {
                data.put(cl, -1);
            }

        }

        if (datatemp == null) {

            if (key.toString().endsWith("_amt_pay") || reportName.equals("roi_byinterval")) {
                datatemp = RandomUtils.nextLong(0L, 229L);
            } else if (key.toString().startsWith("rate_") || key.toString().startsWith("amt_")) {
                datatemp = 0.6;
            } else {
                datatemp = 16;
            }
        }

        if (index == 0) {

            //填充 中文 key
            int keyslength = keys.size();

            String isNatural = null;

            if (keyslength <= lineNum) {
                int indexNow = (int) (Math.random() * (keyslength - 2));

                data.put(key, keys.get(indexNow));

                if ("campaigninfo_bypackagecampaign".equals(reportName)) {
                    data.put("channelname", keys.get(indexNow));
                }


            } else {

                data.put(key, keys.get(lineNum));

                if ("campaigninfo_bypackagecampaign".equals(reportName)) {
                    data.put("channelname", keys.get(lineNum));
                }
            }

            totalLine.put(key, "汇总");
            totalLine.put("isall0", false);

        } else if (isall0) {
            //全零数据
            data.put(key, 0);

        } else if (datatemp.getClass().isAssignableFrom(Long.class)) {

            Long putlong = RandomUtils.nextLong(0L, (Long) datatemp);

            if (totalLine.get(key) == null
                    || key.toString().startsWith("rate_")
                    || key.toString().startsWith("amt_")) {
                totalLine.put(key, putlong);
            } else {
                totalLine.put(key, putlong + (Long) totalLine.get(key));
            }


            data.put(key, putlong);
        } else if (datatemp.getClass().isAssignableFrom(Double.class)) {

            DecimalFormat df = new DecimalFormat("#.00");
            DecimalFormat dfOne = new DecimalFormat("#0.00");

            Double putdouble = RandomUtils.nextDouble(0, (Double) datatemp);

            Double totladata = null;

            if (totalLine.get(key) == null
                    || key.toString().startsWith("rate_")
                    || key.toString().startsWith("amt_")) {
                totladata = putdouble;
            } else {
                totladata = putdouble + Double.valueOf(totalLine.get(key).toString());
            }

            totalLine.put(key, totladata < 1 ? dfOne.format(totladata) : df.format(totladata));

            data.put(key, putdouble < 1 ? dfOne.format(putdouble) : df.format(putdouble));

        } else if (datatemp.toString().endsWith("秒")) {
            Long putlong = RandomUtils.nextLong(0L, getSecondLong(datatemp, "秒"));
            if (totalLine.get(key) == null) {
                totalLine.put(key, putlong + "秒");
            } else {
                totalLine.put(key, (putlong + getSecondLong(totalLine.get(key), "秒")) + "秒");
            }

            data.put(key, putlong + "秒");

        } else {
            data.put(key, datatemp);
        }
    }

    public long getSecondLong(Object data, String code) {
        return Long.parseLong(data.toString().replace(code, ""));
    }

    public void funnelMock(ReportListResult reportResult) {

        String[] view_column = conditions.get("viewcolumn").split(",");
        Map<String, String> clmsMap = new HashMap() {{
            put("install", "激活");
            put("reg", "注册");
            put("loggedin", "登录");
            put("order", "订单");
            put("pay", "付费");
        }};

        List<Map<String, Object>> result = new ArrayList<>();
        long base_start = 42040, base_end = 63092;
        for (int i = 0, j = view_column.length; i < j; i++) {

            Map data = new HashMap();
            long base = RandomUtils.nextLong(base_start, base_end);
            if (clmsMap.containsKey(view_column[i])) {
                data.put("name", clmsMap.get(view_column[i]));
            } else {
                data.put("name", view_column[i]);
            }
            data.put("value", base);
            result.add(data);
            base_end = base;
            base_start = RandomUtils.nextLong(0, base_start);
        }

        Long first = 0L;

        for (int i = 0, j = result.size(); i < j; i++) {
            Long value = (long) result.get(i).get("value");

            if (i == 0) {
                first = value;
                result.get(i).put("rate", 100);
                result.get(i).put("rate_remain", 0.0);
            } else {
                double rate_value = new BigDecimal(value.doubleValue() / first.doubleValue() * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                result.get(i).put("rate", rate_value);
                result.get(i).put("rate_remain", new BigDecimal(100 - rate_value).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            }
            if (i + 1 < j) {
                Long next = (long) result.get(i + 1).get("value");
                double rate_value_sec = new BigDecimal(next.doubleValue() / value.doubleValue() * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                result.get(i).put("section_rate", rate_value_sec);
            } else {
                result.get(i).put("section_rate", -1);
            }
        }

        reportResult.setVal(result);
    }

    public static void main(String[] args) {

//        List<Map> result = new ArrayList<>();
//        long base_start = 42040, base_end = 63092;
//        for (int i = 0, j = 5; i < j; i++) {
//
//            Map data = new HashMap();
//            long base = RandomUtils.nextLong(base_start, base_end);
//            System.out.println(base);
//            data.put("value", base);
//            result.add(data);
//            base_end = base;
//            base_start = RandomUtils.nextLong(0, base_start);
//        }
//
//        Long first = 0L;
//
//        for (int i = 0, j = result.size(); i < j; i++) {
//            Long value = (long) result.get(i).get("value");
//
//            if (i == 0) {
//                first = value;
//                result.get(i).put("rate", 100);
//                result.get(i).put("rate_remain", 0.0);
//            } else {
//                double rate_value = new BigDecimal(value.doubleValue() / first.doubleValue() * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
//                result.get(i).put("rate", rate_value);
//                result.get(i).put("rate_remain", new BigDecimal(100 - rate_value).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
//            }
//            if (i + 1 < j) {
//                Long next = (long) result.get(i + 1).get("value");
//                double rate_value_sec = new BigDecimal(next.doubleValue() / value.doubleValue() * 100).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
//                result.get(i).put("section_rate", rate_value_sec);
//            } else {
//                result.get(i).put("section_rate", -1);
//            }
//        }
//
//        System.out.println(result);
    }

}
