package com.reyun.report.parse;

import com.reyun.context.AppUtils;
import com.reyun.model.Campaign;
import com.reyun.model.Channel;
import com.reyun.model.ReportChartResult;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractListReportParse;
import com.reyun.repository.CampaignRepository;
import com.reyun.service.ChannelService;
import com.reyun.util.ValidateUtil;
import org.json.JSONException;

import java.text.DecimalFormat;
import java.util.*;

public class FakeTop5ListParse extends AbstractListReportParse {


    private static Map<String, Map> RATE_MAP_NAMES = new HashMap<>();
    private static Map<String, Map<Integer, int[]>> TOP5_CHART_RATE_MAP_NAMES = new HashMap<>();

    private static DecimalFormat DF_FORMAT = new DecimalFormat("#0.00");


    private CampaignRepository campaignRepository = AppUtils.getApplicationContext().getBean(CampaignRepository.class);

    private ChannelService channelService = AppUtils.getApplicationContext().getBean(ChannelService.class);

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId() {
        return "campaignid";
    }

    public ReportListResult parse(ReportListResult reportListResult, Long appid, Map<String, String> conditions, String reportName) {


        if (reportName.startsWith("black_top5_list_mtti_")) {

            int start;
            int end;

            List<String> names = reportListResult.getName();
            List<Object> colums = reportListResult.getColumnkey();

            if ("day".equals(conditions.get("mtti_all"))) {
                start = 2;
                end = 14;
            } else {
                start = 14;
                end = names.size();
            }

            List<String> nameNext = new ArrayList<>();
            List<Object> columsNext = new ArrayList<>();
            List<String> colNext = new ArrayList<>();

            nameNext.add(names.get(0));
            nameNext.add(names.get(1));
            columsNext.add(colums.get(0));
            columsNext.add(colums.get(1));
            colNext.add(colums.get(0).toString());
            colNext.add(colums.get(1).toString());


            for (int i = start; i < end; i++) {
                nameNext.add(names.get(i));
                columsNext.add(colums.get(i));
                colNext.add("rate_" + colums.get(i).toString());
            }

            reportListResult.setName(nameNext);
            reportListResult.setColumnkey(columsNext);
            reportListResult.setColumn(colNext);

        }


        if (reportName.endsWith("_campaignid")) {

            return getMapTotalData(reportListResult, "campaignid", getCanmpainNameMap(reportListResult, appid), getRateColums(reportName));

        } else if (reportName.endsWith("_cid")) {

            return getMapTotalData(reportListResult, "cid", getChannelNameMap(reportListResult), getRateColums(reportName));

        } else {
            return getMapTotalData(reportListResult, "ds", null, getRateColums(reportName));
        }

    }

    public ReportChartResult parseByChart(ReportChartResult reportListResult, Map<String, String> conditions, String reportName) {

        List<String> keys = reportListResult.getKey();
        List<String> names = reportListResult.getName();

        this.getTotalRow(reportListResult, keys.size(), names.size(), getRateIndexName(reportName), conditions, reportName);

        return reportListResult;
    }

    private void getTotalRow(ReportChartResult reportListResult, int keylength, int namelength,
                             Map<Integer, int[]> rate_index, Map<String, String> conditions, String reportName) {


        List<Object[]> datas = reportListResult.getVal();

        boolean isMttiChart = false;

        if (reportName != null && reportName.startsWith("black_top5_list_mtti_")) {

            // 截取表头数据
            List<String> names_tmp = new ArrayList<>();

            isMttiChart = true;

            int start = "day".equals(conditions.get("mtti_all")) ? 1 : 13;
            int end = start == 1 ? 13 : namelength;

            List<String> names_value = reportListResult.getName();

            names_tmp.add(names_value.get(0));

            for (int i = start; i < end; i++) {
                names_tmp.add(names_value.get(i));
            }

            namelength = names_tmp.size();

            reportListResult.setName(names_tmp);

        }

        List<Object[]> datas_new = new ArrayList<>();

        for (int i = 0; i < namelength; i++) {

            // 计算汇总数据
            Object[] row = datas.get(i);


            Object[] row_new = new Object[row.length + 1];

            Long total = 0L;

            for (int j = 0; j < keylength; j++) {

                if (isMttiChart && i> 0) {

                    total += Long.parseLong(row[j]==null?"0":row[j].toString());

                    // 计算 百分比数据
                    countMttiRate(datas, row, row_new, j);

                } else if (rate_index == null || rate_index.get(i) == null) {
                    total += Long.parseLong(row[j]==null?"0":row[j].toString());
                    row_new[j + 1] = row[j];
                } else {

                    row_new[j + 1] = row[j];
                }
            }


            if (isMttiChart && i > 0) {

                row_new[0] = countMttiRate(datas_new.get(0)[0], total);
            } else {
                row_new[0] = total;
            }

            datas_new.add(row_new);

        }

        if (!isMttiChart && rate_index!=null) {

            rate_index.forEach((key, value) -> countRate(datas_new, key, value[0], value[1]));
        }


        reportListResult.setVal(datas_new);
        reportListResult.getKey().add(0, "汇总");

    }

    private void countRate(List<Object[]> datas_new, int... index) {

        long num_install = Long.parseLong(datas_new.get(index[2])[0].toString());

        if (num_install == 0) {
            datas_new.get(index[0])[0] = new Double("0.00");
        } else {

            double valueD =  Double.parseDouble(datas_new.get(index[1])[0].toString()) * 100 / num_install;
            datas_new.get(index[0])[0] = new Double(DF_FORMAT.format(valueD));
        }

    }

    private void countMttiRate(List<Object[]> datas, Object[] row, Object[] row_new, int index) {

        long num_install = Long.parseLong(datas.get(0)[index].toString());

        if (num_install == 0) {
            row_new[index + 1] = new Double("0.00");
        } else {

            double valueD = Double.parseDouble(row[index].toString()) * 100 / num_install ;
            row_new[index + 1] = new Double(DF_FORMAT.format(valueD));
        }
    }

    private Object countMttiRate(Object num_install_data, Object total) {

        long num_install = Long.parseLong(num_install_data.toString());

        if (num_install == 0) {
            return new Double("0.00");
        } else {

            double valueD = Double.parseDouble(total.toString()) * 100 / num_install;
            return  new Double(DF_FORMAT.format(valueD)) ;
        }
    }

    private void countRateMtti(List<Object[]> datas, int... index) {

        long num_install = Long.parseLong(datas.get(index[0])[index[1]].toString());

        if (num_install == 0) {
            datas.get(index[0])[index[2]] = new Double("0.00");
        } else {

            double valueD = Double.parseDouble(datas.get(index[0])[index[2]].toString()) * 100 / num_install;
            datas.get(index[0])[index[2]] = new Double(DF_FORMAT.format(valueD));
        }

    }


    private Map<Integer, int[]> getRateIndexName(String reportName) {

        if(reportName.startsWith("black_top5_list_hijacked_pack")){
            return null;
        }

        Map<Integer, int[]> data = TOP5_CHART_RATE_MAP_NAMES.get(reportName);

        if (data != null) {
            return data;
        }

        /**
         * 例如：black_top5_list_click 表头 "渠道", "激活总数", "关联作弊激活数", "关联作弊激活数占比", "渠道刷量", "app刷量", "大点击",
         * report api 处理后的表头为 "激活总数", "关联作弊激活数", "关联作弊激活数占比", "渠道刷量", "app刷量", "大点击",
         * 那么  关联作弊激活数占比 = 关联作弊激活数*100/激活总数
         * putTop5RateIndex 方法中 index:  2 对应 关联作弊激活数 1 对应 激活总数 0 对应 关联作弊激活数占比（要计算的百分比）
         *
         */
        putTop5RateIndex("black_top5_list_click", 2, 1, 0);
        putTop5RateIndex("black_top5_list_device", 4, 1, 0);
        putTop5RateIndex("black_top5_list_corpse", 2, 1, 0);
        putTop5RateIndex("black_top5_list_hijack", 2, 1, 0);
        putTop5RateIndex("black_top5_list_hijackpack", 2, 1, 0);
//        putTop5RateIndex("black_top5_list_mtti", 2, 0, 1);
        putTop5RateIndex("black_top5_list_analysis", 2, 1, 0);
        putTop5RateIndex("black_top5_list_analysis", 4, 3, 0);

        return TOP5_CHART_RATE_MAP_NAMES.get(reportName);
    }

    private void putTop5RateIndex(String reportName, int... index) {

        Map<Integer, int[]> data = TOP5_CHART_RATE_MAP_NAMES.get(reportName);

        if (data == null) {
            data = new HashMap();
            data.put(new Integer(index[0]), new int[]{index[1], index[2]});
            TOP5_CHART_RATE_MAP_NAMES.put(reportName, data);
            TOP5_CHART_RATE_MAP_NAMES.put(reportName + "_ds", data);
            TOP5_CHART_RATE_MAP_NAMES.put(reportName + "_cid", data);
            TOP5_CHART_RATE_MAP_NAMES.put(reportName + "_campaignid", data);
        } else {
            TOP5_CHART_RATE_MAP_NAMES.get(reportName + "_ds").put(new Integer(index[0]), new int[]{index[1], index[2]});
            TOP5_CHART_RATE_MAP_NAMES.get(reportName + "_cid").put(new Integer(index[0]), new int[]{index[1], index[2]});
            TOP5_CHART_RATE_MAP_NAMES.get(reportName + "_campaignid").put(new Integer(index[0]), new int[]{index[1], index[2]});
        }

    }

    private Map<String, String> getRateColums(String reportName) {

        Map<String, String> rateColums = RATE_MAP_NAMES.get(reportName);

        if (rateColums != null) {
            return rateColums;
        }

        if (reportName.startsWith("black_top5_list_click")) {
            fillRateColums(reportName, "rate_install_union", "num_install_union", "num_install");
        } else if (reportName.startsWith("black_top5_list_device")) {
            fillRateColums(reportName, "rate_fake_intall", "fake_intall_all", "num_install");
        } else if (reportName.startsWith("black_top5_list_corpse")) {
            fillRateColums(reportName, "rate_corpse", "num_corpse", "num_install");
        } else if (reportName.startsWith("black_top5_list_hijack")) {
            fillRateColums(reportName, "rate_hijack", "num_click", "num_install");
        } else if (reportName.startsWith("black_top5_list_mtti")) {
            fillRateColums(reportName, null);
        }

        return RATE_MAP_NAMES.get(reportName);
    }

    private Map<String, String> fillRateColums(String reportName, String... keys) {

        Map<String, String> rateColums = RATE_MAP_NAMES.get(reportName);

        if (rateColums == null) {
            rateColums = new HashMap<>();

            RATE_MAP_NAMES.put(reportName, rateColums);
        }

        if (keys != null) {
            rateColums.put(keys[0], keys[1] + "," + keys[2]);
        }

        return rateColums;
    }

    private Map<String, String> getCanmpainNameMap(ReportListResult reportResult, Long app) {
        List<Map<String, Object>> valList = reportResult.getVal();
        List<Map<String, Object>> res = new LinkedList<>();

        Map<String, String> nameMap = new HashMap<>();
        List<String> idList = new ArrayList<>();
        if (ValidateUtil.isValid(valList)) {
            nameMap.put("_default_", "自然量");
            for (Map<String, Object> map : valList) {
                if ((map.get("campaignid") instanceof String)) {
                    idList.add((String) map.get("campaignid"));
                }
            }
            List<Campaign> campaigns = null;
            if (idList.size() != 0) {
                campaigns = campaignRepository.findByCampaignid(idList, app);
            }
            if (ValidateUtil.isValid(campaigns)) {
                for (Campaign cm : campaigns) {
                    nameMap.put(cm.getCampaignid(), cm.getName());
                }
            }
        }

        return nameMap;
    }

    private Map<String, String> getChannelNameMap(ReportListResult reportResult) {
        List<Map<String, Object>> valList = reportResult.getVal();
        Map<String, String> nameMap = new HashMap<>();
        List<Long> idList = new ArrayList<>();
        if (ValidateUtil.isValid(valList)) {
            nameMap.put("-1", "自然量");
            for (Map<String, Object> map : valList) {
                idList.add((Long) map.get("cid"));
            }
            List<Channel> channelList = channelService.listChannelByIds(idList);
            if (ValidateUtil.isValid(channelList)) {
                for (Channel ch : channelList) {
                    nameMap.put(ch.getId().toString(), ch.getName());
                }
            }
        }
        return nameMap;
    }

    public static void main(String[] args) {

        System.out.println(247968*100*1.0/1925571);

        Double d = Double.valueOf(247968*100);

        Long f = Long.valueOf(1925571);

        DecimalFormat df = new DecimalFormat("#0.00");
         String str = df.format(d/f);

        System.out.println(str);
    }


}
