package com.reyun.report.parse;

import com.google.common.base.Function;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.model.*;
import com.reyun.report.AbstractListReportParse;
import com.reyun.util.DateUtil;
import com.reyun.util.ViewColumnDependUtil;
import org.json.JSONException;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * Created by sunhao on 17/7/6.
 * desc:按照推广活动和日期group出的结果处理,无需要汇总。
 */
public class ListCampaignAndDateParse extends AbstractListReportParse {

    private final static String KEY_CAMPAIGN_ID = "campaignid";
    private final static String KEY_CHANNEL_NAME = "channelname";
    private final static String KEY_GROUP_NAME = "groupname";
    private final static Long DEFAULT_ID = -1L;
    private final static String DEFAULT_CHANNEL_NAME = "自然量";
    private final static String DEFAULT_CAMPAIGN_ID = "_default_";
    private final static String DEFAULT_CAMPAIGN_NAME = "自然量";
    private final static String DEFAULT_GROUP_NAME = "无";

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {

        final List<Map<String, Object>> valueList = reportResult.getVal();

        //1,获取所有的推广活动,和对应的活动组已经渠道对应关系
        Map<String, Campaign> campaignMap = new HashMap<>();
        Map<Long, String> channelIdNameMap = new HashMap<>();
        Map<Long, String> campaignIdGroupNameMap = new HashMap<>();

        //获取查出的所有推广活动
        Set<String> campaignSet = new HashSet<>();
        for (Map valMap : valueList) {
            if (valMap.containsKey(KEY_CAMPAIGN_ID) && valMap.get(KEY_CAMPAIGN_ID) != null) {
                campaignSet.add(valMap.get(KEY_CAMPAIGN_ID).toString());
            }
        }

        if (!CollectionUtils.isEmpty(campaignSet)) {
            //找出推广活动的名字
            // List<Campaign> campaignList = super.campaignService.findAllByCampaignIdList(new ArrayList<>(campaignSet));

            List<Campaign> campaignList = new ArrayList<>();
            if (isSearchText || !CollectionUtils.isEmpty(new ArrayList<>(campaignSet)) || "campaigninfo_bypackagecampaign".equals(reportName)) {   // 分包推广活动详情单独处理
                campaignList = super.campaignService.findByCampaignidList(new ArrayList<>(campaignSet), app);
            } else {
                campaignList = super.authService.getAuthCampaignListByAccount(account, app);
            }


            //找出推广活动渠道
            List<Channel> channels = channelService.findCoopAndOwnChannel(account, null);

            //加入自然量
            if (super.authService.isNature(account, app)) {

                Campaign campaign = new Campaign();
                campaign.setId(DEFAULT_ID);
                campaign.setCampaignid(DEFAULT_CAMPAIGN_ID);
                campaign.setName(DEFAULT_CAMPAIGN_NAME);
                campaign.setChannel(DEFAULT_ID);

                campaignList.add(campaign);

                channels.add(new Channel(DEFAULT_ID, DEFAULT_CHANNEL_NAME));
            }

            //构建活动MAP
            campaignMap = Maps.uniqueIndex(campaignList, new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });

            //构建渠道MAP
            for (Channel ch : channels) {
                channelIdNameMap.put(ch.getId(), ch.getName());
            }

            //找出推广活动组和推广活动关系
            if (!CollectionUtils.isEmpty(campaignList)) {

                //推广活动MAP,KEY-活动组ID, VALUE - 活动组信息
                List<CampaignGroup> groups = campaignGroupService.listCamgroupByAccountAndApp(app, account);
                Map<Long, CampaignGroup> campaignGroupIdMap = Maps.uniqueIndex(groups, new Function<CampaignGroup, Long>() {
                    @Override
                    public Long apply(CampaignGroup campaignGroup) {
                        return campaignGroup.getId();
                    }
                });

                //对应关系List 推广活动和活动组的对应关系列表
                List<CamCamgroupMap> campaignGroupList = camCamGroupMapRepository.listAllByCampaignIds(Lists.transform(campaignList, new Function<Campaign, Long>() {
                    @Override
                    public Long apply(Campaign campaign) {
                        return campaign.getId();
                    }
                }));

                //MAP KEY - 推广活动ID    VALUE - 活动组名称
                if (!CollectionUtils.isEmpty(campaignGroupIdMap)) {
                    for (CamCamgroupMap camCamgroupMap : campaignGroupList) {

                        if (campaignGroupIdMap.containsKey(camCamgroupMap.getCampGroup())) {
                            campaignIdGroupNameMap.put(camCamgroupMap.getCampaign(), campaignGroupIdMap.get(camCamgroupMap.getCampGroup()).getName());
                        }
                    }
                }
            }
        }


        //2,替换所有推广活动中文名
        for (Map<String, Object> valMap : valueList) {

            String campaignId = valMap.get(KEY_CAMPAIGN_ID).toString();

            Campaign campaign = campaignMap.get(campaignId);

            //设置默认活动组名
            valMap.put(KEY_GROUP_NAME, DEFAULT_GROUP_NAME);

            if (null != campaign) {

                //渠道名
                String channelName = channelIdNameMap.get(campaign.getChannel());

                if (!StringUtils.isEmpty(channelName)) {
                    valMap.put(KEY_CHANNEL_NAME, channelName);
                } else {
                    valMap.put(KEY_CHANNEL_NAME, campaign.getChannel());
                }

                //活动组名
                String campaignGroupName = campaignIdGroupNameMap.get(campaign.getId());

                if (!StringUtils.isEmpty(campaignGroupName)) {
                    valMap.put(KEY_GROUP_NAME, campaignGroupName);
                }

                //活动名
                valMap.put(KEY_CAMPAIGN_ID, campaign.getName());

                //计算指标点
                //todo 此处未做通用处理（报表sql已计算指标）
                boolean isall0 = true;
                for (String viewColumn : viewColumns) {
                    if (viewColumn.toLowerCase().equals("duration")) {
                        valMap.put(viewColumn, DateUtil.formatCaseTime(Long.valueOf(valMap.get(viewColumn)==null?"0":valMap.get(viewColumn).toString())));
                    }
                    Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewColumn);
                    if (linkfields != null && linkfields.size() > 0) {
                        Object[] objects = linkfields.toArray();
                        String[] linkfield = Arrays.copyOf(objects, objects.length, String[].class);
                        final Object f1 = valMap.get(linkfield[0]);

                        if (f1 == null){
                            continue;
                        }

                        if (f1.getClass().isAssignableFrom(Long.class)) {
                            if (viewColumn.startsWith("amt_ltv")) {
                                valMap.put(viewColumn, divideAmt(((Long) f1) / 100, (Long) valMap.get(linkfield[1])));
                            } else if(viewColumn.toLowerCase().startsWith("avg_time")){
                                valMap.put(viewColumn, divideAvgTime(Long.valueOf(String.valueOf(f1.toString())), Long.valueOf(String.valueOf(valMap.get(linkfield[1])))));
                            }else{

                                Object dataVal = getNewRateInstallData(viewColumn, valMap, valMap.get(linkfield[1]));

                                if (dataVal == null) {
                                    dataVal = "1";
                                }

//                            valuemap.put(viewColumn, divide((Long) f1, (Long) valuemap.get(linkfield[1])));
                                valMap.put(viewColumn, divide((Long) f1, Long.valueOf(dataVal.toString())));
                            }
                        } else if (f1.getClass().isAssignableFrom(Double.class)) {
                            if (viewColumn.startsWith("amt_ltv")) {
                                valMap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valMap.get(linkfield[1])))));
                            } else {

                                Object dataVal = getNewRateInstallData(viewColumn, valMap, valMap.get(linkfield[1]));

                                if (dataVal == null) {
                                    dataVal = "1";
                                }
                                valMap.put(viewColumn, divide((Double) f1, Double.valueOf(dataVal.toString())));
                            }
                        }
                    }
                    if (!"campaignid".equals(viewColumn) && isall0) {
                        Object tmpval = valMap.get(viewColumn);
                        if (tmpval != null && (tmpval instanceof Double && Double.valueOf(String.valueOf(tmpval)) > 0) ||(tmpval instanceof Long && Long.valueOf(String.valueOf(tmpval)) > 0) || (tmpval instanceof String && (tmpval.toString().contains("秒")||tmpval.toString().contains("分")||tmpval.toString().contains("时")||tmpval.toString().contains("天"))&&!tmpval.equals("0秒"))) {
                            isall0 = false;
                        }
                    }
                }
                valMap.put("isall0", isall0);
//                for (String key : valMap.keySet()) {
//
//                    Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(key);
//
//                    if (linkfields != null && linkfields.size() > 0) {
//                        String[] linkfield = new String[linkfields.size()];
//                        linkfield = linkfields.toArray(linkfield);
//                        final Object f1 = valMap.get(linkfield[0]);
//
//                        Object linkfieldVal = getNewRateInstallData(key, valMap, valMap.get(linkfield[1]));
//
//                        if (linkfieldVal == null) {
//                            linkfieldVal = "0";
//                        }
//
//                        if (f1.getClass().isAssignableFrom(Long.class)) {
//                           Double v = multiply((Long) f1, Long.parseLong(linkfieldVal.toString()));
//                           valMap.put(key, v);
//                        }else{
//                            Double v = multiply((Double) f1, Double.valueOf(linkfieldVal.toString()));
//                            valMap.put(key, v);
//                        }
//
//                    }
//
//                }

            }
        }

        //3,return
        reportResult.setVal(valueList);

        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }
}
