package com.reyun.report.parse;

import com.google.common.base.Function;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ImmutableListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimaps;
import com.reyun.model.CamCamgroupMap;
import com.reyun.model.Campaign;
import com.reyun.model.CampaignGroup;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractListReportParse;
import com.reyun.util.ViewColumnDependUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by nolan on 18/01/2017.
 * description:
 */
public class ListCampaignGroupParse
        extends AbstractListReportParse {
    Logger logger = LoggerFactory.getLogger(ListCampaignGroupParse.class);

    /**
     * 激活率=激活设备数/排重点击
     * 注册率=注册设备数/激活设备数
     * 异常点击率=异常点击设备数/点击总数
     * 异常激活率=异常激活设备数/激活设备数
     * n日留存率=sum(激活设备数n日留存设备数)*100.0/sum(激活设备数)
     * ltvn=sum(n日LTV值)/sum(激活设备数)
     */

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {
        List<CamCamgroupMap> querylist = Lists.newArrayList();
        ImmutableListMultimap<Long, CamCamgroupMap> dicCampaignGroupByCampId = ImmutableListMultimap.of();
        Map<String, Campaign> dicCampaignByCampaignid = Maps.newHashMap();

        List<CampaignGroup> itemlist = campaignGroupService.listCamgroupByAccountAndApp(app, account);
        Map<Long, CampaignGroup> dicCampaignGroupByGroupId = Maps.uniqueIndex(itemlist, new Function<CampaignGroup, Long>() {
            @Override
            public Long apply(CampaignGroup campaignGroup) {
                return campaignGroup.getId();
            }
        });
        if (dicCampaignGroupByGroupId.size() > 0) {
            querylist = this.camCamGroupMapRepository.listCamCampMapByIds(dicCampaignGroupByGroupId.keySet());
        }
        if (querylist.size() > 0) {
            dicCampaignGroupByCampId = Multimaps.index(querylist, new Function<CamCamgroupMap, Long>() {
                @Override
                public Long apply(CamCamgroupMap camCamgroupMap) {
                    return camCamgroupMap.getCampaign();
                }
            });
        }
        if (dicCampaignGroupByCampId.size() > 0) {
            dicCampaignByCampaignid = Maps.uniqueIndex(this.campaignService.findAllById(dicCampaignGroupByCampId.keySet()), new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });
        }

        //1. 按活动组分类推广活动
        final List<Map<String, Object>> val = reportResult.getVal();
        ListMultimap<Long, Map> groupMultiMap = ArrayListMultimap.create();
        for (int i = 0, n = val != null ? val.size() : 0; i < n; i++) {
            Map rowItem = val.get(i);
            Campaign campaign = dicCampaignByCampaignid.get(rowItem.get("campaignid"));
            if (campaign == null) {
                continue;
            }

            List<CamCamgroupMap> refCamCamgroup = dicCampaignGroupByCampId.get(campaign.getId());
            if (refCamCamgroup == null || refCamCamgroup.size() == 0) {
                continue;
            }

            for (CamCamgroupMap camgroupMap : refCamCamgroup) {
                //关键字查看报表
                if (isSearchText && !searchIds.contains(camgroupMap.getCampGroup().toString())) {
                    continue;
                }
                groupMultiMap.put(camgroupMap.getCampGroup(), rowItem);
            }
        }

        //2. 按活动组统计推广活动汇总值
        Iterator<Long> iter = groupMultiMap.keySet().iterator();
        List<Map<String, Object>> translist = Lists.newArrayList();
        while (iter.hasNext()) {
            Long groupid = iter.next();
            List<Map> camplist = groupMultiMap.get(groupid);


            ListMultimap<String, Object> attributemaplist = ArrayListMultimap.create();
            for (Map<String, Object> row : camplist) {
                for (Map.Entry<String, Object> entry : row.entrySet()) {
                    attributemaplist.put(entry.getKey(), entry.getValue());
                }
            }

            Map valuemap = Maps.newHashMap();
            final CampaignGroup campaignGroup = dicCampaignGroupByGroupId.get(groupid);
            valuemap.put("campaignid", campaignGroup != null ? campaignGroup.getName() : "");
            for (String viewColumn : viewColumns) {
                final String fieldcode = viewColumn;

                List<Object> listbyAttribute = attributemaplist.get(fieldcode);
                if (listbyAttribute != null && listbyAttribute.size() > 0) {
                    Object[] obj = listbyAttribute.toArray();
                    if (obj[0] instanceof Long) {
                        Long[] f1 = Arrays.copyOf(obj, obj.length, Long[].class);
                        valuemap.put(fieldcode, plus(f1));
                    } else if (obj[0] instanceof Double) {
                        Double[] f1 = Arrays.copyOf(obj, obj.length, Double[].class);
                        valuemap.put(fieldcode, plus(f1));
                    }
                }
            }

            boolean isall0 = true;
            for (String viewColumn : viewColumns) {
                Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewColumn);
                if (linkfields != null && linkfields.size() > 0 && !viewColumn.startsWith("avg_time")) {
                    Object[] objects = linkfields.toArray();
                    String[] linkfield = Arrays.copyOf(objects, objects.length, String[].class);
                    final Object f1 = valuemap.get(linkfield[0]);

                    if (f1.getClass().isAssignableFrom(Long.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Long) f1) / 100, (Long) valuemap.get(linkfield[1])));
                        } else if (viewColumn.equals("amt_use_freq")) {
                            valuemap.put(viewColumn, divideAmt(((Long) f1) / 100.0, Long.valueOf(String.valueOf(valuemap.get(linkfield[1]))) * 1.0));
                        } else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }

//                            valuemap.put(viewColumn, divide((Long) f1, (Long) valuemap.get(linkfield[1])));
                            valuemap.put(viewColumn, divide((Long) f1, Long.valueOf(dataVal.toString())));
                        }
                    } else if (f1.getClass().isAssignableFrom(Double.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        } else if (viewColumn.startsWith("amt_arp") && viewColumn.endsWith("_new")) {
                            valuemap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        } else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }
                            valuemap.put(viewColumn, divide((Double) f1, Double.valueOf(dataVal.toString())));
                        }
                    }
                }
                if (!"campaignid".equals(viewColumn) && isall0) {
                    Object tmpval = valuemap.get(viewColumn);
                    if (tmpval != null && Double.valueOf(String.valueOf(tmpval)) > 0) {
                        isall0 = false;
                    }
                }
            }
            valuemap.put("isall0", isall0);

            translist.add(valuemap);
        }

        //添加全0数据 add by sunhao
        if (!isSearchText) {
            for (Long groupId : dicCampaignGroupByGroupId.keySet()) {
                if (groupMultiMap.containsKey(groupId)) {
                    continue;
                }

                Map<String, Object> map = getAllZeroValue(viewColumns);
                map.put("campaignid", dicCampaignGroupByGroupId.get(groupId).getName());
                translist.add(map);
            }
        } else {
            for (String groupId : searchIds) {
                if (groupMultiMap.containsKey(Long.parseLong(groupId))) {
                    continue;
                }

                Map<String, Object> map = getAllZeroValue(viewColumns);
                map.put("campaignid", dicCampaignGroupByGroupId.get(Long.parseLong(groupId)).getName());
                map.put("isall0", true);
                translist.add(map);
            }
        }

        reportResult.getName().remove(0);
        reportResult.getName().add(0, "推广活动组");
        reportResult.setVal(translist);

        //计算汇总行
        List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);

        reportResult.setVal(super.transCampaignGroupName(newResult, app, account));
        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) {
        List<CamCamgroupMap> querylist = Lists.newArrayList();
        ImmutableListMultimap<Long, CamCamgroupMap> dicCampaignGroupByCampId = ImmutableListMultimap.of();
        Map<String, Campaign> dicCampaignByCampaignid = Maps.newHashMap();

        Map<String, String> webQueryCgids = null;

        if (conditions.containsKey("cgroupid") && conditions.get("cgroupid") != null) {
            //搜索的活动组
            webQueryCgids = Arrays.asList(conditions.get("cgroupid").split(",")).stream().collect(Collectors.toMap(v -> v, v -> ""));
        }

        List<CampaignGroup> itemlist = new ArrayList<>();
        List<CampaignGroup> item = campaignGroupService.listCamgroupByAccountAndApp(app, account);

        if (webQueryCgids != null) {
            Map<String, String> finalWebQueryCgids = webQueryCgids;
            item = item.stream().filter(v -> finalWebQueryCgids.containsKey(v.getId().toString())).collect(Collectors.toList());
        }

        if (conditions.containsKey("subscribes")) {
            if ("SUBSCRIBE".equals(conditions.get("subscribes"))) {
                for (CampaignGroup campaignGroup : item) {
                    if (campaignGroup.getType() != null && campaignGroup.getType() == 1) {
                        itemlist.add(campaignGroup);
                    }
                }
            } else {
                for (CampaignGroup campaignGroup : item) {
                    if (campaignGroup.getType() == null || campaignGroup.getType() != 1) {
                        itemlist.add(campaignGroup);
                    }
                }
            }
        } else {
            itemlist = item;
        }
        Map<Long, CampaignGroup> dicCampaignGroupByGroupId = Maps.uniqueIndex(itemlist, new Function<CampaignGroup, Long>() {
            @Override
            public Long apply(CampaignGroup campaignGroup) {
                return campaignGroup.getId();
            }
        });
        if (dicCampaignGroupByGroupId.size() > 0) {
            querylist = this.camCamGroupMapRepository.listCamCampMapByIds(dicCampaignGroupByGroupId.keySet());
        }
        if (querylist.size() > 0) {
            dicCampaignGroupByCampId = Multimaps.index(querylist, new Function<CamCamgroupMap, Long>() {
                @Override
                public Long apply(CamCamgroupMap camCamgroupMap) {
                    return camCamgroupMap.getCampaign();
                }
            });
        }
        if (dicCampaignGroupByCampId.size() > 0) {
            dicCampaignByCampaignid = Maps.uniqueIndex(this.campaignService.findAllById(dicCampaignGroupByCampId.keySet()), new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });
        }

        //1. 按活动组分类推广活动
        final List<Map<String, Object>> val = reportResult.getVal();
        ListMultimap<Long, Map> groupMultiMap = ArrayListMultimap.create();
        for (int i = 0, n = val != null ? val.size() : 0; i < n; i++) {
            Map rowItem = val.get(i);
            Campaign campaign = dicCampaignByCampaignid.get(rowItem.get("campaignid"));
            if (campaign == null) {
                continue;
            }

            List<CamCamgroupMap> refCamCamgroup = dicCampaignGroupByCampId.get(campaign.getId());
            if (refCamCamgroup == null || refCamCamgroup.size() == 0) {
                continue;
            }

            for (CamCamgroupMap camgroupMap : refCamCamgroup) {
                //关键字查看报表
                if (isSearchText && !searchIds.contains(camgroupMap.getCampGroup().toString())) {
                    continue;
                }
                groupMultiMap.put(camgroupMap.getCampGroup(), rowItem);
            }
        }

        //2. 按活动组统计推广活动汇总值
        Iterator<Long> iter = groupMultiMap.keySet().iterator();
        List<Map<String, Object>> translist = Lists.newArrayList();
        while (iter.hasNext()) {
            Long groupid = iter.next();
            List<Map> camplist = groupMultiMap.get(groupid);


            ListMultimap<String, Object> attributemaplist = ArrayListMultimap.create();
            for (Map<String, Object> row : camplist) {
                for (Map.Entry<String, Object> entry : row.entrySet()) {
                    attributemaplist.put(entry.getKey(), entry.getValue());
                }
            }

            Map valuemap = Maps.newHashMap();
            final CampaignGroup campaignGroup = dicCampaignGroupByGroupId.get(groupid);
            valuemap.put("campaignid", campaignGroup != null ? campaignGroup.getName() : "");
            for (String viewColumn : viewColumns) {
                final String fieldcode = viewColumn;

                List<Object> listbyAttribute = attributemaplist.get(fieldcode);
                if (listbyAttribute != null && listbyAttribute.size() > 0) {
                    Object[] obj = listbyAttribute.toArray();
                    if (obj[0] instanceof Long) {
                        Long[] f1 = Arrays.copyOf(obj, obj.length, Long[].class);
                        valuemap.put(fieldcode, plus(f1));
                    } else if (obj[0] instanceof Double) {
                        Double[] f1 = Arrays.copyOf(obj, obj.length, Double[].class);
                        valuemap.put(fieldcode, plus(f1));
                    }
                }
            }

            boolean isall0 = true;
            for (String viewColumn : viewColumns) {
                Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewColumn);
                if (linkfields != null && linkfields.size() > 0 && !viewColumn.startsWith("avg_time")) {
                    Object[] objects = linkfields.toArray();
                    String[] linkfield = Arrays.copyOf(objects, objects.length, String[].class);
                    final Object f1 = valuemap.get(linkfield[0]);

                    if (f1.getClass().isAssignableFrom(Long.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Long) f1) / 100, (Long) valuemap.get(linkfield[1])));
                        } else if (viewColumn.equals("amt_use_freq")) {
                            valuemap.put(viewColumn, divideAmt(((Long) f1) / 100.0, Long.valueOf(String.valueOf(valuemap.get(linkfield[1]))) * 1.0));
                        } else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }

//                            valuemap.put(viewColumn, divide((Long) f1, (Long) valuemap.get(linkfield[1])));
                            valuemap.put(viewColumn, divide((Long) f1, Long.valueOf(dataVal.toString())));
                        }
                    } else if (f1.getClass().isAssignableFrom(Double.class)) {
                        if (viewColumn.startsWith("amt_ltv")) {
                            valuemap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        } else if (viewColumn.startsWith("amt_arp") && viewColumn.endsWith("_new")) {
                            valuemap.put(viewColumn, divideAmt(((Double) f1) / 100, Double.valueOf(String.valueOf(valuemap.get(linkfield[1])))));
                        } else {

                            Object dataVal = getNewRateInstallData(viewColumn, valuemap, valuemap.get(linkfield[1]));

                            if (dataVal == null) {
                                dataVal = "1";
                            }
                            valuemap.put(viewColumn, divide((Double) f1, Double.valueOf(dataVal.toString())));
                        }
                    }
                }
                if (!"campaignid".equals(viewColumn) && isall0) {
                    Object tmpval = valuemap.get(viewColumn);
                    if (tmpval != null && Double.valueOf(String.valueOf(tmpval)) > 0) {
                        isall0 = false;
                    }
                }
            }
            valuemap.put("isall0", isall0);

            translist.add(valuemap);
        }

        //添加全0数据 add by sunhao
        if (!isSearchText) {
            for (Long groupId : dicCampaignGroupByGroupId.keySet()) {
                if (groupMultiMap.containsKey(groupId)) {
                    continue;
                }

                Map<String, Object> map = getAllZeroValue(viewColumns);
                map.put("campaignid", dicCampaignGroupByGroupId.get(groupId).getName());
                translist.add(map);
            }
        } else {
            for (String groupId : searchIds) {
                if (groupMultiMap.containsKey(Long.parseLong(groupId))) {
                    continue;
                }

                Map<String, Object> map = getAllZeroValue(viewColumns);
                map.put("campaignid", dicCampaignGroupByGroupId.get(Long.parseLong(groupId)).getName());
                map.put("isall0", true);
                translist.add(map);
            }
        }

        reportResult.getName().remove(0);
        reportResult.getName().add(0, "推广活动组");
        reportResult.setVal(translist);

        //计算汇总行
        List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);

        reportResult.setVal(super.transCampaignGroupName(newResult, app, account));
        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId() {
        return "campaignid";
    }
}