package com.reyun.report.parse;

import com.google.common.base.Function;
import com.google.common.collect.Maps;
import com.reyun.model.*;
import com.reyun.report.AbstractListReportParse;
import com.reyun.util.ValidateUtil;
import org.json.JSONException;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ListCampaignWxParse extends AbstractListReportParse {

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {

        if (!StringUtils.isEmpty(viewColumns) && !CollectionUtils.isEmpty(reportResult.getColumn())) {
            viewColumns = reportResult.getColumn();
        }

        final List<Map<String,Object>> valList = reportResult.getVal();
        Map<String,Map<String,Object>> valMap = Maps.newHashMap();
        if(valList!=null){
            valMap = Maps.uniqueIndex(valList, new Function<Map<String,Object>, String>() {
                @Override
                public String apply(Map<String,Object> map) {
                    return map.get("campaignid").toString()+"";
                }
            });
        }

        List<Campaign> campaignList = new ArrayList<>();
        if (isSearchText || !CollectionUtils.isEmpty(searchIds)) {
            campaignList = super.campaignService.findByCampaignidList(searchIds, app);
        } else {
            campaignList = super.authService.getAuthCampaignListByAccount(account, app);
        }

        if (super.authService.isNature(account, app)) {
            if(!isSearchText || searchIds.contains("_default_")){
                Campaign cam = new Campaign("_default_","自然量"+"");
                cam.setChannel(-1L);
                campaignList.add(cam);
            }
        }

        List<Map<String, Object>> values = new ArrayList<>();

        if(ValidateUtil.isValid(campaignList)){

            for (Campaign cam : campaignList) {

                if (!valMap.containsKey(cam.getCampaignid())) {
                    Map<String, Object> tmpResultValue = super.getAllZeroValue(viewColumns);
                    tmpResultValue.put("campaignid"+"", cam.getCampaignid());
                    values.add(tmpResultValue);
                } else {
                    values.add(valMap.get(cam.getCampaignid()));
                }
            }
        }

        reportResult.setVal(transCampaignName(values, app, account));

        List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);

        reportResult.setVal(newResult);

        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId() {
        return "campaignid";
    }
}

