package com.reyun.report.parse;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Splitter;
import com.google.common.collect.*;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractListReportParse;
import com.reyun.util.ViewColumnDependUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author liyin
 * @description
 * @date
 */
public class ListChannelByChannelParse extends AbstractListReportParse {
    private static final Logger logger = LoggerFactory.getLogger(ListChannelByChannelParse.class);
    private String reprtName;

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {
        reprtName = reportName;
        reportResult.setVal(super.transChannelName(reportResult.getVal(), app, account));
        List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);
        reportResult.setVal(newResult);

        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId() {
        return "cid";
    }

    @Override
    protected String firstShowRowNeedReplaceId() {
        return "cid";
    }

    @Override
    protected List<Map<String, Object>> calFirstTotalRow(ReportListResult reportResult, List<String> viewColumns) {
        List<Map<String, Object>> sourceResult = reportResult.getVal();
        Map<String, Object> totalRowMap = Maps.newHashMap();

        if (sourceResult == null || sourceResult.size() == 0) {
            for (String fieldcode : viewColumns) {
                Object tmp = null;
                if (isRateFields(fieldcode)) {
                    tmp = 0.0;
                } else {
                    tmp = 0l;
                }
                totalRowMap.put(fieldcode, tmp);
            }
            totalRowMap.put("isall0", false);
            totalRowMap.put("istotal", true);
            totalRowMap.put(firstTotalRowNeedReplaceId(), "汇总");

            return ImmutableList.of(totalRowMap);
        }
        //分类汇总列数据
        ListMultimap<String, Object> attributeMapList = ArrayListMultimap.create();
        for (int i = 0, n = sourceResult.size(); i < n; i++) {
            Map<String, Object> sourceItem = sourceResult.get(i);

            for (String fieldcode : viewColumns) {
                Object v = sourceItem.get(fieldcode);
                boolean isRateFields = isRateFields(fieldcode);

                if (v == null) {
                    if (isRateFields) {
                        v = 0.0;
                    } else {
                        v = 0l;
                    }
                } else {
                    Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(fieldcode);
                    if (linkfields != null && linkfields.size() > 0) {
                        String[] linkfield = new String[linkfields.size()];
                        linkfield = linkfields.toArray(linkfield);
                        final Object f1 = sourceItem.get(linkfield[0]);
                        if (f1.getClass().isAssignableFrom(Long.class)) {

                            Object linkfieldVal = getNewRateInstallData(fieldcode, sourceItem, sourceItem.get(linkfield[1]));

                            if (linkfieldVal == null) {
                                linkfieldVal = "0";
                            }

                            v = multiply((Long) f1, Long.parseLong(linkfieldVal.toString()));

                        } else if (f1.getClass().isAssignableFrom(Double.class)) {
                            //这个判空n重新赋值，针对修改的retention在report中的计算方法，有些报表以及demo数据，没有指定的属性num_d_install的处理，替换成了正常的激活数
//                            Object ob = sourceItem.get(linkfield[1]);
                            Object ob = getNewRateInstallData(fieldcode, sourceItem, sourceItem.get(linkfield[1]));

                            if (ob == null && fieldcode.contains("retention")) {
                                ob = sourceItem.get("num_install");
                            }
                            v = multiply((Double) f1, Double.valueOf(String.valueOf(ob)));
                        }
                    }
                }
                attributeMapList.put(fieldcode, v);

            }

        }

        boolean isall0 = true;
        //计算汇总行
        for (String fieldcode : viewColumns) {
            List<Object> listbyAttribute = attributeMapList.get(fieldcode);
            if (listbyAttribute == null || listbyAttribute.size() == 0) {
                continue;
            }

            Object calVal = null;
            Object[] obj = listbyAttribute.toArray();

            int line_number = 0;
            try {
                if (obj[0] instanceof Long) {
                    Long[] f2 = new Long[obj.length];
                    for (int i = 0; i < obj.length; i++) {

                        line_number = i;

                        f2[i] = Long.valueOf(String.valueOf(obj[i]));
                    }
                    calVal = plus(f2);
                } else if (obj[0] instanceof Double) {
                    Double[] f2 = new Double[obj.length];
                    for (int i = 0; i < obj.length; i++) {
                        line_number = i;
                        f2[i] = Double.valueOf(String.valueOf(obj[i]));
                    }
                    calVal = plus(f2);
                }
            } catch (ArrayStoreException e) {
                String valueAsString = "";
                try {
                    valueAsString = new ObjectMapper().writerWithDefaultPrettyPrinter().writeValueAsString(obj);
                } catch (JsonProcessingException e1) {
                    logger.error("解析汇总数据错误", valueAsString, e1);
                }
                logger.error("汇总数据复制数据错误", valueAsString, e);
            } catch (Exception e) {
                logger.error("解析汇总数数据时发生错误, 第{}行, colum:{} ", line_number, fieldcode, e);
            }
            totalRowMap.put(fieldcode, calVal);

            if (!firstTotalRowNeedReplaceId().equals(fieldcode) && isall0) {
                if (calVal != null && Double.valueOf(String.valueOf(calVal)) > 0) {
                    isall0 = false;
                }
            }
        }

        //根据依赖字段计算比率值
        for (String fieldcode : viewColumns) {
            Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(fieldcode);
            if (linkfields == null || linkfields.size() == 0) {
                continue;
            }

            String[] linkfield = new String[linkfields.size()];
            linkfield = linkfields.toArray(linkfield);

            final Object f1 = totalRowMap.get(linkfield[0]);
            if (f1 == null) {
                totalRowMap.put(fieldcode, 0);
                continue;
            }

            Object calVal = null;
            final Object linkFieldValue2 = totalRowMap.get(linkfield[1]);
            if (linkFieldValue2 != null) {
                if (f1.getClass().isAssignableFrom(Long.class)) {
                    Object linkFieldVal = getNewRateInstallData(fieldcode, totalRowMap, linkFieldValue2);

                    if (linkFieldVal == null) {
                        linkFieldVal = "1";
                    }
                    calVal = divide((Long) f1, Long.parseLong(linkFieldVal.toString()));
                } else if (f1.getClass().isAssignableFrom(Double.class)) {
                    calVal = divide((Double) f1, Double.valueOf(String.valueOf(
                            getNewRateInstallData(fieldcode, totalRowMap, linkFieldValue2))));
                }
            } else {
                calVal = 0.0d;
            }
            totalRowMap.put(fieldcode, calVal);
        }
        totalRowMap.put("isall0", false);
        totalRowMap.put("istotal", true);
        totalRowMap.put(firstTotalRowNeedReplaceId(), "汇总");

        List<Map<String, Object>> newResult = Lists.newArrayList();
        newResult.add(totalRowMap);

        newResult.addAll(sourceResult);

        return newResult;
    }
}
