package com.reyun.report.parse;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.reyun.context.AppUtils;
import com.reyun.model.Account;
import com.reyun.model.ReportListResult;
import com.reyun.model.SubChannelRename;
import com.reyun.report.AbstractListReportParse;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.SubChannelRenameRepository;
import com.reyun.service.ReportService;
import com.reyun.util.DateUtil;
import com.reyun.util.ValidateUtil;
import com.reyun.util.ViewColumnDependUtil;
import org.apache.commons.lang.StringUtils;
import org.json.JSONException;


import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;

/**
 * Created by nolan on 15/02/2017.
 * description:
 */
public class ListSubChannelParse
        extends AbstractListReportParse
{
    private String firstTotalRowColumnKey = "";
    private SubChannelRenameRepository subChannelRenameRepository = AppUtils.getApplicationContext().getBean(SubChannelRenameRepository.class);
    private ReportService reportService = AppUtils.getApplicationContext().getBean(ReportService.class);
    private AccountRepository authService = AppUtils.getApplicationContext().getBean(AccountRepository.class);

    public static Map<String,String> ignorePorteName = new HashMap(){{
        put("subchannel_bysubchannel_siteid","siteid");
        put("subchannel_bysubchannel_aid","aid");
        put("subchannel_bysubchannel_csite","csite");

    }};

    private static Map<String,String> csiteCodeName = new HashMap(){{
        put("1","头条");
//        put("3","头条");
        put("10001","西瓜");
        put("30001","火山");
        put("40001","抖音");
        put("800000000","穿山甲开屏");
        put("900000000","穿山甲");
    }};


    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions)
    {
        cid = -1L;
        String ss_keyword = "";
        if ("subchannel_bysubchannel".equals(reportName)) {
            cid = Long.parseLong(viewColumns.get(0));
            ss_keyword = viewColumns.get(1);
        }

        if ("subchannel_bycampaign".equals(reportName)) {
            if (reportResult.getColumnkey() != null && reportResult.getColumnkey().size() > 0) {
                viewColumns = Lists.newArrayList();
                for (Object tmpKey : reportResult.getColumnkey()) {
                    if (tmpKey.equals("campaignid")) {
                        continue;
                    }
                    viewColumns.add(String.valueOf(tmpKey));
                }
            }
            List<Map<String, Object>> sourceResult = reportResult.getVal();
            List<Map<String, Object>> newSourceResult = new ArrayList<>();
            if (sourceResult.size() != 0) {
                for (Map<String, Object> num : sourceResult) {
                    num.put("isall0", true);
                    boolean isall0 = true;
                    for (String obj : viewColumns) {
                        Object o = num.get(obj);

                        if (o.getClass().isAssignableFrom(Long.class)) {
                            boolean b = ((Long) o).equals(0L);
                            if (!((Long) o).equals(0L)) {
                                isall0 = false;
                                num.put("isall0", isall0);
                            }
                        }
                        else if (o.getClass().isAssignableFrom(Double.class)) {
                            DecimalFormat df = new DecimalFormat("0.00");
                            String format = df.format((Double) o);
                            if ("0.00".equals(format)) {
                                num.put("isall0", isall0);
                            }
                        }
                    }
                    if(viewColumns.contains("rate_retentiond3") && !num.containsKey("num_d3_install")){
                        num.put("num_d3_install",0l);
                    }
                    if(viewColumns.contains("rate_retentiond7") && !num.containsKey("num_d7_install")){
                        num.put("num_d7_install",0l);
                    }
                    if(viewColumns.contains("rate_retentiond30") && !num.containsKey("num_d30_install")){
                        num.put("num_d30_install",0l);
                    }
                    if(viewColumns.contains("rate_retentiond45") && !num.containsKey("num_d45_install")){
                        num.put("num_d45_install",0l);
                    }
                    if(viewColumns.contains("rate_retentiond60") && !num.containsKey("num_d60_install")){
                        num.put("num_d60_install",0l);
                    }
                    newSourceResult.add(num);

                }
                reportResult.setVal(newSourceResult);
            }
            firstTotalRowColumnKey = "campaignid";
            List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);
            reportResult.setVal(super.transCampaignName(newResult, app, account));
        }
        else {
            if (reportResult.getColumnkey() != null && reportResult.getColumnkey().size() > 0) {
                viewColumns = Lists.newArrayList();
                for (Object tmpKey : reportResult.getColumnkey()) {
                    if (tmpKey.equals("cid") || tmpKey.equals("subchannel")) {
                        continue;
                    }
                    viewColumns.add(String.valueOf(tmpKey));
                }
            }

            if ("subchannel_bysubchannel".equals(reportName)) {

                /**
                 * 替换子渠道的名字
                 */
                    List<Map<String, Object>> source = reportResult.getVal();
                    for (Map<String, Object> map : source) {
                        String subchannel = map.get("subchannel").toString();
                        SubChannelRename byCidAndSubChannel = subChannelRenameRepository.findByCidAndSubChannel(subchannel,cid,app);
                        if(byCidAndSubChannel!=null){
                            map.put("subchannel",byCidAndSubChannel.getSubChannelName());
                        }
                    }
                    /**
                     * 查找被检索的子渠道
                     */
                    /*List<Map<String, Object>> sourceResultFilter = new ArrayList<>();
                    if(!"null".equals(ss_keyword)) {
                        for (Map<String, Object> map : source) {
                            String subchannel = map.get("subchannel").toString();
                            if(subchannel.contains(ss_keyword)){
                                sourceResultFilter.add(map);
                            }
                        }
                        reportResult.setVal(sourceResultFilter);
                    }else {
                        reportResult.setVal(source);
                    }*/
                reportResult.setVal(source);

                if (reportResult.getColumnkey() != null && reportResult.getColumnkey().size() > 0) {
                    viewColumns = Lists.newArrayList();
                    for (Object tmpKey : reportResult.getColumnkey()) {
                        if (tmpKey.equals("subchannel")) {
                            continue;
                        }
                        viewColumns.add(String.valueOf(tmpKey));
                    }
                }
                List<Map<String, Object>> sourceResult = reportResult.getVal();
                List<Map<String, Object>> newSourceResult = new ArrayList<>();
                if (sourceResult.size() != 0) {
                    for (Map<String, Object> num : sourceResult) {
                        num.put("isall0", true);
                        boolean isall0 = true;
                        for (String obj : viewColumns) {
                            Object o = num.get(obj);

                            if (o.getClass().isAssignableFrom(Long.class)) {
                                if (!((Long) o).equals(0L)) {
                                    isall0 = false;
                                    num.put("isall0", isall0);
                                }
                            }
                            else if (o.getClass().isAssignableFrom(Double.class)) {
                                DecimalFormat df = new DecimalFormat("0.00");
                                String format = df.format((Double) o);
                                boolean b;
                                if ("0.00".equals(format)) {
                                    num.put("isall0", isall0);
                                }
                            }
                            if(viewColumns.contains("rate_retentiond3") && !num.containsKey("num_d3_install")){
                                num.put("num_d3_install",0l);
                            }
                            if(viewColumns.contains("rate_retentiond7") && !num.containsKey("num_d7_install")){
                                num.put("num_d7_install",0l);
                            }
                            if(viewColumns.contains("rate_retentiond30") && !num.containsKey("num_d30_install")){
                                num.put("num_d30_install",0l);
                            }
                            if(viewColumns.contains("rate_retentiond45") && !num.containsKey("num_d45_install")){
                                num.put("num_d30_install",0l);
                            }
                            if(viewColumns.contains("rate_retentiond60") && !num.containsKey("num_d60_install")){
                                num.put("num_d60_install",0l);
                            }
                        }
                        newSourceResult.add(num);
                    }
                    reportResult.setVal(newSourceResult);
                }
                firstTotalRowColumnKey = "subchannel";
                List<Map<String, Object>> newResult = calFirstTotalRow(reportResult, viewColumns);
                newResult = parseTotal(newResult, app, account, "subchannel_bysubchannel_sum", conditions);
                reportResult.setVal(newResult);
            }
            else {
                if (reportResult != null && reportResult.getColumnkey() != null && ignorePorteName.get(reportName)==null) {
                    reportResult.getColumnkey().set(0, "channelname");
                }
                List<Map<String, Object>> newResult = new ArrayList<>();
                if(ignorePorteName.get(reportName)==null){
                    newResult  = super.transChannelNameAddName(reportResult.getVal(), app, account);
                }
                if ("subchannel_bychannel".equals(reportName)) {
                    firstTotalRowColumnKey = "channelname";
                    //计算汇总行
                    newResult = calFirstTotalRow(reportResult, viewColumns);
                }

                if(ignorePorteName.get(reportName)!=null){

                    firstTotalRowColumnKey = ignorePorteName.get(reportName);

                    if("csite".equals(firstTotalRowColumnKey)){

                        // 替换指标代码 为中文
                        List<Map<String, Object>> source = reportResult.getVal();
                        for (Map<String, Object> map : source) {
                            String subchannel = map.get("csite").toString();
                            String name = csiteCodeName.get(subchannel);
                            if(!StringUtils.isEmpty(name)){
                                map.put("csite",name);
                            }
                        }
                    }

                    //计算汇总行
                    newResult = calFirstTotalRow(reportResult, viewColumns);
                }

                reportResult.setVal(newResult);
            }
        }
        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId()
    {
        return firstTotalRowColumnKey;
    }

    private List<Map<String, Object>> parseTotal(List<Map<String, Object>> result , Long app, Long account, String reportName, Map<String, String> conditions){
        List<Map<String, Object>> newResult = Lists.newArrayList();

        Account one = authService.findOne(account);
        try {
            Object report = reportService.report(app, reportName, one, conditions);
            ReportListResult listResult = null;
            ReportListResult reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(report.toString(), ReportListResult.class);
            List<Map<String, Object>> val = reportListResult.getVal();
            if(ValidateUtil.isValid(val) && ValidateUtil.isValid(val.get(0))){
                Map<String, Object> map = val.get(0);
                if(reportName.equals("subchannel_bysubchannel_sum")){
                    map.put("subchannel", "汇总");
                    map.put("subchannelname", "汇总");
                    map.put("isall0", false);
                    map.put("istotal", true);
                }
                for (String fieldcode : map.keySet()) {
                    if (fieldcode.toLowerCase().equals("duration")) {
                        map.put(fieldcode, DateUtil.formatCaseTime(Long.valueOf(map.get(fieldcode)==null?"0":map.get(fieldcode).toString())));
                    }
                    Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(fieldcode);
                    if (linkfields == null || linkfields.size() == 0) {
                        continue;
                    }

                    String[] linkfield = new String[linkfields.size()];
                    linkfield = linkfields.toArray(linkfield);

                    final Object f1 = map.get(linkfield[0]);
                    if (f1 == null) {
                        map.put(fieldcode, 0);
                        continue;
                    }
                    Object calVal = null;
                    final Object linkFieldValue2 = map.get(linkfield[1]);
                    if (linkFieldValue2 !=null){
                        if (f1.getClass().isAssignableFrom(Long.class)) {
                            if (fieldcode.startsWith("avg_time")) {
                                calVal = divideAvgTime(((Long) f1), (Long) linkFieldValue2);
                            }else if (fieldcode.equals("amt_use_freq")) {
                                calVal = divideAmt(((Long) f1) / 100.0, Long.valueOf(String.valueOf(linkFieldValue2))*1.0);
                            }else{
                                continue;
                            }
                        }else{
                            continue;
                        }
                        map.put(fieldcode, calVal);
                    }
                }
                newResult.add(map);
                for(Map<String, Object> m : result){
                    if(m.containsKey("istotal") && (m.get("istotal").toString()).equals("true")){

                    }else{
                        newResult.add(m);
                    }
                }
                return newResult;
            }
          
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }
}
