package com.reyun.report.parse;

import com.google.common.base.Function;
import com.google.common.base.Strings;
import com.google.common.collect.Maps;
import com.reyun.model.*;
import com.reyun.report.AbstractListReportParse;
import com.reyun.util.Constant;
import com.reyun.util.ValidateUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by nolan on 18/01/2017.
 * description:
 */
public class RemarketingCampaignParse
        extends AbstractListReportParse {

    private static final Logger logger = LoggerFactory.getLogger(RemarketingCampaignParse.class);

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns,
                                  String reportName, boolean isSearchText, List<String> dataIds) {
        return null;

    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> dataIds, Long cid, Map<String, String> conditions){
        if (isSearchText) {
            setDefaultValue(reportResult, dataIds);
        }

        final List<Map<String, Object>> valList = reportResult.getVal();
        Map<String, Map<String, Object>> valMap = Maps.newHashMap();
        if (valList != null) {
            valMap = Maps.uniqueIndex(valList, new Function<Map<String, Object>, String>() {
                @Override
                public String apply(Map<String, Object> map) {
                    return map.get("campaignid").toString();
                }
            });
        }

        List<Channel> channels = channelService.findCoopAndOwnChannel(account, null);
        List<CampaignGroup> groups = campaignGroupService.listCamgroupByAccountAndApp(app, account);

        Map<Long, String> cid_nameMap = new HashMap<>();
        for (Channel ch : channels) {
            cid_nameMap.put(ch.getId(), ch.getName());
        }

        Map<Long, String> gid_nameMap = new HashMap<>();
        for (CampaignGroup cg : groups) {
            gid_nameMap.put(cg.getId(), cg.getName());
        }

        //modify by sunhao 20170804解决数据补零
        List<Campaign> campaignList = new ArrayList<>();
        if (isSearchText || !CollectionUtils.isEmpty(dataIds) || "campaigninfo_bypackagecampaign".equals(reportName)) {   // 分包推广活动详情单独处理
            campaignList = super.campaignService.findByCampaignidList(dataIds, app);
        } else {
            campaignList = super.authService.getAuthCampaignListByAccount(account, app);
        }

        boolean nature = false;
        List<Long> campaignIds = new ArrayList<>();
        if (ValidateUtil.isValid(campaignList)) {
            for (Campaign cam : campaignList) {
                if (null != cam.getId()) {
                    campaignIds.add(cam.getId());
                } else {
                    nature = true;
                }
            }
        }

        List<CamCamgroupMap> mapList = new ArrayList<>();
        if (campaignIds.size() > 0) {
            mapList = camCamGroupMapRepository.listAllByCampaignIds(campaignIds);
        }
        Map<Long, String> cid_gnameMap = new HashMap<>();
        if (ValidateUtil.isValid(mapList)) {
            for (CamCamgroupMap camCamp : mapList) {
                cid_gnameMap.put(camCamp.getCampaign(), gid_nameMap.get(camCamp.getCampGroup()));
            }
        }

        Map<String, Object> totalResult = new HashMap<>();
        List<Map<String, Object>> values = new ArrayList<>();

        if (ValidateUtil.isValid(campaignList)) {
            for (Campaign cam : campaignList) {
                boolean ispackchannel = !Strings.isNullOrEmpty(cam.getPack_channelid());
                Map<String, Campaign> dimentionCampaignMapValue = new HashMap<>();
                dimentionCampaignMapValue.put(cam.getCampaignid(), cam);
                Map<String, Map<String, Object>> result = super.aggregateByDimension(cam.getName(), dimentionCampaignMapValue, valMap, viewColumns, totalResult);
                Map<String, Object> tmpResultValue = result.get("single");
                tmpResultValue.put("campaignid", cam.getCampaignid());
                tmpResultValue.put("ispackchannel", ispackchannel);
                tmpResultValue.put("channelname", cam.getChannel().equals(-1L) ? "自然量" : cid_nameMap.get(cam.getChannel()));
                tmpResultValue.put("groupname", cid_gnameMap.containsKey(cam.getId()) ? cid_gnameMap.get(cam.getId()) : "无");
                tmpResultValue.put("id", cam.getCampaignid());
                values.add(tmpResultValue);
            }
        }

        //翻译名字,补全全零数据
        List<Map<String, Object>> maps = super.transCampaignName(values, app, account);
        List<Map<String, Object>> maps_new = new ArrayList<>();
        List<String> campaigns = new ArrayList<>();
        if ("ADVERTISING".equals(conditions.get("marketinginfo_appmarket"))){
            campaigns = campaignRepository.findFilterDeeplinkByAppAndChannelCategory(app, "ADVERTISING");
        }else if ("APPMARKET".equals(conditions.get("marketinginfo_appmarket"))){
            campaigns = campaignRepository.findFilterDeeplinkByAppAndChannelCategory(app, "APPMARKET");
        }else{
            campaigns = campaignRepository.findAllFilterDeeplinkByApp(app,Constant.deeplinkHost);
        }
        for (Map<String, Object> map : maps) {
            if(campaigns.contains(map.get("id"))){
                maps_new.add(map);
            }
        }
        reportResult.setVal(maps_new);
        //统计汇总
        reportResult.setVal(super.calFirstTotalRow(reportResult, viewColumns));

        return reportResult;

    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

    @Override
    protected String firstTotalRowNeedReplaceId() {
        return "campaignid";
    }
}