package com.reyun.report.parse;

import com.google.common.base.Function;
import com.google.common.base.Strings;
import com.google.common.collect.*;
import com.reyun.context.AppUtils;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.report.AbstractListReportParse;
import com.reyun.repository.AccountRepository;
import com.reyun.service.AuthService;
import com.reyun.util.ValidateUtil;
import com.reyun.util.ViewColumnDependUtil;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * Created by nolan on 18/01/2017.
 * description:
 */
public class RemarketingSubAccountParse
        extends AbstractListReportParse {

    private AuthService authService = AppUtils.getApplicationContext().getBean(AuthService.class);

    private AccountRepository accountRepository = AppUtils.getApplicationContext().getBean(AccountRepository.class);

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds) {

        final List<Map<String,Object>> valList = reportResult.getVal();
        List<Map<String,Object>> resultMapList = new ArrayList<>();

        //1.先判断当前账户是不是母张号或者是管理员帐号，如果是,则要查出来所有root_parent是当前帐号的账号
        Account acc = accountRepository.findOne(account);
        Long rootParent=acc.getRootParent();
        List<Account> accountList;
        if(acc.getIsSuperUser()){
            accountList = accountRepository.findAccountByRootParent(rootParent);
        }else if(RoleEnumType.MANAGER.getKey().equals(acc.getRoleCategory())){
            accountList = accountRepository.findAccountByRootParentExManager(rootParent, acc.getId());
        } else {
            accountList = accountRepository.findAccountByParent(account);
        }

        //查询该账户及子账户下面所有活动
        Map<Long,Map<String, Campaign>> accountCampaignMap = authService.getAllCampaignByParentAccount(account,app,true);

        if(!CollectionUtils.isEmpty(accountList) && !CollectionUtils.isEmpty(accountCampaignMap)) {

            Map<String,Map<String,Object>> valMap = Maps.uniqueIndex(valList, new Function<Map<String,Object>, String>() {
                @Override
                public String apply(Map<String,Object> map) {
                    return map.get("campaignid").toString();
                }
            });

            //汇总所有数据到子账户下 子账户名称
            Map<String, Map<String, Object>> subAccountResultMap = Maps.newHashMap();

            for (Account act : accountList) {

                String accountName = act.getName();
                Map<String, Campaign> accountCampaignMapValue = accountCampaignMap.get(act.getId());
                Map<String, Object> tmpResultValue = subAccountResultMap.get(accountName);

                Map<String, Map<String, Object>> result = super.aggregateByDimension(accountName, accountCampaignMapValue, valMap, viewColumns, null);

                tmpResultValue = result.get("single");

                subAccountResultMap.put(accountName, tmpResultValue);
            }

            resultMapList = new ArrayList<>(subAccountResultMap.values());
        }

        reportResult.getName().remove(0);
        reportResult.getName().add(0, "子账号");
        reportResult.setVal(resultMapList);

        return reportResult;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid, Map<String, String> conditions) throws JSONException {
        return null;
    }

    @Override
    public ReportListResult parse(ReportListResult reportResult, Long app, Long account, List<String> viewColumns, String reportName, boolean isSearchText, List<String> searchIds, Long cid) throws JSONException {
        return null;
    }

}
