package com.reyun.report.roi;

import com.reyun.context.AppUtils;
import com.reyun.model.Channel;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractListReportParse;
import com.reyun.service.ChannelService;
import com.reyun.util.ValidateUtil;
import org.apache.commons.collections.ArrayStack;
import org.apache.commons.collections.map.HashedMap;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by mxq on 15/02/2017.
 * description:
 */
public class ListRoiByIntervalParse
{

	public ReportListResult parse(ReportListResult reportResult, Map<String,String> conditions)
	{
		List<Map<String, Object>> newResult = reportResult.getVal();
		Map<String, List<Map<String, Object>>> valMap = new HashMap<>();
		for(Map<String, Object> map : newResult){
			String ins_ds = map.get("ins_ds").toString();
			String pay_ds = map.get("pay_ds").toString();
			double amt_amount = Double.parseDouble(map.get("amt_amount").toString());
			if(valMap.containsKey(ins_ds)){
				List<Map<String, Object>> list = valMap.get(ins_ds);
				Map<String, Object> dataMap = new HashMap<>();
				dataMap.put("ins_ds", ins_ds);
				dataMap.put("pay_ds", pay_ds);
				dataMap.put("amt_amount", amt_amount);
				list.add(dataMap);
				valMap.put(ins_ds, list);
			}else{
				List<Map<String, Object>> list = new ArrayList<>();
				Map<String, Object> dataMap = new HashMap<>();
				dataMap.put("ins_ds", ins_ds);
				dataMap.put("pay_ds", pay_ds);
				dataMap.put("amt_amount", amt_amount);
				list.add(dataMap);
				valMap.put(ins_ds, list);
			}
		}

		List<Map<String, Object>> res = new ArrayList<>();
		String install_startdate = conditions.get("install_startdate");
		String install_enddate = conditions.get("install_enddate");
		int install_interval = Integer.parseInt(conditions.get("install_interval"));
		String pay_startdate = conditions.get("pay_startdate");
		String pay_enddate = conditions.get("pay_enddate");
		int pay_interval = Integer.parseInt(conditions.get("pay_interval"));
		List<List<String>> installInterval = getDateInterval(install_startdate, install_enddate, install_interval);
		List<List<String>> payInterval = getDateInterval(pay_startdate, pay_enddate, pay_interval);
		int clumns = payInterval.size();    //列的数目
		for(List<String> list : installInterval){
			String startDate = list.get(0);
			String endDate = list.get(0);
			Map<String, Object> map = new HashMap<>();
			if(startDate.compareTo(pay_startdate) > 0){
				pay_startdate = startDate;
			}
			if(pay_enddate.compareTo(pay_startdate) >= 0){
				List<List<String>> dateInterval = getDateInterval(pay_startdate, pay_enddate, pay_interval);
				for(int m = 0; m < dateInterval.size(); m++){// 初始化
					map.put(m + "", 0d);
				}
				for(String date : list){
					endDate = date;
					if(valMap.containsKey(date)){
						List<Map<String, Object>> valList = valMap.get(date);
						for(Map<String, Object> val : valList){
							String pay_ds = (String)val.get("pay_ds");
							Double amt_amount = (Double) val.get("amt_amount");
							for(int j = 0; j < dateInterval.size(); j++){
								List<String> payList = dateInterval.get(j);
								if(payList.contains(pay_ds)){
									map.put(j+"",map.containsKey(j+"") ? ((double)map.get(j+"") + amt_amount) : amt_amount);
								}
							}
						}
					}
				}
			}
			for(int i = 0; i < clumns; i++){
				if(!map.containsKey(i + "")){
					map.put(i + "","--");
				}
			}
			map.put("key",startDate + "～" + endDate);
			res.add(map);
		}
		reportResult.setVal(res);

		List<Object> keyList = new LinkedList<>();
		List<String> nameList = new LinkedList<>();
		keyList.add("key");
		nameList.add("付费激活周期");
		for(int i = 0; i < clumns; i++){
			keyList.add(i+"");
			nameList.add("第" + (i + 1) + "付费周期");
		}
		reportResult.setColumnkey(keyList);
		reportResult.setName(nameList);
		return reportResult;
	}

	public List<List<String>> getDateInterval(String startDate, String endDate, int num){
		String date = startDate;
		List<List<String>> res = new LinkedList<>();
		while(date.compareTo(endDate) <= 0){
			List<String> list = new LinkedList<>();
			for(int i = 0; i < num; i++){
				list.add(date);
				date = getSpecifiedDayBefore(date, 1);
				if(date.compareTo(endDate) > 0){
					break;
				}
			}
			res.add(list);
		}
		return  res;
	}

	public String getSpecifiedDayBefore(String specifiedDay, int days) {
		Calendar c = Calendar.getInstance();
		Date date = null;
		try {
			date = new SimpleDateFormat("yyyy-MM-dd").parse(specifiedDay);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		c.setTime(date);
		int day = c.get(Calendar.DATE);
		c.set(Calendar.DATE, day + days);

		String dayBefore = new SimpleDateFormat("yyyy-MM-dd").format(c.getTime());
		return dayBefore;
	}

}
