package com.reyun.repository;

import com.reyun.model.App;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;

import java.math.BigInteger;
import java.util.List;

public interface AppRepository extends JpaRepository<App, Long> {

    @Query(value = "select * from app where account in (select id from account where parent = ?1 or id = ?1) and del_flag is not true order by sort", nativeQuery = true)
    List<App> listAppByRootAccount(Long account);

    @Query(value = "select * from app where account in (select id from account where parent = ?1 or id = ?1) ", nativeQuery = true)
    List<App> listAllAppByRootAccount(Long account);



    @Query(value = "select appkey from app where id = ?1", nativeQuery = true)
    String findAppkeyById(Long id);

    List<App> findByAccount(Long account);

    @Query(value = "select appkey from app where account = ?1", nativeQuery = true)
    List<String> findAppkeyByAccount(Long account);

    @Query(value = "select * from app where account =?1 and bundleid = ?2 and del_flag is not true order by sort", nativeQuery = true)
    App findByBundleidInNotDel(Long account, String bundleid);

    @Query(value = "select * from app where account =?1 and name = ?2 and del_flag is not true", nativeQuery = true)
    App findByNameNotDel(Long account, String name);

    @Query(value = "select * from app where sync_ddb is false and del_flag is not true order by sort", nativeQuery = true)
    List<App> findNotSyncAppDebug();

    @Query(value = "select * from app where token = ?1 limit 1", nativeQuery = true)
    App findOneByAppToken(String token);

    @Query(value = "select * from app where id in ?1 order by sort", nativeQuery = true)
    List<App> findByIds(List<Long> ids);

    @Query(value = "SELECT DISTINCT t2.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and (t1.type!='ry_coop' or t1.type is NULL) and t1.category = 'ADVERTISING' and t2.id in ?1 and t2.platform = ?2 and t2.del_flag is not TRUE", nativeQuery = true)
    List<App> findAppsFromOnelink(List<Long> appids, String platform);

    @Query(value = "SELECT DISTINCT t2.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and (t1.type!='ry_coop' or t1.type is NULL) and t1.category = 'ADVERTISING' and t2.id = ?1", nativeQuery = true)
    App findAppFromOnelink(Long appid);

    @Query(value = "select * from app where appkey in ?1", nativeQuery = true)
    List<App> findByAppkeys(List<String> appkeys);

    @Query(value = "select * from app where appkey = ?1", nativeQuery = true)
    App findByAppkey(String appkeys);

    @Query(value = "select * from app a join auth b on a.id = b.app where b.account = ?1 and b.create_account in (select id from account where root_parent = ?2) ", nativeQuery = true)
    List<App> findAuthAppByRootParent(Long subAccountId, Long rootParent);

    @Query(value = "select distinct a.*  from app a join auth b on a.id = b.app where b.account = ?1 and b.create_account in (select id from account where root_parent = ?2) ", nativeQuery = true)
    List<App> findAuthAppByRootParentOnlyApp(Long subAccountId, Long rootParent);

    @Query(value = "select * from app a join auth b on a.id = b.app where b.account = ?1 ", nativeQuery = true)
    List<App> findAuthApp(Long subAccountId);

    @Query(value = "select platform from app where id = ?1", nativeQuery = true)
    String findPlatformByAppId(Long appId);

    @Query(value = "select categoryname from category where categoryid=(select game_genre from app where id=?1);", nativeQuery = true)
    String findGameCategoryByAppID(Long appId);

    @Query(value = "select categoryname from category where categoryid=(select app_genre from app where id=?1);", nativeQuery = true)
    String findAppGenreNameByAppID(Long id);

    @Query(value = "select DISTINCT a.* from app a right join data_auth d on a.id = d.app where d.account = ?1 and d.app in ?2 and d.channel = ?3 and (d.all_campaign is true or d.channel_permit is true)", nativeQuery = true)
    List<App> listTotalAuthorizedApp(Long accountId, List<Long> appIdList, Long channelId);

    @Query(value = "select create_account from app where appkey=?1", nativeQuery = true)
    String findCreateAccountByAppkey(String appKey);

    @Query(value = "select DISTINCT a.* from app a right join data_auth d on a.id = d.app where d.account = ?1  and d.channel = ?2 and (d.all_campaign is true or d.channel_permit is true)", nativeQuery = true)
    List<App> listApps(Long accountId, Long channelId);

    @Query(value = "select count(*) from app where root_parent = ?1 and del_flag is not true", nativeQuery = true)
    BigInteger count(Long accountId);

    @Query(value = "select * from app where root_parent = ?1 and sort < ?2 and del_flag is not true", nativeQuery = true)
    List<App> listAppsBefore(Long accountId, Integer sort);

    @Query(value = "select * from app where root_parent = ?1 and sort > ?2 and del_flag is not true", nativeQuery = true)
    List<App> listAppsAfter(Long accountId, Integer sort);

    @Query(value = "select * from app where root_parent = ?1 and sort >= ?2 and sort <= ?3 and del_flag is not true", nativeQuery = true)
    List<App> listAppsBetween(Long accountId, Integer sortStart, Integer sortEnd);

    @Query(value = "select * from app where root_parent = ?1 and del_flag is not true order by id asc", nativeQuery = true)
    List<App> listAppsAsc(Long accountId);

    @Query(value = "select * from app where root_parent = ?1 and del_flag is not true order by id desc", nativeQuery = true)
    List<App> listAppsDesc(Long accountId);

    @Query(value = "select * from app where root_parent =?1 and mip_appid = ?2 and del_flag is not true", nativeQuery = true)
    List<App> findBymipAppid(Long rootAccount, String mipAppid);

    @Query(value = "select * from app where platform = ?1 and del_flag is not true", nativeQuery = true)
    List<App> findWeiXinApp(String platform);

    @Query(value = "SELECT COUNT(id) FROM app WHERE appkey IN (SELECT appkey FROM channel_account_link_app WHERE  link_user_id = ?1)", nativeQuery = true)
    Long haveLinkApp(Long id);

    @Query(value = "SELECT * FROM app WHERE appkey in (SELECT appkey FROM channel_account_link_app WHERE  link_user_id = ?1)", nativeQuery = true)
    List<App> findLinkedAppName(Long id);

    @Query(value = "select appkey from app where account in (select id from account where parent = ?1 or id = ?1) and del_flag is not true  ", nativeQuery = true)
    List<String> findAppIdsByAccount(Long id);

    @Query(value = "select appkey from app where account in (select id from account where parent = ?1 or id = ?1)  ", nativeQuery = true)
    List<String> findAllAppIdsByAccount(Long id);

    @Query(value = "select appkey from app where account in (select id from account where parent = ?1 or id = ?1) and del_flag is true and modify_time > ?2", nativeQuery = true)
    List<String> findDeleteAppsByDate(Long accountId,String date);

    @Query(value = "select count(*) from app where bundleid = ?1", nativeQuery = true)
    Long checkUniqueBundleId(String bundleid);

    @Query(value = "select * from app where platform = 'ios' and is_debug is not true and del_flag is not true", nativeQuery = true)
    List<App> findAppWithBundleid();

    @Query(value = "select * from app where id in (select app from campaign where id in (?1) )", nativeQuery = true)
    List<App> findAppByCampaignIds(List<Long> campids);

    @Query(value = "select * from app where root_parent = ?1 and is_debug is not true and del_flag is not true", nativeQuery = true)
    List<App> findValidAppByRootParent(Long rootParent);

    @Query(value = "select b.* from (SELECT appkey FROM channel_account_link_app WHERE link_user_id = ?1 ) a left join app b on a.appkey = b.appkey left join (select  app from auth where account = ?2 ) c on b.id = c.app where c.app is not null", nativeQuery = true)
    List<App> listAuthLinkApps(Long linkId, Long accountId);

    @Query(value = "select * from app where whether_online is false and whether_appointment_monitor is true and del_flag is not true", nativeQuery = true)
    List<App> findAllAppByWhetherOnlineAndWhetherAppointmentMonitor();

    @Query(value = "select * from app where appkey = ?1 and token = ?2", nativeQuery = true)
    App getByAppidAndToken(String appid, String appsecret);

    @Query(value = "select appkey,name from app where account = 38 ", nativeQuery = true)
    List<Object[]> findNotDebugApp();

    @Query(value = "select appkey from app where account in (select id from account where id =?1 or root_parent= ?1) ", nativeQuery = true)
    List<String> findAllAppByAccount(Long id);

    @Query(value = "select * from app where wait_stop_campaign is true", nativeQuery = true)
    List<App> findAllAppByWaitStopCampaign();

    @Query(value = "SELECT * FROM app WHERE id IN (SELECT DISTINCT app FROM campaign_wangyan)", nativeQuery = true)
    List<App> findAllByWangyan();

    @Query(value = "select appkey from app where account = ?1", nativeQuery = true)
    List<String> findAppKeysByAccount(long account);
}
