package com.reyun.repository;

import com.reyun.model.CampaignGroup;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;

import java.math.BigInteger;
import java.util.List;

/**
 * 活动组
 */
public interface CampaignGroupRepository extends
		JpaRepository<CampaignGroup, Long> {
	
	@Query(value = "select * from campaign_group where app = ?1 and name = ?2 and create_account = ?3 and del_flag is not true", nativeQuery = true)
	CampaignGroup findByName(Long appid, String name, Long account);

	@Query(value = "select * from campaign_group where app = ?1 and name = ?2 and del_flag is not true", nativeQuery = true)
	CampaignGroup findByName(Long appid, String name);

	@Query(value = "select * from campaign_group where app = ?1 and name = ?2 and del_flag is not true", nativeQuery = true)
	List<CampaignGroup> findByNameAndApp(Long app, String name);

	@Query(value = "select * from campaign_group where app = ?1 and name like %?2% and create_account = ?3 and del_flag is not true", nativeQuery = true)
	List<CampaignGroup> findLikeName(Long appid, String name, Long account);

	@Query(value = "select * from campaign_group where app = ?1 and name like %?2% and del_flag is not true", nativeQuery = true)
	List<CampaignGroup> findAllLikeName(Long appid, String name);

	Iterable<CampaignGroup> findByApp(Long app);

	/**
	 * 基于appid查询所有活动组信息
	 *
	 * @param app 应用id
	 * @return
	 */
	Iterable<CampaignGroup> findAllByApp(Long app);

	/**
	 * 基于appid查询所有活动组信息,并按时间倒序
	 *
	 * @param app 应用id
	 * @return
	 */
	Iterable<CampaignGroup> findAllByAppOrderByCreateTimeDesc(Long app);


	/**
	 *
	 * @param account
	 * @return
	 */
	Iterable<CampaignGroup> findAllByAccount(Long account);

	@Query(value = "select count(id)  from campaign_group where app  = ?1 and name = ?2", nativeQuery = true)
	BigInteger hasExistName(Long app, String name);

	@Query(value = "select * from campaign_group where id in ?1", nativeQuery = true)
	List<CampaignGroup> findById(List<Long> ids);

	@Query(value = "select * from campaign_group where app in ?1", nativeQuery = true)
	List<CampaignGroup> findByApp(List<Long> appids);
	
	@Query(value = "select * from campaign_group where app = ?1 and del_flag is not true",nativeQuery=true)
	List<CampaignGroup> listCamgroupByApp(Long appId);

    @Query(value = "select c.* from campaign_group c right join account a on c.create_account = a.id  where c.app = ?1 and c.del_flag is not true",nativeQuery=true)
    List<CampaignGroup> listCmapaignGroupWithoutChannelPerson(Long appId);

	@Query(value = "select * from campaign_group where app  = ?1 and create_account = ?2 and del_flag is not true",nativeQuery=true)
	List<CampaignGroup> listCamgroupByAccount(Long app, Long account);

}
