package com.reyun.repository;

import com.reyun.model.Campaign;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

@Transactional
public interface CampaignRepository extends JpaRepository<Campaign, Long>, JpaSpecificationExecutor<Campaign> {

    Iterable<Campaign> findByApp(Long id);

    @Query(value = "select distinct app from campaign where create_account in ?1 and del_flag = 0 ", nativeQuery = true)
    List<Long> findByAccount(List<Long> accountidList);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 ", nativeQuery = true)
    List<Campaign> findByAccountApp(Long accountId, Long appId);

    @Query(value = "select * from campaign where channel in ?1 and app = ?2 and del_flag is not true ", nativeQuery = true)
    List<Campaign> findActiveByChannelApp(List<Long> channelIds, Long appId);

    @Query(value = "select * from campaign where channel in ?1 and app = ?2  ", nativeQuery = true)
    List<Campaign> findAllByChannelApp(List<Long> channelIds, Long appId);

    @Query(value = "select * from campaign where channel in ?1 and app = ?2 and del_flag=?3 ", nativeQuery = true)
    List<Campaign> findCampByChannelAppDelflag(List<Long> channelIds, Long appId, Boolean delflag);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 and del_flag is not true ", nativeQuery = true)
    List<Campaign> findActiveByCreate(Long createAccountId, Long appId);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 ", nativeQuery = true)
    List<Campaign> findAllByCreate(Long createAccountId, Long appId);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 and del_flag=?3 ", nativeQuery = true)
    List<Campaign> findActiveByCreateDelflag(Long createAccountId, Long appId, Boolean delflag);

    @Query(value = "select * from campaign c where c.name=?1 and c.app=?2 and del_flag is not true limit 1", nativeQuery = true)
    Campaign findByNameAndApp(String name, Long app);

    @Query(value = "select * from campaign c where c.name=?1 and c.app=?2 limit 1", nativeQuery = true)
    Campaign findByNameAndAppIncludeDel(String name, Long app);

    @Query(value = "select * from campaign c where c.pack_channelid=?1 and c.app=?2 and del_flag is not true limit 1", nativeQuery = true)
    Campaign findPackChannelidAndApp(String pack_channelid, Long app);

    @Query(value = "select * from campaign where (create_account<> ?1 or create_account is null) and  app = ?2 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<Campaign> findDataAuthCampaignListWithoutCreate(Long accountId, Long appId);

    @Query(value = "select * from campaign where create_account<> ?1 and  app = ?2 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false and d.channel = ?3) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true and d.channel = ?3))", nativeQuery = true)
    List<Campaign> findAsmDataAuthCampaignListWithoutCreate(Long accountId, Long appId, Long channel);

    @Query(value = "select * from campaign c where c.app = ?1", nativeQuery = true)
    List<Campaign> findAllByApp(Long app);

    @Query(value = "select * from campaign c where c.app = ?1 and name like %?2% ", nativeQuery = true)
    List<Campaign> findLikeName(Long app, String keyword);

    @Query(value = "select * from campaign c where c.app = ?1 and name like %?2% and del_flag=?3", nativeQuery = true)
    List<Campaign> findLikeNameWithDel(Long app, String keyword, Boolean delFlag);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is not true order by create_time desc ", nativeQuery = true)
    List<Campaign> findAll(Long app);

    @Query(value = "select * from campaign c where c.app=?1 order by create_time desc ", nativeQuery = true)
    List<Campaign> findAllAndDisable(Long app);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is true order by create_time desc ", nativeQuery = true)
    List<Campaign> findAllDisable(Long app);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is not true and channel_type = ?2 order by create_time desc ", nativeQuery = true)
    List<Campaign> findByChannelType(Long app, String channelType);

    @Query(value = "select * from campaign c where c.app=?1 and channel_type = ?2 order by create_time desc ", nativeQuery = true)
    List<Campaign> findByChannelTypeAndDisable(Long app, String channelType);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is true and channel_type = ?2 order by create_time desc ", nativeQuery = true)
    List<Campaign> findByChannelTypeDisable(Long app, String channelType);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is not true and channel_type is null order by create_time desc ", nativeQuery = true)
    List<Campaign> findCustomOrderByCreateTimeDesc(Long app);

    @Query(value = "select * from campaign c where c.app=?1 and channel_type is null order by create_time desc ", nativeQuery = true)
    List<Campaign> findCustomAndDisableOrderByCreateTimeDesc(Long app);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is true and channel_type is null order by create_time desc ", nativeQuery = true)
    List<Campaign> findCustomCampaignDisable(Long app);

    List<Campaign> findCampaignByAppAndName(Long app, String Name);

    @Query(value = "select * from campaign c where c.id in ?1 ", nativeQuery = true)
    List<Campaign> findById(Iterable<Long> id);

    @Query(value = "select * from campaign c where c.id in ?1", nativeQuery = true)
    List<Campaign> findAllById(Iterable<Long> id);

    @Query(value = "select * from campaign c where c.app=?1", nativeQuery = true)
    List<Campaign> listCampByApp(Long app);

    @Query(value = "select * from campaign c where c.app=?1", nativeQuery = true)
    List<Campaign> listAllCampByApp(Long app);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag=?2", nativeQuery = true)
    List<Campaign> listAllCampByAppAndDelflag(Long app, Boolean delFlag);

    @Query(value = "select * from campaign c where c.app=?1 and c.channel=?2 and del_flag is not true", nativeQuery = true)
    List<Campaign> findByAppAndChannel(Long app, Long channel);

    /**
     * 根据app和channel，查询所有活动
     * (跟上面的findByAppAndChannel的区别是，在查活动的时候，针对广点通的做了特殊处理，只查广点通中方案一的活动)
     * <p>
     * add by zhangsy 20180822
     *
     * @param app
     * @param channel
     * @return
     */
    @Query(value = "SELECT c.* FROM campaign c JOIN channel h ON c.channel=h.id WHERE c.app=?1 AND c.channel=?2 AND c.del_flag IS NOT TRUE AND (h.unique_name !='gdt' OR (h.unique_name='gdt' AND c.surl !=NULL AND c.surl !=''))", nativeQuery = true)
    List<Campaign> findByAppAndChannelNew(Long app, Long channel);

    @Query(value = "select * from campaign c where c.channel=?1 and del_flag is not true", nativeQuery = true)
    List<Campaign> findAllByChannel(Long channel);

    @Query(value = "select * from campaign c where c.advertiserid is not null AND c.bundleid is not null AND c.app is not null and c.channel=?1", nativeQuery = true)
    List<Campaign> findGdtByChannel(Long channel);

    @Query(value = "select c.channelid from campaign c where c.id in ?1", nativeQuery = true)
    List<String> findChannelidById(List<Long> id);

    @Query(value = "select count(*) from campaign c where c.app=?1", nativeQuery = true)
    Integer getCount(Long app);

    @Query(value = "select * from campaign c where c.surl=?1", nativeQuery = true)
    Campaign findCampaignBySurl(String surl);

    @Query(value = "select * from campaign c where c.app=?1 and c.campaignpackid=?2", nativeQuery = true)
    Campaign findCampaignByAppAndCampaignpackid(Long app, String campaignpackid);

//	@Query(value = "select * from callback c where c.key=?1", nativeQuery = true)
//	Callback getCallBackId(String key);

    @Query(value = "select c.* from campaign c, channel ch where c.app=?1 and c.channel=ch.id and (ch.type='ry_bd' or ch.type='ry_coop')", nativeQuery = true)
    List<Campaign> findAllInBDChannel(Long app);

    @Query(value = "select * from campaign c where c.channel=?1", nativeQuery = true)
    List<Campaign> findByChannel(Long channel);


    @Query(value = "select c.* from campaign c, channel ch where c.app=?1 and c.channel=ch.id and ch.type=?2", nativeQuery = true)
    List<Campaign> find(Long app, String channeltype);

    @Query(value = "select * from campaign where campaignid = ?1 and app = ?2", nativeQuery = true)
    Campaign findCampaignByCampaignId(String campaignId, Long appId);

    @Query(value = "select c.surl from campaign c where c.del_flag=?1 and c.campaignid in ?2", nativeQuery = true)
    List<String> findAllSurlBydDelFlagAndCampaignids(Boolean del_flag, List<String> campaignids);

//    @Query(value = "select * from campaign c where c.campaignid in ?1", nativeQuery = true)
//    List<Campaign> findByCampaignid(List<String> campaignidList);

    @Query(value = "select * from campaign c where c.campaignid in ?1 and app = ?2 ", nativeQuery = true)
    List<Campaign> findByCampaignid(List<String> campaignidList, Long app);

    @Query(value = "select * from campaign c where c.campaignid in ?1 and app = ?2 ", nativeQuery = true)
    List<Campaign> findByCampaignid(Set<String> campaignidList, Long app);

    @Query(value = "select * from campaign c where c.campaignid in ?1  ", nativeQuery = true)
    List<Campaign> findAllByCampaignId(List<String> campaignidList);

    @Query(value = "select name from campaign c where c.campaignid in ?1 and del_flag is not true", nativeQuery = true)
    List<String> findNameByCampaignid(List<String> campaignidList);

    @Query(value = "select * from campaign c where c.app=?1 and del_flag is not true", nativeQuery = true)
    List<Campaign> findCampaignByApp(Long app);

    @Query(value = "SELECT DISTINCT t0.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and t1.id = ?2 and t2.id = ?1", nativeQuery = true)
    List<Campaign> findFromOnelink(Long appid, Long channel);

    @Query(value = "select count(*) from campaign c ,channel ch where ch.unique_name = 'smsearch' and c.app in ?1 and ch.id = c.channel and c.del_flag is not true ", nativeQuery = true)
    BigInteger findFromAppsAndSM(List<Long> appids);

    @Query(value = "select * from campaign c where c.app = ?1 and url = ?2 and del_flag is not true", nativeQuery = true)
    List<Campaign> findAllByAppAndUrl(Long app, String url);

    //查询非自己创建的授权活动
    @Query(value = "select * from campaign where create_account<> ?1 and  app = ?2 and channel_type is null and del_flag = ?3 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<Campaign> findAuthCampaignByCustomChannel(Long accountId, Long appId, Boolean isDisable);

    @Query(value = "select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true", nativeQuery = true)
    List<Long> findChannelsByDataAuth(Long accountId, Long appId);

    @Query(value = "select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false", nativeQuery = true)
    List<Long> findCampaignIdsByDataAuth(Long accountId, Long appId);


    @Query(value = "select * from campaign where create_account<> ?1 and app = ?2 and channel_type = ?3 and del_flag = ?4 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            " or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<Campaign> findAuthCampaignByChannelType(Long accountId, Long appId, String channelType, Boolean isDisable);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 and channel_type is null and del_flag = ?3", nativeQuery = true)
    List<Campaign> findCreateByCustomChannel(Long accountId, Long appId, Boolean isDisable);

    @Query(value = "select * from campaign where create_account = ?1 and app = ?2 and channel_type = ?3 and del_flag = ?4", nativeQuery = true)
    List<Campaign> findCreateByChannelType(Long accountId, Long appId, String channelType, Boolean isDisable);

    @Query(value = "select * from campaign where app = ?1 and channel in (?2) and del_flag is not true", nativeQuery = true)
    List<Campaign> findByAppAndChannels(Long appId, List<String> channelIds);

    @Transactional
    @Modifying
    @Query(value = "update Campaign c set c.name=?3 where c.id =?1 and c.app = ?2")
    int updateCampaignName(Long id, Long appid, String name);

    @Query(value = "select * from campaign c where c.awssuccess = 0 and c.surl is not null and c.surl != '' and c.del_flag is false", nativeQuery = true)
    List<Campaign> findUnSync();

    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.awssuccess=?2 where c.id = ?1", nativeQuery = true)
    int updateAwsStatus(Long id, Boolean isSuccess);

    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.awssuccess=?2 where c.surl in ?1", nativeQuery = true)
    int updateAwsStatus(List<String> ids, Boolean isSuccess);

    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.url=?2,c.awssuccess = 0 where c.app = ?1", nativeQuery = true)
    int updateDownloadUrl(Long app, String url);


    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.awssuccess = false where c.channel = ?1 and del_flag is false", nativeQuery = true)
    int updateAwsStatusByChannel(Long channelId);


    @Query(value = "select * from campaign c where surl is not null and del_flag is true and awssuccess = 0", nativeQuery = true)
    List<Campaign> findNeedDeleteDDB();


    @Query(value = "select distinct third_account, third_account_alias from campaign c where app=?1 and channel=?2 and del_flag is not true and third_account is not null", nativeQuery = true)
    List<Object> findThirdAccountByAppChannel(Long app, Long channel);

    @Query(value = "select distinct third_account, third_account_alias from campaign c where account in (?1) and channel=?2 and del_flag is not true and third_account is not null", nativeQuery = true)
    List<Object> findThirdAccountByAccountChannel(List<Long> accounts, Long channel);

    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.third_account_alias = ?3 where c.channel = ?1 and third_account=?2 and account in (?4)", nativeQuery = true)
    int updateThirdAccountAlias(Long channel, String third_account, String third_account_alias, List<Long> accounts);

    @Query(value = "select count(*) from campaign where mip_type=?1 and app=?2", nativeQuery = true)
    Integer findbyMipType(String miniQrCode, Long appid);


    @Query(value = "select * from campaign c where c.app=?1 order by create_time desc limit ?2 ", nativeQuery = true)
    List<Campaign> findAllAndDisableLimit(Long appid, int pagesize);

    @Query(value = "select * from campaign c where c.app=?1 and  name like %?2%   order by create_time desc limit ?3 ", nativeQuery = true)
    List<Campaign> findAllAndDisableLimit(Long appid, String name, int pagesize);

    @Query(value = "select count(1) from campaign c where c.app=?1  ", nativeQuery = true)
    int findAllAndDisableLimitCount(Long appid);

    @Query(value = "select count(1) from campaign c where c.app=?1 and ( name like %?2% or surl like %?2% ) ", nativeQuery = true)
    int findAllAndDisableLimitCount(Long appid, String name);

//    @Query(value = "select count(1) from campaign c where c.app=?1 and del_flag is not true and channel_type = ?2 ", nativeQuery = true)
//    int findAllLimitCount(Long appid, String channelType);
//
//
//    @Query(value = "select count(1) from campaign c where c.app=?1 and del_flag is true and channel_type = ?2 ", nativeQuery = true)
//    int findDisableAllCount(Long appid, String channelType);

    @Query(value = "select count(1) from campaign c where c.app=?1 and del_flag is not true ", nativeQuery = true)
    int findAllCount(Long appId);

    @Query(value = "select * from campaign c where c.campaignid in ?1  ", nativeQuery = true)
    List<Campaign> findByCampaignidNotApp(List<String> transCampaignIds);

    @Query(value = "select c.id,c.campaignpackid,c.channel,c.pack_channelid,c.campaignid,c.subscribes from campaign c  where app=?1", nativeQuery = true)
    List<Object[]> listPartCampByApp(Long appid);

    @Query(value = "select campaignid from campaign where app = ?1 and channel in (?2) ", nativeQuery = true)
    List<String> findByAppAndChannelIds(Long appId, List<String> channelIdList);

    @Query(value = "select campaignid from campaign c where c.app = ?1", nativeQuery = true)
    Collection<? extends String> findAllCampaignIdsByApp(Long appId);

    @Query(value = "select campaignid from campaign where create_account = ?1 and app = ?2 ", nativeQuery = true)
    List<String> findCampaignIdsByAccountApp(Long accountId, Long appId);

    @Query(value = "select campaignid,channel from campaign where create_account = ?1 and app = ?2 ", nativeQuery = true)
    List<Object[]> findCampaignIdsCidByAccountApp(Long accountId, Long appId);


    @Query(value = "select campaignid from campaign where (create_account<> ?1 or create_account is null) and  app = ?2 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<String> findDataAuthCampaignIdListWithoutCreate(Long accountId, Long appId);

    @Query(value = "select campaignid,channel from campaign where (create_account<> ?1 or create_account is null) and  app = ?2 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<Object[]> findDataAuthCampaignIdCidListWithoutCreate(Long accountId, Long appId);

    @Query(value = "SELECT * FROM campaign WHERE account IN (SELECT id  FROM account WHERE past_date < ?1 ) ", nativeQuery = true)
    List<Campaign> find15DaysAgo(String date15);

    @Query(value = "select campaignid from campaign where create_account<> ?1 and  app = ?2 and channel_type is null and del_flag = ?3 and channel_category = ?5 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            "or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<String> findAuthCampaignIdsByCustomChannel(Long id, Long appId, boolean isDisable, String APPMARKET);

    @Query(value = "select campaignid from campaign where create_account = ?1 and app = ?2 and channel_type is null and del_flag = ?3 and channel_category = ?5", nativeQuery = true)
    List<String> findIdsCreateByCustomChannel(Long id, Long appId, boolean isDisable, String APPMARKET);

    @Query(value = "select campaignid from campaign where create_account<> ?1 and app = ?2 and channel_type = ?3 and del_flag = ?4 and channel_category = ?5 and  (id in (select d.campaign from data_auth d right join auth a on a.app = d.app and a.account= d.account where  a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is false) " +
            " or channel in (select d.channel from data_auth d right join auth a on a.app = d.app and a.account= d.account where a.account = ?1 and a.app = ?2 and a.status is true and d.all_campaign is true))", nativeQuery = true)
    List<String> findAuthCampaignIdsByChannelType(Long id, Long appId, String channelType, boolean isDisable, String APPMARKET);

    @Query(value = "select campaignid from campaign where create_account = ?1 and app = ?2 and channel_type = ?3 and del_flag = ?4 and channel_category = ?5", nativeQuery = true)
    List<String> findIdsCreateByChannelType(Long id, Long appId, String channelType, boolean isDisable, String APPMARKET);

    @Query(value = "select campaignid from campaign where app = ?1  and  channel_category = ?2   ", nativeQuery = true)
    List<String> findAllCampinIds(Long appId, String appmarket);

    @Query(value = "select * from campaign c where c.app=?1 and channel in (?3) and del_flag=?2", nativeQuery = true)
    List<Campaign> listAllCampaignByDelflagAndCid(Long appid, Boolean delflag, String cid);

    @Query(value = "SELECT  third_account,GROUP_CONCAT(id) FROM campaign c WHERE third_account in ?1 AND channel=5305 AND del_flag IS NOT TRUE AND third_account IS NOT NULL GROUP BY third_account", nativeQuery = true)
    List<Object[]> findByThirdAccount(List<String> o);

    @Query(value = "SELECT id FROM campaign WHERE campaignid IN (?1)", nativeQuery = true)
    List<String> findIdByCam(List<String> camids);

    @Query(value = "SELECT * FROM campaign WHERE new_insert = true", nativeQuery = true)
    List<Campaign> findByNewInsert();


    @Transactional
    @Modifying
    @Query(value = "update campaign set url_name = ?2 where id = ?1", nativeQuery = true)
    int modifyUrlName(Long campaignId, String name);

    @Query(value = "select surl from campaign where surl in (?1) union all select surl from onelink where surl in (?1) union all select surl from package_sub_campaign where surl in (?1) union all select surl from campaign_expired where surl in (?1) union all select feedbackURL surl from gdt_plan where feedbackURL in (?1)", nativeQuery = true)
    List<String> checkSurlExist(List<String> asList);

    @Query(value = "select * from campaign c where c.pack_channelid=?1 and c.app=?2  limit 1", nativeQuery = true)
    Campaign findPackChannelidAndAppExist(String packchannelid, Long appid);

    @Query(value = "select * from campaign where id not in (select campaign from data_auth where app = ?1 and account = ?2) and app = ?1 and id in ?3 AND channel = 5305 AND del_flag = FALSE", nativeQuery = true)
    List<Campaign> findByDataAuth(Long id, Long account, List<String> caids);

    @Query(value = "SELECT count(a.url) FROM (SELECT c.surl as url from campaign c WHERE c.surl = ?1 UNION ALL SELECT c.surl as url from onelink c WHERE c.surl = ?1 UNION ALL SELECT c.surl as url from package_sub_campaign c WHERE c.surl = ?1 UNION ALL SELECT c.surl as url from campaign_expired c WHERE c.surl = ?1 UNION ALL SELECT c.feedbackURL as url from gdt_plan c  WHERE c.feedbackURL = ?1) a", nativeQuery = true)
    Integer findSUrlALLTable(String s);

    @Query(value = "SELECT * FROM campaign WHERE account IN (SELECT id FROM account WHERE root_parent=?1)", nativeQuery = true)
    List<Campaign> findByRootParent(Long rootParent);

    @Query(value = "SELECT * FROM campaign WHERE app IN (SELECT app FROM auth WHERE account  = ?1 AND STATUS IS TRUE)", nativeQuery = true)
    List<Campaign> findByAuth(Long id);

    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.url=?2 ,c.url_name = ?3 where c.id in ?1", nativeQuery = true)
    int updataUlrWithIds(List<Long> campids, String new_url, String urlName);


    @Transactional
    @Modifying
    @Query(value = "update campaign c set c.url_name = ?2 where c.app = ?3 and  c.url=?1", nativeQuery = true)
    int updateUrlName(String new_url, String urlName, Long appid);


    @Query(value = "select * from campaign c where c.app=?1 and c.channel=?2 ", nativeQuery = true)
    List<Campaign> findByAppAndChannelAll(Long appid, Long channelid);

    @Query(value = "select distinct * from (select b.* from data_auth a left join campaign b on a.campaign = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is not true and b.id > 0 union all select * from campaign where channel in (select b.id from data_auth a left join channel b on a.channel = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is true)and app = ?2 union all select * from campaign where create_account = ?1 and app = ?2) a limit ?3 ", nativeQuery = true)
    List<Campaign> findAuthCampaign(Long accountId, Long appid, int limit);


    @Query(value = "select  sum(capcount) from (select count(b.id) capcount from data_auth a left join campaign b on a.campaign = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is not true and b.id > 0 union all select count(*) capcount from campaign where channel in (select b.id from data_auth a left join channel b on a.channel = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is true)and app = ?2 union all select count(*) capcount from campaign where create_account = ?1 and app = ?2) a ", nativeQuery = true)
    BigInteger findAuthCampaignCount(Long accountId, Long appid);


    @Query(value = "select distinct * from (select b.* from data_auth a left join campaign b on a.campaign = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is not true and b.id > 0 and b.name like %?3% union all select * from campaign where channel in (select b.id from data_auth a left join channel b on a.channel = b.id where a.app = ?2 and a.account = ?1 and a.all_campaign is true)and app = ?2 and name like %?3% union all select *from campaign where create_account = ?1 and app = ?2 and name like %?3% ) a limit ?4  ", nativeQuery = true)
    List<Campaign> findAuthCampaignName(Long id, Long appid, String name, int limit);

    @Query(value = " select third_account, third_account_alias, count(*) appcount from (select third_account, third_account_alias from campaign c where account in (?1) and channel = ?2 and del_flag is not true and third_account is not null group by third_account, app) a group by third_account", nativeQuery = true)
    List<Object[]> findThirdBackAccountByAccountChannel(List<Long> accounts, Long channel);

    @Query(value = "select third_account, third_account_alias,count(app) appcount from (select third_account, third_account_alias,app from campaign where third_account in (select distinct third_account from campaign c where app = ?1 and channel = ?2 and del_flag is not true and third_account is not null) group by third_account,app ) a group by third_account", nativeQuery = true)
    List<Object[]> findThirdBackAccountByAppChannel(Long appid, Long channel);

    @Query(value = "select p.id,p.name from (select app from campaign c where account in (?1)and channel = ?2 and del_flag is not true and third_account group by  app ) a left join  app p on a.app = p.id", nativeQuery = true)
    List<Object[]> findThirdBackAccountApp(List<Long> accounts, Long channel);


    @Query(value = "select id,name from app where id in (select distinct app from campaign c where third_account = ?1 and del_flag is not true)", nativeQuery = true)
    List<Object[]> findAppByThirdAccountBack(String thirdAccount);


    @Query(value = "select distinct third_account from campaign where app = ?2 and channel = ?3 and (third_account_alias like %?1% or third_account = ?1)", nativeQuery = true)
    List<String> findThirdBackAccountByAppKeyWorrds(String keyword, Long appId, String channelId);


    @Query(value = "SELECT  third_account,GROUP_CONCAT(id) FROM campaign c WHERE app = ?1 AND channel = 5305 AND del_flag IS NOT TRUE AND third_account IS NOT NULL GROUP BY third_account", nativeQuery = true)
    List<Object[]> findThirdAccountAndId(Long id);

    @Query(value = "select campaignid,name from campaign where campaignid in (?1) and app = ?2", nativeQuery = true)
    List<Object[]> findObjectsByCampaignid(List<String> val, Long app);

    @Query(value = "select campaignid,name from campaign where  app = ?1", nativeQuery = true)
    List<Object[]> findObjectsByApp(Long app);

    @Transactional
    @Modifying
    @Query(value = "update campaign c SET url = REPLACE (url, ?2, ?3) where app=?1", nativeQuery = true)
    void UpdateUrlByAppId(Long appid, String oldLinkId, String newLinkId);

    @Query(value = "select * from  campaign where app=?1 and url like %?2%", nativeQuery = true)
    List<Campaign> findByAppIdAndLikeUrl(Long appid, String linkId);

    @Query(value = "select third_account from  campaign where app=?2 and third_account_alias=?1 limit 1 ", nativeQuery = true)
    String getThirdAccountByAlias(String thirdaccountcode, Long id);

    @Query(value = "select * from (select b.id, b.name, case when type is not null then type else 'custom' end type from (select channel from campaign where app = ?1 group by channel) a left join channel b on a.channel = b.id where b.id is not null) c where c.type = ?2 ", nativeQuery = true)
    List<Object[]> findChannelNames(Long appid, String type);

    @Query(value = "select campaignid from campaign where app=?1 and del_flag is true", nativeQuery = true)
    List<String> findCampaignDsiabledByApp(Long appid);

    @Transactional
    @Modifying
    @Query(value = "update campaign set deeplink = ?2 where campaignid = ?1", nativeQuery = true)
    void updateDeeplinkByCampaignId(String campaignid, boolean deeplink);

    @Query(value = "select campaignid from campaign where campaignid in (?1) and ( url like %?2% or deeplink is true)", nativeQuery = true)
    List<String> findAllFilterDeeplinkByCampaignIds(ArrayList<String> campaignid, String deeplinkHost);

    @Query(value = "select campaignid from  campaign where account = ?1 and app = ?2 and ( url like %?3% or deeplink is true)", nativeQuery = true)
    List<String> findAllFilterDeeplinkByAccountAndApp(Long rootParent, Long id, String deeplinkHost);

    @Query(value = "select campaignid from campaign where campaignid in (?1) and  deeplink is true and channel_category = ?2", nativeQuery = true)
    List<String> findFilterDeeplinkByCampaignIdsAndChannelCategory(ArrayList<String> campaignid, String channelCategory);

    @Query(value = "select campaignid from  campaign where account = ?1 and app = ?2 and deeplink is true and channel_category = ?3", nativeQuery = true)
    List<String> findFilterDeeplinkByAccountAndAppAndChannelCategory(Long rootParent, Long id, String channelCategory);

    @Query(value = "select * from campaign where app=?1 and subscribes = ?2 and del_flag is not true", nativeQuery = true)
    List<Campaign> findByAppAndSubscribes(Long appid, boolean subscribes);

    @Query(value = "select * from campaign c where c.campaignid in ?1 and app = ?2 and  subscribes = ?3", nativeQuery = true)
    List<Campaign> findByCampaignid(List<String> campaignidList, Long app, boolean subscribe);

    @Query(value = "select * from campaign c where c.campaignid in ?1 and app = ?2 and  subscribes is not true", nativeQuery = true)
    List<Campaign> findByCampaignidAndSubscribeNotTrue(List<String> campaignidList, Long app);

    @Query(value = "select * from campaign where app =? and del_flag is false ", nativeQuery = true)
    List<Campaign> findByAppId(Long id);

    @Query(value = "select campaignid from  campaign where app = ?1 and deeplink is true and channel_category = ?2", nativeQuery = true)
    List<String> findFilterDeeplinkByAppAndChannelCategory(Long id, String appmarket);

    @Query(value = "select campaignid from  campaign where app = ?1 and ( url like %?2% or deeplink is true)", nativeQuery = true)
    List<String> findAllFilterDeeplinkByApp(Long id, String deeplinkHost);

    @Query(value = "SELECT c.surl as url from campaign c WHERE c.surl in ?1 UNION ALL SELECT c.surl as url from onelink c WHERE c.surl in ?1 UNION ALL SELECT c.surl as url from package_sub_campaign c WHERE c.surl in ?1 UNION ALL SELECT c.surl as url from campaign_expired c WHERE c.surl in ?1 UNION ALL SELECT c.feedbackURL as url from gdt_plan c  WHERE c.feedbackURL in ?1", nativeQuery = true)
    List<String> findSUrlALLTableBySurls(List<String> asList);

    @Query(value = "select id,campaignid,subscribes from campaign where id in ?1", nativeQuery = true)
    List<Object[]> findCampaignIdsAndSubscribesById(List<Long> authCampaignList);

    @Query(value = "select * from campaign where app=?1 and pack_channelid in ?2",nativeQuery = true)
    List<Campaign> findByAppAndPackcid(Long appid, List<String> packcids);

    @Query(value = "SELECT * FROM campaign WHERE id IN (SELECT campaign FROM `data_auth_pkgsubcampaign` WHERE app = ?2 AND account = ?1 )",nativeQuery = true)
    List<Campaign> findAuthCampaignBySubCam(Long account, Long appId);

    @Query(value = "select * from campaign where app=?1 and subscribes = ?2 limit 1", nativeQuery = true)
    Campaign findByAppAndSubscribesLimitOne(Long app_id, boolean subscribes);

    @Query(value = "select * from campaign where app = ?1 and ( url like '%link.trackingio%' or deeplink is true)  limit 1", nativeQuery = true)
    Campaign findByAppAndDeeplinkLimitOne(Long app_id);

    //网眼家定制
    @Query(value = "SELECT * FROM campaign WHERE id IN (SELECT campaign_id FROM `campaign_wangyan` WHERE app = ?1)",nativeQuery = true)
    List<Campaign> findByWangyanApp(Long app);
}

