package com.reyun.repository;

import com.reyun.model.ChannelAccountMap;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.Collection;
import java.util.List;

/**
 * Created by sunhao on 17/7/26.
 * desc:渠道账号对应关系DAO
 */
@Transactional
public interface ChannelAccountMapRepository extends JpaRepository<ChannelAccountMap, Long> {

    @Query(value = "select * from channel_account_map where root_account = ?1 ", nativeQuery = true)
    List<ChannelAccountMap> findAllChannelAccount(Long rootAccount);

    @Query(value = "select * from channel_account_map where root_account = ?1 and status = ?2", nativeQuery = true)
    List<ChannelAccountMap> findChannelAccount(Long rootAccount, Boolean status);

    @Query(value = "select channel_account from channel_account_map where root_account = ?1 and status = ?2", nativeQuery = true)
    List<Long> findChannelAccountIdByRoot(Long rootAccountId, Boolean status);

    @Query(value = "select * from channel_account_map where root_account = ?1 and status = ?2", nativeQuery = true)
    List<ChannelAccountMap> findChannelAccountByRoot(Long rootAccountId, Boolean status);

    @Query(value = "select channel_account from channel_account_map where sub_account = ?1 and status = ?2", nativeQuery = true)
    List<Long> findChannelAccountByCreate(Long createAccountId, Boolean status);

    @Query(value = "select * from channel_account_map where root_account = ?1 and channel_account = ?2", nativeQuery = true)
    ChannelAccountMap findAccount(Long rootAccount, Long channelAccount);

    @Transactional
    @Modifying
    @Query(value = "update channel_account_map set status = true where root_account = ?1 and channel_account = ?2 and status is false ", nativeQuery = true)
    int enableChannelAccount(Long rootAccountId, Long channelAccountId);

    @Transactional
    @Modifying
    @Query(value = "update channel_account_map set status = false where root_account = ?1 and channel_account = ?2 and status is true ", nativeQuery = true)
    int disableChannelAccount(Long rootAccountId, Long channelAccountId);

    @Query(value = "select root_account from channel_account_map where channel_account = ?1", nativeQuery = true)
    List<BigInteger> findRootAccountsByChannelAcountId(Long channelAccount);

    @Query(value = "select id from account where root_parent = ?1 and role_category < 3 ", nativeQuery = true)
    List<BigInteger> findChannelAccountsByRootAccount(Long rootAccount);
}
