package com.reyun.repository;

import com.reyun.model.ChannelAccount;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.List;

/**
 * Created by mxq on 17/7/11.
 */
public interface ChannelAccountRepository
        extends JpaRepository<ChannelAccount, Long>
{

    List<ChannelAccount> findByCreateAccount(long account);

    @Query(value = "select * from channel_account where root_account = ?1 and channel = ?2", nativeQuery = true)
    List<ChannelAccount> findByCreateAccountCid(long account, long cid);

    @Query(value = "select alias from channel_account where create_account in ?1", nativeQuery = true)
    List<String> listAliasByAccounts(List<Long> accounts);

    @Query(value = "select * from channel_account where create_account in ?1", nativeQuery = true)
    List<ChannelAccount> listByAccounts(List<Long> accounts);

    @Query(value = "select * from channel_account where id in ?1", nativeQuery = true)
    List<ChannelAccount> findByids(List<Long> ids);

    @Query(value = "select * from channel_account where name = ?1 and unique_name = ?2", nativeQuery = true)
    ChannelAccount findByName(String name, String uniqueGdt);
    @Query(value = "select * from channel_account where name = ?1 and unique_name = ?2 and third_account_code = ?3", nativeQuery = true)
    ChannelAccount findByNameAndThirdAccountCode(String name, String uniqueGdt,String ThirdAccountCode);

    @Query(value = "select * from channel_account where name = ?1 and create_account = ?2", nativeQuery = true)
    ChannelAccount findByNameAndAccount(String name, Long account);

    @Query(value = "select * from channel_account where third_account_code = ?1 and unique_name = 'gdt'", nativeQuery = true)
    ChannelAccount findByThirdAccountCode(String code);

    @Query(value = "select * from channel_account where third_account_code = ?1 and unique_name = ?2 ", nativeQuery = true)
    ChannelAccount findByThirdAccountCode(String code, String uniqueName);

    @Query(value = "select * from channel_account where name = ?1 and third_account_code = ?2 and status = 1 and root_account = ?3", nativeQuery = true)
    ChannelAccount findByUser(String userName, String code,Long rootAccount);

    @Query(value = "select * from channel_account where unique_name = ?1 and past_status = false", nativeQuery = true)
    List<ChannelAccount> findValidByUniqueName(String uniqueName);

    @Query(value = "select * from channel_account where unique_name = ?1 and past_status = false and status=1", nativeQuery = true)
    List<ChannelAccount> findValidEnableByUniqueName(String uniqueName);

    @Transactional
    @Modifying
    @Query(value = "update ChannelAccount set past_status = 0 where id = ?1")
    void invalidAccount(Long id);

    @Query(value = "select * from channel_account where root_account = ?1 and unique_name = 'gdt'", nativeQuery = true)
    List<ChannelAccount> findByRootAccount(Long rootAccount);

    @Query(value = "select * from channel_account where root_account = ?1 and channel = ?2 and status = 1", nativeQuery = true)
    List<ChannelAccount> findEnableByRootAccount(Long rootAccount, Long cid);


    List<ChannelAccount> findByAliasAndCreateAccount(String alias, Long createAccount);

    List<ChannelAccount> findByStatusTrueAndPastStatusFalse();

    @Query(value = "select * from channel_account where root_account = ?1 and unique_name = ?2", nativeQuery = true)
    List<ChannelAccount> findByRootAccount(Long rootAccount, String unique_name);

    @Query(value = "select * from channel_account where root_account = ?1 and unique_name = ?2 and status=1", nativeQuery = true)
    List<ChannelAccount> findByAbleRootAccount(Long rootAccount, String unique_name);

    @Query(value = "select count(*) from channel_account where alias = ?1", nativeQuery = true)
    int checkAlias(String alias);

    @Query(value = "select * from channel_account where alias = ?1", nativeQuery = true)
    ChannelAccount getByAlias(String alias);

    @Query(value = "SELECT * FROM channel_account WHERE id IN (SELECT link_user_id FROM channel_account_link_app WHERE  link_user_id IN (SELECT id FROM channel_account WHERE  channel = 5305 ))\n", nativeQuery = true)
    List<ChannelAccount> findAllLinkUser();

    @Modifying
    @Query(value = "update channel_account set status = false where STATUS=TRUE and create_account in (SELECT id  FROM account WHERE past_date < ?1)", nativeQuery = true)
    int ableExpired(String date15);


    @Query(value = "select * from channel_account where status is true and past_status is not true and unique_name ='gdt'",nativeQuery = true)
    List<ChannelAccount> findgdtStatusTrueAndNotPast();


    @Query(value = "select count(*) a  from channel_account where root_account = ?1 and channel = ?2 " +
            " union all select count(*) a from channel_account where root_account = ?1 and channel = ?3" +
            " union all select count(*) a from channel_account where root_account = ?1 and channel = ?4",nativeQuery = true)
    List<BigInteger> findChannelAcountCount(Long rootParent, String shma, String adwords, String gdt);

    @Query(value = "select * from channel_account where root_account = ?1 and channel = 514",nativeQuery = true)
    List<ChannelAccount> findValidByToutiao(Long rootAccount);
    @Modifying
    @Query(value = "delete from channel_account where root_account = ?1 and channel = 514)", nativeQuery = true)
    int deleteToutiaoByAccount(Long rootAccount);

    @Query(value = "select * from channel_account where channel = 514 and status is true",nativeQuery = true)
    List<ChannelAccount> findAllValidToutiao();

    @Query(value = "select * from channel_account where channel = 514 and status is true and role = 2",nativeQuery = true)
    List<ChannelAccount> findAllValidToutiaoManager();

    @Query(value = "select * from channel_account where channel = 6481 and status is true",nativeQuery = true)
    List<ChannelAccount> findAllValidBaidu();

    @Query(value = "select * from channel_account where channel = 2586 and status is true",nativeQuery = true)
    List<ChannelAccount> findAllValidKuaishou();

    @Query(value = "select * from channel_account where channel = 5306 and status is true",nativeQuery = true)
    List<ChannelAccount> findAllValidWeixinMP();

    @Query(value = "select * from channel_account where alias = ?1 and channel = ?2 and status is false",nativeQuery = true)
    ChannelAccount getByAliasAndChannel(String state, Long id);

    @Query(value = "select * from channel_account where alias = ?1 and channel = ?2 and status is true",nativeQuery = true)
    ChannelAccount getByAliasAndChannelTrue(String alias, long channel);

    List<ChannelAccount> findByAccessToken(String s);

    @Query(value = "SELECT DISTINCT access_token FROM channel_account WHERE channel = 514",nativeQuery = true)
    List<String> findToutiaoTokens();
}
