package com.reyun.repository;

import com.reyun.model.Channel;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;

@Transactional
public interface ChannelRepository extends JpaRepository<Channel, Long> {

    @Query(value = "select * from channel where type is not null", nativeQuery = true)
    List<Channel> findCoopIdChannel();

    @Query(value = "select * from channel where type is not null", nativeQuery = true)
    List<Channel> findByTypeIsNotNull();

    @Query(value = "select * from channel where del_flag is not true and type is not null and  case when 'Android' = ?1 then os_android = 1  when 'iOS' = ?1 then  os_ios = 1  when 'selfBuilt' = ?1 then  os_selfbuilt = 1 else os_web = 1 end ", nativeQuery = true)
    List<Channel> findCoopByAppType(String appType);

    @Query(value = " select c.* from channel c left join account a on c.account = a.id where a.root_parent = ?1 and c.type is null and  case when 'Android' = ?2 then c.os_android = 1   when 'iOS' = ?2 then  c.os_ios = 1  WHEN 'wx' = ?2 or 'wg' = ?2 THEN c.os_weixin = 1 else c.os_web = 1 end ",nativeQuery = true)
    List<Channel> findCreatedChannelByAppType(Long rootParentId, String appType);

    @Query(value = "select * from channel where name = ?1 and type is not null limit 1 ", nativeQuery = true)
    Channel findByNames(String name);

    @Query(value = "SELECT * from channel where type is not null ORDER BY orderindex asc", nativeQuery = true)
    List<Channel> findByTypeIsNotNullOrderByOrderindexAsc();

    Channel findById(Long id);

    @Query(value = "select * from channel where unique_name = ?1 and  type is not null", nativeQuery = true)
    Channel findByUniqueName(String uniqueName);

    Channel findByName(String name);

    @Query(value = "select * from channel where id in ?1 ", nativeQuery = true)
    List<Channel> findById(List<Long> ids);

    @Query(value = "select name from channel where id in ?1 ", nativeQuery = true)
    List<String> findNameByIds(List<Long> ids);

    List<Channel> findByIshttps(boolean ishttps);

    @Query(value = "select * from channel where imp_is_https = ?1 ", nativeQuery = true)
    List<Channel> findByImpIshttps(boolean impIshttps);

    @Query(value = "select * from channel where account in (select id from account where id = ?1 or parent = ?1 ) and del_flag is not true", nativeQuery = true)
    List<Channel> listAllByMainAccount(Long accountId);

    @Query(value = "select * from channel where account in (select id from account where id = ?1 or id = ?2 or parent = ?2 ) and del_flag is not true", nativeQuery = true)
    List<Channel> listAllByManager(Long accountId, Long parent);

    @Query(value = "select * from channel where id in (select distinct channel from data_auth where account = ?1 ) and del_flag is not true", nativeQuery = true)
    List<Channel> listChannelIdByDataAuth(Long accountId);

    @Query(value = "select * from channel where id in (select distinct channel from data_auth where account = ?1 and app = ?2 ) and del_flag is not true", nativeQuery = true)
    List<Channel> listChannelIdByDataAuth(Long accountId, Long appId);

    @Query(value = "select * from channel where id in (select distinct channel from data_auth where account = ?1 and app = ?2 ) ", nativeQuery = true)
    List<Channel> listChannelIdByDataAuthIgnoreDel(Long accountId, Long appId);

    @Query(value = "select * from channel where id in (select distinct channel from data_auth where account = ?1 and app = ?2) and del_flag is not true ", nativeQuery = true)
    List<Channel> listChannelByDataAuth(Long accountId, Long appId);

    @Query(value = "select distinct c.* from data_auth d join channel c on c.id=d.channel and d.account=?1 and c.type is null and (d.all_campaign is true or d.channel_permit is true)", nativeQuery = true)
    List<Channel> findByAccountWithPermission(Long accountid);

    @Query(value = "select * from channel where (create_account=?1 and del_flag is not true ) or type is not null ", nativeQuery = true)
    List<Channel> findChannelsByAccount(Long accountid);

    @Query(value = "select * from channel where create_account =?1 and del_flag is not true", nativeQuery = true)
    List<Channel> listOwnChannelByAccount(Long accountid);

    @Query(value = "select * from channel where create_account =?1 ", nativeQuery = true)
    List<Channel> listOwnChannelByAccountDeled(Long accountid);


	@Query(value = "select * from channel where create_account =?1",nativeQuery=true)
	List<Channel> listChannelByAccount(Long accountid);

	@Query(value = "select distinct b.* from campaign a join channel b on a.channel = b.id where a.app = ?1 ", nativeQuery = true)
	List<Channel> listCampaignChannel(Long appid);

    @Query(value = "select distinct b.* from campaign a join channel b on a.channel = b.id where a.app = ?2 and a.account = ?1 " +
            " and b.id not in(select distinct channel from data_auth where account = ?1 and app = ?2 )", nativeQuery = true)
    List<Channel> listSubAccountCampaignChannel(Long accountId, Long appId);



    @Query(value = "select distinct b.* from campaign a join channel b on a.channel = b.id where a.app = ?1", nativeQuery = true)
    List<Channel> listAllCampaignChannel(Long appid);

    @Query(value = "select distinct b.* from campaign a join channel b on a.channel = b.id where a.create_account = ?1 and  a.app = ?2", nativeQuery = true)
    List<Channel> listCampaignChannelByAccount(Long accountId, Long appid);

    @Query(value = "select distinct b.* from campaign a join channel b on a.channel = b.id where a.app = ?1 and b.name like %?2% ", nativeQuery = true)
    List<Channel> findLikeName(Long appid, String keyword);

   // @Query(value = "SELECT DISTINCT t1.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and (t1.type!='ry_coop' or t1.type is NULL) and t1.category = 'ADVERTISING' and t1.del_flag is not true and t2.id = ?1", nativeQuery = true)
    @Query(value = "SELECT DISTINCT t1.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and (t1.type!='ry_coop' or t1.type is NULL) and t1.category = 'ADVERTISING' and t1.del_flag is not true and t2.id = ?1  " +
        "union all SELECT DISTINCT t1.* FROM campaign t0 JOIN channel t1 ON t0.channel = t1.id JOIN app t2 ON t0.app = t2.id WHERE t0.del_flag is not true and t1.type = 'ry_coop' and  t1.onelink_open is not true and t1.category = 'ADVERTISING' and t1.del_flag is not true and t2.id = ?1", nativeQuery = true)
    List<Channel> listChannelFromOnelink(Long appid);

//    @Query(value = "select DISTINCT c.* from channel c right join data_auth d on c.id = d.channel where d.account = ?1 and d.app = ?2 and (d.all_campaign is true or d.channel_permit is true)", nativeQuery = true)
    @Query(value = "select c.* from data_auth a  left join channel c on a.channel = c.id where a.account = ?1  and a.app =?2  group by  a.channel", nativeQuery = true)
    List<Channel> listTotalAuthorizedChannel(Long accountId, Long appId);

    @Query(value = "select DISTINCT c.* from channel c right join data_auth d on c.id = d.channel where d.account = ?1 and (d.all_campaign is true or d.channel_permit is true) and c.type is not null Order By c.orderindex Asc ", nativeQuery = true)
    List<Channel> listAllAuthorizedChannelByCoop(Long accountId);

	@Query(value = "select * from channel where unique_name in ?1", nativeQuery = true)
	List<Channel> findByUniqueNames(List<String> uniqueNames);

    @Query(value = "select * from channel where category = ?1 and name = ?2 and del_flag is not true", nativeQuery = true)
    List<Channel> findByNameAndCategroy(String catagroy, String name);

    @Query(value = "select * from channel where name = ?1 and create_account in (select id from account where root_parent = ?2) ", nativeQuery = true)
    List<Channel> findByChName(String name, Long rootParent);

    @Query(value = "select * from channel where id in (select channel from campaign where id in (?1) )",nativeQuery = true)
    List<Channel> findChannelByCampaignIds(List<Long> campids);

    @Query(value = "select * from channel where id in ?1 ", nativeQuery = true)
    List<Channel> findByIdSet(Set<Long> idSet);

    @Query(value = "select * from (select a.id, a.name,case when (a.type is not null or a.type <>'') then a.type else 'custom' end type from channel a join (select channel from data_auth where account = ?2 and app = ?1 group by channel) b on a.id = b.channel ) c where c.type = ?3 ",nativeQuery = true)
    List<Object[]> findAuthChannelNames(Long appid, Long id, String type);

    @Query(value ="select name from channel where id in ?1",nativeQuery=true)
    List<String> findNamesById(List<String> duration_cid);
}
