package com.reyun.repository;

import com.reyun.model.DataAuth;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Transactional
public interface DataAuthRepository extends JpaRepository<DataAuth, Long> {

    @Query(value = "select * from data_auth where account in ?1", nativeQuery = true)
    List<DataAuth> findDataAuthByAccount(List<Long> account);

    @Query(value = "select * from data_auth where account = ?1", nativeQuery = true)
    List<DataAuth> findDataAuthByAccount(Long account);

    @Query(value = "select * from data_auth where account = ?1 and app=?2", nativeQuery = true)
    List<DataAuth> findDataAuthByAccountAndApp(Long account, Long app);

   // @Query(value = "select * from data_auth where account = ?1 and app=?2 and campaign in (select id from campaign where app=?2 and del_flag=?3)", nativeQuery = true)
    @Query(value= "select a.* from (select * from data_auth where account = ?1 and app = ?2 and campaign <> 0) a left join campaign c on a.campaign = c.campaignid where c.del_flag = ?3 union all select * from data_auth where account = ?1 and app = ?2 and all_campaign is true",nativeQuery = true)
    List<DataAuth> findDataAuthByAccountAndAppDelflag(Long account, Long app, Boolean delflag);

    @Query(value = "select * from data_auth where account = ?2 and app in (select app from auth where create_account in ?1 and account = ?2)", nativeQuery = true)
    List<DataAuth> findDataAuthByCreateAccount(List<Long> createAccount, Long channelAccount);

    @Transactional
    @Modifying
    @Query(value = "delete from data_auth where account = ?1", nativeQuery = true)
    void deleteAuthByAccount(Long account);

    @Query(value = "select * from data_auth where app = ?1 and channel =?2 and campaign =?3", nativeQuery = true)
    List<DataAuth> listDataAuthByAppChnCam(Long app, Long channel, Long campain);

    Iterable<DataAuth> findByApp(Long id);

    @Query(value = "select account from data_auth where app = ?1 and channel =?2 and campaign =?3", nativeQuery = true)
    List<Long> listAccountByAppChnCam(Long app, Long channel, Long campain);

    @Query(value = "select * from data_auth where channel =?1", nativeQuery = true)
    List<DataAuth> listDataAuthByChannel(Long channel);

    @Query(value = "select * from data_auth where account = ?1 and app = ?2 and all_campaign = 1", nativeQuery = true)
    List<DataAuth> listByAllCampaign(Long account, Long app);

    @Query(value = "select * from data_auth where account = ?1 and app = ?2 and all_campaign = 1 and channel = ?3 limit 1", nativeQuery = true)
    DataAuth getByAllCampaign(Long account, Long app,Long channel);

    @Query(value = "select DISTINCT(all_campaign) from data_auth where account =?1 and channel=?2", nativeQuery = true)
    List<Boolean> findDataAuthByAccountAndCid(Long account, Long cid);

    @Query(value = "select account from data_auth where app=?1 and campaign=0 and all_campaign=1 and channel=5305", nativeQuery = true)
    List<String> getDataAuthByAppAndAllCampaign(Long app);

    @Query(value = "select count(*) from data_auth where app=?1 and account = ?2 and all_campaign=0 and channel=5305 and  campaign in (?3)", nativeQuery = true)
    int checkAuth(Long appId, Long account,List<String> campaigns);

    @Query(value = "select * from data_auth where app = ?1 and channel = ?2 and account = ?3 and channel_permit IS NOT FALSE limit 1", nativeQuery = true)
    DataAuth getByAppAndChannelAndAccountAndPermit(Long app, Long channel, Long account);
}
