package com.reyun.repository;

import com.reyun.model.Campaign;
import com.reyun.model.PackageSubCampaign;
import com.reyun.model.SubCampaign;
import com.reyun.util.Constant;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.Date;
import java.util.List;

/**
 * Created by sunhao on 17/5/22.
 * desc:子活动DAO
 */
public interface PackageSubCampaignRepository extends JpaRepository<PackageSubCampaign,Long>{

    @Query(value = "select count(*) from package_sub_campaign where parent = ?1 and del_flag is false ",nativeQuery = true)
    BigInteger countSubCampaignByCampaign(Long campaignId);
    @Query(value = "select * from package_sub_campaign where parent in ?1 and del_flag is false ",nativeQuery = true)
    List<PackageSubCampaign> subCampaignInParent(List<Long> ids);
    @Query(value = "select * from package_sub_campaign where parent in ?1 and del_flag is true ",nativeQuery = true)
    List<PackageSubCampaign> subCampaignInParentAndDel(List<Long> ids);

    @Query(value = "select * from package_sub_campaign c where c.parent_campaignid = ?1 and name like %?2% ", nativeQuery = true)
    List<PackageSubCampaign> findLikeName(String parentCampaignid, String keyword);

    @Query(value = "select * from package_sub_campaign where id in ?1 and del_flag is false ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignById(List<Long> ids);

    @Query(value = "select * from package_sub_campaign where campaignid in ?1 and del_flag is false ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignByCampaignid(List<String> ids);


    @Query(value = "select name from package_sub_campaign where parent = ?1 ", nativeQuery = true)
    List<String> listAllSubCampaignNameByCampaign(Long campaignId);

    @Query(value = "select * from package_sub_campaign where parent = ?1 and del_flag is false ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignByCampaign(Long campaignId);

    @Query(value = "select * from package_sub_campaign where parent = ?1 and del_flag is true ", nativeQuery = true)
    List<PackageSubCampaign> listDisableByCampaign(Long campaignId);

    @Query(value = "select * from package_sub_campaign c where c.app=?1", nativeQuery = true)
    List<PackageSubCampaign> listAllCampByApp(Long app);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign c set c.awssuccess=?2 where c.id = ?1", nativeQuery = true)
    int updateAwsStatus(Long id, Boolean isSuccess);

    @Query(value = "select c.surl from package_sub_campaign c", nativeQuery = true)
    List<String> findAllSurl();

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign set del_flag = false, modify_account = ?2, modify_time = ?3 where id in ?1 and del_flag is true ", nativeQuery = true)
    int enableSubCampaign(List<Long> subCampaignIdList, Long modifyAccount, Date modifyDate);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign set del_flag = true , modify_account = ?2, modify_time = ?3 where id in ?1 and del_flag is false ", nativeQuery = true)
    int forbidenSubCampaign(List<Long> subCampaignIdList, Long modifyAccount, Date modifyDate);

    @Query(value = "select * from package_sub_campaign c where c.app = ?1 and del_flag is not true", nativeQuery = true)
    List<PackageSubCampaign> findAllByApp(Long app);

    @Query(value = "select * from package_sub_campaign c where c.app = ?1 and url = ?2 and del_flag is not true", nativeQuery = true)
    List<PackageSubCampaign> findAllByAppAndUrl(Long app, String url);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign c set c.url=?2 where c.app = ?1", nativeQuery = true)
    int updateDownloadUrl(Long app, String url);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign c set c.pack_channelid=?2 where c.parent = ?1", nativeQuery = true)
    int updatePackChannelid(Long campaign, String packchannel);

    @Query(value = "select count(1) from package_sub_campaign where parent = ?1 and del_flag = 0 and name = ?2 ", nativeQuery = true)
    int exist (Long parent,String subCamName);

    @Query(value = "select parent,count(*) from package_sub_campaign where name != '爱奇艺模糊归因' and parent in ?1 group by parent",nativeQuery = true)
    List<Object[]> countSubCampaignByCampaignByIds(List<Long> ids);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign set url_name = ?2 where id = ?1",nativeQuery = true)
    int modifyUrlName(Long campaignId, String name);


    @Query(value = "select * from package_sub_campaign where id in ?1 and del_flag is true ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignByDelAndId(List<Long> subCampaignList);

    @Query(value = "select * from package_sub_campaign where campaignid in ?1 and del_flag is false and parent_campaignid = ?2  ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignByCampaignidAndPrarent(List<String> idList, String parent_campaignid);


    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign set url = ?2,url_name=?3 where id in ?1",nativeQuery = true)
    int updataUlrWithIds(List<Long> campids, String new_url, String urlName);

    @Transactional
    @Modifying
    @Query(value = "update package_sub_campaign  set url_name=?2 where app = ?3 and  url = ?1",nativeQuery = true)
    int updateUrlName(String new_url, String urlName, Long appid);

    @Query(value = "select * from package_sub_campaign where id in ?1 ",nativeQuery = true)
    List<PackageSubCampaign> findById(List<Long> camp_ids);

    @Query(value = "select * from package_sub_campaign where campaignid in ?1 and del_flag = ?3 and parent_campaignid = ?2  ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignByCampaignidAndPrarentAndDelFlag(List<String> idList, String parent_campaignid,Boolean del_flag);


    @Query(value = "select * from package_sub_campaign where campaignid in ?1 and parent_campaignid = ?2  ", nativeQuery = true)
    List<PackageSubCampaign> listSubCampaignAllByCampaignidAndPrarent(List<String> idList, String parent_campaignid);

    @Query(value = "select * from package_sub_campaign where campaignid = ?1", nativeQuery = true)
    PackageSubCampaign findSubCampaignBycampaignid(String sonCampaignid);

    @Transactional
    @Modifying
    @Query(value = "delete from package_sub_campaign where parent = ?1 and name = '爱奇艺模糊归因'",nativeQuery = true)
    int deleteIqiyiByParent(Long id);

    @Query(value = "select * from package_sub_campaign where  url like %?2% and parent_campaignid = ?1  ", nativeQuery = true)
    List<PackageSubCampaign> findFilterDeeplinkByParentCampaignid(String parentCampaignid,String deeplinkHost);

    @Query(value = "select * from package_sub_campaign where app = ?1 and del_flag is false ", nativeQuery = true)
    List<PackageSubCampaign> subCampaignByApp(Long id);

    @Query(value = "select * from package_sub_campaign where parent = ?1",nativeQuery = true)
    List<PackageSubCampaign> listSubCampaign(Long id);
}
