package com.reyun.repository;

import com.reyun.model.RecycleChannelAccount;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * Created by sunhao on 18/5/17.
 * 回本渠道账号DAO
 */
public interface RecycleChannelAccountRepository extends JpaRepository<RecycleChannelAccount, Long> {

    @Query(value = "select * from recycle_channel_account where root_account = ?1 and status = ?2", nativeQuery = true)
    List<RecycleChannelAccount> findChannelAccountByAccount(Long rootParent, Boolean enable);

    @Query(value = "select * from recycle_channel_account where root_account = ?1 ", nativeQuery = true)
    List<RecycleChannelAccount> findChannelAccountByAccount(Long rootParent);

    @Query(value = "select * from recycle_channel_account where user_name = ?1 and channel_account_id = ?2 and cid=?3", nativeQuery = true)
    RecycleChannelAccount findChannelAccountByAccount(String userName, String channelAccountId, Long cid);

    @Query(value = "select * from recycle_channel_account where root_account = ?1 and cid = ?2 and status =1", nativeQuery = true)
    List<RecycleChannelAccount> findRecycleAccountByChannel(Long rootParent, Long cid);

    @Query(value = "select count(distinct unique_name) from recycle_channel_account where root_account = ?1 and status =1", nativeQuery = true)
    Long findAddedChannelNumber(Long rootParent);

    @Query(value = "select count(1) from recycle_channel_account where account = ?1 and user_name =?2 ", nativeQuery = true)
    Long findAccountidAndUserNameNumber(String accountid,String username);


    @Query(value = "select * from recycle_channel_account where root_account = ?1 and channel_alias =?2 and cid = ?3 ", nativeQuery = true)
    List<RecycleChannelAccount> findRootAccountidAndChannelAliasAndCid(long rootAccountId,String channelAlias, long cid);

    RecycleChannelAccount findByUserName(String name);

    RecycleChannelAccount findByAccount(long account);

    @Query(value = "select * from recycle_channel_account where root_account = ?1 and user_name =?2 and cid = ?3", nativeQuery = true)
    RecycleChannelAccount findByRootAccountIdAndUserNameAndCid(long rootAccountId, String username, long cid );

    @Query(value = "select * from recycle_channel_account where root_account = ?1 and unique_account =?2 and cid = ?3", nativeQuery = true)
    RecycleChannelAccount findByRootAccountIdAndUniqueAccountAndCid(long rootAccountId, String uniqueAccount, long cid );

    @Query(value = "select * from recycle_channel_account where unique_name = ?1 and status = 1 and valid_flag = 1 ", nativeQuery = true)
    List<RecycleChannelAccount> findByUniqueNameAndEnable(String uniqueName);

    @Query(value = "select * from recycle_channel_account where status = 1 and valid_flag = 0 and DATE_ADD(utime,INTERVAL 3 DAY) > NOW() ", nativeQuery = true)
    List<RecycleChannelAccount> findDisableAccount();

    @Query(value = "select * from recycle_channel_account where status = 1 and valid_flag = 1 ", nativeQuery = true)
    List<RecycleChannelAccount> findEnableAccount();

    @Transactional
    @Modifying
    @Query(value = "update recycle_channel_account set status = 1, utime = now() where root_account = ?1 and unique_account =?2 and cid = ?3 ", nativeQuery = true)
   int enableRecycleChannelAccount(long rootAccountId, String uniqueAccount, long cid );

    @Transactional
    @Modifying
    @Query(value = "update recycle_channel_account set status = 0, utime = now() where root_account = ?1 and unique_account =?2 and cid = ?3 ", nativeQuery = true)
    int disableRecycleChannelAccount(long rootAccountId, String uniqueAccount, long cid );

    @Query(value = "select DISTINCT cid from recycle_channel_account where root_account = ?1 and status =1", nativeQuery = true)
    List<BigInteger> findCidsByAccount(Long rootParent);

    @Query(value = "select * from recycle_channel_account where root_account = ?1 and status =1", nativeQuery = true)
    List<RecycleChannelAccount> findEnableAccount(Long rootParent);

    @Transactional
    @Modifying
    @Query(value = "update recycle_channel_account set status = 0, utime = now() where root_account = ?1 and cid = ?2 ", nativeQuery = true)
    int disablePastRecycleChannelAccount(long rootAccountId, long cid );

    @Query(value = "select * from recycle_channel_account where account=?1 and cid = ?2 and user_name=?3", nativeQuery = true)
    List<RecycleChannelAccount> findByAccountAndCidAndUserName(long account, long cid, String userName);
}


