package com.reyun.security.interceptor;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.security.RedisLoginStatusManager;
import com.reyun.security.RedisLoginStatusManagerImpl;
import com.reyun.security.TokenManager;
import com.reyun.security.annotation.AuthorizationMenu;
import com.reyun.service.AccountService;
import com.reyun.service.AuthService;
import com.reyun.service.ConfigParamService;
import com.reyun.service.LoginService;
import com.reyun.util.*;
import jdk.internal.dynalink.linker.LinkerServices;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

public class AuthorizationInterceptor extends HandlerInterceptorAdapter {
    private static final Logger logger = LoggerFactory.getLogger(AuthorizationInterceptor.class);

    /**
     * 存放登录用户模型Key的Request Key
     */
    public static final String REQUEST_CURRENT_KEY = "REQUEST_CURRENT_KEY";

    public static final String MENU_AUTH_PREKEY = ":AUTH:MENUS";

    //管理身份验证操作的对象
    @Autowired
    private TokenManager manager;
    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private AuthService authService;
    @Autowired
    private RedisLoginStatusManager redisLoginStatusManager;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ConfigParamService configParamService;
    @Autowired
    private LoginService loginService;


    @Autowired
    private AccountService accountService;

    //存放鉴权信息的Header名称，默认是Authorization
    public static String httpHeaderName = "Authorization";

    public static Map<String, String> reportAuthMap = new HashMap() {{
        put("", "");

    }};

    //鉴权信息的无用前缀，默认为空
    private String httpHeaderPrefix = "";

    public void setManager(TokenManager manager) {
        this.manager = manager;
    }

    public void setHttpHeaderName(String httpHeaderName) {
        AuthorizationInterceptor.httpHeaderName = httpHeaderName;
    }

    public void setHttpHeaderPrefix(String httpHeaderPrefix) {
        this.httpHeaderPrefix = httpHeaderPrefix;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        try {
            //如果不是映射到方法直接通过
            if (!(handler instanceof HandlerMethod)) {
                return true;
            }
            //放开登录,第三方下载,注册,短信验证码
            if (request.getRequestURL().indexOf("login/login") != -1
                    || request.getRequestURL().indexOf("login/loginweb") != -1
                    || request.getRequestURL().indexOf("login/getvalidcode") != -1
                    || request.getRequestURL().indexOf("login/getopenid") != -1
                    || request.getRequestURL().indexOf("external/download") != -1
                    || request.getRequestURL().indexOf("openapi/toutiao") != -1
                    || request.getRequestURL().indexOf("event/find/valuespecail") != -1
                    || request.getRequestURL().indexOf("findchannelcamp4io") != -1
                    || request.getRequestURI().startsWith("/api/reged")
                    || request.getRequestURI().startsWith("/api/demo")
                    || request.getRequestURI().startsWith("/api/messagecode")
                    || request.getRequestURI().startsWith("/api/report/all/home")
                    || request.getRequestURI().startsWith("/api/mng/app/findWxMes")
                    || request.getRequestURI().startsWith("/api/mng/recycle/accountlist")
                    || request.getRequestURI().startsWith("/api/mng/recycle/disableaccount")
                    || request.getRequestURI().startsWith("/api/mng/recycle/getloginlicense")
                    || request.getRequestURI().startsWith("/api/mng/recycle/addloginlicense")
                    || request.getRequestURI().startsWith("/api/mng/recycle/jrtt/cb")
                    || request.getRequestURI().startsWith("/api/mng/accountRestrict/apiflow")
                    || request.getRequestURI().startsWith("/api/reged/apiaccount")
                    || request.getRequestURI().startsWith("/api/mng/accountRestrict/mail/test")
                    || request.getRequestURI().endsWith("/api/ClickMonitor/click")
                    || request.getRequestURI().endsWith("/api/logo/dic")
                    || request.getRequestURI().endsWith("/api/adsAccount/bindUser")
                    || request.getRequestURI().endsWith("/api/adsAccount/updateCampaign")
                    || request.getRequestURI().endsWith("/api/adsAccount/findCampaign")
                    || request.getRequestURI().endsWith("/api/adsAccount/addCampaign")
                    || request.getRequestURI().endsWith("/api/adsAccount/findApp")
                    || request.getRequestURI().startsWith("/druid")
                    || request.getRequestURI().startsWith("/api/dingtalk/callback")
                    || request.getRequestURI().startsWith("/api/third/gdt")
                    || request.getRequestURI().startsWith("/api/server/check")
                    || request.getRequestURI().startsWith("/api/tkio/authorization_token")
                    || request.getRequestURI().startsWith("/api/tkio/refresh_token")
                    || request.getRequestURI().startsWith("/api/tkio/openapi")
                    || request.getRequestURI().startsWith("/api/app/createchannelwebeye")
                    || request.getRequestURI().startsWith("/api/tkio/report")
					){
    			return true;
    		}

    		HandlerMethod handlerMethod = (HandlerMethod) handler;
    		Method method = handlerMethod.getMethod();


            //从header中得到token
            String token = request.getHeader(httpHeaderName);
            token = StringUtil.isEmpty(token) ? getCookieToken(request.getCookies(), "TOKEN") : token;

            long start_time = System.currentTimeMillis();

            if (token != null && token.startsWith(httpHeaderPrefix) && token.length() > 0) {

                token = token.substring(httpHeaderPrefix.length());
                //验证token
                String key = manager.getKeyFromToken(token);
                //登陆状态
                boolean loginStatus = redisLoginStatusManager.getLoginStatusByKey(token);

                if (!StringUtil.isEmpty(key)) {

                    Account account = accountRepository.findOne(Long.parseLong(key));
                    //加上密码校验
                    if (account == null || !redisLoginStatusManager.validateLoginPassword(token, account.getPassword(), loginStatus)) {

                        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
                        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                        request.getSession().removeAttribute(Constant.CURRENT_ACCOUNT);
                        manager.delRelationshipByToken(token);
                        redisLoginStatusManager.deleteLoginStatus(token);
                        String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.USERNAME_LOGIN_EXPIRE));
                        writer.write(json);
                        writer.close();
                        return false;
                    }

                    boolean isVerify = true;
                    //2. 帐户过期验证
                    if (request.getRequestURL().indexOf("login/logout") == -1
                            && request.getRequestURL().indexOf("tip") == -1
                            && request.getRequestURL().indexOf("notice") == -1
                            && request.getRequestURL().indexOf("account/detail") == -1
                            && request.getRequestURL().indexOf("account/updatePwd") == -1
                            && request.getRequestURL().indexOf("account/updateBase") == -1) {

                        Account verifyAct = account;
                        if (!account.getIsSuperUser()) {
                            verifyAct = accountRepository.findOne(account.getRootParent());
                        }
                        if (verifyAct.getStatus() != 1
                                || (!StringUtil.isEmpty(verifyAct.getPastDate())
                                && DateUtil.compare_date(DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT), verifyAct.getPastDate()) == 1)) {
                            isVerify = false;
                        }

                        //渠道账号不判断过期或停用
                        if (account.getIsChannelPerson()) {
                            isVerify = true;
                        }

                        if (!isVerify) {
                            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
                            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                            request.getSession().removeAttribute(Constant.CURRENT_ACCOUNT);
                            manager.delRelationshipByToken(token);
                            redisLoginStatusManager.deleteLoginStatus(token);
                            String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.ACCOUNT_BUSINESS_EXPIRE));
                            writer.write(json);
                            writer.close();
                            return false;
                        }
                    }

                    //3. 子帐号权限验证

                    //如果token验证成功，将token对应的用户id存在request中，便于之后注入
                    //TODO 暂时用于解决下载时前端无法传递token的问题
                    Account sessionAct = (Account) request.getSession().getAttribute(Constant.CURRENT_ACCOUNT);
                    if (sessionAct == null || !account.getId().equals(sessionAct.getId())) {
                        Account sessionAccount = authService.getSubAccountAuthById(account.getId());
                        sessionAccount.setIsMasterLogin(loginStatus);

                        if ("demo@chinacfsc.com".equals(sessionAccount.getEmail())) {
                            sessionAccount.setId(982L);
                        }

                        request.getSession().setAttribute(Constant.CURRENT_ACCOUNT, sessionAccount);
                    }
                    //校验母账号删除app，子账号还能看的问题
                    if (needAppDelFilter(request.getRequestURL().toString())) {
                        String lastApp = getCookieToken(request.getCookies(), "ryioLastApp");
                        if (!StringUtil.isEmpty(lastApp)) {
                            App app = appRepository.findOne(Long.parseLong(lastApp));
                            if (app != null && app.getDelFlag()) {
                                Cookie cookie = new Cookie("ryioLastApp", null);
                                cookie.setPath("/");
                                cookie.setMaxAge(0);
                                response.addCookie(cookie);
                                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
                                response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                                String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.APP_DEL));
                                writer.write(json);
                                writer.close();
                                return false;
                            }

                            /**
                             *此处限制了渠道帐号，但是应该只限制此渠道帐号下的一个app即可
                             */
							/* else if (account.getIsChannelPerson() && authService.isOriginAccountPast(app.getAccount())) {
    							BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
    							response.setContentType(MediaType.APPLICATION_JSON_VALUE);
    							String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.AUTH_ACCOUNT_BUSINESS_EXPIRE));
    							writer.write(json);
    							writer.close();
    							return false;
    						}*/
                        }
                    }

                    String domain = accountService.getCookieData(request.getCookies(), "sys_domain");
                    //延长cookie
                    Cookie cookie = new Cookie("TOKEN", token);

                    if (StringUtils.isEmpty(domain)) {
                            domain = Constant.getDomainStr();
                    }

                    cookie.setDomain(domain);
                    cookie.setMaxAge(3600 * 24 * 7);
                    cookie.setPath("/");
                    response.addCookie(cookie);

                    // 校验用户权限
                    return userMenuAuthoriztion(account, request, response, handler);

                }
            }

            //如果验证token失败，并且方法注明了Authorization，返回401错误
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.USERNAME_LOGIN_EXPIRE));
            writer.write(json);
            writer.close();
            return false;

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("", e);

        }
        return false;
    }

    private boolean userMenuAuthoriztion(Account account, HttpServletRequest request, HttpServletResponse response, Object handler) {

        if ("demo@reyun.com".equals(account.getEmail())) {
            return true;
        }

        String authMenuKey = loginService.getAuthMeunCheckKey(account);
        boolean report_auth_check = redisLoginStatusManager.getAuthMenusByKey(RedisLoginStatusManagerImpl.SHOULD_AUTH_REPORT_NAMES, null);
        boolean menu_check = redisLoginStatusManager.getAuthMenusByKey(authMenuKey, null);
        loginService.getAuthMenusToRedis(account,!report_auth_check || !menu_check);

        if (redisLoginStatusManager.getAuthMenusByKey(authMenuKey, "not_setting")) {
            return true;
        }

        String authHashKey = null;
        StringBuffer requestUrl = request.getRequestURL();

        if (requestUrl.indexOf("/api/report") > 0) {
            String[] urls = requestUrl.toString().split("/");
            //报表接口 通过 URL控制权限
            authHashKey = urls[urls.length - 1];
            if (!redisLoginStatusManager.getAuthMenusByKey(RedisLoginStatusManagerImpl.SHOULD_AUTH_REPORT_NAMES, authHashKey)) {
                authHashKey = null;
            }
            if ("number".equals(authHashKey)) {
                authHashKey = null;
            }
        } else {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            Method method = handlerMethod.getMethod();
            AuthorizationMenu authorizationMenu = method.getAnnotation(AuthorizationMenu.class);
            if (authorizationMenu != null) {
                authHashKey = authorizationMenu.value();
            }
        }

        if (authHashKey != null && !redisLoginStatusManager.getAuthMenusByKey(authMenuKey, authHashKey)) {
            //拦截权限
            try {
                String checkResulJson = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.USERNAME_AUTH_ERRO));
                writeJsonData(response, checkResulJson);
                return false;

            } catch (Exception e) {
                logger.error("校验权限时返回值错误", e);
                return false;
            }

        } else {
            return true;
        }

    }


    private void writeJsonData(HttpServletResponse response, String json) throws IOException {
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(response.getOutputStream()));
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
//		String json = new ObjectMapper().writeValueAsString(ResultModel.ERROR(ResultStatus.USERNAME_LOGIN_EXPIRE));
        writer.write(json);
        writer.close();
    }


    private boolean needAuthFilter(String url) {
        if (url.indexOf("login") != -1
                || url.indexOf("logout") != -1
                || url.indexOf("external/download") != -1
                || url.startsWith("/api/reged")
                || url.startsWith("/api/demo")
                || url.indexOf("tip") != -1
                || url.indexOf("notice") != -1) {
            return false;
        }
        return true;
    }

    private boolean needAppDelFilter(String url) {
        if (url.indexOf("login") != -1
                || url.indexOf("logout") != -1
                || url.indexOf("external/download") != -1
                || url.startsWith("/api/reged")
                || url.startsWith("/api/demo")
                || url.startsWith("api/app")
                || url.startsWith("api/mng/auth")
                || url.startsWith("api/mng/channel")
                || url.indexOf("tip") != -1
                || url.indexOf("notice") != -1) {
            return false;
        }
        return true;
    }

    private String getCookieToken(Cookie[] cookies, String name) {
        String token = null;
        if (null != cookies) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals(name)) {
                    token = cookie.getValue();
                }
            }
        }
        return token;
    }


    public static void main(String[] args) {


        String ss = "http://localhost:8091/api/report/5487/campaigninfo_byds";


    }
}