package com.reyun.service;

import com.reyun.model.*;
import com.reyun.util.ResultModel;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;


public interface CampaignService {

    /**
     * 根据主键检索单条记录
     */
    Campaign4Web findById(Long id);

    Campaign findOneById(Long id);

    /**
     * 根据ids批量检索推广活动
     */
    List<Campaign> findById(Iterable<Long> campids);

    /**
     * 根据ids批量检索推广活动
     */
    List<Campaign> findAllById(Iterable<Long> campids);

    /**
     * 检索某一应用下某个渠道下的所有推广活动
     */
    List<Campaign> listCampaignByAppAndChannel(Long appid, Long channelid);

    /************************************修**改**操**作**********************************************************/
    /**
     * 创建单个活动
     */
    Campaign create(Long appid, Campaign resource);

    /**
     * 批量创建
     */
    List<Campaign> createBatch(Long appid, Iterable<String> surls, Campaign resource);

    /**
     * 修改推广活动
     */
    Campaign update(Account loginAccount, Long appId, Campaign resource);

    /**
     * 修改推广活动名称
     */
    int updateName(Long appid, Long id, String name);


    int updateUrl(Long app, String url);

    /**
     * 修改推广活动名称
     */
    Boolean updateCampaignName(Long appid, Long id, String name);

    /**
     * 删除推广活动(假删除)
     */
    Campaign delete(Long appid, Long id, Long loginActId);

    /**
     * 启用推广活动
     * created by sunhao 2017
     */
    Campaign enableCampaign(Long appid, Long canpaignId, Long loginActId);

    /**
     * 批量删除活动--抽离删除操作
     */
    List<Campaign> batchdelete(Long appid, Iterable<Long> campids, Long loginActId);

    /**
     * 批量启用活动
     */
    List<Campaign> batchEnable(Long appid, Iterable<Long> campids, Long loginActId);

    /****************************************短**链**服**务**********************************************************/
    /**
     * 生成单条短链
     */
    String makesurl(Long appid, Account loginAccount);

    /**
     * 批量生成短链
     */
    List<String> makesurls(Long appid, Account loginAccount, int genNum);


    /**
     * 批量导出短链
     */
    StringBuffer exportsurls(Long appid, Iterable<Long> campids, Long loginAccountId, String type, String lanType, String showurl);

    /**
     * 批量导出短链2列
     */
    StringBuffer exportsurls2column(Long appid, Iterable<Long> campids, Long loginAccountId, String lanType);


    /**
     * 根据appid列出推广活动
     */
    List<Campaign> listCampaignByApp(Long accountid);

    /**
     * 列出活动
     */
    List<Campaign> listCampaign(Long appid, Long loginAccountId);

    List<Campaign> listAllCampaignByDelflag(Long appid, Boolean delflag);

    /**
     * 列出活动
     */
    List<Campaign> listAllCampaign(Long appid, Long loginAccountId);

    Iterable<Campaign4Web> listAll(Long appid, Account loginAccount, String channelType);

    /**
     * 数据导出展示全活动列表
     *
     * @param appid
     * @param loginAccount
     * @param channelType
     * @return
     */
    Iterable<Campaign4Web> listAllAndDisable(Long appid, Account loginAccount, String channelType);

    Iterable<Campaign4Web> listAllDisable(Long appid, Account loginAccount, String channelType);

    /**
     * 名字获取活动
     */
    Campaign findByName(String name, Long app);


    /**
     * 之前不包括已经停用的，现在修改为包含已经停用的推广活动
     */
    Campaign findByNameIncludeDel(String name, Long app);

    /**
     * 名字获取活动
     */
    Campaign findPackChannelid(String channelid, Long app);

	/*//TODO 待实现downloadCampaign
	String downloadCampaign(HttpServletRequest request, Long appid, Integer downtype, HttpServletResponse response, Account loginAccount);*/

    /**
     * 查询所有未同步成功的推广活动列表
     */
    void doSyncSurl2Aws();


    List<Campaign> findByCampaignidList(List<String> campaignidList, Long app);

    List<Campaign> findAllByCampaignIdList(List<String> campaignIdList);


    Map<String, String> findCampaignMapWithDefault(Long appId, Long accountId);


    void updateCampaignAwsStatusByChannel(Long channelId);

    /**
     * 查询所有下载地址列表
     */
    List<CampaignUrl4Web> findUrlList(Long appid);

    /**
     * 把当前URL的推广活动和子活动修改为新URL
     */
    void updateUrlList(Long account, Long appid, String oldUrl, String newUrl);

    /**
     * 查询下载地址下的推广活动列表
     */
    List<CampaignUrl4Web> findCampaignByUrl(Long appid, String url);

    List<Campaign> findByKeyword(Account loginAccount, Long app, String keyword, Boolean del_flag);

    Iterable<Campaign4Web> listAuthCampaignByType(Account loginAccount, Long app, String channelType, boolean includeDisable);

    List<Campaign4Web> listAuthCampaignByType(Account loginAccount, Long appid, String type, Campaign campaign, Map data);

    List<Campaign4Web> listCampaignByLink(Iterable<Campaign4Web> iterable);

    List<Campaign4Web> listCampaignByPackage(Iterable<Campaign4Web> iterable);

    List<String> filterChannelAndAuthCampaignId(Long accountId, Long appId, String channelIdString);

    void doDisableCampaign2ddb();

    Boolean checkActiveCampaign(Long app, Long channel);

    Boolean checkActiveCampaignNew(Long app, Long channel);

    List<Campaign> listAllAndDisableLimit(Long appid, Account loginAccount, String name, Map jsond, String channelType);

    List<Object[]> listPartCampaign(Long appid);

    void deleteExpiredCampaign(String date15);

    Iterable<String> listAuthCampaignIds(Account account, Long appid, String custom, boolean b, String AppMarket);

    List<Campaign> listAllCampaignByDelflagAndCid(Long appid, Boolean del_flag, String cid);

    int modifyUrlName(Long compainId, String name, String type);

    Campaign findPackChannelidExist(String packchannelid, Long appid);

    void updateUrlListByCampin(Long id, Long appid, CampaignUrl4Web web);

    List<Campaign> listCampaignByAppAndChannelAll(Long appid, Long channelid, Account loginAccount);

    List<String> findThirdAccountByArails(String keyword, App account, String s);

    List<Object[]> listAllCampaignObject(List<String> val, Long app, Long account);

    /**
     * 功能描述:刷新微信MPToken
     *
     * @author liyin
     * @date 2019/9/29
     */
    void autoRefreshWeChatMPToken();

    void UpdateUrlByAppId(Long appid, String oldLinkId, String newLinkId);

    List<Campaign> findByAppIdAndLikeUrl(Long appid, String linkId);

    List<Map> getChannelsNameAndId(Long appid, Account loginAccount, String type);

    void createVivoParam(List<Campaign> result, Campaign resource);

    void doOnlineApp(Long appid);

    void doOnlineAppSyncDingDingAndEmailMSGTask();

    List<Campaign> findByKeyword(Account loginAccount, Long app, String keyword, Boolean del_flag, boolean subscribe);

    List<Campaign> findByCampaignidList(List<String> campaignidList, Long app, boolean subscribe);

    Campaign deleteCampaign(Long id, Campaign campaign, Long account);

    Integer getSulrOnServer(String surls);

    List<Campaign> findCampaignIdsAndSubscribesById(List<Long> authCampaignList);

    boolean checkPackChannle(Campaign resource, HttpServletRequest request);

    Map<String,Boolean> dropDownList(Long appid);

    void syncDeleteSubscribesSurlTask(Long appid);

    SurlModel transform(Campaign campaign, App app, Channel channel);
}
