package com.reyun.service;

import com.reyun.model.Account;
import com.reyun.model.Channel;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface ChannelService {

	/**
	 * id获取channel
	 *
	 * @param id
	 */
	Channel findoneById(Long id);

	/**
	 * 保存
	 *
	 * @param resource
	 * @return
	 */
	Channel save(Channel resource);

	/**
	 * 修改
	 *
	 * @param resource
	 * @return
	 */
	Channel update(Channel resource, Long accountid);

	/**
	 * channel管理--假删除--只是修改delFlag字段
	 *
	 * @param channel
	 */
	void delete(Channel channel, Long accountid);

	/**
	 * channel管理启用
	 *
	 * @param id
	 */
	Channel enable(long id, Long accountid);

	/**
	 * 渠道授权相关
	 *
	 * @param holderids
	 * @param appid
	 * @param channelid
	 * @param currentaccountid
	 */
	void syncChannelAuth(String holderids, Long appid, Long channelid, Long currentaccountid);

	/**
	 * 删除与该渠道相关的所有权限
	 *
	 * @param channelid
	 */
	void delChannelAuth(Long channelid);

	/**
	 * 修改渠道下关联商务的权限
	 *
	 * @param holderids
	 * @param appid
	 * @param channelid
	 * @param currentaccountid
	 */
	void updateChannelAuth(String holderids, Long appid, Long channelid, Long currentaccountid);

	/**
	 * 展示app下的channel信息
	 *
	 * @param accountid
	 * @return
	 */
	List<Channel> listChannelByAccount(Long accountid);

	/**
	 * 展示该账号下的自定义channel信息
	 *
	 * @param accountid
	 * @return
	 */
	List<Channel> listOwnChannelByCreateAccount(Long accountid);

	/**
	 * 展示该账号下的非自定义channel信息
	 *
	 *
	 * @return
	 */
	List<Channel> listUnOwnChannel(Account loginAccount, String delflg);
	/**
	 * 通过名儿查找channel
	 *
	 * @param name
	 * @return
	 */
	Channel valid(String name);

	/**
	 * 展示已有推广活动的渠道列表
	 *
	 * @param accountid
	 * @return
	 */
	List<Channel> listCampaignChannel(Long appid, Long accountid);

	List<Channel> listCampaignChannelAndDisable(Long appid, Long accountid);

	List<Channel> listChannelByIds(List<Long> ids);

    Map<String, String> findChannelMapWithDefault(Long appId, Long accountId);

    List<Channel> findCoopAndOwnChannel(Long account, String accountAll);


	/**
	 * channel管理--删除前验证
	 *
	 * @param cid
	 */
	int deleteValid(Long cid);

	List<Channel> findByKeyword(Account loginAccount, Long app, String keyword);

    List<Channel> listChannel4CreateCampaign(Long accountId, Long appId);

	/**
	 * 去掉合并的渠道
	 * @param accountId
	 * @param appId
	 * @param showDel
     * @param accountAll
     * @param filter
	 * @return
	 */
	List<Channel> listChannel4CreateCampaignShow(Account accountId, Long appId, String lanType, String showDel, String accountAll, Boolean filter);

    List<Channel> listAllCooperationChannel();

	Channel findByUniqueName(String uniqueName);

    List<Channel> findAllChannelByApp(Long appId);

	/**
	 * 创建渠道时发送邮件
	 */
	//void sendEmail(Account loginAccount, Channel channel);

    List<Channel> findByChannelName(String name, Account loginAccount);

	Channel findByUniqueName(String uniquename, Account loginAccount);

    List<Channel> listChannelByIds(Set<Long> idSet);

	List<Channel> filterChannelControl(Long rootParent,List<Channel> channels);
}
