package com.reyun.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.Account;
import com.reyun.model.AdsAccount;
import com.reyun.model.App;
import com.reyun.repository.*;
import com.reyun.service.AccountFlowRestrictService;
import com.reyun.service.AdsAccountService;
import com.reyun.service.AppService;
import com.reyun.service.AuthService;
import com.reyun.util.DateUtil;
import com.reyun.util.StringUtil;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;
import org.apache.http.protocol.HTTP;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class AdsAccountServiceImpl implements AdsAccountService {

    private static final Logger logger = LoggerFactory.getLogger(AdsAccountServiceImpl.class);

    private static final String APPLICATION_JSON = "application/json";

    private static final String CONTENT_TYPE_TEXT_JSON = "application/json;charset=UTF-8";
    //测试
    //private static final String URL = "http://pre-api.adsdesk.cn/adsdesk/api/open_api/tkio/tkioAppId";
    //线上
    private static final String URL = "http://api.adsdesk.cn/adsdesk/api/open_api/tkio/tkioAppId";
    private static final ArrayList<Long> cids = new ArrayList<>();
    static {
        cids.add(514L);// 今日头条
        cids.add(2586L);// 快手
    }

    @Autowired
    AccountFlowRestrictService accountFlowRestrictService;
    @Autowired
    private AppService appService;
    @Autowired
    private AdsAccountRepository adsAccountRepository;
    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private AuthService authService;
    @Autowired
    private DataAuthRepository dataAuthRepository;


    @Override
    public List<JSONObject> findApp(AdsAccount adsAccount) {

        List<App> listApp = appService.listAppByAccount(adsAccount.getAccount());
        Account one = accountRepository.findOne(adsAccount.getAccount());
        boolean isSubAccount = false;
        if (one.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())
                || one.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())
                || one.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

            isSubAccount = true;
        }
        List<JSONObject> apps = new ArrayList<>(listApp.size());
        for (App app : listApp) {
            if(app.getIsDebug())continue;
            JSONObject appsJson = new JSONObject(6);
            appsJson.put("app",app.getId());
            appsJson.put("appkey",app.getAppkey());
            appsJson.put("name",app.getName());
            appsJson.put("platform",app.getPlatform());
            appsJson.put("createTime",app.getCreateTime());

            JSONObject jb = new JSONObject(cids.size());

             for (Long cid : cids) {
                 if(isSubAccount){
                     //普通账号需要查询是否可以创建活动
                     jb.put(cid.toString(),dataAuthRepository.getByAppAndChannelAndAccountAndPermit(app.getId(),cid,one.getId())!=null);
                 }else{
                     jb.put(cid.toString(),true);
                 }
            }
            appsJson.put("addCampaign",jb);
            apps.add(appsJson);
        }
        return apps;
    }

    @Override
    public AdsAccount findByAccount(Long rootParent) {

        return adsAccountRepository.findByAccount(rootParent);
    }

    @Override
    public void updatePermissionById(Long id, int i) {
        adsAccountRepository.updatePermissionById(id,2);
    }

    @Override
    public String checkAccount(Account account) {
        if (!account.getRoleCategory().equals(5L)) {
            Boolean trackExhaust = accountFlowRestrictService.isTrackExhaust(account.getId());
            if (trackExhaust) {
                return "广告效果监控流量已经用尽！";
            }
        }
        String nowDate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);
        if (account.getIsSuperUser()) {

            //母账号
            if (account.getStatus() == 0) {
                //主账号禁用
                return "主账号禁用";

            } else if (account.getStatus() == -1 || account.getStatus() == -2 || account.getStatus() == -3) {
                //主账号还没通过审核
                return "主账号还没通过审核";
            }

            //主账号过期
            if (!StringUtil.isEmpty(account.getPastDate()) && DateUtil.compare_date(nowDate, account.getPastDate()) == 1) {
                return "主账号过期";
            }

        } else {

            //子账号
            if (account.getStatus() == 0) {
                // 子账号禁用
                return "子账号禁用";

            }else if(account.getStatus() == -1 || account.getStatus() == -2 || account.getStatus() == -3){
                return "主账号还没通过审核";
            } else if (!account.getRoleCategory().equals(5L)) {
                //获取根节点母账号
                Account parent = authService.findRootParentAccount(account.getId());

                if (parent.getStatus() == 0) {
                    //子账号所在主账号被禁用
                    return "子账号所在主账号被禁用";

                } else if (!StringUtil.isEmpty(parent.getPastDate()) && DateUtil.compare_date(nowDate, parent.getPastDate()) == 1) {
                    //子账号所在主账号过期
                    return "子账号所在主账号过期";
                }
            }

        }
        return "true";
    }

    public void sendAds(String json) throws Exception {
        ResponseHandler<String> responseHandler = new BasicResponseHandler();
        DefaultHttpClient httpClient = new DefaultHttpClient();
        HttpPost httpPost = new HttpPost(URL);
        httpPost.addHeader(HTTP.CONTENT_TYPE, APPLICATION_JSON);

        StringEntity se = new StringEntity(json);
        se.setContentType(CONTENT_TYPE_TEXT_JSON);
        se.setContentEncoding(new BasicHeader(HTTP.CONTENT_TYPE, APPLICATION_JSON));
        httpPost.setEntity(se);
        String execute = httpClient.execute(httpPost, responseHandler);
        logger.info(execute);
    }
}
