package com.reyun.service.impl;

import com.reyun.model.Account;
import com.reyun.model.AdvancedFunctionApply;
import com.reyun.model.AdvancedFunctionRenew;
import com.reyun.repository.AdvancedFunctionApplyRepository;
import com.reyun.repository.AdvancedFunctionRenewRepository;
import com.reyun.service.AdvancedFunctionApplyService;
import com.reyun.taskexecute.SendApplyEmailThread;
import com.reyun.util.Constant;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by sunhao on 18/5/17.
 * 高级功能service implements
 */
@Service
public class AdvancedFunctionApplyServiceImpl implements AdvancedFunctionApplyService {

    //申请待审核状态
    private final static Integer STATUS_APPLY = 0;
    //正常状态
    private final static Integer STATUS_NORMAL = 1;
    //续订状态
    private final static Integer STATUS_RENEW = 2;

    @Autowired
    private AdvancedFunctionApplyRepository advancedFunctionApplyRepository;

    @Autowired
    private AdvancedFunctionRenewRepository advancedFunctionRenewRepository;

    @Override
    public AdvancedFunctionApply addApply(Account account, AdvancedFunctionApply advancedFunctionApply) {

        AdvancedFunctionApply result = null;

        //查询联系方式是否申请过。
        AdvancedFunctionApply queryApply = advancedFunctionApplyRepository.findByRootAccount(account.getRootParent(),Constant.ADVANCED_FUNCTION_MENU_RECYCLE);

        if (null == queryApply) {

            advancedFunctionApply.setAccount(account.getId());
            advancedFunctionApply.setRootParent(account.getRootParent());

            advancedFunctionApply.setApplyDate(new Date());
            //未审核状态
            advancedFunctionApply.setStatus(STATUS_APPLY);

            result = advancedFunctionApplyRepository.save(advancedFunctionApply);

            //发送开通申请邮件
            if (null != result) {
                SendApplyEmailThread sendApplyEmailThread = new SendApplyEmailThread(account.getCompany(), account.getEmail(), result.getApplierContact(), false);
                sendApplyEmailThread.start();
            }
        }

        return result;
    }

    @Override
    public AdvancedFunctionApply listApplyByAccount(Account account) {

        return advancedFunctionApplyRepository.findByRootAccount(account.getRootParent(),Constant.ADVANCED_FUNCTION_MENU_RECYCLE);
    }

    @Override
    public AdvancedFunctionRenew addRenewApply(Account account, AdvancedFunctionRenew advancedFunctionRenew) {

        AdvancedFunctionRenew result = null;

        AdvancedFunctionApply advancedFunctionApply = advancedFunctionApplyRepository.findNormalByAccount(account.getRootParent());

        if (null != advancedFunctionApply) {
            result = new AdvancedFunctionRenew();

            result.setPhone(advancedFunctionRenew.getPhone());
            result.setApplyId(advancedFunctionApply.getId());
            result.setCreateDate(new Date());
            result.setContact(advancedFunctionRenew.getContact());
            result.setStatus(STATUS_APPLY);

            result = advancedFunctionRenewRepository.save(result);

            //修改申请状态为renew
            advancedFunctionApply.setStatus(STATUS_RENEW);
            advancedFunctionApply = advancedFunctionApplyRepository.save(advancedFunctionApply);

            //发送续订申请邮件
            if (null != result && STATUS_RENEW.equals(advancedFunctionApply.getStatus())) {
                SendApplyEmailThread sendApplyEmailThread = new SendApplyEmailThread(account.getCompany(), account.getEmail(), result.getPhone(), false);
                sendApplyEmailThread.start();
            }
        }

        return result;
    }
}
