package com.reyun.service.impl;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AdvertisingService;
import com.reyun.util.*;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.POIXMLDocument;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class AdvertisingServiceImpl implements AdvertisingService {

    private static Logger logger = LoggerFactory.getLogger(AdvertisingServiceImpl.class);

    @Autowired
    private AdvertisingPlatformRepository advertisingPlatformRepository;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private ConfigParamRepository configParamRepository;

    @Override
    public ResultStatus importDataWithExcel(String typeName, Long appid, MultipartFile file) {

        App app = appRepository.findOne(appid);

        InputStream stream = null;
        try {
            stream = file.getInputStream();
            if (stream == null) {
                return ResultStatus.FILE_TYPE_ERROR;
            }
            Workbook workbook = WorkbookFactory.create(stream);
            Sheet sheet = workbook.getSheetAt(0);

            if (sheet.getPhysicalNumberOfRows() > 2000) {
                return ResultStatus.EXPORT_PHYSICALNUMBER_LARGE;
            }

            Row row = sheet.getRow(0);
            AdvertisingPlatform platformData = advertisingPlatformRepository.findByPlatform(typeName);
            String[] keyNames = platformData.getTitleKey().split(",");
            String[] titleName = platformData.getTitleName().split(",");
            Map<String, String> keyNameMaps = new HashMap(keyNames.length);
            for (int t = 0, k = keyNames.length; t < k; t++) {
                keyNameMaps.put(titleName[t], keyNames[t]);
            }

            StringBuffer sql_value_part = new StringBuffer();
            String[] key_index_value = new String[row.getLastCellNum()];

            //校验表头
            for (int e = 0, w = row.getLastCellNum(); e < w; e++) {
                Cell cell = row.getCell(e);
                String title = cell.getStringCellValue().trim();

                if (keyNameMaps.containsKey(title)) {
                    if (sql_value_part.length() > 0) {
                        sql_value_part.append(",");
                    }
                    sql_value_part.append(keyNameMaps.get(title));
                    key_index_value[e] = keyNameMaps.get(title);
                }
            }

            if (keyNameMaps.size() != sql_value_part.toString().split(",").length) {
                //表头配置不正确
                return ResultStatus.SHEET_TITLE_ERRO;
            }

            //添加固定扩展字段 ds,plat_type,up_date,appkey
            sql_value_part.append(",plat_type").append(",plat_type_name").append(",up_date").append(",appid");

            //数据部分 sql
            StringBuffer sql_data_part = new StringBuffer();

            int lastNum = sheet.getLastRowNum();
            String update_time = new DateTime().toString("yyyy-MM-dd HH:mm:ss");
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

            for (int v = platformData.getStartIndex(); v <= lastNum; v++) {
                Row rowValue = sheet.getRow(v);
                boolean lineDataErro = false;

                StringBuffer sql_data_part_line = new StringBuffer();

                for (int e = 0, w = row.getLastCellNum(); e < w; e++) {
                    if (key_index_value[e] == null) {
                        continue;
                    }
                    Cell data_cell = rowValue.getCell(e);

                    if (data_cell == null) {
                        lineDataErro = true;
                        break;
                    }

                    if (data_cell.getCellTypeEnum().equals(CellType.STRING) && data_cell.getStringCellValue().trim().equals("")) {
                        lineDataErro = true;
                        break;
                    }


                    if (e > 0) {
                        sql_data_part_line.append(",");
                    }


                    if ("adv_id".equals(key_index_value[e])) {
                        if (data_cell.getCellTypeEnum().equals(CellType.STRING)) {
                            sql_data_part_line.append("'").append(data_cell.getStringCellValue()).append("'");
                        } else {
                            sql_data_part_line.append(new Double(data_cell.getNumericCellValue()).longValue());
                        }

                    } else if ("rate_click".equals(key_index_value[e])) {

                        if (data_cell.getCellTypeEnum().equals(CellType.STRING)) {
                            String data = data_cell.getStringCellValue();
                            if (data != null && data.endsWith("%")) {
                                //处理百分号
                                sql_data_part_line.append(data.replace("%", ""));
                            }
                        } else {
                            double value = new BigDecimal(data_cell.getNumericCellValue() * 100).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue();
                            sql_data_part_line.append(value);
                        }


                    } else if ("ds".equals(key_index_value[e])) {

                        if (data_cell.getCellTypeEnum().equals(CellType.STRING)) {
                            sql_data_part_line.append("'").append(data_cell.getStringCellValue()).append("'");
                        } else if (data_cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                            String value = data_cell.getNumericCellValue() + "";
                            value = value.substring(0, 9).replace(".", "");
                            DateTime yyyyMMdd = DateTimeFormat.forPattern("yyyyMMdd").parseDateTime(value);
                            sql_data_part_line.append("'")
                                    .append(yyyyMMdd.toString("yyyy-MM-dd")).append("'");

                        } else {
                            if (data_cell.getDateCellValue() == null) {
                                lineDataErro = true;
                                break;
                            }
                            sql_data_part_line.append("'")
                                    .append(format.format(data_cell.getDateCellValue())).append("'");
                        }

                    } else if (data_cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                        double data = data_cell.getNumericCellValue();
                        data = new BigDecimal(data).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue();
                        sql_data_part_line.append(data);
                    } else {
                        sql_data_part_line.append("'").append(data_cell.getStringCellValue()).append("'");
                    }

                }

                if (!lineDataErro) {

                    if (sql_data_part.length() > 0) {
                        sql_data_part.append(",");
                    }
                    sql_data_part.append("(")
                            .append(sql_data_part_line)
                            .append(",'").append(platformData.getType())
                            .append("','").append(platformData.getTypeName())
                            .append("','").append(update_time).append("','").append(app.getAppkey()).append("')");

                }
            }

            if (sql_data_part.length() > 0) {
                StringBuffer sql = new StringBuffer("replace into tkio_rpt_advertising_cash_day ( ");
                sql.append(sql_value_part).append(") values ");
                sql.append(sql_data_part);
                //插入数据
                TKIOdbUtil.newInstance().insert(sql.toString());
            }

        } catch (Exception e) {
            logger.error("流量变现导入数据错误", e);
        } finally {
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e) {
                    logger.error("close stream erro", e);
                }
            }
        }

        return null;
    }

    @Override
    public ResultStatus importDataWithCSV(String typeName, Long appid, MultipartFile file) {

        //处理csv导入
        List<String> dataLineAll = new ArrayList<>();
        ResultStatus status = readCsvFile(file, dataLineAll);
        if (status != null) {
            return status;
        }

        AdvertisingPlatform platformData = advertisingPlatformRepository.findByPlatform(typeName);

        String[] keyNames = platformData.getTitleKey().split(",");//需要用到的字段对应的表头
        String[] titleName = platformData.getTitleName().split(",");
        Map<String, String> keyNameMaps = new HashMap(keyNames.length);
        for (int t = 0, k = keyNames.length; t < k; t++) {
            keyNameMaps.put(titleName[t].trim(), keyNames[t].trim());
        }

        String[] lineFirst = platformData.getCsvTitle().split(",");//文件的表头

        String[] keysFlag = new String[lineFirst.length];

        int checkedNum = 0;

        StringBuffer sqlPartOne = new StringBuffer();
        for (int i = 0; i < lineFirst.length; i++) {
            String name = lineFirst[i].trim();
            if (keyNameMaps.containsKey(name)) {

                if (sqlPartOne.length() > 0) sqlPartOne.append(",");

                keysFlag[i] = keyNameMaps.get(name);
                sqlPartOne.append(keysFlag[i]);
                checkedNum++;
            }
        }

        App app = appRepository.findOne(appid);
        String updateTime = new DateTime().toString("yyyy-MM-dd HH:mm:ss");


        //添加固定扩展字段 ds,plat_type,up_date,appkey
        sqlPartOne.append(",plat_type").append(",plat_type_name").append(",up_date").append(",appid");


        if (checkedNum != keyNames.length) {
            return ResultStatus.SHEET_TITLE_ERRO;
        }

        StringBuffer sqlPartTwo = new StringBuffer();
        String[] appid_adid_adname = new String[4];
        String regex = "(\".*?),(.*?\")";
        for (int i = platformData.getStartIndex(); i < dataLineAll.size(); i++) {
            String[] lineItem = dataLineAll.get(i).replaceAll(regex, "$1$2").split(",");
            if (i > 1) sqlPartTwo.append(",");

            sqlPartTwo.append("(");

            for (int j = 0; j < lineItem.length; j++) {

                lineItem[j] = lineItem[j].trim();
                if (lineItem[j].startsWith("\"")) {
                    lineItem[j] = lineItem[j].replace("\"", "").trim();
                }
                if (keysFlag[j] != null) {
                    if (j > 0) sqlPartTwo.append(",");
                    if ("rate_click".equals(keysFlag[j])) {
                        lineItem[j] = lineItem[j].replace("%", "");
                    } else if ("adv_id".equals(keysFlag[j])) {
                        appid_adid_adname[0] = lineItem[j];
                    } else if ("adv_name".equals(keysFlag[j])) {
                        appid_adid_adname[1] = lineItem[j];
                    }


                    sqlPartTwo.append("'").append(lineItem[j]).append("'");

                }
            }

            sqlPartTwo.append(",'" + platformData.getType() + "'")
                    .append(",'" + platformData.getTypeName() + "'")
                    .append(",'" + updateTime + "'")
                    .append(",'" + app.getAppkey() + "')");


            //更新广告位名称
            updateAdName(app.getAppkey(), appid_adid_adname[0], platformData.getTypeName(), appid_adid_adname[1]);

        }

        if (sqlPartTwo.length() > 0) {
            StringBuffer sql = new StringBuffer();
            sql.append("replace into tkio_rpt_advertising_cash_day (")
                    .append(sqlPartOne).append(") values ").append(sqlPartTwo);

            //插入数据
            TKIOdbUtil.newInstance().insert(sql.toString());
        }


        return null;
    }

    @Override
    public ResultStatus importDataOnCSV(String typeName, Long appid, MultipartFile file) {
        //处理csv导入
        List<String> dataLineAll = new ArrayList<>();
        ResultStatus status = readCsvFile(file, dataLineAll);
        if (status != null) {
            return status;
        }

        StringBuffer sql = new StringBuffer();
        sql.append("ds,adv_id,adv_name,income_all,plat_type,plat_type_name,up_date,appid");
        AdvertisingPlatform platformData = advertisingPlatformRepository.findByPlatform(typeName);
        App app = appRepository.findOne(appid);
        String regex = "(\".*?),(.*?\")";
        String updateTime = new DateTime().toString("yyyy-MM-dd HH:mm:ss");

        StringBuffer sqlTwo = new StringBuffer();

        for (int i = 1, lines = dataLineAll.size(); i < lines; i++) {
            String[] lineItem = dataLineAll.get(i).replace("\"", "").replaceAll(regex, "$1$2").split("\t");
            DataModel dataModel = new DataModel();
            dataModel.setUpdateTime(updateTime);
            dataModel.setAppid(app.getAppkey());
            dataModel.setPlName(platformData.getTypeName());
            dataModel.setPlType(platformData.getType() + "");
            dataModel = fillDataModeInfo(dataModel, platformData, lineItem);
            //处理sql
            appendSQL(dataModel, sqlTwo);
            //更新广告位名称
            updateAdName(app.getAppkey(), dataModel.getAdid(), platformData.getTypeName(), dataModel.getAdName());
        }

        if (sqlTwo.length() > 0) {
            StringBuffer sqlMain = new StringBuffer();
            sqlMain.append("replace into tkio_rpt_advertising_cash_day (").append(sql).append(") values ").append(sqlTwo);

            //插入数据
            TKIOdbUtil.newInstance().insert(sqlMain.toString());
        }


        return null;
    }


    private DataModel fillDataModeInfo(DataModel dataModel, AdvertisingPlatform platformData, String[] lineItem) {

        if ("mintegral".equals(platformData.getTypeName())) {
            DateTime dateTime = DateTime.parse(lineItem[0], DateTimeFormat.forPattern("yyyyMMdd"));
            dataModel.setDs(dateTime.toString("yyyy-MM-dd"));
            String unit = lineItem[2];
            unit = unit.replaceFirst("-", "ry@@");
            String[] valueNameId = unit.split("ry@@");
            dataModel.setAdid(valueNameId[0].trim());
            dataModel.setAdName(valueNameId[1].trim());
            String income = lineItem[12];
            Double value = new Double(income.replace("$", ""));
            value = BigDecimal.valueOf(value * 6.6).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            dataModel.setInCome(value + "");

            return dataModel;

        }
        return dataModel;
    }

    private void appendSQL(DataModel model, StringBuffer sql) {

        if (sql != null && sql.length() > 0) {
            sql.append(",");
        }

        //ds,adv_id,adv_name,income_all,plat_type,plat_type_name,up_date,appid
        sql.append("('").append(model.getDs()).append("','")
                .append(model.getAdid()).append("','")
                .append(model.getAdName()).append("','")
                .append(model.getInCome()).append("','")
                .append(model.getPlType()).append("','")
                .append(model.getPlName()).append("','")
                .append(model.getUpdateTime()).append("','")
                .append(model.getAppid()).append("')");

    }


    private class DataModel {
        private String ds;
        private String adid;
        private String adName;
        private String inCome;
        private String plType;
        private String plName;
        private String updateTime;
        private String appid;

        public String getDs() {
            return ds;
        }

        public void setDs(String ds) {
            this.ds = ds;
        }

        public String getAdid() {
            return adid;
        }

        public void setAdid(String adid) {
            this.adid = adid;
        }

        public String getAdName() {
            return adName;
        }

        public void setAdName(String adName) {
            this.adName = adName;
        }

        public String getInCome() {
            return inCome;
        }

        public void setInCome(String inCome) {
            this.inCome = inCome;
        }

        public String getPlType() {
            return plType;
        }

        public void setPlType(String plType) {
            this.plType = plType;
        }

        public String getPlName() {
            return plName;
        }

        public void setPlName(String plName) {
            this.plName = plName;
        }

        public String getUpdateTime() {
            return updateTime;
        }

        public void setUpdateTime(String updateTime) {
            this.updateTime = updateTime;
        }

        public String getAppid() {
            return appid;
        }

        public void setAppid(String appid) {
            this.appid = appid;
        }
    }


    public ResultStatus readCsvFile(MultipartFile file, List<String> dataLineAll) {
        //处理csv导入
        if (file.isEmpty()) {
            return ResultStatus.FILE_EMPFFTY;
        }

        InputStream inputStream = null;
        InputStreamReader reader = null;
        BufferedReader bufferedReader = null;
        try {

            inputStream = file.getInputStream();
            reader = new InputStreamReader(inputStream, "UTF-8");
            bufferedReader = new BufferedReader(reader);

            String line;

            int sizeValue = 0;
            while ((line = bufferedReader.readLine()) != null) {

                if (line != null && line.startsWith(",,,,,,,")) {
                    break;
                }

                dataLineAll.add(line);
                sizeValue++;
                if (sizeValue > 2000) {
                    return ResultStatus.EXPORT_PHYSICALNUMBER_LARGE;
                }
            }

        } catch (Exception e) {

        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    logger.error("", e);
                }
            }
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    logger.error("", e);
                }
            }
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (IOException e) {
                    logger.error("", e);
                }
            }
        }
        return null;
    }

    @Deprecated
    public ResultStatus readExclFile(MultipartFile file, List<String> dataLineAll, int dsIndex) {
        InputStream stream = null;
        boolean lineDataErro = false;
        String fileName = file.getOriginalFilename();
        try {

            stream = new ByteArrayInputStream(file.getBytes());

            if (stream == null) {
                return ResultStatus.FILE_TYPE_ERROR;
            }
            Workbook workbook = null;

            if (fileName.endsWith(".xls")) {

                if (POIFSFileSystem.hasPOIFSHeader(stream)) {
                    workbook = new HSSFWorkbook(stream);
                }
                if (POIXMLDocument.hasOOXMLHeader(stream)) {
                    workbook = new XSSFWorkbook(OPCPackage.open(stream));
                }


            } else {
                workbook = WorkbookFactory.create(stream);
            }
            Sheet sheet = workbook.getSheetAt(0);

            if (sheet.getPhysicalNumberOfRows() > 2000) {
                return ResultStatus.EXPORT_PHYSICALNUMBER_LARGE;
            }

            int lastNum = sheet.getLastRowNum();

            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

            for (int v = 1; v <= lastNum; v++) {
                Row rowValue = sheet.getRow(v);

                StringBuffer dataLine = new StringBuffer();
                for (int e = 0, w = rowValue.getLastCellNum(); e < w; e++) {

                    Cell data_cell = rowValue.getCell(e);

                    if (dataLine.length() > 0) {
                        dataLine.append("\t");
                    }

                    if (e == dsIndex) {
                        if (data_cell.getCellTypeEnum().equals(CellType.STRING)) {
                            dataLine.append(data_cell.getStringCellValue()).append("'");
                        } else if (data_cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                            String value = data_cell.getNumericCellValue() + "";
                            value = value.substring(0, 9).replace(".", "");
                            DateTime yyyyMMdd = DateTimeFormat.forPattern("yyyyMMdd").parseDateTime(value);
                            dataLine.append("'")
                                    .append(yyyyMMdd.toString("yyyy-MM-dd")).append("'");

                        } else {
                            if (data_cell.getDateCellValue() == null) {
                                lineDataErro = true;
                                break;
                            }
                            dataLine.append("'")
                                    .append(format.format(data_cell.getDateCellValue())).append("'");
                        }
                    } else if (data_cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                        double data = data_cell.getNumericCellValue();
                        data = new BigDecimal(data).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue();
                        dataLine.append(data);
                    } else {
                        dataLine.append("'").append(data_cell.getStringCellValue()).append("'");
                    }
                }

                dataLineAll.add(dataLine.toString());

            }

            if (lineDataErro) {
                return ResultStatus.FILE_TYPE_ERROR;
            }

        } catch (Exception e) {
            //e.printStackTrace();

        }
        return null;
    }


    @Override
    public List getAdNamesList(String appid, String adType, String searchName, String cid) {

        //adType  adgroup_id  adplan_id   adcreative_id
        App app = appRepository.findOne(Long.parseLong(appid));
        String url = Constant.reportUrl + "/api/trackingio/advertisinginfo_group_plan_creative/" + app.getAppkey();
        Map conditions = new HashMap();
        conditions.put("appid", app.getAppkey());
        conditions.put("iscache", Constant.iscache);
        if (!StringUtils.isEmpty(adType)) {
            conditions.put(adType, "1");
        }
        if (!StringUtils.isEmpty(searchName)) {
            conditions.put("searchName", searchName);
        }
        if (!StringUtils.isEmpty(cid)) {
            conditions.put("cid", cid);
        }
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        JSONObject object = JSONObject.fromObject(responseJson);
        List values = (List) object.getJSONArray("val").get(0);
        List keys = object.getJSONArray("key");
        List result = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            Map data = new HashMap();
            data.put("value", values.get(i));
            data.put("key", keys.get(i));
            result.add(data);
        }
        return result;
    }

    @Override
    public List adChannels() {
        return JSONArray.fromObject(configParamRepository.findParamsByKey("adv_channels_settings"));
    }

    @Override
    public void saveAdRenameData(AdNameSetting adName) {
        String updateTime = new DateTime().toString("yyyy-MM-dd HH:mm:ss");
        StringBuffer sql = new StringBuffer();
        sql.append("replace into tkio_rpt_advertising_adrename ( appid, ad_id, adplatform, ad_name, ad_type, ad_type_name, up_date) values (");
        sql.append("'").append(adName.getAppid()).append("'")
                .append(",'").append(adName.getAdid()).append("'")
                .append(",'").append(adName.getPlatform()).append("'")
                .append(",'").append(adName.getAdName()).append("'")
                .append(",'").append(adName.getAdType()).append("'")
                .append(",'").append(adName.getAdTypeName()).append("'")
                .append(",'").append(updateTime).append("')");

        //插入数据
        TKIOdbUtil.newInstance().insert(sql.toString());
    }

    private void updateAdName(String appid, String adid, String adplatform, String adName) {
        String sql = "select * from tkio_rpt_advertising_adrename where appid ='" + appid + "' and ad_id ='" + adid + "' and adplatform ='" + adplatform + "'  limit 1";
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        String adidQuery = null;
        try {
            conn = TKIOdbUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                adidQuery = rs.getString("ad_id");
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }

        if (adidQuery == null) {
            sql = "insert into tkio_rpt_advertising_adrename (appid, ad_id, adplatform, ad_name) value ('" + appid + "','" + adid + "','" + adplatform + "','" + adName + "')";
        } else {
            sql = "update tkio_rpt_advertising_adrename set ad_name = '" + adName + "' where appid ='" + appid + "' and ad_id ='" + adid + "' and adplatform ='" + adplatform + "'";
        }

        TKIOdbUtil.newInstance().insert(sql);

    }

    @Override
    public List<String> advertiseridByReport(HttpServletRequest request, Account account) {

        Map<String, String> conditions = new HashMap<>();
        Collections.list(request.getParameterNames()).forEach(v -> {
            conditions.put(v, request.getParameter(v));
        });

        if (StringUtils.isEmpty(conditions.get("appid"))) {
            throw new TipException("app不存在");
        }

        String appKey = appRepository.findAppkeyById(Long.parseLong(conditions.get("appid")));
        if (appKey == null) {
            throw new TipException("app不存在");
        }
        conditions.put("appid", appKey);
        ReportListResult reportListResult = null;
        List<String> itmeList = new ArrayList<>();
        try {
            String url4id = Constant.reportUrl + "/api/trackingio/advertisinginfo_byadvertiserid_list/1";
            String idItemsVale = HttpClientUtil.doHttpPostRequest(url4id, "trackingio", conditions);
            if (conditions.containsKey("api2sql")) {
                itmeList.add(idItemsVale);
                return itmeList;
            }
            reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(idItemsVale, ReportListResult.class);

        } catch (Exception e) {
            logger.error("获取广告投放->投放账号错误", e);
        }

        if (reportListResult != null && reportListResult.getVal() != null) {
            reportListResult.getVal().forEach(v -> itmeList.add(v.get("advertiserid") + ""));
        }

        return itmeList;

    }


    public static void main(String[] args) {
//        String t = "\"world\"";
        /*String t = "2020-05-20,Banner,火力飞机(iOS),1110457966\t,banner,1071710264131232\t,-,\"23,955\",\"3,039\",405.59,16.93,12.69%";
        String p = "(?<=\").*?(?=\")";
        Pattern P = Pattern.compile(p);
        Matcher matcher1 = P.matcher(t);*/
//        if (matcher1.find()) {
//            System.out.println(matcher1.group(0).replaceAll(p, "$1"));
//        }

        String regex = "(\".*?),(.*?\")";
//        String regex= "\"(.*?)\"";
//        System.out.println(t.replaceAll(regex, "$1$2"));

        // DATA,PLACEMENT ID,PLACEMENT NAME,REVENUE

        //System.out.println(new DateTime("20210112").toString("yyyy-MM-dd"));
        /*DateTime dateTime = DateTime.parse("20210112", DateTimeFormat.forPattern("yyyyMMdd"));
        System.out.println(dateTime.toString("yyyy-MM-dd"));


        String unit = "373465 - ChaPing_10USD/65RMB\n";
        unit = unit.replaceFirst("-", "ry@@");
        String[] valueNameId = unit.split("ry@@");

        System.out.println(valueNameId[0] + "=====" + valueNameId[1]);*/

        String a = "\"20210112\"\t\"Banner(246507)\"\t\"365506 - Banner_all\"\t\"SDK Banner\"\t\"9\"\t\"9\"\t\"100%\"\t\"0\"\t\"0\"\t\"0%\"\t\"$0.00\"\t\"$0.00\"\t\"$0.00\"";

        System.out.println(a.replace("\"", ""));
    }
}
