package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.dic.ChannelEnumType;
import com.reyun.model.App;
import com.reyun.model.ChannelAccountLinkApp;
import com.reyun.model.ChannelClassMapConfig;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ChannelClassMapConfigRepository;
import com.reyun.repository.ChannnelAccountLinkAppRepository;
import com.reyun.service.AdwordsService;
import com.reyun.util.HttpClientUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * description:
 * Adwords接口对接
 *
 * @author nolan
 * @date 18/10/2017
 */
@Service
public class AdwordsServiceImpl
        implements AdwordsService
{
    private Logger logger = LoggerFactory.getLogger(AdwordsServiceImpl.class);
    @Autowired
    private ChannelClassMapConfigRepository channelClassMapConfigRepository;
    @Autowired
    private AppRepository appRepository;


    @Autowired
    private ChannnelAccountLinkAppRepository channnelAccountLinkAppRepository;

    @Override
    public boolean isValid(String linkId,String appType)
    {
        ChannelClassMapConfig channelClassMapConfig = channelClassMapConfigRepository.findByUniqueName(ChannelEnumType.ADWORDS.getUniqueName());
        if (channelClassMapConfig == null) {
            logger.error("adwords组件未配置");
            return false;
        }

        boolean result = false;
        String idType;
        if(appType.toLowerCase().equals("ios")){
            idType = "idfa";
        }else{
            idType = "advertisingid";
        }

        //create by 20180725--此处调用第三方接口校验linkId失效（时间戳固定，导致校验返回一直都是失败，而下方取数有问题一直返回true）
        String url = "http://www.googleadservices.com/pagead/conversion/app/1.0?dev_token=" + channelClassMapConfig.getDevToken()
                + "&link_id=" + linkId + "&app_event_type=first_open&rdid=3A2288B6-B103-43AF-8F73-B66CEFFC30A5&id_type="
                +idType+"&lat=0&app_version=3.31.1&os_version=10.3.3&sdk_version=3.0.4&timestamp="+new Date().getTime()/1000;
        //------------------------end
        String res = HttpClientUtil.doHttpPostRequest(url, "");
        try {
            Map resMap = new ObjectMapper().readValue(res, Map.class);
            List errors = (List) resMap.get("errors");
            logger.info("first_open api url :"+url);
            logger.info("first_open api response :"+resMap.toString());
            result = errors == null || errors.size() == 0;
            if(!result){
                logger.info("记录linkedId校验结果 linkedId:{} res:{}",linkId , res);
            }
            return result;
        }
        catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public String isValid(String linkId, String what,String appType)
    {
        ChannelClassMapConfig channelClassMapConfig = channelClassMapConfigRepository.findByUniqueName(ChannelEnumType.ADWORDS.getUniqueName());
        if (channelClassMapConfig == null) {
            logger.error("adwords组件未配置");
            return "adwords组件未配置";
        }
        String idType;
        if(appType.toLowerCase().equals("ios")){
            idType = "idfa";
        }else{
            idType = "advertisingid";
        }
        String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
        String url = "http://www.googleadservices.com/pagead/conversion/app/1.0?dev_token=" + channelClassMapConfig.getDevToken() + "&link_id=" + linkId + "&app_event_type=custom&app_event_name="+what+"&rdid=45F38BD4-64DA-485E-BAFD-D1ECEBBCAF18&id_type="+idType+"&lat=0&app_version=3.31.1&os_version=10.3.3&sdk_version=3.0.4&timestamp=" + timestamp;
        if (what.equals("payment")) {
            url = "http://www.googleadservices.com/pagead/conversion/app/1.0?dev_token="+channelClassMapConfig.getDevToken()+"&link_id=" + linkId + "&app_event_type=in_app_purchase&rdid=0F7AB11F-DA50-498E-B225-21AC1977A85D&id_type="+idType+"&lat=0&app_version=1.2.4&os_version=9.3.2&sdk_version=1.9.5r6&timestamp=" + timestamp + ".123456&value=1.99&currency_code=USD";
        }
        String res = HttpClientUtil.doHttpPostRequest(url, "");

        try {
            Map resMap = new ObjectMapper().readValue(res, Map.class);
            List errors = (List) resMap.get("errors");
            logger.info("custom api url :"+url + ".\n" + "custom api response :"+resMap.toString());
            return errors.toString();
        }
        catch (IOException e) {
            logger.error(e.getMessage());
            return "发送请求失败,"+e.getMessage();
        }
    }

    @Override
    public int findLinkIdCount(String userActionSetId) {
        BigDecimal bigDecimal = channnelAccountLinkAppRepository.findCountByLinkId(userActionSetId);
        return bigDecimal == null ? 0 : bigDecimal.intValue();
    }
}
