package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.collect.Maps;
import com.reyun.controller.manage.MngCampaignController;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.ReportEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.*;
import com.reyun.util.*;
import com.reyun.util.NoSQLDBUtil.Result;

import net.sf.json.JSONObject;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AppServiceImpl implements AppService {

    @Autowired
    AppRepository appRepository;

    @Autowired
    CategoryRepository categoryRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    CampaignGroupRepository campaignGroupRepository;

    @Autowired
    CallbackCustomizeRepository callbackCustomizeRepository;

    @Autowired
    AuthRepository authRepository;

    @Autowired
    DataAuthRepository dataAuthRepository;

    @Autowired
    CamCamgroupMapRepository camCamgroupMapRepository;

    @Autowired
    AuthService authService;

    @Autowired
    SubCampaignRepository subCampaignRepository;

    @Autowired
    INoSQLDBService iNoSQLDBService;

    @Autowired
    PackageSubCampaignRepository packageSubCampaignRepository;

    @Autowired
    ConfigParamService configParamService;

    @Autowired
    AccountFlowRestrictServiceImpl accountFlowRestrictService;

    @Autowired
    private IAppTagService iAppTagService;

    @Autowired
    private IDicService iDicService;

    @Autowired
    private PackageSubCampaignService packageSubCampaignService;

    @Autowired
    private CampaignService campaignService;

    @Autowired
    private WxAppRepository wxAppRepository;

    @Autowired
    GdtPlanRepository gdtPlanRepository;

    @Autowired
    private AppTipRepository appTipRepository;

    @Autowired
    private ChannelWangyanRepository channelWangyanRepository;
    @Autowired
    private CampaignWangyanRepository campaignWangyanRepository;
    @Autowired
    private ChannelRepository channelRepository;

    protected Logger logger = LoggerFactory.getLogger(AppServiceImpl.class);

    @Override
    public List<App> listAuthAppByAccount(Account account, Long subAccountId) {

        Account subAccount = accountRepository.findOne(subAccountId);

        if (subAccount.getIsChannelPerson()) {

            //查询母账号下面所有授权的数据
            List<App> apps = appRepository.findAuthAppByRootParentOnlyApp(subAccountId, account.getRootParent());
            return apps;

        } else {

            return appRepository.findAuthApp(subAccountId);
        }
    }

    public App delete(final Long id, final Long account) {
        App app = appRepository.findOne(id);
        app.setModifyAccount(account + "");
        app.setModifyTime(new Date());
        app.setDelFlag(true);
        appRepository.save(app);

        //后面的APP顺序依次提前
        Account one = accountRepository.findOne(account);
        List<App> appList = appRepository.listAppsAfter(one.getRootParent(), app.getSort());
        if (ValidateUtil.isValid(appList)) {
            List<App> list = new ArrayList<>();
            for (App a : appList) {
                a.setSort(a.getSort() - 1);
                list.add(a);
            }
            appRepository.save(list);
        }

        //删除APP在ddb中的状态
        if (Constant.awsneed) {
            iNoSQLDBService.deleteAppSdkDebugItem(app.getAppkey());

            if (!StringUtils.isEmpty(app.getBundleid())) {
                // 删除bundleid 在DDB中的同步
                iNoSQLDBService.sycDDbByBundleId(app, false);
            }

        }

        // 修改已授权子账号对该App的可见状态
        authRepository.updateStatusByRootApp(id, false);

        //停用活动
        //子活动停用
        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.subCampaignByApp(app.getId());
        String pkgCampaignIds = "";
        if (!CollectionUtils.isEmpty(packageSubCampaigns)) {
            for (PackageSubCampaign packageSubCampaign : packageSubCampaigns) {
                pkgCampaignIds = pkgCampaignIds + packageSubCampaign.getId() + ",";
            }
            packageSubCampaignService.forbiden(pkgCampaignIds.substring(0, pkgCampaignIds.length() - 1), app.getAccount());
        }

        //停用活动
        List<Campaign> campaigns = campaignRepository.findByAppId(app.getId());
        for (Campaign campaign : campaigns) {
            campaignService.deleteCampaign(app.getId(), campaign, app.getAccount());
        }

        return app;
    }

    public App delete_Old(final Long id, final Long account) {
        App app = appRepository.findOne(id);
        app.setModifyAccount(account + "");
        app.setModifyTime(new Date());
        app.setDelFlag(true);
        appRepository.save(app);

        //后面的APP顺序依次提前
        Account one = accountRepository.findOne(account);
        List<App> appList = appRepository.listAppsAfter(one.getRootParent(), app.getSort());
        if (ValidateUtil.isValid(appList)) {
//            List<App> list = new ArrayList<>();
//            for (App a : appList) {
//                a.setSort(a.getSort() - 1);
//                list.add(a);
//            }
//            appRepository.save(list);
        }

        //删除APP在ddb中的状态
        if (Constant.awsneed) {
            iNoSQLDBService.deleteAppSdkDebugItem(app.getAppkey());
        }

        Thread alterRelationThread = new Thread() {
            @Override
            public void run() {

                //回调列表
                Iterable<CallbackCustomize> callbackList = callbackCustomizeRepository.findByApp(id);
                if (callbackList != null) {
                    for (CallbackCustomize callbackCustomize : callbackList) {
                        callbackCustomize.setDelFlag(true);
                        callbackCustomize.setModifyAccount(account + "");
                        callbackCustomize.setModifyTime(new Date());
                    }
                    callbackCustomizeRepository.save(callbackList);
                }

                Iterable<DataAuth> dataAuthList = dataAuthRepository.findByApp(id);
                if (dataAuthList != null)
                    dataAuthRepository.delete(dataAuthList);

                Iterable<Auth> authList = authRepository.findByApp(id);
                if (authList != null)
                    authRepository.delete(authList);

                Iterable<Campaign> campaignList = campaignRepository.findByApp(id);
                if (campaignList != null) {
                    for (Campaign campaign : campaignList) {
                        campaign.setDelFlag(true);
                        campaign.setModifyAccount(account);
                        campaign.setModifyTime(new Date());
                    }
                    campaignRepository.save(campaignList);
                }

                Iterable<CampaignGroup> campaignGroupList = campaignGroupRepository.findByApp(id);
                if (campaignGroupList != null) {
                    for (CampaignGroup campaignGroup : campaignGroupList) {
                        campaignGroup.setDelFlag(true);
                        campaignGroup.setModifyAccount(account);
                        campaignGroup.setModifyTime(new Date());
                    }
                    campaignGroupRepository.save(campaignGroupList);
                }

                List<CamCamgroupMap> camCamgroupMapList = null;
                if (campaignList != null) {
                    List<Long> list = null;
                    for (Campaign campaign : campaignList) {
                        list = new ArrayList<Long>();
                        list.add(campaign.getId());
                    }
                    if (list != null)
                        camCamgroupMapList = camCamgroupMapRepository.listAllByCampaignIds(list);
                }
                List<CamCamgroupMap> listMaps = null;
                if (campaignGroupList != null) {
                    List<Long> list = null;
                    for (CampaignGroup campaignGroup : campaignGroupList) {
                        list = new ArrayList<Long>();
                        list.add(campaignGroup.getId());
                    }
                    if (list != null)
                        listMaps = camCamgroupMapRepository.listAllByCampaignGroupIds(list);
                }

                if (listMaps != null) {
                    if (camCamgroupMapList != null) {
                        for (CamCamgroupMap camCamgroupMap : camCamgroupMapList) {
                            if (!listMaps.contains(camCamgroupMap)) {
                                listMaps.add(camCamgroupMap);
                            }
                        }
                    }
                    camCamgroupMapRepository.delete(listMaps);
                } else {
                    if (camCamgroupMapList != null) {
                        camCamgroupMapRepository.delete(camCamgroupMapList);
                    }
                }
            }
        };
        alterRelationThread.setName("alterAppRelationThread");
        alterRelationThread.start();

        //TODO 删除应用时，考虑新标签如何进行删除
        return app;
    }

    @Override
    public List<App> listAppByAccount(Long accountId) {

        List<App> appList = new ArrayList<>();

        Account oneAccount = accountRepository.findOne(accountId);

        if ("demo@reyun.com".equals(oneAccount.getEmail())) {
            //demo账号默认显示 自定义权限app
            List<Long> appIdList = authService.findAuthAppListByAccount(oneAccount.getId());
            if (!CollectionUtils.isEmpty(appIdList)) {
                appList = appRepository.findByIds(appIdList);
            }

        } else if (oneAccount.getIsSuperUser() || oneAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {
            //母账号,或者管理员创建的APP
            accountId = oneAccount.getIsSuperUser() ? accountId : oneAccount.getParent();
            appList = appRepository.listAppByRootAccount(accountId);

            if (accountId.equals(6380L)) {
                List<App> keyNext = appRepository.findByAccount(17108L);
                if (keyNext != null) {
                    appList.addAll(keyNext);
                }
            }
        } else {
            //自定义角色,自定义权限
            List<Long> appIdList = authService.findAuthAppListByAccount(oneAccount.getId());
            if (!CollectionUtils.isEmpty(appIdList)) {
                appList = appRepository.findByIds(appIdList);
            }
        }
        HashMap<String, Boolean> TrackAndIoBothExhaustMap = null;
        if (oneAccount.getRoleCategory().equals(5L)) {
            TrackAndIoBothExhaustMap = accountFlowRestrictService.getTrackExhaust(appList);
        }

        for (App app : appList) {
            //String gameCategory=appRepository.findGameCategoryByAppID(app.getId());
            String appGenreName = appRepository.findAppGenreNameByAppID(app.getId());
            //app.setGameCategory(gameCategory);
            app.setAppGenreName(appGenreName);

            /**
             * 判断app所属的主张号是否过期或者流量用尽
             */
            if (oneAccount.getRoleCategory().equals(5L)) {
                Account rootAccount = accountRepository.getRootAccountByCreateAccount(app.getCreateAccount());
                boolean isMainAccountExpire = false;
                if (DateUtil.compareDate(DateUtil.getCurrentDateStr(), rootAccount.getPastDate()) == 1) {
                    isMainAccountExpire = true;
                }
                app.setTrackAndIoBothExhaust(TrackAndIoBothExhaustMap.get(app.getCreateAccount()));
                app.setMainAccountExpire(isMainAccountExpire);
            }
        }
        return appList;
    }

    @Override
    public List<App> listAppByAccountForMobile(Long accountId) throws IOException {

        //获取LIST
        List<App> appList = listAppByAccount(accountId);

        if (!CollectionUtils.isEmpty(appList)) {

            //构建条件
            Map<String, String> conditions = Maps.newHashMap();
            conditions.put("appid", Joiner.on(",").join(Iterables.transform(appList, new Function<App, String>() {
                @Override
                public String apply(App app) {
                    return app.getAppkey();
                }
            })));
            conditions.put("datatype", ReportEnumType.LIST.getCode());

            //获取注册数据  请求报表系统
            final String url = Constant.reportUrl + "/api/trackingio/app_install_register_byappids/0";
            String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
            ReportListResult reportListResult = new ObjectMapper().readValue(responseJson, ReportListResult.class);


            Map<String, App> appMap = Maps.uniqueIndex(appList, new Function<App, String>() {
                @Override
                public String apply(App app) {
                    app.setInstallNum(0L);
                    app.setRegisterNum(0L);
                    return app.getAppkey();
                }
            });

            //合并注册数据和APP数据
            if (!CollectionUtils.isEmpty(reportListResult.getVal())) {

                for (Map<String, Object> map : reportListResult.getVal()) {

                    String appKey = (String) map.get("appid");
                    App app = appMap.get(appKey);
                    if (!StringUtil.isEmpty(appKey) && null != app) {
                        app.setInstallNum(new Long((Integer) map.get("cum_num_install")));
                        app.setRegisterNum(new Long((Integer) map.get("cum_dupnum_reged_mon")));
                    }
                }
            }
        }

        return appList;
    }

    @Override
    public App findById(Long id) {
        App one = appRepository.findOne(id);
        one.setAppTagList(iAppTagService.findByApp(id));
        return one;
    }

    @Override
    public App create(App resource, Long accountId) {

        //校验bundleid 全局唯一
        if (checkBundleidAndSycDDb(resource)) {
            App message = new App();
            if ("iOS".equalsIgnoreCase(resource.getPlatform())) {
                message.setErroMessage("Bundleid已存在!");
            } else if ("Android".equalsIgnoreCase(resource.getPlatform())) {
                //网眼定制需求 选择Android的时候Bundleid 填包名
                message.setErroMessage("包名已存在!");
            }

            return message;
        }

        Account account = accountRepository.findOne(accountId);
        resource.setRootParent(account.getRootParent());
        resource.setAccount(account.getId());
        resource.setCreateAccount(account.getId() + "");
        resource.setCreateTime(new Date());
        resource.setModifyAccount(account.getId() + "");
        resource.setModifyTime(new Date());
        resource.setDelFlag(false);

        BigInteger count = appRepository.count(account.getRootParent());
        resource.setSort(count.intValue());

        String dateStr = DateUtil.getCurrentDateStr(DateUtil.C_TIME_PATTON_DEFAULT);
        String key = ShortUrlGenerator.md5(account.getEmail() + resource.getName() + resource.getPlatform() + dateStr);
        resource.setAppkey(ShortUrlGenerator.md5(key));
        resource.setToken(CipherUtil.generatePassword("reged_" + ShortUrlGenerator.md5(key)));
        resource.setRegedbutton(false);
        //新建APP默认debug模式 true 是测试,false不是测试
        if ("selfBuilt".equals(resource.getPlatform())) {
            //自建站跳过调试
            resource.setIsDebug(false);
        } else {
            resource.setIsDebug(true);
        }
        resource.setSyncDdb(false);

        App app = appRepository.save(resource);
        //网眼定制需求 2116 选择Android和ios的时候自动创建三条活动
        if (27289L == account.getRootParent() || 38L == account.getRootParent()) {
            List<ChannelWangyan> all = channelWangyanRepository.findAll();
            for (ChannelWangyan channelWangyan : all) {
                putWangyan(channelWangyan, app, account.getId());
            }
        }

        if (app.getAppTagList() != null && app.getAppTagList().size() > 0) {
            List<String> tagIdsList = app.getAppTagList().stream().map(AppTag::getTagId).collect(Collectors.toList());
            final List<SyDicItem> subNodeList = iDicService.find(tagIdsList);

            List<String> parentTagIdsList = subNodeList.stream().map(SyDicItem::getItemPid).collect(Collectors.toList());
            final List<SyDicItem> parentNodeList = iDicService.find(parentTagIdsList);

            Map<String, SyDicItem> dicMap = Maps.uniqueIndex(subNodeList, SyDicItem::getItemCode);
            Map<String, SyDicItem> dicParentMap = Maps.uniqueIndex(parentNodeList, SyDicItem::getItemCode);

            app.getAppTagList().forEach(appTag -> {
                SyDicItem syDicItem = dicMap.get(appTag.getTagId());
                if (syDicItem != null) {
                    appTag.setTagName(syDicItem.getItemValue());
                    appTag.setParentTagId(syDicItem.getItemPid());

                    if (dicParentMap.containsKey(syDicItem.getItemPid())) {
                        appTag.setParentTagName(dicParentMap.get(syDicItem.getItemPid()).getItemValue());
                    }
                }
            });
            this.iAppTagService.saveOrUpdate(app.getId(), app.getAppTagList());
        }

        //同步DDB,同步APP的测试模式状态
        this.syncAppDebugStatus(app);

        //入crateDB
        //this.insertCrateDB(app);

        // 如果bundleid 唯一则同步到DDb
        if (Constant.awsneed) {
            if (!iNoSQLDBService.sycDDbByBundleId(resource, true)) {
                App message = new App();
                message.setErroMessage("Bundleid同步错误!");
                return message;
            }
        }

        //同步wx_app到ddb
        synWxApp(app);

        //TODO 创建应用时，考虑新标签如何进行添加保存
        return app;
    }

    @Override
    public void putWangyan(ChannelWangyan channelWangyan, App app, long account) {
        Channel one = channelRepository.findOne(channelWangyan.getChannel());
        Campaign campaign = new Campaign();
        campaign.setApp(app.getId());
        campaign.setBatchNumber(1);
        campaign.setChannel(one.getId());
        campaign.setClickLinkType(false);
        campaign.setName(app.getName() + "-" + one.getName());
        campaign.setOs(app.getPlatform());
        campaign.setUniqueName(one.getUniqueName());
        campaign.setUrl(app.getUrl());
        campaign.setSurl(app.getBundleid() + "@ry@" + channelWangyan.getPid());
        campaign.setCreateTime(new Date());
        campaign.setCreateAccount(account);
        Campaign cam = campaignService.create(app.getId(), campaign);
        CampaignWangyan campaignWangyan = new CampaignWangyan();
        campaignWangyan.setApp(app.getId());
        campaignWangyan.setCampaignId(cam.getId());
        campaignWangyanRepository.save(campaignWangyan);
    }

    private Boolean checkBundleidAndSycDDb(App resource) {

        if (!StringUtils.isEmpty(resource.getBundleid())) {
            Long count = appRepository.checkUniqueBundleId(resource.getBundleid());

            if (count != null && count.intValue() > 0) {
                return true;
            }
        }
        return false;

    }

    @Override
    public void syncAppDebugStatus(App app) {

        //同步DDB,同步APP的测试模式状态
        if (null != app && Constant.awsneed) {

            boolean syncSuccess = iNoSQLDBService.createAppSdkDebugItem(app.getAppkey(), app);

            if (syncSuccess) {
                app.setSyncDdb(true);
                appRepository.save(app);
            }
        }
    }

    @Override
    public App update(App resource, Long account) {

        App app = appRepository.findOne(resource.getId());

        String oldBundleid = app.getBundleid();

        if (app.getBundleid() != null && !app.getBundleid().equals(resource.getBundleid())) {
            if (checkBundleidAndSycDDb(resource)) {
                App message = new App();
                if ("iOS".equalsIgnoreCase(resource.getPlatform())) {
                    message.setErroMessage("Bundleid已存在!");
                } else if ("Android".equalsIgnoreCase(resource.getPlatform())) {
                    //网眼定制需求 选择Android的时候Bundleid 填包名
                    message.setErroMessage("包名已存在!");
                }
                return message;
            }
        }

        if (resource.getName() != null && !resource.getName().equals(app.getName())) {
            //更新ddb app name
            resource.setAppkey(app.getAppkey());
            NoSQLDBUtil.putAppName(resource);
        }

        Account loginAccount = accountRepository.findOne(account);
        if (app.getWhetherOnline() != resource.getWhetherOnline()) {
            List<String> deeplink = campaignRepository.findAllFilterDeeplinkByAccountAndApp(loginAccount.getRootParent(), app.getId(), Constant.deeplinkHost);
            if (deeplink != null && deeplink.size() > 0) {
                App message = new App();
                message.setErroMessage("该产品存在再营销活动，不可更改为未上线状态");
                return message;
            }
        }

        //如果是未上线改为已上线，或者反之，需要对相关的预约活动和广点通活动进行停用/启用
        if (app.getWhetherOnline() != null && app.getWhetherOnline() && !resource.getWhetherOnline()) {
            //关闭广点通渠道的预约app推广
            GdtPlan gdtPlan = gdtPlanRepository.getGdtPlanByAppkeyAndPlanAndSubscribes(app.getAppkey(), 1, true);
            if (gdtPlan != null) {
                gdtPlan.setSubscribes(false);
                gdtPlanRepository.save(gdtPlan);
            }

            //停用预约活动
            List<Campaign> campaigns = campaignRepository.findByAppAndSubscribes(app.getId(), true);
            for (Campaign campaign : campaigns) {
                try {
                    if (!campaign.getDelFlag()) {
                        campaignService.delete(app.getId(), campaign.getId(), 0L);
                    }
                } catch (Exception e) {
                    logger.error("#####自动上线活动停用失败campaignid:" + campaign.getCampaignid() + "");
                }
            }
        } else if (app.getWhetherOnline() != null && !app.getWhetherOnline() && resource.getWhetherOnline()) {
            //开启广点通渠道的预约app推广
            GdtPlan gdtPlan = gdtPlanRepository.getGdtPlanByAppkeyAndPlanAndSubscribes(app.getAppkey(), 1, false);
            if (gdtPlan != null) {
                gdtPlan.setSubscribes(true);
                gdtPlanRepository.save(gdtPlan);
            }

            //开启预约活动
            List<Campaign> campaigns = campaignRepository.findByAppAndSubscribes(app.getId(), true);
            for (Campaign campaign : campaigns) {
                try {
                    if (campaign.getDelFlag()) {
                        campaignService.enableCampaign(app.getId(), campaign.getId(), 0L);
                    }
                } catch (Exception e) {
                    logger.error("#####自动上线活动停启用失败campaignid:" + campaign.getCampaignid() + "");
                }
            }
        }

        app.setModifyAccount(account + "");
        app.setModifyTime(new Date());
        app.setName(resource.getName());
        app.setBundleid(resource.getBundleid());
        app.setUrl(resource.getUrl());
        app.setAppTagList(resource.getAppTagList());
        app.setTagType(resource.getTagType());
        app.setIndustory(resource.getIndustory());
        app.setAccessToken(resource.getAccessToken());
        app.setMipAppid(resource.getMipAppid());
        app.setMipSecret(resource.getMipSecret());
        if (app.getOnlineDate() != null) {
            Date today = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, DateUtil.getCurrentDateStr());
            Date onlineDate = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, app.getOnlineDate());
            if (onlineDate.compareTo(today) != -1) {
                app.setWhetherOnline(resource.getWhetherOnline());
                app.setWhetherAppointmentMonitor(resource.getWhetherAppointmentMonitor());
                app.setOnlineDate(resource.getOnlineDate());
            }
        } else {
            app.setWhetherOnline(resource.getWhetherOnline());
            app.setWhetherAppointmentMonitor(resource.getWhetherAppointmentMonitor());
            app.setOnlineDate(resource.getOnlineDate());
        }

        if (resource.isSync()) {
            campaignRepository.updateDownloadUrl(resource.getId(), resource.getUrl());
            subCampaignRepository.updateDownloadUrl(resource.getId(), resource.getUrl());
            packageSubCampaignRepository.updateDownloadUrl(resource.getId(), resource.getUrl());
        }
        final List<AppTag> appTagList = resource.getAppTagList();
        if (appTagList != null && appTagList.size() > 0) {
            List<String> tagIdsList = appTagList.stream().map(AppTag::getTagId).collect(Collectors.toList());
            Map<String, SyDicItem> dicMap = Maps.uniqueIndex(iDicService.find(tagIdsList), SyDicItem::getItemCode);

            appTagList.forEach(appTag -> {
                SyDicItem syDicItem = dicMap.get(appTag.getTagId());
                if (syDicItem != null) {
                    appTag.setTagName(syDicItem.getItemValue());
                    appTag.setParentTagId(syDicItem.getItemPid());
                }
            });
            this.iAppTagService.saveOrUpdate(resource.getId(), appTagList);
        }

        //同步wxApp到ddb
        synWxApp(app);

        if (!iNoSQLDBService.sycDDbByBundleId(app, true)) {
            logger.error("[{}]同步bundleid[{}]错误", app.getAppkey(), app.getBundleid());
        }
        //网眼定制需求 2116 更改Bundleid或者包名时修改之前自动创建的活动,campaignid保持不变
        if (!StringUtil.isEmpty(resource.getBundleid()) && !resource.getBundleid().equals(oldBundleid) && (27289L == loginAccount.getRootParent() || 38L == loginAccount.getRootParent())) {
            //List<ChannelWangyan> all = channelWangyanRepository.findAll();
            List<Campaign> byWangyanApp = campaignRepository.findByWangyanApp(app.getId());
            for (Campaign campaign : byWangyanApp) {
                this.iNoSQLDBService.delete(campaign.getSurl());
                ChannelWangyan channelWangyan = channelWangyanRepository.findByChannel(campaign.getChannel());
                campaign.setSurl(app.getBundleid() + "@ry@" + channelWangyan.getPid());
                campaign.setModifyTime(new Date());
                Channel channel = channelRepository.findOne(campaign.getChannel());
                final SurlModel transform = campaignService.transform(campaign, app, channel);
                if (!Strings.isNullOrEmpty(transform.getSurl())//短链
                        && !Strings.isNullOrEmpty(transform.getCampaignid())//推广活动id
                        && !Strings.isNullOrEmpty(transform.getAppid())//应用id
                        && !Strings.isNullOrEmpty(transform.getCid())) {//渠道id
                    iNoSQLDBService.create(transform);
                }
                campaignRepository.save(campaign);
            }
        }


        return appRepository.save(app);
    }


    @Override
    public App enable(Long id, Long account) {
        App app = appRepository.findOne(id);
        app.setModifyAccount(account + "");
        app.setModifyTime(new Date());
        app.setRegedbutton(true);
        return appRepository.save(app);

    }

    @Override
    public App forbidden(Long id, Long account) {
        App app = appRepository.findOne(id);
        app.setModifyAccount(account + "");
        app.setModifyTime(new Date());
        app.setRegedbutton(false);
        return appRepository.save(app);

    }

    @Override
    public Boolean validName(String name) {

        return null;
    }

    @Override
    public Boolean validAppleId(Long account, String appleId) {
        boolean flag = true;
        App app = appRepository.findByBundleidInNotDel(account, appleId);
        if (app == null) {
            flag = false;
        }
        return flag;
    }

    @Override
    public Boolean validName(Long account, String name) {
        boolean flag = true;
        App app = appRepository.findByNameNotDel(account, name);
        if (app == null) {
            flag = false;
        }
        return flag;
    }

    @Override
    public List<Category> listCategory() {

        return categoryRepository.findAll();
    }

    @Override
    public void updateSort(Account account, Long appBefore, Long appAfter) {
        App app = appRepository.findOne(appBefore);
        App app2 = appRepository.findOne(appAfter);

        if (app.getSort() < app2.getSort()) {
            int afterSort = app2.getSort();
            List<App> apps = appRepository.listAppsBetween(account.getRootParent(), app.getSort(), app2.getSort());
            List<App> list = new ArrayList<>();
            for (App a : apps) {
                Integer sort = a.getSort() - 1;
                if (a.getId().equals(app.getId())) {
                    a.setSort(afterSort);
                } else {
                    a.setSort(sort);
                }
                list.add(a);
            }
            appRepository.save(list);
        } else {
            int beforeSort = app2.getSort();
            List<App> apps = appRepository.listAppsBetween(account.getRootParent(), app2.getSort(), app.getSort());
            List<App> list = new ArrayList<>();
            for (App a : apps) {
                Integer sort = a.getSort() + 1;
                if (a.getId().equals(app.getId())) {
                    a.setSort(beforeSort);
                } else {
                    a.setSort(sort);
                }
                list.add(a);
            }
            appRepository.save(list);
        }
    }

    @Override
    public void sort(Account account, Integer isAsc) {
        List<App> appList = new ArrayList<>();
        if (isAsc.intValue() == 1) {
            appList = appRepository.listAppsAsc(account.getRootParent());
        } else {
            appList = appRepository.listAppsDesc(account.getRootParent());
        }

        if (ValidateUtil.isValid(appList)) {
            List<App> list = new ArrayList<>();
            for (int i = 0; i < appList.size(); i++) {
                App app = appList.get(i);
                app.setModifyTime(new Date());
                app.setModifyAccount(account.getId().toString());
                app.setSort(i + 1);
                list.add(app);
            }
            appRepository.save(list);
        }

    }

    /**
     * @param appkey
     * @param wxAppid     微信的appid
     * @param wxAppSecret 微信的appsecret
     * @return 插入wxApp到ddb是否成功 true 成功  false失败 (wx_app存储appkey 对应的微信appid与appsecret)
     * <p>
     * putItem方法会没有对应数据的时候插入，存在则更新
     */
    public boolean insertOrUpdateWxAppToDDB(String appkey, String wxAppid, String wxAppSecret) {
        try {
            Map<String, Object> wxAppItem = new HashMap<>();
            wxAppItem.put("appid", appkey);
            wxAppItem.put("wx_appid", wxAppid);
            wxAppItem.put("wx_secret", wxAppSecret);
            return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_WX_APP, Result.fromMap(wxAppItem, "appid"));
        } catch (Exception e) {
            logger.info("insertOrUpdateWxAppToDDB 失败：{}", e);
            return false;
        }
    }

    /**
     * @param app App 不允许为空
     *            同步wxApp 到 ddb
     *            如果记录同步ddb数据的时候失败，对业务影响不大（可以忽略此处.所以不打印日志错误详情）
     *            当记录表存在的情况下再次同步失败，会重置记录表
     */
    public void synWxApp(App app) {
        boolean appIsWxOrWg = false;
        try {
            if (app != null) {
                appIsWxOrWg = AppPlatformEnum.WX.getKey().equals(app.getPlatform()) || AppPlatformEnum.WG.getKey().equals(app.getPlatform());
            }
            if (appIsWxOrWg) {
                boolean pushSussess = insertOrUpdateWxAppToDDB(app.getAppkey(), app.getMipAppid(), app.getMipSecret());
                WxApp wxApp = wxAppRepository.findByAppid(app.getAppkey());
                Date now = new Date();
                if (wxApp == null) {
                    wxApp = new WxApp(app.getAppkey(), app.getMipAppid(), app.getMipSecret(), now, now, pushSussess, 0);
                } else {
                    wxApp.setWxAppid(app.getMipAppid());
                    wxApp.setWxSecret(app.getMipSecret());
                    wxApp.setCtime(now);
                    wxApp.setUtime(now);
                    wxApp.setFlag(pushSussess);
                    wxApp.setPushTime(0);
                }
                wxAppRepository.save(wxApp);
            }
        } catch (Exception e) {
            logger.info("保存wxApp失败,{}", e);
        }

    }

    /*
        查询此账号是否绑定产品    已绑定 true
        create by zhangyao 2018-11-22
     */
    @Override
    public Boolean haveLinkApp(Long id) {
        Long number = this.appRepository.haveLinkApp(id);
        return number.intValue() > 0;
    }

    /**
     * 处理历史数据中的bundleId
     *
     * @param apps
     * @return
     */
    @Override
    public List<App> syncAppBundleInfo(List<App> apps) {

        if (apps == null || apps.size() == 0) {
            apps = appRepository.findAppWithBundleid();
        }

        List<App> errored = new ArrayList<>();
        for (App app : apps) {

            if (!StringUtils.isEmpty(app.getBundleid())) {

                if (!iNoSQLDBService.sycDDbByBundleId(app, true)) {
                    errored.add(app);
                    logger.info("sync failed !");
                }
            }
        }

        if (errored.size() > 0) {
            //递归
            syncAppBundleInfo(errored);

            logger.info("sync loop now !");
        }

        return errored;

    }

    @Override
    public Map<String, Object> tipRemind(Long id, Account loginAccount) {
        App one = appRepository.findOne(id);
        Map<String, Object> res = new HashMap<>();
        if (
                (
                        loginAccount.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey())
                                || loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                                || loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())
                )
                        && one.getOnlineDate() != null
                        && !one.getWhetherOnline()
                        && one.getWhetherAppointmentMonitor()) {
            //是否显示提示条
            int remianDays = Days.daysBetween(new DateTime(new DateTime().toString("yyyy-MM-dd")), new DateTime(one.getOnlineDate())).getDays();
            AppTip apptip = appTipRepository.findByAppId(id);
            if (remianDays >= 0 && (apptip == null || !org.apache.commons.lang.StringUtils.equals(apptip.getTipDate(), new DateTime().toString("yyyy-MM-dd")))) {
                res.put("appOnlineTips", true);
                res.put("appName", one.getName());
                res.put("appOnlineRemindDays", remianDays < 0 ? "0" : remianDays + "");
            } else {
                res.put("appOnlineTips", false);
            }
        } else {
            res.put("appOnlineTips", false);
        }
        return res;
    }

    @Override
    public Object todayCloseTip(Long id) {
        AppTip appTip = new AppTip();
        AppTip apptip = appTipRepository.findByAppId(id);
        if (apptip == null) {
            apptip = new AppTip();
            apptip.setTipDate(DateUtil.getCurrentDateStr());
            apptip.setApp(id);
            appTip = appTipRepository.save(apptip);
        } else {
            apptip.setTipDate(DateUtil.getCurrentDateStr());
            appTip = appTipRepository.save(apptip);
        }
        return appTip;
    }

    @Override
    public Object checkBundleidAndSycDDb(String bundleid) {
        App app = new App();
        app.setBundleid(bundleid);
        return checkBundleidAndSycDDb(app);
    }

    @Override
    public App getByAppidAndToken(String appid, String appsecret) {
        return appRepository.getByAppidAndToken(appid, appsecret);
    }


    @Override
    public List findDownlist(Long accountId) {

        List<App> appList = new ArrayList<>();

        Account oneAccount = accountRepository.findOne(accountId);

        if (oneAccount.getIsSuperUser() || oneAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {
            //母账号,或者管理员创建的APP
            accountId = oneAccount.getIsSuperUser() ? accountId : oneAccount.getParent();
            appList = appRepository.listAppByRootAccount(accountId);

        } else {

            //自定义角色,自定义权限
            List<Long> appIdList = authService.findAuthAppListByAccount(oneAccount.getId());

            if (!CollectionUtils.isEmpty(appIdList)) {
                appList = appRepository.findByIds(appIdList);
            }
        }

        return appList.stream().filter(v ->
                v.getPlatform().equalsIgnoreCase("iOS")
                        || v.getPlatform().equalsIgnoreCase("Android")
        ).map(v -> {
            Map item = new HashMap(4);
            item.put("id", v.getId());
            item.put("appid", v.getAppkey());
            item.put("name", v.getName());
            item.put("platform", v.getPlatform());
            return item;
        }).collect(Collectors.toList());

    }
}
