package com.reyun.service.impl;

import com.google.common.base.Function;
import com.google.common.collect.Ordering;
import com.reyun.model.*;
import com.reyun.repository.AppRepository;
import com.reyun.repository.CampaignRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.repository.PackageSubCampaignRepository;
import com.reyun.service.AttributionService;
import com.reyun.util.BeanUtils;
import com.reyun.util.NoSQLDBUtil;
import com.reyun.util.NoSQLDBUtil.Result;
import com.reyun.util.RandomUtils;
import com.reyun.util.StringUtil;
import com.reyun.util.ValidateUtil;

import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by sunhao on 17/3/14.
 * Description 归因查询服务
 */
@Service
public class AttributionServiceImpl implements AttributionService {

    private static final String TBL_TKIO_INSTALL_KEY = "installid";

    private static final String MAP_CLICK_HISTORY = "clickHistory";

    //分包渠道
    private static final String CAMPAIGN_TYPE_PACKAGE = "pack";

    private static final String STATUS_TRUE = "1";

    private static final String ATTRIBUTION_RESULT_NORMAL = "自然量";

    private static final String ATTRIBUTION_RESULT_CHEAT = "激活作弊";

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    private ConfigParamServiceImpl configParamService;

    @Autowired
    private PackageSubCampaignRepository packageSubCampaignRepository;

    @Override
    public TrackInstallModel findAttributionByInstallId(Long appId, String deviceId1) {
        TrackInstallModel demoModel = this.mockAttributionData(appId,deviceId1);
        if(demoModel!=null){
            return demoModel;
        }
        String deviceId=deviceId1.toUpperCase();
        TrackInstallModel trackInstallModel = null;
        String appKey = appRepository.findAppkeyById(appId);
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TKIO_INSTALL, TBL_TKIO_INSTALL_KEY,
                appKey + ":" + deviceId);
        if (ValidateUtil.isValid(resultItem) && null != resultItem.asMap(TBL_TKIO_INSTALL_KEY)) {
            //处理结果集中的Map List
            Map<String, Object> queryMap = resultItem.asMap(TBL_TKIO_INSTALL_KEY);
            if (null != queryMap.get(MAP_CLICK_HISTORY)) {
                Map<String, Object> clickHistoryMap = (Map) queryMap.get(MAP_CLICK_HISTORY);
                List<Object> clickHistoryList = new ArrayList<>(clickHistoryMap.values());
                queryMap.put(MAP_CLICK_HISTORY, clickHistoryList);
            }
            trackInstallModel = BeanUtils.map2Bean(queryMap, TrackInstallModel.class);
            //归因结果
            if (null != trackInstallModel) {
                trackInstallModel = transferChannelAndCampaign(trackInstallModel,appId);
            }
        }
        return trackInstallModel;
    }



    private TrackInstallModel mockAttributionData(Long appId, String deviceId1){
        String  demoApps = configParamService.getConfigParamByKey("demo_appkey");
        if(demoApps==null){
            return null;
        }
        List<String> demoAppList = Arrays.asList(demoApps.split(","));

        App app = appRepository.findOne(appId);

        if(demoAppList!=null && demoAppList.contains(app.getAppkey())){

            TrackInstallModel model = new TrackInstallModel();

            int type = RandomUtils.nextInt(0,20);

            if(type%2==0){
                //自然量
                DateTime installTime = new DateTime().plusMinutes(RandomUtils.nextInt(120,300)*(-1));
                model.setChannelname("自然量");
                model.setWhen(installTime.toString("yyyy-MM-dd HH:mm:ss"));

            }else{
                model.setChannelname("今日头条");
                model.setClick_ip("62.21.212.55");
                model.setIsNormInstall("1");
                model.setJoinType("imei");
                model.setCampaignName("快手2_致维_356");
                model.setDeviceid(deviceId1);
                model.setImpressionMatch(type%3==0?true:false);

                TrackInstallClickModel clickModel = new TrackInstallClickModel();

                clickModel.setCampaignName(model.getCampaignName());
                clickModel.setChannelName(model.getChannelname());
                clickModel.setIp(model.getIp());
                clickModel.setImpressionMatch(model.getImpressionMatch());
                clickModel.setIsLastClick("1");
                clickModel.setIsNormClick("1");
                List hist = new ArrayList();
                hist.add(clickModel);
                model.setClickHistory(hist);

                DateTime installTime = new DateTime().plusMinutes(RandomUtils.nextInt(120,300)*(-1));
                model.setWhen(installTime.toString("yyyy-MM-dd HH:mm:ss"));
                installTime = installTime.plusMinutes(RandomUtils.nextInt(120,300)*(-1));
                model.setClicktime(installTime.toString("yyyy-MM-dd HH:mm:ss"));

            }

            return model;
        }

        return null;

    }


    /**
     * 将渠道和活动翻译成中文,排序点击
     * 1.最后点击有效,激活有效,归因到此渠道
     * 2.最后点击有效,激活作弊,归因到此渠道且激活作弊
     * 3.无最后点击,或者全部点击作弊,激活不作弊,归因自然量
     * 4.无最后点击,且激活作弊,归因作弊
     * 5.分包渠道,直接归因到相应推广活动
     */
    private TrackInstallModel transferChannelAndCampaign(TrackInstallModel trackInstallModel, Long appId) {

        //初始化归因结果
        boolean isAttribution = false;

        //分链接,处理点击,找到归因渠道
        if (!CollectionUtils.isEmpty(trackInstallModel.getClickHistory())) {

            //补充点击内的渠道和活动
            for (TrackInstallClickModel trackInstallClickModel : trackInstallModel.getClickHistory()) {

                Campaign campaign = campaignRepository.findCampaignByCampaignId(trackInstallClickModel.getCampaignid(),appId);
                trackInstallClickModel.setCampaignName(null != campaign && !StringUtil.isEmpty(campaign.getName()) ? campaign.getName() : null);
                trackInstallClickModel.setCampaignSubscribes(null != campaign && campaign.getSubscribes()!=null ? campaign.getSubscribes() : false);

                Channel channel = channelRepository.findById(Long.parseLong(trackInstallClickModel.getCid()));
                trackInstallClickModel.setChannelName(null != channel && !StringUtil.isEmpty(channel.getName()) ? channel.getName() : null);

                //最后一次正常点击
                if (STATUS_TRUE.equals(trackInstallClickModel.getIsLastClick()) && STATUS_TRUE.equals(trackInstallClickModel.getIsNormClick())) {

                    trackInstallModel.setChannelname(trackInstallClickModel.getChannelName());
                    trackInstallModel.setCampaignName(trackInstallClickModel.getCampaignName());
                    trackInstallModel.setCampaignSubscribes(trackInstallClickModel.getCampaignSubscribes());
                    isAttribution = true;
                }
            }

            //点击事件排序
            List<TrackInstallClickModel> sortedClickHistoryList = Ordering.natural().onResultOf(
                    new Function<TrackInstallClickModel, String>() {
                        @Override
                        public String apply(TrackInstallClickModel trackInstallClickModel) {
                            return trackInstallClickModel.getTimestamp();
                        }
                    }).sortedCopy(trackInstallModel.getClickHistory());

            trackInstallModel.setClickHistory(sortedClickHistoryList);

        } else if (CAMPAIGN_TYPE_PACKAGE.equals(trackInstallModel.getJoinType())) {

            //分包处理
            Campaign campaign = campaignRepository.findCampaignByCampaignId(trackInstallModel.getCampaignid(), appId);

            if(null != campaign){
                trackInstallModel.setCampaignName(campaign.getName());

                Channel channel = channelRepository.findById(campaign.getChannel());
                trackInstallModel.setChannelname(channel.getName());
                if(trackInstallModel.getSonCampaignid()!=null){
                    if("_default_".equals(trackInstallModel.getSonCampaignid())){
                        trackInstallModel.setSonCampaignName("其他");
                    }else{
                        PackageSubCampaign packageSubCampaign = packageSubCampaignRepository.findSubCampaignBycampaignid(trackInstallModel.getSonCampaignid());
                        trackInstallModel.setSonCampaignName(packageSubCampaign.getName());

                    }
                }
                trackInstallModel.setCampaignSubscribes(false);
                isAttribution = true;
            }
        }else if(trackInstallModel.getChannelname()!=null && trackInstallModel.getCid()!=null){
            ArrayList<TrackInstallClickModel> trackInstallClickModels = new ArrayList<>();
            TrackInstallClickModel trackInstallClickModel = new TrackInstallClickModel();

            trackInstallClickModel.setCampaignid(trackInstallModel.getCampaignid());
            trackInstallClickModel.setCid(trackInstallModel.getCid());
            trackInstallClickModel.setIsLastClick("1");
            trackInstallClickModel.setIsNormClick("1");
            trackInstallClickModel.setImpressionMatch(trackInstallModel.getImpressionMatch());

            String clicktime = trackInstallModel.getClicktime();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date parse=new Date();
            try {
                parse = simpleDateFormat.parse(clicktime);
            } catch (ParseException e) {
                e.printStackTrace();
            }
            long time = parse.getTime();

            trackInstallClickModel.setTimestamp(String.valueOf(time));
            trackInstallClickModel.setIp(trackInstallModel.getClick_ip());

            Campaign campaign = campaignRepository.findCampaignByCampaignId(trackInstallModel.getCampaignid(), appId);
            trackInstallClickModel.setCampaignName(null != campaign && !StringUtil.isEmpty(campaign.getName()) ? campaign.getName() : null);
            trackInstallClickModel.setCampaignSubscribes(null != campaign && campaign.getSubscribes()!=null ? campaign.getSubscribes() : false);

            Channel channel = channelRepository.findById(Long.parseLong(trackInstallModel.getCid()));
            trackInstallClickModel.setChannelName(null != channel && !StringUtil.isEmpty(channel.getName()) ? channel.getName() : null);

            trackInstallClickModels.add(trackInstallClickModel);
            trackInstallModel.setClickHistory(trackInstallClickModels);
            trackInstallModel.setChannelname(trackInstallClickModel.getChannelName());
            trackInstallModel.setCampaignName(trackInstallClickModel.getCampaignName());
            trackInstallModel.setCampaignSubscribes(trackInstallClickModel.getCampaignSubscribes());
            isAttribution = true;
        }

        //为匹配处理
        if (!isAttribution && !STATUS_TRUE.equals(trackInstallModel.getIsNormInstall())) {
            //以上都没归因,激活作弊,归因激活作弊
            trackInstallModel.setChannelname(ATTRIBUTION_RESULT_CHEAT);

        } else if (!isAttribution) {
            //以上都没归因,激活正常,则是自然量
            trackInstallModel.setChannelname(ATTRIBUTION_RESULT_NORMAL);
        }

        return trackInstallModel;
    }
}
