package com.reyun.service.impl;

import com.google.common.base.Function;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.security.TokenManager;
import com.reyun.service.*;
import com.reyun.util.*;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional
public class AuthServiceImpl implements AuthService {

    protected Logger logger = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private TokenManager tokenManager;

    @Autowired
    private DataAuthRepository dataAuthRepository;

    @Autowired
    private ThirdAccountAuthRepository thirdAccountAuthRepository;

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    RoleAuthDetailRepository roleAuthDetailRepository;

    @Autowired
    AuthRepository authRepository;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    AppRepository appRepository;

    @Autowired
    RoleAuthRepository roleAuthRepository;

    @Autowired
    ChannelAccountMapRepository channelAccountMapRepository;

    @Autowired
    DataAuthPkgsubcampaignRepository dataAuthPkgsubcampaignRepository;

    @Autowired
    UserViewColumnRepository userViewColumnRepository;

    @Autowired
    ChannelAccountService channelAccountService;

    @Autowired
    AccountService accountService;
    @Autowired
    CampaignService campaignService;


    @Autowired
    TipService tipService;

    @Autowired
    private ChannelService channelServiceImpl;

    @PersistenceContext
    private EntityManager entityManager;

    private static final int MAX_CUSTOM_ROLE_NUM = 5;

    private static final int MAX_MANAGE_NUM = 3;

    /**
     * 获取自定义角色和渠道账号的MENU
     * created by sunhao 20170607
     */
    @Override
    public RoleAuth getMenuAthByRole(Account loginAccount, Long roleId, String lanType) {
        RoleAuth roleAuth = roleAuthRepository.findOne(roleId);

        //自定义角色,渠道账号
        if (null != roleAuth && (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())
                || roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())
                || roleAuth.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey()))) {

            List<RoleAuthDetail> roleAuthDetailList = roleAuthDetailRepository.findDetailByRole(roleId);

            List<RoleAuthDetail> roleAuthDetails = new ArrayList<>();
            for (RoleAuthDetail roleAuthDetail : roleAuthDetailList) {
                if (entityManager.contains(roleAuthDetail)) {//判断是否对象处于托管状态（取消托管，防止自动同步到数据库）
                    entityManager.clear();
                }
                roleAuthDetail = roleAuthDetail.toTransEng(roleAuthDetail, lanType);
                roleAuthDetails.add(roleAuthDetail);
            }

            roleAuth.setRoleAuthDetails(roleAuthDetails);
        }

        return roleAuth;
    }

    @Override
    public Account getSubAccountInfo(Long subAccountId) {
        return accountRepository.findOne(subAccountId);
    }

    @Override
    public Account getChannelAccountInfoByEmail(Account loginAccount, String email) {

        Account channelAccount = accountRepository.findAccountByEmail(email);

        //判断此渠道账号是否已经授权
        if (null != channelAccount && channelAccount.getIsChannelPerson()) {

            List<Long> subAccountIds = this.getAllSubAccountList(loginAccount);

            List<Auth> authList = authRepository.findAuthByCreateAndAuth(subAccountIds, channelAccount.getId());

            channelAccount.setAuthList(authList);
        }

        return channelAccount;
    }

    /**
     * 获取所有的角色
     * created by sunhao 20170607
     */
    @Override
    public List<RoleAuth> getAllCustomRoleList(Account loginAccount) {

        List<RoleAuth> roleAuthList = new ArrayList<>();

        RoleAuth roleAuth = roleAuthRepository.findOne(loginAccount.getRoleCategory());

        //母账号、管理员、子账号管理员
        if (loginAccount.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            List<Long> accountList = this.getAllSubAccountList(loginAccount);
            roleAuthList = roleAuthRepository.findAllCustomRoleList(accountList);
        }

        return roleAuthList;
    }

    /**
     * 查询子账户菜单权限信息,用于登陆
     */
    @Override
    public Account getSubAccountAuthById(Long subAccountId) {

        Account account = accountRepository.findOne(subAccountId);

        if (null != account && !account.getIsSuperUser()) {
            //角色
            RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

            //角色为空,跳出
            if (null == roleAuth) {
                return account;
            }

            //构建菜单权限信息
            if (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

                //2,子应用管理员,只有App权限
                List<Auth> authList = authRepository.findAuthByAccountId(subAccountId);
                account.setAuthStr(this.parseAuthList2Str(authList));

            } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())
                    || roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())
                    || roleAuth.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

                //3,自定义权限,4,自定义角色,5,渠道账号,菜单权限来自auth.getControlAuth()
                List<Auth> authList = authRepository.findAuthByAccountId(subAccountId);
                account.setAuthStr(this.parseAuthList2Str(authList));

            }

        }

        return account;
    }

    /**
     * 获取单个子账号信息,权限信息为全量信息
     * created bu sunhao 20170606
     */
    @Override
    public Account getSubAccountById(Account loginAccount, Long subAccountId) {

        List<Auth> authList = new ArrayList<>();


        Account account = accountRepository.findOne(subAccountId);
        if (null != account) {

            //查询角色
            RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());
            //角色为空,跳出
            if (null == roleAuth) {
                return account;
            }

            //设置母账号的角色
            Account parentAccount = accountRepository.findOne(account.getParent());
            account.setParentRole(null != parentAccount ? parentAccount.getRoleCategory() : 0L);

            //查询权限
            if (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

                //2,子应用管理员
                authList = authRepository.findAuthByAccountId(subAccountId);

            } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())) {

                //3,自定义角色,
                authList = authRepository.findAuthByAccountId(subAccountId);
                List<DataAuth> dataAuthList = dataAuthRepository.findDataAuthByAccount(subAccountId);

                authList = this.buildAuthListWithDataAuth(authList, dataAuthList, true, false);

            } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())) {

                //4,选择的保存自定义角色,无需填充菜单权限
                authList = authRepository.findAuthByAccountId(subAccountId);
                List<DataAuth> dataAuthList = dataAuthRepository.findDataAuthByAccount(subAccountId);

                authList = this.buildAuthListWithDataAuth(authList, dataAuthList, false, false);

            } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

                //5,渠道账号
//                List<Long> accountList = this.getAllSubAccountList(loginAccount);
                // modify on 2019/09/29
                List<Long> accountList = accountRepository.findAllAccountByAccount(loginAccount);
                authList = authRepository.findAuthByCreateIds(subAccountId, accountList);
                List<DataAuth> dataAuthList = dataAuthRepository.findDataAuthByAccount(subAccountId);

                authList = this.buildAuthListWithDataAuth(authList, dataAuthList, true, true);
            }

            //设置权限LIST
            account.setAuthList(authList);
        }

        return account;
    }

    /**
     * 构建AuthList,包含数据权限,参数addMenuAuth 控制是否加入menu权限
     * created by sunhao
     */
    private List<Auth> buildAuthListWithDataAuth(List<Auth> authList, List<DataAuth> dataAuthList, boolean addMenuAuth, boolean isChannelPerson) {

        //数据权限
        Map<Long, Map<Long, DataAuth>> appDataAuthMap = this.getAppChannelCampaignMap(dataAuthList);

        //APP权限
        for (Auth auth : authList) {

            if (addMenuAuth) {

                //全部的菜单权限
                Map<String, RoleAuthDetail> roleAuthDetailMap = isChannelPerson ? this.getChannelAccountRoleAuthMap() : this.getAllRoleAuthMap();

                //拥有的菜单权限
                JSONArray jsonArray = JSONArray.fromObject(auth.getControlAuth());

                //聚合权限到列表中
                for (int i = 0; i < jsonArray.size(); i++) {

                    JSONObject authObject = jsonArray.getJSONObject(i);
                    RoleAuthDetail roleAuthDetail = roleAuthDetailMap.get(authObject.getString("auth"));

                    if (null != roleAuthDetail) {

                        roleAuthDetail.setEdit(authObject.getBoolean("edit"));
                        roleAuthDetail.setView(authObject.getBoolean("view"));
                    }
                }

                auth.setRoleAuthDetailList(new ArrayList<>(roleAuthDetailMap.values()));
            }

            //设置数据权限
            Map<Long, DataAuth> channelDataAuth = appDataAuthMap.get(auth.getApp());
            auth.setDataAuth(null != channelDataAuth ? new ArrayList<>(channelDataAuth.values()) : new ArrayList<DataAuth>());
        }

        return authList;
    }

    /**
     * 获取APP-渠道-活动MAP
     */
    private Map<Long, Map<Long, DataAuth>> getAppChannelCampaignMap(List<DataAuth> dataAuthList) {

        Map<Long, Map<Long, DataAuth>> appDataAuthMap = new HashMap<>();

        if (!CollectionUtils.isEmpty(dataAuthList)) {

            for (DataAuth dataAuth : dataAuthList) {

                Long channelId = dataAuth.getChannel();
                Long campaignId = dataAuth.getCampaign();

                //是否包含此渠道
                Map<Long, DataAuth> dataAuthMap = appDataAuthMap.get(dataAuth.getApp());
                dataAuthMap = null != dataAuthMap ? dataAuthMap : new HashMap<Long, DataAuth>();

                //是否包含此推广活动
                DataAuth dataAuthTemp = dataAuthMap.get(channelId);
                dataAuthTemp = null != dataAuthTemp ? dataAuthTemp : new DataAuth();
                dataAuthTemp.setChannel(channelId);

                //写入推广活动ID
                if (dataAuth.getAllCampaign()) {

                    dataAuthTemp.setAllCampaign(true);

                } else {

                    List<Long> campaignList = dataAuthTemp.getCampaignList();
                    campaignList = null != campaignList ? campaignList : new ArrayList<Long>();

                    if (!campaignList.contains(campaignId) && 0 != campaignId) {
                        campaignList.add(campaignId);
                    }
                    dataAuthTemp.setCampaignList(campaignList);
                    Channel channel = channelRepository.findOne(channelId);
                    if (channel != null && "APPMARKET".equals(channel.getCategory())) {
                        Map<String, List<String>> subMap = new HashMap<>();
                        Account loginaccount = null;
                        if (loginaccount == null) {
                            loginaccount = accountRepository.findOne(dataAuth.getAccount());
                        }
                        List<Campaign> result = campaignService.listCampaignByAppAndChannelAll(dataAuth.getApp(), channelId, loginaccount);
                        List<DataAuthPkgsubcampaign> pkgsubcampaignListByApp = dataAuthPkgsubcampaignRepository.findPkgsubcampaignListByApp(loginaccount.getId(), dataAuth.getApp());
                        Multimap<Long, String> map = ArrayListMultimap.create();
                        for (DataAuthPkgsubcampaign dataAuthPkgsubcampaign : pkgsubcampaignListByApp) {
                            map.put(dataAuthPkgsubcampaign.getCampaign(), dataAuthPkgsubcampaign.getPkgSubCampaign().toString());
                        }
                        for (Campaign campaign : result) {
                            //List<DataAuthPkgsubcampaign> pkgsubcampaignList = dataAuthPkgsubcampaignRepository.findPkgsubcampaignList(loginaccount.getId(), campaign.getApp(), campaign.getId());
                            Collection<String> pkgsubcampaignList = map.get(campaign.getId());
                            if (!CollectionUtils.isEmpty(pkgsubcampaignList)) {
                                List<String> SubIdList = new ArrayList<>();
                                SubIdList.addAll(pkgsubcampaignList);
                                subMap.put(campaign.getId().toString(), SubIdList);
                            }
                        }
                        dataAuthTemp.setPkgsubcampaign(subMap);
                    }

                    dataAuthTemp.setAllCampaign(false);

                    if (dataAuth.getChannelPermit() == null) {
                        if (dataAuthTemp.getChannelPermit() == null || dataAuthTemp.getChannelPermit() == false) {
                            dataAuthTemp.setChannelPermit(false);
                        }
                    } else {
                        if (dataAuthTemp.getChannelPermit() == null || dataAuthTemp.getChannelPermit() == false) {
                            dataAuthTemp.setChannelPermit(dataAuth.getChannelPermit());
                        }
                    }

                }

                //写入渠道
                dataAuthMap.put(channelId, dataAuthTemp);
                appDataAuthMap.put(dataAuth.getApp(), dataAuthMap);
            }
        }

        return appDataAuthMap;
    }

    /**
     * 获取母账号及下面所有子账号列表
     * created bu sunhao 20170606
     */

    private List<Long> getAllSubAccountList(Account loginAccount) {

        Long rootAccount;

        if (loginAccount.getIsSuperUser()) {
            rootAccount = loginAccount.getId();
        } else {
            rootAccount = this.findRootParentAccount(loginAccount.getId()).getId();
        }

        return accountRepository.findAllAccountList(rootAccount);
    }

    /**
     * 获取子账号的菜单权限。仅仅输出已授权的信息
     * created by sunhao 20170612
     */
    private String parseAuthList2Str(List<Auth> authList) {

        JSONObject resultObject = new JSONObject();

        for (Auth auth : authList) {

            JSONObject authJsonObject = new JSONObject();

            //菜单权限
            JSONArray menuAuthArray = JSONArray.fromObject(auth.getControlAuth() == null ? "[]" : auth.getControlAuth());

            authJsonObject.put("payAuth", auth.isPayAuth());
            authJsonObject.put("retentionAuth", auth.isRetentionAuth());
            authJsonObject.put("isNatureOpen", auth.getIsNatureOpen());
            authJsonObject.put("topAuth", auth.getTopAuth());
            authJsonObject.put("orderAuth", auth.getOrderAuth());
            authJsonObject.put("roleAuthDetailList", menuAuthArray);
            authJsonObject.put("eventAuth", auth.getEventAuth());
            resultObject.put(auth.getApp(), authJsonObject);
        }

        return resultObject.toString();
    }

    /**
     * 获取有次APP权限的所有有效的子账号和渠道账号
     *
     * @param loginAccount
     * @return
     */
    @Override
    public List<Account> getAuthorizedSubAndChannelAccount(Account loginAccount, Long appId) {

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

        return accountRepository.findAuthorizedSubAndChannelAccount(appId, simpleDateFormat.format(new Date()));
    }

    /**
     * 获取所属生效的子账号
     * created by sunhao 20170606
     */
    @Override
    public List<Account> getAllActiveSubAccount(Account loginAccount) {

        List<Account> activeSubAccountList;
        RoleAuth roleAuth = roleAuthRepository.findByRoleCategory(loginAccount.getRoleCategory());

        if (null != roleAuth && roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {
            //管理员,需要看到同级别的管理员
            activeSubAccountList = accountRepository.findActiveByRootExcludeSelf(loginAccount.getRootParent(), loginAccount.getId());

        } else if (null != roleAuth && roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
            //子应用管理员
            activeSubAccountList = accountRepository.findActiveByParent(loginAccount.getId());

        } else {
            //母账号
            activeSubAccountList = accountRepository.findActiveByRoot(loginAccount.getId());

        }

        activeSubAccountList = this.dealWithOriginalAccount(loginAccount, activeSubAccountList);

        return activeSubAccountList;
    }

    /**
     * 查询所属失效的子账号
     * created by sunhao 20170606
     */
    @Override
    public List<Account> getAllDisableSubAccount(Account loginAccount) {

        List<Account> disableSubAccountList;

        RoleAuth roleAuth = roleAuthRepository.findByRoleCategory(loginAccount.getRoleCategory());

        if (null != roleAuth && roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {
            //管理员,需要看到同级别的管理员
            disableSubAccountList = accountRepository.findDisableByRootExcludeSelf(loginAccount.getRootParent(), loginAccount.getId());

        } else if (null != roleAuth && roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
            //子应用管理员
            disableSubAccountList = accountRepository.findDisableByParent(loginAccount.getId());

        } else {
            //母账号
            disableSubAccountList = accountRepository.findDisableByRoot(loginAccount.getId());
        }

        disableSubAccountList = this.dealWithOriginalAccount(loginAccount, disableSubAccountList);

        return disableSubAccountList;
    }

    /**
     * 查询有效的渠道账号
     * created by sunhao 20170612
     */
    @Override
    public List<Account> getAllActiveChannelAccount(Account loginAccount) {

        List<Account> channelAccountList = new ArrayList<>();

        List<Long> channelAccountIds;

        RoleAuth roleAuth = roleAuthRepository.findByRoleCategory(loginAccount.getRoleCategory());

        if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(roleAuth.getRoleCategory())) {

            //查询root_parent下面
            channelAccountIds = channelAccountMapRepository.findChannelAccountIdByRoot(loginAccount.getRootParent(), true);

        } else {
            //查询自己名下的渠道账号
            channelAccountIds = channelAccountMapRepository.findChannelAccountByCreate(loginAccount.getId(), true);

        }

        //查询这些有权限的所有渠道账号
        if (!CollectionUtils.isEmpty(channelAccountIds)) {
            channelAccountList = accountRepository.findChannelAccountByAccountIds(channelAccountIds);
        }

        channelAccountList = this.dealWithOriginalAccount(loginAccount, channelAccountList);

        return channelAccountList;
    }

    /**
     * 查询停用的渠道账号
     * created by sunhao 20170612
     */
    @Override
    public List<Account> getAllDisableChannelAccount(Account loginAccount) {

        List<Account> channelAccountList = new ArrayList<>();

        List<Long> channelAccountIds;

        RoleAuth roleAuth = roleAuthRepository.findByRoleCategory(loginAccount.getRoleCategory());

        if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(roleAuth.getRoleCategory())) {

            //查询root_parent下面
            channelAccountIds = channelAccountMapRepository.findChannelAccountIdByRoot(loginAccount.getRootParent(), false);

        } else {
            //查询自己名下的渠道账号
            channelAccountIds = channelAccountMapRepository.findChannelAccountByCreate(loginAccount.getId(), false);
        }

        //查询这些有权限的所有渠道账号
        if (!CollectionUtils.isEmpty(channelAccountIds)) {
            channelAccountList = accountRepository.findChannelAccountByAccountIds(channelAccountIds);
        }

        channelAccountList = this.dealWithOriginalAccount(loginAccount, channelAccountList);

        return channelAccountList;
    }

    /**
     * 填充角色名字,修改人名字,检验账户是否过期,
     */
    private List<Account> dealWithOriginalAccount(Account loginAccount, List<Account> subAccountList) {
        //设置角色名字
        if (!CollectionUtils.isEmpty(subAccountList)) {

            //账号列表
            List<Long> accountIdList = Lists.transform(subAccountList, new Function<Account, Long>() {
                @Override
                public Long apply(Account account) {
                    return account.getId();
                }
            });

            //角色列表
            List<Long> roleCategoryList = Lists.transform(subAccountList, new Function<Account, Long>() {
                @Override
                public Long apply(Account account) {
                    return account.getRoleCategory();
                }
            });

            //所有角色MAP
            List<RoleAuth> roleAuthList = roleAuthRepository.findCustomRoleList(roleCategoryList);
            Map<Long, RoleAuth> roleAuthMap = Maps.uniqueIndex(roleAuthList, new Function<RoleAuth, Long>() {
                @Override
                public Long apply(RoleAuth roleAuth) {
                    return roleAuth.getId();
                }
            });

            //所有管理账户MAP(此处作为优化，将复杂的子语句分开处理)
//            List<Account> accountList = accountRepository.findCreateAndModifyAccount(accountIdList);
            List<Long> accountIds = accountRepository.findCreateAndModifyAccountIds(accountIdList);
            List<Account> accountList = accountRepository.findAccounts(accountIds);

            Map<Long, Account> accountMap = Maps.uniqueIndex(accountList, new Function<Account, Long>() {
                @Override
                public Long apply(Account account) {
                    return account.getId();
                }
            });

            //渠道账号备注映射
            Long rootAccount = this.findRootParentAccount(loginAccount.getId()).getId();
            List<ChannelAccountMap> channelAccountMapList = channelAccountMapRepository.findAllChannelAccount(rootAccount);
            Map<Long, ChannelAccountMap> channelRemarkMap = Maps.uniqueIndex(channelAccountMapList, new Function<ChannelAccountMap, Long>() {
                @Override
                public Long apply(ChannelAccountMap channelAccountMap) {
                    return channelAccountMap.getChannelAccount();
                }
            });

            //设置名字
            for (Account account : subAccountList) {

                RoleAuth roleAuth = roleAuthMap.get(account.getRoleCategory());
                account.setRoleName(null != roleAuth ? roleAuth.getRoleName() : null);

                //创建和修改人
                Account modifyAccount = accountMap.get(account.getModifyAccount());
                Account createAccount = accountMap.get(account.getCreateAccount());
                account.setModifyAccountName(null != modifyAccount ? modifyAccount.getEmail() : null);
                account.setCreateAccountName(null != createAccount ? createAccount.getEmail() : null);

                //账户是否过期 true 有效,  false 过期
                account.setValid(true);

                //渠道账号备注
                if (RoleEnumType.CHANNEL_PERSON.getKey().equals(account.getRoleCategory()) && channelRemarkMap.containsKey(account.getId())) {
                    account.setRemark(channelRemarkMap.get(account.getId()).getRemark());
                    if (channelRemarkMap.get(account.getId()).getAuthConfig() != null)
                        account.setAuthConfig(channelRemarkMap.get(account.getId()).getAuthConfig());
                }
            }
        }

        return subAccountList;
    }

    /**
     * 启用子账号
     * created by sunhao 20170606
     */
    @Override
    public int enableSubAccountById(Account loginAccount, Long subAccountId) {
        return accountRepository.enableSubAccount(loginAccount.getId(), new Date(), subAccountId);
    }

    /**
     * 停用子账号
     * created by sunhao 20170606
     */
    @Override
    public int disableSubAccountById(Account loginAccount, Long subAccountId) {

        int affectNum = accountRepository.disableSubAccount(loginAccount.getId(), new Date(), subAccountId);

        if (affectNum > 0) {

            //禁用后踢出所有已经登陆的此子账号
            tokenManager.delMultiRelationshipByKey(subAccountId.toString());
        }

        return affectNum;
    }

    /**
     * 启用渠道账号
     */
    @Override
    public int enableChannelAccountById(Account loginAccount, Long channelAccountId) {

        List<Long> accountIds = this.getAllSubAccountList(loginAccount);

        channelAccountMapRepository.enableChannelAccount(loginAccount.getRootParent(), channelAccountId);

        return authRepository.updateStatusByCreateAccount(channelAccountId, accountIds, true);
    }

    /**
     * 停用渠道账号
     */
    @Override
    public int disableChannelAccountById(Account loginAccount, Long channelAccountId) {

        List<Long> accountIds = this.getAllSubAccountList(loginAccount);

        channelAccountMapRepository.disableChannelAccount(loginAccount.getRootParent(), channelAccountId);

        return authRepository.updateStatusByCreateAccount(channelAccountId, accountIds, false);
    }

    @Override
    public RoleAuth getRoleAuthById(Long roleId) {
        return roleAuthRepository.findOne(roleId);
    }


    /**
     * 创建渠道子账号
     * created by sunhao 20170608
     */
    @Override
    public Account createChannelAccount(Account loginAccount, Account subAccount, String lanType) {

        //是都需要邮件激活
        boolean needMail2Active = false;

        //查询渠道账号是否存在
        Account account = accountRepository.findAccountByEmail(subAccount.getEmail());

        if (null == account) {

            //渠道账号不存在
            account = this.buildAccount(loginAccount, subAccount);
            account = accountRepository.save(account);

            needMail2Active = true;

        } else if (!StringUtils.isEmpty(subAccount.getName()) && !subAccount.getName().equals(account.getName())) {
            //保存姓名
            account.setName(subAccount.getName());
            account = accountRepository.save(account);
        }

        //保存APP权限和数据权限
//        this.saveChannelAuthAndDataAuth(subAccount.getAuthList(), loginAccount, account);

        //创建渠道账号备注对应关系,设置权限配置状态为false
        this.createOrUpdateChannelRemark(loginAccount.getId(), account.getId(), subAccount.getRemark());

        if (needMail2Active) {
            //激活邮件
            this.sendSubAccountActivateEmail(loginAccount, account, lanType);
            account.setFirstSend(true);
            return account;
        } else {
            //通知邮件
            this.sendChannelAccountAuthEmail(loginAccount, account, lanType);
            return account;
        }


    }

    /**
     * 保存渠道账号的权限和数据权限
     */
    public boolean saveChannelAuthAndDataAuth(List<Auth> authList, Account loginAccount, Account channelAccount) {

        List<Auth> saveAuthList = new ArrayList<>();
        List<DataAuth> saveDataAuthList = new ArrayList<>();

        //查询这个渠道账号下的和登陆账号有关联的账号给此渠道账号的授权。
        List<Long> allAccount = this.getAllSubAccountList(loginAccount);
        List<Auth> oldAuthList = authRepository.findAuthByCreateAccount(allAccount, channelAccount.getId());
        List<DataAuth> oldDataAuthList = dataAuthRepository.findDataAuthByCreateAccount(allAccount, channelAccount.getId());

        if (!CollectionUtils.isEmpty(authList)) {
            //删除老权限
            authRepository.delete(oldAuthList);
//            dataAuthRepository.delete(oldDataAuthList);
        }
        this.saveDataAuthPkgsubcampaign(loginAccount, channelAccount);
        //新权限加入。
        for (Auth auth : authList) {
            //构建数据权限
            saveDataAuthList.addAll(this.getDataAuthByAuth(channelAccount.getId(), auth));
            //构建auth权限
            List<RoleAuthDetail> roleAuthDetailList = auth.getRoleAuthDetailList();
            Auth saveAuth = this.buildAuth(auth, channelAccount, loginAccount.getId(), roleAuthDetailList);

            saveAuthList.add(saveAuth);
        }

        if (!CollectionUtils.isEmpty(saveAuthList)) {
            authRepository.save(saveAuthList);
        }

        saveNewOnlyDataAuth(saveDataAuthList, oldDataAuthList);


        //修改对应关系表中，配置权限的状态为true
        Long rootAccountId = loginAccount.getRootParent();
        ChannelAccountMap channelAccountMap = channelAccountMapRepository.findAccount(rootAccountId, channelAccount.getId());
        channelAccountMap.setAuthConfig(true);
        channelAccountMapRepository.save(channelAccountMap);

        return true;
    }


    /**
     * 保存新增的权限  删除取消的老权限
     *
     * @param saveDataAuthList
     * @param oldDataAuthList
     */
    public void saveNewOnlyDataAuth(List<DataAuth> saveDataAuthList, List<DataAuth> oldDataAuthList) {
        //处理老权限
        List<DataAuth> shouldSaveList = new ArrayList();
        List<DataAuth> shouldDelList = new ArrayList();

        if (oldDataAuthList == null) {
            oldDataAuthList = new ArrayList<>();
        }

        //删除部分老权限  保存新增权限
        Map<String, DataAuth> saveDataAuthMap = saveDataAuthList.stream().collect(Collectors.toMap(
                k -> k.getApp() + "_" + k.getCampaign() + "_" + k.getChannel() + "_" + k.getAccount()
                        + "_" + ((k.getChannelPermit() == null || k.getChannelPermit()) ? 1 : 0)
                        + "_" + ((k.getAllCampaign() == null || k.getAllCampaign()) ? 1 : 0),
                java.util.function.Function.identity(), (k1, k2) -> k2));


        Map<String, DataAuth> oldDataAuthMap = oldDataAuthList.stream().collect(Collectors.toMap(
                k -> k.getApp() + "_" + k.getCampaign() + "_" + k.getChannel() + "_" + k.getAccount()
                        + "_" + ((k.getChannelPermit() == null || k.getChannelPermit()) ? 1 : 0)
                        + "_" + ((k.getAllCampaign() == null || k.getAllCampaign()) ? 1 : 0),
                java.util.function.Function.identity(), (k1, k2) -> k2
        ));


        for (DataAuth dataAuth : saveDataAuthList) {
            if (oldDataAuthMap.get(dataAuth.getApp() + "_" + dataAuth.getCampaign() + "_" + dataAuth.getChannel() + "_" + dataAuth.getAccount()
                    + "_" + ((dataAuth.getChannelPermit() == null || dataAuth.getChannelPermit()) ? 1 : 0)
                    + "_" + ((dataAuth.getAllCampaign() == null || dataAuth.getAllCampaign()) ? 1 : 0)) == null) {
                shouldSaveList.add(dataAuth);
            }
        }

        for (DataAuth dataAuth : oldDataAuthList) {
            if (saveDataAuthMap.get(dataAuth.getApp() + "_" + dataAuth.getCampaign() + "_" + dataAuth.getChannel() + "_" + dataAuth.getAccount()
                    + "_" + ((dataAuth.getChannelPermit() == null || dataAuth.getChannelPermit()) ? 1 : 0)
                    + "_" + ((dataAuth.getAllCampaign() == null || dataAuth.getAllCampaign()) ? 1 : 0)) == null) {
                shouldDelList.add(dataAuth);
            }
        }

//        if (!CollectionUtils.isEmpty(saveDataAuthList)) {
//            dataAuthRepository.save(saveDataAuthList);
//        }

        if (!CollectionUtils.isEmpty(shouldSaveList)) {
            dataAuthRepository.save(shouldSaveList);
        }

        if (!CollectionUtils.isEmpty(shouldDelList)) {
            dataAuthRepository.delete(shouldDelList);
        }
    }


    /**
     * 修改渠道账号,无法修改渠道账号的名称。编辑账号的时候，不涉及权限修改03-01，zxy
     * created by sunhao 20170612
     */
    @Override
    public Account modifyChannelAccount(Account loginAccount, Account subAccount) {

        Account account = accountRepository.findAccountByEmail(subAccount.getEmail());

        logger.info(entityManager.contains(account) + "");

        if (null != account && account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

            //如果渠道账号姓名为空,则更新渠道账号名字
            if (StringUtils.isEmpty(account.getName()) && !StringUtils.isEmpty(subAccount.getName())) {
                account.setName(subAccount.getName());
            }

            //更新时间
            account.setModifyAccount(loginAccount.getId());
            account.setModifyTime(new Date());
            account.setRemark(subAccount.getRemark());
            accountRepository.save(account);

            //更新备注
            Long rootAccount = this.findRootParentAccount(loginAccount.getId()).getId();
            ChannelAccountMap accountMap = channelAccountMapRepository.findAccount(rootAccount, account.getId());
            accountMap.setRemark(account.getRemark());
            channelAccountMapRepository.save(accountMap);

        }

        return account;
    }

    /**
     * 创建子账号,渠道账号
     * created by sunhao 20170607
     */
    @Override
    public Account createSubAccount(Account loginAccount, Account subAccount, RoleAuth roleAuth, String lanType) {

        //新建账号 account  渠道账号
        Account saveAccount = this.buildAccount(loginAccount, subAccount);
        //新创建的账号(除去管理员，管理员不需要配置权限)，还没有配置权限
        if (!subAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {
            saveAccount.setAuthConfig(false);
        }

        saveAccount = accountRepository.save(saveAccount);

        //构建权限 auth  权限配置和账号创建分开  03-01，zxy
//        this.saveAuthAndDataAuth(subAccount.getAuthList(), roleAuth, saveAccount, loginAccount.getId());

        this.sendSubAccountActivateEmail(loginAccount, saveAccount, lanType);


        return saveAccount;
    }

    /**
     * 构建数据和菜单权限
     */
    @Override
    public boolean saveAuthAndDataAuth(List<Auth> authList, RoleAuth roleAuth, Account saveAccount, Long loginAccountId, List<DataAuth> dataAuthList) {

        List<Auth> saveAuthList = new ArrayList<>();
        List<DataAuth> saveDataAuthList = new ArrayList<>();

        if (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            //2,子应用管理员,创建auth 无需创建data_auth ;无需control_auth字段数据
            for (Auth auth : authList) {
                //默认设置
                auth.setIsNatureOpen(true);
                auth.setRetentionAuth(true);
                auth.setTopAuth(true);
                auth.setCampaign(true);
                auth.setOrderAuth(true);
                auth.setArouseAuth(true);
                Auth saveAuth = this.buildAuth(auth, saveAccount, loginAccountId, null);
                saveAuthList.add(saveAuth);
            }

        } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())) {

            //3,自定义,创建auth和data_auth
            for (Auth auth : authList) {
                //构建数据权限
                saveDataAuthList.addAll(this.getDataAuthByAuth(saveAccount.getId(), auth));
                //构建auth权限
                List<RoleAuthDetail> roleAuthDetailList = auth.getRoleAuthDetailList();
                Auth saveAuth = this.buildAuth(auth, saveAccount, loginAccountId, roleAuthDetailList);
                saveAuthList.add(saveAuth);
            }

        } else if (roleAuth.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())) {

            //5,选定的自定义角色,创建auth ,创建data_auth ;
            List<RoleAuthDetail> roleAuthDetailList = roleAuthDetailRepository.findValidDetailByRole(roleAuth.getId());

            for (Auth auth : authList) {
                //构建数据权限
                saveDataAuthList.addAll(this.getDataAuthByAuth(saveAccount.getId(), auth));

                //自定义角色设置
                auth.setPayAuth(roleAuth.getPayAuth());
                auth.setIsNatureOpen(roleAuth.getIsNatureOpen());
                auth.setRetentionAuth(roleAuth.getRetentionAuth());
                auth.setTopAuth(roleAuth.getTopAuth());
                auth.setOrderAuth(roleAuth.getOrderAuth());
                auth.setArouseAuth(roleAuth.getArouseAuth());
                auth.setEventAuth(roleAuth.getEventAuth());
                //构建auth权限
                Auth saveAuth = this.buildAuth(auth, saveAccount, loginAccountId, roleAuthDetailList);
                saveAuthList.add(saveAuth);
            }

        }

        //保存 auth
        if (!CollectionUtils.isEmpty(saveAuthList)) {
            authRepository.save(saveAuthList);
        }
//        //保存 data_auth
//        if (!CollectionUtils.isEmpty(saveDataAuthList)) {
//            dataAuthRepository.save(saveDataAuthList);
//        }


        //仅保存本次新增权限，删除取消掉的权限
        saveNewOnlyDataAuth(saveDataAuthList, dataAuthList);

        Account accountNew = accountRepository.findOne(saveAccount.getId());
        //保存完权限之后，更新账号权限配置状态为true
        accountNew.setAuthConfig(true);
        accountNew.setRoleCategory(saveAccount.getRoleCategory());
        accountRepository.save(accountNew);
        return true;
    }

    /**
     * 菜单权限转换
     */
    private JSONArray parseAuth2JsonArray(List<RoleAuthDetail> roleAuthDetailList) {

        JSONArray authArray = new JSONArray();

        if (!CollectionUtils.isEmpty(roleAuthDetailList)) {

            for (RoleAuthDetail roleAuthDetail : roleAuthDetailList) {

                JSONObject authObject = new JSONObject();
                authObject.put("auth", roleAuthDetail.getAuth());
                authObject.put("view", roleAuthDetail.getView());
                authObject.put("edit", roleAuthDetail.getEdit());
                authObject.put("sort", roleAuthDetail.getSort());
                authObject.put("parentAuth", roleAuthDetail.getParentAuth());

                authArray.add(authObject);
            }
        }

        return authArray;
    }

    /**
     * 构建Auth
     */
    private Auth buildAuth(Auth auth, Account subAccount, Long createAccount, List<RoleAuthDetail> roleAuthDetailList) {

        Auth saveAuth = new Auth();

        Date operateDate = new Date();

        saveAuth.setApp(auth.getApp());
        saveAuth.setAccount(subAccount.getId());
        saveAuth.setRoleCategory(subAccount.getRoleCategory());
        saveAuth.setControlAuth(this.parseAuth2JsonArray(roleAuthDetailList).toString());
        saveAuth.setModifyTime(operateDate);
        saveAuth.setModifyAccount(createAccount.toString());
        saveAuth.setCreateAccount(null != auth.getCreateAccount() ? auth.getCreateAccount() : createAccount);
        saveAuth.setCreateTime(null != auth.getCreateTime() ? auth.getCreateTime() : operateDate);
        saveAuth.setStatus(true);
        //付费,留存,自然量,Top指标，订单,调起,自定义事件
        saveAuth.setPayAuth(auth.isPayAuth());
        saveAuth.setIsNatureOpen(auth.getIsNatureOpen());
        saveAuth.setRetentionAuth(auth.isRetentionAuth());
        saveAuth.setTopAuth(auth.getTopAuth());
        saveAuth.setOrderAuth(auth.getOrderAuth());
        saveAuth.setArouseAuth(auth.getArouseAuth());
        saveAuth.setEventAuth(auth.getEventAuth());
        return saveAuth;
    }

    /**
     * 构建Account
     */
    private Account buildAccount(Account loginAccount, Account subAccount) {

        Account account = new Account();

        account.setName(subAccount.getName());
        account.setCompany(loginAccount.getCompany());
        account.setCreateTime(new Date());
        account.setDelFlag(false);
        account.setCreateAccount(loginAccount.getId());
        account.setModifyAccount(loginAccount.getId());
        account.setModifyTime(new Date());
        account.setEmail(subAccount.getEmail());
        account.setIsChannelPerson(false);
        account.setIsSuperUser(false);
        account.setIsChannelPerson(subAccount.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey()));
        //未激活,需要发送邮件后激活
        account.setStatus(-3);
        account.setPricelevel(loginAccount.getPricelevel());
        account.setParent(loginAccount.getId());
        //根节点账号,取登陆账号的rootParent
        account.setRootParent(loginAccount.getRootParent());
        account.setPastDate(loginAccount.getPastDate());
        account.setRoleCategory(subAccount.getRoleCategory());
        account.setRemark(subAccount.getRemark());

        return account;
    }

    /**
     * 构建数据权限data_auth
     */
    private List<DataAuth> getDataAuthByAuth(Long subAccountId, Auth auth) {

        List<DataAuth> result = new ArrayList<>();

        //渠道和活动列表
        List<DataAuth> dataAuthList = auth.getDataAuth();

        for (DataAuth dataAuthTemp : dataAuthList) {

            //校验channel和campaign
            Long channelId = dataAuthTemp.getChannel();
            List<Long> campaignList = dataAuthTemp.getCampaignList();

            //构建data_auth
            if (null != channelId) {
                if (dataAuthTemp.getAllCampaign()) {
                    //按渠道授权
                    DataAuth dataAuth = this.buildDataAuth(subAccountId, auth.getApp(), channelId, null);
                    result.add(dataAuth);

                } else {
                    Boolean flag;
                    if (dataAuthTemp.getChannelPermit() == null) {
                        flag = false;
                    } else {
                        flag = dataAuthTemp.getChannelPermit();
                    }
                    if (!CollectionUtils.isEmpty(campaignList)) {
                        //按照渠道和推广活动授权
                        for (Long campaignId : campaignList) {
                            DataAuth dataAuth = this.buildDataAuth(subAccountId, auth.getApp(), channelId, campaignId);
                            dataAuth.setChannelPermit(flag);

                            result.add(dataAuth);
                        }
                    } else {
                        if (flag) {//改成新版的只按渠道授权
                            DataAuth dataAuth = this.buildDataAuth(subAccountId, auth.getApp(), channelId, null);

                            dataAuth.setChannelPermit(flag);
                            dataAuth.setAllCampaign(false);//非第一种全选按渠道授权
                            result.add(dataAuth);
                        } else if (!CollectionUtils.isEmpty(dataAuthTemp.getPkgsubcampaign())) {
                            DataAuth dataAuth = this.buildDataAuth(subAccountId, auth.getApp(), channelId, null);
                            dataAuth.setChannelPermit(flag);
                            dataAuth.setAllCampaign(false);
                            result.add(dataAuth);
                        }
                    }
                }
            }
        }

        return result;
    }

    /**
     * 构建Data_Auth
     */
    private DataAuth buildDataAuth(Long accountId, Long appId, Long channelId, Long campaignId) {

        DataAuth dataAuth = new DataAuth();

        dataAuth.setAccount(accountId);
        dataAuth.setApp(appId);
        dataAuth.setChannel(channelId);
        dataAuth.setCampaign(null == campaignId ? 0L : campaignId);
        dataAuth.setModifyTime(new Date());
        dataAuth.setAllCampaign(null == campaignId);
        dataAuth.setDelFlag(false);

        return dataAuth;
    }

    /**
     * 修改子账户信息
     * create by sunhao 20170609
     */
    @Override
    public Account modifySubAccount(Account loginAccount, Account subAccount, RoleAuth roleAuth) {

        Account account = accountRepository.findOne(subAccount.getId());

        boolean isRoleChange = !account.getRoleCategory().equals(subAccount.getRoleCategory());


        //3,更新基本信息,激活之前，email可修改 03-01，zxy
        account.setEmail(subAccount.getEmail());
        account.setName(subAccount.getName());
        account.setRoleCategory(subAccount.getRoleCategory());
        account.setRemark(subAccount.getRemark());

        account.setModifyTime(new Date());
        account.setModifyAccount(loginAccount.getId());

        if (isRoleChange) {
            account.setAuthConfig(false);
        }
        accountRepository.save(account);

        //删除来源分析显示的列数据，判断角色发生变化时候再删除03-01，zxy
        if (isRoleChange) {
            List<UserViewColumn> userViewColumn = userViewColumnRepository.findByAccount(account.getId());
            if (null != userViewColumn && userViewColumn.size() > 0) {
                userViewColumnRepository.delete(userViewColumn);
            }

            //1,删除旧权限,渠道账号的不在这里操作
            List<Auth> authList = authRepository.findAuthByAccountId(subAccount.getId());
            List<DataAuth> dataAuthList = dataAuthRepository.findDataAuthByAccount(subAccount.getId());

            if (!CollectionUtils.isEmpty(authList)) {
                authRepository.delete(authList);
            }

            if (!CollectionUtils.isEmpty(authList)) {
                dataAuthRepository.delete(dataAuthList);
            }
        }

        return account;
    }

    /**
     * 修改子账户信息
     * create by sunhao 20170609
     */
    @Override
    public boolean modifySubAccountAuth(Account loginAccount, Account subAccount, RoleAuth roleAuth) {

        List<DataAuth> dataAuthList = null;
        if (subAccount.getAuthConfig() == null || subAccount.getAuthConfig()) {
            //1,删除旧权限,渠道账号的不在这里操作
            List<Auth> authList = authRepository.findAuthByAccountId(subAccount.getId());
            long st = System.currentTimeMillis();
            dataAuthList = dataAuthRepository.findDataAuthByAccount(subAccount.getId());
            logger.info("获取data_auth用时{}", (System.currentTimeMillis() - st) / 1000);
//            String sql = "select * from data_auth where account =" + subAccount.getId();
//            RowMapper<DataAuth> rowMapper = new BeanPropertyRowMapper<DataAuth>(DataAuth.class);
//            List<DataAuth> users = jdbcTemplate.query(sql, rowMapper);

            if (!CollectionUtils.isEmpty(authList)) {
                authRepository.delete(authList);
            }
            this.saveDataAuthPkgsubcampaign(loginAccount, subAccount);

//            if (!CollectionUtils.isEmpty(authList)) {
//                dataAuthRepository.delete(dataAuthList);
//            }
        }


        //2,保存新权限
        boolean result = this.saveAuthAndDataAuth(subAccount.getAuthList(), roleAuth, subAccount, loginAccount.getId(), dataAuthList);
        return result;
    }

    public void saveDataAuthPkgsubcampaign(Account loginAccount, Account subAccount) {
        dataAuthPkgsubcampaignRepository.delByAccount(subAccount.getId());
        List<Auth> authList = subAccount.getAuthList();
        List<DataAuthPkgsubcampaign> pkgs = new ArrayList<>();
        if (!CollectionUtils.isEmpty(authList)) {
            for (Auth auth : authList) {
                List<DataAuth> dataAuths = auth.getDataAuth();
                if (!CollectionUtils.isEmpty(dataAuths)) {
                    for (DataAuth dataAuth : dataAuths) {
                        Map<String, List<String>> pkgsubcampaignMap = dataAuth.getPkgsubcampaign();
                        if (!CollectionUtils.isEmpty(pkgsubcampaignMap)) {
                            for (String campaign : pkgsubcampaignMap.keySet()) {
                                List<String> pkgCampaigns = pkgsubcampaignMap.get(campaign);
                                if (!CollectionUtils.isEmpty(pkgCampaigns)) {
                                    for (String pkgCampaign : pkgCampaigns) {
                                        DataAuthPkgsubcampaign dataAuthPkgsubcampaign = new DataAuthPkgsubcampaign();
                                        dataAuthPkgsubcampaign.setAccount(subAccount.getId());
                                        dataAuthPkgsubcampaign.setApp(auth.getApp());
                                        dataAuthPkgsubcampaign.setCampaign(Long.valueOf(campaign));
                                        dataAuthPkgsubcampaign.setChannel(dataAuth.getChannel());
                                        dataAuthPkgsubcampaign.setModifyAccount(loginAccount.getId());
                                        dataAuthPkgsubcampaign.setPkgSubCampaign(Long.valueOf(pkgCampaign));
                                        pkgs.add(dataAuthPkgsubcampaign);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        if (pkgs.size() > 0) {
            dataAuthPkgsubcampaignRepository.save(pkgs);
        }
    }

    @Override
    public int updateAuthConfig(Long id, Boolean status) {
        return accountRepository.updateAuthConfig(id, status);
    }

    @Override
    public boolean isNeedAuthConfig(Long roleCategory) {
        return roleCategory.equals(RoleEnumType.CUSTOM_AUTH.getKey()) || roleCategory.equals(RoleEnumType.CHANNEL_PERSON.getKey());
    }

    /**
     * 创建自定义角色
     * create by sunhao 20170607
     */
    @Override
    public RoleAuth createCustomRole(Account loginAccount, RoleAuth roleAuth) {

        RoleAuth saveRoleAuth = new RoleAuth();

        //创建并保存 ROLE_AUTH
        saveRoleAuth.setCreateAccount(loginAccount.getId());
        saveRoleAuth.setRoleName(roleAuth.getRoleName());
        saveRoleAuth.setCreateTime(new Date());
        saveRoleAuth.setRoleCategory(RoleEnumType.CUSTOM_ROLE.getKey());
        saveRoleAuth.setDelFlag(false);
        saveRoleAuth.setPayAuth(roleAuth.getPayAuth());
        saveRoleAuth.setRetentionAuth(roleAuth.getRetentionAuth());
        saveRoleAuth.setIsNatureOpen(roleAuth.getIsNatureOpen());
        saveRoleAuth.setTopAuth(roleAuth.getTopAuth());
        saveRoleAuth.setOrderAuth(roleAuth.getOrderAuth());
        saveRoleAuth.setArouseAuth(roleAuth.getArouseAuth());
        saveRoleAuth.setEventAuth(roleAuth.getEventAuth());
        saveRoleAuth = roleAuthRepository.save(saveRoleAuth);

        //构建 ROLE_AUTH_DETAIL,全量detail
        List<RoleAuthDetail> roleAuthDetailList = this.getInitRoleAuthList();

        Map<String, RoleAuthDetail> roleAuthDetailMap = Maps.uniqueIndex(roleAuth.getRoleAuthDetails(), new Function<RoleAuthDetail, String>() {
            @Override
            public String apply(RoleAuthDetail roleAuthDetail) {
                return roleAuthDetail.getAuth();
            }
        });

        //构建选中信息
        for (RoleAuthDetail roleAuthDetail : roleAuthDetailList) {
            RoleAuthDetail newRoleAuthDetail = roleAuthDetailMap.get(roleAuthDetail.getAuth());

            if (null != newRoleAuthDetail) {
                roleAuthDetail.setEdit(newRoleAuthDetail.getEdit());
                roleAuthDetail.setView(newRoleAuthDetail.getView());
            }

            roleAuthDetail.setRoleId(saveRoleAuth.getId());
        }

        //保存 ROLE_AUTH_DETAIL
        roleAuthDetailRepository.save(roleAuthDetailList);

        return saveRoleAuth;
    }

    /**
     * 获取全部未选中的所有的菜单权限MAP,
     */
    private Map<String, RoleAuthDetail> getAllRoleAuthMap() {
        return getRoleAuthMapByRoleType(RoleEnumType.CUSTOM_AUTH.getKey());
    }

    /**
     * 获取渠道账号全部的菜单权限MAP,
     */
    private Map<String, RoleAuthDetail> getChannelAccountRoleAuthMap() {
        return getRoleAuthMapByRoleType(RoleEnumType.CHANNEL_PERSON.getKey());
    }

    /**
     * 获取全部未选中的所有的菜单权限MAP,
     */
    private Map<String, RoleAuthDetail> getRoleAuthMapByRoleType(Long roleType) {

        Map<String, RoleAuthDetail> result = new HashMap<>();

        List<RoleAuthDetail> roleAuthDetailList = roleAuthDetailRepository.findDetailByRole(roleType);

        for (RoleAuthDetail roleAuthDetail : roleAuthDetailList) {
            result.put(roleAuthDetail.getAuth(), new RoleAuthDetail(roleAuthDetail));
        }

        return result;
    }

    /**
     * 获取全部未选中的所有的菜单权限LIST,
     */
    private List<RoleAuthDetail> getInitRoleAuthList() {

        List<RoleAuthDetail> result = new ArrayList<>();

        List<RoleAuthDetail> roleAuthDetailList = roleAuthDetailRepository.findDetailByRole(RoleEnumType.CUSTOM_AUTH.getKey());

        for (RoleAuthDetail roleAuthDetail : roleAuthDetailList) {
            result.add(new RoleAuthDetail(roleAuthDetail));
        }

        return result;
    }


    /**
     * 修改自定义角色,参数只传了选中的权限
     * created by sunhao 20170609
     */
    @Override
    public RoleAuth modifyCustomRole(Account loginAccount, RoleAuth roleAuth) {

        //修改保存基础信息
        RoleAuth customRoleAuth = roleAuthRepository.findOne(roleAuth.getId());
        customRoleAuth.setRoleName(!StringUtils.isEmpty(roleAuth.getRoleName()) ? roleAuth.getRoleName() : customRoleAuth.getRoleName());
        customRoleAuth.setPayAuth(null != roleAuth.getPayAuth() ? roleAuth.getPayAuth() : customRoleAuth.getPayAuth());
        customRoleAuth.setRetentionAuth(null != roleAuth.getRetentionAuth() ? roleAuth.getRetentionAuth() : customRoleAuth.getRetentionAuth());
        customRoleAuth.setIsNatureOpen(null != roleAuth.getIsNatureOpen() ? roleAuth.getIsNatureOpen() : customRoleAuth.getIsNatureOpen());
        customRoleAuth.setTopAuth(null != roleAuth.getTopAuth() ? roleAuth.getTopAuth() : customRoleAuth.getTopAuth());
        customRoleAuth.setOrderAuth(null != roleAuth.getOrderAuth() ? roleAuth.getOrderAuth() : customRoleAuth.getOrderAuth());
        customRoleAuth.setArouseAuth(null != roleAuth.getArouseAuth() ? roleAuth.getArouseAuth() : customRoleAuth.getArouseAuth());
        customRoleAuth.setEventAuth(null != roleAuth.getEventAuth() ? roleAuth.getEventAuth() : customRoleAuth.getEventAuth());
        customRoleAuth.setModifyTime(new Date());
        customRoleAuth.setModifyAccount(loginAccount.getId());

        customRoleAuth = roleAuthRepository.save(customRoleAuth);

        //原始权限
        List<RoleAuthDetail> customRoleAuthDetailList = roleAuthDetailRepository.findDetailByRole(roleAuth.getId());

        //新权限
        List<RoleAuthDetail> sourceRoleAuthDetailList = roleAuth.getRoleAuthDetails();
        Map<String, RoleAuthDetail> sourceRoleAuthDetailMap = Maps.uniqueIndex(sourceRoleAuthDetailList, new Function<RoleAuthDetail, String>() {
            @Override
            public String apply(RoleAuthDetail roleAuthDetail) {
                return roleAuthDetail.getAuth();
            }
        });

        for (RoleAuthDetail oldRoleAuthDetail : customRoleAuthDetailList) {

            RoleAuthDetail newRoleAuthDetail = sourceRoleAuthDetailMap.get(oldRoleAuthDetail.getAuth());

            if (null != newRoleAuthDetail) {
                //修改
                oldRoleAuthDetail.setView(newRoleAuthDetail.getView());
                oldRoleAuthDetail.setEdit(newRoleAuthDetail.getEdit());

            } else {
                //初始化
                oldRoleAuthDetail.setEdit(false);
                oldRoleAuthDetail.setView(false);
            }
        }

        //保存
        roleAuthDetailRepository.save(customRoleAuthDetailList);

        //修改相应自定义角色的权限
        this.updateCustomRoleAuthByRoleId(roleAuth.getRoleAuthDetails(), customRoleAuth);

        return customRoleAuth;
    }

    /**
     * 批量更新相应自定义角色的权限
     */
    private void updateCustomRoleAuthByRoleId(List<RoleAuthDetail> roleAuthDetailList, RoleAuth customRoleAuth) {

        List<Auth> authList = authRepository.findAuthByRoleCategory(customRoleAuth.getId());

        for (Auth auth : authList) {
            auth.setControlAuth(parseAuth2JsonArray(roleAuthDetailList).toString());
            auth.setPayAuth(customRoleAuth.getPayAuth());
            auth.setRetentionAuth(customRoleAuth.getRetentionAuth());
            auth.setIsNatureOpen(customRoleAuth.getIsNatureOpen());
            auth.setTopAuth(customRoleAuth.getTopAuth());
            auth.setOrderAuth(customRoleAuth.getOrderAuth());
            auth.setEventAuth(customRoleAuth.getEventAuth());
        }

        authRepository.save(authList);
    }

    /**
     * 删除自定义角色
     * created by sunhao 20170609
     */
    @Override
    public int deleteCustomRole(Account loginAccount, Long roleId) {

        return roleAuthRepository.deleteCustomRole(roleId, loginAccount.getId(), new Date());
    }

    /**
     * 删除未激活的子账号
     * created by sunhao 20170609
     */
    @Override
    public boolean deleteInactivateSubAccount(Long subAccountId) {

        Account account = accountRepository.findOne(subAccountId);

        if (account.getStatus() == -3) {

            //删除账户
            accountRepository.delete(account);

            //删除权限
            authRepository.deleteAuthByAccount(subAccountId);
            dataAuthRepository.deleteAuthByAccount(subAccountId);

            return true;
        }

        return false;
    }

    /**
     * 校验自定义权限名称是否重复,true:重复,false:没重复
     * create by sunhao 20170608
     */
    @Override
    public boolean checkCustomRoleName(Account loginAccount, String roleName, Long roleId) {

        boolean result = false;

        //如果检查编辑角色的名字,查看名字是否和之前一样,不一样就按原逻辑检查,一样就直接返回false
        if (null != roleId) {

            RoleAuth roleAuth = roleAuthRepository.findOne(roleId);
            if (!roleAuth.getRoleName().equals(roleName)) {
                roleId = null;
            }
        }

        //检查名字
        if (null == roleId) {

            List<Long> subAccountList = this.getAllSubAccountList(loginAccount);

            BigInteger number = roleAuthRepository.findByAccountAndName(subAccountList, roleName);

            result = null != number && number.intValue() > 0;

        }

        return result;
    }

    /**
     * 校验自定义权限的数量,true:超过,false:没超过
     * create by sunhao 20170608
     */
    @Override
    public boolean checkCustomRoleNumber(Account loginAccount) {

        List<Long> subAccountList = this.getAllSubAccountList(loginAccount);

        BigInteger number = roleAuthRepository.findCustomNumByAccount(subAccountList);

        return null != number && number.intValue() >= MAX_CUSTOM_ROLE_NUM;
    }

    /**
     * 校验已开启子管理员的数量 true:超过,false:没超过
     * create by sunhao 20170608
     * update by zhangyao  2018-11-19   管理员数量可以根据数据库中的mng_count判断最高数量
     */
    @Override
    public boolean checkSubMangerNumber(Account loginAccount) {

        BigInteger number = accountRepository.findSubMangerNumByParent2(loginAccount.getIsSuperUser() ? loginAccount.getId() : loginAccount.getParent());
        return number != null && number.intValue() >= accountRepository.checkMngCount(loginAccount.getIsSuperUser() ? loginAccount.getId() : loginAccount.getParent());
    }

    @Override
    public boolean checkUpdateSubMangerNumber(Account loginAccount, Account updateAccount) {

        Account originalAccount = accountRepository.findOne(updateAccount.getId());

        if (!originalAccount.getRoleCategory().equals(updateAccount.getRoleCategory()) && RoleEnumType.MANAGER.getKey().equals(updateAccount.getRoleCategory())) {

            return this.checkSubMangerNumber(loginAccount);

        } else {
            return false;
        }
    }

    /**
     * 校验Email存在,true:存在,false:不存在
     * create by sunhao 20170616
     */
    @Override
    public boolean checkEmailExists(String email) {

        Account account = accountRepository.findAccountByEmail(email);

        return null != account;
    }

    @Override
    public boolean checkChannelEmailExists(String email) {

        Account account = accountRepository.findAccountByEmail(email);
        if (null != account && account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {
            return true;
        } else if (null == account) {
            return true;
        } else {
            return false;
        }

    }

    /**
     * 校验此角色下面有无子账号 true:有,false:没有
     * created by sunhao 20170609
     */
    @Override
    public List<String> checkCustomRoleAccount(Long roleId) {

        List<String> result = accountRepository.findAccountByRoleId(roleId);
        return result;
    }

    /**
     * 根据账户和APP获取权限数据
     */
    @Override
    public Auth findAuth(Long account, Long app) {
        return authRepository.findAuthByAccountAndApp(account, app);
    }

    /**
     * 是否包含自然量
     * modify by sunhao 20170613
     */
    @Override
    public boolean isNature(Long accountId, Long app) {

        boolean isNatureOpen = true;

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        //不是管理员且不是应用管理员的时候,通过auth判断
        if (!account.getIsSuperUser() && null != roleAuth && !roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && !roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            Auth auth = this.findAuth(accountId, app);
            isNatureOpen = auth != null && auth.getIsNatureOpen();
        }

        return isNatureOpen;
    }

    /**
     * 查询子账号的根节点母账号
     */
    @Override
    public Account findRootParentAccount(Long childAccountId) {

        Account account = accountRepository.findOne(childAccountId);

        while (null != account.getParent() && 0 != account.getParent()) {

            account = accountRepository.findOne(account.getParent());
        }

        return account;
    }

    /**
     * 校验母账号是否过期
     */
    @Override
    public boolean isOriginAccountPast(Long account) {

        Account accountObject = accountRepository.findOne(account);

        String nowDate = DateUtil.format(new Date(), DateUtil.C_DATE_PATTON_DEFAULT);

        return !StringUtil.isEmpty(accountObject.getPastDate()) && DateUtil.compare_date(nowDate, accountObject.getPastDate()) == 1;
    }

    @Override
    public Account check(String email) {
        return accountRepository.findAccountByEmail(email);
    }


    /**
     * 查询账号下面所有的渠道ID
     */
    @Override
    public List<Long> getAuthChannelList(Long accountId) {

        List<Long> result = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());


        //母账号,管理员,子应用管理员。合作渠道和创建的所有自定义渠道
        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            //合作渠道
            List<Channel> coopChannelList = channelRepository.findCoopIdChannel();
            result.addAll(this.transformChannelToChannelId(coopChannelList));

            //自己以及所属子账号创建的自定义渠道
            List<Channel> createChannelList = account.getIsSuperUser() ? channelRepository.listAllByMainAccount(account.getId())
                    : channelRepository.listAllByManager(account.getId(), account.getParent());
            result.addAll(this.transformChannelToChannelId(createChannelList));

        } else {

            //自定义角色,自定义权限,渠道账号。授权的渠道权限
            List<Channel> authorizedChannelList = channelRepository.listChannelIdByDataAuth(accountId);
            result.addAll(this.transformChannelToChannelId(authorizedChannelList));

            //自己创建的渠道
            List<Channel> createdChannelList = channelRepository.listOwnChannelByAccount(accountId);
            result.addAll(this.transformChannelToChannelId(createdChannelList));

        }

        return result;
    }

    private List<Long> transformChannelToChannelId(List<Channel> channelList) {

        List<Long> result = new ArrayList<>();

        if (!CollectionUtils.isEmpty(channelList)) {
            result = Lists.transform(channelList, new Function<Channel, Long>() {
                @Override
                public Long apply(Channel channel) {
                    return channel.getId();
                }
            });
        }

        return result;
    }

    @Override
    public List<Channel> getAuthChannelListByAppIngnorDel(Long accountId, Long appId) {

        return getAuthChannelListByApp(accountId, appId, true);
    }

    @Override
    public List<String> getAuthCampaignIdListByAccount(Long accountId, Long appId) {
        return getAuthCampaignListIdByAccount(accountId, appId, false);
    }

    @Override
    public List<String> getAuthCampaignIdListByAccount(Long accountId, Long appId, int type) {

        return getAuthCampaignListIdByAccount(accountId, appId, true);
    }

    private List<String> getAuthCampaignListIdByAccount(Long accountId, Long appId, boolean forReport) {
        List<String> resultList = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());


        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), appId))) {

            //母账号,管理员,有此权限的子应用管理员
            resultList.addAll(this.campaignRepository.findAllCampaignIdsByApp(appId));

        } else {

            resultList = ChannelPersionAuth(accountId, appId);

        }

        return resultList;
    }

    private List<String> ChannelPersionAuth(Long accountId, Long appId) {

        List<String> resultList = new ArrayList<>();

        //自定义角色,渠道账号,授权+自己创建
        Auth auth = this.findAuth(accountId, appId);

        if (null != auth) {

            //自己创建的活动
            List<String> campaignIdList = campaignRepository.findCampaignIdsByAccountApp(accountId, appId);

            if (!CollectionUtils.isEmpty(campaignIdList)) {
                resultList.addAll(campaignIdList);
            }

            //授权的活动
            List<String> dataAuthCampaignIdListWithoutCreate = campaignRepository.findDataAuthCampaignIdListWithoutCreate(accountId, appId);

            if (!CollectionUtils.isEmpty(dataAuthCampaignIdListWithoutCreate)) {
                resultList.addAll(dataAuthCampaignIdListWithoutCreate);
            }

        }

        return resultList;
    }


    @Override
    public List<Object[]> getCampaignIdsByAuth(Account accountnew, App app) {
        Account account = accountRepository.findOne(accountnew.getId());
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        if (account.getIsSuperUser()
                || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())
                && null != authRepository.findAuthByAccountAndApp(account.getId(), app.getId()))) {
            return new ArrayList<>();

        } else {
            List<Object[]> resultList = new ArrayList<>();

            Long accountId = account.getId();
            Long appId = app.getId();

            //自定义角色,渠道账号,授权+自己创建
            Auth auth = this.findAuth(accountnew.getId(), app.getId());

            if (null != auth) {

                //自己创建的活动
                List<Object[]> campaignIdList = campaignRepository.findCampaignIdsCidByAccountApp(accountId, appId);

                if (!CollectionUtils.isEmpty(campaignIdList)) {
                    resultList.addAll(campaignIdList);
                }

                //授权的活动
                List<Object[]> dataAuthCampaignIdListWithoutCreate = campaignRepository.findDataAuthCampaignIdCidListWithoutCreate(accountId, appId);

                if (!CollectionUtils.isEmpty(dataAuthCampaignIdListWithoutCreate)) {
                    resultList.addAll(dataAuthCampaignIdListWithoutCreate);
                }

            }

            return resultList;
        }
    }


    @Override
    public List<Channel> getAuthChannelListByApp(Long accountId, Long appId) {
        return getAuthChannelListByApp(accountId, appId, false);
    }


    public List<Channel> getAuthChannelListByApp(Long accountId, Long appId, boolean ignore) {

        List<Channel> result = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        //母账号,管理员,子应用管理员。
        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), appId))) {

            //根据创建的活动查APP下所有渠道
            result = channelRepository.listAllCampaignChannel(appId);
            //筛选改渠道是否是更细粒度的单独用户控制
            //result=channelServiceImpl.filterChannelControl(accountId,result);
        } else {

            if (ignore) {
                //自定义角色,自定义权限,渠道账号。授权的渠道权限
                List<Channel> authorizedChannelList = channelRepository.listChannelIdByDataAuthIgnoreDel(accountId, appId);
                result.addAll(authorizedChannelList);

            } else {
                //自定义角色,自定义权限,渠道账号。授权的渠道权限
                List<Channel> authorizedChannelList = channelRepository.listChannelIdByDataAuth(accountId, appId);
                result.addAll(authorizedChannelList);
            }

            //自己创建的推广活动的渠道
            List<Channel> createdChannelList = channelRepository.listSubAccountCampaignChannel(accountId, appId);
            result.addAll(createdChannelList);
        }

        return result;
    }


    /**
     * 根据账户和APP获取使用的渠道
     * 1,母账号,管理员,子应用管理员:APP下所使用的所有渠道
     * 2,自定义角色,渠道账号:授权的渠道+自己创建的活动的渠道
     */
    @Override
    public List<Channel> getUsedChannelByAccountAndApp(Long accountId, Long appId) {

        List<Channel> result = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        //母账号，非渠道账号
        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), appId))) {

            //查询APP下面所有的推广活动的渠道
            List<Channel> appChannelList = channelRepository.listAllCampaignChannel(appId);

            if (ValidateUtil.isValid(appChannelList)) {
                result.addAll(appChannelList);
            }

        } else {

            //自定义角色,自定义权限,渠道账号。授权的渠道权限
            List<Channel> authorizedChannelList = channelRepository.listChannelByDataAuth(accountId, appId);

            if (ValidateUtil.isValid(authorizedChannelList)) {
                result.addAll(authorizedChannelList);
            }

            //自己创建的推广活动渠道
            List<Channel> createdChannelList = channelRepository.listCampaignChannelByAccount(accountId, appId);

            if (ValidateUtil.isValid(createdChannelList)) {
                result.addAll(createdChannelList);
            }
        }

        return result;
    }

    /**
     * 根据APP获取所有授权的活动,只包含生效的
     */
    @Override
    public List<Long> getAuthCampaignList(Long accountId, Long app) {

        List<Long> result = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), app))) {

            //母账号,管理员,子应用管理员
            List<Campaign> campaignList = campaignRepository.findCampaignByApp(app);

            if (CollectionUtils.isEmpty(campaignList)) {
                result = Lists.transform(campaignList, new Function<Campaign, Long>() {
                    @Override
                    public Long apply(Campaign campaign) {
                        return campaign.getId();
                    }
                });
            }

        } else {

            //自定义权限,自定义角色,渠道账号
            result = getCustomRoleAuthorizedCampaignList(accountId, app, true);
        }

        return result;
    }


    @Override
    public List<Campaign> getAuthCampaignListLimit(Account account, String name, Long appid, Map jsond, int limit) {
        // 自定义权限,自定义角色,渠道账号

        BigInteger count = campaignRepository.findAuthCampaignCount(account.getId(), appid);
        jsond.put("total", count);

        List<Campaign> campaign4Webs;
        if (StringUtil.isEmpty(name)) {
            campaign4Webs = campaignRepository.findAuthCampaign(account.getId(), appid, limit);
        } else {
            campaign4Webs = campaignRepository.findAuthCampaignName(account.getId(), appid, name, limit);
        }

        return campaign4Webs;
    }

    /**
     * 根据APP获取所有授权的活动,只包含生效的
     */
    @Override
    public List<Long> getAuthCampaignListByDelflag(Long accountId, Long app, String delflag_condition) {

        List<Long> result = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), app))) {

            //母账号,管理员,子应用管理员
            List<Campaign> campaignList = new ArrayList<>();
            if (!StringUtil.isEmpty(delflag_condition)) {
                campaignList = campaignRepository.listAllCampByAppAndDelflag(app, Boolean.valueOf(delflag_condition));
            } else {
                campaignList = campaignRepository.listCampByApp(app);
            }


            if (CollectionUtils.isEmpty(campaignList)) {
                result = Lists.transform(campaignList, new Function<Campaign, Long>() {
                    @Override
                    public Long apply(Campaign campaign) {
                        return campaign.getId();
                    }
                });
            }

        } else {

            //自定义权限,自定义角色,渠道账号
            result = getCustomRoleAuthorizedCampaignListByDelflag(accountId, app, StringUtil.isEmpty(delflag_condition) ? null : Boolean.valueOf(delflag_condition));
        }

        return result;
    }


    /**
     * 获取自定义权限授权的推广活动列表
     * 参数isIncludeDisable表示是否查出停用的推广活动
     */
    private List<Long> getCustomRoleAuthorizedCampaignList(Long accountId, Long appId, boolean isIncludeDisable) {

        List<Long> result = new ArrayList<>();

        //自定义权限,自定义角色,渠道账号
        List<DataAuth> dataList = dataAuthRepository.findDataAuthByAccountAndApp(accountId, appId);

        if (ValidateUtil.isValid(dataList)) {

            //渠道权限的渠道ID
            List<Long> channelIds = new ArrayList<>();

            for (DataAuth data : dataList) {

                if (data.getAllCampaign() && !channelIds.contains(data.getChannel())) {
                    //全部推广活动
                    channelIds.add(data.getChannel());

                } else if (!data.getAllCampaign()) {
                    //部分推广活动
                    result.add(data.getCampaign());
                }
            }

            //有按渠道授权的
            if (!CollectionUtils.isEmpty(channelIds)) {

                List<Campaign> authCampaignList = isIncludeDisable ? campaignRepository.findAllByChannelApp(channelIds, appId)
                        : campaignRepository.findActiveByChannelApp(channelIds, appId);

                if (!CollectionUtils.isEmpty(authCampaignList)) {
                    for (Campaign campaign : authCampaignList) {
                        result.add(campaign.getId());
                    }
                }
            }
        }

        //自己创建的推广活动
        List<Campaign> createdCampaignList = isIncludeDisable ? campaignRepository.findAllByCreate(accountId, appId)
                : campaignRepository.findActiveByCreate(accountId, appId);

        if (!CollectionUtils.isEmpty(createdCampaignList)) {
            for (Campaign campaign : createdCampaignList) {
                result.add(campaign.getId());
            }
        }

        return result;
    }

    /**
     * 获取自定义权限授权的推广活动列表
     * 参数isIncludeDisable表示是否查出停用的推广活动
     */
    private List<Long> getCustomRoleAuthorizedCampaignListByDelflag(Long accountId, Long appId, Boolean delflag) {

        List<Long> result = new ArrayList<>();

        //自定义权限,自定义角色,渠道账号
        List<DataAuth> dataList = delflag == null ? dataAuthRepository.findDataAuthByAccountAndApp(accountId, appId) : dataAuthRepository.findDataAuthByAccountAndAppDelflag(accountId, appId, delflag);

        if (ValidateUtil.isValid(dataList)) {

            //渠道权限的渠道ID
            List<Long> channelIds = new ArrayList<>();

            for (DataAuth data : dataList) {

                if (data.getAllCampaign() && !channelIds.contains(data.getChannel())) {
                    //全部推广活动
                    channelIds.add(data.getChannel());

                } else if (!data.getAllCampaign()) {
                    //部分推广活动
                    result.add(data.getCampaign());
                }
            }

            //有按渠道授权的
            if (!CollectionUtils.isEmpty(channelIds)) {

                List<Campaign> authCampaignList = delflag == null ? campaignRepository.findAllByChannelApp(channelIds, appId)
                        : campaignRepository.findCampByChannelAppDelflag(channelIds, appId, delflag);

                if (!CollectionUtils.isEmpty(authCampaignList)) {
                    for (Campaign campaign : authCampaignList) {
                        result.add(campaign.getId());
                    }
                }
            }
        }

        //自己创建的推广活动
        List<Campaign> createdCampaignList = delflag == null ? campaignRepository.findAllByCreate(accountId, appId)
                : campaignRepository.findActiveByCreateDelflag(accountId, appId, delflag);

        if (!CollectionUtils.isEmpty(createdCampaignList)) {
            for (Campaign campaign : createdCampaignList) {
                result.add(campaign.getId());
            }
        }

        return result;
    }


    public List<Campaign> getAuthCampaignListByAccount(Long accountId, Long appId) {

        return getAuthCampaignListByAccount(accountId, appId, false);
    }


    /**
     * 获取授权活动列表
     * 1.母账号,管理员,子应用管理员:获取APP下所有
     * 2.自定义角色,渠道账号:获取授权+创建
     */
    public List<Campaign> getAuthCampaignListByAccount(Long accountId, Long appId, boolean forReport) {

        List<Campaign> resultList = new ArrayList<>();

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || (account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey()) && null != authRepository.findAuthByAccountAndApp(account.getId(), appId))) {

            //母账号,管理员,有此权限的子应用管理员
            List<Campaign> allByApp = this.campaignRepository.findAllByApp(appId);
            for (Campaign campaign : allByApp) {
                campaign.setDeeplink(this.isDeeplink(campaign));
            }
            resultList.addAll(allByApp);

            if (forReport) {
                Campaign natureCamp = new Campaign();
                natureCamp.setChannel(-1L);
                natureCamp.setCampaignid("_default_");
                natureCamp.setName("自然量");
                resultList.add(natureCamp);
            }

        } else {

            //自定义角色,渠道账号,授权+自己创建
            Auth auth = this.findAuth(accountId, appId);

            if (null != auth) {

                //自己创建的活动
                List<Campaign> campaignList = campaignRepository.findByAccountApp(accountId, appId);

                if (!CollectionUtils.isEmpty(campaignList)) {
                    for (Campaign campaign : campaignList) {
                        campaign.setDeeplink(this.isDeeplink(campaign));
                    }
                    resultList.addAll(campaignList);
                }

                //授权的活动
                List<Campaign> dataAuthCampaignListWithoutCreate = campaignRepository.findDataAuthCampaignListWithoutCreate(accountId, appId);

                if (!CollectionUtils.isEmpty(dataAuthCampaignListWithoutCreate)) {
                    for (Campaign campaign : dataAuthCampaignListWithoutCreate) {
                        campaign.setDeeplink(this.isDeeplink(campaign));
                    }
                    resultList.addAll(dataAuthCampaignListWithoutCreate);
                }

                if (auth.getIsNatureOpen()) {
                    Campaign natureCamp = new Campaign();
                    natureCamp.setChannel(-1L);
                    natureCamp.setCampaignid("_default_");
                    natureCamp.setName("自然量");
                    resultList.add(natureCamp);
                }
            }
        }

        return resultList;
    }

    private boolean isDeeplink(Campaign campaign) {
        String containsUrl = Constant.deeplinkHost;
        if ((!StringUtil.isEmpty(campaign.getUrl()) && campaign.getUrl().contains(containsUrl)) || campaign.getDeeplink()) {
            return true;
        }
        return false;
    }

    // modify 2018-12-06
    @Override
    public Map<Long, Map<String, Campaign>> getAllCampaignByParentAccount(Long accountId, Long appId) {

        return getAllCampaignByParentAccount(accountId, appId, false);

    }

    /**
     * 查询母账号下面所有子账号的创建活动以及权限活动
     * created by sunhao
     */
    @Override
    public Map<Long, Map<String, Campaign>> getAllCampaignByParentAccount(Long accountId, Long appId, boolean forReport) {

        Map<Long, Map<String, Campaign>> result = Maps.newHashMap();

        Account acc = accountRepository.findOne(accountId);
        Long rootParent = acc.getRootParent();
        List<Account> accountList;
        if (acc.getIsSuperUser()) {
            accountList = accountRepository.findAccountByRootParent(rootParent);
        } else if (RoleEnumType.MANAGER.getKey().equals(acc.getRoleCategory())) {
            accountList = accountRepository.findAccountByRootParentExManager(rootParent, acc.getId());
        } else {
            accountList = accountRepository.findAccountByParent(accountId);
        }


        //获取子账号所有授权的活动数据
        for (Account act : accountList) {

            List<Campaign> campaignList = getAuthCampaignListByAccount(act.getId(), appId, forReport);

            if (!CollectionUtils.isEmpty(campaignList)) {
                Map<String, Campaign> campaignMap = Maps.uniqueIndex(campaignList, new Function<Campaign, String>() {
                    @Override
                    public String apply(Campaign campaign) {
                        return campaign.getCampaignid();
                    }
                });

                result.put(act.getId(), campaignMap);
            }
        }

        return result;
    }


    /**
     * 根据账号查询授权的APP
     */
    @Override
    public List<Long> findAuthAppListByAccount(Long accountId) {

        List<Long> idListOne = authRepository.findAppsByAccountAuthRole(accountId, 2);
        List<Long> idListTwo = authRepository.findAppsByAccountWithOutNew(accountId);
        List allIds = new ArrayList();
        allIds.addAll(idListOne);
        allIds.addAll(idListTwo);
        return allIds;

        //return authRepository.findAppListByAccount(accountId);
    }

    /**
     * 发送激活邮件,激活链接复用忘记密码方法
     * 亲爱的用户，已为您xxxxx@xxx.com邮箱创建TrackingIO子账号。点击此链接设置密码:http://trackingio.com/resetpwd.html#?
     */
    @Override
    public boolean sendSubAccountActivateEmail(Account loginAccount, Account subAccount, String lanType) {

        boolean result = true;

        try {
            //修改忘记密码时间
            subAccount.setForgetPwdTime(new Date().getTime());
            subAccount.setForgetPwdStatus(false);
            accountRepository.save(subAccount);

            Account root = subAccount;

            if (!subAccount.getId().equals(subAccount.getRootParent())) {
                root = accountRepository.findOne(subAccount.getRootParent());
            }

            Map<String, String> logo_txt = tipService.getLogoData(root, "logo_res_company", "active_email_url", "logo_res_create", SendCommonPostMail.EMAIL_HOST, SendCommonPostMail.EMAIL_USERNAME, SendCommonPostMail.EMAIL_PASSWORD);


//            final String url = Constant.accountCheckUrl + "/resetpwd.html#?message=" + HashAlgorithms.MD5("superuser_" + subAccount.getId() + subAccount.getForgetPwdTime()) + subAccount.getId();
            final String url = logo_txt.get("active_email_url") + "/resetpwd.html#?message=" + HashAlgorithms.MD5("superuser_" + subAccount.getId() + subAccount.getForgetPwdTime()) + subAccount.getId();

//            String sourceName = accountService.getAccountSoureceName(loginAccount,null,1);

            //邮件主题

            String subject = logo_txt.get("logo_res_company") + (subAccount.getIsChannelPerson() ? "渠道账号" : "子账号") + "激活";
            //邮件内容
            StringBuilder content = new StringBuilder();
            content.append("<!doctype html> <html><head></head><body> ")
                    .append("亲爱的用户：</br>")
                    .append("您好，").append(loginAccount.getEmail())
                    .append("已为您的").append(subAccount.getEmail()).append(logo_txt.get("logo_res_create")).append(subAccount.getIsChannelPerson() ? "渠道账号。" : "子账号。")
                    .append("点击此链接设置密码激活账号: </br>")
                    .append("<a href=\"").append(url).append("\">").append(url).append("</a>")
                    .append("</body></html>");

            //处理国际化
            if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                subject = (subAccount.getIsChannelPerson() ? "channel account" : "subaccount") + " activation for TrackingIO";

                content = new StringBuilder();
                content.append("<!doctype html> <html><head></head><body> ")
                        .append("Dear user：</br>")
                        .append(loginAccount.getEmail())
                        .append(" create TrackingIO").append(subAccount.getIsChannelPerson() ? " channel account" : " subaccount").append(" for your ").append(subAccount.getEmail())
                        .append(". You can activate the account by clicking the following link: </br>")
                        .append("<a href=\"").append(url).append("\">").append(url).append("</a>")
                        .append("</body></html>");
            }
            //end

            //收件人列表
            List<String> mailList = new ArrayList<>();
            Map<String, String> mailConfig = null;
            if (root.getRegSource() != 0) {
                mailConfig = logo_txt;
            }
            mailList.add(subAccount.getEmail());
            //MailUtils.sendHtmlEmail(subject, content.toString(), mailList, mailConfig);
            SendCommonPostMail.sendMailReuse(mailList, subject, content.toString(), null, null, mailConfig);
//            MailUtils.sendHtmlEmail(subject, content.toString(), mailList);
        } catch (Exception e) {

            logger.error(e.getMessage(), e);
            result = false;
        }

        return result;
    }

    @Override
    public boolean sendChannelAccountAuthEmail(Account loginAccount, Account subAccount, String lanType) {

        boolean result = true;
        //邮件主题
        Account root = subAccount;
        if (!subAccount.getId().equals(subAccount.getRootParent())) {
            root = accountRepository.findOne(subAccount.getRootParent());
        }
        Map<String, String> logo_txt = tipService.getLogoData(root, "logo_res_company", "active_email_url", "logo_res_create",
                "logo_res_host", SendCommonPostMail.EMAIL_HOST, SendCommonPostMail.EMAIL_USERNAME, SendCommonPostMail.EMAIL_PASSWORD);


        String subject = logo_txt.get("logo_res_company") + "渠道账号权限分配";
        //邮件内容
        StringBuilder content = new StringBuilder();
        content.append("<!doctype html> <html><head></head><body> ")
                .append("亲爱的用户：</br>")
                .append("您好，").append(loginAccount.getEmail())
                .append("已为您的 ").append(subAccount.getEmail()).append(" 邮箱分配" + logo_txt.get("logo_res_company") + "渠道账号权限。")
                .append("点击此链接查看详情: </br>")
                .append("<a href=\"http://" + logo_txt.get("logo_res_host") + "/login.html\">").append("http://" + logo_txt.get("logo_res_host") + "/login.html").append("</a>")
                .append("</body></html>");

        //处理国际化
        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            subject = "Allocation of channel account rights for TrackingIO";

            content = new StringBuilder();
            content.append("<!doctype html> <html><head></head><body> ")
                    .append("Dear user：</br>")
                    .append(loginAccount.getEmail())
                    .append(" have allocated  TrackingIO account authority").append(" for your ").append(subAccount.getEmail())
                    .append(". You can activate the account by clicking the following link: </br>")
                    .append("<a href=\"http://" + logo_txt.get("logo_res_host") + "/login.html\">").append("http://" + logo_txt.get("logo_res_host") + "/login.html").append("</a>")
                    .append("</body></html>");
        }
        //end

        //收件人列表
        List<String> mailList = new ArrayList<>();
        mailList.add(subAccount.getEmail());

        try {
            //发送邮件
//            Thread emailThread = new EmailThread(subject, content.toString(), mailList);
//            emailThread.start();
            Map<String, String> mailConfig = null;
            if (root.getRegSource() != 0) {
                mailConfig = logo_txt;
            }
            //MailUtils.sendHtmlEmail(subject,  content.toString(), mailList, mailConfig);
            SendCommonPostMail.sendMailReuse(mailList, subject, content.toString(), null, null, mailConfig);
        } catch (Exception e) {

            logger.error(e.getMessage(), e);
            result = false;
        }

        return result;
    }

    @Override
    public void createOrUpdateChannelRemark(Long loginAccountId, Long channelAccountId, String remark) {

        Long rootAccountId = this.findRootParentAccount(loginAccountId).getId();

        ChannelAccountMap channelAccountMap = channelAccountMapRepository.findAccount(rootAccountId, channelAccountId);

        if (null != channelAccountMap) {

            channelAccountMap.setRemark(remark);
            channelAccountMap.setAuthConfig(false);

        } else {

            channelAccountMap = new ChannelAccountMap(rootAccountId, loginAccountId, channelAccountId, remark, true);
            channelAccountMap.setAuthConfig(false);
        }

        channelAccountMapRepository.save(channelAccountMap);
    }

    /**
     * 上线数据处理
     * 临时处理
     */
    @Override
    public boolean dealWithOldAuth() {

        //获取全部的菜单
        Map<String, RoleAuthDetail> roleAuthDetailMap = this.getAllRoleAuthMap();

        //所有的权限
        List<Auth> authList = authRepository.findAll();

        for (Auth auth : authList) {

            if (null != auth.getControlAuth() && auth.getControlAuth().startsWith("[") && auth.getControlAuth().endsWith("]")) {

                JSONArray authArray = JSONArray.fromObject(auth.getControlAuth());
                JSONArray newAuthArray = new JSONArray();

                if (!CollectionUtils.isEmpty(authArray)) {

                    Set<String> authNameSet = new HashSet<>();

                    for (int i = 0; i < authArray.size(); i++) {

                        JSONObject authObject = authArray.getJSONObject(i);
                        if (authObject != null) {
                            String authName = authObject.getString("auth");

                            authNameSet.add(authName);

                            if (roleAuthDetailMap.containsKey(authName)) {

                                RoleAuthDetail roleAuthDetail = roleAuthDetailMap.get(authName);

                                String parentName = roleAuthDetail.getParentAuth();
                                if (!StringUtil.isEmpty(parentName) && !authNameSet.contains(parentName) && roleAuthDetailMap.containsKey(parentName)) {
                                    RoleAuthDetail roleAuthDetailParent = roleAuthDetailMap.get(parentName);
                                    JSONObject authObjectParent = new JSONObject();
                                    authObjectParent.put("auth", roleAuthDetailParent.getAuth());
                                    authObjectParent.put("view", true);
                                    authObjectParent.put("edit", true);
                                    authObjectParent.put("sort", roleAuthDetailParent.getSort());
                                    authObjectParent.put("parentAuth", roleAuthDetailParent.getParentAuth());
                                    newAuthArray.add(authObjectParent);
                                    authNameSet.add(parentName);
                                }

                                authObject.put("parentAuth", parentName);
                                authObject.put("sort", roleAuthDetail.getSort());

                                newAuthArray.add(authObject);
                            }
                        }


                    }

                    auth.setControlAuth(newAuthArray.toString());
                }
            }
        }

        authRepository.save(authList);

        return true;
    }

    @Override
    public void dealWithOldRoleDetail() {
        //获取全部的菜单
        Map<String, RoleAuthDetail> roleAuthDetailMap = this.getAllRoleAuthMap();
        List<RoleAuthDetail> detailList = roleAuthDetailRepository.findByNeedAddFirstMenu();
        Map<String, List<String>> middle = new HashMap<>();
        List<RoleAuthDetail> result = new ArrayList<>();
        for (RoleAuthDetail detail : detailList) {
            List<String> names = null;
            if (middle.containsKey(detail.getRoleId().toString())) {
                names = middle.get(detail.getRoleId().toString());
            } else {
                names = new ArrayList<>();
            }
            if (!names.contains(detail.getParentAuth())) {
                RoleAuthDetail newFirstMenu = new RoleAuthDetail();
                newFirstMenu.setAuth(detail.getParentAuth());
                newFirstMenu.setRoleId(detail.getRoleId());
                newFirstMenu.setEdit(detail.getEdit());
                newFirstMenu.setView(detail.getView());
                RoleAuthDetail custom = roleAuthDetailMap.get(detail.getParentAuth());
                newFirstMenu.setSort(custom.getSort());
                newFirstMenu.setAuthName(custom.getAuthName());
                result.add(newFirstMenu);
                names.add(detail.getParentAuth());
                middle.put(detail.getRoleId().toString(), names);
            }

        }
        roleAuthDetailRepository.save(result);
    }

    @Override
    public Map<String, Object> findGuangDianTongChannel(Long appId, long cid) {
        HashMap<String, Object> map = new HashMap<>();
        //HashMap<String, String> resultValMap = new HashMap<>();

        //广点通和微信mp
        Channel gdt = new Channel();
        Channel weixinmp = new Channel();
        gdt.setName("广点通");
        gdt.setId(1L);
        weixinmp.setName("微信mp");
        weixinmp.setId(21L);
        List<Channel> channellist = new ArrayList<Channel>(2);
        channellist.add(gdt);
        channellist.add(weixinmp);
        map.put("gdtMap", channellist);
        App app = appRepository.findOne(appId);
        List<String> spAccountNumber = this.findSpAccountNumber(app);
        List<Object> splist = new ArrayList<>(spAccountNumber.size());
        HashMap<String, String> nameMap = this.thirdAccount(appId);
        for (String s : spAccountNumber) {
            HashMap<Object, Object> mp = new HashMap<>(2);
            mp.put("sp_account_number", s);
            mp.put("alias", nameMap.containsKey(s) ? nameMap.get(s) : s);
            splist.add(mp);
        }
        map.put("channelAccounts", splist);
        return map;
    }


    private HashMap<String, String> thirdAccount(Long appid) {
        List<Object> rs = campaignRepository.findThirdAccountByAppChannel(appid, 5305L);
        HashMap<String, String> nameMap = new HashMap<>();

        if (ValidateUtil.isValid(rs)) {
            for (Object ob : rs) {
                JSONArray jsonA = JSONArray.fromObject(ob);
                String key = jsonA.getString(0);
                String value = jsonA.getString(1);
                if (!(StringUtil.isEmpty(value) || value == "null")) {
                    nameMap.put(key, value);
                }
            }
        }
        return nameMap;
    }

    @Override
    public Map<String, Object> findGuangDianTongChannelByRole(Long appId, long cid, Long accountid) {
        HashMap<String, Object> map = new HashMap<>();
        //广点通和微信mp
        Channel gdt = new Channel();
        Channel weixinmp = new Channel();
        gdt.setName("广点通");
        gdt.setId(1L);
        weixinmp.setName("微信mp");
        weixinmp.setId(21L);
        List<Channel> channellist = new ArrayList<Channel>(2);
        channellist.add(gdt);
        channellist.add(weixinmp);
        map.put("gdtMap", channellist);
        App app = appRepository.findOne(appId);
        List<String> spAccountNumber = this.findSpAccountNumber(app);
        List<String> spaccounts = null;
        HashMap<String, String> nameMap = thirdAccount(appId);
        if (!CollectionUtils.isEmpty(spAccountNumber)) {
            spaccounts = this.thirdAccountAuthRepository.findByThirdAccountAndAccount(spAccountNumber, accountid);
        }
        List<Object> splist = new ArrayList<>(spAccountNumber.size());
        if (!CollectionUtils.isEmpty(spaccounts)) {
            for (String s : spaccounts) {
                HashMap<String, Object> mp = new HashMap<>(2);
                mp.put("sp_account_number", s);
                mp.put("alias", nameMap.containsKey(s) ? nameMap.get(s) : s);
                splist.add(mp);
            }
        }
        map.put("channelAccounts", splist);
        return map;
    }

    @Override
    public List<Map<String, Object>> findGdtThirdAccount(Long appId, Long account) {
        //System.out.println("开始查询");
        //long l = System.currentTimeMillis();
        App app = appRepository.findOne(appId);
        List<Map<String, Object>> list = new ArrayList<>();
        if (app != null) {
            //List<Map<String, Object>> thirdAccounts = this.findThirdAccount(app);
            List<Object[]> thirdAccountAndId = campaignRepository.findThirdAccountAndId(app.getId());
            DataAuth dataAuth = dataAuthRepository.getByAllCampaign(account, app.getId(), 5305L);
            if (!CollectionUtils.isEmpty(thirdAccountAndId)) {
                for (Object[] thirdAccountObj : thirdAccountAndId) {
                    String thirdAccount = (String) thirdAccountObj[0];
                    if (!StringUtil.isEmpty(thirdAccount)) {
                        Map<String, Object> objects = new HashMap<>(3);
                        objects.put("id", thirdAccount);
                        //List<String> campaignids = (List<String>)thirdAccountMap.get("campaignids");
                        String camids = (String) thirdAccountObj[1];
                        List<String> campaignids = StringUtil.transString2List(camids);
                        objects.put("campaign", campaignids);
                        if (dataAuth != null || thirdAccountAuthRepository.getThirdAccount(thirdAccount, account) != null) {
                            objects.put("checked", true);
                        } else {
                            if (CollectionUtils.isEmpty(campaignids)) {
                                continue;
                            }
                            objects.put("checked", this.dataAuthRepository.checkAuth(appId, account, campaignids) > 0 ? false : null);
                        }
                        list.add(objects);
                    }
                }
                //System.out.println("用时:"+(1.0*(System.currentTimeMillis()-l)/1000));
                return list;
            }
        }
        return null;
    }

    private List<String> findSpAccountNumber(App app) {
        JdbcTemplate tkioJdbcTemplate = TkioStreamDBUtil.newInstance().getTkioJdbcTemplate();
        String sql = "SELECT DISTINCT sp_account_number FROM (SELECT sp_account_number FROM tkio.tkio_rlt_account_history_info WHERE appid= '" + app.getAppkey() + "' UNION ALL SELECT sp_account_number FROM tkio_stream.tkio_rpt_source_analysis_day WHERE appid='" + app.getAppkey() + "' AND cid = 5305) t where t.sp_account_number!='unknown' AND sp_account_number!=-1";
        //System.out.println("sql = " + sql);
        List<String> query = tkioJdbcTemplate.query(sql, new RowMapper<String>() {
            @Override
            public String mapRow(ResultSet rs, int rowNum) throws SQLException {
                return rs.getString("sp_account_number");
            }
        });
        return query;
    }

    @Override
    public boolean checkRoleTempCategory(Account account) {
        if (account.getRoleCategory() > 5) {
            RoleAuth category = roleAuthRepository.findOne(account.getRoleCategory());
            account.setRoleCategory(category.getRoleCategory());
            return true;
        }

        return false;
    }
}
