package com.reyun.service.impl;

import com.google.common.base.Strings;
import com.reyun.model.AuthModel;
import com.reyun.model.ChannelAccount;
import com.reyun.repository.ChannelAccountRepository;
import com.reyun.service.AutoSyncGdtTokenService;
import com.reyun.service.ChannelAccountService;
import com.reyun.task.AutoSyncGdtToken;
import com.reyun.util.CipherUtil;
import com.reyun.util.HttpClientUtil;
import org.joda.time.DateTime;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author: zhangyao
 * @create: 2018-12-20 17:21
 **/
@Service
@Transactional
public class AutoSyncGdtTokenServiceImpl implements AutoSyncGdtTokenService {

    private static Logger logger = LoggerFactory.getLogger(AutoSyncGdtTokenServiceImpl.class);
    private static final String GDT_REFRESH_TOKEN_URL = "https://api.e.qq.com/oauth/token?client_id=%s&client_secret=%s&grant_type=refresh_token&refresh_token=%s&timestamp=%s&nonce=%s";


    @Autowired
    private ChannelAccountRepository channelAccountRepository;

    @Autowired
    private ChannelAccountService channelAccountService;

    @Override
    public void refreshToken() {


        Optional<List<ChannelAccount>> listOptional = Optional.ofNullable(channelAccountRepository.findValidEnableByUniqueName("gdt"));
        if (!listOptional.isPresent() || listOptional.get().size() == 0) {
            return;
        }

        long st = System.currentTimeMillis();

        int acount_count = 0;

        for (ChannelAccount channelAccount : listOptional.get()) {

            try {
              acount_count +=  syncAccessToken(channelAccount);
            } catch (Exception e) {
                logger.error("GDT渠道帐号刷新出错! 账号: "+channelAccount.getName(), e);
            }
        }

        logger.info("GDT 共{}个账号, {}刷新成功, 用时{} 秒",listOptional.get().size(),acount_count ,timeLongForLog(st));
    }

    private int syncAccessToken(ChannelAccount channelAccount) {
        if (checkDueRule(channelAccount, 3)) {

            if (Strings.isNullOrEmpty(channelAccount.getRefreshToken()) || (channelAccount.getPastStatus() != null && channelAccount.getPastStatus())) {
                return 0;
            }
            AuthModel authModel = this.getToken(channelAccount.getRefreshToken());
            if (authModel==null) {
                logger.info("GDT TOKEN 刷新失败:{}",channelAccount.getName());
                return 0;
            }

            channelAccountService.refreshAcessToken(channelAccount.getId(), authModel.getAccessToken(), authModel.getAccessTokenExpire(), authModel.getRefreshToken(), authModel.getRefreshTokenExpire());

            return 1;
        }

        return 0;
    }



    public AuthModel getToken(String refreshToken) {

        String url = String.format(GDT_REFRESH_TOKEN_URL, "1106237187", "BvITHJpxu0gufAxF", refreshToken, System.currentTimeMillis() / 1000, CipherUtil.generatePassword(new Date().toString()));
        try {

            String res = HttpClientUtil.doHttpGetRequest(url,"");

            JSONObject resJson = new JSONObject(res);
            if(!resJson.has("data")){
                logger.info("GDT刷新token的返回值没有data属性,resJson:{},url:{}",resJson.toString(),url);
                return null;
            }
            JSONObject data = resJson.getJSONObject("data");
            String access_token = data.getString("access_token");
            String refresh_token = data.getString("refresh_token");
            long access_token_expires = data.getLong("access_token_expires_in");

            AuthModel model = new AuthModel();
            model.setAccessToken(access_token);
            model.setAccessTokenExpire(access_token_expires);
            model.setRefreshToken(refresh_token);
            if(data.has("refresh_token_expires_in")){
                model.setRefreshTokenExpire(data.getLong("refresh_token_expires_in"));
            }

            return model;

        }catch (Exception e){

            logger.error("GDT刷新token失败, refreshToken:"+refreshToken,e);
        }
        return null;
    }


    private boolean checkDueRule(ChannelAccount channelAccount, int beforeHour) {
        return channelAccount.getLastUpdateTime() == null
                || channelAccount.getAccessTokenExpire() == null
                || new DateTime(channelAccount.getLastUpdateTime())
                .plusSeconds(channelAccount.getAccessTokenExpire().intValue())
                .minusHours(beforeHour).isBeforeNow();
    }

    private long timeLongForLog(long st) {
        return (System.currentTimeMillis() - st) / 1000;
    }

    public static void main(String[] args) {
        /*AutoSyncGdtTokenServiceImpl syncGdtTokenService = new AutoSyncGdtTokenServiceImpl();
        AuthModel model = syncGdtTokenService.getToken("737577131a3dc3dc6b264f0af08e99a6");
        System.out.println(model.getAccessToken());*/
    }


}
