package com.reyun.service.impl;

import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AccountFlowRestrictService;
import com.reyun.service.AutoTodayFlowService;
import com.reyun.service.TipService;
import com.reyun.util.Constant;
import com.reyun.util.HttpClientUtil;
import com.reyun.util.SendCommonPostMail;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AutoTodayFlowServiceImpl implements AutoTodayFlowService {

    private static Logger logger = LoggerFactory.getLogger(AutoTodayFlowServiceImpl.class);

    private static final String NUM_IMPRESSION = "num_impression";
    private static final String NUM_CLICK_TOTAL = "num_click_total";

    public static final Integer TODAY_FLOW_ONLY = 1;
    public static final Integer SAVE_PRE_FLOW_TO_ALL = 2;


    @Autowired
    private AppRepository appRepository;

    @Autowired
    private AccountRestrict4WebRepository accountRestrict4WebRepository;

    @Autowired
    private ConfigParamRepository configParamRepository;

    @Autowired
    private BussinessManRepository bussinessManRepository;

    @Autowired
    private PackageTypeRepository packageTypeRepository;

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private TipService tipService;

    @Autowired
    private PackageFlowService packageFlowService;


    @Autowired
    private AccountFlowRestrictService accountFlowRestrictServiceImpl;

    @Autowired
    private PackageFlowRepository packageFlowRepository;

    @Autowired
    private RestrictFlowsLogRepository restrictFlowsLogRepository;

    @Autowired
    private AccountFlowHourLogRepository accountFlowHourLogRepository;


    @Override
    public AccountRestrict4Web autoTodayFlowTask(Account account) {

        return autoTodayFlowTaskWithNotify(account, 0);

    }

    //定时计算实时流量
    @Override
    public AccountRestrict4Web autoTodayFlowTaskWithNotify(Account account, int type) {

        AccountRestrict4Web restrict4Web;

        BigInteger beforeFlow = BigInteger.ZERO;

        if (type == TODAY_FLOW_ONLY) {
            // 只获取流量数据
            restrict4Web = new AccountRestrict4Web();
        } else {
            restrict4Web = accountRestrict4WebRepository.findByAccountId(account.getId());
            beforeFlow = restrict4Web.getTrackTotalFlow();
        }

        if (restrict4Web == null) {
            return restrict4Web;
        }

        //查询 2天内的实时数据
        Map<String, String> conditions = new HashMap<>();
        DateTime taday = new DateTime();
        conditions.put("startdate", taday.plusDays(-1).toString("yyyy-MM-dd"));
        conditions.put("enddate", taday.toString("yyyy-MM-dd"));
        conditions.put("datatype", "list");
        conditions.put("countdata", "1");
        conditions.put("iscache", Constant.iscache);

        List<String> appids;
        if (account.getEmail().trim().equals("jiboxu773966295525@163.com")) {
            appids = appRepository.findAllAppIdsByAccount(account.getId());
        } else if (account.getPubDate() != null && new DateTime(account.getPubDate()).isAfter(new DateTime("2021-11-14"))) {
            appids = appRepository.findAllAppIdsByAccount(account.getId());
        } else {
            appids = appRepository.findAppIdsByAccount(account.getId());
            List<String> delApps = appRepository.findDeleteAppsByDate(account.getId(), "2021-11-14");
            appids.addAll(delApps);
        }

        if (appids == null || appids.size() == 0) {
            return null;
        }

        if (account.getId().equals(6380L)) {
            List<String> keysNext = appRepository.findAppKeysByAccount(17108L);
            if (keysNext != null) {
                appids.addAll(keysNext);
            }
        }

        String appkeyStrs = appids.stream().collect(Collectors.joining("','", "'", "'"));

        conditions.put("appids", appkeyStrs);

        List<Map<String, String>> dataAll = getTotalClickImpression(conditions, "account_today_click_flow");

        BigInteger today_click = new BigInteger("0");
        BigInteger today_impression = new BigInteger("0");
        BigInteger today_all = new BigInteger("0");

        for (Map<String, String> data : dataAll) {
            BigInteger click = new BigInteger(data.get(NUM_CLICK_TOTAL) == null ? "0" : data.get(NUM_CLICK_TOTAL));
            BigInteger impress = new BigInteger(data.get(NUM_IMPRESSION) == null ? "0" : data.get(NUM_IMPRESSION));
            today_click = today_click.add(click);
            today_impression = today_impression.add(impress);

            if (conditions.get("enddate").equals(data.get("dt"))) {
                today_all = today_all.add(new BigInteger(data.get(NUM_CLICK_TOTAL) == null ? "0" : data.get(NUM_CLICK_TOTAL)))
                        .add(new BigInteger(data.get(NUM_IMPRESSION) == null ? "0" : data.get(NUM_IMPRESSION)));
            }
        }

        BigInteger increment_flow = today_click.add(today_impression);

        if (increment_flow.compareTo(new BigInteger("0")) <= 0) {
            //实时流量为0  不更新
            return restrict4Web;
        }

        //记录日志  两天前流量存在被串改情况
        restrict4Web = flowQueyCheck(account, restrict4Web, beforeFlow, appkeyStrs);


        restrict4Web.setModifyTime(new Date());
        restrict4Web.setIncrementFlow(increment_flow);// 实时流量
        restrict4Web.setIncrementToday(today_all);// 当天实时流量

        boolean openPackageFlow = account.getPackageFlowId() != null && account.getPackageFlowId().intValue() != -1;

        if (DateTime.now().isAfter(new DateTime(account.getPackageFlowDs()))
                || DateTime.now().isEqual(new DateTime(account.getPackageFlowDs()))) {
            //流量包开通时间起开始计算流量包流量
            openPackageFlow = openPackageFlow ? true : false;
        } else {
            openPackageFlow = false;
        }


        if (openPackageFlow) {
            //当用户开通流量包
            String todayDs = taday.toString("yyyy-MM-dd");
            String packStart = todayDs.equals(new DateTime(account.getPackageFlowDs()).toString("yyyy-MM-dd"))
                    ? todayDs : taday.plusDays(-1).toString("yyyy-MM-dd");
            BigInteger two = accountFlowRestrictServiceImpl.getTotalNum(packStart, todayDs, appkeyStrs, "account_package_today_click_flow", "click_sum");
            restrict4Web.setPackageIncrementFlow(two == null ? BigInteger.ZERO : two);
            restrict4Web.setPackageIncrementContractFlow(restrict4Web.getPackageIncrementFlow());
        }

        if (account.getNewContract() != null && account.getNewContract()) {
            //对于新合同用户
            if (conditions.get("enddate").equals(account.getModifyPricelevelTime())) {
                // 合同生效当天  只计算当天实时流量
                restrict4Web.setIncrementFlow(today_all);
                restrict4Web.setIncrementToday(today_all);
                restrict4Web.setTrackTotalFlow(BigInteger.ZERO);

                //当用户开通流量包，当天流量包使用量
                if (openPackageFlow) {
                    BigInteger today = accountFlowRestrictServiceImpl.getTotalNum(taday.toString("yyyy-MM-dd"), taday.toString("yyyy-MM-dd"), appkeyStrs, "account_package_today_click_flow", "click_sum");
                    restrict4Web.setPackageIncrementContractFlow(today);
                    restrict4Web.setPackageContractTotalFlow(BigInteger.ZERO);
                }


            } else if (conditions.get("startdate").equals(account.getModifyPricelevelTime())) {
                restrict4Web.setIncrementFlow(increment_flow);// 近两天实时流量
                restrict4Web.setIncrementToday(today_all);// 当天实时流量
                restrict4Web.setTrackTotalFlow(BigInteger.ZERO);
                if (openPackageFlow) {
                    restrict4Web.setPackageContractTotalFlow(BigInteger.ZERO);
                }

            }
        }

        if (type == TODAY_FLOW_ONLY) {
            //获取当天实时流量数据 结束
            return restrict4Web;
        }

        //更新流量信息
        accountRestrict4WebRepository.save(restrict4Web);

        if (restrict4Web.getTrackLimit().compareTo(new BigInteger("0")) > 0) {
            // 非包年用户计算 流量百分比
            double remainPercent = 1.0 -
                    restrict4Web.getTrackTotalFlow().add(restrict4Web.getIncrementFlow()).doubleValue() / restrict4Web.getTrackLimit().doubleValue();
            restrict4Web.setTrackRemainPercent(remainPercent);

            //非包年用户 流量邮件提醒
            restrict4Web = trackLimitNotify(account, restrict4Web, increment_flow);

        }

        if (account.getPackageFlowId() != null && account.getPackageFlowId().intValue() != -1) {
            //流量包用户 流量邮件提醒
            packageFlowNotify(restrict4Web, account);
        }


        if (account.getId().equals(19575L)) {
            AccountFlowHourLog hourLog = accountFlowHourLogRepository.findLatest(account.getId());
            int hourPre = hourLog == null ? -1 : new DateTime(hourLog.getAddTime()).hourOfDay().get();
            int hourNow = DateTime.now().hourOfDay().get();
            if (hourPre == hourNow) {
                //每小时记录一次
                return restrict4Web;
            }
            AccountFlowHourLog flowHourLog = new AccountFlowHourLog();
            flowHourLog.setAddTime(new Date());
            flowHourLog.setDs(DateTime.now().toString("yyyy-MM-dd"));
            flowHourLog.setTrackTotalFlow(restrict4Web.getTrackTotalFlow());
            flowHourLog.setIncrementFlow(restrict4Web.getIncrementFlow());
            flowHourLog.setIncrementToday(restrict4Web.getIncrementToday());
            flowHourLog.setAccountId(restrict4Web.getAccountId());
            accountFlowHourLogRepository.save(flowHourLog);
        }

        return restrict4Web;

    }

    private AccountRestrict4Web flowQueyCheck(Account account, AccountRestrict4Web restrict4Web, BigInteger beforeFlow, String appid) {
        //记录信息防止 两天前流量被串改
        RestrictFlowsLog flowsLog = restrictFlowsLogRepository.getLatesByAccount(restrict4Web.getAccountId());
        String start = StringUtils.isEmpty(account.getModifyPricelevelTime()) ? new DateTime(account.getPubDate()).toString("yyyy-MM-dd") : account.getModifyPricelevelTime();

        if (flowsLog == null) {
            BigInteger clickNum = getTotalNum(start, DateTime.now().plusDays(-2).toString("yyyy-MM-dd"), appid, "account_track_flow_restrict", "click_sum");
            restrict4Web.setTrackTotalFlow(clickNum);
            restrictFlowsLogRepository.save(buildFlowLog(restrict4Web));
        } else {

            boolean reQuery = false;
            if (flowsLog.getQueryFlow() != null
                    && flowsLog.getQueryFlow().doubleValue() != restrict4Web.getTrackTotalFlow().doubleValue()) {
                //流量被 串改
                reQuery = true;
            } else if (!DateTime.now().toString("yyyy-MM-dd").equals(flowsLog.getDsValue())) {
                //记录的是三日前的流量 时
                reQuery = true;
            }

            if (reQuery) {
                //数据被串改 记录log
                BigInteger clickNum = getTotalNum(start, DateTime.now().plusDays(-2).toString("yyyy-MM-dd"), appid.toString(), "account_track_flow_restrict", "click_sum");
                RestrictFlowsLog log = buildFlowLog(restrict4Web);
                log.setBeforeFlow(beforeFlow);
                log.setAfterFlow(restrict4Web.getTrackTotalFlow());
                log.setQueryFlow(clickNum);
                restrictFlowsLogRepository.save(log);
                restrict4Web.setTrackTotalFlow(clickNum);
            }

        }

        return restrict4Web;
    }

    public RestrictFlowsLog buildFlowLog(AccountRestrict4Web byAccountId) {
        RestrictFlowsLog flowsLog = new RestrictFlowsLog();
        flowsLog.setAccountId(byAccountId.getAccountId());
        flowsLog.setAfterFlow(BigInteger.ZERO);
        flowsLog.setBeforeFlow(BigInteger.ZERO);
        flowsLog.setQueryFlow(byAccountId.getTrackTotalFlow());
        flowsLog.setCreatTime(new Date());
        flowsLog.setDsValue(DateTime.now().toString("yyyy-MM-dd"));
        return flowsLog;
    }

    //流量包邮件提醒
    public void packageFlowNotify(AccountRestrict4Web restrict4Web, Account account) {

        // 流量邮件提醒
        if (account.getFlowRestrict() != null && !account.getFlowRestrict()) {
            return;
        }

        PackageFlow packageFlow = packageFlowRepository.findOne(account.getPackageFlowId());
        AccountRestrict4Web flow = new AccountRestrict4Web();
        if (restrict4Web.getPackageIncrementFlow() == null) {
            restrict4Web.setPackageIncrementFlow(BigInteger.ZERO);
        }
        if (account.getAppentPackageFlow() == null) {
            account.setAppentPackageFlow(BigInteger.ZERO);
        }
        flow.setPackageTotalFlow(restrict4Web.getPackageTotalFlow().add(restrict4Web.getPackageIncrementFlow()));
        flow.setPackageLimit(packageFlow.getFlow().add(account.getAppentPackageFlow()));
        flow.setTrackNotifyFlag(restrict4Web.getTrackNotifyFlag());

        if (getPackageFlowUsedPercent(flow, 0.8, 0.9)) {
            sendPackageFlowMails(account, flow, 0.8);
        } else if (getPackageFlowUsedPercent(flow, 0.9, 1.0)) {
            sendPackageFlowMails(account, flow, 0.9);
        } else if (getPackageFlowUsedPercent(flow, 1.0, 1.0)) {
            sendPackageFlowMails(account, flow, 1.0);
        }

        if (flow.getTrackNotifyFlag() != null) {
            restrict4Web.setTrackNotifyFlag(flow.getTrackNotifyFlag());
        }

    }

    //流量包提醒邮件发送
    public void sendPackageFlowMails(Account account, AccountRestrict4Web flow, Double percent) {

        String notifyFlag = flow.getTrackNotifyFlag();
        if (notifyFlag != null && notifyFlag.indexOf("_pfl" + percent) >= 0) {
            //已提示
            return;
        }

        if (notifyFlag == null) {
            notifyFlag = "";
        }

        notifyFlag += "_pfl" + percent;
        flow.setTrackNotifyFlag(notifyFlag);


        String companyName = StringUtils.isEmpty(account.getCompany()) ? "尊敬的客户您好" : "尊敬的" + account.getCompany();
        StringBuffer content = new StringBuffer();
        content.append(companyName);
        content.append(",您购买的流量包套餐");
        content.append(getValueTile(flow.getPackageLimit()));
        if (flow.getPackageTotalFlow().doubleValue() >= flow.getPackageLimit().doubleValue()) {
            //流量已用尽
            content.append("流量已经用尽,为了不影响您的正常使用，请尽快联系商务续订服务。流量包用尽后我们将会默认消耗“广告效果监测”中的套餐流量，继续支持您的数据上报和接收。");
        } else {
            content.append("流量已经使用");
            content.append(getValueTile(flow.getPackageTotalFlow()));
            content.append(",为了不影响您的正常使用，请尽快联系商务续订服务。流量包用尽后我们将会继续消耗“广告效果监测”中的套餐流量继续支持您的数据上报和接收。");
        }

        Map<String, String> logo_txt = tipService.getLogoData(account, "logo_res_taocan", SendCommonPostMail.EMAIL_HOST, SendCommonPostMail.EMAIL_USERNAME, SendCommonPostMail.EMAIL_PASSWORD);

        String subject = logo_txt.get("logo_res_taocan");

        List<String> accountMail = new ArrayList<>();
        List<String> bccList = new ArrayList<>();

        //处理收件人信息
        getBussiness(account, bccList, accountMail);

        Map<String, String> mailConfig = null;
        if (account.getRegSource() != 0) {
            mailConfig = logo_txt;
        }

        try {
            SendCommonPostMail.sendMailReuse(accountMail, subject, content.toString(), null, bccList, mailConfig);
        } catch (Exception e) {
            logger.error("流量包流量提醒 邮件发送失败", e);
        }

    }


    //实时套餐流量邮件提醒
    public AccountRestrict4Web trackLimitNotify(Account rootAccount, AccountRestrict4Web restrict4Web, BigInteger incrementFlow) {
        // 流量邮件提醒
        if (rootAccount.getFlowRestrict() != null && !rootAccount.getFlowRestrict()) {
            return restrict4Web;
        }

        restrict4Web.setPackageFlowId(rootAccount.getPackageFlowId());

        PackageFlow packageFlow = null;
        if (restrict4Web.getPackageFlowId() != null) {
            packageFlow = packageFlowRepository.findOne(restrict4Web.getPackageFlowId());
        }

        BigInteger usedFlows = packageFlowService.getUsedTrackFlow(restrict4Web, rootAccount);

        if (getUsedPercent(restrict4Web, 0.5, 0.6, packageFlow, rootAccount)) {
            // 50%
            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.5);

        } else if (getUsedPercent(restrict4Web, 0.6, 0.7, packageFlow, rootAccount)) {
            // 60%
            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.6);

        } else if (getUsedPercent(restrict4Web, 0.7, 0.8, packageFlow, rootAccount)) {
            // 70%
            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.7);

        } else if (getUsedPercent(restrict4Web, 0.8, 0.9, packageFlow, rootAccount)) {
            // 80%
            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.8);

        } else if (getUsedPercent(restrict4Web, 0.9, 0.95, packageFlow, rootAccount)) {

            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.9);

        } else if (getUsedPercent(restrict4Web, 0.95, 0.99, packageFlow, rootAccount)) {

            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 0.95);

        } else if (getUsedPercent(restrict4Web, 1.0, 1.0, packageFlow, rootAccount)) {

            restrict4Web = sendMail(rootAccount, restrict4Web, usedFlows, 1.0);
        }

        return restrict4Web;


    }

    //当前已使用流量所在百分比区间
    public boolean getUsedPercent(AccountRestrict4Web restrict4Web, Double percent, Double percentEnd, PackageFlow packageFlow, Account account) {
        BigInteger used = packageFlowService.getUsedTrackFlow(restrict4Web, account);
        if (percent == 1.0) {
            return used.doubleValue() >= restrict4Web.getTrackLimit().doubleValue();
        }
        return restrict4Web.getTrackLimit().doubleValue() * percent <= used.doubleValue()
                && restrict4Web.getTrackLimit().doubleValue() * percentEnd > used.doubleValue();

    }


    //流量包已使用流量所在百分比区间
    public boolean getPackageFlowUsedPercent(AccountRestrict4Web restrict4Web, Double percent, Double percentEnd) {
        BigInteger used = restrict4Web.getPackageTotalFlow();
        if (percent == 1.0) {
            return used.doubleValue() >= restrict4Web.getPackageLimit().doubleValue();
        }

        return restrict4Web.getPackageLimit().doubleValue() * percent <= used.doubleValue()
                && restrict4Web.getPackageLimit().doubleValue() * percentEnd > used.doubleValue();

    }


    public String checkNotifyFlag(String restrict4Web, Double percent) {
        // 标记对应百分比区间是否已发邮件提醒
        String notify = restrict4Web;

        if (StringUtils.isEmpty(notify)) {
            notify = "_" + percent;
        } else {

            if (notify.indexOf("_" + percent) < 0) {

                notify += "_" + percent;
            } else {
                notify = null;

            }
        }

        return notify;

    }


    /**
     * 实时流量邮件提醒
     *
     * @param rootAccount
     * @param restrict4Web
     * @param usedTotal
     * @param percent
     * @return
     */

    public AccountRestrict4Web sendMail(Account rootAccount, AccountRestrict4Web restrict4Web, BigInteger usedTotal, Double percent) {

        String notify = checkNotifyFlag(restrict4Web.getTrackNotifyFlag(), percent);

        if (notify == null) {
            // 当前百分比范围 已经流量邮件 提醒，不在提醒
            return restrict4Web;
        }

        Map<String, String> logo_txt = tipService.getLogoData(rootAccount, "logo_res_taocan", SendCommonPostMail.EMAIL_HOST, SendCommonPostMail.EMAIL_USERNAME, SendCommonPostMail.EMAIL_PASSWORD);

        restrict4Web.setTrackNotifyFlag(notify);
        //String subject = rootAccount.getRegSource() == 0 ? "热云数据广告效果监控套餐提醒" : "新华频媒广告效果监控套餐提醒";
        String subject = logo_txt.get("logo_res_taocan");

        List<String> accountMail = new ArrayList<>();
        List<String> bccList = new ArrayList<>();

        //处理收件人信息
        getBussiness(rootAccount, bccList, accountMail);

        try {
            String mailContent;
            if (percent == 0.95 || percent == 1.0) {
                mailContent = getEailContent(rootAccount, usedTotal, 1);
            } else {
                mailContent = getEailContent(rootAccount, usedTotal, 0);
            }

            Map<String, String> mailConfig = null;
            if (rootAccount.getRegSource() != 0) {
                mailConfig = logo_txt;
            }

            //MailUtils.sendHtmlEmailWithCC(subject, mailContent, accountMail, null, bccList,mailConfig);
            SendCommonPostMail.sendMailReuse(accountMail, subject, mailContent, null, bccList, mailConfig);
        } catch (Exception e) {

            logger.error("实时流量邮件错误", e);

        }

        return restrict4Web;
    }


    @Override
    public void getBussiness(Account rootAccount, List<String> bccList, List<String> accountMail) {
        String emialConfig = configParamRepository.findParamsByKey("package_notify_email");

        //密送列表
        //配置的收件人
        if (!StringUtils.isEmpty(emialConfig)) {
            bccList.addAll(Arrays.asList(emialConfig.split(",")));
        }

        //销售收件人
        String salesManLeaderEmail = bussinessManRepository.findEmialWithLeader(rootAccount.getBussinessman());
        String[] busEmails = salesManLeaderEmail == null ? new String[]{} : salesManLeaderEmail.split(",");

        for (String sal : busEmails) {
            // 销售不在配置收件人中时
            if (!bccList.contains(sal)) {
                bccList.add(sal);
            }
        }

        if (Constant.mailNeed != null && Boolean.parseBoolean(Constant.mailNeed)) {
            //正式环境 发送客户邮件
            accountMail.add(rootAccount.getEmail());
        }
    }

    @Override
    public String getEailContent(Account rootAccount, BigInteger totalUse, int type) {

        PackageType packageType = packageTypeRepository.findPackageTypeByRootParent(rootAccount.getId());
        String companyName = StringUtils.isEmpty(rootAccount.getCompany()) ? "尊敬的客户您好" : "尊敬的" + rootAccount.getCompany();

        String pakName = packageType.getPackageName();
        if (pakName != null) {
            pakName = pakName.split("_")[0];
        }

        BigInteger trackTotal = packageType.getTrackFlow().add(BigInteger.valueOf(rootAccount.getAppentTrackFlow()));


        StringBuffer content = new StringBuffer();
        content.append(companyName);
        content.append(",您购买的广告效果监控\"");
        content.append(pakName + "\"中的");
        content.append(getValueTile(trackTotal));


        if (packageType.getTrackFlow().add(BigInteger.valueOf(rootAccount.getAppentTrackFlow())).compareTo(totalUse) < 0) {
            //流量用尽时
            content.append("流量已经用尽,");
            content.append("为了不影响您的正常使用，请尽快联系商务续订服务。流量用尽后我们将会继续支持您的数据上报和接收，但数据展示服务将暂时无法使用。");

        } else {
            content.append("流量已经使用");
            content.append(getValueTile(totalUse));

            ////流量未用尽时
            if (type == 0) {
                content.append("为了不影响您的正常使用，请关注您的套餐流量使用情况，并联系商务续订服务。");
            } else {
                content.append("为了不影响您的正常使用，请尽快联系商务续订服务。流量用尽后我们将会继续支持您的数据上报和接收，但数据展示服务将暂时无法使用。");
            }
        }

        return content.toString();

    }

    private String getValueTile(BigInteger val) {

        if (val.compareTo(new BigInteger("100000000")) >= 0) {
            return new DecimalFormat("#.00").format(val.doubleValue() / 100000000) + "亿";
        } else if (val.compareTo(new BigInteger("10000")) >= 0) {
            return new DecimalFormat("#.00").format(val.doubleValue() / 10000) + "万";
        } else {
            return val.toString();
        }
    }


    //流量使用情况页面 tip 提醒
    @Override
    public Map getUserFlowOrTip(Account loginAccount) {

        if (!loginAccount.getId().equals(loginAccount.getRootParent())) {
            loginAccount = accountRepository.getRootAccountByCreateAccount(loginAccount.getId() + "");
        }

        AccountRestrict4Web restrict4Web = accountRestrict4WebRepository.findByAccountId(loginAccount.getId());
        Map<String, Object> tips = new HashMap();
        String notify = null;


        if (restrict4Web != null && restrict4Web.getTrackLimit() != null && restrict4Web.getTrackLimit().compareTo(new BigInteger("0")) > 0) {

            if (restrict4Web.getIncrementFlow() == null) {
                restrict4Web.setIncrementFlow(new BigInteger("0"));
            }
            if (restrict4Web.getPackageIncrementFlow() == null) {
                restrict4Web.setPackageIncrementFlow(new BigInteger("0"));
            }

            if (restrict4Web.getPackageContractTotalFlow() == null) {
                restrict4Web.setPackageContractTotalFlow(BigInteger.ZERO);
            }
            if (restrict4Web.getPackageIncrementContractFlow() == null) {
                restrict4Web.setPackageIncrementContractFlow(BigInteger.ZERO);
            }


            PackageFlow packageFlow = null;
            if (restrict4Web.getPackageFlowId() != null) {
                packageFlow = packageFlowRepository.findOne(restrict4Web.getPackageFlowId());
            }

            BigInteger totolUsed = packageFlowService.getUsedTrackFlow(restrict4Web, loginAccount);

            tips.put("track_limit", restrict4Web.getTrackLimit());
            tips.put("track_flow_total", totolUsed);

            if (getUsedPercent(restrict4Web, 0.8, 0.9, packageFlow, loginAccount)) {
                // 80%
                notify = checkNotifyFlag(restrict4Web.getTrackTipshowFlag(), 0.8);
                tips.put("percent_flag", 0.8);

            } else if (getUsedPercent(restrict4Web, 0.9, 0.95, packageFlow, loginAccount)) {

                notify = checkNotifyFlag(restrict4Web.getTrackTipshowFlag(), 0.9);
                tips.put("percent_flag", 0.9);

            } else if (getUsedPercent(restrict4Web, 0.95, 0.99, packageFlow, loginAccount)) {

                notify = checkNotifyFlag(restrict4Web.getTrackTipshowFlag(), 0.95);
                tips.put("percent_flag", 0.95);

            } else if (getUsedPercent(restrict4Web, 1.0, 1.0, packageFlow, loginAccount)) {

                notify = checkNotifyFlag(restrict4Web.getTrackTipshowFlag(), 1.0);
                tips.put("percent_flag", 1.0);

            }

            if (notify != null) {
                //当前百分比范围 还未弹窗提示用户
                if (tips.get("percent_flag").toString().equals("1.0") || tips.get("percent_flag").toString().equals("0.95")) {
                    tips.put("tip", getEailContent(loginAccount, totolUsed, 1));
                } else {
                    tips.put("tip", getEailContent(loginAccount, totolUsed, 0));
                }

                restrict4Web.setTrackTipshowFlag(notify);
                accountRestrict4WebRepository.save(restrict4Web);
            }

        }

        if (tips.get("tip") != null) {

            tips.put("show", true);
        } else {

            tips.put("show", false);
        }


        return tips;
    }

    @Override
    public List getTotalClickImpression(Map data, String reportName) {
        Long appId = 1L;
        Map<String, String> conditions = data;
        conditions.put("datatype", "list");
        conditions.put("iscache", Constant.iscache);
        String url = Constant.reportUrl + "/api/trackingio/" + reportName + "/" + appId;
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        List allclick = new ArrayList();
        if (responseJson != null) {
            try {
                JSONObject jsonObject = new JSONObject(responseJson);
                JSONArray array = jsonObject.getJSONArray("val");
                if (array != null) {
                    for (int i = 0; i < array.length(); i++) {
                        Map dataJson = new HashMap();
                        JSONObject jsonObject_val = array.getJSONObject(i);
                        dataJson.put("dt", jsonObject_val.get("dt"));
                        dataJson.put(NUM_IMPRESSION, getJsonAttrData(jsonObject_val, NUM_IMPRESSION));
                        dataJson.put(NUM_CLICK_TOTAL, getJsonAttrData(jsonObject_val, NUM_CLICK_TOTAL));
                        allclick.add(dataJson);
                    }
                }

            } catch (JSONException e) {
                logger.error("实时流量计算错误", e);
            }
        }
        return allclick;
    }

    public String getJsonAttrData(JSONObject jsonObject, String attr) throws JSONException {
        return jsonObject.get(attr) == null ? "0" : jsonObject.get(attr).toString();
    }


    public BigInteger getTotalNum(String createTime, String pastDate, String appids, String reportName, String sumType) {
        Long appId = 1218L;
        Map<String, String> conditions = new HashMap<String, String>();
        conditions.put("appids", appids);
        conditions.put("startdate", createTime);
        conditions.put("enddate", pastDate);
        conditions.put("datatype", "list");
        conditions.put("iscache", Constant.iscache);
        String url = Constant.reportUrl + "/api/trackingio/" + reportName + "/" + appId;
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        HashMap<String, String> resultValMap = new HashMap<>();
        String click_sum = "";
        String s = "0";
        if (responseJson != null) {
            try {
                JSONObject jsonObject = new JSONObject(responseJson);
                Iterator keys = jsonObject.keys();
                while (keys.hasNext()) {
                    String key = (String) keys.next();
                    String value = jsonObject.getString(key);
                    resultValMap.put(key, value);
                }
                String val = resultValMap.get("val");
                JSONArray jsonArray = new JSONArray(val);
                JSONObject jsonObject1 = jsonArray.getJSONObject(0);
                click_sum = jsonObject1.getString(sumType);
                if (org.springframework.util.StringUtils.isEmpty(click_sum) || "0".equals(click_sum)) {
                    click_sum = "0";
                }
                Double aDouble = Double.valueOf(click_sum);
                BigDecimal bigDecimal = new BigDecimal(aDouble);
                s = bigDecimal.toPlainString();
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        return new BigInteger(s);
    }

    public static void main(String[] args) {

        //System.out.println("99_fl0.8__".indexOf("_fl0.8"));

        System.out.println(new BigInteger("110000000001").doubleValue() / 10000
                + "");

    }


}
