package com.reyun.service.impl;

import com.reyun.model.Account;
import com.reyun.model.BlacklistApplication;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.BlacklistApplicationRepository;
import com.reyun.repository.BussinessManRepository;
import com.reyun.repository.ConfigParamRepository;
import com.reyun.service.AccountFlowRestrictService;
import com.reyun.service.BlacklistApplicationCheckService;
import com.reyun.service.TipService;
import com.reyun.util.Constant;
import com.reyun.util.SendCommonPostMail;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

@Service
public class BlacklistApplicationCheckServiceImpl implements BlacklistApplicationCheckService {

    private static Logger logger = LoggerFactory.getLogger(BlacklistApplicationCheckServiceImpl.class);

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    BussinessManRepository bussinessManRepository;

    @Autowired
    ConfigParamRepository configParamRepository;

    @Autowired
    BlacklistApplicationRepository blacklistApplicationRepository;

    @Autowired
    AccountFlowRestrictService accountFlowRestrictService;

    static List<Integer> ranges = Arrays.asList(1, 3, 7, 15);

    @Autowired
    TipService tipService;

    @Override
    public void remainDaysNotify(BlacklistApplication application, int remainDays) {

        if (!ranges.contains(remainDays)) {
            //不在时间点 不发送提醒
            return;
        }

        Account rootAcc = accountRepository.findOne(application.getParentAccount());

        Map<String,String> logo_txt = tipService.getLogoData(rootAcc,"logo_res_fzbtx","logo_res_zhuce",SendCommonPostMail.EMAIL_HOST,SendCommonPostMail.EMAIL_USERNAME,SendCommonPostMail.EMAIL_PASSWORD);

       // String email_suject = rootAcc.getRegSource()==0?"热云数据“高级防作弊”过期提醒":"新华频媒“高级防作弊”过期提醒";
        String email_suject = logo_txt.get("logo_res_fzbtx");

        String company = StringUtils.isEmpty(rootAcc.getCompany()) ? "" : "尊敬的" + rootAcc.getCompany() + ",";

        String mail_content = company + "您购买的“高级防作弊”功能还有" + remainDays + "天(" + application.getPastDate() + ")即将过期，请关注您的使用情况，并联系商务及时续订。";

        //主账号商务信息
        String bus_email = bussinessManRepository.findBussinessManByRootAccount(application.getParentAccount());

        String configEmails = configParamRepository.findParamsByKey("black_app_notify_email");

        if (StringUtils.isEmpty(configEmails)) {
            configEmails = "yangfei@reyun.com";
        }

        List<String> bccList = new ArrayList<>();
        bccList.addAll(Arrays.asList(configEmails.split(",")));

        if (!bccList.contains(bus_email)) {
            //商务不在配置表 则抄送
            bccList.add(bus_email);
        }

        List<String> accountMail = new ArrayList<>();
//            if (Constant.mailNeed != null && Boolean.parseBoolean(Constant.mailNeed)) {
        if (StringUtils.equalsIgnoreCase(Constant.mailNeed, "true")) {
            //正式环境 发送客户邮件
            accountMail.add(application.getParentEmail());
        }

        try {

            Map<String,String> mailConfig = null;
            if(rootAcc.getRegSource()!=0){
                mailConfig = logo_txt;
            }
            //MailUtils.sendHtmlEmailWithCC(email_suject, mail_content, accountMail, null, bccList,mailConfig);
            SendCommonPostMail.sendMailReuse(accountMail, email_suject, mail_content,null,bccList,mailConfig);
        } catch (Exception e) {
            logger.error("防作弊提醒邮件发送错误", e);
        }


    }
}
