package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.model.*;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.BlacklistApplicationRepository;
import com.reyun.repository.BlacklistTimeRepository;
import com.reyun.repository.BussinessManRepository;
import com.reyun.service.AppService;
import com.reyun.service.BlacklistService;
import com.reyun.taskexecute.SendEmailByDefenceFakeThread;
import com.reyun.util.*;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * description:
 *
 * @author nolan
 * @date 25/09/2017
 */
@Service
public class BlacklistServiceImpl implements BlacklistService {
    protected Logger logger = LoggerFactory.getLogger(BlacklistServiceImpl.class);
    @Autowired
    private BlacklistApplicationRepository blacklistApplicationRepository;
    @Autowired
    private BlacklistTimeRepository blacklistTimeRepository;
    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private AppService appService;
    @Autowired
    private BussinessManRepository bussinessManRepository;

    @Override
    public Map<String, Object> limit(Account account) {
        Map<String, Object> res = new HashMap<>();
        boolean flag = false;
        BlacklistApplication bla = blacklistApplicationRepository.findByParent(account.getRootParent());

        if (null == bla) {//未申请
            flag = false;
            res.put("fakeStatusDetail", -1);
        } else if (bla.getStatus().equals(2) || bla.getStatus().equals(1)) {//试用或正式
            String today = DateUtil.getBeforeDays(0);
            res.put("pastDate", bla.getPastDate());
            if (today.compareTo(bla.getPastDate()) <= 0) {
                flag = true;
            }
            res.put("fakeStatusDetail", bla.getStatus());

            //是否显示提示条
            int remianDays = Days.daysBetween(new DateTime(), new DateTime(bla.getPastDate())).getDays()+1;

            if (remianDays <= 3 && !StringUtils.equals(bla.getTipDate(), new DateTime().toString("yyyy-MM-dd"))) {
                res.put("fakeTips", "true");
                res.put("fakeRemianDays", remianDays < 0 ? "0" : remianDays + "");
            } else {
                res.put("fakeTips", "false");
            }

        } else {
            BlacklistApplication past = blacklistApplicationRepository.findByParentAndPast(account.getRootParent());

            if (null != past) { //是否已过期
                res.put("fakeStatusDetail", 3);
                BlacklistApplication one2 = blacklistApplicationRepository.findByAccountAndJustApply(account.getId());

                if (null != one2) {//是否已申请
                    res.put("fakeStatusDetail", one2.getStatus());
                }
            } else { //是否未申请
                res.put("fakeStatusDetail", -1);
                BlacklistApplication one2 = blacklistApplicationRepository.findByAccountAndJustApply(account.getId());

                if (null != one2) {//是否已申请
                    res.put("fakeStatusDetail", one2.getStatus());
                }
            }
        }
        res.put("fakeStatus", flag);
        res.put("msgStatus", false);
        res.put("report", false);
        return res;
    }

    @Override
    public BlacklistApplication create(Account account, BlacklistApplication blacklistApplication) {
        Account parent = accountRepository.findOne(account.getRootParent());
        blacklistApplication.setPubDate(new Date());
        blacklistApplication.setParentAccount(account.getRootParent());
        blacklistApplication.setParentEmail(parent.getEmail());
        blacklistApplication.setParentName(parent.getName());
        blacklistApplication.setCompany(parent.getCompany());
        blacklistApplication.setStatus(0);
        BlacklistApplication save = blacklistApplicationRepository.save(blacklistApplication);
        if(blacklistApplication.getMenu().equals("arouse")){
            String url = "https://oapi.dingtalk.com/robot/send?access_token=2163f280229ca24b31cad3fb5f7d3598b742f3b3ed9049217915e56e336f0304";
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
            String content = "公司名称："+parent.getCompany()+"\n申请邮箱："+account.getEmail()+"\n联系人："
                    +account.getName()+"\n联系电话："+blacklistApplication.getTell()+"\n于"+sdf.format(new Date())+"，申请开通【调起】数据展示";
            DingDingSendMsgUtil dingDing = new DingDingSendMsgUtil();
            dingDing.sendMsg(url,content);
        }else{
            save.setRootAccount(parent);
            SendEmailByDefenceFakeThread t = new SendEmailByDefenceFakeThread(save);
            t.start();
        }
        return save;
    }

    private Long getMoney(Map<String, String> conditions, Map<String, Integer> priceMap) {
        conditions.put("datatype", "list");
        Long money = 0L;
        String url = Constant.reportUrl + "/api/trackingio/black_deviceid/123456";
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        if (responseJson.indexOf("val") != -1) {
            JSONObject rs = null;
            try {
                rs = new JSONObject(responseJson);
                JSONArray valueArr = rs.getJSONArray("val");
                if (valueArr.length() > 0) {
                    for (int i = 0; i < valueArr.length(); i++) {
                        JSONObject val = valueArr.getJSONObject(i);
                        String appid = val.getString("appid");
                        long install = val.getLong("black");
                        money = money + priceMap.get(appid) * install;
                    }
                }
            } catch (JSONException e) {
                logger.error("", e);
            }
        }
        return money;
    }

    private Long getBlack(Map<String, String> conditions) {
        conditions.put("datatype", "list");
        Long black = 0L;
        String url = Constant.reportUrl + "/api/trackingio/black_deviceid/123456";
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        if (responseJson.indexOf("val") != -1) {
            JSONObject rs = null;
            try {
                rs = new JSONObject(responseJson);
                JSONArray valueArr = rs.getJSONArray("val");
                if (valueArr.length() > 0) {
                    for (int i = 0; i < valueArr.length(); i++) {
                        JSONObject val = valueArr.getJSONObject(i);
                        long install = val.getLong("black");
                        black = black + install;
                    }
                }
            } catch (JSONException e) {
                logger.error("", e);
            }
        }
        return black;
    }

    @Override
    public void close(Account account) {
        BlacklistTime blacklistTime = blacklistTimeRepository.findAllByAccount(account.getId());
        if (null == blacklistTime) {
            BlacklistTime bt = new BlacklistTime();
            bt.setAccountId(account.getId());
            bt.setLastTime(DateUtil.getBeforeDays(0));
            bt.setTime(1);
            blacklistTimeRepository.save(bt);
        } else {
            blacklistTime.setLastTime(DateUtil.getBeforeDays(0));
            blacklistTime.setTime(blacklistTime.getTime() + 1);
            blacklistTimeRepository.save(blacklistTime);
        }
    }

    @Override
    public String findTell(Account account) {
        Account one = accountRepository.findOne(account.getRootParent());
        return one.getPhone();
    }

    @Override
    public Map<String, Object> report(Account account) {
        Map<String, Object> result = new HashMap();
        List<App> appList = appService.listAppByAccount(account.getId());
        Map<String, Integer> priceMap = new HashMap<>();
        List<String> list = new ArrayList<>();
        for (App app : appList) {
            list.add(app.getAppkey());
            priceMap.put(app.getAppkey(), "38".equals(app.getAppGenre()) ? 30 : 17);
        }
        String appkeyStr = String.join(",", list);
        String endDate = DateUtil.getBeforeDays(1);
        String satrtDate = DateUtil.getBeforeDays(31);
        Map<String, String> conditions = new HashMap<>();
        conditions.put("startdate", satrtDate);
        conditions.put("enddate", endDate);
        conditions.put("appid", appkeyStr);
        conditions.put("iscache", "0");
        conditions.put("datatype", "list");
        // TODO: 2018/9/28 此处url 拼接 appkeyStr 并非必要，appkeyStr 长度较长时可能严重影响接口性能
        String url = Constant.reportUrl + "/api/trackingio/black_install/" + appkeyStr;
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
        if (responseJson.indexOf("key") != -1) {
            JSONObject rs = null;
            ObjectMapper mapper = new ObjectMapper();
            try {
                rs = new JSONObject(responseJson);
                result = mapper.readValue(rs.toString(), Map.class);
                JSONArray keyArr = rs.getJSONArray("key");
                JSONArray valueArr = rs.getJSONArray("val");
                if (keyArr.length() > 0) {
                    Map<String, Object> mapRes = new HashMap<>();
                    long fakeAll = 0l;
                    long numAll = 0l;
                    long money = 0l;
                    long installAll = 0;
                    for (int i = 0; i < keyArr.length(); i++) {
                        long num = valueArr.getJSONObject(i).getLong("num_click");
                        long install = valueArr.getJSONObject(i).getLong("num_install");
                        installAll = installAll + install;
                        numAll = numAll + num;
                    }
                    money = getMoney(conditions, priceMap);
                    fakeAll = getBlack(conditions);
                    if (installAll != 0) {
                        double v = fakeAll * 100.0 / installAll;
                        mapRes.put("rate_install", String.format("%.2f", v));
                    } else {
                        mapRes.put("rate_install", "0");
                    }
                    mapRes.put("appid", "最近30天");
                    mapRes.put("num_click", numAll);
                    mapRes.put("num_install", installAll);
                    mapRes.put("black", fakeAll);
                    mapRes.put("money", money);
                    List<Object> listRes = new ArrayList<>();
                    listRes.add(mapRes);
                    rs.put("val", listRes);
                    result = mapper.readValue(rs.toString(), Map.class);
                }
            } catch (JSONException e) {
                logger.error("", e);
            } catch (IOException e) {
                logger.error("", e);
            }
        }
        return result;
    }

    @Override
    public BlacklistApplication findOne(Account account) {
        return blacklistApplicationRepository.findByAccountAndJustApply(account.getId());
    }

    @Override
    public BlacklistApplication findOneUsed(Account account) {
        return blacklistApplicationRepository.findByAccount(account.getId());
    }
    @Override
    public BlacklistApplication findOneByParentId(Long parentId) {
        return blacklistApplicationRepository.findByParent(parentId);
    }

    @Override
    public BlacklistApplication renew(Long id, String phone, Account account) {
        BlacklistApplication one = blacklistApplicationRepository.findOne(id);
        one.setPubDate(new Date());
        one.setTell(phone);
        one.setRenew(true);
        if(one.getMenu().equals("arouse")){
            String url = "https://oapi.dingtalk.com/robot/send?access_token=2163f280229ca24b31cad3fb5f7d3598b742f3b3ed9049217915e56e336f0304";
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
            String content = "公司名称："+one.getCompany()+"\n申请邮箱："+account.getEmail()+"\n联系人："
                    +account.getName()+"\n联系电话："+one.getTell()+"\n于"+sdf.format(new Date())+"，申请续订【调起】数据展示";
            DingDingSendMsgUtil dingDing = new DingDingSendMsgUtil();
            dingDing.sendMsg(url,content);
        }else{
            sendEmail(one);
        }
        return blacklistApplicationRepository.save(one);
    }

    private void sendEmail(BlacklistApplication blacklistApplication) {
        String subject = "TrackingIo防作弊续订申请";
        String contents = "<!doctype html> <html><head></head><body> " +
                "Larry：</br>" +
                "在" + DateUtil.format(blacklistApplication.getPubDate()) + "收到 " + (null == blacklistApplication.getCompany() ? "    " : blacklistApplication.getCompany()) + "   " + blacklistApplication.getParentEmail() + "提交的防作弊功能续订申请，联系手机为 " + blacklistApplication.getTell() + "，请反馈至销售同学进行后续跟进。\n" +
                "或至Office快速开通</br>" +
                "<a href=\"" + Constant.adminUrl + "\">" + Constant.adminUrl + "</a>" +
                "</body></html>";
        List<BussinessMan> bussinessList = bussinessManRepository.findAllIncumbency();
        if (ValidateUtil.isValid(bussinessList)) {
            List<String> list = new ArrayList<>();
            for (BussinessMan b : bussinessList) {
                list.add(b.getEmail());
            }
            try {
                //MailUtils.sendHtmlEmail(subject, contents, list);
                SendCommonPostMail.sendMailReuse(list, subject, contents,null,null);
            } catch (Exception e) {
                logger.error("", e);
            }
        }
    }

    @Override
    public Map updateFakeTips(Account loginAccount) {
        BlacklistApplication bla = blacklistApplicationRepository.findByParent(loginAccount.getRootParent());
        bla.setTipDate(new DateTime().toString("yyyy-MM-dd"));
        return new HashMap(){{
            put("tipDate",bla.getTipDate());
        }};
    }

    @Override
    public Map updateArouseTips(Account loginAccount) {
        BlacklistApplication bla = blacklistApplicationRepository.findByParent(loginAccount.getRootParent(),"arouse");
        bla.setTipDate(new DateTime().toString("yyyy-MM-dd"));
        return new HashMap(){{
            put("tipDate",bla.getTipDate());
        }};
    }

    @Override
    public Map<String, Object> hintArouseServiceInfo(Account account) {
            Map<String, Object> res = new HashMap<>();
            boolean flag = false;
            BlacklistApplication bla = blacklistApplicationRepository.findByParentAndArouse(account.getRootParent());
            //arouseStatusDetail：-1：未申请；1：审核中；2：已开通；3：已过期；
            if (null == bla) {//未申请
                flag = false;//这个字段前端没用到，防作弊有这个状态，咱也不敢删除
                res.put("arouseStatusDetail", -1);
            } else if (bla.getStatus().equals(2)) {//已开通
                String today = DateUtil.getBeforeDays(0);
                res.put("pastDate", bla.getPastDate());
                if (today.compareTo(bla.getPastDate()) <= 0) {
                    flag = true;
                }
                res.put("arouseStatusDetail", bla.getStatus());

                //是否显示提示条
                int remianDays = Days.daysBetween(new DateTime(), new DateTime(bla.getPastDate())).getDays()+1;

                if (remianDays <= 3 && !StringUtils.equals(bla.getTipDate(), new DateTime().toString("yyyy-MM-dd"))) {
                    res.put("arouseTips", "true");
                    res.put("arouseRemianDays", remianDays < 0 ? "0" : remianDays + "");
                } else {
                    //预防office定时任务没有执行，处理指标点是否锁定
                    if(remianDays<0)res.put("arouseStatusDetail", 3);

                    res.put("arouseTips", "false");
                }

            } else {//审核中&已过期
                if(bla.getPastDate()==null){
                    res.put("arouseStatusDetail", 1);
                    res.put("arouseTips", "false");
                }else{
                    res.put("arouseStatusDetail", bla.getStatus());
                    res.put("arouseTips", "false");
                }
            }
            res.put("arouseStatus", flag);
            res.put("msgStatus", false);
            res.put("report", false);
            return res;

        }

    public static void main(String[] args) {
        System.out.println(Days.daysBetween(
                new DateTime(2018,12,20,00,41,30),
                new DateTime("2018-12-23")
        ).getDays());
    }
}
