package com.reyun.service.impl;

import com.google.common.base.Function;
import com.google.common.base.Splitter;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.Account;
import com.reyun.model.CamCamgroupMap;
import com.reyun.model.CampaignGroup;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.CamCamgroupMapRepository;
import com.reyun.repository.CampaignGroupRepository;
import com.reyun.repository.CampaignRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.service.CampaignGroupService;
import com.reyun.service.CampaignService;
import com.reyun.util.ValidateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class CampaignGroupServiceImpl implements CampaignGroupService {

	@Autowired
	CampaignGroupRepository campaignGroupRepository;
	@Autowired
	CampaignRepository campaignRepository;
	@Autowired
	ChannelRepository channelRepository;
	@Autowired
	AccountRepository accountrepository;
	@Autowired
	CampaignService campaignService;
	@Autowired
	CamCamgroupMapRepository camCamGroupMapRepository;

	@Override
	public CampaignGroup create(Long appid, CampaignGroup resource,
			Long accountid, String campaigns) {
		resource.setApp(appid);
		resource.setCreateAccount(accountid);
		resource.setAccount(accountid);
		resource.setDelFlag(false);
		resource.setCreateTime(new Date());

		CampaignGroup rtnGroup = this.campaignGroupRepository.save(resource);

		List<Long> ids = this.str2List(campaigns);

		// 批量插入map表
		this.batchInsertCamCamgroupMap(rtnGroup.getId(), ids);

		return rtnGroup;
	}

	@Override
	public CampaignGroup delete(Long id, Long accountid) {

		CampaignGroup resource = this.campaignGroupRepository.findOne(id);

		CampaignGroup result = new CampaignGroup();
		if (resource != null) {
			resource.setDelFlag(true);
			resource.setModifyAccount(accountid);
			resource.setModifyTime(new Date());
			result = this.campaignGroupRepository.save(resource);
			// 删除映射表中活动组相关
			List<CamCamgroupMap> mapList = this.camCamGroupMapRepository
					.listAllByGroupId(id);

			this.camCamGroupMapRepository.delete(mapList);

		}
		return result;

	}

	@Override
	public CampaignGroup update(String campaigns, CampaignGroup campaignGroup) {
		CampaignGroup one = this.campaignGroupRepository.findOne(campaignGroup.getId());
		one.setApp(campaignGroup.getApp());
		one.setName(campaignGroup.getName());
		one.setModifyTime(new Date());
		one.setDeeplink(campaignGroup.isDeeplink());
		
		CampaignGroup result = this.campaignGroupRepository.save(one);
		
		

		// 删除旧的oldCampaignList中的id对应的camCamgroupMap
		List<CamCamgroupMap> oldMap = this.camCamGroupMapRepository
				.listAllByGroupId(campaignGroup.getId());
		this.camCamGroupMapRepository.delete(oldMap);

		// 获得新的推广活动的id组
		List<Long> newCampaignsList = this.str2List(campaigns);

		// 批量插入CamCamgroupMap表
		this.batchInsertCamCamgroupMap(result.getId(), newCampaignsList);
		return result;

	}

	// 判断推广活动组名是否重复
	@Override
	public BigInteger hasExistName(Long app, String name) {
		return this.campaignGroupRepository.hasExistName(app, name);
	}

	// 根据ID获得推广活动组
	@Override
	public CampaignGroup findOneById(Long id) {
		return this.campaignGroupRepository.findOne(id);
	}

	private void batchInsertCamCamgroupMap(Long groupId,
			List<Long> campaignIdList) {
		List<CamCamgroupMap> camCamGroupMapList = new ArrayList<CamCamgroupMap>();

		for (Long id : campaignIdList) {
			CamCamgroupMap temp = new CamCamgroupMap();
			temp.setCampaign(id);
			temp.setCampGroup(groupId);
			camCamGroupMapList.add(temp);
		}
		// 批量插入CamCamGroupMap表
		this.camCamGroupMapRepository.save(camCamGroupMapList);
	}

	// String转List
	private List<Long> str2List(String strs) {
		Iterable<String> stringIterable = Splitter.on(",").omitEmptyStrings()
				.trimResults().split(strs);
		return Lists.newArrayList(Iterables.transform(stringIterable,
				new Function<String, Long>() {
					@Override
					public Long apply(String s) {
						return Long.parseLong(s);
					}
				}));
	}

	@Override
	public List<CampaignGroup> listCamgroupByAccountAndApp(Long appId, Long accountId) {

        List<CampaignGroup> result = new ArrayList<>();
        List<CampaignGroup> campaignGroupList = new ArrayList<>();

        Account account = accountrepository.findOne(accountId);

        if (account.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(account.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(account.getRoleCategory())) {
            //母账号或管理员返回APP下的活动组
            campaignGroupList = this.campaignGroupRepository.listCmapaignGroupWithoutChannelPerson(appId);

        } else {
            // 子账号只能看自己的
            campaignGroupList = this.campaignGroupRepository.listCamgroupByAccount(appId, accountId);
        }

        //查询所有group下面的推广活动
		if(ValidateUtil.isValid(campaignGroupList)){

			for (CampaignGroup temp : campaignGroupList) {

				List<Long> campaignsList = this.camCamGroupMapRepository.findCampaignByGroup(temp.getId());

				if (ValidateUtil.isValid(campaignsList)) {

					StringBuilder sb = new StringBuilder();

					for (int i = 0; i < (campaignsList.size() - 1); i++) {
						sb.append(campaignsList.get(i)).append(",");
					}

					sb.append(campaignsList.get(campaignsList.size() - 1));

                    temp.setCampaigns(sb.toString());

				}

				result.add(temp);
			}
		}

		return result;
	}

	@Override
	public List<Long> listCamCamgroupMapByGroupid(Long groupid) {
		return this.camCamGroupMapRepository.findCampaignByGroup(groupid);
	}

    @Override
    public List<Long> listCampaignIdByGroupIds(List<Long> campaignGroupIdList) {
        return this.camCamGroupMapRepository.findCampaignByGroup(campaignGroupIdList);
    }

    @Override
	public CampaignGroup findByName(Long app, String name) {
		return this.campaignGroupRepository.findByName(app, name);
	}

	@Override
	public List<CampaignGroup> findByKeyword(Long app, String keyword, Long account)
	{
		return this.campaignGroupRepository.findLikeName(app, keyword, account);
	}

	@Override
	public List<CampaignGroup> findAllByKeyword(Long app, String keyword, Account account, Boolean subscribe)
	{
		List<CampaignGroup> likeName;
		if (account.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(account.getRoleCategory())
				|| RoleEnumType.SUB_APP_MANAGER.getKey().equals(account.getRoleCategory())) {
			likeName = this.campaignGroupRepository.findAllLikeName(app, keyword);
		} else{
			likeName = this.campaignGroupRepository.findLikeName(app, keyword, account.getId());
		}
		if(subscribe==null){
			return likeName;
		}else{
			List<CampaignGroup> likeName_new = new ArrayList<>();
			if(subscribe==true){
				for (CampaignGroup campaignGroup : likeName) {
					if(campaignGroup.getType()==1){
						likeName_new.add(campaignGroup);
					}
				}

			}else{
				for (CampaignGroup campaignGroup : likeName) {
					if(campaignGroup.getType()!=1){
						likeName_new.add(campaignGroup);
					}
				}
			}
			return likeName_new;
		}
	}

	@Override
	public boolean valiad(CampaignGroup resource) {
		boolean flag = false;

		CampaignGroup campaignGroup = this.campaignGroupRepository.findByName(resource.getApp(), resource.getName(), resource.getCreateAccount());
		if(null == campaignGroup){
			flag = true;
		}else if(null != resource.getId() && resource.getId().equals(campaignGroup.getId())){
			flag = true;
		}
		return flag;
	}

	@Override
	public boolean nameValiad(CampaignGroup resource, Account loginAccount) {
		boolean flag = false;
		List<CampaignGroup> campaignGroups = this.campaignGroupRepository.findByNameAndApp(resource.getApp(), resource.getName());
		if(campaignGroups.size()==0){
			flag = true;
		}else if(null != resource.getId() && campaignGroups.size()!=0 ){
			for(CampaignGroup campaignGroup:campaignGroups){
				if(resource.getId().equals(campaignGroup.getId())){
					flag = true;
				}
			}
		}
		return flag;
	}
}
