package com.reyun.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.RoleEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.*;
import com.reyun.util.*;
import org.apache.commons.collections.map.HashedMap;
import org.mortbay.util.UrlEncoded;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class CampaignServiceImpl implements CampaignService {
    private static final Logger logger = LoggerFactory.getLogger(CampaignServiceImpl.class);

    @Autowired
    private CampaignRepository campaignRepository;
    @Autowired
    private CampaignExpiredRepository campaignExpiredRepository;
    @Autowired
    private INoSQLDBService iNoSQLDBService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    private ParamRepository paramRepository;
    @Autowired
    private CallbackRepository callbackRepository;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ChannelRepository channelRepository;
    @Autowired
    private AuthService authService;
    @Autowired
    private DataAuthRepository dataAuthRepository;
    @Autowired
    private SubCampaignRepository subCampaignRepository;
    @Autowired
    OnelinkRepository onelinkRepository;
    @Autowired
    SubCampaignService subCampaignService;
    @Autowired
    RoleAuthRepository roleAuthRepository;
    @Autowired
    GdtPlanRepository gdtPlanRepository;
    @Autowired
    PackageSubCampaignRepository packageSubCampaignRepository;
    @Autowired
    PackageSubCampaignService packageSubCampaignService;
    @Autowired
    SubCampaignINoSQLService subCampaignINoSQLService;
    @Autowired
    ConfigParamService configParamService;
    @Autowired
    ChannelAccountRepository channelAccountRepository;
    @Autowired
    AccountRepository accountRepository;
    @Autowired
    MiniProgramService miniProgramService;
    @Autowired
    LogoDicRepository logoDicRepository;
    @Autowired
    AppDeepLinkRepository appDeepLinkRepository;
    @Autowired
    AppDeepLinkWakeRepository appDeepLinkWakeRepository;
    @Autowired
    CampaignWeChatLinkRepository campaignWeChatLinkRepository;
    @Autowired
    CampaignWeChatAccountRepository campaignWeChatAccountRepository;
    @Autowired
    private ExportCampaignConfigRepository exportCampaignConfigRepository;
    @Autowired
    private IqiyiCampaignRepository iqiyiCampaignRepository;

    private final static String CHANNEL_TYPE_CUSTOM = "custom";
    private final static String CHANNEL_TYPE_SEM = "ry_bd";
    private final static String VIVO_API_SIGN_URL = "https://ad-market.vivo.com.cn/test/sign";

    @Override
    public Campaign4Web findById(Long id) {
        Campaign campaign = campaignRepository.findOne(id);

        Campaign4Web campaign4Web = new Campaign4Web(campaign);
        campaign4Web.setBundleid(campaign.getBundleid());
        campaign4Web.setAdvertiserid(campaign.getAdvertiserid());
        campaign4Web.setPack_channelid(campaign.getPack_channelid());
        campaign4Web.setImpressionStop(campaign.getImpressionStop());
        campaign4Web.setSubscribes(campaign.getSubscribes());
        Channel channel = channelRepository.findById(campaign.getChannel());
        if (10454L == channel.getId()) {
            IqiyiCampaign IqiyiCampaign = iqiyiCampaignRepository.getByParent(campaign4Web.getId());
            if (IqiyiCampaign != null) campaign4Web.setAdvertisid(IqiyiCampaign.getAdvertiserid());

        }
        Account account = accountRepository.findRootAccount(campaign.getCreateAccount());

        if (account != null && account.getRegSource() != 0) {
            String nodeurl = logoDicRepository.findDicNodeURL(account.getRegSource(), channel.getId());
            channel.setNodeUrl(nodeurl);
        }

        campaign4Web.setChannelUniqueName(channel.getUniqueName());
        campaign4Web.setChannelNodeUrl(channel.getNodeUrl());

        String trackUrl = Constant.trackurl;
        String impTrackUrl = Constant.trackurl;

        if (channel != null) {

            campaign4Web.setChannelobj(channel);

            if (campaign.getClickLinkType() == null) {
                if (channel.isIshttps()) {
                    campaign4Web.setIshttps(true);
                    trackUrl = trackUrl.replace("http:", "https:");
                }
            } else {
                if (campaign.getClickLinkType()) {
                    campaign4Web.setIshttps(true);
                    trackUrl = trackUrl.replace("http:", "https:");
                }
            }


            if (channel.isImpIsHttps()) {
                campaign4Web.setImpIshttps(true);
                impTrackUrl = impTrackUrl.replace("http:", "https:");
            }

//            //设置SpecialKeys
//            if (!StringUtil.isEmpty(channel.getUniqueName()) && "adwords".equals(channel.getUniqueName())) {
//
//                String specialKeys = "";
//
//                List<Param> paramList = paramRepository.findByCampaignid(id);
//                for (Param param : paramList) {
//                    specialKeys = specialKeys + "&" + param.getKey() + "=" + param.getValue();
//                }
//
//                if (specialKeys.contains("&")) {
//                    specialKeys = specialKeys.substring(1);
//                }
//
//                campaign4Web.setSpecialKeys(specialKeys);
//            }
        }

        //campaignPackId,构建url参数
        if (!StringUtil.isEmpty(campaign4Web.getCampaignpackid())) {

            trackUrl = campaign4Web.getCampaignpackid();

        } else {

            String surl = campaign4Web.getSurl();

            if (null != channel) {

                trackUrl = this.buildUrlParamByChannel(surl, channel, campaign4Web.getOs());
                impTrackUrl = this.buildImpressionUrlParamByChannel(surl, channel, campaign4Web.getOs());

                if (campaign.getClickLinkType() == null) {
                    if (channel.isIshttps()) {
                        campaign4Web.setIshttps(true);
                        trackUrl = trackUrl.replace("http:", "https:");
                    }
                } else {
                    if (campaign.getClickLinkType()) {
                        campaign4Web.setIshttps(true);
                        trackUrl = trackUrl.replace("http:", "https:");
                    }
                }

                if (channel.isImpIsHttps()) {
                    campaign4Web.setImpIshttps(true);
                    impTrackUrl = impTrackUrl.replace("http:", "https:");
                }

            } else {

                trackUrl = trackUrl + "/" + surl;
                impTrackUrl = impTrackUrl + "/" + surl + "?impression=true";
            }
        }

        campaign4Web.setTrackurl(trackUrl);
        campaign4Web.setImpTrackurl(impTrackUrl);

        // 拼接holderids
        List<Long> accountList = this.dataAuthRepository.listAccountByAppChnCam(campaign.getApp(), campaign.getChannel(), campaign.getId());

        if (ValidateUtil.isValid(accountList)) {

            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < (accountList.size() - 1); i++) {
                sb.append(accountList.get(i)).append(",");
            }

            sb.append(accountList.get(accountList.size() - 1));
            campaign4Web.setHolderids(sb.toString());
        }
        if (campaign4Web.getSubscribes() != null && campaign4Web.getSubscribes()) {
            campaign4Web.setImpTrackurl(null);
        }
        return campaign4Web;
    }

    @Override
    public Campaign findOneById(Long id) {
        return this.campaignRepository.findOne(id);
    }

    @Override
    public List<Campaign> findById(Iterable<Long> campids) {
        return campaignRepository.findById(campids);
    }

    @Override
    public List<Campaign> findAllById(Iterable<Long> campids) {
        return campaignRepository.findAllById(campids);
    }

    @Override
    public List<Campaign> findCampaignIdsAndSubscribesById(List<Long> authCampaignList) {
        List<Object[]> items = campaignRepository.findCampaignIdsAndSubscribesById(authCampaignList);

        try (Stream<Object[]> list = items.stream()) {
            return list.map((v) -> {
                //select id,campaignid,subscribes
                Campaign campaign = new Campaign();
                campaign.setId(Long.parseLong(v[0] + ""));
                campaign.setCampaignid(v[1] + "");
                campaign.setSubscribes(v[2] == null || v[2].equals(false) ? false : true);
                return campaign;

            }).collect(Collectors.toList());

        } catch (Exception e) {

        }
        return new ArrayList<>();
    }

    @Override
    public List<Campaign> listCampaignByAppAndChannel(Long appid, Long channelid) {
        return campaignRepository.findByAppAndChannel(appid, channelid);
    }

    @Override
    public List<Campaign> listCampaignByAppAndChannelAll(Long appid, Long channelid, Account loginAccount) {


        List<Campaign> result = campaignRepository.findByAppAndChannelAll(appid, channelid);

        //非母账号,非管理员和子应用管理员时候,筛选出已授权活动
        if (!loginAccount.getIsSuperUser() && loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                && loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            final List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), appid);
            result = Lists.newArrayList(Iterables.filter(result, new Predicate<Campaign>() {
                @Override
                public boolean apply(Campaign campaign4Web) {
                    return authCampaignList.contains(campaign4Web.getId());
                }
            }));
        }

        return result;

    }

    @Override
    public Campaign create(Long appid, Campaign resource) {

        if (Strings.isNullOrEmpty(resource.getName())) {
            throw new TipException("名称不允许为空");
        }

        if (resource.getName().contains("\t")) {
            throw new TipException("名称中存在不合法字符");
        }

        if (Strings.isNullOrEmpty(resource.getSurl())) {
            throw new TipException("短链为空");
        }
        App app = this.appRepository.findOne(appid);

        Channel channel = channelService.findoneById(resource.getChannel());

        String surl = resource.getSurl();
        String channelcategory = channel.getCategory();
        if (!Strings.isNullOrEmpty(channelcategory) && channelcategory.equals("APPMARKET")) {
            surl = app.getAppkey() + ":" + resource.getPack_channelid() + ":" + new Date().getTime();
        }

        if (resource.getSubscribes() != null && resource.getSubscribes()) {
            //查找当前app的预约上线日期是否更改
            if (!app.getOnlineDate().equals(resource.getOnlineDate()) && !app.getWhetherOnline()) {
                Account account = accountRepository.findOne(app.getAccount());
                if (account.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey()) || account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
                    app.setOnlineDate(resource.getOnlineDate());
                    app = appRepository.save(app);
                }
            }
        }
        //在保存推广活动之前处理特殊渠道比如微信mp行为数据源的判断，这里必须保证数据源正确才可以创建活动
        if (resource.getChannel().equals(5306L)) {
            String result = createUserActionSetId(app, resource, surl);
            if (!result.equals("success")) {
                throw new TipException(result);
            }
        }
        if (resource.getMipType() != null) {
            if (resource.getMipType().equals("link")) {
                String url = resource.getUrl() + "?cid=" + channel.getId() + "&surl=" + resource.getSurl();
                resource.setUrl(url);
                resource.setQrCodeAddress(url);
                //create by 2018.07.17（此处小游戏特殊处理：url只是？+以后的参数，其他均为完整的）
                resource = operateWgShortUrl(resource);
                //----------------------------------------------------------end
            } else if (resource.getMipType().equals("miniCode") || resource.getMipType().equals("miniQrCode")) {
                //create by 2018.07.17（此处小游戏特殊处理：url只是？+以后的参数，其他均为完整的）
                resource = operateWgShortUrl(resource);
                //----------------------------------------------------------end
                Map<Integer, String> qrCodeMap = miniProgramService.getQrCode(app, channel.getId(), resource.getUrl(), resource.getMipType(), resource.getSurl());
                if (qrCodeMap.get(201) == null) {
                    String fileName = qrCodeMap.get(200);
                    resource.setQrCodeAddress(fileName);
                } else {
                    throw new TipException(qrCodeMap.get(201));
                }
            }
        }

        //不用考虑特殊渠道比如微信mp等活动创建，前端控制不传deeplink对象即可
        AppDeepLink appDeepLink = resource.getAppDeepLink();
        //保存deeplink配置，安卓或ios，deeplink对象不为空
        if (appDeepLink != null && (app.getPlatform().toLowerCase().equals("ios") || app.getPlatform().toLowerCase().equals("android"))) {
            //启用deeplink的活动(app的)appkey存库给实时使用
            AppDeepLinkWake wake = appDeepLinkWakeRepository.findByAppkey(app.getAppkey());
            if (wake == null) {
                AppDeepLinkWake linkWake = new AppDeepLinkWake();
                linkWake.setRootAccountId(app.getRootParent());
                linkWake.setAppkey(app.getAppkey());
                linkWake.setCreateDate(DateUtil.getBeforeDays(0));
                appDeepLinkWakeRepository.save(linkWake);
            }

            //AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByLinkId(appDeepLink.getLinkId());
            AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByAppId(app.getId());
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
            }
            //顺序不能变
            String download = UrlEncoded.encodeString(resource.getUrl());
            String scheme = UrlEncoded.encodeString(appDeepLink.getUrlScheme());
            if (appDeepLink.getCallType() == 0) {
                dbAppDeepLink.setMyDoMain("https://urlschemecommon." + Constant.deeplinkHost + "/ry?ry_dp=" + scheme + "&download=" + download);
            } else if (appDeepLink.getCallType() == 1) {
                String path = appDeepLink.getPaths() == null ? "" : appDeepLink.getPaths();
                if (app.getPlatform().toLowerCase().equals("android")) {
                    dbAppDeepLink.setMyDoMain("https://" + appDeepLink.getLinkId() + "." + Constant.deeplinkHost + "/ry/" + path + "?ry_dp=" + scheme + "&download=" + download);
                } else {
                    dbAppDeepLink.setMyDoMain("https://" + appDeepLink.getLinkId() + "." + Constant.deeplinkHost + "/ry/" + path + "?ry_sc=" + scheme + "&download=" + download);
                }
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
                dbAppDeepLink.setPaths(appDeepLink.getPaths());
            } else {
                if (dbAppDeepLink.getPaths() == null) {
                    dbAppDeepLink.setPaths(appDeepLink.getPaths());
                } else {
                    if (appDeepLink.getPaths() != null && !dbAppDeepLink.getPaths().contains(appDeepLink.getPaths()))
                        dbAppDeepLink.setPaths(dbAppDeepLink.getPaths() + "," + appDeepLink.getPaths());
                }
            }
            //修改活动的url，活动修改回显时候需要查找替换
            resource.setUrl(dbAppDeepLink.getMyDoMain());
            appDeepLinkRepository.save(dbAppDeepLink);
            try {
                //同步deeplink至dynampdb
                if (app.getPlatform().toLowerCase().equals("ios")) {
                    //ios修改paths
                    if (appDeepLink.getCallType() != 0) {
                        //开启了Universal Links
                        iNoSQLDBService.updateIOSPaths(dbAppDeepLink);
                    }
                }
            } catch (Exception e) {
                logger.error("同步deeplink发生错误", e);
            }
        }/*else{
            //mysql逻辑删除、ddb真删除
            AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByAppId(app.getId());
            if(dbAppDeepLink!=null){
                appDeepLinkRepository.delete(dbAppDeepLink.getId());
                iAwsDynamoDBService.deleteDeepLinkByLinkId(dbAppDeepLink.getLinkId());
            }

        }*/


        // 1. 保存推广活动
        resource.setOs(app.getPlatform());
        resource.setSurl(surl);
        resource.setApp(app.getId());
        resource.setImpressionStop(0);// 展示默认启用
        if (resource.getOs() != null && "wx".equals(resource.getOs()) || AppPlatformEnum.WG.getKey().equals(resource.getOs())) {
            resource.setCampaignid(surl);
        } else {
            resource.setCampaignid(ShortUrlGenerator.md5(surl));
        }
        resource.setChannelCategory(channel.getCategory());
        resource.setChannelType(channel.getType());
        resource = this.campaignRepository.save(resource);
        //爱奇艺_分包默认创建一个无法编辑的模糊归因子活动
        if (10454L == resource.getChannel() && !StringUtil.isEmpty(resource.getAdvertisid())) {
            Account account = accountRepository.findOne(resource.getAccount());
            PackageSubCampaign packageSubCampaign = new PackageSubCampaign();
            packageSubCampaign.setCreateTime(new Date());
            packageSubCampaign.setAccount(account.getId());
            packageSubCampaign.setUrl(resource.getUrl());
            packageSubCampaign.setParent(resource.getId());
            packageSubCampaign.setParentCampaignid(resource.getCampaignid());
            packageSubCampaign.setBatchNumber(1);
            packageSubCampaign.setPackChannelid(resource.getPack_channelid());
            packageSubCampaign.setChannel(resource.getChannel());
            packageSubCampaign.setName("爱奇艺模糊归因");
            packageSubCampaign.setApp(resource.getApp());
            PackageSubCampaign save = packageSubCampaignService.save(packageSubCampaign, account);
            IqiyiCampaign iqiyiCampaign = new IqiyiCampaign();
            iqiyiCampaign.setAdvertiserid(resource.getAdvertisid());
            iqiyiCampaign.setAppkey(appRepository.findOne(save.getApp()).getAppkey());
            iqiyiCampaign.setCampaignid(save.getCampaignid());
            iqiyiCampaign.setCreateAccount(account.getId());
            iqiyiCampaign.setPackChannelid(save.getPackChannelid());
            iqiyiCampaign.setRootAccount(account.getRootParent());
            iqiyiCampaign.setSurl(save.getSurl());
            iqiyiCampaignRepository.save(iqiyiCampaign);
        }

        // 2. 同步创建短链信息至aws-dynamodb
        if (resource.getId() != null) {
            boolean isAwsSucces = true;
            try {
                final SurlModel transform = transform(resource, app, channel);
                if (!Strings.isNullOrEmpty(transform.getSurl())//短链
                        && !Strings.isNullOrEmpty(transform.getCampaignid())//推广活动id
                        && !Strings.isNullOrEmpty(transform.getAppid())//应用id
                        && !Strings.isNullOrEmpty(transform.getCid())) {//渠道id
                    isAwsSucces = iNoSQLDBService.create(transform);
                }
            } catch (Exception e) {
                isAwsSucces = false;
                logger.error("同步短链发生错误", e);
            }
            this.campaignRepository.updateAwsStatus(resource.getId(), isAwsSucces);
            resource.setAwssuccess(isAwsSucces);
        }

        // 4. 特殊渠道处理
        if (!Strings.isNullOrEmpty(channel.getUniqueName()) && "adwords".equals(channel.getUniqueName())
                && !Strings.isNullOrEmpty(resource.getSpecialKeys())) {

            Callback callback = callbackRepository.findByKey(channel.getUniqueName());

            if (callback != null) {

                List<Param> saveList = Lists.newArrayList();
                List<String> itemList = Splitter.on("&").omitEmptyStrings().trimResults().splitToList(resource.getSpecialKeys());

                for (String param : itemList) {
                    String[] paramItem = param.split("=");
                    Param paramModel = new Param();
                    paramModel.setAppId(app.getAppkey());
                    paramModel.setKey(paramItem[0]);
                    paramModel.setValue(paramItem[1]);
                    paramModel.setCallbackId(callback.getId());
                    paramModel.setCampaignid(resource.getId());
                    saveList.add(paramModel);
                }

                paramRepository.save(saveList);
            }
        }

        // 5. 授权应用,授权给选择的子账号
        String subAccountIds = resource.getHolderids();
        if (ValidateUtil.isValid(subAccountIds)) {

            String[] subAccountIdArray = subAccountIds.split(",");
            for (String subAccountId : subAccountIdArray) {

                DataAuth dataAuth = new DataAuth();
                dataAuth.setAccount(Long.parseLong(subAccountId));
                dataAuth.setApp(appid);
                dataAuth.setCampaign(resource.getId());
                dataAuth.setChannel(resource.getChannel());
                dataAuth.setAllCampaign(false);
                dataAuth.setDelFlag(false);

                this.dataAuthRepository.save(dataAuth);
            }
        }
        return resource;
    }

    public String createUserActionSetId(App app, Campaign resource, String surl) {
        Account account = accountRepository.findOne(app.getAccount());
        String specialKeys = resource.getSpecialKeys();
        if (!Strings.isNullOrEmpty(specialKeys)) {
            String[] split = specialKeys.split("&");
            Map<String, String> specialKeyMap = new HashMap<>();
            for (String s : split) {
                String[] split1 = s.split("=");
                if (split1.length == 2) {
                    specialKeyMap.put(split1[0], split1[1]);
                } else {
                    specialKeyMap.put(split1[0], null);
                }
            }
            String mpappid = specialKeyMap.get("mpappid");
            String mpappsecret = specialKeyMap.get("mpappsecret");
            String mpmobile_app_id = specialKeyMap.get("mpmobile_app_id");
            String mpaccess_token_url = specialKeyMap.get("mpaccess_token");
            String mptoken = null;
            Object mptokenOBJ = null;
            String response = null;
            //先查询库里token是否已存在
            // CampaignWechatAccount weChatAccount = campaignWeChatAccountRepository.findByMpappidAndMpappsecret(mpappid,mpappsecret);
            CampaignWechatAccount weChatAccount = campaignWeChatAccountRepository.findByMpappid(mpappid, account.getRootParent());
            if (resource.getId() != null && weChatAccount != null) {
                //根据活动id确认是否是修改行为,然后进行修改
                campaignWeChatAccountRepository.updateAccessTokenUrlAndMpappsecretById(mpaccess_token_url, mpappsecret, weChatAccount.getId(), DateUtil.getCurrentDateStr(DateUtil.C_TIME_PATTON_DEFAULT));
                weChatAccount = campaignWeChatAccountRepository.findOne(weChatAccount.getId());
            }
            if (weChatAccount != null) {
                mptoken = weChatAccount.getMpaccesstoken();
            } else {
                //不存在则去微信/中控获取token，成功后添加至库
                if (!Strings.isNullOrEmpty(mpaccess_token_url)) {
                    //请求用户提供的中控台获取token
                    response = HttpClientUtil.doHttpGetRequest(mpaccess_token_url, "", 2000);
                    JSONObject requestData = JSONObject.parseObject(response);
                    mptokenOBJ = requestData.getJSONObject("data").get("access_token");
                } else {
                    //直接从微信请求token
                    response = HttpClientUtil.doHttpGetRequest("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + mpappid + "&secret=" + mpappsecret, "", 2000);
                    JSONObject jsonObject = JSONObject.parseObject(response);
                    mptokenOBJ = jsonObject.get("access_token");
                }
                if (mptokenOBJ == null) {
                    return response;
                }
                mptoken = mptokenOBJ.toString();
                CampaignWechatAccount campaignWeChatAccount = new CampaignWechatAccount();
                campaignWeChatAccount.setMpaccesstoken(mptoken);
                campaignWeChatAccount.setMpappid(mpappid);
                campaignWeChatAccount.setMpappsecret(mpappsecret);
                campaignWeChatAccount.setMpaccess_token_url(mpaccess_token_url);
                campaignWeChatAccount.setRootAccountId(account.getRootParent());
                campaignWeChatAccountRepository.save(campaignWeChatAccount);
            }

            //获取行为数据源
            Map<String, String> map = new HashMap<>();
            map.put("type", app.getPlatform().toUpperCase());
            map.put("mobile_app_id", mpmobile_app_id);
            map.put("name", app.getName() + "_" + resource.getName());
            map.put("description", "");
            response = HttpClientUtil.doHttpPostRequest("https://api.weixin.qq.com/marketing/user_action_sets/add?version=v1.0&access_token=" + mptoken, "", map);
            JSONObject jsonResponse = JSONObject.parseObject(response);
            JSONObject dataSource = jsonResponse.getJSONObject("data");
            String user_action_set_id = null;
            //如果没有获取到则再请求微信接口查询
            if (dataSource != null) {
                    /*//保证数据源唯一
                    CampaignWechatLink weChatLink =  campaignWeChatLinkRepository.findByUserActionSetId(dataSource.get("user_action_set_id").toString());
                    if(weChatLink!=null){
                        //行为数据源重复,则不再添加
                        return "success";
                    }*/
                user_action_set_id = dataSource.get("user_action_set_id").toString();

            } else {
                String errcode = jsonResponse.get("errcode").toString();
                String errmsg = jsonResponse.get("errmsg").toString();
                if ("900351000".equals(errcode) && errmsg.contains("registered")) {
                    user_action_set_id = errmsg.split(":")[1].trim();
                } else {
                    return response;
                }
            }
            CampaignWechatLink campaignWeChatLink = new CampaignWechatLink();
            campaignWeChatLink.setAppkey(app.getAppkey());
            if (resource.getOs() != null && "wx".equals(resource.getOs()) || AppPlatformEnum.WG.getKey().equals(resource.getOs())) {
                campaignWeChatLink.setCampaignid(surl);
            } else {
                campaignWeChatLink.setCampaignid(ShortUrlGenerator.md5(surl));
            }
            campaignWeChatLink.setUser_action_set_id(user_action_set_id);
            campaignWeChatLink.setMpappid(mpappid);
            campaignWeChatLink.setStatus(0);
            campaignWeChatLink.setRootAccountId(account.getRootParent());
            //先查询是否已有该活动行为数据源的数据，有则修改（赋值id即可），没有即save新增
            CampaignWechatLink ca = campaignWeChatLinkRepository.findByCampaignId(campaignWeChatLink.getCampaignid());
            if (ca != null) {
                campaignWeChatLink.setId(ca.getId());
            }
            campaignWeChatLinkRepository.save(campaignWeChatLink);
        } else {
            return "缺失参数!";
        }
        return "success";
    }

    /**
     * 功能描述:定时任务刷新微信MP-token
     *
     * @author liyin
     * @date 2019/9/29
     */
    public void autoRefreshWeChatMPToken() {

            List<CampaignWechatAccount> all = campaignWeChatAccountRepository.findAll();
            for (CampaignWechatAccount campaignWeChatAccount : all) {
                try {
                    String response = null;
                    String mptoken = null;
                    Object mptokenOBJ = null;
                    for (int i = 0; i < 8; i++) {
                        if (!StringUtil.isEmpty(campaignWeChatAccount.getMpaccess_token_url())) {
                            response = HttpClientUtil.doHttpGetRequest(campaignWeChatAccount.getMpaccess_token_url(), "", 2000);
                            if (response != null) {
                                JSONObject requestData = JSONObject.parseObject(response);
                                mptokenOBJ = requestData.getJSONObject("data").get("access_token");
                            }
                        } else {
                            response = HttpClientUtil.doHttpGetRequest("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + campaignWeChatAccount.getMpappid() + "&secret=" + campaignWeChatAccount.getMpappsecret(), "", 2000);
                            if (response != null) {
                                JSONObject jsonObject = JSONObject.parseObject(response);
                                mptokenOBJ = jsonObject.get("access_token");
                            }
                        }
                        if (mptokenOBJ != null) {
                            mptoken = mptokenOBJ.toString();
                            campaignWeChatAccountRepository.updateTokenAndDateById(campaignWeChatAccount.getId(), mptoken, DateUtil.getCurrentDateStr(DateUtil.C_TIME_PATTON_DEFAULT));
                            break;
                        }
                    }
                    if (mptokenOBJ == null) {
                        logger.info("微信MP-token刷新失败：" + response);
                        DingDingSendMsgUtil dingding = new DingDingSendMsgUtil();
                        Account one = accountRepository.findOne(campaignWeChatAccount.getRootAccountId());
                        List<CampaignWechatLink> list = campaignWeChatLinkRepository.findByMpAappId(campaignWeChatAccount.getMpappid());
                        if(!CollectionUtils.isEmpty(list)){
                            List<String> campaignids = list.stream().map(CampaignWechatLink::getCampaignid).collect(Collectors.toList());
                            List<String> surls = campaignRepository.findAllSurlBydDelFlagAndCampaignids(false, campaignids);
                            if (response == null) response = "";
                            if (!CollectionUtils.isEmpty(surls)){
                                dingding.sendMsg("https://oapi.dingtalk.com/robot/send?access_token=aba28788ae0d6dca9936b64919c13e6ab95e3e04a0af1b63fadb2124a947f66b", "微信MPtoken刷新失败：\n" + "[主]" + one.getName() + "：" + one.getEmail() + "\nappid:" + campaignWeChatAccount.getMpappid() + "\naccess_token_url:" + campaignWeChatAccount.getMpaccess_token_url() + "\n失败信息：" + response.replace("\"", "'") + "\n\n涉及短链：" + surls);
                            }
                        }
                    }
                } catch (Exception e) {
                    logger.warn(e.getMessage());
                }
            }


        List<Object[]> allSynDDB = campaignWeChatAccountRepository.findAllSynDDB();
        Map<String, String> map = new HashMap<>();
        for (Object[] objects : allSynDDB) {
            if (objects.length == 4) {
                String key = objects[0] + ":" + objects[1];
                String value = objects[2] + ":" + objects[3];
                map.put(key, value);
            }
        }
        NoSQLDBUtil.writeItem("tkio_setting_new",
                new NoSQLDBUtil.Result().withPrimaryKey("id", "WxCallbacks").withMap("setting", map));
    }

    @Override
    public void UpdateUrlByAppId(Long appid, String oldLinkId, String newLinkId) {
        campaignRepository.UpdateUrlByAppId(appid, oldLinkId, newLinkId);
    }

    @Override
    public List<Campaign> findByAppIdAndLikeUrl(Long appid, String linkId) {
        return campaignRepository.findByAppIdAndLikeUrl(appid, linkId);
    }

    @Override
    public List<Campaign> createBatch(Long appid, Iterable<String> surls, Campaign resource) {

        List<Campaign> rtnList = Lists.newArrayList();
        Iterator<String> iterator = surls.iterator();
        App app = appRepository.findOne(appid);
        if (resource.getSubscribes() != null && resource.getSubscribes()) {
            //查找当前app的预约上线日期是否更改
            if (!app.getOnlineDate().equals(resource.getOnlineDate()) && !app.getWhetherOnline()) {
                Account account = accountRepository.findOne(app.getAccount());
                if (account.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey()) || account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
                    app.setOnlineDate(resource.getOnlineDate());
                    app = appRepository.save(app);
                }
            }
        }
        int i = resource.getStartNumber() - 1;
        while (iterator.hasNext()) {
            i++;
            String surl = iterator.next();

            Campaign campaign = new Campaign();
            campaign.setApp(appid);
            campaign.setName(resource.getName() + "_" + i);
            campaign.setSurl(surl);
            campaign.setUrl(resource.getUrl());
            campaign.setAccount(resource.getAccount());
            campaign.setChannel(resource.getChannel());
            campaign.setCreateTime(new Date());
            campaign.setCreateAccount(resource.getCreateAccount());
            campaign.setSpecialKeys(resource.getSpecialKeys());
            campaign.setHolderids(resource.getHolderids());
            campaign.setAppDeepLink(resource.getAppDeepLink());
            campaign.setClickLinkType(resource.getClickLinkType());
            campaign.setSubscribes(resource.getSubscribes());
            campaign.setOnlineDate(resource.getOnlineDate());
            //create by 2018.07.17（此处小游戏特殊处理：url只是？+以后的参数，其他均为完整的）
            campaign = operateWgShortUrl(campaign);
            //----------------------------------------------------------end
            campaign = this.create(appid, campaign);

            rtnList.add(campaign);
        }

        return rtnList;
    }

    @Override
    public Campaign update(Account loginAccount, Long appid, Campaign resource) {


        Campaign retrievedResource = this.campaignRepository.findOne(resource.getId());
        App app = this.appRepository.findOne(appid);

        //在保存推广活动之前处理特殊渠道比如微信mp行为数据源的判断，这里必须保证数据源正确才可以创建活动
        if (resource.getChannel().equals(5306L)) {
            String result = createUserActionSetId(app, resource, retrievedResource.getSurl());
            if (!result.equals("success")) {
                throw new TipException(result);
            }
        }
        if (resource.getSubscribes() != null && resource.getSubscribes()) {
            //查找当前app的预约上线日期是否更改
            if (!app.getOnlineDate().equals(resource.getOnlineDate()) && !app.getWhetherOnline()) {
                Account account = accountRepository.findOne(app.getAccount());
                if (account.getRoleCategory().equals(RoleEnumType.SUPER_USER.getKey()) || account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey()) || account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
                    app.setOnlineDate(resource.getOnlineDate());
                    app = appRepository.save(app);
                }
            }
        }
        //deeplink同步到ddb
        AppDeepLink appDeepLink = resource.getAppDeepLink();
        //保存deeplink配置，安卓或ios，deeplink对象不为空
        if (appDeepLink != null && (app.getPlatform().toLowerCase().equals("ios") || app.getPlatform().toLowerCase().equals("android"))) {
            AppDeepLink dbAppDeepLink = null;
            if (appDeepLink.getId() != null) {
                AppDeepLink one = appDeepLinkRepository.findOne(appDeepLink.getId());
                dbAppDeepLink = one;
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
            }
            //顺序不能变
            String download = UrlEncoded.encodeString(resource.getUrl());
            String scheme = UrlEncoded.encodeString(appDeepLink.getUrlScheme());
            if (appDeepLink.getCallType() == 0) {
                dbAppDeepLink.setMyDoMain("https://urlschemecommon." + Constant.deeplinkHost + "/ry?ry_dp=" + scheme + "&download=" + download);
            } else if (appDeepLink.getCallType() == 1) {
                String path = appDeepLink.getPaths() == null ? "" : appDeepLink.getPaths();
                if (app.getPlatform().toLowerCase().equals("android")) {
                    dbAppDeepLink.setMyDoMain("https://" + dbAppDeepLink.getLinkId() + "." + Constant.deeplinkHost + "/ry/" + path + "?ry_dp=" + scheme + "&download=" + download);
                } else {
                    dbAppDeepLink.setMyDoMain("https://" + dbAppDeepLink.getLinkId() + "." + Constant.deeplinkHost + "/ry/" + path + "?ry_sc=" + scheme + "&download=" + download);
                }
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
                dbAppDeepLink.setPaths(appDeepLink.getPaths());
            } else {
                if (dbAppDeepLink.getPaths() == null) {
                    dbAppDeepLink.setPaths(appDeepLink.getPaths());
                } else {
                    if (appDeepLink.getPaths() != null && !dbAppDeepLink.getPaths().contains(appDeepLink.getPaths()))
                        dbAppDeepLink.setPaths(dbAppDeepLink.getPaths() + "," + appDeepLink.getPaths());
                }
            }
            //修改活动的url，活动修改回显时候需要查找替换
            resource.setUrl(dbAppDeepLink.getMyDoMain());
            appDeepLinkRepository.save(dbAppDeepLink);
            try {
                //同步deeplink至dynampdb
                if (app.getPlatform().toLowerCase().equals("ios")) {
                    //ios，修改不需要同步
                    if (appDeepLink.getCallType() != 0) {
                        //开启了Universal Links     ios修改paths
                        iNoSQLDBService.updateIOSPaths(dbAppDeepLink);
                    }
                }
            } catch (Exception e) {
                logger.error("修改同步deeplink发生错误", e);
            }
        }/*else{
            AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByAppId(app.getId());
            if(dbAppDeepLink!=null){
                appDeepLinkRepository.delete(dbAppDeepLink.getId());
                iAwsDynamoDBService.deleteDeepLinkByLinkId(dbAppDeepLink.getLinkId());
            }
        }
*/
        //修复修改推广活动时候报错 add by sunhao 2017-04-05
        resource.setCampaignid(retrievedResource.getCampaignid());

        retrievedResource.setChannel(resource.getChannel());
        retrievedResource.setName(resource.getName());
        retrievedResource.setCallback(resource.getCallback());
        retrievedResource.setUrl(resource.getUrl());
        retrievedResource.setSurl(resource.getSurl());
        retrievedResource.setOs(resource.getOs());
        retrievedResource.setApp(resource.getApp());
        retrievedResource.setBundleid(resource.getBundleid());
        retrievedResource.setAdvertiserid(resource.getAdvertiserid());
        retrievedResource.setSpecialKeys(resource.getSpecialKeys());
        retrievedResource.setModifyAccount(resource.getModifyAccount());
        retrievedResource.setModifyTime(resource.getModifyTime());
        retrievedResource.setPack_channelid(resource.getPack_channelid());
        if (resource.getSubscribes() != null && resource.getSubscribes()) {
            retrievedResource.setSubscribes(resource.getSubscribes());
        }
        resource.setVivo2DDb(false);
//        if(resource.getChannel().equals(767L) || resource.getChannel().equals(13296L)){
        if (resource.getChannel().equals(13296L)) {
            resource.setVivo2DDb(true);
        }

        // 1. 保存推广活动
        Campaign result = this.campaignRepository.save(retrievedResource);
        packageSubCampaignRepository.updatePackChannelid(result.getId(), result.getPack_channelid());
        if (10454L == result.getChannel()) {
            IqiyiCampaign iqiyiCampaign = iqiyiCampaignRepository.getByParent(result.getId());
            if (StringUtil.isEmpty(resource.getAdvertisid())) {
                packageSubCampaignRepository.deleteIqiyiByParent(result.getId());
                iqiyiCampaignRepository.deleteByParent(result.getId());
            } else if (!StringUtil.isEmpty(resource.getAdvertisid())) {
                if (iqiyiCampaign == null) {
                    PackageSubCampaign packageSubCampaign = new PackageSubCampaign();
                    packageSubCampaign.setCreateTime(new Date());
                    packageSubCampaign.setAccount(loginAccount.getId());
                    packageSubCampaign.setUrl(result.getUrl());
                    packageSubCampaign.setParent(result.getId());
                    packageSubCampaign.setParentCampaignid(result.getCampaignid());
                    packageSubCampaign.setBatchNumber(1);
                    packageSubCampaign.setPackChannelid(result.getPack_channelid());
                    packageSubCampaign.setChannel(result.getChannel());
                    packageSubCampaign.setName("爱奇艺模糊归因");
                    packageSubCampaign.setApp(result.getApp());
                    PackageSubCampaign save = packageSubCampaignService.save(packageSubCampaign, loginAccount);
                    IqiyiCampaign ic = new IqiyiCampaign();
                    ic.setAdvertiserid(resource.getAdvertisid());
                    ic.setAppkey(appRepository.findOne(save.getApp()).getAppkey());
                    ic.setCampaignid(save.getCampaignid());
                    ic.setCreateAccount(loginAccount.getId());
                    ic.setPackChannelid(save.getPackChannelid());
                    ic.setRootAccount(loginAccount.getRootParent());
                    ic.setSurl(save.getSurl());
                    iqiyiCampaignRepository.save(ic);
                } else {
                    iqiyiCampaign.setPackChannelid(resource.getPack_channelid());
                    iqiyiCampaign.setAdvertiserid(resource.getAdvertisid());
                    iqiyiCampaignRepository.save(iqiyiCampaign);
                }
            }
        }
        //2. 同步更新短链信息至aws-dynamodb
        if (Constant.awsneed) {
            Channel channel = channelService.findoneById(resource.getChannel());
            resource.setImpressionStop(retrievedResource.getImpressionStop());// 取值
            SurlModel surlModel = transform(resource, app, channel);
            if (resource.isVivo2DDb()) {
                surlModel.setSpecialkeys("");
            }

            if (!this.iNoSQLDBService.update(surlModel)) {
                this.campaignRepository.updateAwsStatus(resource.getId(), false);
            }
        }

        //母账号和管理员时候,操作授权
        if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(loginAccount.getRoleCategory())) {
            // 3.1 授权应用,删除老授权
            List<DataAuth> oldDataAuthList = this.dataAuthRepository.listDataAuthByAppChnCam(appid, resource.getChannel(), resource.getId());
            this.dataAuthRepository.delete(oldDataAuthList);

            // 3.2 授权应用,写入新授权
            String subAccountIds = resource.getHolderids();
            if (ValidateUtil.isValid(subAccountIds)) {

                String[] subAccountIdArray = subAccountIds.split(",");

                for (String subAccountId : subAccountIdArray) {

                    DataAuth dataAuth = new DataAuth();
                    dataAuth.setAccount(Long.parseLong(subAccountId));
                    dataAuth.setApp(appid);
                    dataAuth.setCampaign(resource.getId());
                    dataAuth.setChannel(resource.getChannel());
                    dataAuth.setAllCampaign(false);
                    dataAuth.setDelFlag(false);

                    this.dataAuthRepository.save(dataAuth);
                }
            }
        }

        //处理 vivo 渠道特定参数
        resource.setApp(result.getApp());
        resource.setCampaignid(result.getCampaignid());
        this.updateVivoParam(resource, resource.getSpecialKeys());

        return result;
    }

    private void updateVivoParam(Campaign resource, String speciakeys) {
        if (!resource.isVivo2DDb()) {
            return;
        }

        Map data = this.fillVivoParamMap(speciakeys);
        App app = appRepository.findOne(resource.getApp());
        iNoSQLDBService.addVivoParamDatas(data, app.getAppkey(), resource.getCampaignid());
    }

    @Override
    public int updateName(Long appid, Long id, String name) {
        //判断重名
        List<Campaign> campaignList = campaignRepository.findCampaignByAppAndName(appid, name);
        if (campaignList != null && campaignList.size() > 1) {
            return 0;
        }

        Campaign resource = this.campaignRepository.findOne(id);
        resource.setName(name);
        resource.setAwssuccess(false);
        Campaign campaign = this.campaignRepository.save(resource);
        return 1;
    }

    @Override
    public Boolean updateCampaignName(Long appid, Long id, String name) {
        Boolean flag = true;

        // 1. 业务参数校验
        if (Strings.isNullOrEmpty(name)) {
            throw new TipException("名称不允许为空");
        }
        if (name.contains("\t")) {
            throw new TipException("名称中存在不合法字符");
        }
        Campaign resource = this.campaignRepository.findOne(id);
        if (!resource.getApp().equals(appid)) {
            throw new TipException("推广活动与应用不匹配");
        }
        List<Campaign> retrievedResources = campaignRepository.findCampaignByAppAndName(appid, name);

        if (null != retrievedResources && retrievedResources.size() > 0) {
            flag = false;
            return flag;
        }

        // 2. 修改名称
        this.campaignRepository.updateCampaignName(id, appid, name);

        if (Constant.awsneed) {
            App app = this.appRepository.findOne(appid);
            Channel channel = channelService.findoneById(resource.getChannel());
            resource.setName(name);


            this.iNoSQLDBService.update(transform(resource, app, channel));
        }

        return flag;
    }

    @Override
    public Campaign deleteCampaign(Long appid, Campaign campaign, Long account) {
        return delete(appid, 0L, campaign, account);
    }

    @Override
    public Campaign delete(Long appid, Long id, Long loginActId) {

        return delete(appid, id, null, loginActId);
    }

    public Campaign delete(Long appid, Long id, Campaign campaign, Long loginActId) {
        if (campaign == null && id != null) {
            campaign = this.campaignRepository.findOne(id);
        }

        if (campaign == null) {
            throw new TipException("应用不存在");
        }

        // 1. 删除用户授权应用
        // TODO 删除用户授权应用

        // 2. 同步删除aws-dynamodb
        if (Constant.awsneed) {
            if (!this.iNoSQLDBService.delete(campaign.getSurl())) {
                logger.error("删除dynamodb推广活动短链失败. info: {}", campaign);
                return campaign;
            }

            App app = appRepository.findOne(campaign.getApp());
            iNoSQLDBService.deleteVivoParam(app.getAppkey(), campaign.getCampaignid());

        }

        campaign.setDelFlag(true);
        campaign.setModifyAccount(loginActId);
        campaign.setModifyTime(new Date());
        // 3. 删除推广活动--假删除
        Campaign result = this.campaignRepository.save(campaign);
        // 4. 删除cam_camgroup_map表的推广活动信息
        //List<CamCamgroupMap> camCampaignMap = this.camCamgroupMapRepository.listCamCampMapByCamp(campaign.getId());
        //this.camCamgroupMapRepository.delete(camCampaignMap);

        //5.删除推广活动对应的data_auth表中信息
        //List<DataAuth> dataAuthList = this.dataAuthRepository.listDataAuthByAppChnCam(appid, campaign.getChannel(), id);
        //this.dataAuthRepository.delete(dataAuthList);;
        //改变新版微信mp关联表的状态
        if (campaign.getChannel().equals(5306L)) {
            campaignWeChatLinkRepository.updateStatusByCampaignId(campaign.getCampaignid(), 1);
        }
        //子活动停用
        List<Long> ids = new ArrayList<>();
        ids.add(result.getId());
        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.subCampaignInParent(ids);
        String pkgCampaignIds = "";
        if (!CollectionUtils.isEmpty(packageSubCampaigns)) {
            for (PackageSubCampaign packageSubCampaign : packageSubCampaigns) {
                if ("爱奇艺模糊归因".equals(packageSubCampaign.getName())) {
                    iqiyiCampaignRepository.deleteByCampaignid(packageSubCampaign.getCampaignid());
                }
                pkgCampaignIds = pkgCampaignIds + packageSubCampaign.getId() + ",";
            }
            packageSubCampaignService.forbiden(pkgCampaignIds.substring(0, pkgCampaignIds.length() - 1), loginActId);
        }
        return result;
    }

    /**
     * 启用推广活动
     * created by sunhao 20170602
     */
    @Override
    public Campaign enableCampaign(Long appId, Long campaignId, Long loginActId) {
        Campaign campaign = this.campaignRepository.findOne(campaignId);

        if (null == campaign) {
            throw new TipException("应用不存在!");
        }

        //保存
        campaign.setDelFlag(false);
        campaign.setModifyAccount(loginActId);
        campaign.setModifyTime(new Date());
        campaign = this.campaignRepository.save(campaign);

        App app = this.appRepository.findOne(appId);
        //启用活动判断当前app是否为预约转上线
        if (app.getOnlineDate() != null && campaign.getSubscribes() != null && "ry_coop".equals(campaign.getChannelType())) {
            Date today = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, DateUtil.getCurrentDateStr());
            Date onlineDate = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, app.getOnlineDate());
            if (onlineDate.compareTo(today) == -1) {
                throw new TipException("产品已上线，无法再启用预约活动");
            }
        }

        Channel channel = channelService.findoneById(campaign.getChannel());

        //同步创建短链信息至aws-dynamodb
        if (Constant.awsneed) {
            if (campaign.getId() != null) {

                boolean isAwsSucces = true;

                try {

                    final SurlModel transform = transform(campaign, app, channel);

                    if (!Strings.isNullOrEmpty(transform.getSurl()) && !Strings.isNullOrEmpty(transform.getCampaignid())
                            && !Strings.isNullOrEmpty(transform.getAppid()) && !Strings.isNullOrEmpty(transform.getCid())) {

                        isAwsSucces = iNoSQLDBService.create(transform);
                        SurlModel fetchRemoteItem = iNoSQLDBService.findBySurl(campaign.getSurl());

                        if (fetchRemoteItem == null || Strings.isNullOrEmpty(fetchRemoteItem.getSurl()) || Strings.isNullOrEmpty(fetchRemoteItem.getCampaignid())
                                || Strings.isNullOrEmpty(fetchRemoteItem.getAppid()) || Strings.isNullOrEmpty(fetchRemoteItem.getCid())) {

                            String contents = String.format("本地SurlModel: %s;\n远程SurlModel: %s.\n\ncampaininfo: {}",
                                    new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(fetchRemoteItem),
                                    new ObjectMapper().writeValueAsString(campaign));
                            //MailUtils.sendSimpleEmail("短链数据不一致(业务库与dynamodb)", contents, Constant.mlist);
                            SendCommonPostMail.sendMailReuse(Constant.mlist, "短链数据不一致(业务库与dynamodb)", contents, null, null);
                        }
                    } else {

                        try {
                            String contents = String.format("本地SurlModel信息: %s.\n\ncampaininfo: {}", new ObjectMapper().writeValueAsString(transform),
                                    new ObjectMapper().writeValueAsString(campaign));
                            //MailUtils.sendSimpleEmail("SurlModel信息缺失,无法同步Dynamodb", contents, Constant.mlist);
                            SendCommonPostMail.sendMailReuse(Constant.mlist, "SurlModel信息缺失,无法同步Dynamodb", contents, null, null);
                        } catch (Exception e1) {
                            logger.error("fail to send email for failing to generat surl");
                        }
                    }

                    //vivo 参数恢复到ddb
                    campaign.setVivo2DDb(false);
//                    if ("vivoclick".equals(channel.getUniqueName()) || "vivo".equals(channel.getUniqueName())) {
                    if (channel.getId().equals(13296L)) {
                        campaign.setVivo2DDb(true);
                    }

                    createVivoParam(null, campaign);


                } catch (Exception e) {
                    isAwsSucces = false;
                    e.printStackTrace();
                }
                this.campaignRepository.updateAwsStatus(campaign.getId(), isAwsSucces);
                campaign.setAwssuccess(isAwsSucces);
            }
        }

        //改变新版微信mp关联表的状态
        if (campaign.getChannel().equals(5306L)) {
            campaignWeChatLinkRepository.updateStatusByCampaignId(campaign.getCampaignid(), 0);
        }
        //子活动启用
        List<Long> ids = new ArrayList<>();
        ids.add(campaign.getId());
        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.subCampaignInParentAndDel(ids);
        String pkgCampaignIds = "";
        if (!CollectionUtils.isEmpty(packageSubCampaigns)) {
            for (PackageSubCampaign packageSubCampaign : packageSubCampaigns) {
                pkgCampaignIds = pkgCampaignIds + packageSubCampaign.getId() + ",";
            }
            packageSubCampaignService.enable(pkgCampaignIds.substring(0, pkgCampaignIds.length() - 1), loginActId);
        }
        return campaign;
    }

    @Override
    public List<Campaign> batchdelete(Long appid, Iterable<Long> campids, Long loginActId) {

        List<Campaign> rtnList = Lists.newArrayList();

        Iterator<Long> iter = campids.iterator();
        while (iter.hasNext()) {
            rtnList.add(delete(appid, iter.next(), loginActId));
        }

        return rtnList;
    }

    /**
     * 批量启用推广活动
     * created by sunhao 20170602
     */
    @Override
    public List<Campaign> batchEnable(Long appId, Iterable<Long> campids, Long loginActId) {

        List<Campaign> rtnList = Lists.newArrayList();

        Iterator<Long> iter = campids.iterator();
        while (iter.hasNext()) {
            rtnList.add(enableCampaign(appId, iter.next(), loginActId));
        }

        return rtnList;
    }

    @Override
    public String makesurl(Long appid, Account loginAccount) {
        final List<String> makesurls = makesurls(appid, loginAccount, 1);
        return makesurls != null && makesurls.size() > 0 ? makesurls.get(0) : null;
    }

    @Override
    public List<String> makesurls(Long appid, Account loginAccount, int genNum) {

        List<String> resultList = Lists.newArrayList();

        App app = appRepository.findOne(appid);

        boolean success = false;

        for (int i = 0; i < 3 && !success; i++) {

            String dateStr = DateUtil.getCurrentDateStr(DateUtil.C_TIME_PATTON_DEFAULT);

            String[] surls = ShortUrl.generateCodeN(loginAccount.getEmail() + app.getAppkey() + dateStr + UUID.randomUUID().toString(), genNum * 2);

            for (String s : surls) {
                Integer surlNum = campaignRepository.findSUrlALLTable(s);
                //todo 服务器迁移过渡
               /* if (Constant.qCloudOnline()) {
                    for (int j = 0; j < 3; j++) {
                        try {
                            String resultStr = HttpClientUtil.doHttpGetRequest(Constant.getServerDomain() + "/api/server/check/surl?surls=" + s, "");
                            net.sf.json.JSONObject fromObject = net.sf.json.JSONObject.fromObject(resultStr);
                            Object accountJSON = fromObject.get("content");
                            if (surlNum == 0 && (accountJSON.equals("0") || accountJSON.equals(0))) {
                                logger.info("服务器迁移过渡 校验通过");
                                resultList.add(s);
                            }
                            if (!StringUtil.isEmpty(resultStr)) break;
                        } catch (Exception e) {

                        }
                    }
                } else {
                    if (surlNum == 0) {
                        resultList.add(s);
                    }
                }*/

                if (surlNum == 0) {
                    resultList.add(s);
                }

                if (resultList.size() == genNum) {
                    success = true;
                    break;
                }
            }

            if (resultList.size() == genNum) {
                success = true;
                break;
            }
        }

        if (success) {
            return resultList;
        } else {
            try {
                //  MailUtils.sendSimpleEmail("推广活动短链循环3次，每次生成" + genNum * 2 + "个短链，没达到客户批量生成" + genNum + "个短链的目的！！",
                //        "推广活动短链循环3次，每次生成" + genNum * 2 + "个短链，没达到客户批量生成" + genNum + "个短链的目的！！请及时处理！！", Constant.mlist);

                SendCommonPostMail.sendMailReuse(Constant.mlist, "推广活动短链循环3次，每次生成" + genNum * 2 + "个短链，没达到客户批量生成" + genNum + "个短链的目的！！", "推广活动短链循环3次，每次生成" + genNum * 2 + "个短链，没达到客户批量生成" + genNum + "个短链的目的！！请及时处理！！", null, null);
            } catch (Exception e1) {
                logger.error("fail to send email for failing to generat surl", e1);
            }
            throw new TipException("生成推广活动短链失败，请联系与贵司对接的技术.");
        }
    }

    @Override
    public Integer getSulrOnServer(String surls) {
        if (StringUtils.isEmpty(surls)) {
            return 0;
        }
        List<String> surlsList = campaignRepository.findSUrlALLTableBySurls(Arrays.asList(surls.split(",")));
        return surlsList == null ? 0 : surlsList.size();
    }

    @Override
    public StringBuffer exportsurls(Long appid, Iterable<Long> campids, Long loginAccountId, String type, String lanType, String showurl) {
        String platformByAppId = appRepository.findPlatformByAppId(appid);
        boolean isShowMonitor = exportCampaignConfigRepository.findOne(appid) != null;

        StringBuffer sb = new StringBuffer("活动名称\t类型\t分包渠道ID");

        boolean is_export_url = "2".equals(showurl) ? false : true;

        if (is_export_url) {
            sb.append("\t追踪短链\r\n");
        } else {
            sb.append("\r\n");
        }


        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            sb = new StringBuffer("Campaign\tType\tPartner ID (Package)\tTracking URL\r\n");


        }

        if (type != null && !"H5".equals(platformByAppId) && !type.equals("ry_bd")) {
            sb = new StringBuffer("活动名称\t类型\t分包渠道ID");

            if (is_export_url) {
                if (isShowMonitor) {
                    sb.append("\t展示监测短链\t点击监测短链\r\n");
                } else {
                    sb.append("\t点击监测短链\r\n");
                }
            } else {
                sb.append("\r\n");
            }
            if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                sb = new StringBuffer("Campaign\tType\tPartner ID (Package)\tImpression Tracking Link\tClick Tracking Link\r\n");
            }
        }

        List<Campaign> camList = campaignRepository.findById(Lists.newArrayList(campids));
        if (camList == null || camList.size() == 0) {
            return sb;
        }

        List<Long> typeChannelIdList = authService.getAuthChannelList(loginAccountId);

        List<Channel> typeChannelList = this.channelRepository.findById(typeChannelIdList);

        Map<Long, Channel> channelMap = Maps.uniqueIndex(typeChannelList,
                new Function<Channel, Long>() {
                    @Override
                    public Long apply(Channel channel) {
                        return channel.getId();
                    }
                });

        for (Campaign cam : camList) {
            Channel c = channelMap.get(cam.getChannel());
            if (c == null) {
                continue;
            }

            if (!is_export_url) {
                // 去掉短链导出
                sb.append(cam.getName()).append("\t");
                sb.append(StringUtil.isEmpty(cam.getPack_channelid()) ? "分链接\t" : "分包\t");
                sb.append(cam.getPack_channelid()).append("\r\n");
                continue;

            }

            String trackUrl1 = Constant.trackurl + "/$surl";
            //String trackurl = c.getTrackurl().replace("$surl", cam.getSurl());
            String trackurl = trackUrl1.replace("$surl", cam.getSurl());
            String impTrackUrl = trackurl;
            //TODO qiqiqi
            if (cam.getClickLinkType() == null) {
                if (c.isIshttps()) {
                    trackurl = trackurl.replace("http:", "https:");
                }
            } else {
                if (cam.getClickLinkType()) {
                    trackurl = trackurl.replace("http:", "https:");
                }
            }
            if (c.isImpIsHttps()) {
                impTrackUrl = impTrackUrl.replace("http://", "https://");
            }

            if (c.isIstrackurlspecial()) {
                String androidparam = c.getAndroidparam();
                String iosparam = c.getIosparam();
                String selfbuiltparam = c.getSelfbuiltparam();
                //广点通同步的推广活动排除掉  规则为surl为空
                if (!Strings.isNullOrEmpty(cam.getSurl()) && "Android".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(androidparam)) {
                    androidparam = androidparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + androidparam;
                } else if (!Strings.isNullOrEmpty(cam.getSurl()) && "iOS".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(iosparam)) {
                    iosparam = iosparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + iosparam;
                } else if (!Strings.isNullOrEmpty(cam.getSurl()) && "selfBuilt".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(selfbuiltparam)) {
                    //String s = "idfa=__IDFA__&openudid=__OPENUDID__&imei=__IMEI__&mac=__MAC1__&androidid=__ANDROIDID1__&oaid=__OAID__&ip=__IP__&ts=__TS__&os=__OS__&subchannel=__CID__&callback_param=__CALLBACK_PARAM__&callback=__CALLBACK_URL__&noredirect=true&aid=__AID__&csite=__CSITE__&ctype=__CTYPE__&siteid=__UNION_SITE__";
                    selfbuiltparam = selfbuiltparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + selfbuiltparam;
                }
            }

            if (c.isImpIsTrackUrlSpecial() && isShowMonitor) {

                String impAndroidParam = c.getImpAndroidParam();
                String impIosParam = c.getImpIosParam();
                String impSelfbuiltparam = c.getImpSelfbuiltparam();

                if ("Android".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(impAndroidParam)) {
                    impAndroidParam = impAndroidParam.replace("$surl", cam.getSurl());
                    impTrackUrl = impTrackUrl + "?impression=true&" + impAndroidParam;
                } else if ("iOS".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(impIosParam)) {
                    impIosParam = impIosParam.replace("$surl", cam.getSurl());
                    impTrackUrl = impTrackUrl + "?impression=true&" + impIosParam;
                } else if ("selfBuilt".equalsIgnoreCase(cam.getOs()) && !Strings.isNullOrEmpty(impSelfbuiltparam)) {
                    impSelfbuiltparam = impSelfbuiltparam.replace("$surl", cam.getSurl());
                    impTrackUrl = impTrackUrl + "?impression=true&" + impSelfbuiltparam;
                }
            } else {
                impTrackUrl = impTrackUrl + "?impression=true";
            }

            if (cam.getUrl() != null && cam.getAccount() != null) {
                //拼接deeplink的标识

                Long appId = cam.getApp();
                App app = appRepository.findOne(appId);

                String containsUrl = Constant.deeplinkHost;

                String url = cam.getUrl();
                if (url.contains(containsUrl)) {
                    if (impTrackUrl.contains("?")) {
                        impTrackUrl += "&ry_dp=true";
                    } else {
                        impTrackUrl += "?ry_dp=true";
                    }
                    if (trackurl.contains("?")) {
                        trackurl += "&ry_dp=true";
                    } else {

                        trackurl += "?ry_dp=true";
                    }
                    AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(appId);
                    if (url.contains("ry/")) {
                        if ((appDeepLink.getPackageName() == null && app.getPlatform().equals("Android") || (appDeepLink.getAppIdPrefix() == null && app.getPlatform().equals("iOS")))) {
                            impTrackUrl += "&rydp_path=null";
                            trackurl += "&rydp_path=null";
                        } else {
                            String encodeString = UrlEncoded.encodeString(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                            trackurl += "&rydp_path=" + encodeString;
                            impTrackUrl += "&rydp_path=" + encodeString;
                        }

                    } else {
                        trackurl += "&rydp_path=null";
                        impTrackUrl += "&rydp_path=null";
                    }
                    if (url.contains("ry_dp=")) {
                        String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                        trackurl += "&rydp_url=" + substring2;
                        impTrackUrl += "&rydp_url=" + substring2;
                    } else if (url.contains("ry_sc=") && appDeepLink.getCallType() == null) {
                        String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                        trackurl += "&rydp_url=" + substring2;
                        impTrackUrl += "&rydp_url=" + substring2;
                    }
                }
            }

            if (cam.getSubscribes() != null && cam.getSubscribes()) {
                String preOrder = "&preorder=true";
                String pre_order = "?preorder=true";
                if (trackurl.contains("?")) {
                    trackurl += preOrder;
                } else {
                    trackurl += pre_order;
                }
                if (impTrackUrl.contains("?")) {
                    impTrackUrl += preOrder;
                } else {
                    impTrackUrl += pre_order;
                }
            }

            if (type != null && !"H5".equals(platformByAppId) && !type.equals("ry_bd")) {
                sb.append(cam.getName())
                        .append("\t");

                if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                    sb.append(StringUtil.isEmpty(cam.getPack_channelid()) ? "Track by Link\t" : "Track by Package\t");
                } else {
                    sb.append(StringUtil.isEmpty(cam.getPack_channelid()) ? "分链接\t" : "分包\t");
                }

                sb.append(cam.getPack_channelid()).append("\t");
                if (isShowMonitor) sb.append(impTrackUrl).append("\t");
                sb.append(trackurl).append("\r\n");
            } else {
                sb.append(cam.getName())
                        .append("\t");

                if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                    sb.append(StringUtil.isEmpty(cam.getPack_channelid()) ? "Track by Link\t" : "Track by Package\t");
                } else {
                    sb.append(StringUtil.isEmpty(cam.getPack_channelid()) ? "分链接\t" : "分包\t");
                }
                sb.append(cam.getPack_channelid())
                        .append("\t")
                        .append(trackurl)
                        .append("\r\n");
            }
        }
        return sb;
    }

    @Override
    public StringBuffer exportsurls2column(Long appid, Iterable<Long> campids, Long loginAccountId, String lanType) {
        String platformByAppId = appRepository.findPlatformByAppId(appid);
        boolean isShowMonitor = exportCampaignConfigRepository.findOne(appid) != null;
        StringBuffer sb;
        if (isShowMonitor) {
            sb = new StringBuffer("活动名称,展现监测短链,点击监测短链\n");
        } else {
            sb = new StringBuffer("活动名称,点击监测短链\n");
        }
        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            sb = new StringBuffer("Campaign,Impression Tracking Link,Click Tracking Link\n");
        }

        if ("H5".equals(platformByAppId)) {
            sb = new StringBuffer("活动名称,点击监测短链\n");
            if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                sb = new StringBuffer("Campaign,Click Tracking Link\n");
            }
        }

        if ("wx".equals(platformByAppId) || AppPlatformEnum.WG.getKey().equals(platformByAppId)) {
            sb = new StringBuffer("活动名称,监测链接\n");
            if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                sb = new StringBuffer("Campaign,Impression Tracking Link,Tracking Link\n");
            }
        }

        List<Campaign> camList = campaignRepository.findById(Lists.newArrayList(campids));
        if (camList == null || camList.size() == 0) {
            return sb;
        }

        boolean isSEM = false;
        if (camList.get(0).getChannelType() != null && camList.get(0).getChannelType().equalsIgnoreCase("ry_bd")) {
            isSEM = true;
        }


        if (isSEM) {
            sb = new StringBuffer("活动名称,点击监测短链\n");
            if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                sb = new StringBuffer("Campaign,Click Tracking Link\n");
            }
        }

        List<Long> typeChannelIdList = authService.getAuthChannelList(loginAccountId);

        List<Channel> typeChannelList = this.channelRepository.findById(typeChannelIdList);

        Map<Long, Channel> channelMap = Maps.uniqueIndex(typeChannelList,
                new Function<Channel, Long>() {
                    @Override
                    public Long apply(Channel channel) {
                        return channel.getId();
                    }
                });

        for (Campaign cam : camList) {
            Channel c = channelMap.get(cam.getChannel());
            if (c == null) {
                continue;
            }

            String trackUrl1 = Constant.trackurl + "/$surl";
            String trackurl = trackUrl1.replace("$surl", cam.getSurl());
            String impTrackUrl = trackurl;
            if (cam.getClickLinkType() == null) {
                if (c.isIshttps()) {
                    trackurl = trackurl.replace("http:", "https:");
                }
            } else {
                if (cam.getClickLinkType()) {
                    trackurl = trackurl.replace("http:", "https:");
                }
            }

            if (c.isImpIsHttps()) {
                impTrackUrl = impTrackUrl.replace("http://", "https://");
            }

            if (c.isIstrackurlspecial()) {
                String androidparam = c.getAndroidparam();
                String iosparam = c.getIosparam();
                String selfbuiltparam = c.getSelfbuiltparam();
                if (cam.getOs().equalsIgnoreCase("Android") && !Strings.isNullOrEmpty(androidparam)) {
                    androidparam = androidparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + androidparam;
                } else if (cam.getOs().equalsIgnoreCase("iOS") && !Strings.isNullOrEmpty(iosparam)) {
                    iosparam = iosparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + iosparam;
                } else if (cam.getOs().equalsIgnoreCase("selfBuilt") && !Strings.isNullOrEmpty(selfbuiltparam)) {
                    selfbuiltparam = selfbuiltparam.replace("$surl", cam.getSurl());
                    trackurl = trackurl + "?" + selfbuiltparam;
                }
            }
            if (isShowMonitor) {
                if (c.isImpIsTrackUrlSpecial()) {

                    String impAndroidParam = c.getImpAndroidParam();
                    String impIosParam = c.getImpIosParam();
                    String impSelfbuiltparam = c.getImpSelfbuiltparam();
                    if (cam.getOs().equalsIgnoreCase("Android") && !Strings.isNullOrEmpty(impAndroidParam)) {
                        impAndroidParam = impAndroidParam.replace("$surl", cam.getSurl());
                        impTrackUrl = impTrackUrl + "?impression=true&" + impAndroidParam;
                    } else if (cam.getOs().equalsIgnoreCase("iOS") && !Strings.isNullOrEmpty(impIosParam)) {
                        impIosParam = impIosParam.replace("$surl", cam.getSurl());
                        impTrackUrl = impTrackUrl + "?impression=true&" + impIosParam;
                    } else if (cam.getOs().equalsIgnoreCase("selfBuilt") && !Strings.isNullOrEmpty(impSelfbuiltparam)) {
                        impSelfbuiltparam = impSelfbuiltparam.replace("$surl", cam.getSurl());
                        impTrackUrl = impTrackUrl + "?impression=true&" + impSelfbuiltparam;
                    }
                } else {
                    impTrackUrl = impTrackUrl + "?impression=true";
                }
            }
            if (cam.getUrl() != null && cam.getAccount() != null) {
                //拼接deeplink的标识

                Long appId = cam.getApp();
                App app = appRepository.findOne(appId);

                String containsUrl = Constant.deeplinkHost;

                String url = cam.getUrl();
                if (url.contains(containsUrl)) {
                    if (impTrackUrl.contains("?")) {
                        impTrackUrl += "&ry_dp=true";
                    } else {
                        impTrackUrl += "?ry_dp=true";
                    }
                    if (trackurl.contains("?")) {
                        trackurl += "&ry_dp=true";
                    } else {

                        trackurl += "?ry_dp=true";
                    }
                    AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(appId);
                    if (url.contains("ry/")) {
                        if ((appDeepLink.getPackageName() == null && app.getPlatform().equals("Android") || (appDeepLink.getAppIdPrefix() == null && app.getPlatform().equals("iOS")))) {
                            impTrackUrl += "&rydp_path=null";
                            trackurl += "&rydp_path=null";
                        } else {
                            String encodeString = UrlEncoded.encodeString(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                            trackurl += "&rydp_path=" + encodeString;
                            impTrackUrl += "&rydp_path=" + encodeString;
                        }

                    } else {
                        trackurl += "&rydp_path=null";
                        impTrackUrl += "&rydp_path=null";
                    }
                    if (url.contains("ry_dp=")) {
                        String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                        trackurl += "&rydp_url=" + substring2;
                        impTrackUrl += "&rydp_url=" + substring2;
                    } else if (url.contains("ry_sc=") && appDeepLink.getCallType() == null) {
                        String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                        trackurl += "&rydp_url=" + substring2;
                        impTrackUrl += "&rydp_url=" + substring2;

                    }
                }
            }

            if (cam.getSubscribes() != null && cam.getSubscribes()) {
                String preOrder = "&preorder=true";
                String pre_order = "?preorder=true";
                if (trackurl.contains("?")) {
                    trackurl += preOrder;
                } else {
                    trackurl += pre_order;
                }
                if (impTrackUrl.contains("?")) {
                    impTrackUrl += preOrder;
                } else {
                    impTrackUrl += pre_order;
                }
            }
            if ("H5".equals(platformByAppId) || isSEM) {
                sb.append(cam.getName()).append(",").append(trackurl).append("\n");
            } else if ("wx".equals(platformByAppId) || AppPlatformEnum.WG.getKey().equals(platformByAppId)) {
                sb.append(cam.getName()).append(",").append(cam.getUrl()).append("\n");
            } else {
                sb.append(cam.getName()).append(",");
                if (isShowMonitor) {
                    sb.append(impTrackUrl).append(",");
                }
                sb.append(trackurl).append("\n");
            }
        }
        return sb;
    }

    @Override
    public List<Campaign> listCampaignByApp(Long appid) {
        return this.campaignRepository.listCampByApp(appid);
    }

    public List<Campaign> listCampaign(Long appid, Long accountid) {
        //TODO 需根据不同角色查询可查看活动
        return this.campaignRepository.listCampByApp(appid);
    }

    public List<Campaign> listAllCampaign(Long appid, Long accountid) {
        //TODO 需根据不同角色查询可查看活动
        return this.campaignRepository.listAllCampByApp(appid);
    }

    @Override
    public List<Object[]> listAllCampaignObject(List<String> val, Long app, Long account) {

        if (val != null && val.size() < 2000) {
            return campaignRepository.findObjectsByCampaignid(val, app);
        } else {
            return campaignRepository.findObjectsByApp(app);
        }

    }

    public List<Object[]> listPartCampaign(Long appid) {
        return this.campaignRepository.listPartCampByApp(appid);
    }

    public List<Campaign> listAllCampaignByDelflag(Long appid, Boolean delflag) {
        //TODO 需根据不同角色查询可查看活动
        return this.campaignRepository.listAllCampByAppAndDelflag(appid, delflag);
    }

    public List<Campaign> listAllCampaignByDelflagAndCid(Long appid, Boolean delflag, String cid) {

        return this.campaignRepository.listAllCampaignByDelflagAndCid(appid, delflag, cid);
    }

    @Override
    public Iterable<Campaign4Web> listAll(Long appid, Account loginAccount, String channelType) {
        List<Campaign> campaignList;

        if (StringUtils.isEmpty(channelType)) {
            //查所有
            campaignList = campaignRepository.findAll(appid);

        } else if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
            //查询自定义渠道
            campaignList = campaignRepository.findCustomOrderByCreateTimeDesc(appid);

        } else {
            //查合作渠道和SEM渠道
            campaignList = campaignRepository.findByChannelType(appid, channelType);
        }

        return buildCampaign4WebWithCampaign(campaignList, channelType);
    }

    @Override
    public Iterable<Campaign4Web> listAllDisable(Long appid, Account loginAccount, String channelType) {
        List<Campaign> campaignList;

        if (StringUtils.isEmpty(channelType)) {
            //查所有
            campaignList = campaignRepository.findAllDisable(appid);

        } else if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
            //查询自定义渠道
            campaignList = campaignRepository.findCustomCampaignDisable(appid);

        } else {
            //查合作渠道和SEM渠道
            campaignList = campaignRepository.findByChannelTypeDisable(appid, channelType);
        }

        return buildCampaign4WebWithCampaign(campaignList, channelType);
    }

    private List<Campaign4Web> listAllDataPage(Campaign campaign, Long appid, String channelType, Pageable pageable, Map data) {

        List<Campaign> campaignList;
        campaign.setApp(appid);
        if (StringUtils.isEmpty(channelType)) {
            //查所有
            campaign.setChannelType("");

        } else if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
            //查询自定义渠道
            campaign.setChannelType(null);
        } else {
            campaign.setChannelType(channelType);
        }
        Page<Campaign> page = campaignRepository.findAll(getCampineSpecification(campaign, null), pageable);
        campaignList = page.getContent();


        //查询总数时 去掉搜索条件
        String name = campaign.getName();
        campaign.setName("");
        campaign.setChannel(0l);
        data.put("total", campaignRepository.count(getCampineSpecification(campaign, null)));
        campaign.setName(name);

        return buildCampaign4WebWithCampaign(campaignList, channelType);
    }

    private Specification<Campaign> getCampineSpecification(Campaign campaign, Map idsMap) {

        return new Specification<Campaign>() {
            @Override
            public javax.persistence.criteria.Predicate toPredicate(Root<Campaign> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {

                List<javax.persistence.criteria.Predicate> predicates = new ArrayList<>();

                if (!StringUtils.isEmpty(campaign.getName())) {
                    predicates.add(
                            criteriaBuilder.or(
                                    criteriaBuilder.like(root.get("name").as(String.class), "%" + campaign.getName() + "%"),
                                    criteriaBuilder.like(root.get("surl").as(String.class), "%" + campaign.getName() + "%")));
                }

                if (campaign.getChannel() != null && campaign.getChannel() != 0) {
                    predicates.add(criteriaBuilder.equal(root.get("channel").as(Long.class), campaign.getChannel()));
                }

                if (campaign.getChannelType() == null) {
                    predicates.add(criteriaBuilder.isNull(root.get("channelType").as(String.class)));
                } else if (!StringUtils.isEmpty(campaign.getChannelType())) {
                    predicates.add(criteriaBuilder.equal(root.get("channelType").as(String.class), campaign.getChannelType()));
                }

                if (campaign.getDelFlag()) {
                    predicates.add(criteriaBuilder.isTrue(root.get("delFlag").as(Boolean.class)));
                } else {
                    predicates.add(criteriaBuilder.isFalse(root.get("delFlag").as(Boolean.class)));
                }

                if (idsMap != null) {
                    if (idsMap.get("channels") != null) {

                        predicates.add(criteriaBuilder.or(
                                root.get("channel").as(Long.class).in((List<Long>) idsMap.get("channels")),
                                root.get("id").as(Long.class).in((List<Long>) idsMap.get("campaigns"))));
                    }

                    if (campaign.getCreateAccount() != null && "1".equals(idsMap.get("equal"))) {
                        predicates.add(criteriaBuilder.notEqual(root.get("createAccount").as(Long.class), campaign.getCreateAccount()));
                    }

                    if (campaign.getCreateAccount() != null && "2".equals(idsMap.get("equal"))) {
                        predicates.add(criteriaBuilder.equal(root.get("createAccount").as(Long.class), campaign.getCreateAccount()));
                    }
                }

                predicates.add(criteriaBuilder.notEqual(root.get("channel").as(Long.class), 5305L));

                predicates.add(criteriaBuilder.equal(root.get("channelCategory").as(String.class), "ADVERTISING"));

                predicates.add(criteriaBuilder.equal(root.get("app").as(Long.class), campaign.getApp()));

                javax.persistence.criteria.Predicate[] predicatesArr = new javax.persistence.criteria.Predicate[predicates.size()];

                return criteriaBuilder.and(predicates.toArray(predicatesArr));
            }
        };
    }

    @Override
    public List<Campaign> listAllAndDisableLimit(Long appid, Account loginAccount, String name, Map jsond, String channelType) {
        List<Campaign> campaignList;
        int count = campaignRepository.findAllAndDisableLimitCount(appid);
        jsond.put("total", count);
        if (StringUtils.isEmpty(name)) {
            //查合作渠道和SEM渠道
            campaignList = campaignRepository.findAllAndDisableLimit(appid, 500);
        } else {
            campaignList = campaignRepository.findAllAndDisableLimit(appid, name, 500);
        }

        return campaignList;
//        return buildCampaign4WebWithCampaign(campaignList, channelType);
    }

    /*@Autowired
    JdbcTemplate jdbcTemplate;*/

    /**
     * 删除DDB和mysql的过期短链并且备份
     */
    @Override
    @Transactional
    public void deleteExpiredCampaign(String date15) {
        List<Campaign> campaigns = this.campaignRepository.find15DaysAgo(date15);
        if (!CollectionUtils.isEmpty(campaigns)) {
            List<CampaignExpired> campaignExpireds = new ArrayList<>(campaigns.size());
            List<String> oldCampaignExpireds = this.campaignExpiredRepository.findDinstinctCampaignid(date15);
            for (Campaign campaign : campaigns) {
                if (!oldCampaignExpireds.contains(campaign.getCampaignid())) {
                    campaignExpireds.add(new CampaignExpired(campaign));
                }
                if (!StringUtil.isEmpty(campaign.getSurl())) {
                    NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TABLE_SURL, "surl", campaign.getSurl());
                }
            }
            this.channelAccountRepository.ableExpired(date15);
            this.campaignExpiredRepository.save(campaignExpireds);
            this.campaignRepository.delete(campaigns);
        }
    }

    @Override
    public Iterable<Campaign4Web> listAllAndDisable(Long appid, Account loginAccount, String channelType) {

        List<Campaign> campaignList;

        if (StringUtils.isEmpty(channelType)) {
            //查所有
            campaignList = campaignRepository.findAllAndDisable(appid);

        } else if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
            //查询自定义渠道
            campaignList = campaignRepository.findCustomAndDisableOrderByCreateTimeDesc(appid);

        } else {
            //查合作渠道和SEM渠道
            campaignList = campaignRepository.findByChannelTypeAndDisable(appid, channelType);
        }

        return buildCampaign4WebWithCampaign(campaignList, channelType);
    }

    /**
     * 用List<Campaign> 构建 List<Campaign4Web>
     */
    private List<Campaign4Web> buildCampaign4WebWithCampaign(List<Campaign> campaignList, String channelType) {
        /*if(campaignList.isEmpty()){
            AppDeepLink deeplink = appDeepLinkRepository.findOneByAppId(campaignList.get(0).getApp());

        }*/
        List<Campaign4Web> resultCampaignList = new ArrayList<>();

        if (CollectionUtils.isEmpty(campaignList)) {
            return resultCampaignList;
        }

        //所有https的渠道
        List<Channel> channelList = channelRepository.findByIshttps(true);
        Map<Long, Channel> httpsChannelMap = Maps.uniqueIndex(channelList, new Function<Channel, Long>() {
            @Override
            public Long apply(Channel channel) {
                return channel.getId();
            }
        });

        //查询所有展示https的渠道
        List<Channel> impChannelList = channelRepository.findByImpIshttps(true);
        Map<Long, Channel> impHttpsChannelMap = Maps.uniqueIndex(impChannelList, new Function<Channel, Long>() {
            @Override
            public Long apply(Channel channel) {
                return channel.getId();
            }
        });

        //所有的渠道信息
        List<Channel> typeChanelList = channelRepository.findByTypeIsNotNull();
        Map<Long, Channel> typeChannelMap = Maps.uniqueIndex(typeChanelList, new Function<Channel, Long>() {
            @Override
            public Long apply(Channel channel) {
                return channel.getId();
            }
        });
        // 对循环进行优化  update by zhangyao 2018-11-22
        List<Long> ids = new ArrayList<>(campaignList.size());
        for (Campaign campaign : campaignList) {
            ids.add(campaign.getId());
        }
        List<Object[]> CampaignNumbers = subCampaignService.findSubCampaignNumberByIds(ids);
        Map<Long, Long> subCampaignNumberMap = new HashMap<>(CampaignNumbers.size());
        for (Object[] objects : CampaignNumbers) {
            subCampaignNumberMap.put(Long.parseLong(objects[0].toString()), Long.parseLong(objects[1].toString()));
        }
        List<Object[]> packageCampaignNumbers = packageSubCampaignService.findSubCampaignNumberByIds(ids);
        Map<Long, Long> packageCampaignNumberMap = new HashMap<>(packageCampaignNumbers.size());
        for (Object[] packageCampaignNumber : packageCampaignNumbers) {
            packageCampaignNumberMap.put(Long.parseLong(packageCampaignNumber[0].toString()), Long.parseLong(packageCampaignNumber[1].toString()));
        }


        //构建参数
        // 对循环进行优化  update by zhangyao 2018-11-22
        for (Campaign campaign : campaignList) {

            Campaign4Web campaign4Web = new Campaign4Web(campaign);
            campaign4Web.setSubscribes(campaign.getSubscribes());
/*            if("wx".equals(campaign.getOs())){
                campaign4Web.setMipType(campaign.getMipType());
                campaign4Web.setQrCodeAddress(campaign.getQrCodeAddress());
            }*/

            campaign4Web.setSubCampaignNumber(0L);
            campaign4Web.setImpressionStop(campaign.getImpressionStop());

            //子活动数量 add by sunhao 20170522
            if (CHANNEL_TYPE_SEM.equals(channelType)) {
                Long aLong = subCampaignNumberMap.get(campaign.getId()) == null ? 0L : subCampaignNumberMap.get(campaign.getId()).longValue();
                campaign4Web.setSubCampaignNumber(aLong);
            }
            Long aLong = packageCampaignNumberMap.get(campaign.getId()) == null ? 0L : packageCampaignNumberMap.get(campaign.getId()).longValue();

            campaign4Web.setSubCampaignNumber(campaign4Web.getSubCampaignNumber() + aLong);

            String trackUrl = Constant.trackurl;
            String impTrackUrl = Constant.trackurl;
            if (campaign.getClickLinkType() == null) {
                if (httpsChannelMap.containsKey(campaign.getChannel())) {
                    campaign4Web.setIshttps(true);
                    trackUrl = trackUrl.replace("http:", "https:");
                }
            } else {
                if (campaign.getClickLinkType()) {
                    campaign4Web.setIshttps(true);
                    trackUrl = trackUrl.replace("http:", "https:");
                }
            }

            if (impHttpsChannelMap.containsKey(campaign.getChannel())) {
                campaign4Web.setImpIshttps(true);
                impTrackUrl = impTrackUrl.replace("http:", "https:");
            }


            if (!StringUtil.isEmpty(campaign4Web.getCampaignpackid())) {

                trackUrl = campaign4Web.getCampaignpackid();

            } else {

                String surl = campaign4Web.getSurl();

                Channel channel = typeChannelMap.get(campaign4Web.getChannel());

                //填充参数
                if (null != channel) {

                    trackUrl = this.buildUrlParamByChannel(surl, channel, campaign4Web.getOs());
                    impTrackUrl = this.buildImpressionUrlParamByChannel(surl, channel, campaign4Web.getOs());

                    if (campaign.getClickLinkType() == null) {
                        if (httpsChannelMap.containsKey(campaign.getChannel())) {
                            campaign4Web.setIshttps(true);
                            trackUrl = trackUrl.replace("http:", "https:");
                        }
                    } else {
                        if (campaign.getClickLinkType()) {
                            campaign4Web.setIshttps(true);
                            trackUrl = trackUrl.replace("http:", "https:");
                        }
                    }
                    if (impHttpsChannelMap.containsKey(campaign.getChannel())) {
                        campaign4Web.setImpIshttps(true);
                        impTrackUrl = impTrackUrl.replace("http:", "https:");
                    }

                    campaign4Web.setChannelName(channel.getName());

                } else {

                    trackUrl = trackUrl + "/" + surl;
                    impTrackUrl = impTrackUrl + "/" + surl + "?impression=true";
                }
            }

            campaign4Web.setTrackurl(trackUrl);
            campaign4Web.setImpTrackurl(impTrackUrl);


            if (campaign.getUrl() != null && campaign.getAccount() != null) {
                //拼接deeplink的标识
                Long app = campaign4Web.getApp();

                String containsUrl = Constant.deeplinkHost;

                String url = campaign.getUrl();
                if (url.contains(containsUrl)) {
                    campaign4Web.setDeeplink(true);
                    if (campaign4Web.getTrackurl().contains("?")) {
                        campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&ry_dp=true");
                    } else {
                        campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "?ry_dp=true");
                    }
                    if (campaign4Web.getImpTrackurl().contains("?")) {
                        campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&ry_dp=true");
                    } else {
                        campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "?ry_dp=true");
                    }
                    AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(app);
                    if (url.contains("ry/")) {
                        if ((appDeepLink.getPackageName() == null && campaign4Web.getOs().equals("Android") || (appDeepLink.getAppIdPrefix() == null && campaign4Web.getOs().equals("iOS")))) {
                            campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&rydp_path=null");
                            campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&rydp_path=null");
                        } else {
                            String encodeString = UrlEncoded.encodeString(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                            campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&rydp_path=" + encodeString);
                            campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&rydp_path=" + encodeString);
                        }

                    } else {
                        campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&rydp_path=null");
                        campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&rydp_path=null");
                    }
                    if (url.contains("ry_dp=")) {
                        String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                        campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&rydp_url=" + substring2);
                        campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&rydp_url=" + substring2);
                    } else if (url.contains("ry_sc=") && appDeepLink.getCallType() == null) {
                        String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                        campaign4Web.setTrackurl(campaign4Web.getTrackurl() + "&rydp_url=" + substring2);
                        campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + "&rydp_url=" + substring2);
                    }
                }
            }
            if (campaign.getSubscribes() != null && campaign.getSubscribes()) {
                String preOrder = "&preorder=true";
                String pre_order = "?preorder=true";
                if (campaign4Web.getTrackurl().contains("?")) {
                    campaign4Web.setTrackurl(campaign4Web.getTrackurl() + preOrder);
                } else {
                    campaign4Web.setTrackurl(campaign4Web.getTrackurl() + pre_order);
                }
                if (campaign4Web.getImpTrackurl().contains("?")) {
                    campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + preOrder);
                } else {
                    campaign4Web.setImpTrackurl(campaign4Web.getImpTrackurl() + pre_order);
                }
            }
            resultCampaignList.add(campaign4Web);
        }
        return resultCampaignList;
    }

    /**
     * 根据渠道构建trackurl
     */
    private String buildUrlParamByChannel(String surl, Channel channel, String appPlatform) {

        String baseurl = Constant.trackurl + "/$surl";

        String trackUrl = baseurl.replace("$surl", surl);

        boolean isTrackUrlSpecial = channel.isIstrackurlspecial();

        String androidParam = channel.getAndroidparam();
        String iosParam = channel.getIosparam();
        String selfbuiltparam = channel.getSelfbuiltparam();
        if (isTrackUrlSpecial && !StringUtil.isEmpty(androidParam) && AppPlatformEnum.ANDROID.getKey().equalsIgnoreCase(appPlatform)) {

            androidParam = androidParam.replace("$surl", surl);
            trackUrl = trackUrl + "?" + androidParam;

        } else if (isTrackUrlSpecial && !StringUtil.isEmpty(iosParam) && AppPlatformEnum.IOS.getKey().equalsIgnoreCase(appPlatform)) {

            iosParam = iosParam.replace("$surl", surl);
            trackUrl = trackUrl + "?" + iosParam;
        } else if (isTrackUrlSpecial && "selfBuilt".equalsIgnoreCase(appPlatform)) {

            selfbuiltparam = selfbuiltparam.replace("$surl", surl);
            trackUrl = trackUrl + "?" + selfbuiltparam;
        }

        return trackUrl;
    }

    /**
     * 根据渠道构建impression_trackurl
     */
    private String buildImpressionUrlParamByChannel(String surl, Channel channel, String appPlatform) {
        String baseurl = Constant.trackurl + "/$surl";
        String impTrackUrl = baseurl.replace("$surl", surl);

        boolean impIsTrackUrlSpecial = channel.isImpIsTrackUrlSpecial();

        String impIosParam = channel.getImpIosParam();
        String impAndroidParam = channel.getImpAndroidParam();
        String impSelfbuiltparam = channel.getImpSelfbuiltparam();
        if (impIsTrackUrlSpecial && !StringUtil.isEmpty(impAndroidParam) && AppPlatformEnum.ANDROID.getKey().equalsIgnoreCase(appPlatform)) {

            impAndroidParam = impAndroidParam.replace("$surl", surl);
            impTrackUrl = impTrackUrl + "?impression=true&" + impAndroidParam;

        } else if (impIsTrackUrlSpecial && !StringUtil.isEmpty(impIosParam) && AppPlatformEnum.IOS.getKey().equalsIgnoreCase(appPlatform)) {

            impIosParam = impIosParam.replace("$surl", surl);
            impTrackUrl = impTrackUrl + "?impression=true&" + impIosParam;
        } else if (impIsTrackUrlSpecial && "selfBuilt".equalsIgnoreCase(appPlatform)) {

            impSelfbuiltparam = impSelfbuiltparam.replace("$surl", surl);
            impTrackUrl = impTrackUrl + "?impression=true&" + impSelfbuiltparam;
        } else {
            impTrackUrl = impTrackUrl + "?impression=true";
        }
        return impTrackUrl;
    }

    @Override
    public Campaign findByName(String name, Long app) {
        return this.campaignRepository.findByNameAndApp(name, app);
    }

    @Override
    public Campaign findByNameIncludeDel(String name, Long app) {
        return this.campaignRepository.findByNameAndAppIncludeDel(name, app);
    }

    @Override
    public Campaign findPackChannelid(String channelid, Long app) {
        return this.campaignRepository.findPackChannelidAndApp(channelid, app);
    }

    @Override
    public Campaign findPackChannelidExist(String packchannelid, Long appid) {
        return campaignRepository.findPackChannelidAndAppExist(packchannelid, appid);
    }

    @Override
    public void doSyncSurl2Aws() {
        if (!Constant.awsneed) {
            logger.info("开发环境不进行同步数据操作");
            return;
        }
        List<Campaign> querylist = this.campaignRepository.findUnSync();
        if (querylist == null || querylist.size() == 0) {
            return;
        }

        //字典数据准备
        //1. 渠道字典
        List<Long> qChannelIds = Lists.transform(querylist, new Function<Campaign, Long>() {
            @Override
            public Long apply(Campaign campaign) {
                return campaign.getChannel();
            }
        });
        List<Channel> queryChannelList = channelRepository.findById(qChannelIds);
        Map<Long, Channel> dicChannelMap = Maps.uniqueIndex(queryChannelList, new Function<Channel, Long>() {
            @Override
            public Long apply(Channel channel) {
                return channel.getId();
            }
        });
        //1. 应用字典
        List<Long> qAppIds = Lists.transform(querylist, new Function<Campaign, Long>() {
            @Override
            public Long apply(Campaign campaign) {
                return campaign.getApp();
            }
        });
        List<App> queryAppList = appRepository.findByIds(qAppIds);
        Map<Long, App> dicAppMap = Maps.uniqueIndex(queryAppList, new Function<App, Long>() {
            @Override
            public Long apply(App app) {
                return app.getId();
            }
        });

        //更新短链至dynamodb(aws)
        logger.info("本次共计同步条数: {}", querylist.size());
        int suc_num = 0;
        List<String> failedList = Lists.newArrayList();
        for (Campaign var : querylist) {
            final App app = dicAppMap.get(var.getApp());
            final Channel channel = dicChannelMap.get(var.getChannel());
            if (app == null || channel == null) {
                logger.warn("数据错误: 该推广活动对应应用或渠道信息不存在。 info: {}", var.toString());
                continue;
            }
            final SurlModel transform = transform(var, app, channel);
            if (!Strings.isNullOrEmpty(transform.getSurl())//短链
                    && !Strings.isNullOrEmpty(transform.getCampaignid())//推广活动id
                    && !Strings.isNullOrEmpty(transform.getAppid())//应用id
                    && !Strings.isNullOrEmpty(transform.getCid())) {//渠道id
                boolean isAwsSucces = iNoSQLDBService.create(transform);
                SurlModel fetchRemoteItem = iNoSQLDBService.findBySurl(var.getSurl());
                if (fetchRemoteItem == null
                        || Strings.isNullOrEmpty(fetchRemoteItem.getSurl())
                        || Strings.isNullOrEmpty(fetchRemoteItem.getCampaignid())
                        || Strings.isNullOrEmpty(fetchRemoteItem.getAppid())
                        || Strings.isNullOrEmpty(fetchRemoteItem.getCid())
                ) {
                    try {
                        String contents = String.format("本地SurlModel: %s;\n远程SurlModel: %s.\n\ncampaininfo: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(fetchRemoteItem), new ObjectMapper().writeValueAsString(var));
                        // MailUtils.sendSimpleEmail("(调度任务)短链数据不一致(业务库与dynamodb)", contents, Constant.mlist);
                        SendCommonPostMail.sendMailReuse(Constant.mlist, "(调度任务)短链数据不一致(业务库与dynamodb)", contents, null, null);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("fail to send email for failing to generat surl");
                    }
                }

                suc_num++;
                this.campaignRepository.updateAwsStatus(var.getId(), isAwsSucces);
            } else {
                try {
                    String contents = String.format("本地SurlModel信息: %s.\n\ncampaininfo: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(var));
                    // MailUtils.sendSimpleEmail("(调度任务)SurlModel信息缺失,无法同步Dynamodb", contents, Constant.mlist);
                    SendCommonPostMail.sendMailReuse(Constant.mlist, "(调度任务)SurlModel信息缺失,无法同步Dynamodb", contents, null, null);
                } catch (Exception e1) {
                    e1.printStackTrace();
                    logger.error("fail to send email for failing to generat surl");
                }
                failedList.add(var.getSurl());
            }
        }

        logger.info("成功条数: {}, 失败条数: {}", suc_num, failedList.size());
        logger.info("失败surl列表: {}", failedList.toArray());
    }

    @Override
    public void doDisableCampaign2ddb() {
        List<Campaign> itemList = this.campaignRepository.findNeedDeleteDDB();

        logger.warn("同步删除DDB推广活动......, 总计条数: {}\\begin", itemList.size());
        List<String> delList = Lists.newArrayList();
        for (Campaign campaign : itemList) {
            delList.add(campaign.getSurl());

            if (delList.size() == 25) {
                this.campaignRepository.updateAwsStatus(delList, this.iNoSQLDBService.delete(delList));
                delList.clear();
            }
        }

        if (delList.size() > 0) {
            this.campaignRepository.updateAwsStatus(delList, this.iNoSQLDBService.delete(delList));
        }
        logger.warn("同步删除DDB推广活动结束......, 总计条数: {}\\end", itemList.size());
    }

    public SurlModel transform(Campaign campaign, App app, Channel channel) {
        //h5的APP需要同步apptype="wap"到ddb ,其他类型的无需
        String appType = AppPlatformEnum.H5.getKey().equals(app.getPlatform()) ? "wap" : null;

        SurlModel surlModel = transform(channel.getType(), campaign.getSurl(), campaign.getCampaignid(), campaign.getName(), campaign.getUrl(), campaign.getSpecialKeys(), app.getAppkey(), appType, app.getUrl(), channel.getId(), channel.getName(), channel.getUniqueName(), channel.getCategory(), channel.getClipboardParam());

        if (campaign.getImpressionStop() != null && campaign.getImpressionStop().intValue() == 1 && !"ry_bd".equals(channel.getType())) {
            surlModel.setIsclose("impression");
        }

        return surlModel;
    }

    private SurlModel transform(String type, String surl, String md5CampaignId, String campaignName, String campaignUrl, String specialKeys, String appkey, String appType, String appUrl, Long channelid, String channelName, String channelUniqueName, String channelcategory, String clipboardParam) {
        SurlModel surlModel = new SurlModel(surl);

        /*推广活动信息*/
        //活动id
        surlModel.setCampaignid(md5CampaignId);
        //活动名称
        surlModel.setName(campaignName);
        //应用地址或活动地址
        surlModel.setUrl(Strings.isNullOrEmpty(campaignUrl) ? appUrl : campaignUrl);
        surlModel.setSpecialkeys(specialKeys);
        //黏贴版参数
        if (!StringUtil.isEmpty(clipboardParam)) surlModel.setClipboardParam(clipboardParam);

        /*渠道信息*/
        if (!StringUtil.isEmpty(type) && type.equals("ry_bd")) {
            //关键词类型的将type放到ddb中.
            surlModel.setType(type);
        }
        //渠道id
        surlModel.setCid(String.valueOf(channelid));
        //渠道名称
        surlModel.setChannel(channelName);
        //渠道英文名称
        surlModel.setChannelname(Strings.isNullOrEmpty(channelUniqueName) ? "" : channelUniqueName);
        surlModel.setCategory(channelcategory);

        //应用编码
        surlModel.setAppid(appkey);
        //app类型
        surlModel.setApptype(appType);
        return surlModel;
    }

    @Override
    public List<Campaign> findByCampaignidList(List<String> campaignidList, Long app) {

        return CollectionUtils.isEmpty(campaignidList) ? new ArrayList<Campaign>() : campaignRepository.findByCampaignid(campaignidList, app);
    }

    @Override
    public List<Campaign> findAllByCampaignIdList(List<String> campaignidList) {
        return campaignRepository.findAllByCampaignId(campaignidList);
    }

    @Override
    public int updateUrl(Long app, String url) {
        return campaignRepository.updateDownloadUrl(app, url);
    }

    @Override
    public Map<String, String> findCampaignMapWithDefault(Long appId, Long accountId) {

        List<Campaign> campaignList = this.listCampaign(appId, accountId);
        campaignList = (null != campaignList) ? campaignList : new ArrayList<Campaign>();

        //加入自然量
        Campaign defaultCampaign = new Campaign();
        defaultCampaign.setName("自然量");
        defaultCampaign.setCampaignid("_default_");

        campaignList.add(defaultCampaign);

        Map<String, String> result = new HashMap<>();

        for (Campaign camp : campaignList) {
            result.put(camp.getCampaignid(), camp.getName());
        }

        return result;
    }

    /**
     * 将channel下面所有的推广活动改为未同步
     * created by sunhao 2017-05-17
     */
    @Override
    public void updateCampaignAwsStatusByChannel(Long channelId) {
        this.campaignRepository.updateAwsStatusByChannel(channelId);
    }

    @Override
    public List<CampaignUrl4Web> findUrlList(Long appid) {
        List<CampaignUrl4Web> result = new ArrayList<>();

        List<Campaign> campaigns = campaignRepository.findAllByApp(appid);
        List<SubCampaign> subCampaigns = subCampaignRepository.findAllByApp(appid);
        List<PackageSubCampaign> packageSubCampaignList = packageSubCampaignRepository.findAllByApp(appid);
        Map<String, CampaignUrl4Web> map = new HashMap<>();

        if (ValidateUtil.isValid(campaigns)) {
            for (Campaign ca : campaigns) {

                String url = ca.getUrl();

                if (ValidateUtil.isValid(url)) {
                    if (map.containsKey(url)) {
                        CampaignUrl4Web web = map.get(url);
//                        web.setCampaignNum(web.getCampaignNum() + 1);
                        web.setCampaignNum(web.getCampaignNum() + (ca.getDelFlag() ? 0 : 1));
                        web.setUrlName(StringUtils.isEmpty(ca.getUrlName()) ? url : ca.getUrlName());
                        web.setTabType("camp_tab");
                        web.setCampaignId(ca.getId());
                        map.put(url, web);
                    } else {

                        if (!ca.getDelFlag()) {
                            CampaignUrl4Web web = new CampaignUrl4Web();
                            web.setUrl(url);
                            web.setCampaignNum(1);
                            web.setSubCampaignNum(0);

                            web.setUrlName(StringUtils.isEmpty(ca.getUrlName()) ? url : ca.getUrlName());
                            web.setTabType("camp_tab");
                            web.setCampaignId(ca.getId());

                            map.put(url, web);
                        }


                    }
                }
            }
        }

        if (ValidateUtil.isValid(subCampaigns)) {
            for (SubCampaign sc : subCampaigns) {
                String url = sc.getUrl();
                if (ValidateUtil.isValid(url)) {
                    if (map.containsKey(url)) {
                        CampaignUrl4Web web = map.get(url);
//                        web.setSubCampaignNum(web.getSubCampaignNum() + 1);
                        web.setSubCampaignNum(web.getSubCampaignNum() + (sc.getDelFlag() ? 0 : 1));

                        web.setUrlName(StringUtils.isEmpty(sc.getUrlName()) ? url : sc.getUrlName());
                        web.setTabType("sub_camp_tab");
                        web.setCampaignId(sc.getId());


                        map.put(url, web);
                    } else {

                        if (!sc.getDelFlag()) {
                            CampaignUrl4Web web = new CampaignUrl4Web();
                            web.setUrl(url);
                            web.setSubCampaignNum(1);

                            web.setUrlName(StringUtils.isEmpty(sc.getUrlName()) ? url : sc.getUrlName());
                            web.setTabType("sub_camp_tab");
                            web.setCampaignId(sc.getId());

                            map.put(url, web);
                        }


                    }
                }
            }
        }

        if (ValidateUtil.isValid(packageSubCampaignList)) {
            for (PackageSubCampaign sc : packageSubCampaignList) {
                String url = sc.getUrl();
                if (ValidateUtil.isValid(url)) {
                    if (map.containsKey(url)) {
                        CampaignUrl4Web web = map.get(url);
                        web.setSubCampaignNum(web.getSubCampaignNum() + (sc.getDelFlag() ? 0 : 1));

                        web.setUrlName(StringUtils.isEmpty(sc.getUrlName()) ? url : sc.getUrlName());
                        web.setTabType("pk_sub_camp_tab");
                        web.setCampaignId(sc.getId());

                        map.put(url, web);
                    } else {

                        if (!sc.getDelFlag()) {

                            CampaignUrl4Web web = new CampaignUrl4Web();
                            web.setUrl(url);
                            web.setSubCampaignNum(1);

                            web.setUrlName(StringUtils.isEmpty(sc.getUrlName()) ? url : sc.getUrlName());
                            web.setTabType("pk_sub_camp_tab");
                            web.setCampaignId(sc.getId());
                            map.put(url, web);
                        }

                    }
                }
            }
        }

        if (ValidateUtil.isValid(map)) {
            for (String url : map.keySet()) {
                CampaignUrl4Web web = map.get(url);
                result.add(web);
            }
        }

        return result;
    }

    @Override
    public int modifyUrlName(Long campaignId, String name, String type) {

        int line = 0;

        if ("camp_tab".equals(type)) {

            line = campaignRepository.modifyUrlName(campaignId, name);
        } else if ("sub_camp_tab".equals(type)) {

            line = subCampaignRepository.modifyUrlName(campaignId, name);

        } else if ("pk_sub_camp_tab".equals(type)) {
            line = packageSubCampaignRepository.modifyUrlName(campaignId, name);
        }
        return line;
    }

    @Override
    public void updateUrlList(Long account, Long appid, String oldUrl, String newUrl) {
        List<Campaign> campaignList = campaignRepository.findAllByAppAndUrl(appid, oldUrl);
        List<SubCampaign> subCampaignList = subCampaignRepository.findAllByAppAndUrl(appid, oldUrl);
        List<PackageSubCampaign> packageSubCampaignList = packageSubCampaignRepository.findAllByAppAndUrl(appid, oldUrl);
        List<Campaign> saveList = new ArrayList<>();
        List<PackageSubCampaign> subList = new ArrayList<>();
        List<Long> appidList = new ArrayList<>();
        List<Long> channelIdList = new ArrayList<>();

        if (ValidateUtil.isValid(campaignList)) {
            List<Campaign> newCampaignList = new ArrayList<>();
            for (Campaign ca : campaignList) {
                Long app = ca.getApp();
                Long channel = ca.getChannel();
                if (!appidList.contains(app)) {
                    appidList.add(app);
                }
                if (!channelIdList.contains(channel)) {
                    channelIdList.add(channel);
                }

                ca.setUrl(newUrl);
                ca.setModifyAccount(account);
                ca.setModifyTime(new Date());
                ca.setAwssuccess(false);
                newCampaignList.add(ca);
            }
            saveList = campaignRepository.save(newCampaignList);
        }

        if (ValidateUtil.isValid(packageSubCampaignList)) {
            List<PackageSubCampaign> newSubCapaignList = new ArrayList<>();
            for (PackageSubCampaign sc : packageSubCampaignList) {
                sc.setUrl(newUrl);
                sc.setModifyAccount(account);
                sc.setModifyTime(new Date());
                newSubCapaignList.add(sc);
            }
            subList = packageSubCampaignRepository.save(newSubCapaignList);
        }
        if (ValidateUtil.isValid(subCampaignList)) {
            List<SubCampaign> newSubCapaignList = new ArrayList<>();
            for (SubCampaign sc : subCampaignList) {
                sc.setUrl(newUrl);
                sc.setModifyAccount(account);
                sc.setModifyTime(new Date());
                newSubCapaignList.add(sc);
            }
            subCampaignRepository.save(newSubCapaignList);
        }


        Map<Long, App> appMap = new HashMap<>();
        if (ValidateUtil.isValid(appidList)) {
            List<App> appList = appRepository.findByIds(appidList);
            for (App app : appList) {
                appMap.put(app.getId(), app);
            }
        }
        Map<Long, Channel> channelMap = new HashMap<>();
        if (ValidateUtil.isValid(channelIdList)) {
            List<Channel> channelList = channelRepository.findById(channelIdList);
            for (Channel channel : channelList) {
                channelMap.put(channel.getId(), channel);
            }
        }

        //2. 同步更新短链信息至aws-dynamodb
        if (ValidateUtil.isValid(saveList)) {
            for (Campaign campaign : saveList) {
                if (Constant.awsneed) {
                    if (!this.iNoSQLDBService.update(transform(campaign, appMap.get(campaign.getApp()), channelMap.get(campaign.getChannel())))) {
                        this.campaignRepository.updateAwsStatus(campaign.getId(), false);
                    }
                }
            }
        }


        //2. 子活动同步更新短链信息至aws-dynamodb
        if (ValidateUtil.isValid(subList)) {
            for (PackageSubCampaign campaign : subList) {
                if (Constant.awsneed) {
                    if (!this.subCampaignINoSQLService.update(packageSubCampaignService.transform(campaign))) {
                        this.packageSubCampaignRepository.updateAwsStatus(campaign.getId(), false);
                    }
                }
            }
        }
    }

    @Override
    public List<CampaignUrl4Web> findCampaignByUrl(Long appid, String url) {
        List<CampaignUrl4Web> result = new ArrayList<>();
        List<CampaignUrl4Web> temp = new ArrayList<>();
        List<Long> cidList = new ArrayList<>();

        if (!StringUtils.isEmpty(url)) {
            url = JSONObject.parseObject(url).get("url").toString();
        }

        List<Campaign> campaignList = campaignRepository.findAllByAppAndUrl(appid, url);
        if (ValidateUtil.isValid(campaignList)) {
            for (Campaign ca : campaignList) {
                Long cid = ca.getChannel();
                if (!cidList.contains(cid)) {
                    cidList.add(cid);
                }
                CampaignUrl4Web web = new CampaignUrl4Web();
                web.setCampaignName(ca.getName());
                web.setChannelId(cid);
                web.setTabType(CampaignUrl4Web.TAB_CAMP);
                web.setCampaignId(ca.getId());
                temp.add(web);
            }
        }

        Map<Long, String> channelMap = new HashMap<>();
        if (ValidateUtil.isValid(cidList)) {
            List<Channel> channelList = channelRepository.findById(cidList);
            if (ValidateUtil.isValid(channelList)) {
                for (Channel ch : channelList) {
                    channelMap.put(ch.getId(), ch.getName());
                }
            }
        }

        if (ValidateUtil.isValid(temp)) {
            for (CampaignUrl4Web web : temp) {
                Long cid = web.getChannelId();
                if (channelMap.containsKey(cid)) {
                    web.setChannelName(channelMap.get(cid));
                }
                result.add(web);
            }
        }

        return result;
    }

    @Override
    public void updateUrlListByCampin(Long id, Long appid, CampaignUrl4Web web) {

        List<CampaignUrl4Web> listdata = web.getCampaings();

        List<Long> campids = new ArrayList();
        List<Long> camp_sub_ids = new ArrayList();
        List<Long> camp_subpk_ids = new ArrayList();

        for (CampaignUrl4Web capweb : listdata) {

            if (CampaignUrl4Web.TAB_CAMP.equals(capweb.getTabType())) {

                campids.add(capweb.getCampaignId());

            } else if (CampaignUrl4Web.TAB_CAMP_SUN.equals(capweb.getTabType())) {

                camp_sub_ids.add(capweb.getSubCampaignId());

            } else if (CampaignUrl4Web.TAB_CAMP_SUBPK.equals(capweb.getTabType())) {

                camp_subpk_ids.add(capweb.getSubCampaignId());
            }

        }

        String new_url = web.getNewUrl();

        String urlName = web.getUrlName();

        if (StringUtils.isEmpty(urlName)) {
            urlName = new_url;
        }

        if (!StringUtils.isEmpty(new_url)) {
            if (campids.size() > 0) {

                int upline = campaignRepository.updataUlrWithIds(campids, new_url, urlName);
//              campaignRepository.updateUrlName(new_url,urlName);
                List<Campaign> campaigns = campaignRepository.findById(campids);
                List<App> apps = appRepository.findAppByCampaignIds(campids);
                List<Channel> channels = channelRepository.findChannelByCampaignIds(campids);
                Map<Long, App> appMap = new HashMap<>();
                Map<Long, Channel> channelMap = new HashMap<>();

                for (App app : apps) {
                    appMap.put(app.getId(), app);
                }

                for (Channel channel : channels) {
                    channelMap.put(channel.getId(), channel);
                }

                //2. 同步更新短链信息至aws-dynamodb
                for (Campaign campaign : campaigns) {
                    campaign.setUrl(new_url);
                    if (Constant.awsneed) {
                        if (!this.iNoSQLDBService.update(transform(campaign, appMap.get(campaign.getApp()), channelMap.get(campaign.getChannel())))) {
                            this.campaignRepository.updateAwsStatus(campaign.getId(), false);
                        }
                    }
                }
            }

            if (camp_sub_ids.size() > 0) {
                int upline = subCampaignRepository.updataUlrWithIds(camp_sub_ids, new_url, urlName);
            }

            if (camp_subpk_ids.size() > 0) {
                int upline = packageSubCampaignRepository.updataUlrWithIds(camp_subpk_ids, new_url, urlName);

                List<PackageSubCampaign> subCampaigns = packageSubCampaignRepository.findById(camp_subpk_ids);
                for (PackageSubCampaign campaign : subCampaigns) {
                    campaign.setUrl(new_url);
                    if (Constant.awsneed) {
                        if (!this.subCampaignINoSQLService.update(packageSubCampaignService.transform(campaign))) {
                            this.packageSubCampaignRepository.updateAwsStatus(campaign.getId(), false);
                        }
                    }
                }

            }
        }

        campaignRepository.updateUrlName(new_url, urlName, appid);
        subCampaignRepository.updateUrlName(new_url, urlName, appid);
        packageSubCampaignRepository.updateUrlName(new_url, urlName, appid);


    }

    @Override
    public List<Campaign> findByKeyword(Account loginAccount, Long app, String keyword, Boolean del_flag) {
        List<Campaign> result = new ArrayList<>();

        if (loginAccount.getIsSuperUser() || loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            if (del_flag == null) {
                result = this.campaignRepository.findLikeName(app, keyword);
            } else {
                result = this.campaignRepository.findLikeNameWithDel(app, keyword, del_flag);
            }


        } else {

            List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), app);

            if (!CollectionUtils.isEmpty(authCampaignList)) {

                if (del_flag == null) {
                    result = this.campaignRepository.findLikeName(app, keyword);
                } else {
                    result = this.campaignRepository.findLikeNameWithDel(app, keyword, del_flag);
                }


                //筛选授权渠道
                if (!CollectionUtils.isEmpty(result)) {

                    Iterator<Campaign> campaignIterator = result.iterator();

                    while (campaignIterator.hasNext()) {
                        Campaign campaign = campaignIterator.next();

                        if (!authCampaignList.contains(campaign.getId())) {
                            campaignIterator.remove();
                        }
                    }
                }
            }
        }


        return result;
    }

    @Override
    public Iterable<String> listAuthCampaignIds(Account account, Long appId, String channelType, boolean isDisable, String APPMARKET) {
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());
        if (null != roleAuth && (RoleEnumType.CHANNEL_PERSON.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_AUTH.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_ROLE.getKey().equals(roleAuth.getRoleCategory()))) {
            List<String> campaignList = new ArrayList<>();

            if (!StringUtils.isEmpty(channelType)) {

                if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {

                    List<String> authCampaignList = campaignRepository.findAuthCampaignIdsByCustomChannel(account.getId(), appId, isDisable, APPMARKET);
                    if (!CollectionUtils.isEmpty(authCampaignList)) {
                        campaignList.addAll(authCampaignList);
                    }

                    //查询创建活动
                    List<String> createCampaignList = campaignRepository.findIdsCreateByCustomChannel(account.getId(), appId, isDisable, APPMARKET);
                    if (!CollectionUtils.isEmpty(createCampaignList)) {
                        campaignList.addAll(createCampaignList);
                    }
                } else {

                    //查询授权的活动
                    List<String> authCampaignList = campaignRepository.findAuthCampaignIdsByChannelType(account.getId(), appId, channelType, isDisable, APPMARKET);
                    if (!CollectionUtils.isEmpty(authCampaignList)) {
                        campaignList.addAll(authCampaignList);
                    }

                    //查询创建活动
                    List<String> createCampaignList = campaignRepository.findIdsCreateByChannelType(account.getId(), appId, channelType, isDisable, APPMARKET);
                    if (!CollectionUtils.isEmpty(createCampaignList)) {
                        campaignList.addAll(createCampaignList);
                    }
                }
            }
        } else {

            return campaignRepository.findAllCampinIds(appId, APPMARKET);
        }

        return null;

    }


    @Override
    public Iterable<Campaign4Web> listAuthCampaignByType(Account loginAccount, Long appId, String channelType, boolean isDisable) {
        RoleAuth roleAuth = roleAuthRepository.findOne(loginAccount.getRoleCategory());

        if (null != roleAuth && (RoleEnumType.CHANNEL_PERSON.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_AUTH.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_ROLE.getKey().equals(roleAuth.getRoleCategory()))) {

            List<Campaign> campaignList = new ArrayList<>();

            if (!StringUtils.isEmpty(channelType)) {

                if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
                    //查询授权的活动
                    List<Campaign> authCampaignList = campaignRepository.findAuthCampaignByCustomChannel(loginAccount.getId(), appId, isDisable);
                    if (!CollectionUtils.isEmpty(authCampaignList)) {
                        campaignList.addAll(authCampaignList);
                    }

                    //查询创建活动
                    List<Campaign> createCampaignList = campaignRepository.findCreateByCustomChannel(loginAccount.getId(), appId, isDisable);
                    if (!CollectionUtils.isEmpty(createCampaignList)) {
                        campaignList.addAll(createCampaignList);
                    }

                } else {

                    //查询授权的活动
                    List<Campaign> authCampaignList = campaignRepository.findAuthCampaignByChannelType(loginAccount.getId(), appId, channelType, isDisable);
                    if (!CollectionUtils.isEmpty(authCampaignList)) {
                        campaignList.addAll(authCampaignList);
                    }
                    //查询创建活动
                    List<Campaign> createCampaignList = campaignRepository.findCreateByChannelType(loginAccount.getId(), appId, channelType, isDisable);
                    if (!CollectionUtils.isEmpty(createCampaignList)) {
                        campaignList.addAll(createCampaignList);
                    }
                }
            }

            //构建后返回
            return this.buildCampaign4WebWithCampaign(campaignList, channelType);

        } else {

            return isDisable ? this.listAllDisable(appId, loginAccount, channelType) : this.listAll(appId, loginAccount, channelType);
        }
    }

    @Override
    public List<Campaign4Web> listAuthCampaignByType(Account loginAccount, Long appId, String channelType, Campaign campaign, Map data) {

        authService.checkRoleTempCategory(loginAccount); //使用权限模板配置权限时做处理 及 角色类型兼容处理
        App app = appRepository.findOne(appId);
//        RoleAuth roleAuth = roleAuthRepository.findOne(loginAccount.getRoleCategory());

        RoleAuth roleAuth = new RoleAuth();
        roleAuth.setRoleCategory(loginAccount.getRoleCategory());

        campaign.setApp(appId);
        campaign.setChannelType(campaign.getType());
        channelType = campaign.getType();

        if (campaign.getChannel() != null && campaign.getChannel() == 1) {
            campaign.setChannel(0l);
        }
        Pageable pageable = null;
        if(campaign.isNoLimit()){
             pageable = new PageRequest(0,100000, Sort.Direction.DESC, "createTime");
        }else{
             pageable = new PageRequest(0, 1000, Sort.Direction.DESC, "createTime");
        }

//        data.put("total_flag", campaignRepository.findAllCount(appId));
        List<Campaign4Web> campaign4Webs;
        if (null != roleAuth && (RoleEnumType.CHANNEL_PERSON.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_AUTH.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_ROLE.getKey().equals(roleAuth.getRoleCategory()))) {

            List<Campaign> campaignList = new ArrayList<>();

            if (!StringUtils.isEmpty(channelType)) {

                //查询授权的活动
                List<Long> channels = campaignRepository.findChannelsByDataAuth(loginAccount.getId(), appId);
                List<Long> campaigns = campaignRepository.findCampaignIdsByDataAuth(loginAccount.getId(), appId);

                if (channels.size() == 0) {
                    channels.add(-1l);
                }
                if (campaigns.size() == 0) {
                    campaigns.add(-1l);
                }

                Map idsMap = new HashMap();
                idsMap.put("channels", channels);
                idsMap.put("campaigns", campaigns);
                idsMap.put("equal", "1");
                campaign.setCreateAccount(loginAccount.getId());
                if (CHANNEL_TYPE_CUSTOM.equals(channelType)) {
                    //查询授权的活动
                    campaign.setChannelType(null);
                } else {
                    //查询创建活动
                    campaign.setChannelType(channelType);
                }

                //查询授权的活动
                this.getCustomerCampins(campaign, idsMap, pageable, data, campaignList);

                idsMap = new HashMap();
                idsMap.put("equal", "2");
                //查询创建活动
                this.getCustomerCampins(campaign, idsMap, pageable, data, campaignList);

            }

            //构建后返回
            campaign4Webs = this.buildCampaign4WebWithCampaign(campaignList, channelType);
            campaign4Webs = campaign4Webs.stream()
                    .sorted(Comparator.comparing(Campaign4Web::getPubDate).reversed()).collect(Collectors.toList());

        } else {

            campaign4Webs = this.listAllDataPage(campaign, appId, channelType, pageable, data);
        }
        for (Campaign4Web campaign4Web : campaign4Webs) {
            campaign4Web.setOnlineDate(app.getOnlineDate());
            if (campaign4Web.getSubscribes() != null && campaign4Web.getSubscribes()) {
                campaign4Web.setImpTrackurl(null);
            }
        }


        return campaign4Webs;
    }


    @Override
    public List<Map> getChannelsNameAndId(Long appid, Account loginAccount, String type) {


        authService.checkRoleTempCategory(loginAccount); //使用权限模板配置权限时做处理 及 角色类型兼容处理
        RoleAuth roleAuth = new RoleAuth();
        roleAuth.setRoleCategory(loginAccount.getRoleCategory());

        List<Object[]> channels;

        if (null != roleAuth && (RoleEnumType.CHANNEL_PERSON.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_AUTH.getKey().equals(roleAuth.getRoleCategory())
                || RoleEnumType.CUSTOM_ROLE.getKey().equals(roleAuth.getRoleCategory()))) {

            channels = channelRepository.findAuthChannelNames(appid, loginAccount.getId(), type);
        } else {
            channels = campaignRepository.findChannelNames(appid, type);
        }
        List<Map> result = new ArrayList<>();
        if (channels != null) {
            result = channels.stream().map(v -> {
                Map data = new HashedMap(2);
                data.put("id", v[0]);
                data.put("name", v[1]);
                return data;
            }).collect(Collectors.toList());
            return result;
        }

        return result;
    }

    private void getCustomerCampins(Campaign campaign, Map idsMap, Pageable pageable, Map data, List campaignList) {
        Page<Campaign> pageCreat = campaignRepository.findAll(getCampineSpecification(campaign, idsMap), pageable);
        List<Campaign> createCampaignList = pageCreat.getContent();

        // 查询总数 时去掉搜索条件
        campaign.setChannel(0l);
        String name = campaign.getName();
        campaign.setName("");
        long count = campaignRepository.count(getCampineSpecification(campaign, idsMap));
        // 恢复模糊查询名称
        campaign.setName(name);

        if (data.get("total") == null) {
            data.put("total", count);
        } else {
            long total = Long.parseLong(data.get("total").toString());
            data.put("total", count + total);
        }

        if (!CollectionUtils.isEmpty(createCampaignList)) {
            campaignList.addAll(createCampaignList);
        }
    }

    @Override
    public List<Campaign4Web> listCampaignByLink(Iterable<Campaign4Web> iterable) {
        List<Campaign4Web> res = new ArrayList<>();
       /* if (ValidateUtil.isValid(iterable)) {
            for (Campaign4Web cam : iterable) {
                *//*      Long channel = cam.getChannel();
                Channel one = channelRepository.findOne(channel);*//*
                //去掉广点通渠道的显示
                if (cam.getChannel() == 5305L && cam.getSurl() != null && cam.getSurl().length() == 7) {
                    continue;
                }
                if ("ADVERTISING".equals(cam.getChannelCategory())) {
                    //cam.setChannelName(one.getName());
                    res.add(cam);
                }
            }
        }*/
        Collections.sort(res, new Comparator<Campaign4Web>() {
            @Override
            public int compare(Campaign4Web o1, Campaign4Web o2) {
                if (DateUtil.compareDate(o1.getPubDate(), o2.getPubDate()) == 1) {
                    return -1;
                }

                if (DateUtil.compareDate(o1.getPubDate(), o2.getPubDate()) == -1) {
                    return 1;
                }
                return 0;
            }
        });

        return res;
    }

    @Override
    public List<Campaign4Web> listCampaignByPackage(Iterable<Campaign4Web> iterable) {
        List<Campaign4Web> res = new ArrayList<>();
        if (ValidateUtil.isValid(iterable)) {
            for (Campaign4Web cam : iterable) {
                if ("APPMARKET".equals(cam.getChannelCategory())) {
                    res.add(cam);
                }
            }
        }
        return res;
    }

    @Override
    public List<String> filterChannelAndAuthCampaignId(Long accountId, Long appId, String channelIdString) {

        List<String> resultCampaignList = new ArrayList<>();

        if (StringUtils.isEmpty(channelIdString)) {
            return resultCampaignList;
        }

        //idList
        List<String> channelIdList = Arrays.asList(channelIdString.split(","));

        //app和渠道下的
        //List<Campaign> campaignList = campaignRepository.findByAppAndChannels(appId, channelIdList);

        // bugfix_32409
        List<String> campaignIdList = campaignRepository.findByAppAndChannelIds(appId, channelIdList);

       /* if (!CollectionUtils.isEmpty(campaignList)) {
            campaignIdList = Lists.transform(campaignList, new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });
        }*/

        //授权的所有渠道
        //List<Campaign> authCampaignList = authService.getAuthCampaignListByAccount(accountId, appId);

        List<String> authCampaignIdList = authService.getAuthCampaignIdListByAccount(accountId, appId, 1);

        if (!CollectionUtils.isEmpty(authCampaignIdList)) {

            /* List<String> authCampaignIdList = Lists.transform(authCampaignList, new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });*/

            //筛选出权限内的活动
            resultCampaignList = Lists.newArrayList(Iterables.filter(campaignIdList, new Predicate<String>() {
                @Override
                public boolean apply(String campaignid) {
                    return authCampaignIdList.contains(campaignid);
                }
            }));
        }

        return resultCampaignList;
    }

    /**
     * 处理小游戏的特殊url
     *
     * @return 去掉？之前映射的短链接
     */
    public Campaign operateWgShortUrl(Campaign resource) {


        boolean objIsNull = resource == null;
        boolean OsIsNull = !objIsNull && StringUtils.isEmpty(resource.getOs());

        if (objIsNull || OsIsNull) {

            return resource;
        }

        boolean isWg = AppPlatformEnum.WG.getKey().equals(resource.getOs());

        if (!isWg) {
            return resource;
        }

        if (!StringUtils.isEmpty(resource.getUrl())) {
            String str = resource.getUrl();
            String newUrl = null;
            if (str.contains("?")) {
                String[] strArr = str.split("\\?");
                newUrl = "?" + strArr[1];
            } else {
                newUrl = str;
            }

            if (resource.getUrl().equals(resource.getQrCodeAddress())) {
                resource.setQrCodeAddress(newUrl);
            }
            resource.setUrl(newUrl);

        }
        return resource;
    }

    /**
     * @param app
     * @param channel
     * @return 是否产品下存在该渠道活跃的推广活动  true 存在   false 不存在
     */
    @Override
    public Boolean checkActiveCampaign(Long app, Long channel) {
        List<Campaign> activeCampaign = campaignRepository.findByAppAndChannel(app, channel);
        Boolean hasActiveCampaign = false;//默认不存在活跃的推广活动
        if (activeCampaign != null && activeCampaign.size() > 0) {
            hasActiveCampaign = true;
        }
        return hasActiveCampaign;
    }

    /**
     * (跟上面的checkActiveCampaign的区别是，在查活动的时候，针对广点通的做了特殊处理，只查广点通中方案一的活动)
     * add by zhangsy 20180822
     *
     * @param app
     * @param channel
     * @return
     */
    @Override
    public Boolean checkActiveCampaignNew(Long app, Long channel) {
        List<Campaign> activeCampaign = campaignRepository.findByAppAndChannelNew(app, channel);
        Boolean hasActiveCampaign = false;//默认不存在活跃的推广活动
        if (activeCampaign != null && activeCampaign.size() > 0) {
            hasActiveCampaign = true;
        }
        return hasActiveCampaign;
    }

    @Override
    public List<String> findThirdAccountByArails(String keyword, App app, String channelId) {

        return campaignRepository.findThirdBackAccountByAppKeyWorrds(keyword, app.getId(), channelId);

    }

    @Override
    public void createVivoParam(List<Campaign> result, Campaign resource) {

        if (!resource.isVivo2DDb()) {
            return;
        }

        Map data = this.fillVivoParamMap(resource.getSpecialKeys());
        App app = appRepository.findOne(resource.getApp());

        if (result == null) {
            iNoSQLDBService.addVivoParamDatas(data, app.getAppkey(), resource.getCampaignid());
        } else {
            result.forEach(v -> {
                iNoSQLDBService.addVivoParamDatas(data, app.getAppkey(), v.getCampaignid());
            });
        }


    }

    /**
     * 功能描述:预约APP上线同步
     *
     * @author liyin
     * @date 2020/7/1
     */
    @Override
    public void doOnlineApp(Long appid) {

        List<App> apps;
        List<App> appsDB = new ArrayList<>();
        if (appid == null) {
            apps = appRepository.findAllAppByWhetherOnlineAndWhetherAppointmentMonitor();
        } else {
            App one = appRepository.findOne(appid);
            apps = new ArrayList();
            apps.add(one);
        }
        for (App app : apps) {
            if (app.getOnlineDate() != null) {
                Date today = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, DateUtil.getCurrentDateStr());
                Date onlineDate = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, app.getOnlineDate());
                if (onlineDate.compareTo(today) == -1 || onlineDate.compareTo(today) == 0) {//上线日期小于等于今日
                    //app上线
                    App one = appRepository.findOne(app.getId());
                    one.setWhetherOnline(true);
                    one.setWaitStopCampaign(true);
                    appsDB.add(one);
                }
            } else {
                logger.error("#####App预约自动上线AppKey:" + app.getAppkey() + ",缺少上线时间");
            }
        }
        appRepository.save(appsDB);

    }
    /**
     * 功能描述:预约App上线次日停用短链服务
     * @author liyin
     * @date 2021/1/28
     */
    @Override
    public void syncDeleteSubscribesSurlTask(Long appid) {
        List<App> apps;
        if (appid == null) {
            apps = appRepository.findAllAppByWaitStopCampaign();
        } else {
            App one = appRepository.findOne(appid);
            apps = new ArrayList();
            apps.add(one);
        }
        for (App app : apps) {
            if (app.getOnlineDate() != null) {
                Date today = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, DateUtil.getCurrentDateStr());
                Date onlineDate = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, app.getOnlineDate());
                if (onlineDate.compareTo(today) == -1) {//上线日期小于今日
                    App one = appRepository.findOne(app.getId());
                    one.setWaitStopCampaign(false);
                    appRepository.save(one);
                    //关闭广点通渠道的预约app推广
                    GdtPlan gdtPlan = gdtPlanRepository.getGdtPlanByAppkeyAndPlanAndSubscribes(app.getAppkey(), 1, true);
                    if (gdtPlan != null) {
                        gdtPlan.setSubscribes(false);
                        gdtPlanRepository.save(gdtPlan);
                    }
                    //停用预约活动
                    List<Campaign> campaigns = campaignRepository.findByAppAndSubscribes(one.getId(), true);
                    for (Campaign campaign : campaigns) {
                        try {
                            this.delete(app.getId(), campaign.getId(), 0L);
                        } catch (Exception e) {
                            logger.error("#####自动上线活动停用失败campaignid:" + campaign.getCampaignid() + "");
                        }
                    }
                }
            } else {
                logger.error("#####停用短链服务AppKey:" + app.getAppkey() + ",缺少上线时间");
            }
        }
    }
    @Override
    public void doOnlineAppSyncDingDingAndEmailMSGTask() {
        if (!"true".equals(Constant.mailProd)) {
            logger.info("非线上环境不进行钉钉邮件通知操作");
            return;
        }
        List<App> apps = appRepository.findAllAppByWhetherOnlineAndWhetherAppointmentMonitor();
        DingDingSendMsgUtil dingDingUtil = new DingDingSendMsgUtil();
        StringBuilder sb = new StringBuilder();
        for (App app : apps) {
            if (app.getOnlineDate() != null) {
                try {
                    int between = DateUtil.daysBetween(DateUtil.getCurrentDateStr(), app.getOnlineDate());
                    if (between == 1 || between == 3 || between == 7 || between == 15) {
                        Account rootAccount = accountRepository.findOne(app.getRootParent());
                        //发送邮件
                        List<String> email = new ArrayList<String>();
                        email.add(rootAccount.getEmail());
                        String subject = "产品上线提醒【热云数据】";
                        String contents = "<!doctype html><html><head></head><body> " +
                                "尊敬的用户：</br>" +
                                "距离您的产品\"" + app.getName() + "\"上线还有" + between + "天，如您已在App Store修改了预计上线日期，请将最新的产品上线日期同步更新至TrackingIO，" +
                                "<a href=\"http://" + Constant.getDomainStr() + "/\">点击进入TrackingIO</a>。" +
                                "</body></html>";
                        boolean b = false;
                        b = SendCommonPostMail.sendMailReuse(email, subject, contents, null, null, new HashMap<>());
                        //钉钉
                        sb.append("Company：").append(rootAccount.getCompany()).append("\n")
                                .append("Email：").append(rootAccount.getEmail()).append("\n")
                                .append("App Key：").append(app.getAppkey()).append("\n")
                                .append("App Name：").append(app.getName()).append("\n")
                                .append("预计上线时间：").append(app.getOnlineDate()).append("\n")
                                .append("距离上线天数：").append(between + "天").append("\n")
                                .append("客户邮件是否下达：").append(b).append("\n\n");
                    }

                } catch (Exception e) {
                    logger.error("#####邮件发送错误AppKey:" + app.getAppkey(), e);
                }
            } else {
                logger.error("#####AppKey:" + app.getAppkey() + ",缺少上线时间");
            }
        }
        if (!StringUtil.isEmpty(sb.toString())) {
            //发送钉钉提醒
            long timeMillis = System.currentTimeMillis();
            String sign = dingDingUtil.getSignNew(timeMillis, DingDingSendMsgUtil.APP_ONLINE_MSG);
            String url = "https://oapi.dingtalk.com/robot/send?access_token=d2b4b62809fe0e469d8487ffe2e8678d596fb3ed10a2fc731df274fd7c473d3a&timestamp=" + timeMillis + "&sign=" + sign;
            dingDingUtil.sendMsg(url, sb.toString());
        }
    }

    @Override
    public List<Campaign> findByKeyword(Account loginAccount, Long app, String keyword, Boolean del_flag, boolean subscribe) {
        List<Campaign> result = new ArrayList<>();

        if (loginAccount.getIsSuperUser() || loginAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            if (del_flag == null) {
                result = this.campaignRepository.findLikeName(app, keyword);
            } else {
                result = this.campaignRepository.findLikeNameWithDel(app, keyword, del_flag);
            }


        } else {

            List<Long> authCampaignList = authService.getAuthCampaignList(loginAccount.getId(), app);

            if (!CollectionUtils.isEmpty(authCampaignList)) {

                if (del_flag == null) {
                    result = this.campaignRepository.findLikeName(app, keyword);
                } else {
                    result = this.campaignRepository.findLikeNameWithDel(app, keyword, del_flag);
                }


                //筛选授权渠道
                if (!CollectionUtils.isEmpty(result)) {

                    Iterator<Campaign> campaignIterator = result.iterator();

                    while (campaignIterator.hasNext()) {
                        Campaign campaign = campaignIterator.next();

                        if (!authCampaignList.contains(campaign.getId())) {
                            campaignIterator.remove();
                        }
                    }
                }
            }
        }
        List<Campaign> result_new = new ArrayList<>();
        if (subscribe) {
            for (Campaign campaign : result) {
                if (campaign.getSubscribes() != null && true == campaign.getSubscribes()) {
                    result_new.add(campaign);
                }
            }
            return result_new;
        } else {
            for (Campaign campaign : result) {
                if (campaign.getSubscribes() == null || true != campaign.getSubscribes()) {
                    result_new.add(campaign);
                }
            }
            return result_new;
        }
    }

    @Override
    public List<Campaign> findByCampaignidList(List<String> campaignidList, Long app, boolean subscribe) {
        if (!subscribe) {
            return CollectionUtils.isEmpty(campaignidList) ? new ArrayList<Campaign>() : campaignRepository.findByCampaignidAndSubscribeNotTrue(campaignidList, app);
        }
        return CollectionUtils.isEmpty(campaignidList) ? new ArrayList<Campaign>() : campaignRepository.findByCampaignid(campaignidList, app, subscribe);
    }

    private Map<String, String> fillVivoParamMap(String speciakey) {
        Map<String, String> data = new HashMap();
        Arrays.asList(speciakey.split("&")).forEach(v -> {
            String[] kv = v.replaceFirst("=", "@").split("@");
            data.put(kv[0], kv[1]);
        });

        //获取签名
        Map apiQuery = new HashMap();
        apiQuery.put("apiKey", data.get("apiKey"));
        apiQuery.put("requestStr", "{\"reportDate\":\"20181112\"}");
        String response = HttpClientUtil.doHttpPostRequest(VIVO_API_SIGN_URL, "", apiQuery);
        data.put("sign", response);
        return data;
    }


    @Override
    public boolean checkPackChannle(Campaign resource, HttpServletRequest request) {

        if (resource.getChannel() == null) {
            return false;
        }

        Channel channel = channelRepository.findById(resource.getChannel());

        if (channel == null || ("APPMARKET".equals(channel.getCategory()) && StringUtils.isEmpty(resource.getPack_channelid()))) {
            //分包渠道时 Pack_channelid 不能为空
            return false;
        }

        return true;
    }

    @Override
    public Map<String,Boolean> dropDownList(Long appid) {
        Map<String, Boolean> map = new HashMap<>();
        Campaign campaign = campaignRepository.findByAppAndSubscribesLimitOne(appid,true);
        if(campaign!=null){
            map.put("subscribes",true);
        }else{
            map.put("subscribes",false);
        }
        Campaign campaignDeeplink = campaignRepository.findByAppAndDeeplinkLimitOne(appid);
        if(campaignDeeplink!=null){
            map.put("deepLink",true);
        }else{
            map.put("deepLink",false);
        }
        return map;
    }



    public static void main(String[] args) {
//        System.out.println("123,1231,123434".indexOf("123") >= 0);

//        Map apiQuery = new HashMap();
//        apiQuery.put("apiKey", "c3d200288f50ccc3c1e77a68e9afdd22");
//        apiQuery.put("requestStr", "{\"reportDate\":\"20181112\"}");
//        String response = HttpClientUtil.doHttpPostRequest(VIVO_API_SIGN_URL, "", apiQuery);
//        JSONObject jsonResponse = JSONObject.parseObject(response);
//        System.out.println(jsonResponse);

        String appkey = "apiKey===jn+rY7ZgrIbo7XQzsrbd/Q==";

        System.out.println(appkey.replaceFirst("=", ":@"));
    }

}
