package com.reyun.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Preconditions;
import com.google.common.base.Strings;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.OperateObjectEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.*;
import com.reyun.util.*;
import com.reyun.util.NoSQLDBUtil.Result;
import org.apache.commons.lang.StringUtils;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.joda.time.DateTime;
import org.json.JSONException;
import org.json.JSONObject;
import org.mortbay.util.UrlEncoded;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigInteger;
import java.net.URI;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * Created by mxq on 2017/7/11.
 */
@Service
@Transactional
public class ChannelAccountServiceImpl
        implements ChannelAccountService {
    private static final Logger logger = LoggerFactory.getLogger(ChannelAccountServiceImpl.class);
    private static final String CLIENT_ID = "1106237187";
    private static final String CLIENT_SECRET = "BvITHJpxu0gufAxF";
    //今日头条appid
    private static final String JRTT_APPID = "1601401724412931";
    //今日头条secret
    private static final String JRTT_SECRET = "7d4ac56b0c42873c9ee879c320bd89b05b5788b4";
    private static final String redirectUrl = "api/mng/channelaccount/code";
    private static final String UNIQUE_GDT = "gdt";
    private final String UCTOUTIAO = "uctoutiao";
    private final String SOHU = "huisuan";

    @Autowired
    private UserLogService userLogService;

    @Autowired
    private ChannelAccountRepository channelAccountRepository;

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private AuthService authService;

    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    private ChannnelAccountLinkAppRepository channnelAccountLinkAppRepository;

    @Autowired
    private ChannnelAccountLinkAppService channnelAccountLinkAppService;

    @Autowired
    private RecycleChannelAccountRepository recycleChannelAccountRepository;

    @Autowired
    private AppService appService;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private GdtPlanRepository gdtPlanRepository;
    @Autowired
    private GdtPlanLogRepository gdtPlanLogRepository;
    @Autowired
    private ToutiaoNumRepository toutiaoNumRepository;
    @Autowired
    private BlacklistApplicationRepository blacklistApplicationRepository;

    static Map<Long, String> apiMap;

    static {
        apiMap = new HashMap<>();
        apiMap.put(514L, "toutiao_");
        apiMap.put(5305L, "gdt_");
        apiMap.put(6481L, "baiduysgg_");
        apiMap.put(2586L, "kuaishou_");
        apiMap.put(5306L, "wxmp_");
    }

    @Override
    public boolean simulationLogin(ChannelAccount resource) {
        Boolean flag = false;
        Channel channel = channelRepository.findOne(resource.getChannel());

        //广点通另行处理
        if ("gdt".equals(channel.getUniqueName())) {
            return true;
        }

        String url = Constant.simulationLoginUrl + channel.getUniqueName() + "?name=" + resource.getName() + "&pwd=" + URLEncoder.encode(resource.getPassword());
        String res = HttpClientUtil.doHttpPostRequest(url, "");

        JSONObject jsonObject = null;
        try {
            jsonObject = new JSONObject(res);
            JSONObject content = jsonObject.getJSONObject("content");
            flag = content.getBoolean("success");
        } catch (JSONException e) {
            logger.error(e.getMessage(), e);
        }

        return flag;
    }

    @Override
    public boolean validAlias(String alias, long account) {
        List<Long> accountIds = getLinkAccounts(account);

        boolean flag = true;
        List<String> aliasList = channelAccountRepository.listAliasByAccounts(accountIds);
        if (ValidateUtil.isValid(aliasList)) {
            if (aliasList.contains(alias)) {
                flag = false;
            }
        }

        return flag;
    }

    @Override
    public List<Long> getLinkAccounts(long account) {
        List<Long> accountIds = new ArrayList<>();
        Account one = accountRepository.findOne(account);
        if (one.getIsSuperUser()) {
            accountIds.add(account);
            List<Account> sonAccounts = accountRepository.findAccountByParent(account);
            if (ValidateUtil.isValid(sonAccounts)) {
                for (Account ac : sonAccounts) {
                    accountIds.add(ac.getId());
                }
            }
        } else {
            Account rootParentAccount = authService.findRootParentAccount(account);
            accountIds.add(rootParentAccount.getId());
            List<Account> sonAccounts = accountRepository.findAccountByParent(rootParentAccount.getId());
            if (ValidateUtil.isValid(sonAccounts)) {
                for (Account ac : sonAccounts) {
                    accountIds.add(ac.getId());
                }
            }
        }
        return accountIds;
    }

    @Override
    public ChannelAccount create(ChannelAccount resource, long loginAccount) {
        Preconditions.checkArgument(!Strings.isNullOrEmpty(resource.getAlias()), "渠道别名不允许为空");
        List<ChannelAccount> channelAccounts = channelAccountRepository.findByAliasAndCreateAccount(resource.getAlias(), loginAccount);
        if (channelAccounts != null && channelAccounts.size() > 0) {
            throw new TipException("渠道别名已存在.");
        }

        resource.setStatus(true);
        resource.setCreateAccount(loginAccount);
        resource.setCreateTime(new Date());

        Account rootParentAccount = authService.findRootParentAccount(loginAccount);
        resource.setRootAccount(rootParentAccount != null ? rootParentAccount.getId() : loginAccount);
        return channelAccountRepository.save(resource);
    }

    @Override
    public ChannelAccount update(ChannelAccount resource, long account) {
        ChannelAccount one = channelAccountRepository.findOne(resource.getId());
        one.setStatus(true);
        one.setAlias(resource.getAlias());
        one.setName(resource.getName());
        one.setPassword(resource.getPassword());
        one.setRebate(resource.getRebate());
        one.setModifyTime(new Date());
        one.setModifyAccount(account);

        ChannelAccount channelAccount = channelAccountRepository.save(one);
        return channelAccount;
    }

    @Override
    public ChannelAccount delete(long id, long account) {
        ChannelAccount one = channelAccountRepository.findOne(id);

        one.setModifyAccount(account);
        one.setModifyTime(new Date());
        one.setStatus(false);

        ChannelAccount channelAccount = channelAccountRepository.save(one);
        delDDB(channelAccount);
        return channelAccount;
    }
    public void delDDB(ChannelAccount channelAccount){
        if (apiMap.containsKey(channelAccount.getChannel())) {
            String fix = apiMap.get(channelAccount.getChannel());
            NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TABLE_MARKETKNGAPI, "id",
                    fix + channelAccount.getThirdAccountCode());
            Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_MARKETKNGAPI, "id",
                    fix + "advertiser_id_list");
            if (ValidateUtil.isValid(resultItem)) {
                Set<String> advertiserIdSet = resultItem.getStringSet("datas");
                advertiserIdSet.remove(channelAccount.getThirdAccountCode());
                resultItem.withStringSet("datas", advertiserIdSet);
                if (CollectionUtils.isEmpty(advertiserIdSet)) {
                    NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TABLE_MARKETKNGAPI, "id",
                            fix + "advertiser_id_list");
                } else {
                    NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TABLE_MARKETKNGAPI, resultItem);
                }
            }
        }
    }
    @Override
    public ChannelAccount enable(long id, long account) {
        ChannelAccount one = channelAccountRepository.findOne(id);
        one.setStatus(true);
        one.setModifyAccount(account);
        one.setModifyTime(new Date());

        ChannelAccount channelAccount = channelAccountRepository.save(one);
        if (apiMap.containsKey(channelAccount.getChannel())) {
            //禅道号459  要求广点通在2020-01-16后授权或重新授权的入ddb,因为之前授权的广点通token都没有获取消耗的权限
            //微信mp绑定完直接是开启状态,再次开启需要重新绑定
            if (channelAccount.getChannel() == 5305L || (channelAccount.getChannel() == 5305L && DateUtil.compareDate(channelAccount.getCreateTime(), DateUtil.parseDate("2020-01-16")) < 1)) {
                return channelAccount;
            }
            List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
            NoSQLDBUtil.putMarketingapi(channelAccount, apps, true, apiMap.get(channelAccount.getChannel()));
        }

        return channelAccount;
    }

    @Override
    public boolean createByBaidu(ChannelAccount resource, Long loginAccount) {
        boolean flag = false;
        Preconditions.checkArgument(!Strings.isNullOrEmpty(resource.getAlias()), "渠道别名不允许为空");
        Channel one = channelRepository.findOne(6481L);
        ChannelAccount byName = channelAccountRepository.findByName(resource.getName(), one.getUniqueName());
        if (byName != null) return false;
        //百度原生ocpc
        String url = "https://api.baidu.com/json/sms/service/AccountService/getAccountInfo";

        // 请求参数
        List<String> objects = new ArrayList(2) {
            {
                add("budget");
                add("balance");
            }

            ;
        };
        HashMap<Object, Object> body = new HashMap<>(1);
        body.put("accountFields", objects);
        Map<String, Object> header = new HashMap(3) {
            {
                put("username", resource.getName());
                put("password", resource.getPassword());
                put("token", resource.getAccessToken());
/*
                put("username", "BJ热云");
                put("password", "Reyun888");
                put("token", "36b7c34b5a47267fa72cd582910fde99");
*/
            }
        };
        Map<String, Object> data = new HashMap(2) {
            {
                put("body", body);
                put("header", header);
            }
        };
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                if (jsonObject.containsKey("header") && "success".equals(jsonObject.getJSONObject("header").getString("desc"))) {
                    String userId = jsonObject.getJSONObject("body").getJSONArray("data").getJSONObject(0).getString("userId");
                    resource.setCreateAccount(loginAccount);
                    resource.setCreateTime(new Date());
                    resource.setRootAccount(accountRepository.findOne(loginAccount).getRootParent());
                    resource.setChannel(one.getId());
                    resource.setUniqueName(one.getUniqueName());
                    resource.setThirdAccountCode(userId);
                    ChannelAccount channelAccount = channelAccountRepository.save(resource);
                    flag = true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return flag;
    }

    public boolean createByKuaishou(ChannelAccount resource, Long loginAccount) {
        String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
        // 请求参数
        Map<String, Object> data = new HashMap(3) {
            {
                put("app_id", resource.getName());
                put("secret", resource.getPassword());
                put("refresh_token", resource.getRefreshToken());
            }
        };
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                if ("OK".equals(jsonObject.getString("message")) && jsonObject.getJSONObject("data").size() > 0) {
                    Account account = accountRepository.findOne(loginAccount);
                    Preconditions.checkArgument(!Strings.isNullOrEmpty(resource.getAlias()), "渠道别名不允许为空");
                    Channel one = channelRepository.findOne(2586L);
                    /*ChannelAccount byName = channelAccountRepository.findByName(resource.getName(), one.getUniqueName());
                    if (byName != null) return false;*/
                    resource.setCreateAccount(loginAccount);
                    resource.setCreateTime(new Date());
                    resource.setChannel(one.getId());
                    resource.setUniqueName(one.getUniqueName());
                    resource.setRootAccount(account.getRootParent());
                    resource.setStatus(false);
                    com.alibaba.fastjson.JSONObject rs = jsonObject.getJSONObject("data");
                    String access_token = rs.getString("access_token");
                    resource.setAccessToken(access_token);
                    String refresh_token_expires_in = rs.getString("refresh_token_expires_in");
                    resource.setRefreshTokenExpire(Long.parseLong(refresh_token_expires_in));
                    String refresh_token = rs.getString("refresh_token");
                    resource.setRefreshToken(refresh_token);
                    String access_token_expires_in = rs.getString("access_token_expires_in");
                    resource.setAccessTokenExpire(Long.parseLong(access_token_expires_in));
                    String advertiser_id = rs.getString("advertiser_id");
                    resource.setThirdAccountCode(advertiser_id);
                    channelAccountRepository.save(resource);
                    return true;
                } else {
                    logger.info("绑定百度账号返回错误[{}]", result);
                }
            }
        } catch (Exception e) {
            logger.error("绑定百度账号错误", e);
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return false;
    }

    @Override
    public boolean updateByKuaishou(ChannelAccount resource, Long loginAccount) {
        ChannelAccount channelAccount = channelAccountRepository.findOne(resource.getId());
        channelAccount.setAlias(resource.getAlias());
        channelAccount.setName(resource.getName());
        channelAccount.setPassword(resource.getPassword());
        channelAccount.setRefreshToken(resource.getRefreshToken());
        String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
        // 请求参数
        Map<String, Object> data = new HashMap(3) {
            {
                put("app_id", channelAccount.getName());
                put("secret", channelAccount.getPassword());
                put("refresh_token", channelAccount.getRefreshToken());
            }
        };
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                if ("OK".equals(jsonObject.getString("message")) && jsonObject.getJSONObject("data").size() > 0) {
                    Account account = accountRepository.findOne(loginAccount);
                    Preconditions.checkArgument(!Strings.isNullOrEmpty(channelAccount.getAlias()), "渠道别名不允许为空");
                    Channel one = channelRepository.findOne(2586L);
                    /*ChannelAccount byName = channelAccountRepository.findByName(channelAccount.getName(), one.getUniqueName());
                    if (byName == null) return false;*/
                    channelAccount.setCreateAccount(loginAccount);
                    channelAccount.setChannel(one.getId());
                    channelAccount.setUniqueName(one.getUniqueName());
                    channelAccount.setRootAccount(account.getRootParent());
                    channelAccount.setModifyTime(new Date());
                    //channelAccount.setStatus(false);
                    com.alibaba.fastjson.JSONObject rs = jsonObject.getJSONObject("data");
                    String access_token = rs.getString("access_token");
                    channelAccount.setAccessToken(access_token);
                    String refresh_token_expires_in = rs.getString("refresh_token_expires_in");
                    channelAccount.setRefreshTokenExpire(Long.parseLong(refresh_token_expires_in));
                    String refresh_token = rs.getString("refresh_token");
                    channelAccount.setRefreshToken(refresh_token);
                    String access_token_expires_in = rs.getString("access_token_expires_in");
                    channelAccount.setAccessTokenExpire(Long.parseLong(access_token_expires_in));
                    String advertiser_id = rs.getString("advertiser_id");
                    channelAccount.setThirdAccountCode(advertiser_id);
                    ChannelAccount save = channelAccountRepository.save(channelAccount);
                    List<App> apps = appRepository.findValidAppByRootParent(save.getRootAccount());
                    NoSQLDBUtil.putMarketingapi(save, apps, true, apiMap.get(save.getChannel()));
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return false;
    }

    @Override
    public boolean createByWeixinMP(String state, String authorization_code, Account loginAccount) {
        //微信mp启用和创建共用一个接口
        Account account = accountRepository.findOne(loginAccount.getId());
        String url = new StringBuffer("https://api.e.qq.com/oauth/token?")
                .append("client_id=").append("1106237187")
                .append("&client_secret=").append("BvITHJpxu0gufAxF")
                .append("&grant_type=authorization_code")
                .append("&authorization_code=").append(authorization_code)
                .append("&redirect_uri=http://"+Constant.getDomainStr()+"/api/mng/channelaccount/createByWeixinMP").toString();
        // 请求参数
        // 构造请求
        HttpGet httpEntity = new HttpGet(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());

                if (jsonObject.getInteger("code") == 0) {
                    Channel channel = channelRepository.findOne(5306L);
                    com.alibaba.fastjson.JSONObject rs = jsonObject.getJSONObject("data");
                    com.alibaba.fastjson.JSONObject authorizerInfo = rs.getJSONObject("authorizer_info");
                    String account_id = authorizerInfo.getString("account_id");
                    ChannelAccount channelAccount = channelAccountRepository.findByThirdAccountCode(account_id, channel.getUniqueName());
                    if (channelAccount == null) {
                        channelAccount = new ChannelAccount();
                        channelAccount.setCreateTime(new Date());
                    }
                    channelAccount.setModifyTime(new Date());
                    channelAccount.setAlias(state);

                    String access_token = rs.getString("access_token");
                    channelAccount.setAccessToken(access_token);
                    String refresh_token_expires_in = rs.getString("refresh_token_expires_in");
                    channelAccount.setRefreshTokenExpire(Long.parseLong(refresh_token_expires_in));
                    String refresh_token = rs.getString("refresh_token");
                    channelAccount.setRefreshToken(refresh_token);
                    String access_token_expires_in = rs.getString("access_token_expires_in");
                    channelAccount.setAccessTokenExpire(Long.parseLong(access_token_expires_in));

                    channelAccount.setThirdAccountCode(account_id);
                    String wechat_account_id = authorizerInfo.getString("wechat_account_id");
                    channelAccount.setName(wechat_account_id);
                    channelAccount.setRootAccount(account.getRootParent());
                    channelAccount.setChannel(channel.getId());
                    channelAccount.setUniqueName(channel.getUniqueName());
                    channelAccount.setStatus(true);
                    channelAccount.setCreateAccount(loginAccount.getId());
                    channelAccountRepository.save(channelAccount);
                    List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                    NoSQLDBUtil.putMarketingapi(channelAccount, apps, true, apiMap.get(channelAccount.getChannel()));
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    @Override
    public ChannelAccount getByAliasAndChannelTrue(String alias, long channel) {
        return channelAccountRepository.getByAliasAndChannelTrue(alias, channel);
    }

    @Override
    public boolean renewalDeepLink(Account loginAccount, String feedbackurl, String urlScheme, String state) {
        GdtPlan gdtPlan = this.gdtPlanRepository.findByRootAccountAndUrl(loginAccount.getRootParent(), feedbackurl);
        if (gdtPlan == null) return false;
        if (state.equals("open")) {
            urlScheme = UrlEncoded.encodeString(urlScheme);
            this.gdtPlanRepository.updateUrlSchemeById(urlScheme, gdtPlan.getId());
            return true;
        } else if (state.equals("close")) {
            this.gdtPlanRepository.updateUrlSchemeById("", gdtPlan.getId());
            return true;
        }
        return false;
    }

    @Override
    public boolean renewalSubscribes(Account loginAccount, String feedbackurl, String state) {
        GdtPlan gdtPlan = this.gdtPlanRepository.findByRootAccountAndUrl(loginAccount.getRootParent(), feedbackurl);
        if (gdtPlan == null || gdtPlan.getPlan() == 2) return false;
        App app = appRepository.findByAppkey(gdtPlan.getAppkey());
        if(app.getOnlineDate()!=null){
            Date today = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, DateUtil.getCurrentDateStr());
            Date onlineDate = DateUtil.parseDate(DateUtil.C_DATE_PATTON_DEFAULT, app.getOnlineDate());
            if(onlineDate.compareTo(today)!=-1){
                if (state.equals("open")) {
                    this.gdtPlanRepository.updateSubscribesById(true,gdtPlan.getId());
                    return true;
                } else if (state.equals("close")) {
                    this.gdtPlanRepository.updateSubscribesById(false,gdtPlan.getId());
                    return true;
                }
            }
        }

        return false;
    }

    @Override
    public boolean updateByBaidu(ChannelAccount resource, Long loginAccount) {
        ChannelAccount one = channelAccountRepository.findOne(resource.getId());
        if (one == null) return false;
        boolean flag = false;
        Preconditions.checkArgument(!Strings.isNullOrEmpty(resource.getAlias()), "渠道别名不允许为空");

        //百度原生ocpc
        String url = "https://api.baidu.com/json/sms/service/AccountService/getAccountInfo";

        // 请求参数
        List<String> objects = new ArrayList(2) {
            {
                add("budget");
                add("balance");
            }

            ;
        };
        HashMap<Object, Object> body = new HashMap<>(1);
        body.put("accountFields", objects);
        Map<String, Object> header = new HashMap(3) {
            {
                /*put("username", "BJ热云");
                put("password", "Reyun888");
                put("token", "36b7c34b5a47267fa72cd582910fde99");*/
                put("username", resource.getName());
                put("password", resource.getPassword());
                put("token", resource.getAccessToken());
            }
        };
        Map<String, Object> data = new HashMap(2) {
            {
                put("body", body);
                put("header", header);
            }
        };
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                if (jsonObject.containsKey("header") && "success".equals(jsonObject.getJSONObject("header").getString("desc"))) {
                    String userId = jsonObject.getJSONObject("body").getJSONArray("data").getJSONObject(0).getString("userId");
                    one.setModifyAccount(loginAccount);
                    one.setModifyTime(new Date());
                    one.setThirdAccountCode(userId);
                    one.setName(resource.getName());
                    one.setAlias(resource.getAlias());
                    one.setAccessToken(resource.getAccessToken());
                    one.setStatus(false);
                    one.setPassword(resource.getPassword());
                    channelAccountRepository.save(one);
                    flag = true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return flag;
    }

    @Override
    public Object channellist(Account loginAccount) {
        List<String> names = new ArrayList<>();
        names.add("广点通");
        names.add("神马搜索");
        names.add("adwords");
        names.add("今日头条");
        names.add("百度原生OCPC");
        names.add("快手");
        names.add("微信mp");
        List<Channel> channels = new ArrayList<>();
        for (String name : names) {
            Channel channel = channelRepository.findByNames(name);
            if(channel!=null)channels.add(channel);
        }
        for (Channel channel : channels) {
            channel.setThirdAccountNum(channelAccountRepository.findByCreateAccountCid(loginAccount.getRootParent(), channel.getId()).size());
        }
        return channels;
    }

    @Override
    public List<ChannelAccount> find(Account account, long cid) {
        List<ChannelAccount> result = new ArrayList<>();
        List<ChannelAccount> channelAccountList = channelAccountRepository.findByCreateAccountCid(account.getRootParent(), cid);

        if (ValidateUtil.isValid(channelAccountList)) {
            List<Long> idList = new ArrayList<>();
            for (ChannelAccount channelAccount : channelAccountList) {
                idList.add(channelAccount.getId());
                //广点通账号id需要拼到一条中
                if (cid == 5305) {
                    String thirdAccountCode = channelAccount.getThirdAccountCode();
                    String acccountid = channnelAccountLinkAppRepository.getStringByChannelAccountId(channelAccount.getId());
                    //广点通存在新版旧版兼容 禅道1490
                    if(thirdAccountCode!=null&&acccountid!=null&&!thirdAccountCode.equals(acccountid)){
                        channelAccount.setThirdAccountCode(acccountid);
                    }
                }
            }
            List<ChannelAccountLinkApp> linkAppList = channnelAccountLinkAppRepository.findByLinkUserIds(idList);
            Map<Long, List<ChannelAccountLinkApp>> map = new HashMap<>();
            if (ValidateUtil.isValid(linkAppList)) {
                for (ChannelAccountLinkApp la : linkAppList) {
                    if (map.containsKey(la.getLinkUserId())) {
                        List<ChannelAccountLinkApp> list = map.get(la.getLinkUserId());
                        list.add(la);
                        map.put(la.getLinkUserId(), list);
                    } else {
                        List<ChannelAccountLinkApp> list = new ArrayList<>();
                        list.add(la);
                        map.put(la.getLinkUserId(), list);
                    }
                }
            }
            for (ChannelAccount ca : channelAccountList) {
                if(ca.getChannel()!=514L&&ca.getChannel()!=2586L) {//这些渠道不需要查询绑定app
                ca.setHaveLinkApp(this.appService.haveLinkApp(ca.getId()));
                if (map.containsKey(ca.getId())) {
                    ca.setTestSuccess(true);
                    List<ChannelAccountLinkApp> list = map.get(ca.getId());
                    for (ChannelAccountLinkApp la : list) {
                        if (null != la.getTestSuccess() && !la.getTestSuccess()) {
                            ca.setTestSuccess(false);
                        }
                    }
                } else {
                    ca.setTestSuccess(true);
                }
                }
                result.add(ca);
            }

        }
        return result;
    }

    @Override
    public Map findChannelAccountCount(Account account, Map<String, String> channleids) {

//        gdt,adwords,smsearch

        List<BigInteger> count = channelAccountRepository.findChannelAcountCount(account.getRootParent(),
                channleids.get("smsearch") == null ? "0" : channleids.get("smsearch"),
                channleids.get("adwords") == null ? "0" : channleids.get("adwords"),
                channleids.get("gdt") == null ? "0" : channleids.get("gdt"));

        Map dataFlag = new HashMap();

        if (count.size() >= 3) {
            if (count.get(0) != null && count.get(0).intValue() > 0) {
                dataFlag.put("smsearch", true);
            } else {
                dataFlag.put("smsearch", false);
            }

            if (count.get(1) != null && count.get(1).intValue() > 0) {
                dataFlag.put("adwords", true);
            } else {
                dataFlag.put("adwords", false);
            }

            if (count.get(2) != null && count.get(2).intValue() > 0) {
                dataFlag.put("gdt", true);
            } else {
                dataFlag.put("gdt", false);
            }

        } else {
            dataFlag.put("gdt", true);
            dataFlag.put("adwords", true);
            dataFlag.put("smsearch", true);
        }

        return dataFlag;
    }

    @Override
    public ChannelAccount getGDTTokenAndSave(HttpServletRequest request, String autnCode, String state, Account account)
            throws JSONException {
        String url = "https://api.e.qq.com/oauth/token?client_id=" + CLIENT_ID + "&client_secret=" + CLIENT_SECRET +
                "&grant_type=authorization_code&authorization_code=" + autnCode +
                "&timestamp=" + System.currentTimeMillis() / 1000 + "&nonce=" + CipherUtil.generatePassword(new Date().toString())
                + "&redirect_uri=" + Constant.tkioUrl + redirectUrl;
        String res = HttpClientUtil.doHttpGetRequest(url, "");

        Account rootParentAccount = authService.findRootParentAccount(account.getId());
        JSONObject obj = new JSONObject(res);
        JSONObject data = obj.getJSONObject("data");
        String account_id = data.getJSONObject("authorizer_info").getString("account_id");
        String qq = data.getJSONObject("authorizer_info").getString("account_uin");
        String refresh_token = data.getString("refresh_token");
        String access_token = data.getString("access_token");
        long access_token_expires = data.getLong("access_token_expires_in");
        long refresh_token_expires = data.getLong("refresh_token_expires_in");

        Date date = new Date();//取时间
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, 730);//把日期往后增加730天.
        Date pastDate = calendar.getTime();

        ChannelAccount channelAccountSave;
        ChannelAccount channelAccount = channelAccountRepository.findByNameAndThirdAccountCode(qq, UNIQUE_GDT,account_id);
        //ChannelAccount channelAccount = channelAccountRepository.getByAlias(state);
        if (null == channelAccount) {
            Channel channel = channelRepository.findByUniqueName("gdt");
            ChannelAccount ca = new ChannelAccount();
            ca.setName(qq);
            ca.setChannel(channel.getId());
            ca.setUniqueName("gdt");
            ca.setAccessToken(access_token);
            ca.setAccessTokenExpire(access_token_expires);
            ca.setRefreshToken(refresh_token);
            ca.setRefreshTokenExpire(refresh_token_expires);
            ca.setLastUpdateTime(date);
            ca.setStatus(true);
            ca.setCreateAccount(account.getId());
            ca.setCreateTime(date);
            ca.setRootAccount(rootParentAccount.getId());
            ca.setPastDate(pastDate);
            ca.setPastStatus(false);
            ca.setAlias(state);
            ca.setThirdAccountCode(account_id);
            channelAccountSave = channelAccountRepository.save(ca);
            userLogService.insertLog(account, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "" + channelAccountSave.getName(), channelAccountSave, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        } else {
            channelAccount.setAccessToken(access_token);
            channelAccount.setAccessTokenExpire(access_token_expires);
            channelAccount.setRefreshToken(refresh_token);
            channelAccount.setRefreshTokenExpire(refresh_token_expires);
            channelAccount.setLastUpdateTime(date);
            channelAccount.setModifyAccount(account.getId());
            channelAccount.setModifyTime(date);
            channelAccount.setPastDate(pastDate);
            channelAccount.setPastStatus(false);
            channelAccount.setCreateTime(new Date());
            channelAccount.setThirdAccountCode(account_id);
            channelAccount.setAlias(state);
            channelAccountSave = channelAccountRepository.save(channelAccount);
            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号" + channelAccount.getName(), channelAccount, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
        }

        return channelAccountSave;
    }

    @Override
    public int getJRTTTokenAndSave(HttpServletRequest request, String autnCode, String loginAccountid, String alias)
            throws JSONException {
        Account account = accountRepository.findOne(Long.valueOf(loginAccountid));

        //今日头条获取access_token的接口地址
        Map<String, String> params = new HashMap<>();
        params.put("appid", JRTT_APPID);
        params.put("secret", JRTT_SECRET);
        params.put("grant_type", "auth_code");
        params.put("auth_code", autnCode);

        String localhost = HttpClientUtil.doHttpPostRequest("https://ad.toutiao.com/open_api/oauth2/access_token/", "localhost", params);
        JSONObject obj = new JSONObject(localhost);
        int code = obj.getInt("code");

        if (code == 0) {

            JSONObject data = obj.getJSONObject("data");
            String access_token = data.getString("access_token");
            long access_token_expires = data.getLong("expires_in");
            String refresh_token = data.getString("refresh_token");
            long refresh_token_expires = data.getLong("refresh_token_expires_in");
            String advertiser_id = data.getString("advertiser_id");
            Date date = new Date();//取时间

            RecycleChannelAccount channelAccountSave;
            RecycleChannelAccount channelAccount = recycleChannelAccountRepository.findByRootAccountIdAndUniqueAccountAndCid(account.getRootParent(), advertiser_id, 514L);

            if (null == channelAccount) {
                RecycleChannelAccount ca = new RecycleChannelAccount();
                ca.setUserName(advertiser_id);
                //头条回调的广告主id
                ca.setChannelAccountId(advertiser_id);
                ca.setUniqueName("toutiao");
                ca.setToken(access_token);
                ca.setTokenExpire(access_token_expires);
                ca.setRefreshToken(refresh_token);
                ca.setRefreshTokenExpire(refresh_token_expires);
                //tkioing平台登录账号
                ca.setAccount(account.getId());
                ca.setRootAccount(account.getRootParent());
                ca.setStatus(1);
                ca.setChannelAlias(alias);
                ca.setInsertDate(date);
                ca.setCid(514L);
                ca.setChannelName("今日头条");
                ca.setUserName(advertiser_id);

                ca.setUniqueAccount(advertiser_id);

                ca.setUtime(date);
                ca.setChannelAlias(alias);
                ca.setValidFlag(RecycleChannelAccount.VALID_FLAG_VALID);

                channelAccountSave = recycleChannelAccountRepository.save(ca);
                userLogService.insertLog(account, LogEnumType.CREATE.getCode(), LogEnumType.CREATE.getName() + "" + channelAccountSave.getUserName(), channelAccountSave, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
            } else {
                channelAccount.setToken(access_token);
                channelAccount.setTokenExpire(access_token_expires);
                channelAccount.setRefreshToken(refresh_token);
                channelAccount.setRefreshTokenExpire(refresh_token_expires);
                channelAccount.setModifyDate(date);

                channelAccount.setUtime(date);
                channelAccount.setChannelAlias(alias);
                channelAccount.setValidFlag(RecycleChannelAccount.VALID_FLAG_VALID);

                channelAccountSave = recycleChannelAccountRepository.save(channelAccount);
                userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + "渠道账号" + channelAccountSave.getUserName(), channelAccountSave, null, OperateObjectEnumType.CHANNEL_ACCOUNT.getCode(), request);
            }

        }

        return code;
    }

    @Override
    public ChannelAccount authorize(ChannelAccount resource, long account) {
        //ChannelAccount channelAccount = channelAccountRepository.findByName(resource.getName(), UNIQUE_GDT);
        ChannelAccount channelAccount = channelAccountRepository.getByAlias(resource.getAlias());
        channelAccount.setModifyTime(new Date());
        channelAccount.setModifyAccount(account);
        //channelAccount.setThirdAccountCode(resource.getThirdAccountCode());
        //channelAccount.setUserActionSetId(resource.getUserActionSetId());
        ChannelAccount save = channelAccountRepository.save(channelAccount);
        if (null != save && 5305L != save.getChannel()) {
            List<ChannelAccountLinkApp> linkApps = channnelAccountLinkAppRepository.findByLinkUserId(save.getId());
            if (ValidateUtil.isValid(linkApps)) {
                List<ChannelAccountLinkApp> validList = new ArrayList<>();
                for (ChannelAccountLinkApp linkApp : linkApps) {
                    linkApp.setAccountId(save.getThirdAccountCode());
                    validList.add(linkApp);
                }
                try {
                    channnelAccountLinkAppService.reValid(validList);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        return save;
    }

    @Override
    public void deleteAuthorize(String qq) {
        ChannelAccount channelAccount = channelAccountRepository.findByName(qq, UNIQUE_GDT);
        channelAccountRepository.delete(channelAccount);
        return;
    }

    @Override
    public ChannelAccount alias(ChannelAccount resource, long account) {
        ChannelAccount channelAccount = channelAccountRepository.findOne(resource.getId());
        channelAccount.setModifyTime(new Date());
        channelAccount.setModifyAccount(account);
        channelAccount.setAlias(resource.getAlias());
        channelAccount.setRebate(resource.getRebate());
        ChannelAccount save = channelAccountRepository.save(channelAccount);

        return save;
    }

    @Override
    public boolean validQQ(String qq, Long account) {
        ChannelAccount channelAccount = channelAccountRepository.findByName(qq, UNIQUE_GDT);
        return null != channelAccount;
    }

    @Override
    public boolean validAccountId(String accountId, Long account) {
        ChannelAccount channelAccount = channelAccountRepository.findByThirdAccountCode(accountId);
        return null != channelAccount;
    }

    @Override
    public List<ChannelAccount> findValidByUniqueName(String uniqueName) {
        return channelAccountRepository.findValidByUniqueName(uniqueName);
    }

    @Override
    public ChannelAccount findById(Long id) {
        return channelAccountRepository.findOne(id);
    }

    @Override
    public Optional<List<ChannelAccount>> findByStatusTrue() {
//        return Optional.ofNullable(channelAccountRepository.findByStatusTrueAndPastStatusFalse());
        //修改为只校验 GDT 账号
        return Optional.ofNullable(channelAccountRepository.findgdtStatusTrueAndNotPast());
    }

    @Override
    public List<ChannelAccount> listByAccounts(List<Long> accounts) {
        return channelAccountRepository.listByAccounts(accounts);
    }

    @Override
    public boolean checkAlias(String alias) {

        return channelAccountRepository.checkAlias(alias) > 0;
    }

    /**
     * 绑定产品时候的第一步 - 查询已经绑定的商品
     * create by zhangyao 2018-11-23
     */
    @Override
    public List<ChannelAccount> findLinkedApp(Account loginAccount, long cid) {
        List<ChannelAccount> channelAccounts = this.find(loginAccount, cid);
        List<ChannelAccount> newChannelAccounts = new ArrayList<>();
        BlacklistApplication blacklistApplication = blacklistApplicationRepository.findByParentAndArouse(loginAccount.getRootParent());

        for (ChannelAccount channelAccount : channelAccounts) {
            if (channelAccount.getHaveLinkApp()) {
                List<App> linkedApps = null;
                if (loginAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
                    //子应用管理员 已授权绑定列表下的 app
                    linkedApps = this.appRepository.listAuthLinkApps(channelAccount.getId(), loginAccount.getId());
                } else {
                    linkedApps = this.appRepository.findLinkedAppName(channelAccount.getId());
                }

                for (App linkedApp : linkedApps) {
                    channelAccount.setAppName(linkedApp.getName());
                    channelAccount.setAppKey(linkedApp.getAppkey());
                    GdtPlan gdtplan = gdtPlanRepository.getGdtPlanByAppkey(linkedApp.getAppkey());

                    channelAccount.setFeedbackurl(gdtplan == null ? "暂无" : gdtplan.getFeedbackURL());
                    channelAccount.setPlan(gdtplan == null ? 0 : gdtplan.getPlan());
                    if (channelAccount.getPlan() == 1) {
                        App app = appRepository.findByAppkey(gdtplan.getAppkey());
                        if (!StringUtil.isEmpty(gdtplan.getUrlScheme())) {
                            channelAccount.setOpenDeepLink(1);
                            channelAccount.setUrlScheme(gdtplan.getUrlScheme());
                        } else {
                            channelAccount.setOpenDeepLink(0);
                        }
                        channelAccount.setOpenSubscribes(gdtplan.getSubscribes()==null?false:gdtplan.getSubscribes());

                        //控制广点通已绑定app的再营销和预约app的开关按钮是否展示：0全部不展示，1展示再营销，2展示预约
                        if(app.getWhetherOnline()!=null&&!app.getWhetherOnline()){
                            channelAccount.setType(2);
                        }else{
                            if(blacklistApplication!=null&&blacklistApplication.getStatus()==2){
                                channelAccount.setType(1);
                            }else{
                                channelAccount.setType(0);
                            }
                        }
                    }else{
                        channelAccount.setType(0);
                    }
                    newChannelAccounts.add(new ChannelAccount(channelAccount));
                }
            }
        }

        return newChannelAccounts;
    }

    @Override
    public void refreshAcessToken(Long id, String accessToken, Long expireTime, String refreshToken, Long refreshTokenExpireTime) {
        ChannelAccount channelAccount = this.channelAccountRepository.getOne(id);
        String old_accessToken = channelAccount.getAccessToken();
        channelAccount.setAccessToken(accessToken);
        channelAccount.setAccessTokenExpire(expireTime);
        channelAccount.setRefreshToken(refreshToken);
        channelAccount.setRefreshTokenExpire(refreshTokenExpireTime);
        channelAccount.setLastUpdateTime(new Date());
        channelAccount.setPastStatus(false);
        ChannelAccount account = this.channelAccountRepository.save(channelAccount);
        List<App> linkedApps = this.appRepository.findLinkedAppName(account.getId());

        //禅道号459  要求广点通在2020-01-16后授权或重新授权的入ddb,因为之前授权的广点通token都没有获取消耗的权限
        if (DateUtil.compareDate(channelAccount.getCreateTime(), DateUtil.parseDate("2020-01-16")) == 1) {
            List<ChannelAccountLinkApp> byLinkUserId = channnelAccountLinkAppRepository.findByLinkUserId(account.getId());
            for (ChannelAccountLinkApp channelAccountLinkApp : byLinkUserId) {
                account.setThirdAccountCode(channelAccountLinkApp.getAccountId());
                NoSQLDBUtil.putMarketingapi(account, linkedApps, false, apiMap.get(5305L));
            }
        }

        if (!StringUtils.equals(account.getAccessToken(), old_accessToken)) {
            logger.info("GDT_accessToken刷新成功 旧:{} 新:{} 时间:{}, name_qq:{}",
                    old_accessToken,
                    account.getAccessToken(),
                    new DateTime().toString("yyyy-MM-dd HH:mm:ss"),
                    channelAccount.getName());
        }

        List<RecycleChannelAccount> recycleChannelAccounts = recycleChannelAccountRepository.findByAccountAndCidAndUserName(channelAccount.getCreateAccount(), channelAccount.getChannel(), channelAccount.getName());
        if (recycleChannelAccounts != null && recycleChannelAccounts.size() > 0) {
            for (RecycleChannelAccount recycleChannelAccount : recycleChannelAccounts) {
                recycleChannelAccount.setToken(accessToken);
                recycleChannelAccount.setTokenExpire(expireTime);
                recycleChannelAccount.setRefreshToken(refreshToken);
                recycleChannelAccount.setRefreshTokenExpire(refreshTokenExpireTime);
                recycleChannelAccount.setModifyDate(new Date());
                recycleChannelAccount.setUtime(new Date());
                recycleChannelAccount.setUniqueAccount(channelAccount.getThirdAccountCode());
                try {
                    recycleChannelAccountRepository.save(recycleChannelAccount);
                    logger.info(recycleChannelAccount.getChannelName() + "同步token数据到recycleChannelAccount：ChannelAccountId：" + recycleChannelAccount.getChannelAccountId() + "ChannelName:" + recycleChannelAccount.getChannelName());
                } catch (Exception e) {
                    logger.error("同步token数据到recycleChannelAccount 失败  message:", e);

                }
            }
        }
    }

    @Override
    @Transactional
    public boolean updatePlan(Account account, int plan, String cip, String shortUrl) {
        GdtPlan gdtPlan = this.gdtPlanRepository.findByRootAccountAndUrl(account.getRootParent(), shortUrl);
        if (gdtPlan == null || gdtPlan.getPlan() == plan) return false;
        App app = this.appRepository.findByAppkey(gdtPlan.getAppkey());
        boolean b = this.gdtPlanRepository.updatePlanById(plan, gdtPlan.getId()) > 0;
        if (b) {
            GdtPlanLog gdtPlanLog = new GdtPlanLog();
            gdtPlanLog.setCip(cip);
            gdtPlanLog.setPlan(plan);
            gdtPlanLog.setRootParent(account.getRootParent().intValue());
            gdtPlanLog.setUpdateAccount(account.getEmail());
            gdtPlanLog.setUpdateTime(new Date());
            this.gdtPlanLogRepository.save(gdtPlanLog);
            if (plan == 1 && "true".equals(Constant.mailProd)) {
                String url = "https://oapi.dingtalk.com/robot/send?access_token=aba28788ae0d6dca9936b64919c13e6ab95e3e04a0af1b63fadb2124a947f66b";
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
                String content = account.getName() + "用户的" + app.getName() + "产品于" + sdf.format(new Date()) + "选择广点通方案一\n"
                        + account.getName() + ":" + account.getEmail() + "\n" + app.getName() + ":" + app.getAppkey();
                DingDingSendMsgUtil dingDing = new DingDingSendMsgUtil();
                dingDing.sendMsg(url, content);
            }
        }

        return b;
    }

    @Override
    public int checkPlan(Account loginAccount) {
        GdtPlan gdtPlan = this.gdtPlanRepository.getGdtlanByRootAccount(loginAccount.getRootParent());
        //默认是广点通方案2
        return gdtPlan == null ? 2 : gdtPlan.getPlan();
    }

    @Override
    public Map<String, Object> checkToutiaoNum(Account loginAccount) {
        Account account = accountRepository.findOne(loginAccount.getId());
        Map<String, Object> rs = new HashMap<>();
        List<ChannelAccount> validByToutiao = channelAccountRepository.findValidByToutiao(account.getRootParent());
        ToutiaoNum toutiaoNum = toutiaoNumRepository.findOne(account.getRootParent());
        int toutiaoCount = toutiaoNum == null ? 10 : toutiaoNum.getCount();
        if (validByToutiao.size() >= toutiaoCount) {
            rs.put("msg", "今日头条投放账号可绑定数量超额,可以去office进行数量设置");
            rs.put("status", false);
            rs.put("toutiaoCount", toutiaoCount);
            rs.put("channelAccountNum", validByToutiao.size());
            return rs;
        }
        rs.put("status", true);
        return rs;
    }

    @Override
    public Object getToutiaoAccessToken(Long appId, String secret, String grantType, String authCode, String alias, Account loginAccount) {

        //code 0.成功  2.失败
        Map<String, Object> rs = new HashMap<>();
        Account account = accountRepository.findOne(loginAccount.getId());

        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/oauth2/access_token/";

        // 请求参数
        Map<String, Object> data = new HashMap() {
            {
                put("app_id", appId);
                put("secret", secret);
                put("grant_type", grantType);
                put("auth_code", authCode);
            }
        };

        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                if (jsonObject != null && jsonObject.containsKey("data") && jsonObject.getInteger("code") == 0) {
                    Channel channel = channelRepository.findOne(514L);
                    com.alibaba.fastjson.JSONObject dataResult = jsonObject.getJSONObject("data");
                    if (dataResult == null) return "头条返回data为空:" + jsonObject.toJSONString();
                    //JSONArray advertiser_ids = dataResult.getJSONArray("advertiser_ids");
                    HashMap<String, String> resultMap = getToutiaoName(appId, secret, dataResult.getString("access_token"));
                    List<App> apps = appRepository.findValidAppByRootParent(account.getRootParent());
                    String sgin = UUID.randomUUID().toString().replace("-", "");

                    List<ChannelAccount> validByToutiao = channelAccountRepository.findValidByToutiao(account.getRootParent());
                    Map<String, ChannelAccount> collect = validByToutiao.stream().collect(Collectors.toMap(ChannelAccount::getThirdAccountCode, Function.identity(), (v1, v2) -> v1));
                    //channelAccountRepository.deleteToutiaoByAccount(account.getRootParent());

                    for (String advertiser_id : resultMap.keySet()) {
                        ChannelAccount channelAccount = new ChannelAccount();
                        if(!CollectionUtils.isEmpty(collect)&&collect.containsKey(advertiser_id)){
                            channelAccount = collect.get(advertiser_id);
                        }
                        channelAccount.setAlias(alias);
                        channelAccount.setChannel(channel.getId());
                        channelAccount.setAccessToken(dataResult.getString("access_token"));
                        channelAccount.setAccessTokenExpire(dataResult.getLong("expires_in"));
                        channelAccount.setRefreshToken(dataResult.getString("refresh_token"));
                        channelAccount.setRefreshTokenExpire(dataResult.getLong("refresh_token_expires_in"));
                        channelAccount.setCreateAccount(account.getId());
                        channelAccount.setName(resultMap.get(advertiser_id));
                        channelAccount.setThirdAccountCode(advertiser_id);
                        channelAccount.setUniqueName(channel.getUniqueName());
                        channelAccount.setRootAccount(account.getRootParent());
                        channelAccount.setStatus(true);
                        channelAccount.setModifyTime(new Date());
                        channelAccount.setSgin(sgin);
                        channelAccount.setRole(1);
                        ChannelAccount save = channelAccountRepository.save(channelAccount);
                        NoSQLDBUtil.putMarketingapi(save,apps,true, "toutiao_");
                    }
                    rs.put("msg", "绑定成功");
                    rs.put("status", true);
                    rs.put("code", 0);
                }
                return rs;
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        rs.put("msg", "绑定失败");
        rs.put("status", false);
        rs.put("code", 2);
        return rs;
    }
    @Override
    public  HashMap<String, String> getToutiaoName(Long appId, String secret, String access_token) {
        HashMap<String, String> resultMap = null;
        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/oauth2/advertiser/get/?app_id="+appId+"&secret="+secret+"&access_token="+access_token;


        // 构造请求
        HttpGet httpEntity = new HttpGet(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        /*HashMap<Long, String> nameMap = new HashMap<>();
        HashMap<Long, Integer> roleMap = new HashMap<>();*/
        try {
            client = HttpClientBuilder.create().build();
            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                com.alibaba.fastjson.JSONArray dataResult = jsonObject.getJSONObject("data").getJSONArray("list");
                resultMap = new HashMap<>();
                if(!CollectionUtils.isEmpty(dataResult)){
                    for (Object o : dataResult) {
                        com.alibaba.fastjson.JSONObject jo = com.alibaba.fastjson.JSONObject.parseObject(o.toString());
                        Long advertiser_id = jo.getLong("advertiser_id");
                        //nameMap.put(advertiser_id,jo.getString("advertiser_name"));
                        Integer advertiser_role = jo.getInteger("advertiser_role");
                        //roleMap.put(advertiser_id,advertiser_role);
                        //2为管理员需要再次获取子账号
                        if(2==advertiser_role){
                            putToutiaoAdvertiserByManager(access_token, advertiser_id ,resultMap);
                        }
                    }
                }
                /*resultMap.put("nameMap",nameMap);
                resultMap.put("roleMap",roleMap);*/
                return resultMap;
            }

        } catch (Exception e) {
            logger.error("获取头条advertiser异常：", e);
            resultMap = new HashMap<>();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                if (client != null) {
                    client.close();
                }
            } catch (IOException e) {
                logger.error("", e);
            }
        }
        return resultMap;
    }

    public void putToutiaoAdvertiserByManager(String access_token ,Long advertiser_id ,HashMap<String, String> resultMap) {
        // 请求地址
        String open_api_url_prefix = "https://ad.oceanengine.com/open_api/2/";
        String uri = "majordomo/advertiser/select/";

        // 请求参数
        Map data = new HashMap(){
            {
                put("advertiser_id", advertiser_id);
            }
        };

        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };

        httpEntity.setHeader("Access-Token",access_token);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(open_api_url_prefix + uri));
            httpEntity.setEntity(new StringEntity(com.alibaba.fastjson.JSONObject.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader  = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                com.alibaba.fastjson.JSONObject jsonObject = com.alibaba.fastjson.JSONObject.parseObject(result.toString());
                com.alibaba.fastjson.JSONArray dataResult = jsonObject.getJSONObject("data").getJSONArray("list");
                for (Object o : dataResult) {
                    com.alibaba.fastjson.JSONObject jo = com.alibaba.fastjson.JSONObject.parseObject(o.toString());
                    resultMap.put(jo.getString("advertiser_id"),jo.getString("advertiser_name"));
                    //roleMap.put(jo.getLong("advertiser_id"),1);//默认获取的都是子账号
                }
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public ChannelAccount getByAlias(String alias) {
        return channelAccountRepository.getByAlias(alias);
    }

    public static void main(String[] args) {
        /*String s = getToutiaoAdvertiserByManager("eb2fb8a8213f4781596ae25cd6fa66f01c51cfa6", 2981528437277035L).toJSONString();
        System.out.println("s = " + s);*/
        //System.out.println(getToutiaoName(1653688605754372L,"1e858757bdd8beaf7ca68579a8ab4d2a7b3296a3","80c8be764bad176cc9b13f2a9f939f919522793c"));
    }
}
