package com.reyun.service.impl;

import com.google.common.base.Predicate;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AuthService;
import com.reyun.service.CampaignService;
import com.reyun.service.ChannelService;
import com.reyun.util.SendCommonPostMail;
import com.reyun.util.ValidateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
@Transactional
public class ChannelServiceImpl implements ChannelService {
    protected Logger logger = LoggerFactory.getLogger(ChannelServiceImpl.class);

    @Autowired
    private ChannelRepository repository;
    @Autowired
    private AuthService authService;
    @Autowired
    private DataAuthRepository dateAuthRepository;
    @Autowired
    private CampaignRepository campaignRepository;
    @Autowired
    private CampaignService campaignService;
    @Autowired
    private CallbackCustomizeRepository callbackCustomizeRepository;
    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private RoleAuthRepository roleAuthRepository;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private LogoDicRepository logoDicRepository;
    @Autowired
    private ChannelControlRepository channelControlRepository;

    /**
     * 保存渠道
     */
    @Override
    public Channel save(Channel resource) {
        resource.setCreateTime(new Date());
        Channel rtnChannel = this.repository.save(resource);
        return rtnChannel;
    }

    /**
     * 修改渠道
     */
    @Override
    public Channel update(Channel resource, Long accountid) {

        Channel one = this.findoneById(resource.getId());

        //验证修改前后名字,分类.只要出现不一样,就需要同步,(去掉url)
        boolean isNeedSyncSurl = !resource.getName().equals(one.getName()) || !resource.getCategory().equals(one.getCategory());

        one.setName(resource.getName());
        one.setCategory(resource.getCategory());
        one.setUrl(resource.getUrl());
        one.setModifyAccount(accountid);
        one.setModifyTime(new Date());
        Channel rtnChannel = this.repository.save(one);

        //修改后需要同步短链
        if (isNeedSyncSurl && null != rtnChannel) {
            campaignService.updateCampaignAwsStatusByChannel(rtnChannel.getId());
        }

        return rtnChannel;
    }

    @Override
    public Channel findoneById(Long id) {
        Channel channel = this.repository.findById(id);
        return channel;
    }

    /**
     * 渠道关联授权
     */
    @Override
    public void syncChannelAuth(String holderids, Long appid, Long channelid,
                                Long currentaccountid) {

        // 存放关联账号的list容器
        List<Long> holderidsList = this.getHolderidsLiast(holderids, appid,
                channelid, currentaccountid);

        // 批量插入关联商务
        if (holderidsList.size() > 0) {
            // authRepository.batchInsert(holderidsList,appid,channelid);
        }

    }

    /**
     * 拼接权限相关的list
     *
     * @param holderids
     * @param appid
     * @param channelid
     * @param currentaccountid
     * @return
     */
    private List<Long> getHolderidsLiast(String holderids, Long appid,
                                         Long channelid, Long currentaccountid) {
        // 存放关联账号的list容器
        List<Long> holderidsList = Lists.newArrayList();
        // 将关联账号分解放入迭代器
        holderids = Strings.isNullOrEmpty(holderids) ? "" : holderids; // 关联帐号ids
        holderids += "," + currentaccountid;
        Iterable<String> linkaccounts = Splitter.on(",").trimResults()
                .omitEmptyStrings().split(holderids);
        Iterator<String> iterator = linkaccounts.iterator();
        // 循环迭代器设置ChannelPermission信息
        while (iterator.hasNext()) {
            String linkactid = iterator.next();
            holderidsList.add(Long.parseLong(linkactid));
        }
        return holderidsList;
    }

    @Override
    public void delChannelAuth(Long channelid) {

        // this.authRepository.delete(channelid);

    }

    @Override
    public void delete(Channel channel, Long accountid) {
        channel.setDelFlag(true);

        // 找出该渠道下的所有推广活动
        List<Campaign> campaignList = this.campaignRepository
                .findByChannel(channel.getId());
        // 对渠道下的推广活动进行假删除以及camcamgroupmap表真删除
        if (ValidateUtil.isValid(campaignList)) {
            for (Campaign cam : campaignList) {
                this.campaignService.delete(cam.getApp(), cam.getId(),
                        cam.getAccount());
            }
        }

        // 对与渠道关联的CallbackCustomize进行假删除
        List<CallbackCustomize> dataCallback = this.callbackCustomizeRepository
                .listByChannel(channel.getId());

        for (CallbackCustomize call : dataCallback) {
            call.setDelFlag(false);
            call.setModifyAccount(accountid.toString());
            call.setModifyTime(new Date());
        }
        this.callbackCustomizeRepository.save(dataCallback);

        // 删除渠道对应的dataauth表中的信息
        List<DataAuth> dataAuthList = this.dateAuthRepository
                .listDataAuthByChannel(channel.getId());
        this.dateAuthRepository.delete(dataAuthList);

        this.repository.save(channel);
    }

    @Override
    public Channel enable(long id, Long accountid) {
        Channel one = this.repository.findOne(id);
        one.setModifyTime(new Date());
        one.setModifyAccount(accountid);
        one.setDelFlag(false);

        return this.repository.save(one);
    }

    @Override
    public void updateChannelAuth(String holderids, Long appid, Long channelid,
                                  Long currentaccountid) {

        // 存放关联账号的list容器
        List<Long> holderidsList = this.getHolderidsLiast(holderids, appid,
                channelid, currentaccountid);

        // 批量插入关联商务
        if (holderidsList.size() > 0) {
            // this.authRepository.update(holderidsList,appid,channelid)
        }

    }

    // app下的所有渠道信息
    @Override
    public List<Channel> listChannelByAccount(Long accountid) {
        return repository.findChannelsByAccount(accountid);
    }

    /**
     * 自定义渠道
     */
    @Override
    public List<Channel> listOwnChannelByCreateAccount(Long accountId) {

        List<Channel> resultList = new ArrayList<>();
        Account oneAccount = accountRepository.findOne(accountId);

        //母账号,管理员,子应用管理员,拥有所有的渠道权限,合作渠道+自创建渠道
        if (oneAccount.getIsSuperUser() || oneAccount.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || oneAccount.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            //母账号ID

            //获取母账号下所有子账号的自定义渠道
            //List<Account> sonList = accountRepository.findAccountByParent(rootAccount);
            List<Account> sonList;
            if (oneAccount.getIsSuperUser()) {
                //主账号 拥有所有子账号的自定义渠道
                sonList = accountRepository.findAccountByRootParentNoStatus(oneAccount.getId());
            } else {
                //管理员,子应用管理员 用户自身父账号下子账号的所有自定义渠道
                sonList = accountRepository.findAccountByParent(oneAccount.getParent());
            }

            for (Account a : sonList) {
                List<Channel> list = this.repository.listChannelByAccount(a.getId());
                if (ValidateUtil.isValid(list)) {
                    resultList.addAll(list);
                }
            }
            Long rootAccount = oneAccount.getIsSuperUser() ? accountId : oneAccount.getParent();
            //添加母账号的自定义渠道
            List<Channel> parentList = this.repository.listChannelByAccount(rootAccount);
            if (ValidateUtil.isValid(parentList)) {
                resultList.addAll(parentList);
            }

        } else {
            //自定义角色和渠道账号
            resultList = this.repository.listChannelByAccount(accountId);
            resultList.addAll(this.repository.findByAccountWithPermission(accountId));
        }

        Map<Long, Channel> map = new HashMap<>();
        for (Channel ch : resultList) {
            map.put(ch.getId(), ch);
        }
        resultList.clear();
        resultList.addAll(map.values());

        return resultList;
    }

    @Override
    public List<Channel> listUnOwnChannel(Account loginAccount, String delflg) {
        List<Channel> resultList = new ArrayList<>();
        List<Channel> result = new ArrayList<>();

        if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(loginAccount.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(loginAccount.getRoleCategory())) {
            resultList = this.repository.findByTypeIsNotNullOrderByOrderindexAsc();
            //筛选改渠道是否是更细粒度的单独用户控制
            resultList = filterChannelControl(loginAccount.getRootParent(), resultList);
        } else {
            resultList = repository.listAllAuthorizedChannelByCoop(loginAccount.getId());
        }

        boolean hideChannel = "1".equals(delflg) ? true : false;

        if (!loginAccount.getRootParent().equals(loginAccount.getId())) {
            loginAccount = accountRepository.findOne(loginAccount.getRootParent());
        }

        Map<String, String> channlenodeurl = new HashMap();
        if (loginAccount.getRegSource() != 0) {
            channlenodeurl = this.getChannelNodeUrl(loginAccount);
        }

        if (ValidateUtil.isValid(resultList)) {
            for (Channel ch : resultList) {

                if (channlenodeurl.size() > 0) {
                    if (channlenodeurl.get(ch.getId() + "") != null) {
                        ch.setNodeUrl(channlenodeurl.get(ch.getId() + ""));
                    }
                }

                if (hideChannel) {
                    if ((null == ch.getNotShow() || !ch.getNotShow()) && (ch.getDelFlag() == null || !ch.getDelFlag())) {
                        result.add(ch);
                    }
                } else {
                    if (null == ch.getNotShow() || !ch.getNotShow()) {
                        result.add(ch);
                    }
                }
            }
        }
        return result;
    }


    @Override
    public Channel valid(String name) {
        return this.repository.findByName(name);
    }

    @Override
    public List<Channel> listCampaignChannel(Long appId, Long accountId) {

        final List<Long> authChannelList = authService.getAuthChannelList(accountId);

        List<Channel> channels = this.repository.listAllCampaignChannel(appId);

        return Lists.newArrayList(Iterables.filter(channels, new Predicate<Channel>() {
            @Override
            public boolean apply(Channel channel) {
                return authChannelList.contains(channel.getId());
            }
        }));
    }

    @Override
    public List<Channel> listCampaignChannelAndDisable(Long appId, Long accountId) {

        Account account = accountRepository.findOne(accountId);
        RoleAuth roleAuth = roleAuthRepository.findOne(account.getRoleCategory());

        List<Channel> resultList = new ArrayList<>();

        List<Channel> result = new ArrayList<>();

        //母账号,管理员,子应用管理员。
        if (account.getIsSuperUser() || roleAuth.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                || roleAuth.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

            resultList = findCoopAndCreateChannelByApp(account, appId);

        } else {

            resultList = authService.getAuthChannelListByApp(accountId, appId);
        }

        if (ValidateUtil.isValid(resultList)) {
            for (Channel ch : resultList) {
                if (null == ch.getNotShow() || !ch.getNotShow()) {
                    result.add(ch);
                } else {
                    List<Campaign> campaignList = campaignRepository.findByAppAndChannel(appId, ch.getId());
                    if (null != campaignList && campaignList.size() > 0) {
                        result.add(ch);
                    }
                }
            }
        }
        return result;
    }

    /**
     * 根据APP类型查询合作渠道和自定义渠道
     * created by sunhao 20170913
     */
    private List<Channel> findCoopAndCreateChannelByApp(Account account, Long appId) {

        List<Channel> channelList = new ArrayList<>();

        App app = appRepository.findOne(appId);

        //合作渠道
        List<Channel> coopByAppType = repository.findCoopByAppType(app.getPlatform());
        coopByAppType = filterChannelControl(account.getRootParent(), coopByAppType);
        channelList.addAll(coopByAppType);

        //自定义渠道
        channelList.addAll(repository.findCreatedChannelByAppType(account.getRootParent(), app.getPlatform()));

        return channelList;
    }

    @Override
    public List<Channel> listChannelByIds(List<Long> ids) {
        return this.repository.findById(ids);
    }

    @Override
    public List<Channel> listChannelByIds(Set<Long> idSet) {
        return this.repository.findByIdSet(idSet);
    }

    @Override
    public Map<String, String> findChannelMapWithDefault(Long appId, Long accountId) {

        List<Channel> channelList = this.listCampaignChannel(appId, accountId);
        channelList = (null != channelList) ? channelList : new ArrayList<Channel>();

        Channel channel = new Channel();
        channel.setId(-1L);
        channel.setName("自然量");

        channelList.add(channel);
        Map<String, String> result = new HashMap<>();

        for (Channel ch : channelList) {
            result.put(ch.getId().toString(), ch.getName());
        }

        return result;
    }

    /**
     * 自定义渠道和合作渠道 合集
     */
    @Override
    public List<Channel> findCoopAndOwnChannel(Long account, String accountAll) {
        return findCoopAndOwnChannel(account, false, accountAll);
    }


    public Map getChannelNodeUrl(Account account) {

        List<LogoDic> logoDics = logoDicRepository.getChannelNodeUrl(account.getRegSource() + "");

        Map data = new HashMap();

        for (LogoDic dic : logoDics) {
            data.put(dic.getDicKey(), dic.getDicContent());
        }
        return data;
    }


    public List<Channel> findCoopAndOwnChannel(Long account, boolean delflag, String accountAll) {
        Account accountInfo = accountRepository.findOne(account);
        List<Channel> result = new ArrayList<>();
        result.addAll(this.repository.findByTypeIsNotNull());
        if (delflag) {
            result.addAll(this.findAllChannel(accountInfo.getIsSuperUser() ? account : accountInfo.getRootParent(), delflag, accountAll));
        } else {

            result.addAll(this.findAllChannel(accountInfo.getIsSuperUser() ? account : accountInfo.getRootParent(), accountAll));
        }

        Map<Long, Channel> map = new HashMap<>();
        for (Channel ch : result) {

            map.put(ch.getId(), ch);


        }
        result.clear();
        result.addAll(map.values());
        return result;
    }

    @Override
    public int deleteValid(Long cid) {
        int num = 0;

        List<Campaign> campaignList = this.campaignRepository.findAllByChannel(cid);
        if (null != campaignList && campaignList.size() > 0) {
            num = campaignList.size();
        }

        return num;
    }


    private List<Channel> findAllChannel(Long accountId, String accountAll) {

        return findAllChannel(accountId, false, accountAll);

    }


    private List<Channel> findAllChannel(Long accountId, boolean delflag, String accountAll) {

        Account oneAccount = accountRepository.findOne(accountId);
        Long id = accountId;

        // 判断是否是母账号
        if (!oneAccount.getIsSuperUser() || oneAccount.getIsChannelPerson()) {
            id = oneAccount.getParent();
        }

        List<Channel> resultList = new ArrayList<>();

        //获取母账号下所有子账号的自定义渠道
        List<Account> sonList;

        if (StringUtils.endsWithIgnoreCase(accountAll, "1")) {
            sonList = accountRepository.findAccountByParentWithOutStatus(id);
        } else {
            sonList = accountRepository.findAccountByParent(id);
        }

        for (Account a : sonList) {

            List<Channel> list;
            if (delflag) {
                list = this.repository.listOwnChannelByAccountDeled(a.getId());
            } else {
                list = this.repository.listOwnChannelByAccount(a.getId());
            }

            if (ValidateUtil.isValid(list)) {
                resultList.addAll(list);
            }
        }


        List<Channel> parentList;
        if (delflag) {
            parentList = this.repository.listOwnChannelByAccountDeled(id);
        } else {
            //添加母账号的自定义渠道
            parentList = this.repository.listOwnChannelByAccount(id);
        }

        if (ValidateUtil.isValid(parentList)) {
            resultList.addAll(parentList);
        }


        return resultList;
    }

    public List<Channel> filterChannelControl(Long rootParent, List<Channel> channels) {
        List<Channel> parentListFilter = new ArrayList<>();
        //如何判断该渠道是否对各个用户进行控制展示：只要channel_control里面有该cid，则代表该渠道del状态为false开启状态
        Long[] allCIDs = channelControlRepository.findAllDistinctChannelId();
        Long[] accountCIDs = channelControlRepository.findByAccountId(rootParent);
        for (Channel channel : channels) {
            if (Arrays.asList(allCIDs).contains(channel.getId())) {//当用户查出的常规渠道为已开启状态，则看该渠道是否是用户维度控制
                if (Arrays.asList(accountCIDs).contains(channel.getId())) {//查看是否再控制列表中
                    parentListFilter.add(channel);
                }
            } else {
                parentListFilter.add(channel);
            }
        }
        return parentListFilter;
    }

    @Override
    public List<Channel> findByKeyword(Account loginAccount, Long app, String keyword) {
        List<Channel> result = new ArrayList<>();

        List<Long> channelList = authService.getAuthChannelList(loginAccount.getId());

        if (!CollectionUtils.isEmpty(channelList)) {
            result = repository.findLikeName(app, keyword);

            if (!CollectionUtils.isEmpty(result)) {

                Iterator<Channel> channelIterator = result.iterator();

                while (channelIterator.hasNext()) {

                    if (!channelList.contains(channelIterator.next().getId())) {
                        channelIterator.remove();
                    }
                }
            }
        }

        return this.repository.findLikeName(app, keyword);
    }

    @Override
    public List<Channel> listChannel4CreateCampaign(Long accountId, Long appId) {

        List<Channel> resultList;

        Account loginAccount = accountRepository.findOne(accountId);

        if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(loginAccount.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(loginAccount.getRoleCategory())) {

            resultList = findCoopAndOwnChannel(accountId, null);

        } else {
            //查询所有全选授权的渠道
            resultList = repository.listTotalAuthorizedChannel(accountId, appId);
        }

        return resultList;
    }

    /**
     * @param account
     * @param appId
     * @param lanType    语种类型（zh 中文  en 英文）
     * @param showDel
     * @param accountAll
     * @param filter
     * @return
     */
    @Override
    public List<Channel> listChannel4CreateCampaignShow(Account account, Long appId, String lanType, String showDel, String accountAll, Boolean filter) {
        List<Channel> resultList;
        List<Channel> result = new ArrayList<>();

        Long accountId = account.getId();

        Account loginAccount = accountRepository.findOne(accountId);

       /* if ("demo@reyun.com".equals(loginAccount.getEmail())) {
            //demo 账号查询主账号能看到的渠道
            loginAccount.setIsSuperUser(true);
        }*/

        // 不传参为false,(不传参为改动之前的业务流程)
        boolean show_del_channel = "1".equals(showDel) ? true : false;

        if (loginAccount.getIsSuperUser() || "demo@reyun.com".equals(loginAccount.getEmail())
                || RoleEnumType.MANAGER.getKey().equals(loginAccount.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(loginAccount.getRoleCategory())) {

            if (show_del_channel) {
                resultList = findCoopAndOwnChannel(accountId, accountAll);
            } else {
                resultList = findCoopAndOwnChannel(accountId, true, accountAll);
            }
            if (filter != null && filter) {
                Long rootId = loginAccount.getIsSuperUser() ? loginAccount.getId() : loginAccount.getRootParent();
                resultList = filterChannelControl(rootId, resultList);
            }

        } else {
            //查询所有全选授权的渠道
            resultList = repository.listTotalAuthorizedChannel(accountId, appId);
        }

        //默认显示已隐藏的渠道
        boolean delflag = true;

        if (show_del_channel) {
            //不显示已隐藏的需要
            delflag = false;
        }
        if (!account.getRootParent().equals(account.getId())) {
            account = accountRepository.findOne(loginAccount.getRootParent());
        }

        Map<String, String> channlenodeurl = new HashMap();
        if (account.getRegSource() != 0) {
            channlenodeurl = this.getChannelNodeUrl(account);
        }


        if (ValidateUtil.isValid(resultList)) {
            for (Channel ch : resultList) {

                if (channlenodeurl.size() > 0) {
                    if (channlenodeurl.get(ch.getId() + "") != null) {
                        ch.setNodeUrl(channlenodeurl.get(ch.getId() + ""));
                    }
                }

                if (delflag) {

                    //显示 隐藏的渠道
                    if (ch != null && (null == ch.getNotShow() || !ch.getNotShow())
//                        && (null == ch.getDelFlag() || !ch.getDelFlag())
                            && (null == ch.getNotCover() || !ch.getNotCover())) {
                        //渠道名转化语种

                        ch = ch.toTransEng(ch, lanType);
                        //end
                        result.add(ch);
                    }
                } else {
                    //不显示 隐藏的渠道
                    if (ch != null && (null == ch.getNotShow() || !ch.getNotShow())
                            && (null == ch.getDelFlag() || !ch.getDelFlag())
                            && (null == ch.getNotCover() || !ch.getNotCover())) {
                        //渠道名转化语种
                        ch = ch.toTransEng(ch, lanType);
                        //end
                        result.add(ch);
                    }
                }

            }
        }

        return result;
    }

    @Override
    public List<Channel> listAllCooperationChannel() {
        return repository.findCoopIdChannel();
    }

    @Override
    public Channel findByUniqueName(String uniqueName) {

        return repository.findByUniqueName(uniqueName);
    }

    @Override
    public Channel findByUniqueName(String uniquename, Account loginAccount) {

        Channel channel = repository.findByUniqueName(uniquename);

        if (!loginAccount.getRootParent().equals(loginAccount.getId())) {
            loginAccount = accountRepository.findOne(loginAccount.getRootParent());
        }

        if (loginAccount.getRegSource() != 0) {
            String nodeUrl = logoDicRepository.findDicNodeURL(loginAccount.getRegSource(), channel.getId());
            channel.setNodeUrl(nodeUrl);
        }

        return channel;
    }

    @Override
    public List<Channel> findAllChannelByApp(Long appId) {

        return repository.listCampaignChannel(appId);
    }

    //@Override
    public void sendEmail(Account loginAccount, Channel channel) {
        String subject = "TrackingIo创建渠道";
        if ("APPMARKET".equals(channel.getCategory())) {
            List<Channel> channelList = repository.findByNameAndCategroy("APPMARKET", channel.getName());
            if (ValidateUtil.isValid(channelList) && channelList.size() == 1) {
                Account account = loginAccount;
                if (null != loginAccount.getRootParent()) {
                    account = accountRepository.findOne(loginAccount.getRootParent());
                }
                StringBuilder content = new StringBuilder();
                content.append("<!doctype html> <html><head></head><body> ")
                        .append("Larry：</br>")
                        .append("客户 " + account.getCompany() + " 创建了应用市场类型渠道，名称为“" + channel.getName() + "”，请做对应处理。")
                        .append("</body></html>");
                String contents = content.toString();
                List<String> list = new ArrayList<>();
                list.add("support@reyun.com");
                //list.add("manxiaoqiang@reyun.com");
                try {
                    //MailUtils.sendHtmlEmail(subject,contents,list);
                    SendCommonPostMail.sendMailReuse(list, subject, contents, null, null);
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                }
            }
        }
    }


    @Override
    public List<Channel> findByChannelName(String name, Account loginAccount) {

        if (loginAccount != null && !loginAccount.getId().equals(loginAccount.getParent())) {
            loginAccount = accountRepository.findOne(loginAccount.getRootParent());
        }

        return repository.findByChName(name, loginAccount.getRootParent());
    }
}
