package com.reyun.service.impl;

import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.CallBackTemplet;
import com.reyun.model.Cpcallback;
import com.reyun.repository.*;
import com.reyun.service.CpcallbackService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by Administrator on 2019/3/18 0007.
 */
@Service
public class CpcallbackServiceImpl implements CpcallbackService {

	@Autowired
	CpcallbackRepository cpcallbackRepository;

	@Autowired
	CallBackTempletRepository callBackTempletRepository;

	@Autowired
	AppRepository appRepository;

	@Autowired
	AccountRepository accountRepository;


	static final Map<String, String> eventMap = new HashMap();

	{
		eventMap.put("install", "激活");
		eventMap.put("reged", "注册");
		eventMap.put("loggedin", "登录");
		eventMap.put("click", "点击");
		eventMap.put("backflow", "回流");
		eventMap.put("invoke", "调起");
	}


	@Override
	public void closeExpiredCpcallback(String date15) {

		List<Cpcallback> daysAgo = this.cpcallbackRepository.find15DaysAgo(date15);
		//System.out.println("关闭"+daysAgo.size()+"条回调");
		if (!CollectionUtils.isEmpty(daysAgo)) {
			for (Cpcallback cpcallback : daysAgo) {
				cpcallback.setStatus(0);
				this.cpcallbackRepository.save(cpcallback);
			}
		}
	}

	@Override
	public CallBackTemplet getTemplate(Long appId, String event) {

		App app = appRepository.findOne(appId);
		if (app == null) {
			return null;
		}

		return callBackTempletRepository.findTemplet(event, app.getPlatform());
	}

	@Override
	public List<Map> getAvlibalEvents(String appkey) {

		String[] allEvent = {"install", "reged", "loggedin"};
		String[] name = {"激活", "注册", "登录"};
		List<String> events = cpcallbackRepository.findEventsByApp(appkey);
		Map<String, String> itemMap = events.stream().collect(Collectors.toMap(v -> v.toLowerCase(), v -> v, (v1, v2) -> v1));
		List<Map> result = new ArrayList<>(3);

		for (int i = 0; i < allEvent.length; i++) {
			if (!itemMap.containsKey(allEvent[i])) {
				Map item = new HashMap<>(2);
				item.put("name", name[i]);
				item.put("id", allEvent[i]);
				result.add(item);
			}
		}

		return result;
	}


	@Override
	public List getCpCallBackList(Account account) {

		List<String> appkeys = appRepository.findAllAppByAccount(account.getRootParent());
		List<Cpcallback> cpcallbacks = cpcallbackRepository.findByAppkeys(appkeys);
		cpcallbacks = cpcallbacks.stream().map(v -> {
			v.setEvnetName(eventMap.get(v.getEventType()));
			return v;
		}).collect(Collectors.toList());
		return cpcallbacks;

	}


	@Override
	public void putOrSave(Cpcallback cpcallback) {

		App app = appRepository.findByAppkey(cpcallback.getAppid());
		CallBackTemplet templet = callBackTempletRepository.findTemplet(cpcallback.getEventType(), app.getPlatform());
		String parmas = cpcallback.getCustermParams();
		StringBuffer pbuffer = new StringBuffer();
		pbuffer.append(cpcallback.getCustermUrl());
		pbuffer.append(templet.getTemplet());
		if ("install".equals(cpcallback.getEventType())) {
			Account account = accountRepository.findRootAccountById(app.getAccount());
			if (account != null && "trackingio@webeye.com".equals(account.getEmail())) {
				pbuffer.append("&mac=$!args.get('_mac')&manufacturer=$!args.get('_manufacturer')&op=$!args.get('_op')&network=$!args.get('_network')&join_type=$!args.get('joinType')&accountid=$!args.get('accountid')");
			}
		}
		if (parmas != null) {
			for (String parma : parmas.split(",")) {
				pbuffer.append("&").append(parma).append("=$!args.get('").append(parma).append("')");
			}
		}

		if (cpcallback.getId() != null) {
			Cpcallback cpcallbackOne = cpcallbackRepository.findOne(cpcallback.getId());
			if (cpcallbackOne != null) {
				cpcallbackOne.setUrl(pbuffer.toString());
				cpcallbackOne.setSpecialkeys("key=" + cpcallback.getSpecialkeys());
				cpcallbackOne.setUpdatetime(new Date());
				cpcallbackOne.setCustermParams(cpcallback.getCustermParams());
				cpcallbackOne.setCustermUrl(cpcallback.getCustermUrl());
				cpcallbackRepository.save(cpcallbackOne);
			}
		} else {
			cpcallback.setName(app.getName());
			cpcallback.setAppid(app.getAppkey());
			cpcallback.setUrl(pbuffer.toString());
			cpcallback.setSpecialkeys("key=" + cpcallback.getSpecialkeys());
			cpcallback.setTimeout(100);
			cpcallback.setStatus(1);
			cpcallback.setRespStatus("200");
			cpcallback.setInserttime(new Date());
			cpcallbackRepository.save(cpcallback);
		}

	}


	@Override
	public void updateStatus(Cpcallback resource) {
		Cpcallback cpcallback = cpcallbackRepository.findOne(resource.getId());
		if (cpcallback != null) {
			cpcallback.setStatus(resource.getStatus());
			cpcallback.setUpdatetime(new Date());
			cpcallbackRepository.save(cpcallback);
		}
	}


}
