package com.reyun.service.impl;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.WhatEventEnumType;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.DataCallBackService;
import com.reyun.service.INoSQLDBService;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.NoSQLDBUtil;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional
public class DataCallBackServiceImpl implements DataCallBackService {
    private static Logger logger = LoggerFactory.getLogger(DataCallBackServiceImpl.class);

    @Autowired
    CallbackCustomizeRepository callbackCustomizeRepository;

    @Autowired
    DataParamRepository dataParamRepository;

    @Autowired
    ChannelIntegrateRepository channelIntegrateRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    private AppRepository appRepository;
    @Autowired
    private INoSQLDBService iNoSQLDBService;

    @Autowired
    private CallbackNextdayRepository callbackNextdayRepository;
    @Autowired
    private CallbackFirstdayPaymentAuthRepository callbackFirstdayPaymentAuthRepository;


    @Override
    public JSONObject listParams() {
        List<DataParam> allList = dataParamRepository.findAll();
        JSONObject json = new JSONObject();
        for (DataParam data : allList) {
            if (json.get(data.getEvent()) != null) {
                JSONArray jsonArray = json.getJSONArray(data.getEvent());
                JSONObject obj = new JSONObject();
                obj.put("key", data.getDataKey());
                obj.put("desc", data.getDescp());
                obj.put("value", data.getValue());
                jsonArray.add(obj);
            } else {
                JSONArray jsonArray = new JSONArray();
                JSONObject obj = new JSONObject();
                obj.put("key", data.getDataKey());
                obj.put("desc", data.getDescp());
                obj.put("value", data.getValue());
                jsonArray.add(obj);
                json.put(data.getEvent(), jsonArray);
            }
        }
        return json;
    }

    @Override
    public JSONArray listEvents(Long channel, String lanType, Long id) {
        List<ChannelIntegrate> channelList = channelIntegrateRepository.findByChannelSort(channel);
        JSONArray arr = new JSONArray();
        if (channelList != null) {
            for (ChannelIntegrate ci : channelList) {

                if (ci.getWhat().equals("payment_amount")) {
                    ChannelIntegrate integrate = channelIntegrateRepository.findByChannelAndWhat(channel, "payment");
                    if (integrate != null) {
                        //已配置付费模板时 跳过
                        continue;
                    } else {
                        //只配置带金额付费时 按付费逻辑显示事件
                        ci.setWhat("payment");
                    }
                }

                WhatEventEnumType what = WhatEventEnumType.get(ci.getWhat());
                JSONObject obj = new JSONObject();
                obj.put("codename", what.getId());
                if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                    obj.put("name", what.getId());
                } else {
                    obj.put("name", what.getValue());

                }
                if (channel.longValue() == 514L && "payment".equals(ci.getWhat())
                        && callbackFirstdayPaymentAuthRepository.findOne(accountRepository.findOne(id).getRootParent()) != null) {
                    JSONObject o = new JSONObject();
                    o.put("codename", "firstdayPayment");
                    o.put("name", "新增付费");

                    arr.add(o);
                }
                arr.add(obj);
            }
        }
        return arr;
    }

    @Override
    public CallbackCustomize save(CallbackCustomize callbackCustomize, Long account, Boolean isCustom) {

        CallbackCustomize customize = this.nextDayData(callbackCustomize, account);
        if (customize != null) {
            // 处理次日留存
            return customize;
        }

        callbackCustomize.setAccount(account);
        callbackCustomize.setActive(true);
        callbackCustomize.setModifyAccount(account + "");
        callbackCustomize.setCreateTime(new Date());
        callbackCustomize.setPubdatetime(DateUtil.format(new Date(), DateUtil.C_TIME_PATTON_DEFAULT));
//		callbackCustomize.setDelFlag(false);

        Long channelId = callbackCustomize.getChannel();

        if (isCustom == null) {
            Channel channel = channelRepository.findOne(channelId);
            if (channel.isIsintegrate()) {
                ChannelIntegrate integrate = null;
                if ("payment".equals(callbackCustomize.getWhat())
                        && callbackCustomize.getHasPaymentAmount() != null
                        && callbackCustomize.getHasPaymentAmount() == 1 &&
                        (channelId.equals(514L)
                                || channelId.equals(501L)
                                || channelId.equals(10452L)
                                || channelId.equals(9219L))) {
                    //付费及新增付费带金额 模板处理
                    integrate = channelIntegrateRepository.findByChannelAndWhat(channelId, "payment_amount");
                    if (integrate == null) {
                        callbackCustomize.setErroMessage("未配置带金额付费模板");
                        return callbackCustomize;
                    }
                } else {
                    integrate = channelIntegrateRepository.findByChannelAndWhat(channelId, callbackCustomize.getWhat());
                }

                callbackCustomize.setUrl(integrate.getCallbackurl());
                callbackCustomize.setSuccesscode(integrate.getSuccesscode());
                callbackCustomize.setSuccesskey(integrate.getSuccesskey());
                callbackCustomize.setSuccessvalue(integrate.getSuccessvalue());
                callbackCustomize.setIscallback(integrate.getIscallback());
                callbackCustomize.setCallbackKey(integrate.getCallbackKey());
            }
        }

        if (Constant.awsneed) {
            syncDDB(callbackCustomize, true);
        }
        return callbackCustomizeRepository.save(callbackCustomize);
    }

    private CallbackCustomize nextDayData(CallbackCustomize callbackCustomize, Long account) {

        if (callbackCustomize != null && WhatEventEnumType.NEXTDAY.getId().equals(callbackCustomize.getWhat())) {

            App app = appRepository.findOne(callbackCustomize.getApp());
            CallbackNextday nextday = new CallbackNextday();
            //处理次日留存 保存与app的对应关系
            nextday.setAccount(account);
            nextday.setApp(callbackCustomize.getApp());
            nextday.setChannel(callbackCustomize.getChannel());
            nextday.setCreateTime(new Date());
            nextday.setName(callbackCustomize.getName());
            nextday.setWhat(callbackCustomize.getWhat());
            nextday.setCreateAccount(account + "");
            nextday.setActive(1);
            nextday.setAppkey(app.getAppkey());
            nextday.setSpecial_event(callbackCustomize.getSpecial_event());
            nextday.setSpecial_nextday(callbackCustomize.getSpecial_nextday());
            callbackNextdayRepository.save(nextday);
            return callbackCustomize;
        }

        return null;
    }


    @Override
    public CallbackCustomize delete(Long id, Long account) {

        CallbackCustomize findOne = callbackCustomizeRepository.findOne(id);
        findOne.setModifyAccount(account + "");
        findOne.setModifyTime(new Date());
        findOne.setDelFlag(true);
        findOne.setActive(false);

        //region 同步ddb
        if (Constant.awsneed) {
            syncDDB(findOne, false);
        }

        //endregion
        return callbackCustomizeRepository.save(findOne);
    }

    @Override
    public CallbackCustomize update(CallbackCustomize callbackCustomize, Long account, Boolean isCustom) {

        CallbackCustomize customize = this.nextDayData(callbackCustomize, account);
        if (customize != null) {
            //删除原 回调事件
            this.delete(callbackCustomize.getId(), account);
            // 处理次日留存
            return customize;
        }

        CallbackCustomize one = callbackCustomizeRepository.findOne(callbackCustomize.getId());
        callbackCustomize.setAccount(one.getAccount());
        callbackCustomize.setUpdatetime(DateUtil.format(new Date(),
                DateUtil.C_TIME_PATTON_DEFAULT));
        callbackCustomize.setPubdatetime(one.getPubdatetime());
        callbackCustomize.setModifyAccount(account + "");
        callbackCustomize.setModifyTime(new Date());
//		callbackCustomize.setDelFlag(false);
        callbackCustomize.setActive(one.isActive());

        Long channelId = callbackCustomize.getChannel();
        if (isCustom == null) {
            Channel channel = channelRepository.findOne(channelId);
            if (channel.isIsintegrate()) {

                ChannelIntegrate integrate = null;
                if ("payment".equals(callbackCustomize.getWhat())
                        && callbackCustomize.getHasPaymentAmount() != null
                        && callbackCustomize.getHasPaymentAmount() == 1 &&
                        (channelId.equals(514L)
                                || channelId.equals(501L)
                                || channelId.equals(10452L)
                                || channelId.equals(9219L))) {
                    //付费及新增付费带金额 模板处理
                    integrate = channelIntegrateRepository.findByChannelAndWhat(channelId, "payment_amount");
                    if (integrate == null) {
                        callbackCustomize.setErroMessage("未配置带金额付费模板");
                        return callbackCustomize;
                    }
                } else {
                    integrate = channelIntegrateRepository.findByChannelAndWhat(channelId, callbackCustomize.getWhat());
                }

                callbackCustomize.setUrl(integrate.getCallbackurl());
                callbackCustomize.setSuccesscode(integrate.getSuccesscode());
                callbackCustomize.setSuccesskey(integrate.getSuccesskey());
                callbackCustomize.setSuccessvalue(integrate.getSuccessvalue());
                callbackCustomize.setIscallback(integrate.getIscallback());
                callbackCustomize.setCallbackKey(integrate.getCallbackKey());
            }
        }


        if (Constant.awsneed) {
            //region 当渠道与历史渠道不一致时，需删除之前的ddb数据
            if (!one.getChannel().equals(callbackCustomize.getChannel())) {
                syncDDB(one, false);
            }
            syncDDB(callbackCustomize, true);
            //endregion
        }


        return callbackCustomizeRepository.save(callbackCustomize);
    }

    @Override
    public List<CallbackCustomize> list(Long account, Long app) {
        List<CallbackCustomize> list = new ArrayList<CallbackCustomize>();
        list = callbackCustomizeRepository.listByAccount(account, app);
        return list;
    }

    @Override
    public List<CallbackCustomize> listAll(Long app) {
        List<CallbackCustomize> list = new ArrayList<CallbackCustomize>();
        list = callbackCustomizeRepository.listByApp(app);
        return list;
    }

    @Override
    public CallbackCustomize enable(Long id, Long account) {
        CallbackCustomize callback = callbackCustomizeRepository.findOne(id);
        callback.setActive(true);
        callback.setModifyAccount(account + "");
        callback.setModifyTime(new Date());
        CallbackCustomize save = callbackCustomizeRepository.save(callback);
        syncDDB(save, true);
        return save;
    }

    @Override
    public CallbackCustomize forbidden(Long id, Long account) {
        CallbackCustomize callback = callbackCustomizeRepository.findOne(id);
        callback.setActive(false);
        callback.setModifyAccount(account + "");
        callback.setModifyTime(new Date());
        CallbackCustomize save = callbackCustomizeRepository.save(callback);
        syncDDB(save, false);
        return save;
    }

    @Override
    public List<CallbackCustomize> find(Long app, Long channelId) {
        return this.callbackCustomizeRepository.findByChannelAndApp(channelId, app);
    }

    @Override
    public Boolean valid(Long account, Long app, String name) {
        boolean flag = true;
        CallbackCustomize callback = callbackCustomizeRepository
                .findByNameAndAccount(account, app, name);
        if (callback == null) {
            flag = false;
        }
        return flag;
    }

    private void syncDDB(CallbackCustomize callbackCustomize, boolean isAdd) {

        //region 同步dynamodb
        App app = appRepository.findOne(callbackCustomize.getApp());
        if (app == null) {
            logger.error("应用不存在, app: {}", callbackCustomize.getApp());
            throw new TipException("应用不存在");
        }

        String appkey = app.getAppkey();
        Long channelid = callbackCustomize.getChannel();

        List<Map> itemList = Lists.newArrayList();
        List<CallbackCustomize> byChannelAndApp = callbackCustomizeRepository.findByChannelAndApp(channelid, app.getId());

        byChannelAndApp = byChannelAndApp.stream().filter(customize -> {
            return customize != null && customize.isActive() && !customize.getId().equals(callbackCustomize.getId());
        }).collect(Collectors.toList());
        if (isAdd) {
            byChannelAndApp.add(callbackCustomize);
        }
        for (CallbackCustomize customize : byChannelAndApp) {
            String what = "keybehavior".equals(customize.getWhat()) ? "kb_" + customize.getEvent() : customize.getWhat();
            itemList.add(ImmutableMap.of("appid", appkey, "what", what, "cid", customize.getChannel()));
        }

        if (itemList != null && itemList.size() > 0) {
            Map callbackMap = ImmutableMap.of("id", keyCallbackCustomize(appkey, channelid), "config_type", "customCallback", "setting", itemList);
            boolean status = iNoSQLDBService.updateSettingNewTable(NoSQLDBUtil.TBL_TKIO_SETTING_NEW, callbackMap);
            //endregion
            if (!status) {
                throw new TipException("同步callback错误");
            }
        } else {
            iNoSQLDBService.deleteFromSettingNewTable(NoSQLDBUtil.TBL_TKIO_SETTING_NEW, "id", keyCallbackCustomize(appkey, channelid));
        }
    }

    private String keyCallbackCustomize(String appkey, Long channelid) {
        return String.format("%s:%s:%s", "customCallback", appkey, channelid);
    }

    @Override
    public List<CallbackNextday> nextCallList(Long appid) {
        return null;
    }


    @Override
    public int setNextActiveData(Long id, int statu) {

        CallbackNextday nextday = callbackNextdayRepository.findOne(id);
        nextday.setUpdatetime(new Date());
        nextday.setActive(statu);
        callbackNextdayRepository.save(nextday);
        return 0;
    }


    @Override
    public List<CallbackCustomize> nextCallApp(Long appid) {

        List<CallbackNextday> nextday = callbackNextdayRepository.findByAppNotDel(appid);
        if (nextday != null) {

            List<CallbackCustomize> dataList = new ArrayList<>();

            for (CallbackNextday nextdayone : nextday) {
                CallbackCustomize customize = new CallbackCustomize();
                customize.setId(nextdayone.getId());
                customize.setActive(nextdayone.getActive() == 1 ? true : false);
                customize.setChannel(nextdayone.getChannel());
                customize.setCreateTime(nextdayone.getCreateTime());
                customize.setPubdatetime(new DateTime(nextdayone.getCreateTime()).toString("yyyy-MM-dd HH:mm:ss"));
                customize.setWhat(nextdayone.getWhat());
                customize.setName(nextdayone.getName());
                dataList.add(customize);
            }
            return dataList;
        }

        return null;
    }
}
