package com.reyun.service.impl;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.Account;
import com.reyun.model.CallbackDisable;
import com.reyun.model.DataAuth;
import com.reyun.repository.DataAuthRepository;
import com.reyun.service.DataCallbackSettingService;
import com.reyun.util.DBUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

/**
 * description:
 *
 * @author nolan
 * @date 05/09/2017
 */
@Service
public class DataCallbackSettingServiceImpl
        implements DataCallbackSettingService
{
    @Autowired
    private DataAuthRepository dataAuthRepository;

    @Override
    public List<CallbackDisable> findAll(Long appid, Account currentAccount ,String lanType)
    {
        List<CallbackDisable> rtnList = Lists.newArrayList();
        //国际化处理
        String flag = "激活";
        if(LanguageTypeEnum.ENGLISH.getCode().equals(lanType)){
            flag="install";
        }
        String sql = "select distinct  b.name as appname, c.name as channelname, '"+flag+"' as eventname, isnull(d.id) as status,  b.id as app, c.id as channel, d.id from campaign a " +
                "join app b on a.app = b.id " +
                "join channel c on a.channel = c.id " +
                "left join callback_disable d on d.app = a.app and d.channel = a.channel " +
                " left join callback cb on cb.key = c.unique_name where a.app = " + appid + " and (c.type = 'ry_coop' or c.type = 'ry_bd') and cb.id is not null order by b.name, c.name";
        Connection conn = DBUtil.newInstance().getConn();
        Statement statement = null;
        ResultSet rs = null;
        try {
            statement = conn.createStatement();
            rs = statement.executeQuery(sql);
            while (rs.next()) {
                CallbackDisable callbackDisable = new CallbackDisable();
                callbackDisable.setApp(rs.getLong("app"));
                callbackDisable.setAppname(rs.getString("appname"));
                callbackDisable.setChannel(rs.getLong("channel"));
                callbackDisable.setChannelname(rs.getString("channelname"));
                callbackDisable.setEventname(rs.getString("eventname"));
                callbackDisable.setStatus(rs.getBoolean("status"));
                rtnList.add(callbackDisable);
            }
        }
        catch (SQLException e) {
            e.printStackTrace();
        }
        finally {
            DBUtil.newInstance().release(rs, statement, conn);
        }

        if (currentAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(currentAccount.getRoleCategory())
                || RoleEnumType.SUB_APP_MANAGER.getKey().equals(currentAccount.getRoleCategory())) {
        }
        else {
            List<DataAuth> dataAuths = dataAuthRepository.listByAllCampaign(currentAccount.getId(), appid);

            final List<Long> channelids = Lists.transform(dataAuths, new Function<DataAuth, Long>()
            {
                @Override
                public Long apply(DataAuth dataAuth)
                {
                    return dataAuth.getChannel();
                }
            });
            rtnList = Lists.newArrayList(Iterables.filter(rtnList, new Predicate<CallbackDisable>()
            {
                @Override
                public boolean apply(CallbackDisable input)
                {
                    return channelids.contains(input.getChannel());
                }
            }));
        }
        return rtnList;
    }
}
