package com.reyun.service.impl;

import com.reyun.dic.DownloadEnumType;
import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.model.Account;
import com.reyun.model.DownloadInfo;
import com.reyun.repository.DownloadInfoRepository;
import com.reyun.service.IDownloadService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by nolan on 04/01/2017.
 * description:
 */
@Service
public class DownloadServiceImpl implements IDownloadService {

    private static Map<String, String> rangeNameMap = new HashMap() {{
        put("5s", "小于5秒");
        put("15s", "小于15秒");
        put("30s", "小于30秒");
        put("60s", "小于1分钟");
        put("300s", "小于5分钟");
        put("c1", "近一天");
        put("c3", "近三天");
        put("c7", "近七天");
    }};

    @Autowired
    private DownloadInfoRepository downloadInfoRepository;

    @Override
    public DownloadInfo save(DownloadInfo downloadInfo) {
        return downloadInfoRepository.saveAndFlush(downloadInfo);
    }

    @Override
    public DownloadInfo updateStatus(Long id, DownloadStatusEnumType downloadStatusEnumType) {
        downloadInfoRepository.update(id, downloadStatusEnumType.getCode());
		return this.findById(id);
    }

    @Override
    public boolean exist(Long account, String filename, DownloadInfo downloadInfo) {
        DownloadInfo downloadInfoRe = downloadInfoRepository.findByDtype(account, filename, downloadInfo.getDownloadtype());
        if(downloadInfoRe!=null){
            return true;
        }
        return false;
    }

    @Override
    public DownloadInfo findById(Long id) {
        return this.downloadInfoRepository.findOne(id);
    }

    @Override
    public List<DownloadInfo> find(Long app, Long accountid, DownloadEnumType downloadEnumType) {
        return this.downloadInfoRepository.find(app, accountid, downloadEnumType.getCode());
    }

    @Override
    public List<DownloadInfo> find(Long appId, Account loginAccount) {

        //母账号能看到所有的日志导出,子账号只能看到自己创建的。add by sunhao 20170828

        List<DownloadInfo> datas;

        if (loginAccount.getIsSuperUser()) {

            datas =  downloadInfoRepository.findByApp(appId);

        } else {

            datas = downloadInfoRepository.findByAccountAndApp(appId, loginAccount.getId());
        }

        datas.forEach(v -> {
            if (v.getFakeRange() != null) {
                String range = v.getDownloadtype().equals(DownloadEnumType.HIJACK.getCode()) ? v.getFakeRange() + "s" : "c" + v.getFakeRange();
                v.setBusiname(v.getBusiname() + "," + rangeNameMap.get(range));
            }
        });

        return datas;
    }

    @Override
    public DownloadInfo delete(Long id) {
        this.downloadInfoRepository.delete(id);
		return this.findById(id);
    }
}
