package com.reyun.service.impl;

import com.reyun.dic.DownloadEnumType;
import com.reyun.model.App;
import com.reyun.model.DownloadInfo;
import com.reyun.repository.AppRepository;
import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;

import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class DownloadSizeService {
    private Logger logger = LoggerFactory.getLogger(DownloadSizeService.class);
    private static final String OLD_CODE = "_old";
    private static final String READ_S3_BUCKET = "reyuntkio";

    private static final Map<String, String> READ_KEY_BASEMAP = new HashMap<String, String>() {{
        put(DownloadEnumType.INSTALL.getCode(), "tkio_mid_install_day");
        put(DownloadEnumType.REGED.getCode() + OLD_CODE, "tkio_mid_reg_day");
        put(DownloadEnumType.REGED.getCode(), "tkio_mid_reg_download");
        put(DownloadEnumType.PAY.getCode() + OLD_CODE, "tkio_mid_pay_day");
        put(DownloadEnumType.PAY.getCode(), "tkio_mid_pay_download");
        put(DownloadEnumType.CLICK.getCode(), "tkio_mid_clk_normal_day");
        put(DownloadEnumType.BLACKLIST.getCode(), "tkio_mid_clk_detail_day_down");
        put(DownloadEnumType.FAKE.getCode(), "tkio_rpt_channel_black_info");
        put(DownloadEnumType.ORDER.getCode() + OLD_CODE, "tkio_mid_order_day");
        put(DownloadEnumType.ORDER.getCode(), "tkio_mid_order_download");
        put(DownloadEnumType.LAUNCH.getCode(), "mid_launch_day");
        put(DownloadEnumType.REGISTER.getCode(), "mid_register_day");
        put(DownloadEnumType.PAYMENT.getCode(), "mid_payment_day");
        put(DownloadEnumType.DEVICELOGIN.getCode() + OLD_CODE, "tkio_mid_loggedin_day");
        put(DownloadEnumType.DEVICELOGIN.getCode(), "tkio_mid_loggedin_download");
        put(DownloadEnumType.H5EVENTS.getCode(), "tkio_phone_day");

    }};

    private static final Map<String, String> readDbBaseMap = new HashMap<String, String>() {{
        put("tkio_mid_install_day", "tkio_realtime_v2.db");
        put("tkio_mid_reg_day", "tkio_realtime_v2.db");
        put("tkio_mid_pay_day", "tkio_realtime_v2.db");
        put("tkio_mid_clk_normal_day", "tkio_v2.db");
        put("tkio_mid_clk_detail_day_down", "tkio_v2.db");
        put("tkio_mid_loggedin_day", "tkio_v2.db");
    }};

    @Autowired
    private AppRepository appRepository;


    public boolean checkByteSize(DownloadInfo downloadInfo) {

        if (!"default".equals(downloadInfo.getDivtype()) && !"all".equals(downloadInfo.getDivtype())) {
            return false;
        }

        if ("a6fc6c73c31f27b25824ff210621095d".equals(downloadInfo.getAppkey())
                || "228ffed7b5c5cfc0091152c225ccd305".equals(downloadInfo.getAppkey())) {
            return false;
        }


        long bytesize = getDownloadFileByteSize(downloadInfo);

        logger.info("导出日志字节大小[{}]MB,ACCOUNT[{}]", bytesize / (1024 * 1024), downloadInfo.getCreateAccount());

        return bytesize > 1024 * 1024 * 100;
    }


    private long getDownloadFileByteSize(DownloadInfo downloadInfo) {

        String downloadtype = downloadInfo.getDownloadtype();
        String appkey = downloadInfo.getAppkey();
        String startdate = downloadInfo.getStartdate();
        String enddate = downloadInfo.getEnddate();

        App app = appRepository.findOne(downloadInfo.getApp());

        String keyBase = READ_KEY_BASEMAP.get(downloadtype);
        int days = Days.daysBetween(new DateTime(startdate), new DateTime(enddate)).getDays();
        long fileByteLength = 0L;

        for (int i = 0; i <= days; i++) {
            final DateTime dsDate = new DateTime(startdate).plusDays(i);
            String ds = dsDate.toString("yyyy-MM-dd");

            List<String> keylist;
            String readFormatS3Key;
            String readS3Bucket = Constant.bucket;

            if (dsDate.isBefore(DateTime.parse("2018-01-07"))) {
                readS3Bucket = READ_S3_BUCKET;
                readFormatS3Key = readFormatS3Key(keyBase, appkey, ds);
            } else {
                if (dsDate.isBefore(DateTime.parse("2019-04-23"))) {
                    String keyBaseOld = READ_KEY_BASEMAP.get(downloadtype + OLD_CODE);
                    if (!StringUtils.isEmpty(keyBaseOld)) {
                        keyBase = keyBaseOld;
                    }
                } else if (dsDate.isAfter(DateTime.parse("2019-06-13"))) {
                    keyBase = READ_KEY_BASEMAP.get(downloadtype + OLD_CODE);
                    if (keyBase == null) {
                        keyBase = READ_KEY_BASEMAP.get(downloadtype);
                    }
                } else {
                    keyBase = READ_KEY_BASEMAP.get(downloadtype);
                }


                readFormatS3Key = newReadFormatS3Key(keyBase, appkey, ds);
                if ("wx".equals(app.getPlatform()) || "wg".equals(app.getPlatform())) {
                    readFormatS3Key = newReadWeiXinFormatS3Key(keyBase, appkey, ds);
                }
            }

            keylist = ObjectStorageUtil.listObjects(readS3Bucket, readFormatS3Key);

            for (String s3key : keylist) {
                fileByteLength += ObjectStorageUtil.getLength(readS3Bucket, s3key);
            }
        }

        return fileByteLength;


    }

    private String readFormatS3Key(String keyBase, String appkey, String ds) {
        return "warehouse/tkio/" + readDbBaseMap.get(keyBase) + "/" + keyBase + "/ds=" + ds + "/appid=" + appkey + "/";
    }

    private String newReadFormatS3Key(String keyBase, String appkey, String ds) {
        return "tkio/" + keyBase + "/ds=" + ds + "/appid=" + appkey + "/";
    }

    private String newReadWeiXinFormatS3Key(String keyBase, String appkey, String ds) {
        return "wx/" + keyBase + "/ds=" + ds + "/appid=" + appkey + "/";
    }

}
