package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Function;
import com.google.common.collect.Maps;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.CommonParamService;
import com.reyun.service.EventService;
import com.reyun.util.*;
import org.apache.commons.collections.CollectionUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class EventServiceImpl implements EventService {
    private static final Logger logger = LoggerFactory.getLogger(EventServiceImpl.class);

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    EventRepository eventRepository;

    @Autowired
    AppRepository appRepository;

    @Autowired
    CommonEventRepository commonEventRepository;

    @Autowired
    CommonParamService commonParamService;


    @Autowired
    EventMetaRepository eventMetaRepository;


    @Autowired
    AttributeRestrictRepository attributeRestrictRepository;


    @Autowired
    CommonParamRepository commonParamRepository;


    @Autowired
    EventService eventService;

    private static final String EVENT_ATTR_CID = "_cid";

    private static final String EVENT_ATTR_CAMPAIGNID = "_campaignid";


    @Override
    public List<Event4Web> listEvent(Long app) {

        String appkey = appRepository.findAppkeyById(app);
        List<Event> eventList = eventRepository.findByAppkey(appkey);
        List<EventMeta> metas = eventMetaRepository.findByAppkey(appkey);
        List<CommonEvent> commonEventList = commonEventRepository.findAll();

        Map<String, EventMeta> metaMap = new HashMap<>();
        if (ValidateUtil.isValid(metas)) {
            for (EventMeta e : metas) {
                metaMap.put(e.getEventId(), e);
            }
        }

        Map<String, CommonEvent> commonMap = new HashMap<>();
        if (ValidateUtil.isValid(commonEventList)) {
            for (CommonEvent e : commonEventList) {
                commonMap.put(e.getEvent(), e);
            }
        }

        LinkedList<Event4Web> result = new LinkedList<>();
        LinkedList<Event4Web> temp = new LinkedList<>();
        List<String> nameList = new ArrayList<>();
        if (ValidateUtil.isValid(eventList)) {
            for (Event e : eventList) {
                if (!nameList.contains(e.getEventName())) {
                    if (commonMap.containsKey(e.getEventName())) {
                        nameList.add(e.getEventName());

                        CommonEvent commonEvent = commonMap.get(e.getEventName());
                        Event4Web event4Web = commonEventTo4Web(commonEvent);
                        result.add(event4Web);
                    } else {
                        nameList.add(e.getEventName());
                        if (metaMap.containsKey(e.getEventName())) {
                            EventMeta eventMeta = metaMap.get(e.getEventName());
                            Event4Web event4Web = eventMetaTo4Web(eventMeta);
                            temp.add(event4Web);
                        } else {
                            Event4Web event4Web = eventTo4Web(e);
                            temp.add(event4Web);
                        }
                    }
                }
            }
            result.addAll(temp);
        }
        return result;
    }

    /**
     * 查询所有的事件以及名字,排重后
     */
    @Override
    public List<Event4Web> listAllEvent(Long app, String lanType) {

        List<Event4Web> result = new ArrayList<>();
        Event4Web eventreged = new Event4Web();
        eventreged.setEventName("reged");
        eventreged.setEventNameAlias("注册");
        eventreged.setCommon(true);
        eventreged.setVirtual(false);
        eventreged.setStatus(true);
        result.add(eventreged);

        Event4Web eventlogin = new Event4Web();
        eventlogin.setEventName("loggedin");
        eventlogin.setEventNameAlias("登录");
        eventlogin.setCommon(true);
        eventlogin.setVirtual(false);
        eventlogin.setStatus(true);
        result.add(eventlogin);

        Event4Web eventpayment = new Event4Web();
        eventpayment.setEventName("payment");
        eventpayment.setEventNameAlias("付费");
        eventpayment.setCommon(true);
        eventpayment.setVirtual(false);
        eventpayment.setStatus(true);
        result.add(eventpayment);


        List<String> eventIdList = new ArrayList<>();
        eventIdList.add("reged");
        eventIdList.add("loggedin");
        eventIdList.add("payment");

        App appObject = appRepository.findOne(app);
        String url = Constant.ioUrl + "/api/" + app + "/event/find4tracking?appkey=" + appObject.getAppkey();
        try {
            String response = HttpClientUtil.doHttpGetRequestNew(url, "tracking");
            if (!StringUtil.isEmpty(response)) {

                JSONObject ob = new JSONObject(response);
                if (ob.has("content")) {
                    JSONArray content = ob.getJSONArray("content");
                    for (int i = 0; i < content.length(); i++) {
                        JSONObject object = content.getJSONObject(i);
                        ObjectMapper mapper = new ObjectMapper();
                        Event4Web event4Web = mapper.readValue(object.toString(), Event4Web.class);
                        if (event4Web.getStatus() && !eventIdList.contains(event4Web.getEventName())) {
                            result.add(event4Web);
                            eventIdList.add(event4Web.getEventName());
                        }

                    }
                }
            }
        } catch (Exception e) {
            logger.error("fail to get events from IO......... appkey is " + appObject.getAppkey());
        }

        List<Event4Web> event4Webs = listEvent(app);
        if (ValidateUtil.isValid(event4Webs)) {
            for (Event4Web e : event4Webs) {
                if (e.getStatus() && !eventIdList.contains(e.getEventName())) {
                    result.add(e);
                    eventIdList.add(e.getEventName());
                }
            }
        }

        //国际化处理
         List<Event4Web> newResult = new ArrayList<>();

        if(LanguageTypeEnum.ENGLISH.getCode().equals(lanType)){
            for (Event4Web event4Web : result){
                event4Web.setEventNameAlias(event4Web.getEventName());
                newResult.add(event4Web);
            }
            return newResult;
        }else{
            return result;
        }
        //end


    }

    @Override
    public List<Event4Web> listAllEventToCallback(Long app) {
        List<Event4Web> result = new ArrayList<>();

        List<Event4Web> event4Webs = listEvent(app);
        if (ValidateUtil.isValid(event4Webs)) {
            for (Event4Web e : event4Webs) {
                if (e.getStatus() && !e.getEventName().equals("reged")
                        && !e.getEventName().equals("loggedin") && !e.getEventName().equals("payment")) {
                    result.add(e);
                }
            }
        }
        return result;
    }

    @Override
    public Map<String, List> getValueResult(String attrName, List<String> valueList) {
        Map<String, List> result = new LinkedHashMap<>();
        if (EVENT_ATTR_CID.equals(attrName)) {
            List<Long> channelIdList = transChannelAttr2Long(valueList);
            List<String> channelNameList = this.getChannelNameList(channelIdList);
            result.put("key", channelIdList);
            result.put("value", channelNameList);
        } else if (EVENT_ATTR_CAMPAIGNID.equals(attrName)) {
            List<String> campaignNameList = this.getCampaignNameList(valueList);
            result.put("key", valueList);
            result.put("value", campaignNameList);
        } else {
            result.put("value", valueList);
        }
        return result;
    }

    /**
     * _campaignid 获取活动名称列表
     * add by sunhao 2017-04-06
     */
    private List<String> getCampaignNameList(List<String> campaignKeyList) {

        List<String> nameList = new ArrayList<>();

        Map<String, Campaign> campaignMap = new HashMap<>();

        List<Campaign> campaignList = campaignRepository.findAllByCampaignId(campaignKeyList);

        campaignList.add(new Campaign("_default_", "自然量"));

        if (!CollectionUtils.isEmpty(campaignList)) {
            campaignMap = Maps.uniqueIndex(campaignList, new Function<Campaign, String>() {
                @Override
                public String apply(Campaign campaign) {
                    return campaign.getCampaignid();
                }
            });
        }

        //填充values
        for (String campaignid : campaignKeyList) {

            if (campaignMap.containsKey(campaignid)) {
                nameList.add(campaignMap.get(campaignid).getName());
            } else {
                nameList.add(campaignid);
            }
        }

        return nameList;
    }

    /**
     * _cid 获取渠道名称列表
     * add by sunhao 2017-04-06
     */
    private List<String> getChannelNameList(List<Long> channelIdList) {

        List<String> nameList = new ArrayList<>();

        Map<Long, Channel> channelMap = new HashMap<>();

        List<Channel> channelList = channelRepository.findById(channelIdList);

        channelList.add(new Channel(-1L, "自然量"));

        if (!CollectionUtils.isEmpty(channelIdList)) {

            channelMap = Maps.uniqueIndex(channelList, new Function<Channel, Long>() {
                @Override
                public Long apply(Channel channel) {
                    return channel.getId();
                }
            });
        }

        //填充values
        for (Long channelId : channelIdList) {

            if (channelMap.containsKey(channelId)) {
                nameList.add(channelMap.get(channelId).getName());
            } else {
                nameList.add(channelId.toString());
            }
        }

        return nameList;
    }

    /**
     * 将字符串ID转换成LONG 非字符串跳过
     */
    private List<Long> transChannelAttr2Long(List<String> channelIdList) {

        List<Long> longList = new ArrayList<>();

        for (String channelId : channelIdList) {

            try {

                longList.add(Long.parseLong(channelId));

            } catch (NumberFormatException e) {
                logger.error(e.getMessage());

//                e.printStackTrace();
            }
        }

        return longList;
    }


    private Event4Web eventMetaTo4Web(EventMeta e) {
        Event4Web ew = new Event4Web();
        ew.setCommon(false);
        ew.setEventName(e.getEventId());
        ew.setEventNameAlias(e.getAlias() == null ? e.getEventId() : e.getAlias());
        ew.setStatus(e.getStatus());

        return ew;
    }

    private Event4Web eventTo4Web(Event e) {
        e = killDumpName(e);
        Event4Web ew = new Event4Web();
        ew.setCommon(false);
        ew.setEventName(e.getEventName());
        ew.setEventNameAlias(e.getEventName());
        ew.setStatus(true);
        return ew;
    }

    public Event4Web commonEventTo4Web(CommonEvent e) {
        Event4Web ew = new Event4Web();
        ew.setCommon(true);
        ew.setEventName(e.getEvent());
        ew.setEventNameAlias(e.getAlias());
        ew.setStatus(true);
        return ew;
    }

    // 处理 dupnum 前缀导致自定义事件异常问题
    private Event killDumpName(Event e) {
        String event_name = e.getEventName();
        if (event_name != null && event_name.startsWith("dupnum")) {
            e.setEventName(event_name.substring(7, event_name.length()));
        }
        return e;

    }

    public static void main(String[] args) {
        String ename = "dupnum_event_1";
        System.out.println(ename.substring(7, ename.length()));
    }
}
