package com.reyun.service.impl;

import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.model.Account;
import com.reyun.model.App;
import com.reyun.model.ExportReportInfo;
import com.reyun.model.ExportReportParam;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ExportReportInfoRepository;
import com.reyun.repository.ExportReportParamRepository;
import com.reyun.service.ExportReportInfoService;
import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;

import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class ExportReportInfoServiceImpl implements ExportReportInfoService {

    @Autowired
    ExportReportInfoRepository repository;

    @Autowired
    AppRepository appRepository;

    @Autowired
    ExportReportParamRepository exportReportParamRepository;


    private final static int EXPORT_INVALID_DAYS = 3;
    private final static int EXPORT_INVALID_MONTH = 1;

    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;

    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    @Override
    public ExportReportInfo save(ExportReportInfo exportReport) {
        exportReport.setCreateTime(new Date());
        return repository.save(exportReport);
    }

    @Override
    public void delete(Long id) {
        ExportReportInfo reportInfo = repository.findOne(id);
        final String s3key = DOWNLOAD_PATH + reportInfo.getFileName();
        //删除文件
        if (ObjectStorageUtil.doesObjectExist(S3_DOWNLOAD_BUCKET, s3key)) {
            ObjectStorageUtil.deleteObject(S3_DOWNLOAD_BUCKET, s3key);
        }
        //删除记录
        repository.delete(id);

    }

    @Override
    public List<ExportReportInfo> findByApp(Account loginAccount, Long app) {

        Date startDate = new DateTime().minusMonths(EXPORT_INVALID_MONTH).toDate();

        //母账号查看所有导出报表,子账号只能查看自己创建的 add by sunhao 20170828
        if (loginAccount.getIsSuperUser()) {

            return repository.findByApp(app, startDate);

        } else {

            return repository.findByAccountAndApp(app, startDate, loginAccount.getId());
        }
    }

    @Override
    public boolean validFileName(ExportReportInfo exportReport) {
        App app = appRepository.findOne(exportReport.getApp());
        String fileName = app.getAppkey() + "_" + exportReport.getReportName() + "_" + exportReport.getStartDate() + "_" + exportReport.getEndDate() + ".txt";
        ExportReportInfo export = repository.findByAppAndFileName(exportReport.getApp(), fileName);
        if (export == null) {
            return true;
        }
        return false;
    }

    @Override
    public int updateStatus(Long id, String status, Long account) {
        return repository.updateStatus(id, status, new Date(), account);
    }




    @Override
    public int updateStatus(Long id, String status) {

        if(DownloadStatusEnumType.FAILED.getCode().equals(status)){
            ExportReportInfo exportReportInfo =  repository.findOne(id);
            if(exportReportInfo.getExceptTimes()>=3){
                //控制重试 3 次
                return repository.updateStatus(id, DownloadStatusEnumType.STOPRETRY.getCode(), new Date());
            }

            return repository.updateFailedStatus(id,status,exportReportInfo.getExceptTimes()+1,new Date());

        }

        return repository.updateStatus(id, status, new Date());
    }

    @Override
    public ExportReportInfo findOne(Long id) {
        return repository.findOne(id);
    }

    @Override
    public List<ExportReportInfo> findExport2Deal() {
        return repository.findByStatus(DownloadStatusEnumType.INIT.getCode(), DownloadStatusEnumType.FAILED.getCode());
    }

    @Override
    public List<ExportReportInfo> findInvalidExportList() {

        //获取三天前的日期
        Date threeDaysAgo = DateTime.now().minusDays(EXPORT_INVALID_DAYS).toDate();
        //查找三天前的已生成的数据
        return repository.listInvalidExport(threeDaysAgo);
    }





    @Override
    public ExportReportParam openExportParma(ExportReportParam resource, Long accountId) {

        if(resource==null){
            return null;
        }

        if(resource.getOpened()==null){
            resource.setOpened(0);
        }

        ExportReportParam param = null;

        if(resource.getId() > 0){
            param = exportReportParamRepository.findOne(resource.getId());
        }

        if (param == null) {
            // 新增
            resource.setCreateTime(new Date());
            resource.setCreateAccount(accountId);
            exportReportParamRepository.save(resource);
            return resource;

        } else {
            // 更新
            param.setOpened(resource.getOpened());
            param.setModifyAccount(accountId);
            param.setModifyTime(new Date());
            exportReportParamRepository.save(param);
            return param;
        }

    }

    @Override
    public ExportReportParam getExportParam(Long app, String reportName) {

        return  exportReportParamRepository.findByAppReportName(app,reportName);
    }

    @Override
    public List<ExportReportParam> openExportParmaList(Long app, Long accountId) {

        List<ExportReportParam> list = exportReportParamRepository.findByApp(app);

        if(list == null || list.size()== 0){
            list = new ArrayList<>();
            ExportReportParam param_1 = new ExportReportParam();
            param_1.setName("推广活动详情-活动");
            param_1.setOpened(0);
            param_1.setApp(app);
            param_1.setCreateAccount(accountId);
            param_1.setCreateTime(new Date());
            param_1.setExportName("campaigninfo_bycampgroup");
            list.add(param_1);

            ExportReportParam param_2 = new ExportReportParam();
            param_2.setName("推广活动详情-渠道");
            param_2.setOpened(0);
            param_2.setApp(app);
            param_2.setCreateAccount(accountId);
            param_2.setCreateTime(new Date());
            param_2.setExportName("campaigninfo_bychannel");
            list.add(param_2);
            ExportReportParam param_3 = new ExportReportParam();
            param_3.setName("推广活动详情-活动组");
            param_3.setOpened(0);
            param_3.setApp(app);
            param_3.setCreateAccount(accountId);
            param_3.setCreateTime(new Date());
            param_3.setExportName("campaigninfo_bycampgroup_campgroup");
            list.add(param_3);

            exportReportParamRepository.save(list);
        }else if( list.size()== 2 ){
            ExportReportParam param_3 = new ExportReportParam();
            param_3.setName("推广活动详情-活动组");
            param_3.setOpened(0);
            param_3.setApp(app);
            param_3.setCreateAccount(accountId);
            param_3.setCreateTime(new Date());
            param_3.setExportName("campaigninfo_bycampgroup_campgroup");
            list.add(param_3);
            exportReportParamRepository.save(param_3);
        }

        return list;
    }

    /**
     * 更新下载信息
     */
    @Override
    public int updateDownloadInfo(Long downloadId, Account account, String downloadIp) {

        return repository.updateDownloadInfo(downloadId, account.getId(), account.getName(), downloadIp, new Date());
    }
}
