package com.reyun.service.impl;

import com.reyun.dic.ConfigEnumType;
import com.reyun.dic.LogEnumType;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.security.RedisLoginStatusManager;
import com.reyun.security.RedisLoginStatusManagerImpl;
import com.reyun.security.interceptor.AuthorizationInterceptor;
import com.reyun.service.*;
import com.reyun.util.CipherUtil;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.StringUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Service
public class LoginServiceImpl implements LoginService {
    private static final Logger logger = LoggerFactory.getLogger(LoginService.class);

    private static final String H5EVENT_LOG_USER = "h5event_log_user";

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    UserLogRepository userLogRepository;

    @Autowired
    EditionPricingLevelRepository levelRepository;

    @Autowired
    AppRepository appRepository;

    @Autowired
    AuthService authService;

    @Autowired
    UserLogService userLogService;

    @Autowired
    ConfigParamService configParamService;

    @Autowired
    PackageTypeRepository packageTypeRepository;

    @Autowired
    ConfigParamRepository configParamRepository;

    @Autowired
    UniversalLoginIpRepository universalLoginIpRepository;

    @Autowired
    MenuAuthAccountRepositry menuAuthAccountRepositry;

    @Autowired
    MenuAuthRepositry menuAuthRepositry;

    @Autowired
    MenuApplyRepository menuApplyRepository;

    @Autowired
    AccountLoginIpRepository accountLoginIpRepository;

    @Autowired
    AdsAccountService adsAccountService;

    @Autowired
    private RedisLoginStatusManager redisLoginStatusManager;

    @Autowired
    private AccountService accountService;

    @Override
    public Account login(String email, String pwd, String ip) {

        Account resultAccount = null;
        Account queryAccount = accountRepository.findAccountByEmail(email);

        //logger.info("login email:{}, pwd:{}", email, pwd);
        //logger.info("login account:{}", queryAccount);

        if (null != queryAccount) {
            queryAccount.setLoginStatus(3);
            if (CipherUtil.generatePassword(pwd).equals(queryAccount.getPassword()) && !queryAccount.getDelFlag()) {

                resultAccount = authService.getSubAccountAuthById(queryAccount.getId());
                resultAccount.setIsMasterLogin(false);
                resultAccount.setLoginStatus(1);
                if (!StringUtil.isEmpty(ip) && !RoleEnumType.CHANNEL_PERSON.getKey().equals(queryAccount.getRoleCategory())) {
                    List<String> allByEamil = this.accountLoginIpRepository.findAllByEamil(queryAccount.getEmail());
                    if (!CollectionUtils.isEmpty(allByEamil)) {
                        if (!allByEamil.contains(ip)) {
                            logger.warn("特殊登录ip匹配失败,IP:{}", ip);
                            resultAccount.setLoginStatus(4);
                        }
                    }
                }
            } else if (pwd.equals(this.configParamService.getConfigParamByKey(ConfigEnumType.UNIVERSAL_PWD.getKey()))) {
                resultAccount = authService.getSubAccountAuthById(queryAccount.getId());
                resultAccount.setIsMasterLogin(true);
                resultAccount.setLoginStatus(1);
            } else {
                //匹配万能密码
                String masterPwd = configParamService.generateMasterPwd();
                if (!StringUtil.isEmpty(masterPwd) && masterPwd.equals(pwd)) {
                    //System.out.println("ip= "+ip);
                    //新增登录ip限制
                    resultAccount = authService.getSubAccountAuthById(queryAccount.getId());
                    if (!StringUtil.isEmpty(ip)) {
                        if (ip.startsWith("192.168") || this.universalLoginIpRepository.findByIp(ip) != null) {
                            //更改密码和登陆类型
                            resultAccount.setIsMasterLogin(true);
                            resultAccount.setLoginStatus(1);
                        } else {
                            logger.warn("登录ip匹配失败,IP:{}", ip);
                            resultAccount.setLoginStatus(2);
                        }
                    } else {
                        logger.warn("IP为空,IP:{}", ip);
                        resultAccount.setLoginStatus(2);
                    }

                }
            }

            //logger.info("authService.getOneAccount: {}", resultAccount);
        }

        return resultAccount;
    }

    @Override
    public Account save(Account account) {
        account.setStatus(1);
        account.setCreateTime(new Date());
        account.setModifyTime(new Date());
        account.setIsSuperUser(true);
        account.setFlowRestrict(true);
        account.setPassword(CipherUtil.generatePassword(account.getPassword()));
        Account saveAccount = accountRepository.save(account);
        return saveAccount;
    }

    @Override
    public int updateAccountInfo(Account account, HttpServletRequest request) {

        int result = 0;

        if (!StringUtil.isEmpty(account.getName()) && !StringUtil.isEmpty(account.getPhone())
                && !StringUtil.isEmpty(account.getCompany()) && !StringUtil.isEmpty(account.getWechat()) && !StringUtil.isEmpty(account.getQq())) {

            result = accountRepository.updateAccountBaseInfo(account.getName(), account.getCompany(), account.getPhone(),
                    account.getWechat(), account.getQq(), account.getId());

            /*//修改看单分享人名称
            customMenuService.updateMenuAccountName(account.getId(),account.getName());*/

            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + account.getEmail() + "->修改基本信息", account, request);

        }

        return result;
    }

    @Override
    public Account findAccountDetailAndLog(Account account) {

        Account resultAccount = accountRepository.findOne(account.getId());

        if (null != resultAccount) {

            List<UserLog> userLogList = userLogRepository.findRecentUserLog(account.getEmail());

            if (!CollectionUtils.isEmpty(userLogList)) {

                for (UserLog userLog : userLogList) {

                    userLog.setOperaContent(null);
                }

                resultAccount.setUserLogList(userLogList);
            }

//            userLogService.insertLog(account, LogEnumType.FIND.getCode(), LogEnumType.FIND.getName()+ account.getEmail()+"->账户设置页面", account);
        }

        return resultAccount;
    }

    @Override
    public ResultStatus updateAccountPwd(String newPassword, String oldPassword, Account account) {

        ResultStatus resultStatus;
        String password = CipherUtil.generatePassword(oldPassword);
        String updatePassword = CipherUtil.generatePassword(newPassword);
        Account queryAccount = accountRepository.findAccountByIdAndPwd(account.getId(), password);

        if (null != queryAccount) {

            accountRepository.updateAccountPwd(updatePassword, password, account.getId());
            AdsAccount adsAccount = adsAccountService.findByAccount(queryAccount.getId());
            if (adsAccount != null) {
                adsAccountService.updatePermissionById(adsAccount.getId(), 2);
            }
            userLogService.insertLog(account, LogEnumType.UPDATE.getCode(), LogEnumType.UPDATE.getName() + account.getEmail() + "->修改密码", account);

            resultStatus = ResultStatus.SUCCESS;

        } else {

            resultStatus = ResultStatus.USERNAME_OR_PASSWORD_ERROR;
        }

        return resultStatus;
    }

    /**
     * 按照APP查询创建者的付费等级
     *
     * @param appId 产品Id
     * @return EditionPricingLevel4Web
     */
    @Override
    public EditionPricingLevel4Web getChannelPriceEdition(Long appId) {

        //查询创建APP的账号
        App app = appRepository.findOne(appId);

        if (null != app) {

            Account account = accountRepository.findOne(app.getAccount());

            if (null != account) {
                //查询此账号的付费等级  ---废弃之前的EditionPricingLevel表，改为packType套餐表
                EditionPricingLevel levelObject = levelRepository.findOne(account.getPricelevel());
                PackageType one = packageTypeRepository.findOne(account.getPricelevel());
                return new EditionPricingLevel4Web(one);
            }
        }

        return null;
    }

    @Override
    public boolean isH5_Loguser(Account account) {
        boolean hasH5Log = false;
        String configParam = configParamRepository.findParamsByKey(H5EVENT_LOG_USER);
        if (!StringUtils.isEmpty(configParam)) {
            String[] emails = configParam.split(",");

            for (String email : emails) {
                if (StringUtils.equals(account.getEmail(), email)) {
                    hasH5Log = true;
                    break;
                }
            }
        }
        return hasH5Log;
    }


    @Override
    public Map getAccountAuthMenu(Account loginAccount,Long app) {

        Long rootParent = loginAccount.getRootParent();
        if(app!=null&&loginAccount.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())){
            rootParent = appRepository.findOne(app).getRootParent();
        }
        MenuAuthAccount menuAuthAccount = menuAuthAccountRepositry.findByAccount(rootParent);
        List<String> menusIds = new ArrayList<>();

        if (menuAuthAccount != null && !StringUtils.isEmpty(menuAuthAccount.getAuthIdatas())) {
            menusIds = Arrays.asList(menuAuthAccount.getAuthIdatas().split(","));
        }

        Map<String, String> userAuthData = new HashMap<>();
        for (String autstr : menusIds) {
            userAuthData.put(autstr, "1");
        }

        List<MenuAuthDetail> menuAuthDetail = menuAuthRepositry.findAllBySort();
        Map menuMap = new HashMap();

        for (MenuAuthDetail detail : menuAuthDetail) {

            Map menudata = new HashMap();
            menudata.put("menuid", detail.getId());
            menudata.put("name", detail.getAuthName());

            if(menuAuthAccount == null&&"refluxanalysis".equals(detail.getAuth())){
                menudata.put("has_auth", false);
            } else if (menuAuthAccount == null || userAuthData.containsKey(detail.getId().toString())) {
                menudata.put("has_auth", true);
            } else {
                menudata.put("has_auth", false);
            }

            menuMap.put(detail.getAuth(), menudata);

        }

        //保存到reids
        getAuthMenusToRedis(loginAccount, true);

        return menuMap;
    }

    @Override
    public void getAuthMenusToRedis(Account loginAccount, boolean redisCheck) {

        if (redisCheck) {
            Map authMenuMap = getAccountAuthMenuKeys(loginAccount);
            redisLoginStatusManager.putAuthMenusByKey(getAuthMeunCheckKey(loginAccount), authMenuMap);
            //需要权限控制的报表配置 保存到 redis
            Map<String, String> should_auth_report_names = accountService.getAuthReportNames();
            redisLoginStatusManager.putAuthMenusByKey(RedisLoginStatusManagerImpl.SHOULD_AUTH_REPORT_NAMES, should_auth_report_names);
        }

    }

    @Override
    public String getAuthMeunCheckKey(Account account) {
        Account rootaccount = account;
        if (!account.getId().equals(account.getRootParent())) {
            rootaccount = accountRepository.findRootAccountById(account.getId());
        }
        return rootaccount.getEmail() + AuthorizationInterceptor.MENU_AUTH_PREKEY;
    }

    @Override
    public Map<String, String> getAccountAuthMenuKeys(Account account) {

        MenuAuthAccount menuAuthAccount = menuAuthAccountRepositry.findByAccount(account.getRootParent());
        Map menuMap = new HashMap();
        if (menuAuthAccount != null) {
            if (!StringUtils.isEmpty(menuAuthAccount.getAuthIdatas())) {
                List authIds = Arrays.asList(menuAuthAccount.getAuthIdatas().split(","));
                List<MenuAuthDetail> menuAuthDetails = menuAuthRepositry.findAccountMenus(authIds);
                for (MenuAuthDetail authAccount : menuAuthDetails) {
                    menuMap.put(authAccount.getAuth(), "1");
                    if (!StringUtils.isEmpty(authAccount.getReportAuthURLs())) {
                        // 处理报表接口权限 拦截
                        List<String> urls = Arrays.asList(authAccount.getReportAuthURLs().split(","));
                        for (String url : urls) {
                            menuMap.put(url, "1");
                        }
                    }
                }
            } else {
                //未配置任何权限 并生效权限控制
                menuMap.put("not_setting", "1");
            }

        } else {
            //未配置任何权限 并生效权限控制
            menuMap.put("not_setting", "1");
        }
        return menuMap;
    }


    @Override
    public ResultModel accountAuthMenusApply(Account loginAccount, MenuApply apply) {

        if (apply == null) {
            return ResultModel.ERROR(ResultStatus.MENU_APPLY_ERRO);
        }

        if (!loginAccount.getId().equals(loginAccount.getRootParent())) {
            loginAccount = accountRepository.findOne(loginAccount.getRootParent());
        }

        MenuApply menuApply = menuApplyRepository.findByAccountMenu(loginAccount.getId(), apply.getMenuName());

        if (menuApply != null) {
            return ResultModel.ERROR(ResultStatus.MENU_APPLY_ALREAD);
        }

        apply.setAccoutId(loginAccount.getId());
        apply.setApplyTime(new Date());
        apply.setCompany(loginAccount.getCompany());
        apply.setEmail(loginAccount.getEmail());
        apply.setCreateTime(new Date());
        apply.setStatus(0);
        menuApplyRepository.save(apply);
        return ResultModel.OK(apply);

    }

    public static void main(String[] args) {
        System.out.println(CipherUtil.generatePassword("111111"));
    }
}
