package com.reyun.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.reyun.controller.manage.MngMiniProgramController;
import com.reyun.model.*;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.repository.CampaignRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.service.CampaignService;
import com.reyun.service.MiniProgramService;
import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.io.IOUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.*;

@SuppressWarnings("SpringAutowiredFieldsWarningInspection")
@Service
public class MiniProgramServiceImpl implements MiniProgramService {
    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(MngMiniProgramController.class);
    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;
    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";
    private static final String GET_ACCESS_TOKEN_URL = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=${user_appid}&secret=${user_secret}";
    private static final String GET_MINI_CODE_URL = "https://api.weixin.qq.com/wxa/getwxacode?access_token=${access_token}";
    private static final String GET_MINI_QR_CODE_URL = "https://api.weixin.qq.com/cgi-bin/wxaapp/createwxaqrcode?access_token=${access_token}";

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private CampaignService campaignService;

    @Autowired
    private ChannelRepository channelRepository;

    @Override
    public HashMap<Integer, String> generateAccessToken(App resource) {
        String url = GET_ACCESS_TOKEN_URL.replace("${user_appid}", resource.getMipAppid()).replace("${user_secret}", resource.getMipSecret());
        HttpClient httpClient = new HttpClient();
        GetMethod getMethod = new GetMethod(url);
        HashMap<Integer, String> map = new HashMap<>();
        try {
            httpClient.executeMethod(getMethod);
            String responseBody = getMethod.getResponseBodyAsString();

            if (getMethod.getStatusCode() == HttpStatus.SC_OK && responseBody != null && !"".equals(responseBody)) {
                JSONObject jsonObject = JSONObject.parseObject(responseBody);
                Integer errcode = jsonObject.getInteger("errcode");
                String access_token = jsonObject.getString("access_token");
                if (errcode != null) {
                    map.put(errcode, jsonObject.getString("errmsg"));
                } else {
                    map.put(200, access_token);
                }
            }
        } catch (IOException e) {
            logger.error(e.getMessage());
        } finally {
            getMethod.releaseConnection();
        }
        return map;
    }

    @Override
    public Boolean appidValid(Long loginAccount, String mipAppid) {
        Account one = accountRepository.findOne(loginAccount);
        List<App> bymipAppid = appRepository.findBymipAppid(one.getRootParent(), mipAppid);
        return bymipAppid.size() <= 0;
    }


    @Override
    public Map<Integer, String> getQrCode(App app, Long cid, String page, String type, String surl) {

        Map<Integer, String> tokenResult = this.generateAccessToken(app);

        if (tokenResult.containsKey(200)) {
            String accesstoken = tokenResult.get(200);
            app.setAccessToken(accesstoken);
            app.setTokenValid(true);
            appRepository.save(app);

            HashMap<Integer, String> map = new HashMap<>();
            String fileName = "mini_program" + "_" + app.getAppkey() + "_" + cid + "_" + type + "_" + new Date().getTime() + ".jpeg";
            String shenmaDownloadDir = Constant.SHENMA_DOWNLOAD_DIR;
            InputStream inputStream;
            OutputStream outputStream = null;
            HttpClient httpClient = new HttpClient(new MultiThreadedHttpConnectionManager());
            String postUrl = "";
            String requestBody = "";

            if ("miniCode".equals(type)) {
                postUrl = GET_MINI_CODE_URL.replace("${access_token}", accesstoken);
                if(page==null){
                    page="";
                }
                requestBody = "{" + "\"path\":\"" + page + "?" + "cid=" + cid + "&surl=" + surl + "\"" + "}";
            } else if ("miniQrCode".equals(type)) {
                postUrl = GET_MINI_QR_CODE_URL.replace("${access_token}", accesstoken);
                requestBody = "{" + "\"path\":\"" + page + "?" + "cid=" + cid + "&surl=" + surl + "\"" + "}";
            }

            try {
                PostMethod postMethod = new PostMethod(postUrl);
                StringRequestEntity requestEntity = new StringRequestEntity(requestBody, "application/json", "UTF-8");
                postMethod.setRequestEntity(requestEntity);
                try {
                    httpClient.executeMethod(postMethod);
                    if (postMethod.getStatusCode() == HttpStatus.SC_OK) {
                        Header responseHeader = postMethod.getResponseHeader("content-type");
                        if (responseHeader != null && !responseHeader.getValue().equals("image/jpeg")) {
                            String errorMessage = postMethod.getResponseBodyAsString();
                            logger.error("请求小程序码失败:\r\n" + "appKey=" + app.getAppkey() + "  cid=" + cid + "  errorMessage=" + errorMessage);
                            map.put(201, errorMessage);
                        } else {
                            inputStream = postMethod.getResponseBodyAsStream();
                            File file = new File(shenmaDownloadDir + fileName);
                            try {
                                outputStream = new FileOutputStream(file);
                                int len;
                                byte[] buf = new byte[1024];
                                while ((len = inputStream.read(buf, 0, 1024)) != -1) {
                                    outputStream.write(buf, 0, len);
                                    outputStream.flush();
                                }
                            } catch (IOException e) {
                                e.printStackTrace();
                                map.put(201, e.getMessage());
                                logger.error(e.getMessage());
                            } finally {
                                try {
                                    inputStream.close();
                                    if (outputStream != null) {
                                        outputStream.close();
                                    }
                                } catch (IOException e) {
                                    e.printStackTrace();
                                    logger.error(e.getMessage());
                                    map.put(201, e.getMessage());
                                }
                            }
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    map.put(201, e.getMessage());
                    logger.error(e.getMessage());
                }
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
                map.put(201, e.getMessage());
            }

            String filePath = Constant.SHENMA_DOWNLOAD_DIR + fileName;
            File file = new File(filePath);
            if (file.exists()) {
                try {
                    FileInputStream in = new FileInputStream(filePath);
                    final String s3key = DOWNLOAD_PATH + fileName;
                    ObjectStorageUtil.putObjectInStream(S3_DOWNLOAD_BUCKET, s3key, in, "jpeg", file.length());
                } catch (IOException e) {
                    e.printStackTrace();
                    logger.error(e.getMessage());
                    map.put(201, e.getMessage());
                }
            }
            map.put(200, fileName);
            return map;
        } else {
            Map<Integer, String> map = new HashMap<>();
            map.put(201, tokenResult.values().toString());
            return map;
        }
    }

    @Override
    public ResponseEntity<byte[]> downloadQrCode(Long id) {
        Campaign one = campaignRepository.findOne(id);

        final String s3key = DOWNLOAD_PATH + one.getQrCodeAddress();

        String fileName = one.getQrCodeAddress();

        InputStream in = null;
        byte[] content;
        String fileType = "image/jpeg";
        try {
            in = ObjectStorageUtil.getObjectContentStream(S3_DOWNLOAD_BUCKET, s3key);
            content = IOUtils.toByteArray(in);

            HttpHeaders headers = new HttpHeaders();
            headers.set("filename", fileName);
            headers.add("content-disposition", "attachment; filename=" + fileName);
            headers.add("Content-Type", fileType);
            return new ResponseEntity<>(content, headers, org.springframework.http.HttpStatus.CREATED);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            IOUtils.closeQuietly(in);
        }
        return null;
    }

    @Override
    public List<Channel> listChannel(Account loginAccount, Long appid, String type) {

        Iterable<Campaign4Web> result = campaignService.listAuthCampaignByType(loginAccount, appid, type, false);
        List<Campaign4Web> res = campaignService.listCampaignByLink(result);

        HashSet<Long> set = new HashSet<>();
        for (Campaign4Web campaign4Web : res) {
            if (campaign4Web.getChannel() != null) {
                Channel one = channelRepository.findOne(campaign4Web.getChannel());
                set.add(one.getId());
            }
        }

        ArrayList<Long> ids = new ArrayList<>(set);

        return channelRepository.findById(ids);
    }

    @Override
    public HashMap<Integer, String> checkAppValid(Long appid, Account loginAccount) {
        HashMap<Integer, String> map = new HashMap<>();
        Integer integer = campaignRepository.findbyMipType("miniQrCode", appid);
        if (integer > 100000) {
            map.put(211, "创建的小程序二维码数量超过限制(100000条)");
        }
        App one = appRepository.findOne(appid);
        if (!(one.getTokenValid() != null && one.getTokenValid())) {
            map.put(212, one.getTokenMessage()==null?"token无效":one.getTokenMessage());
        }
        return map;
    }

    @Override
    public ResponseEntity<byte[]> impression(Long id) {
        Campaign one = campaignRepository.findOne(id);

        final String s3key = DOWNLOAD_PATH + one.getQrCodeAddress();
        String fileName = one.getQrCodeAddress();
        InputStream in = null;
        byte[] content;
        String fileType = "image/jpeg";
        try {
            in = ObjectStorageUtil.getObjectContentStream(S3_DOWNLOAD_BUCKET, s3key);
            content = IOUtils.toByteArray(in);
            HttpHeaders headers = new HttpHeaders();
            headers.set("filename", fileName);
            headers.add("Content-Type", fileType);
            return new ResponseEntity<>(content, headers, org.springframework.http.HttpStatus.CREATED);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            IOUtils.closeQuietly(in);
        }
        return null;
    }

    public static void main(String[] args) {
        MiniProgramServiceImpl miniProgramService = new MiniProgramServiceImpl();
        App app = new App();
        String accessToken = "11_XUyUQgOuNS_ByVexHUmGnEbFB7liYtjTq8XA68Xx1fVSZruGGHsESOdP9C3hTI7UiJCJKhpNNHXx9hMH9FCwm6s0xZr4gOLasRS8asYIExANhxlJd3xiGST1odOToLskTiBrC3ecZJjqpsahWVZgADAGFJ";
        String appkey = "e6e155dfd3ab045322caf94439f6bd9d";
        app.setAccessToken(accessToken);
        app.setAppkey(appkey);
        Map<Integer, String> miniCode = miniProgramService.getQrCode(app, 514L, "pages/index/index", "miniCode", "fy6R3ea");
        System.out.println(miniCode);
        Map<Integer, String> miniQrCode = miniProgramService.getQrCode(app, 514L, "pages/index/index", "miniQrCode", "fy6R3ea");
        System.out.println(miniQrCode);
    }
}
