package com.reyun.service.impl;

import com.google.common.base.Predicate;
import com.google.common.base.Strings;
import com.google.common.collect.Maps;
import com.reyun.model.App;
import com.reyun.model.AppDeepLink;
import com.reyun.model.SurlModel;
import com.reyun.service.INoSQLDBService;
import com.reyun.util.BeanUtils;
import com.reyun.util.NoSQLDBUtil;
import com.reyun.util.NoSQLDBUtil.Result;
import com.reyun.util.NoSQLDBUtil.UpdateResult;
import com.reyun.util.ValidateUtil;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * Created by nolan on 24/12/2016.
 * description:
 */
@Service
public class NoSQLDBServiceImpl implements INoSQLDBService {

    @Override
    public boolean create(SurlModel surlModel) {
        return putItem(surlModel);
    }

    @Override
    public boolean update(SurlModel surlModel) {
        return putItem(surlModel);
    }

    @Override
    public boolean delete(String surl) {
        return NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TABLE_SURL, "surl", surl);
    }

    @Override
    public boolean deleteVivoParam(String appkey, String campaignid) {
        return NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TKIO_VIVO_PARAM, "id", appkey + ":" + campaignid);
    }

    public boolean delete(List<String> surlList) {
        if (CollectionUtils.isEmpty(surlList)) {
            return false;
        }
        return NoSQLDBUtil.batchDeleteSurl(surlList);
    }

    @Override
    public SurlModel findBySurl(String surl) {
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_SURL, "surl", surl);
        if (ValidateUtil.isValid(resultItem)) {
            return BeanUtils.map2Bean(resultItem.asMap("surl"), SurlModel.class);
        }
        return null;
    }

    /**
     * 创建app sdk 测试的ITEM
     *
     * @param appKey APP 32位码
     * @param
     * @return 是否创建成功
     */
    @Override
    public boolean createAppSdkDebugItem(String appKey, App app) {
        if (!haveAppDebugItem(appKey)) {
            Map<String, Object> debugItem = new HashMap<>();
            debugItem.put("appid", appKey);
            //isDebug 改为字符串
            debugItem.put("isDebug", app.getIsDebug().toString());
            debugItem.put("ry_appname", app.getName());
            debugItem.put("num", 0);
            return putAppDebugItem(debugItem);
        } else {
            return false;
        }
    }

    /**
     * 根据APP KEY 获取ITEM
     *
     * @param appKey APP 32位码
     */
    @Override
    public boolean haveAppDebugItem(String appKey) {
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TKIO_DEBUG, "appid", appKey);
        return ValidateUtil.isValid(resultItem);
    }

    /**
     * 根据APP KEY更新item
     *
     * @param appKey  APP 32位码
     * @param isDebug 是否是测试true false
     * @return 是否更新成功
     */
    @Override
    public boolean updateAppSdkDebugItem(String appKey, Boolean isDebug) {
        //FIXME: replace the following two find&update ops w/  findAndUpdate()
        if (this.haveAppDebugItem(appKey)) {
            return NoSQLDBUtil.updateItem(NoSQLDBUtil.TBL_TKIO_DEBUG, "appid", appKey,
                    new UpdateResult().withString("isDebug", isDebug.toString()));
        } else {
            return false;
        }
    }

    /**
     * 删除app的item
     *
     * @param appKey APP 32位码
     * @return 是否删除成功
     */
    @Override
    public boolean deleteAppSdkDebugItem(String appKey) {
        return NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TKIO_DEBUG, "appid", appKey);
    }

    private boolean putItem(SurlModel surlModel) {
        if (Strings.isNullOrEmpty(surlModel.getSurl()))
            throw new IllegalArgumentException("surl不允许为空");
        if (Strings.isNullOrEmpty(surlModel.getCampaignid()))
            throw new IllegalArgumentException("campaignid不允许为空");

        Map<String, Object> attributes = BeanUtils.bean2Map(surlModel);
        final Predicate<Object> valuePredicate = new Predicate<Object>() {
            @Override
            public boolean apply(Object val) {
                return val != null && !Strings.isNullOrEmpty(String.valueOf(val));
            }
        };
        attributes = Maps.filterValues(attributes, valuePredicate);
        return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TABLE_SURL, Result.fromMap(attributes, "surl"));
    }

    /**
     * 新建app Debug 的item
     *
     * @param appDebugItem 测试对象
     * @return 是否新建成功
     */
    private boolean putAppDebugItem(Map<String, Object> appDebugItem) {
        return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TKIO_DEBUG, Result.fromMap(appDebugItem, "appid"));
    }

    @Override
    public boolean updateSettingNewTable(String tableName, Map attributes) {
        final Predicate<Object> valuePredicate = new Predicate<Object>() {
            @Override
            public boolean apply(Object val) {
                return val != null && !Strings.isNullOrEmpty(String.valueOf(val));
            }
        };
        attributes = Maps.filterValues(attributes, valuePredicate);
        return NoSQLDBUtil.writeItem(tableName, Result.fromMap(attributes, "id"));
    }

    @Override
    public boolean deleteFromSettingNewTable(String tableName, String primaryKeyName, String primaryKeyValue) {
        return NoSQLDBUtil.deleteItem(tableName, primaryKeyName, primaryKeyValue);
    }

    /**
     * 功能描述:createOrUpdate开头的方法都是写到ddb的tkio_deeplink的数据，由于trackingio业务活动与app维度原因，将先查询，如果有就修改，没有就新增,scheam没有是吗变化的参数，所以不需要修改
     *
     * @author liyin
     * @date 2019/5/30
     */
    @Override
    public void createOrUpdateAndroidAppLinks(AppDeepLink appDeepLink) {
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_DEEPLINK, "linkid",
                appDeepLink.getLinkId());
        if (!ValidateUtil.isValid(resultItem)) {
            NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TABLE_DEEPLINK,
                    new Result().withPrimaryKey("linkid", appDeepLink.getLinkId())
                            .withString("sha256", appDeepLink.getSha256())
                            .withString("package_name", appDeepLink.getPackageName())
                            .withString("flag", appDeepLink.getFlag()));
        } else {
            NoSQLDBUtil.updateItem(NoSQLDBUtil.TBL_TABLE_DEEPLINK, "linkid", appDeepLink.getLinkId(),
                    new UpdateResult().withString("sha256", appDeepLink.getSha256()).withString("package_name",
                            appDeepLink.getPackageName()));
        }
    }

    @Override
    public void createOrUpdateIOSUniversalLinks(AppDeepLink appDeepLink) {
        List<String> pathsNew = new ArrayList<>();
        List<String> paths = Arrays.asList(appDeepLink.getPaths().split(","));
        for (String path : paths) {
            path = "/ry/" + path;
            pathsNew.add(path);
        }
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_DEEPLINK, "linkid",
                appDeepLink.getLinkId());
        if (!ValidateUtil.isValid(resultItem)) {
            NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TABLE_DEEPLINK,
                    new Result().withPrimaryKey("linkid", appDeepLink.getLinkId())
                            .withString("app_id_prefix", appDeepLink.getAppIdPrefix())
                            .withString("bundle_id", appDeepLink.getBundleId())
                            .withString("flag", appDeepLink.getFlag()).withList("paths", pathsNew));
        } else {
            NoSQLDBUtil.updateItem(NoSQLDBUtil.TBL_TABLE_DEEPLINK, "linkid", appDeepLink.getLinkId(),
                    new UpdateResult().withString("app_id_prefix", appDeepLink.getAppIdPrefix())
                            .withString("bundle_id", appDeepLink.getBundleId()).withList("paths", pathsNew));
        }
    }

    @Override
    public void updateIOSPaths(AppDeepLink appDeepLink) {
        List<String> pathsNew = new ArrayList<>();
        List<String> paths = Arrays.asList(appDeepLink.getPaths().split(","));
        for (String path : paths) {
            path = "/ry/" + path;
            pathsNew.add(path);
        }
        NoSQLDBUtil.updateItem(NoSQLDBUtil.TBL_TABLE_DEEPLINK, "linkid", appDeepLink.getLinkId(),
                new NoSQLDBUtil.UpdateResult().withList("paths", pathsNew));
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    @Override
    public boolean addVivoParamDatas(Map data, String appkey, String campaignid) {
        return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TKIO_VIVO_PARAM,
                new Result().withPrimaryKey("id", appkey + ":" + campaignid).withMap("value", data));
    }

    @Override
    public boolean sycDDbByBundleId(App source, boolean isAdd) {
        if (StringUtils.isEmpty(source.getBundleid())) {
            // 不需要同步
            return true;
        }

        if (isAdd) {
            return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TKIO_BUNDLEID, new Result()
                    .withPrimaryKey("bundleid", source.getBundleid()).withString("appid", source.getAppkey()));
        } else {
            return NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TKIO_BUNDLEID, "bundleid", source.getBundleid());
        }
    }
}
