package com.reyun.service.impl;

import com.google.common.base.Predicate;
import com.google.common.base.Strings;
import com.google.common.collect.Maps;
import com.reyun.model.OnelinkSurlModel;
import com.reyun.service.OnelinkINoSQLDBService;
import com.reyun.util.BeanUtils;
import com.reyun.util.NoSQLDBUtil;
import com.reyun.util.ValidateUtil;
import com.reyun.util.NoSQLDBUtil.Result;

import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * Created by nolan on 24/12/2016.
 * description:
 */
@Service
public class OnelinkNoSQLDBServiceImpl implements OnelinkINoSQLDBService {

    @Override
    public boolean create(OnelinkSurlModel surlModel) {
        return putItem(surlModel);
    }


    @Override
    public boolean update(OnelinkSurlModel surlModel) {
        return putItem(surlModel);
    }

    @Override
    public boolean delete(String surl) {
        return NoSQLDBUtil.deleteItem(NoSQLDBUtil.TBL_TABLE_SURL, "surl", surl);
    }

    @Override
    public OnelinkSurlModel findBySurl(String surl) {
        Result resultItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_SURL, "surl", surl);
        if (ValidateUtil.isValid(resultItem)) {
            return BeanUtils.map2Bean(resultItem.asMap("surl"), OnelinkSurlModel.class);
        }
        return null;
    }

    private boolean putItem(OnelinkSurlModel surlModel) {
        if (Strings.isNullOrEmpty(surlModel.getSurl()))
            throw new IllegalArgumentException("surl不允许为空");

        Map<String, Object> attributes = BeanUtils.bean2Map(surlModel);
        final Predicate<Object> valuePredicate = new Predicate<Object>() {
            @Override
            public boolean apply(Object val) {
                return val != null && !Strings.isNullOrEmpty(String.valueOf(val));
            }
        };
        attributes = Maps.filterValues(attributes, valuePredicate);
        return NoSQLDBUtil.writeItem(NoSQLDBUtil.TBL_TABLE_SURL, Result.fromMap(attributes, "surl"));
    }
}
