package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Strings;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AppService;
import com.reyun.service.OnelinkINoSQLDBService;
import com.reyun.service.PackageSubCampaignService;
import com.reyun.util.*;
import org.apache.commons.io.IOUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.util.*;

@Service
public class OnelinkServiceImpl implements OnelinkService {
    private static final Logger logger = LoggerFactory.getLogger(OnelinkServiceImpl.class);

    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    @Autowired
    AppRepository appRepository;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    OnelinkINoSQLDBService onelinkINoSQLDBService;

    @Autowired
    OnelinkRepository onelinkRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    AppService appService;

    @Autowired
    private PackageSubCampaignService packageSubCampaignService;

    @Override
    public Boolean validName(Long appid, String name, Long id) {
        Boolean flag = true;
        App one = appRepository.findOne(appid);
        List<App> apps = appRepository.findByAccount(one.getAccount());
        List<Long> appids = new ArrayList<>();
        for (App app : apps) {
            appids.add(app.getId());
        }
        List<Onelink> list = onelinkRepository.findByMainApps(appids);
        for (Onelink onelink : list) {
            if (id == null) {
                if (onelink.getName().equals(name)) {
                    flag = false;
                }
            }
            if (!onelink.getId().equals(id)) {
                if (onelink.getName().equals(name)) {
                    flag = false;
                }
            }
        }
        return flag;
    }

    @Override
    public int validAppleId(Long account, Long appid) {

        int flag = 0;
        App app = appRepository.findAppFromOnelink(appid);

        List<App> apps = appService.listAppByAccount(account);
        List<String> appPlats = new ArrayList<>();
        for (App a : apps) {
            if (!appPlats.contains(a.getPlatform())) {
                appPlats.add(a.getPlatform());
            }
        }
        if (!appPlats.contains("iOS") || !appPlats.contains("Android")) {
            flag = 3;
            return flag;
        }

        if (app != null) {
            flag = 1;
            if (app.getPlatform().equals("iOS")) {
                List<App> androids = listApps(account, "Android", "zh");
                if (ValidateUtil.isValid(androids)) {
                    for (App a : androids) {
                        if (a.getPlatform() != null)
                            if (a.getPlatform().equals("Android")) {
                                flag = 2;
                            }
                    }
                }
            } else {
                List<App> ioss = listApps(account, "iOS", "zh");
                if (ValidateUtil.isValid(ioss)) {
                    for (App a : ioss) {
                        if (a.getPlatform() != null)
                            if (a.getPlatform().equals("iOS")) {
                                flag = 2;
                            }
                    }
                }
            }
        }

        return flag;
    }

    @Override
    public List<App> listApps(Long account, String platform, String lanType) {

        List<App> apps = new LinkedList<>();

        List<App> appsList = appService.listAppByAccount(account);
        List<Long> appids = new ArrayList<>();

        for (App a : appsList) {
            appids.add(a.getId());
        }

        apps = appRepository.findAppsFromOnelink(appids, platform);

        App app = new App();
        app.setId(0L);
        app.setName("请选择产品");

        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            app.setName("Please Select the App");
        }

        List<App> result = new LinkedList<>();

        result.add(app);

        if (ValidateUtil.isValid(apps)) {
            result.addAll(apps);
        }

        return result;
    }

    @Override
    public List<Channel> listChannels(Long appid, String lanType) {
        List<Channel> list = new ArrayList<>();
        list = channelRepository.listChannelFromOnelink(appid);

        Channel channel = new Channel();
        channel.setId(0L);
        channel.setName("请选择渠道");

        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            channel.setName("Please Select the Partner");
        }

        List<Channel> result = new LinkedList<>();
        result.add(channel);

        if (ValidateUtil.isValid(list)) {
            result.addAll(list);
        }

        return result;
    }

    @Override
    public List<Campaign4Web> listCampaigns(Long appid, Long channel, String lanType) {
        List<Campaign> list = new ArrayList<>();

        list = campaignRepository.findFromOnelink(appid, channel);

        Channel channel1 = channelRepository.findOne(channel);

        Campaign4Web campaign = new Campaign4Web();
        campaign.setId(0L);
        campaign.setName("请选择推广活动");
        if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
            campaign.setName("Please Select the Campaign");
        }

        List<Campaign4Web> result = new LinkedList<>();
        result.add(campaign);

        if (ValidateUtil.isValid(list)) {
            for (Campaign ca : list) {
                if ("APPMARKET".equals(ca.getChannelCategory()) && channel1.getType() == null & packageSubCampaignService.findSubCampaignNumber(ca.getId()) < 1) {
                    continue;
                }
                Campaign4Web ca4Web = new Campaign4Web();
                ca4Web.setId(ca.getId());
                ca4Web.setName(ca.getName());
                ca4Web.setSurl(ca.getSurl());
                ca4Web.setUrl(ca.getUrl());
                ca4Web.setTrackurl(Constant.trackurl + "/" + ca.getSurl());

                if ("ry_coop".equals(channel1.getType()) && channel1.isIshttps()) {
                    //常规渠道 https 显示时 返回 https 开头链接
                    ca4Web.setTrackurl(ca4Web.getTrackurl().replace("http:", "https:"));
                }

                if (null != channel1.getAndroidparam() && !channel1.getAndroidparam().isEmpty()) {
                    ca4Web.setAndroidParam(channel1.getAndroidparam());
                }
                if (null != channel1.getIosparam() && !channel1.getIosparam().isEmpty()) {
                    ca4Web.setIosParam(channel1.getIosparam());
                }

                if (!StringUtils.isEmpty(channel1.getOnelinkParam())) {
                    ca4Web.setOnelikParam(channel1.getOnelinkParam());
                }

                ca4Web.setType(channel1.getType());
                result.add(ca4Web);
            }

        }
        return result;
    }

    @Override
    public Onelink create(Long account, Onelink onelink) {
        Account accountone = accountRepository.findOne(account);
        onelink.setUrl(Constant.trackurl + "/" + onelink.getSurl());
        Channel channel = channelRepository.findById(onelink.getAndroidChannel());
        // 常规渠道配置 onelink 时 单独配置是否为 https
        if ("ry_coop".equals(channel.getType())) {

            if (channel.getOnelinkHttps() != null && channel.getOnelinkHttps()) {
                onelink.setUrl(onelink.getUrl().replace("http:", "https:"));
            }
            if (channel.getOnelinkSpecial() != null && channel.getOnelinkSpecial()) {
                // 设置常规渠道 宏参
                if (!StringUtils.isEmpty(onelink.getOnelinkParam())) {
                    onelink.setMainParam(onelink.getOnelinkParam());
                }

            } else {
                onelink.setMainParam(null);
            }
        }

        onelink.setCreateAccountName(accountone.getName());
        onelink.setCreateAccount(account);
        onelink.setCreateTime(new Date());
        onelink.setModifyTime(new Date());

        if (onelink.getQcodeEnable() != null && onelink.getQcodeEnable()) {
            // 生成二维码
            this.creatQcode(onelink);
        }

        Onelink save = onelinkRepository.save(onelink);

        if (Constant.awsneed) {
            if (onelink.getId() != null) {
                boolean isAwsSucces = true;
                try {
                    OnelinkSurlModel transform = new OnelinkSurlModel();
                    transform.setSurl(onelink.getSurl());
                    transform.setType("onelink");
                    transform.setAndroid_surl(onelink.getAndroidSurl());
                    transform.setIos_surl(onelink.getIosSurl());
                    transform.setDefault_url(onelink.getDefaultUrl());
                    //文强让cid存String类型
                    transform.setCid(channel.getId().toString());
                    if (!Strings.isNullOrEmpty(transform.getSurl())) {
                        isAwsSucces = onelinkINoSQLDBService.create(transform);
                        OnelinkSurlModel fetchRemoteItem = onelinkINoSQLDBService.findBySurl(onelink.getSurl());
                        if (fetchRemoteItem == null
                                || Strings.isNullOrEmpty(fetchRemoteItem.getSurl())
                                || Strings.isNullOrEmpty(fetchRemoteItem.getAndroid_surl())
                                || Strings.isNullOrEmpty(fetchRemoteItem.getIos_surl())
                        ) {
                            String contents = String.format("本地OnelinkSurlModel: %s;\n远程OnelinkSurlModel: %s.\n\nonelink: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(fetchRemoteItem), new ObjectMapper().writeValueAsString(onelink));
                            //MailUtils.sendSimpleEmail("onelink短链数据不一致(业务库与dynamodb)", contents, Constant.mlist);
                            SendCommonPostMail.sendMailReuse(Constant.mlist, "onelink短链数据不一致(业务库与dynamodb)", contents, null, null);
                        }
                    } else {
                        try {
                            String contents = String.format("本地OnelinkSurlModel: %s.\n\nonelink: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(onelink));
                            //MailUtils.sendSimpleEmail("OnelinkSurlModel信息缺失,无法同步Dynamodb", contents, Constant.mlist);
                            SendCommonPostMail.sendMailReuse(Constant.mlist, "OnelinkSurlModel信息缺失,无法同步Dynamodb", contents, null, null);
                        } catch (Exception e1) {
                            logger.error("fail to send email for failing to generat onelinksurl");
                        }
                    }
                } catch (Exception e) {
                    isAwsSucces = false;
                    e.printStackTrace();
                }
                save.setAwssuccess(isAwsSucces);
                save = onelinkRepository.save(save);
            }
        }
        return save;
    }

    public void creatQcode(Onelink onelink) {
        //生成二维码
        DateTime dateTime = new DateTime();
        String todayTimecode = dateTime.toString("yyyyMMdd");
        //固定格式 保证ddb存一份二维码图片
        String fileName = "onlink_qcode_" + todayTimecode + "_" + onelink.getIosSurl() + "_" + onelink.getSurl();
        try {

            InputStream inLogo = null;
            if (!StringUtils.isEmpty(onelink.getLogoFile())) {
                final String s3keyLogo = DOWNLOAD_PATH + onelink.getLogoFile();
                inLogo = ObjectStorageUtil.getObjectContentStream(Constant.bucket, s3keyLogo);
            }

            String code_logo_dir = Constant.SHENMA_DOWNLOAD_DIR + "onlink_qcode_logo_tmp";
            String coedeFileName = QRCodeUtils.encodeQcode(onelink.getUrl(), code_logo_dir, fileName, inLogo);
            onelink.setQcodeName(coedeFileName + ".jpg");
            String filePath = code_logo_dir + "/" + onelink.getQcodeName();
            File file = new File(filePath);
            if (file.exists()) {
                FileInputStream in = new FileInputStream(filePath);
                final String s3key = DOWNLOAD_PATH + onelink.getQcodeName();
                ObjectStorageUtil.putObjectInStream(Constant.bucket, s3key, in, "jpg", file.length());
                //上传完成  删除文件
                file.delete();
            }
        } catch (Exception e) {
            logger.error("onelink生成二维码错误", e);
        }
    }

    @Override
    public ResponseEntity<byte[]> downloadQrCode(Long id) {

        Onelink onelink = onelinkRepository.findOne(id);
        final String s3key = DOWNLOAD_PATH + onelink.getQcodeName();
        String fileName = onelink.getQcodeName();
        InputStream in = null;
        byte[] content;
        String fileType = "image/jpg";
        try {
            in = ObjectStorageUtil.getObjectContentStream(Constant.bucket, s3key);
            content = IOUtils.toByteArray(in);
            HttpHeaders headers = new HttpHeaders();
            headers.set("filename", fileName);
            headers.add("content-disposition", "attachment; filename=" + fileName);
            headers.add("Content-Type", fileType);
            return new ResponseEntity<>(content, headers, org.springframework.http.HttpStatus.CREATED);
        } catch (Exception e) {
            logger.error("s3key:" + s3key + " bucket:" + Constant.bucket, e);
        } finally {
            IOUtils.closeQuietly(in);
        }
        return null;
    }

    @Override
    public ResultModel codeLogoMake(MultipartFile file, Long appid, Account account, String onelinkSurl) {
        String filename = file.getOriginalFilename();
        String suffixName = filename.substring(filename.lastIndexOf(".") + 1);
        App app = appRepository.findOne(appid);
        String fileName = "logo_" + app.getAppkey() + "_" + account.getId() + "_" + onelinkSurl + "." + suffixName;
        try {
            final String s3key = DOWNLOAD_PATH + fileName;
            ObjectStorageUtil.putObjectInStream(Constant.bucket, s3key, file.getInputStream(), suffixName, file.getSize());
            Map data = new HashMap();
            data.put("url_path", fileName);
            return ResultModel.OK(data);
        } catch (Exception e) {

        }

        return ResultModel.ERROR(ResultStatus.FAILED);
    }


    @Override
    public Onelink update(Long account, Onelink onelink) {

        Onelink one = onelinkRepository.findOne(onelink.getId());

        one.setName(onelink.getName());
        one.setAndroidApp(onelink.getAndroidApp());
        one.setAndroidChannel(onelink.getAndroidChannel());
        one.setAndroidCampaignl(onelink.getAndroidCampaignl());
        one.setAndroidSurl(onelink.getAndroidSurl());
        one.setIosSurl(onelink.getIosSurl());
        one.setIosApp(onelink.getIosApp());
        one.setIosChannel(onelink.getIosChannel());
        one.setIosCampaignl(onelink.getIosCampaignl());
        one.setDefaultUrl(onelink.getDefaultUrl());
        one.setAndroidParam(onelink.getAndroidParam());
        one.setIosParam(onelink.getIosParam());
        one.setModifyAccount(account);
        one.setModifyTime(new Date());
        one.setQcodeEnable(onelink.getQcodeEnable());
        one.setLogoFile(onelink.getLogoFile());
        one.setPakageCampaignl(onelink.getPakageCampaignl());
        //更新二维码
        if (onelink.getQcodeEnable() != null && onelink.getQcodeEnable()) {
            // 生成二维码
            onelink.setUrl(one.getUrl());
            onelink.setSurl(one.getSurl());
            this.creatQcode(onelink);
            one.setQcodeName(onelink.getQcodeName());
        }

        Channel channel = channelRepository.findById(onelink.getAndroidChannel());
        if (!"ry_coop".equals(channel.getType())) {
            one.setMainParam(onelink.getMainParam());
        }

        Onelink save = onelinkRepository.save(one);

        OnelinkSurlModel transform = new OnelinkSurlModel();
        transform.setSurl(save.getSurl());
        transform.setType("onelink");
        transform.setAndroid_surl(save.getAndroidSurl());
        transform.setIos_surl(save.getIosSurl());
        transform.setDefault_url(save.getDefaultUrl());

        if (Constant.awsneed) {
            if (!this.onelinkINoSQLDBService.update(transform)) {
                save.setAwssuccess(false);
                save = onelinkRepository.save(save);
            }
        }

        return save;
    }

    @Override
    public Onelink findone(Long id) {

        Onelink one = onelinkRepository.findOne(id);

        App appAd = appRepository.findOne(one.getAndroidApp());
        Channel channelAd = channelRepository.findOne(one.getAndroidChannel());
        Campaign campaignAd = campaignRepository.findOne(one.getAndroidCampaignl());

        App appIos = appRepository.findOne(one.getIosApp());
        Channel channelIos = channelRepository.findOne(one.getIosChannel());
        Campaign campaignIos = campaignRepository.findOne(one.getIosCampaignl());

        one.setAndroidName(appAd.getName() + "-" + channelAd.getName() + "-" + campaignAd.getName());
        one.setIosName(appIos.getName() + "-" + channelIos.getName() + "-" + campaignIos.getName());

        return one;
    }

    @Override
    public List<Onelink> find(Long account, Long app) {
        List<Onelink> result = new ArrayList<>();

        List<App> appsList = appService.listAppByAccount(account);
        List<Long> appids = new ArrayList<>();

        List<Account> accounts = accountRepository.findAll();
        Map<Long, String> accountMap = new HashMap<>();
        if (ValidateUtil.isValid(accounts)) {
            for (Account ac : accounts) {
                accountMap.put(ac.getId(), ac.getName() == null ? "" : ac.getName());
            }
        }

        for (App a : appsList) {
            appids.add(a.getId());
        }

        List<Onelink> list = new ArrayList<>();
        list = onelinkRepository.findByApp(app);
        List<Long> appidList = new ArrayList<>();
        List<Long> chList = new ArrayList<>();
        List<Long> caList = new ArrayList<>();
        if (ValidateUtil.isValid(list)) {
            for (Onelink o : list) {

                appidList.add(o.getAndroidApp());
                appidList.add(o.getIosApp());

                chList.add(o.getAndroidChannel());
                chList.add(o.getIosChannel());

                caList.add(o.getAndroidCampaignl());
                caList.add(o.getIosCampaignl());
            }
            List<App> apps = appRepository.findByIds(appidList);
            List<Channel> channels = channelRepository.findById(chList);
            List<Campaign> campaigns = campaignRepository.findById(caList);

            Map<Long, String> appMap = new HashMap<>();
            Map<Long, Channel> channelMap = new HashMap<>();
            Map<Long, String> campaignMap = new HashMap<>();

            for (App a : apps) {
                appMap.put(a.getId(), a.getName());
            }
            for (Channel ch : channels) {
                channelMap.put(ch.getId(), ch);
            }
            for (Campaign ca : campaigns) {
                campaignMap.put(ca.getId(), ca.getName());
            }

            for (Onelink o : list) {

                o.setAndroidName(appMap.get(o.getAndroidApp()) + "/" + channelMap.get(o.getAndroidChannel()).getName() + "/" + campaignMap.get(o.getAndroidCampaignl()));
                o.setIosName(appMap.get(o.getIosApp()) + "/" + channelMap.get(o.getIosChannel()).getName() + "/" + campaignMap.get(o.getIosCampaignl()));

                if (o.getMainApp().equals(app)) {
                    o.setMain(true);
                    Long assistantApp = o.getMainApp().equals(o.getAndroidApp()) ? o.getIosApp() : o.getAndroidApp();
                    if (!appids.contains(assistantApp)) {
                        o.setSon(true);
                        o.setSonName(appMap.get(assistantApp));
                    }
                } else {
                    o.setMain(false);
                }
                o.setCreateAccountName(accountMap.get(o.getCreateAccount()));
                if (null != o.getMainParam()) {
                    o.setUrl(o.getUrl() + "?" + o.getMainParam());
                }
                if (null != o.getAndroidParam()) {
                    o.setAndroidName(o.getAndroidName() + "?" + o.getAndroidParam());
                }
                if (null != o.getIosParam()) {
                    o.setIosName(o.getIosName() + "?" + o.getIosParam());
                }

                result.add(o);
            }

        }
        return result;
    }

    @Override
    public Onelink delete(Long account, Long id) {
        Onelink one = onelinkRepository.findOne(id);

        // 2. 同步删除aws-dynamodb
        if (Constant.awsneed) {
            if (!this.onelinkINoSQLDBService.delete(one.getSurl())) {
                logger.error("删除dynamodbonelink短链失败. info: {}", one);
                return one;
            }
        }
        one.setDelFlag(true);
        one.setModifyTime(new Date());
        one.setModifyAccount(account);
        Onelink save = onelinkRepository.save(one);
        return save;
    }

    @Override
    public void taskSurlToDDb() {
        List<Onelink> list = onelinkRepository.findByDDb();

        if (ValidateUtil.isValid(list)) {
            for (Onelink onelink : list) {
                if (Constant.awsneed) {
                    if (onelink.getId() != null && !onelink.getAwssuccess()) {
                        boolean isAwsSucces = true;
                        try {
                            OnelinkSurlModel transform = new OnelinkSurlModel();
                            transform.setSurl(onelink.getSurl());
                            transform.setType("onelink");
                            transform.setAndroid_surl(onelink.getAndroidSurl());
                            transform.setIos_surl(onelink.getIosSurl());
                            transform.setDefault_url(onelink.getDefaultUrl());

                            if (!Strings.isNullOrEmpty(transform.getSurl())) {
                                isAwsSucces = onelinkINoSQLDBService.create(transform);
                                OnelinkSurlModel fetchRemoteItem = onelinkINoSQLDBService.findBySurl(onelink.getSurl());
                                if (fetchRemoteItem == null
                                        || Strings.isNullOrEmpty(fetchRemoteItem.getSurl())
                                        || Strings.isNullOrEmpty(fetchRemoteItem.getAndroid_surl())
                                        || Strings.isNullOrEmpty(fetchRemoteItem.getIos_surl())
                                ) {
                                    String contents = String.format("定时任务OnelinkSurlModel: %s;\n远程OnelinkSurlModel: %s.\n\nonelink: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(fetchRemoteItem), new ObjectMapper().writeValueAsString(onelink));
                                    //MailUtils.sendSimpleEmail("定时任务onelink短链数据不一致(业务库与dynamodb)", contents, Constant.mlist);
                                    SendCommonPostMail.sendMailReuse(Constant.mlist, "定时任务onelink短链数据不一致(业务库与dynamodb)", contents, null, null);
                                }
                            } else {
                                try {
                                    String contents = String.format("定时任务OnelinkSurlModel: %s.\n\nonelink: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(onelink));
                                    //MailUtils.sendSimpleEmail("定时任务OnelinkSurlModel信息缺失,无法同步Dynamodb", contents, Constant.mlist);
                                    SendCommonPostMail.sendMailReuse(Constant.mlist, "定时任务OnelinkSurlModel信息缺失,无法同步Dynamodb", contents, null, null);
                                } catch (Exception e1) {
                                    logger.error("fail to send email for failing to generat onelinksurl");
                                }
                            }
                        } catch (Exception e) {
                            isAwsSucces = false;
                            logger.error(e.getMessage(), e);
                        }
                        onelink.setAwssuccess(isAwsSucces);
                        onelinkRepository.save(onelink);
                    }
                }
            }
        }
    }

    @Override
    public long findOnelinkNumber(Long campaignid) {

        BigInteger number = onelinkRepository.countOnelinkByCampaign(campaignid);
        return (null != number) ? number.longValue() : 0L;
    }


    public static void main(String[] args) {

        String aa[] = "https://itunes.apple.com/cn/app/sha-ba-ke-chuan-qi-sheng-da/id1072126000?mt=8".split("//");

        System.out.println("https://" + aa[1]);
    }
}
