package com.reyun.service.impl;

import com.reyun.model.App;
import com.reyun.model.OpenApi;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.repository.OpenApiRepository;
import com.reyun.service.OpenApiWebService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.UUID;

@Service
public class OpenApiWebServiceImpl implements OpenApiWebService {

    @Autowired
    AppRepository appRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    OpenApiRepository openApiRepository;

    protected Logger logger = LoggerFactory.getLogger(OpenApiWebServiceImpl.class);


    @Override
    public OpenApi create(App app) {
        openApiRepository.delByAppkey(app.getAppkey());

        OpenApi openApi = new OpenApi();
        openApi.setAppkey(app.getAppkey());
        openApi.setAccessToken(UUID.randomUUID().toString().replace("-", ""));
        openApi.setRefreshToken(UUID.randomUUID().toString().replace("-", ""));
        openApi.setAccount(app.getRootParent());
        Date date = new Date();
        openApi.setRefreshTime(date);
        openApi.setAccessTokenExpires(new Date(date.getTime()+(ACCESS_TOKEN_EXPIRES_IN*1000)));
        openApi.setRefreshTokenExpires(new Date(date.getTime()+(REFRESH_TOKEN_EXPIRES_IN*1000)));
        OpenApi save = openApiRepository.save(openApi);
        return save;
    }

    @Override
    public OpenApi getByAppidAndRefreshToken(String appid, String refresh_token) {

        return openApiRepository.getByAppidAndRefreshToken(appid,refresh_token);
    }

    @Override
    public OpenApi refresh(OpenApi openApi) {
        Date date = new Date();
        openApi.setRefreshTime(date);
        openApi.setAccessToken(UUID.randomUUID().toString().replace("-", ""));
        openApi.setAccessTokenExpires(new Date(date.getTime()+(ACCESS_TOKEN_EXPIRES_IN*1000)));
        openApi.setRefreshTokenExpires(new Date(date.getTime()+(REFRESH_TOKEN_EXPIRES_IN*1000)));
        OpenApi save = openApiRepository.save(openApi);
        return save;
    }
}
