package com.reyun.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Function;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.exception.TipException;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.*;
import com.reyun.util.*;
import org.mortbay.util.UrlEncoded;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.*;
import java.math.BigInteger;
import java.util.*;

/**
 * Created by mxq on 2017/8/15.
 */
@Service
public class PackageSubCampaignServiceImpl implements PackageSubCampaignService {
    private static final Logger logger = LoggerFactory.getLogger(PackageSubCampaignServiceImpl.class);

    @Autowired
    PackageSubCampaignRepository packageSubCampaignRepository;

    @Autowired
    AppRepository appRepository;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    SubCampaignINoSQLService subCampaignINoSQLService;

    @Autowired
    CampaignService campaignService;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    private INoSQLDBService iNoSQLDBService;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    CampaignWeChatAccountRepository campaignWeChatAccountRepository;

    @Autowired
    CampaignWeChatLinkRepository campaignWeChatLinkRepository;
    @Autowired
    AppDeepLinkRepository appDeepLinkRepository;
    @Autowired
    AppDeepLinkWakeRepository appDeepLinkWakeRepository;

    @Autowired
    AppService appService;

    private final static String VIVO_API_SIGN_URL = "https://ad-market.vivo.com.cn/test/sign";

    @Override
    public List<PackageSubCampaign> listSubCampaign(Long campaignId) {
        List<PackageSubCampaign> list = packageSubCampaignRepository.listSubCampaignByCampaign(campaignId);
        Channel channel = channelRepository.findById(campaignRepository.findOne(campaignId).getChannel());
        String androidParam = channel.getAndroidparam();
        boolean isTrackUrlSpecial = channel.isIstrackurlspecial();
        boolean istrackUrlSpecailReal = false;
        boolean isTrackHttps = false;

        String impAndroidParam = channel.getImpAndroidParam();
        boolean isImpUrlSpecail = channel.isImpIsTrackUrlSpecial();
        boolean isImpUrlSpecailReal = false;
        boolean isImpHttps = false;

        if (isTrackUrlSpecial && !StringUtil.isEmpty(androidParam)) {
            istrackUrlSpecailReal = true;
        }

        if (isImpUrlSpecail && !StringUtil.isEmpty(impAndroidParam)) {
            isImpUrlSpecailReal = true;
        }

        if (channel.isIshttps()) {
            isTrackHttps = true;
        }
        if (channel.isImpIsHttps()) {
            isImpHttps = true;
        }
        List<PackageSubCampaign> res = new ArrayList<>();
        if (ValidateUtil.isValid(list)) {
            Long appid= list.get(0).getApp();
            AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(appid);
            App app = appRepository.findOne(appid);
            for (PackageSubCampaign sub : list) {
                String trackUrl = Constant.trackurl + "/" + sub.getSurl();

                if (istrackUrlSpecailReal) {
                    trackUrl = trackUrl + "?" + androidParam;
                }


                String impUrl = Constant.trackurl + "/" + sub.getSurl() + "?impression=true";
                if (isImpUrlSpecailReal) {
                    impUrl = impUrl + "&" + androidParam;
                }

                if (isTrackHttps) {
                    trackUrl = trackUrl.replace("http:", "https:");
                }

                if (isImpHttps) {
                    impUrl = impUrl.replace("http:", "https:");
                }
                sub.setTrackUrl(trackUrl);
                sub.setImpUrl(impUrl);
                if(appDeepLink!=null)
                disposeImpUrlAndTackUrl(sub,app,appDeepLink);
                res.add(sub);
            }
        }

        return res;
    }

    @Override
    public List<PackageSubCampaign> listDisableCampaign(Long campaignId) {

        List<PackageSubCampaign> list = packageSubCampaignRepository.listDisableByCampaign(campaignId);
        List<PackageSubCampaign> res = new ArrayList<>();
        if (ValidateUtil.isValid(list)) {
            String containsUrl = Constant.deeplinkHost;
            for (PackageSubCampaign sub : list) {
                if (sub.getUrl().contains(containsUrl)) {
                    sub.setDeeplink(true);
                }
                sub.setTrackUrl(Constant.trackurl + "/" + sub.getSurl());
                res.add(sub);
            }
        }

        return res;
    }

    @Override
    public PackageSubCampaign findSubCampaign(Long id) {
        PackageSubCampaign sub = packageSubCampaignRepository.findOne(id);

        Channel channel = channelRepository.findById(campaignRepository.findOne(id).getChannel());
        String androidParam = channel.getAndroidparam();
        boolean isTrackUrlSpecial = channel.isIstrackurlspecial();
        boolean istrackUrlSpecailReal = false;
        boolean isTrackHttps = false;

        String impAndroidParam = channel.getImpAndroidParam();
        boolean isImpUrlSpecail = channel.isImpIsTrackUrlSpecial();
        boolean isImpUrlSpecailReal = false;
        boolean isImpHttps = false;

        if (isTrackUrlSpecial && !StringUtil.isEmpty(androidParam)) {
            istrackUrlSpecailReal = true;
        }

        if (isImpUrlSpecail && !StringUtil.isEmpty(impAndroidParam)) {
            isImpUrlSpecailReal = true;
        }

        if (channel.isIshttps()) {
            isTrackHttps = true;
        }
        if (channel.isImpIsHttps()) {
            isImpHttps = true;
        }


        String trackUrl = Constant.trackurl + "/" + sub.getSurl();

        if (istrackUrlSpecailReal) {
            trackUrl = trackUrl + "?" + androidParam;
        }


        String impUrl = Constant.trackurl + "/" + sub.getSurl() + "?impression=true";
        if (isImpUrlSpecailReal) {
            impUrl = impUrl + "&" + androidParam;
        }

        if (isTrackHttps) {
            trackUrl = trackUrl.replace("http:", "https:");
        }

        if (isImpHttps) {
            impUrl = impUrl.replace("http:", "https:");
        }
        sub.setTrackUrl(trackUrl);
        sub.setImpUrl(impUrl);
        AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(sub.getApp());
        App app = appRepository.findOne(sub.getApp());
        disposeImpUrlAndTackUrl(sub,app,appDeepLink);
        return sub;
    }

    @Override
    public ResultModel batchSave(List<PackageSubCampaign> list, Account account) {
        for (PackageSubCampaign item : list) {
            Long parentId = item.getParent();
            String name = item.getName();
            if (packageSubCampaignRepository.exist(parentId, name) > 0) {
                return new ResultModel(ResultStatus.PACKAGE_SUB_CAMPAIGN_REPEAT, name);
            }
        }

        List<PackageSubCampaign> rlist = new ArrayList<>();
        for (PackageSubCampaign item : list) {
            rlist.add(save(item, account));
        }
        return ResultModel.OK(rlist);
    }

    @Override
    public PackageSubCampaign save(PackageSubCampaign source, Account account) {
        source.setAccount(account.getId());
        source.setCreateAccount(account.getId());
        source.setCreateTime(new Date());
        App app = appRepository.findOne(source.getApp());
        //不用考虑特殊渠道比如微信mp等活动创建，前端控制不传deeplink对象即可
        AppDeepLink appDeepLink = source.getAppDeepLink();
        //保存deeplink配置，安卓或ios，deeplink对象不为空
        if (appDeepLink != null && (app.getPlatform().toLowerCase().equals("ios") || app.getPlatform().toLowerCase().equals("android"))) {
            //启用deeplink则维护父活动的deeplink的标识字段
            Campaign campaign = campaignRepository.findCampaignByCampaignId(source.getParentCampaignid(), source.getApp());
            if(!campaign.getDeeplink()){
                campaignRepository.updateDeeplinkByCampaignId(source.getParentCampaignid(),true);
            }
            //启用deeplink的活动(app的)appkey存库给实时使用
            AppDeepLinkWake wake = appDeepLinkWakeRepository.findByAppkey(app.getAppkey());
            if (wake == null) {
                AppDeepLinkWake linkWake = new AppDeepLinkWake();
                linkWake.setRootAccountId(app.getRootParent());
                linkWake.setAppkey(app.getAppkey());
                linkWake.setCreateDate(DateUtil.getBeforeDays(0));
                appDeepLinkWakeRepository.save(linkWake);
            }

            //AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByLinkId(appDeepLink.getLinkId());
            AppDeepLink dbAppDeepLink = appDeepLinkRepository.findOneByAppId(app.getId());
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
            }
            //顺序不能变
            String download = UrlEncoded.encodeString(source.getUrl());
            String scheme = UrlEncoded.encodeString(appDeepLink.getUrlScheme());
            if (appDeepLink.getCallType() == 0) {
                dbAppDeepLink.setMyDoMain("https://urlschemecommon."+Constant.deeplinkHost+"/ry?ry_dp=" + scheme + "&download=" + download);
            } else if (appDeepLink.getCallType() == 1) {
                String path = appDeepLink.getPaths() == null ? "" : appDeepLink.getPaths();
                if (app.getPlatform().toLowerCase().equals("android")) {
                    dbAppDeepLink.setMyDoMain("https://" + appDeepLink.getLinkId() + "."+Constant.deeplinkHost+"/ry/" + path + "?ry_dp=" + scheme + "&download=" + download);
                } else {
                    dbAppDeepLink.setMyDoMain("https://" + appDeepLink.getLinkId() + "."+Constant.deeplinkHost+"/ry/" + path + "?ry_sc=" + scheme + "&download=" + download);
                }
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
                dbAppDeepLink.setPaths(appDeepLink.getPaths());
            } else {
                if (dbAppDeepLink.getPaths() == null) {
                    dbAppDeepLink.setPaths(appDeepLink.getPaths());
                } else {
                    if (appDeepLink.getPaths() != null && !dbAppDeepLink.getPaths().contains(appDeepLink.getPaths()))
                        dbAppDeepLink.setPaths(dbAppDeepLink.getPaths() + "," + appDeepLink.getPaths());
                }
            }
            //修改活动的url，活动修改回显时候需要查找替换
            source.setUrl(dbAppDeepLink.getMyDoMain());
            appDeepLinkRepository.save(dbAppDeepLink);
            try {
                //同步deeplink至dynampdb
                if (app.getPlatform().toLowerCase().equals("ios")) {
                    //ios修改paths
                    if (appDeepLink.getCallType() != 0) {
                        //开启了Universal Links
                        iNoSQLDBService.updateIOSPaths(dbAppDeepLink);
                    }
                }
            } catch (Exception e) {
                logger.error("同步deeplink发生错误", e);
            }
        }
        PackageSubCampaign resource = null;

        if (!StringUtils.isEmpty(source.getSurl())) {
            source.setCampaignid(ShortUrlGenerator.md5(source.getSurl()));
            //前端生成的short url
            resource = packageSubCampaignRepository.save(source);
        } else {
            String surl = campaignService.makesurl(source.getApp(), account);
            source.setSurl(surl);
            source.setCampaignid(ShortUrlGenerator.md5(surl));

            resource = packageSubCampaignRepository.save(source);
        }

        resource.setVivo2DDb(false);
        if (resource.getChannel().equals(767L)) {
            resource.setVivo2DDb(true);
        }

        //微信mp 分包
        if (resource.getChannel().equals(17281L)) {

            String result = createUserActionSetId(app, resource, source.getSurl());
            if (!result.equals("success")) {
                throw new TipException(result);
            }
        }



        //2. 同步创建短链信息至aws-dynamodb
        if (Constant.awsneed) {
            this.savePackageSubCampin2DDb(resource);
            //添加vivo信息
            this.vivoParam2DDb(resource, true);

        }
        return resource;
    }


    /**
     * 处理微信MP 分包相关信息 并存入ddb
     * @param app
     * @param resource
     * @param surl
     * @return
     */
    public String createUserActionSetId(App app, PackageSubCampaign resource, String surl) {
        Account account = accountRepository.findOne(app.getAccount());
        String specialKeys = resource.getSpecialKeys();
        if (!Strings.isNullOrEmpty(specialKeys)) {
            String[] split = specialKeys.split("&");
            Map<String, String> specialKeyMap = new HashMap<>();
            for (String s : split) {
                String[] split1 = s.split("=");
                if (split1.length == 2) {
                    specialKeyMap.put(split1[0], split1[1]);
                } else {
                    specialKeyMap.put(split1[0], null);
                }
            }
            String mpappid = specialKeyMap.get("mpappid");
            String mpappsecret = specialKeyMap.get("mpappsecret");
            String mpmobile_app_id = specialKeyMap.get("mpmobile_app_id");
            String mpaccess_token_url = specialKeyMap.get("mpaccess_token");
            String mptoken = null;
            Object mptokenOBJ = null;
            String response = null;
            //先查询库里token是否已存在
            // CampaignWechatAccount weChatAccount = campaignWeChatAccountRepository.findByMpappidAndMpappsecret(mpappid,mpappsecret);
            CampaignWechatAccount weChatAccount = campaignWeChatAccountRepository.findByMpappid(mpappid, account.getRootParent());
            if (weChatAccount != null) {
                mptoken = weChatAccount.getMpaccesstoken();
            } else {
                //不存在则去微信/中控获取token，成功后添加至库
                if (!Strings.isNullOrEmpty(mpaccess_token_url)) {
                    //请求用户提供的中控台获取token
                    response = HttpClientUtil.doHttpGetRequest(mpaccess_token_url, "", 2000);
                    JSONObject requestData = JSONObject.parseObject(response);
                    mptokenOBJ = requestData.getJSONObject("data").get("access_token");
                } else {
                    //直接从微信请求token
                    response = HttpClientUtil.doHttpGetRequest("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + mpappid + "&secret=" + mpappsecret, "", 2000);
                    JSONObject jsonObject = JSONObject.parseObject(response);
                    mptokenOBJ = jsonObject.get("access_token");
                }
                if (mptokenOBJ == null) {
                    return response;
                }
                mptoken = mptokenOBJ.toString();
                CampaignWechatAccount campaignWeChatAccount = new CampaignWechatAccount();
                campaignWeChatAccount.setMpaccesstoken(mptoken);
                campaignWeChatAccount.setMpappid(mpappid);
                campaignWeChatAccount.setMpappsecret(mpappsecret);
                campaignWeChatAccount.setMpaccess_token_url(mpaccess_token_url);
                campaignWeChatAccount.setRootAccountId(account.getRootParent());
                weChatAccount = campaignWeChatAccountRepository.save(campaignWeChatAccount);
            }

            if (resource.getId() != null) {
                //根据活动id确认是否是修改行为,然后进行修改
                campaignWeChatAccountRepository.updateAccessTokenUrlAndMpappsecretById(mpaccess_token_url, mpappsecret, weChatAccount.getId(), DateUtil.getCurrentDateStr(DateUtil.C_TIME_PATTON_DEFAULT));
                weChatAccount = campaignWeChatAccountRepository.findOne(weChatAccount.getId());
            }

            //获取行为数据源
            Map<String, String> map = new HashMap<>();
            map.put("type", app.getPlatform().toUpperCase());
            map.put("mobile_app_id", mpmobile_app_id);
            map.put("name", app.getName() + "_" + resource.getName());
            map.put("description", "");
            response = HttpClientUtil.doHttpPostRequest("https://api.weixin.qq.com/marketing/user_action_sets/add?version=v1.0&access_token=" + mptoken, "", map);
            JSONObject jsonResponse = JSONObject.parseObject(response);
            JSONObject dataSource = jsonResponse.getJSONObject("data");
            String user_action_set_id = null;
            //如果没有获取到则再请求微信接口查询
            if (dataSource != null) {
                    /*//保证数据源唯一
                    CampaignWechatLink weChatLink =  campaignWeChatLinkRepository.findByUserActionSetId(dataSource.get("user_action_set_id").toString());
                    if(weChatLink!=null){
                        //行为数据源重复,则不再添加
                        return "success";
                    }*/
                user_action_set_id = dataSource.get("user_action_set_id").toString();

            } else {
                String errcode = jsonResponse.get("errcode").toString();
                String errmsg = jsonResponse.get("errmsg").toString();
                if ("900351000".equals(errcode) && errmsg.contains("registered")) {
                    user_action_set_id = errmsg.split(":")[1].trim();
                } else {
                    return response;
                }
            }
            CampaignWechatLink campaignWeChatLink = new CampaignWechatLink();
            campaignWeChatLink.setAppkey(app.getAppkey());

           // Campaign campaign = campaignRepository.findCampaignByCampaignId(resource.getParentCampaignid());
            
            if (app.getPlatform() != null && "wx".equals(app.getPlatform()) || AppPlatformEnum.WG.getKey().equals(app.getPlatform())) {
                campaignWeChatLink.setCampaignid(surl);
            } else {
                campaignWeChatLink.setCampaignid(ShortUrlGenerator.md5(surl));
            }
            campaignWeChatLink.setUser_action_set_id(user_action_set_id);
            campaignWeChatLink.setMpappid(mpappid);
            campaignWeChatLink.setStatus(0);
            campaignWeChatLink.setRootAccountId(account.getRootParent());
            //先查询是否已有该活动行为数据源的数据，有则修改（赋值id即可），没有即save新增
            CampaignWechatLink ca = campaignWeChatLinkRepository.findByCampaignId(campaignWeChatLink.getCampaignid());
            if (ca != null) {
                campaignWeChatLink.setId(ca.getId());
            }
            campaignWeChatLinkRepository.save(campaignWeChatLink);
        } else {
            return "缺失参数!";
        }
        return "success";
    }


    private void vivoParam2DDb(PackageSubCampaign subCampaign, boolean isAdd) {
        if (!subCampaign.isVivo2DDb()) {
            return;
        }

        Map data = this.fillVivoParamMap(subCampaign.getSpecialKeys());
        App app = appRepository.findOne(subCampaign.getApp());
        if (isAdd) {
            iNoSQLDBService.addVivoParamDatas(data, app.getAppkey(), subCampaign.getCampaignid());
        } else {
            iNoSQLDBService.deleteVivoParam(app.getAppkey(), subCampaign.getCampaignid());
        }

    }

    private Map<String, String> fillVivoParamMap(String speciakey) {

        Map<String, String> data = new HashMap();
        Arrays.asList(speciakey.split("&")).forEach(v -> {
            String[] kv = v.replaceFirst("=", "@").split("@");
            data.put(kv[0], kv[1]);
        });

        //获取签名
        Map apiQuery = new HashMap();
        apiQuery.put("apiKey", data.get("apiKey"));
        apiQuery.put("requestStr", "{\"reportDate\":\"20181112\"}");
        String response = HttpClientUtil.doHttpPostRequest(VIVO_API_SIGN_URL, "", apiQuery);
        data.put("sign", response);
        return data;

    }


    private boolean savePackageSubCampin2DDb(PackageSubCampaign resource) {
        boolean isAwsSucces = false;
        if (resource.getId() != null) {
            isAwsSucces = true;
            try {
                final SubCampaignSurlModel transform = transform(resource);
                if (!Strings.isNullOrEmpty(transform.getSurl())//短链
                        && !Strings.isNullOrEmpty(transform.getCampaignid())//推广活动id
                        && !Strings.isNullOrEmpty(transform.getAppid())//应用id
                        && !Strings.isNullOrEmpty(transform.getCid())) {//渠道id
                    isAwsSucces = subCampaignINoSQLService.create(transform);
                    SubCampaignSurlModel fetchRemoteItem = subCampaignINoSQLService.findBySurl(resource.getSurl());
                    if (fetchRemoteItem == null
                            || Strings.isNullOrEmpty(fetchRemoteItem.getSurl())
                            || Strings.isNullOrEmpty(fetchRemoteItem.getCampaignid())
                            || Strings.isNullOrEmpty(fetchRemoteItem.getAppid())
                            || Strings.isNullOrEmpty(fetchRemoteItem.getCid())
                    ) {
                        String contents = String.format("本地SubCampaignSurlModel: %s;\n远程SubCampaignSurlModel: %s.\n\ncampaininfo: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(fetchRemoteItem), new ObjectMapper().writeValueAsString(resource));
                        //MailUtils.sendSimpleEmail(, contents, Constant.mlist);
                        SendCommonPostMail.sendMailReuse(Constant.mlist, "子活动数据不一致(业务库与dynamodb)", contents, null, null);
                    }
                } else {
                    try {
                        String contents = String.format("本地SubCampaignSurlModel信息: %s.\n\ncampaininfo: {}", new ObjectMapper().writeValueAsString(transform), new ObjectMapper().writeValueAsString(resource));
                        //MailUtils.sendSimpleEmail("SubCampaignSurlModel信息缺失,无法同步Dynamodb", contents, Constant.mlist);
                        SendCommonPostMail.sendMailReuse(Constant.mlist, "SubCampaignSurlModel信息缺失,无法同步Dynamodb", contents, null, null);
                    } catch (Exception e1) {
                        logger.error("fail to send email for failing to generat surl");
                    }
                }
            } catch (Exception e) {
                isAwsSucces = false;
                e.printStackTrace();
            }
            this.packageSubCampaignRepository.updateAwsStatus(resource.getId(), isAwsSucces);
        }

        return isAwsSucces;
    }


    @Override
    public PackageSubCampaign update(PackageSubCampaign source, Long account) {
        PackageSubCampaign subCampaign = packageSubCampaignRepository.findOne(source.getId());
        //deeplink同步到ddb
        AppDeepLink appDeepLink = source.getAppDeepLink();
        App app = this.appRepository.findOne(source.getApp());
        //保存deeplink配置，安卓或ios，deeplink对象不为空
        if (appDeepLink != null && (app.getPlatform().toLowerCase().equals("ios") || app.getPlatform().toLowerCase().equals("android"))) {
            AppDeepLink dbAppDeepLink = null;
            if (appDeepLink.getId() != null) {
                AppDeepLink one = appDeepLinkRepository.findOne(appDeepLink.getId());
                dbAppDeepLink = one;
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
            }
            //顺序不能变
            String download = UrlEncoded.encodeString(source.getUrl());
            String scheme = UrlEncoded.encodeString(appDeepLink.getUrlScheme());
            if (appDeepLink.getCallType() == 0) {
                dbAppDeepLink.setMyDoMain("https://urlschemecommon."+Constant.deeplinkHost+"/ry?ry_dp=" + scheme + "&download=" + download);
            } else if (appDeepLink.getCallType() == 1) {
                String path = appDeepLink.getPaths() == null ? "" : appDeepLink.getPaths();
                if (app.getPlatform().toLowerCase().equals("android")) {
                    dbAppDeepLink.setMyDoMain("https://" + dbAppDeepLink.getLinkId() + "."+Constant.deeplinkHost+"/ry/" + path + "?ry_dp=" + scheme + "&download=" + download);
                } else {
                    dbAppDeepLink.setMyDoMain("https://" + dbAppDeepLink.getLinkId() + "."+Constant.deeplinkHost+"/ry/" + path + "?ry_sc=" + scheme + "&download=" + download);
                }
            }
            if (dbAppDeepLink == null) {
                dbAppDeepLink = new AppDeepLink();
                dbAppDeepLink.setPaths(appDeepLink.getPaths());
            } else {
                if (dbAppDeepLink.getPaths() == null) {
                    dbAppDeepLink.setPaths(appDeepLink.getPaths());
                } else {
                    if (appDeepLink.getPaths() != null && !dbAppDeepLink.getPaths().contains(appDeepLink.getPaths()))
                        dbAppDeepLink.setPaths(dbAppDeepLink.getPaths() + "," + appDeepLink.getPaths());
                }
            }
            //修改活动的url，活动修改回显时候需要查找替换
            source.setUrl(dbAppDeepLink.getMyDoMain());
            appDeepLinkRepository.save(dbAppDeepLink);
            try {
                //同步deeplink至dynampdb
                if (app.getPlatform().toLowerCase().equals("ios")) {
                    //ios，修改不需要同步
                    if (appDeepLink.getCallType() != 0) {
                        //开启了Universal Links     ios修改paths
                        iNoSQLDBService.updateIOSPaths(dbAppDeepLink);
                    }
                }
            } catch (Exception e) {
                logger.error("修改同步deeplink发生错误", e);
            }
        }
        subCampaign.setUrl(source.getUrl());
        subCampaign.setName(source.getName());
        subCampaign.setModifyTime(new Date());
        subCampaign.setModifyAccount(account);
        subCampaign.setBundleid(source.getBundleid());
        subCampaign.setAdvertiserid(source.getAdvertiserid());
        subCampaign.setSpecialKeys(source.getSpecialKeys());

        PackageSubCampaign save = packageSubCampaignRepository.save(subCampaign);

        //查询子活动列表是否有deeplink活动（子活动更新时需要维护活动的deeplink标志位字段）
        List<PackageSubCampaign> list = packageSubCampaignRepository.findFilterDeeplinkByParentCampaignid(subCampaign.getParentCampaignid(),Constant.deeplinkHost);
        Campaign campaign = campaignRepository.findCampaignByCampaignId(subCampaign.getParentCampaignid(), source.getApp());
        if(list!=null&&list.size()>0){
            campaignRepository.updateDeeplinkByCampaignId(source.getParentCampaignid(),true);
        }else if(campaign.getDeeplink()){
            campaignRepository.updateDeeplinkByCampaignId(source.getParentCampaignid(),false);
        }


        save.setVivo2DDb(false);
        if (save.getChannel().equals(767L)) {
            save.setVivo2DDb(true);
        }

        //2. 同步更新短链信息至aws-dynamodb
        if (Constant.awsneed) {
            if (!this.subCampaignINoSQLService.update(transform(save))) {
                this.packageSubCampaignRepository.updateAwsStatus(save.getId(), false);
            }
            //更新 vivo 参数
            this.vivoParam2DDb(save, true);
        }
        //微信mp 分包
        if (subCampaign.getChannel().equals(17281L)) {
            String result = createUserActionSetId(app, subCampaign, source.getSurl());
            if (!result.equals("success")) {
                throw new TipException(result);
            }
        }

        return save;
    }

    @Override
    public void enable(String ids, Long account) {
        List<Long> subCampaignList = StringUtil.transString2LongList(ids);

//        packageSubCampaignRepository.enableSubCampaign(subCampaignList, account, new Date());

        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.listSubCampaignByDelAndId(subCampaignList);

        Boolean sucess = false;

        // 需要恢复 DDB 中的短链
        for (PackageSubCampaign campaign : packageSubCampaigns) {

            if (Constant.awsneed) {

                campaign.setVivo2DDb(false);
                if (campaign.getChannel().equals(767L)) {
                    campaign.setVivo2DDb(true);
                }

                // 需要恢复 DDB 中的短链
                if (this.savePackageSubCampin2DDb(campaign)) {

                    campaign.setDelFlag(false);
                    campaign.setModifyAccount(account);
                    campaign.setModifyTime(new Date());

                } else {
                    logger.error("启用dynamodb分包推广活动短链失败. info: {}", campaign.getCampaignid());
                }

                this.vivoParam2DDb(campaign, true);


            }
        }

        packageSubCampaignRepository.save(packageSubCampaigns);

    }

    @Override
    public void forbiden(String ids, Long account) {
        List<Long> subCampaignList = StringUtil.transString2LongList(ids);

        List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.listSubCampaignById(subCampaignList);

        for (PackageSubCampaign campaign : packageSubCampaigns) {

            if (Constant.awsneed) {
                campaign.setVivo2DDb(false);
                if (campaign.getChannel().equals(767L)) {
                    campaign.setVivo2DDb(true);
                }

                // 需要停用 DDB 中的短链
                if (this.subCampaignINoSQLService.delete(campaign.getSurl())) {

                    campaign.setDelFlag(true);
                    campaign.setModifyAccount(account);
                    campaign.setModifyTime(new Date());

                } else {
                    logger.error("删除dynamodb分包推广活动短链失败. info: {}", campaign.getCampaignid());
                }

                this.vivoParam2DDb(campaign,false);
            }

        }

        packageSubCampaignRepository.save(packageSubCampaigns);


        // packageSubCampaignRepository.forbidenSubCampaign(subCampaignList, account, new Date());
    }

    @Override
    public boolean valid(Long campaign, String name) {
        boolean flag = true;
        List<String> list = packageSubCampaignRepository.listAllSubCampaignNameByCampaign(campaign);

        if (ValidateUtil.isValid(list)) {
            if (list.contains(name)) {
                flag = false;
            }
        }
        return flag;
    }

    /**
     * 保存导出记录
     * create by sunhao 20170524
     */
    @Override
    public StringBuffer exportSubCampaign(Account account, Long campaignId, String id) throws UnsupportedEncodingException {

        Iterable<String> iterids = Splitter.on(",").trimResults().omitEmptyStrings().split(id);
        Iterable<Long> campids = Iterables.transform(iterids,
                new Function<String, Long>() {
                    @Override
                    public Long apply(String s) {
                        return Long.parseLong(s);
                    }
                });
        ArrayList<Long> ids = Lists.newArrayList(campids);
        Campaign campaign = campaignService.findOneById(campaignId);
        App app = appRepository.findOne(campaign.getApp());
        AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(app.getId());
        Channel channel = channelRepository.findOne(campaign.getChannel());
        StringBuffer sb = new StringBuffer();
        sb.append("子活动名称,推广渠道ID,追踪短链,展示短链,创建时间").append("\r\n");

        if (null != campaign) {

            List<PackageSubCampaign> subList = new ArrayList<>();
            //查询数据条数
            if (!ValidateUtil.isValid(ids)) {
                subList = packageSubCampaignRepository.listSubCampaignByCampaign(campaignId);
            } else {
                subList = packageSubCampaignRepository.listSubCampaignById(ids);
            }
            if (ValidateUtil.isValid(subList)) {
                for (PackageSubCampaign sub : subList) {

                    String androidParam = channel.getAndroidparam();
                    boolean isTrackUrlSpecial = channel.isIstrackurlspecial();
                    boolean istrackUrlSpecailReal = false;
                    boolean isTrackHttps = false;

                    String impAndroidParam = channel.getImpAndroidParam();
                    boolean isImpUrlSpecail = channel.isImpIsTrackUrlSpecial();
                    boolean isImpUrlSpecailReal = false;
                    boolean isImpHttps = false;

                    if (isTrackUrlSpecial && !StringUtil.isEmpty(androidParam)) {
                        istrackUrlSpecailReal = true;
                    }

                    if (isImpUrlSpecail && !StringUtil.isEmpty(impAndroidParam)) {
                        isImpUrlSpecailReal = true;
                    }

                    if (channel.isIshttps()) {
                        isTrackHttps = true;
                    }
                    if (channel.isImpIsHttps()) {
                        isImpHttps = true;
                    }

                    String trackUrl = Constant.trackurl + "/" + sub.getSurl();

                    if (istrackUrlSpecailReal) {
                        trackUrl = trackUrl + "?" + androidParam;
                        sub.setTrackUrl(trackUrl);
                    }

                    String impUrl = Constant.trackurl + "/" + sub.getSurl() + "?impression=true";
                    if (isImpUrlSpecailReal) {
                        impUrl = impUrl + "&" + androidParam;
                        sub.setImpUrl(impUrl);
                    }

                    if (isTrackHttps) {
                        trackUrl = trackUrl.replace("http:", "https:");
                        sub.setTrackUrl(trackUrl);
                    }

                    if (isImpHttps) {
                        impUrl = impUrl.replace("http:", "https:");
                        sub.setImpUrl(impUrl);
                    }

                    disposeImpUrlAndTackUrl( sub, app,  appDeepLink);
                    sb.append(sub.getName()).append(",").append(sub.getPackChannelid()).append(",")
                            .append(sub.getTrackUrl()).append(",").append(sub.getImpUrl()).append(",").append(DateUtil.getFormatDate(sub.getCreateTime())).append("\r\n");
                }
            }
        }

        return sb;
    }

    /**
     * 查询子活动数量
     * Created by sunhao on 17/5/22.
     */
    @Override
    public Long findSubCampaignNumber(Long campaignId) {

        BigInteger number = packageSubCampaignRepository.countSubCampaignByCampaign(campaignId);
        return (null != number) ? number.longValue() : 0L;
    }


    public SubCampaignSurlModel transform(PackageSubCampaign subCampaign) {
        App app = appRepository.findOne(subCampaign.getApp());
        Channel channel = channelRepository.findOne(subCampaign.getChannel());

        SubCampaignSurlModel surlModel = new SubCampaignSurlModel();

        /*推广活动信息*/
        //短链
        surlModel.setSurl(subCampaign.getSurl());

        surlModel.setSpecialkeys(subCampaign.isVivo2DDb() ? "" : subCampaign.getSpecialKeys());
        //活动id
        surlModel.setCampaignid(subCampaign.getCampaignid());
        //活动名称
        surlModel.setName(subCampaign.getName());
        //应用地址或活动地址
        surlModel.setUrl(Strings.isNullOrEmpty(subCampaign.getUrl()) ? app.getUrl() : subCampaign.getUrl());

        //父推广活动的campaignid
        surlModel.setParentCampaignid(subCampaign.getParentCampaignid());

        //渠道id
        surlModel.setCid(String.valueOf(subCampaign.getChannel()));
        //渠道名称
        surlModel.setChannel(channel.getName());
        //渠道英文名称
        surlModel.setChannelname(Strings.isNullOrEmpty(channel.getUniqueName()) ? "" : channel.getUniqueName());
        surlModel.setCategory(channel.getCategory());

        //应用编码
        surlModel.setAppid(app.getAppkey());
        //app类型
        String appType = AppPlatformEnum.H5.getKey().equals(app.getPlatform()) ? "wap" : null;
        surlModel.setApptype(appType);
        return surlModel;
    }

    @Override
    public List<Object[]> findSubCampaignNumberByIds(List<Long> ids) {
        return packageSubCampaignRepository.countSubCampaignByCampaignByIds(ids);
    }

    @Override
    public Object echoDeepLink(Long appid, Long campaignid) {
        AppDeepLink appDeepLink = appDeepLinkRepository.findOneByAppidId(appid);
        AppDeepLink appDeepLink1 = new AppDeepLink();
        appDeepLink1 = appDeepLink;
        PackageSubCampaign oneById = packageSubCampaignRepository.findOne(campaignid);
        App app = appRepository.findOne(appid);

        String url = null;
        ResultModel resultModel = null;
        if (oneById != null) {
            url = oneById.getUrl();
        }
        String containsUrl = Constant.deeplinkHost;
        if (url != null && url.contains(containsUrl)) {
            String substring = url.substring(url.indexOf("download=") + "download=".length());
            if (appDeepLink1 == null) {
                appDeepLink1 = new AppDeepLink();
                appDeepLink1.setCallType(-1);
                appDeepLink1.setLinkId(url.substring(url.indexOf("https://") + "https://".length(), url.indexOf("."+Constant.deeplinkHost)));
            }
            appDeepLink1.setAppDownload(UrlEncoded.decodeString(substring, 0, substring.length(), "UTF-8"));
            if (url.contains("ry/")) {
                if ((appDeepLink1.getPackageName() == null && app.getPlatform().equals("Android") || (appDeepLink1.getAppIdPrefix() == null && app.getPlatform().equals("iOS")))) {
                    appDeepLink1.setPaths(null);
                } else {
                    appDeepLink1.setPaths(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                }
            } else {
                appDeepLink1.setPaths(null);
            }
            if (url.contains("ry_dp=")) {
                String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                appDeepLink1.setUrlScheme(UrlEncoded.decodeString(substring2, 0, substring2.length(), "UTF-8"));
            } else if (url.contains("ry_sc=") && appDeepLink1.getCallType() == null) {
                String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                appDeepLink1.setUrlScheme(UrlEncoded.decodeString(substring2, 0, substring2.length(), "UTF-8"));
            }
            resultModel = new ResultModel(201, "old", appDeepLink1);
        } else {
            if (appDeepLink1 != null) {
                appDeepLink1.setPaths(null);
                appDeepLink1.setUrlScheme(null);
                if (oneById != null)
                    appDeepLink1.setAppDownload(oneById.getUrl());
            }
            resultModel = new ResultModel(201, "new", appDeepLink1);
        }
        //old:根据deeplink的值判断是否开启
        //new：默认关闭，如果content不为null，则照常回显
        return resultModel;
    }

    private PackageSubCampaign disposeImpUrlAndTackUrl(PackageSubCampaign sub,App app, AppDeepLink appDeepLink){
        if (sub.getUrl() != null && sub.getAccount() != null) {
            //拼接deeplink的标识
            String containsUrl = Constant.deeplinkHost;

            String url = sub.getUrl();
            if (url.contains(containsUrl)) {
                sub.setDeeplink(true);
                if (sub.getTrackUrl().contains("?")) {
                    sub.setTrackUrl(sub.getTrackUrl() + "&ry_dp=true");
                } else {
                    sub.setTrackUrl(sub.getTrackUrl() + "?ry_dp=true");
                }
                if (sub.getImpUrl().contains("?")) {
                    sub.setImpUrl(sub.getImpUrl() + "&ry_dp=true");
                } else {
                    sub.setImpUrl(sub.getImpUrl() + "?ry_dp=true");
                }

                if (url.contains("ry/")) {
                    if ((appDeepLink.getPackageName() == null && app.getPlatform().equals("Android") || (appDeepLink.getAppIdPrefix() == null && app.getPlatform().equals("iOS")))) {
                        sub.setTrackUrl(sub.getTrackUrl() + "&rydp_path=null");
                        sub.setImpUrl(sub.getImpUrl() + "&rydp_path=null");
                    } else {
                        String encodeString = UrlEncoded.encodeString(url.substring(url.indexOf("ry/") + "ry/".length(), url.indexOf("?")));
                        sub.setTrackUrl(sub.getTrackUrl() + "&rydp_path=" + encodeString);
                        sub.setImpUrl(sub.getImpUrl() + "&rydp_path=" + encodeString);
                    }

                } else {
                    sub.setTrackUrl(sub.getTrackUrl() + "&rydp_path=null");
                    sub.setImpUrl(sub.getImpUrl() + "&rydp_path=null");
                }
                if (url.contains("ry_dp=")) {
                    String substring2 = url.substring(url.indexOf("ry_dp=") + "ry_dp=".length(), url.indexOf("&download"));
                    sub.setTrackUrl(sub.getTrackUrl() + "&rydp_url=" + substring2);
                    sub.setImpUrl(sub.getImpUrl() + "&rydp_url=" + substring2);
                } else if (url.contains("ry_sc=") && appDeepLink.getCallType() == null) {
                    String substring2 = url.substring(url.indexOf("ry_sc=") + "ry_sc=".length(), url.indexOf("&download"));
                    sub.setTrackUrl(sub.getTrackUrl() + "&rydp_url=" + substring2);
                    sub.setImpUrl(sub.getImpUrl() + "&rydp_url=" + substring2);
                }
            }
        }
        return sub;
    }

}
