package com.reyun.service.impl;

import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.RecoveryReportService;
import com.reyun.util.HttpClientUtil;
import com.reyun.util.ValidateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by mxq on 2017/7/17.
 */
@Service
public class RecoveryReportServiceImpl implements RecoveryReportService {

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    ChannelAccountRepository channelAccountRepository;

    @Autowired
    LinkMetaDataRepository linkMetaDataRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    AppRepository appRepository;



    public ReportListResult parseWeek(ReportListResult listResult){
        List<Map<String, Object>> valList = listResult.getVal();

        Map<String,Map<String,Object>> map = new HashMap<>();
        for(Map<String, Object> obj :valList){
            String ds = (String)obj.get("ds");
            String key = getWeekNoByDate(ds);

            if(map.containsKey(key)){
                Map data = map.get(key);
                data = parseMap(data, obj);
                map.put(key, data);

            }else{
                map.put(key, obj);
            }

            List<Map<String, Object>> arr = new ArrayList<>();
            if(ValidateUtil.isValid(map)){
                Set<String> set = map.keySet();
                for(String mapKey : set){
                    Map<String, Object> data = map.get(mapKey);

                    data.put("ds", mapKey);
                    data = parseMap(data);
                    arr.add(data);
                }
            }
            listResult.setVal(arr);
        }
        return listResult;
    }


    public ReportListResult parseMonth(ReportListResult listResult){
        List<Map<String, Object>> valList = listResult.getVal();

        Map<String,Map<String,Object>> map = new HashMap<>();
        for(Map<String, Object> obj :valList){
            String ds = (String)obj.get("ds");
            String key = ds.substring(0,7);

            if(map.containsKey(key)){
                Map data = map.get(key);
                data = parseMap(data, obj);
                map.put(key, data);
            }else{
                map.put(key, obj);
            }

            List<Map<String, Object>> arr = new ArrayList<>();
            if(ValidateUtil.isValid(map)){
                Set<String> set = map.keySet();
                for(String mapKey : set){
                    Map<String, Object> data = map.get(mapKey);
                    data = parseMap(data);
                    data.put("ds", mapKey);
                    arr.add(data);
                }
            }
            listResult.setVal(arr);
        }
        return listResult;
    }
    public static Map<String, Object> parseMap(Map<String, Object> data, Map<String, Object> obj){

        data.put("amt_pay",(double)data.get("amt_pay") + (double)obj.get("amt_pay"));
        data.put("amt_ammount",(double)data.get("amt_ammount") + (double)obj.get("amt_ammount"));
        data.put("exposure",(int)data.get("exposure") + (int)obj.get("exposure"));
        data.put("click",(int)data.get("click") + (int)obj.get("click") );
        data.put("rate_ctr",Double.parseDouble(data.get("rate_ctr").toString()) + Double.parseDouble(obj.get("rate_ctr").toString()));
        data.put("rate_cpm",Double.parseDouble(data.get("rate_cpm").toString()) + Double.parseDouble(obj.get("rate_cpm").toString()));
        data.put("install",(int)data.get("install") + (int)obj.get("install"));
        data.put("rate_install",Double.parseDouble(data.get("rate_install").toString()) + Double.parseDouble(obj.get("rate_install").toString()));
        data.put("rate_arpu",Double.parseDouble(data.get("rate_arpu").toString()) + Double.parseDouble(obj.get("rate_arpu").toString()));
        data.put("rate_cpa",Double.parseDouble(data.get("rate_cpa").toString()) + Double.parseDouble(obj.get("rate_cpa").toString()));
        data.put("rate_recovery_7",Double.parseDouble(data.get("rate_recovery_7").toString()) + Double.parseDouble(obj.get("rate_recovery_7").toString()));
        data.put("rate_recovery_30",Double.parseDouble(data.get("rate_recovery_30").toString()) + Double.parseDouble(obj.get("rate_recovery_30").toString()));
        data.put("rate_recovery_60",Double.parseDouble(data.get("rate_recovery_60").toString()) + Double.parseDouble(obj.get("rate_recovery_60").toString()));

        return data;
    }

    public static Map<String, Object> parseMap(Map<String, Object> data){

        data.put("rate_ctr", String.format("%.1f", (double)((int)data.get("click") != 0 ? (int)data.get("exposure") / (int)data.get("click") : 0.0)));     //SUM(t1.exposure) / SUM(t1.click)
        data.put("rate_cpm", String.format("%.1f", Double.parseDouble(((int)data.get("exposure") != 0 ? (double)data.get("amt_ammount") * 1000 / (int)data.get("exposure") : 0.0)+"")));   //SUM(t1.ammount) * 1000 / SUM(t1.exposure)
        data.put("rate_install", String.format("%.1f", (double)((int)data.get("install") != 0 ? (double)data.get("rate_install") / (int)data.get("install") : 0.0)));
        data.put("rate_arpu", String.format("%.1f", (double)((int)data.get("install") != 0 ? (double)data.get("amt_pay") / (int)data.get("install") : 0.0)));
        data.put("rate_cpa", String.format("%.1f", (double)((int)data.get("install") != 0 ? (double)data.get("amt_ammount") / (int)data.get("install") : 0.0)));
        data.put("rate_recovery_7", String.format("%.1f", (double)((int)data.get("exposure") != 0 ? (double)data.get("rate_recovery_7") /(int) data.get("exposure") : 0.0)));
        data.put("rate_recovery_30", String.format("%.1f", (double)((int)data.get("exposure") != 0 ? (double)data.get("rate_recovery_30") / (int)data.get("exposure") : 0.0)));
        data.put("rate_recovery_60", String.format("%.1f", (double)((int)data.get("exposure") != 0 ? (double)data.get("rate_recovery_60") / (int)data.get("exposure") : 0.0)));

        return data;
    }

    @Override
    public List<RecoveryParamModel> findChannelByAccount(Long appid,Long account) {
        List<RecoveryParamModel> result = new ArrayList<>();

        String appkey = appRepository.findAppkeyById(appid);

        List<String> uniqueNames = linkMetaDataRepository.findChannelByCreateAccount(account, appkey);
        if(ValidateUtil.isValid(uniqueNames)){
            List<Channel> list = channelRepository.findByUniqueNames(uniqueNames);
            if(ValidateUtil.isValid(list)){
                for(Channel ch : list){
                    RecoveryParamModel model = new RecoveryParamModel();
                    model.setChannelName(ch.getName());
                    model.setUniqueName(ch.getUniqueName());

                    result.add(model);
                }
            }
        }

        return result;
    }


    @Override
    public List<RecoveryParamModel> findChannelAccountByChannel(Long appid,Long account, String uniqueName) {
        List<RecoveryParamModel> result = new ArrayList<>();

        String appkey = appRepository.findAppkeyById(appid);
        List<LinkMetaData> linkMetaDatas = new ArrayList<>();
        if(null ==uniqueName){
            linkMetaDatas   = linkMetaDataRepository.findlink_user_idByCreateAccount(account, appkey);
        }else{
            linkMetaDatas = linkMetaDataRepository.findLinkUserIdByChannel(account, uniqueName, appkey);
        }

        if(ValidateUtil.isValid(linkMetaDatas)){
            List<Long> ids = new ArrayList<>();
            Map<Long, LinkMetaData> map = new HashMap<>();
            for(LinkMetaData data : linkMetaDatas){
                ids.add(data.getLinkUserId());
                map.put(data.getLinkUserId(), data);
            }
            List<ChannelAccount> list = channelAccountRepository.findByids(ids);
            for(ChannelAccount ca : list){
                LinkMetaData linkMetaData = map.get(ca.getId());

                RecoveryParamModel model = new RecoveryParamModel();
                model.setUniqueName(linkMetaData.getUniqueName());
                model.setChannelAccountName(ca.getAlias());
                model.setChannelAccountId(ca.getId());

                result.add(model);
            }
        }
        return result;
    }

    @Override
    public List<RecoveryParamModel> findPlanByChannelAccount(Long appid,Long account, String uniqueName, Long channelAccount) {
        List<RecoveryParamModel> result = new ArrayList<>();

        String appkey = appRepository.findAppkeyById(appid);
        List<LinkMetaData> plans = new ArrayList<>();
        if(null != uniqueName && null != channelAccount){
            plans = linkMetaDataRepository.findPlanByLinkUser(account, uniqueName, channelAccount, appkey);
        }else if(null != uniqueName){
            plans = linkMetaDataRepository.findPlanByChannel(account,uniqueName, appkey);
        }else{
            plans =  linkMetaDataRepository.findPlanByCreateAccount(account, appkey);
        }

        if(ValidateUtil.isValid(plans)){
            for(LinkMetaData data: plans){
                RecoveryParamModel model = new RecoveryParamModel();
                model.setUniqueName(data.getUniqueName());
                model.setChannelAccountId(data.getLinkUserId());
                model.setPlan(data.getPlan());

                result.add(model);
            }
        }
        return result;
    }


    @Override
    public List<RecoveryParamModel> findCampaignByPlan(Long appid,Long account, String uniqueName, Long channelAccount, String plan) {
        List<RecoveryParamModel> result = new ArrayList<>();

        String appkey = appRepository.findAppkeyById(appid);
        List<LinkMetaData> metaDataList = new ArrayList<>();
        if(null != uniqueName && null != channelAccount && null != plan){
            metaDataList = linkMetaDataRepository.findCampaignIdByPlan(account, uniqueName, channelAccount, plan, appkey);
        }else if(null != uniqueName && null != channelAccount){
            metaDataList = linkMetaDataRepository.findCampaignIdByChannelAccount(account,uniqueName,channelAccount, appkey);
        }else if(null != uniqueName ){
            metaDataList = linkMetaDataRepository.findCampaignIdByPlan(account,uniqueName, appkey);
        }else{
            metaDataList = linkMetaDataRepository.findCampaignidByCreateAccount(account, appkey);
        }

        if(ValidateUtil.isValid(metaDataList)){
            List<String> ids = new ArrayList<>();
            Map<String, LinkMetaData> map = new HashMap<>();
            for(LinkMetaData data : metaDataList){
                ids.add(data.getCampaignid());
                map.put(data.getCampaignid(), data);
            }
            List<Campaign> list = campaignRepository.findAllByCampaignId(ids);
            for(Campaign ca : list){
                LinkMetaData linkMetaData = map.get(ca.getCampaignid());

                RecoveryParamModel model = new RecoveryParamModel();
                model.setUniqueName(linkMetaData.getUniqueName());
                model.setChannelAccountId(linkMetaData.getLinkUserId());
                model.setPlan(linkMetaData.getPlan());
                model.setCampaignid(linkMetaData.getCampaignid());
                model.setCampaignName(ca.getName());

                result.add(model);
            }
        }
        return result;
    }


    public static String getWeekNoByDate(String time){
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Date date = null;
        try {
            date = format.parse(time);
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
        calendar.setTime(date);
        int year = calendar.get(Calendar.YEAR);
        int week = calendar.get(Calendar.WEEK_OF_YEAR);

        return year + "-" + week;
    }

    public static void main(String[] args) {
        String url = "http://localhost:8444//testlogin/huisuan?name=larry@reyun.com&pwd=a1b2c3!@#";
        String s = HttpClientUtil.doHttpPostRequest(url, "");
        System.out.println(s);
    }
}
