package com.reyun.service.impl;

import java.io.IOException;
import java.net.URLEncoder;
import java.security.AccessController;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.service.AdvancedFunctionApplyService;
import com.reyun.service.RecycleAnalysisService;
import com.reyun.service.ReportService;
import com.reyun.util.DateUtil;
import com.reyun.util.HttpClientUtil;
import com.reyun.util.ValidateUtil;

import sun.security.action.GetPropertyAction;

/**
 * Created by sunhao on 18/5/17.
 * 回本分析service实现类
 */
@Service
public class RecycleAnalysisServiceImpl implements RecycleAnalysisService {

    private Logger logger = LoggerFactory.getLogger(RecycleAnalysisServiceImpl.class);

    private final static int CHANNEL_LIMIT = 3;

    @Autowired
    private ReportService reportService;

    @Autowired
    private RecycleChannelRepository recycleChannelRepository;

    @Autowired
    private RecycleChannelAccountRepository recycleChannelAccountRepository;

    @Autowired
    private ChannelAccountRepository channelAccountRepository;

    @Autowired
    private AdvancedFunctionApplyService advancedFunctionApplyService;

    @Autowired
    private ChannnelAccountLinkAppRepository channnelAccountLinkAppRepository;

    @Autowired
    private AppRepository appRepository;
    @Autowired
    private ChannelRepository channelRepository;

    @Override
    public List<RecycleChannel> listEnableChannel(int type, String lanType) {
        if(type == 0){
            List<RecycleChannel> recycleChannelList = recycleChannelRepository.findAllEnableChannel();
            List<RecycleChannel> recycleChannels = new ArrayList<>();
            //国际化处理
            for(RecycleChannel recycleChannel: recycleChannelList){
                recycleChannel = recycleChannel.toTransEng(recycleChannel, lanType);
                recycleChannels.add(recycleChannel);
            }
            //end

            return recycleChannels;
        }else{
            List<RecycleChannel> recycleChannelList = recycleChannelRepository.findAllEnableChannel();
            List<RecycleChannel> recycleChannels = new ArrayList<>();
            for(RecycleChannel recycleChannel: recycleChannelList){
                String mcCid = recycleChannel.getMcCid();
                if(!StringUtil.isEmpty(mcCid)){
                    String[] cids = mcCid.split(",");
                    if(cids.length == 1){
                        //国际化处理
                        recycleChannel = recycleChannel.toTransEng(recycleChannel, lanType);
                        //end
                        recycleChannels.add(recycleChannel);
                    }else{
                        for(int i = 0; i < cids.length; i++){
                            RecycleChannel newRecycleChannel = new RecycleChannel(recycleChannel);
                            newRecycleChannel.setCid(Long.valueOf(cids[i]));
                            Channel channel = channelRepository.findById(Long.valueOf(cids[i]));
                            if(channel != null){
                                newRecycleChannel.setName(channel.getName());
                                newRecycleChannel.setEngName(channel.getEngName());
                            }

                            //国际化处理
                            newRecycleChannel = newRecycleChannel.toTransEng(newRecycleChannel, lanType);
                            //end
                            recycleChannels.add(newRecycleChannel);
                        }
                    }
                }

            }
            return  recycleChannels;
        }
    }

    @Override
    public List<RecycleChannelAccount> listAccountByAccount(Account account) {
        return recycleChannelAccountRepository.findChannelAccountByAccount(account.getRootParent());
    }

    @Override
    public RecycleChannelAccount addRecycleChannelAccount(Account account, Long app, RecycleChannelAccount recycleChannelAccount) {

        RecycleChannelAccount result = null;

        //查询渠道是否开启回本
        RecycleChannel recycleChannel = recycleChannelRepository.findAllEnableByChannel(recycleChannelAccount.getCid());

        //查询渠道账号是否添加过
        RecycleChannelAccount queryResult = recycleChannelAccountRepository.findChannelAccountByAccount(recycleChannelAccount.getUserName(), recycleChannelAccount.getChannelAccountId(), recycleChannelAccount.getCid());

        if (null != recycleChannel && null == queryResult) {

            recycleChannelAccount.setAccount(account.getId());

            recycleChannelAccount.setInsertDate(new Date());
            recycleChannelAccount.setModifyDate(new Date());

            recycleChannelAccount.setStatus(1);

            //从已授权的渠道账号获取
            ChannelAccount channelAccount = channelAccountRepository.findByThirdAccountCode(recycleChannelAccount.getChannelAccountId(), recycleChannel.getUniqueName());
            if (null != channelAccount) {
                recycleChannelAccount.setToken(channelAccount.getAccessToken());
                recycleChannelAccount.setTokenExpire(channelAccount.getAccessTokenExpire());
                recycleChannelAccount.setRefreshToken(channelAccount.getRefreshToken());
                recycleChannelAccount.setRefreshTokenExpire(channelAccount.getRefreshTokenExpire());
            }

            result = recycleChannelAccountRepository.save(recycleChannelAccount);
        }

        return result;
    }

    @Override
    public Boolean disableRecycleChannelAccount(String userName, String channelAccountId, Long cid) {

        return modifyChannelAccountStatus(userName, channelAccountId, cid, 0);
    }

    @Override
    public Boolean enableRecycleChannelAccount(String userName, String channelAccountId, Long cid) {
        return modifyChannelAccountStatus(userName, channelAccountId, cid, 1);
    }

    /**
     * 修改渠道账号的状态
     */
    private Boolean modifyChannelAccountStatus(String userName, String channelAccountId, Long cid, int status) {

        Boolean result = false;

        RecycleChannelAccount recycleChannelAccount = recycleChannelAccountRepository.findChannelAccountByAccount(userName, channelAccountId, cid);

        if (null != recycleChannelAccount) {

            recycleChannelAccount.setStatus(status);

            recycleChannelAccount = recycleChannelAccountRepository.save(recycleChannelAccount);

            result = null != recycleChannelAccount;
        }

        return result;
    }

    @Override
    public RecycleChannelAccount modifyRecycleChannelAccount(RecycleChannelAccount recycleChannelAccount) {

        RecycleChannelAccount result = null;

        RecycleChannelAccount queryResult = recycleChannelAccountRepository.findChannelAccountByAccount(recycleChannelAccount.getUserName(), recycleChannelAccount.getChannelAccountId(),
                recycleChannelAccount.getCid());

        if (null != queryResult) {

            //密码
            if (ValidateUtil.isValid(recycleChannelAccount.getPassword()) && !recycleChannelAccount.getPassword().equals(queryResult.getPassword())) {
                queryResult.setPassword(recycleChannelAccount.getPassword());
            }

            //token
            if (ValidateUtil.isValid(recycleChannelAccount.getToken()) && !recycleChannelAccount.getToken().equals(queryResult.getToken())) {
                queryResult.setToken(recycleChannelAccount.getToken());
            }

            //channelAlias
            if (ValidateUtil.isValid(recycleChannelAccount.getChannelAlias()) && !recycleChannelAccount.getChannelAlias().equals(queryResult.getChannelAlias())) {
                queryResult.setChannelAlias(recycleChannelAccount.getChannelAlias());
            }

            result = recycleChannelAccountRepository.save(queryResult);

        }

        return result;
    }

    @Override
    public JSONObject hasRecycleFunctionOpen(Account account) {

        JSONObject result = new JSONObject();

        result.put("isOpen", false);
        result.put("isApply", true);
        result.put("isPay", false);
        result.put("pastDate", account.getPastDate());
        result.put("renew", true);

        //是否申请完整版
        AdvancedFunctionApply advancedFunctionApply = advancedFunctionApplyService.listApplyByAccount(account);

        if (null != advancedFunctionApply) {

            result.put("isOpen", true);
            result.put("isApply", false);

            //是否付费
            if (!advancedFunctionApply.getStatus().equals(0)) {

                result.put("isPay", true);
                result.put("pastDate", advancedFunctionApply.getPastDate());
                //续订状态
                result.put("renew", !advancedFunctionApply.getStatus().equals(2));
            }

        } else {
            //普通版是否绑定渠道
            List<RecycleChannelAccount> recycleChannelAccounts = listAccountByAccount(account);
            result.put("isOpen", ValidateUtil.isValid(recycleChannelAccounts));
        }

        return result;
    }


    /**
     * 查询所有可用的CHANEL ACCOUNT
     */
    @Override
    public List<ChannelAccount> listApplyChannelAccount(Account account, Long cid) {

        //查询channel account 表
        List<ChannelAccount> channelAccounts = channelAccountRepository.findEnableByRootAccount(account.getRootParent(), cid);

        if (ValidateUtil.isValid(channelAccounts)) {

            //查询渠道账号下所有的关联APP
            List<ChannelAccountLinkApp> channelAccountLinkAppList = channnelAccountLinkAppRepository.findAllLinkAppByAccount(account.getRootParent(), cid);
            Map<String, App> appNameMap = getLinkedAppMap(channelAccountLinkAppList);

            //查询已加入渠道的并且状态是有效的。
            List<RecycleChannelAccount> recycleChannelAccounts = recycleChannelAccountRepository.findRecycleAccountByChannel(account.getRootParent(), cid);
            Map<String, RecycleChannelAccount> recycleChannelAccountMap = Maps.uniqueIndex(recycleChannelAccounts, RecycleChannelAccount::getUserName);

            //处理渠道账号数据
            for (ChannelAccount channelAccount : channelAccounts) {

                //填充渠道账号下面所有关联的APP
                for (ChannelAccountLinkApp channelAccountLinkApp : channelAccountLinkAppList) {

                    if (channelAccountLinkApp.getLinkUserId().equals(channelAccount.getId())) {

                        List<String> appList = channelAccount.getAppList();

                        appList = null != appList ? appList : new ArrayList<>();

                        App app = appNameMap.get(channelAccountLinkApp.getAppkey());

                        if (null != app) {
                            appList.add(app.getName());
                        }

                        channelAccount.setAppList(appList);
                    }
                }

                //是否应用到回本
                channelAccount.setApply(recycleChannelAccountMap.containsKey(channelAccount.getName()));
            }
        }

        return channelAccounts;
    }

    private Map<String, App> getLinkedAppMap(List<ChannelAccountLinkApp> channelAccountLinkAppList) {

        Map<String, App> appNameMap = new HashMap<>();


        if (ValidateUtil.isValid(channelAccountLinkAppList)) {
            //appids
            List<String> appIdList = Lists.transform(channelAccountLinkAppList, ChannelAccountLinkApp::getAppkey);

            List<App> appList = appRepository.findByAppkeys(appIdList);

            appNameMap = Maps.uniqueIndex(appList, App::getAppkey);

        }

        return appNameMap;
    }

    @Override
    public RecycleChannelAccount addGdtChannelAccount(Account account, RecycleChannelAccount recycleChannelAccount) {

        RecycleChannelAccount result = null;

        //查出渠道账号
        ChannelAccount channelAccount = channelAccountRepository.findByUser(recycleChannelAccount.getUserName(), recycleChannelAccount.getChannelAccountId(),account.getRootParent());

        if (null != channelAccount) {

            //查出是否已经加入过渠道
            result = recycleChannelAccountRepository.findChannelAccountByAccount(recycleChannelAccount.getUserName(), recycleChannelAccount.getChannelAccountId(), channelAccount.getChannel());

            if (null == result) {

                result = new RecycleChannelAccount(channelAccount);

                result.setChannelName("广点通");
                result.setAccount(account.getId());
                result.setRootAccount(account.getRootParent());
                result.setInsertDate(new Date());
                result.setModifyDate(new Date());

                result.setStatus(1);
                
                result.setValidFlag(1);
                result.setUtime(new Date());
                result.setUniqueAccount(recycleChannelAccount.getChannelAccountId());

            } else {
                result.setStatus(1);
            }

            result = recycleChannelAccountRepository.save(result);
        }

        return result;
    }


    @Override
    public List<RecycleChannelAccount> listChannelAccountInfo(Long app, Account account, Boolean enable) {

        //1.查询该账号下面所有的渠道账号;
        List<RecycleChannelAccount> recycleChannelAccountList = recycleChannelAccountRepository.findChannelAccountByAccount(account.getRootParent(), enable);

        if (ValidateUtil.isValid(recycleChannelAccountList)) {

            List<String> accountList = Lists.transform(recycleChannelAccountList, RecycleChannelAccount::getUniqueAccount);
//            String accountIds = String.join(",", accountList);
            String accountIds = "";
             StringBuffer  accountIdSb =  new StringBuffer();
             for(String accountId : accountList){
                 accountIdSb.append("'").append(accountId).append("',");
             }
             if(accountIdSb.length() > 0){
                 accountIds = accountIdSb.substring(0, accountIdSb.length()-1).toString();
             }

            try{
                accountIds = URLEncoder.encode(accountIds, "utf-8");
            }catch (Exception e){

            }
            //2.查询账号的账户信息
            Map<String, Map<String, Object>> accountInfoMap = requestAccountInfoFromReport(app, account, accountIds);

            //3.组合
            for (RecycleChannelAccount recycleChannelAccount : recycleChannelAccountList) {

                Map<String, Object> accountInfo = accountInfoMap.get(recycleChannelAccount.getUniqueAccount());
                accountInfo = (null == accountInfo ? accountInfoMap.get(recycleChannelAccount.getUserName()) : accountInfo);

                if (null == accountInfo) {

                    recycleChannelAccount.setFirstLevelNumber(0L);
                    recycleChannelAccount.setSecondLevelNumber(0L);
                    recycleChannelAccount.setThirdLevelNumber(0L);
                    recycleChannelAccount.setVirtualAccountAmount(0D);
                    recycleChannelAccount.setCashAccountAmount(0D);

                } else {
                    recycleChannelAccount.setFirstLevelNumber((Long) accountInfo.get("num_first_level"));
                    recycleChannelAccount.setSecondLevelNumber((Long) accountInfo.get("num_second_level"));
                    recycleChannelAccount.setThirdLevelNumber((Long) accountInfo.get("num_third_level"));
                    recycleChannelAccount.setVirtualAccountAmount((Double) accountInfo.get("amt_virtual_account") );
                    recycleChannelAccount.setCashAccountAmount((Double) accountInfo.get("amt_cash_account") );
                }
            }
        }

        return recycleChannelAccountList;
    }

    /**
     * 获取报表数据
     */
    private Map<String, Map<String, Object>> requestAccountInfoFromReport(Long app, Account account, String accountIds) {

        Map<String, Map<String, Object>> result = new HashMap<>();

        Map<String, String> condition = new HashMap<>();
        condition.put("startdate", DateUtil.getCurrentDateStr());
        condition.put("accountid", accountIds);
        condition.put("datatype", "list");

        try {

            ReportListResult reportListResult = (ReportListResult) reportService.report(app, "recycle_analysis_account", account, condition);

            if (null != reportListResult && null != reportListResult.getVal()) {
                result = Maps.uniqueIndex(reportListResult.getVal(), val -> val.get("accountid").toString());
            }

        } catch (IOException e) {
            logger.error("get report data error:", e);
        }

        return result;
    }


    @Override
    public Boolean checkChannelNumberLimit(Account account) {

        Boolean result = false;

        Long channelNumber = recycleChannelAccountRepository.findAddedChannelNumber(account.getRootParent());

        if (null != channelNumber) {
            //付费申请
            AdvancedFunctionApply advancedFunctionApply = advancedFunctionApplyService.listApplyByAccount(account);
            if (null != advancedFunctionApply) {

                if (advancedFunctionApply.getStatus().equals(0) || (advancedFunctionApply.getStatus().equals(2) && new Date().after(advancedFunctionApply.getPastDate()))) {
                    //申请未审核 或者 续订为审核,且之前已过期
                    result = channelNumber >= CHANNEL_LIMIT;
                }
            }
        }

        return result;
    }

    @Override
    public RecycleChannelAccount findJRTTToken(Long rootAccountid,Long cid,String accountid) {
//        RecycleChannelAccount channelAccount = recycleChannelAccountRepository.findByUserName(accountid);
    	RecycleChannelAccount channelAccount = recycleChannelAccountRepository.findByRootAccountIdAndUniqueAccountAndCid(rootAccountid, accountid,cid);
        return channelAccount;
    }

    @Override
    public boolean testJRTTToken(Long rootAccountid,Long cid,String accountid) {
        //url编码固定参数
        String ENC =  AccessController.doPrivileged(new GetPropertyAction("file.encoding"));
        boolean result = false;

        //获取今日头条的该id的token
        RecycleChannelAccount channelAccount = findJRTTToken(rootAccountid,cid,accountid);
        try{
            if(channelAccount != null){
                //根据该id的token请求头条api
                String url = "https://ad.toutiao.com/open_api/2/campaign/select/?advertiser_id=" + accountid + "&fields=" + URLEncoder.encode("[\"id\", \"name\",\"status\"]",ENC) + "&offset=" + 10;
                String response = HttpClientUtil.doHttpGetRequest(url, "localhost", channelAccount.getToken());
                org.json.JSONObject obj = new org.json.JSONObject(response);
                int code = obj.getInt("code");
                //code返回0 则为成功请求
                if(code == 0){
                    result = true;
                }
            }
        }catch (Exception e){
            logger.error("" +e.getMessage());
        }
        return result;
    }

    @Override
    public boolean successJRTTAouth(Long rootAccountid,Long cid,String accountid, String channelalias) {
        boolean result = false;
        try{
            RecycleChannelAccount channelAccount = findJRTTToken(rootAccountid,cid,accountid);
            channelAccount.setChannelAlias(channelalias);
            recycleChannelAccountRepository.save(channelAccount);
            return true;
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public boolean saveSpiderAccountAndPassword(String accountid, String channelalias, String password,String mccid,String mcuniquename,String type,Account account) {
        try{


            if(type.equals("baiduyuansheng")){
                String[] mccids = mccid.split(",");
                String[] mcuniquenames = mcuniquename.split(",");

            }else {
                RecycleChannelAccount rc = new RecycleChannelAccount();
                rc.setChannelAlias(channelalias);
                rc.setPassword(password);
                rc.setUserName(accountid);
                rc.setAccount(account.getId());
                rc.setRootAccount(account.getRootParent());
                rc.setStatus(1);
                rc.setTpye(1);
                switch (type){
                    case  "uctoutiao"://uc
                        rc.setCid(530L);
                        rc.setUniqueName("uctoutiao");
                        rc.setChannelName("UC头条");
                        break;
                }
                recycleChannelAccountRepository.save(rc);
                return true;
            }
        }catch (Exception e){
            logger.error(e.getMessage());
        }
        return false;
    }

    @Override
    public Long checkSpiderAccount(String accountid, Account account) {
        long result = recycleChannelAccountRepository.findAccountidAndUserNameNumber(accountid,String.valueOf(account.getId()));
        return result;
    }

}
