
package com.reyun.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.reyun.model.Account;
import com.reyun.model.RecycleChannel;
import com.reyun.model.RecycleChannelAccount;
import com.reyun.repository.RecycleChannelAccountRepository;
import com.reyun.service.RecycleHandleService;
import com.reyun.util.Constant;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import com.reyun.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.reyun.util.*;

import java.net.URLDecoder;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Lzz on 18/8/29.
 * 百度的回本分析service实现类
 */
@Service
public class RecycleBaiduServiceImpl implements RecycleHandleService {

    private Logger logger = LoggerFactory.getLogger(RecycleBaiduServiceImpl.class);
    @Autowired
    private RecycleChannelAccountRepository recycleChannelAccountRepository;



    /**
     *
     * 测试渠道授权(成功插入数据，其他都后续校验时候插入)
     * @return
     */
    @Override
    public ResultModel authChannel(RecycleChannelAccount recycleChannelAccount, Account account, RecycleChannel recycleChannel) {
        if (checkAuthParam(recycleChannelAccount)) {
            String url = Constant.spiderUrl + "/adt/baidu/v1/login_verify";
            Map<String, String> params = new HashMap<>();
            params.put("username", recycleChannelAccount.getN());
            params.put("password", recycleChannelAccount.getW());


            String responseJson = HttpClientUtil.doHttpPostRequestNew(url, "trackingio", params);
            if (!StringUtil.isEmpty(responseJson)) {
                JSONObject json = JSON.parseObject(responseJson);
                int result = Integer.valueOf((String)json.get("result"));
                if (result == 0) {//成功
                    //成功的保存账号
                    String cookie = (String)json.get("cookie");//登陆凭证
                    recycleChannelAccount.setLoginLicense(cookie);
                    recycleChannelAccount = addRecycleChannelAccount(account, recycleChannelAccount, recycleChannel);
                    if (recycleChannelAccount == null) {
                        return ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_DATA_ADD_FAIL);
                    }
                    return ResultModel.OK();
                } else if(result == -1){//需要手机验证
                    return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUNT_NEED_PHOEN_AUTH);
                }else if(result == -2){//需要问题验证
                    //TODO 需要返回前端对应的问题
                    return ResultModel.ERROR(ResultStatus.RECYCLE_ACCOUNT_NEED_QUESTION_AUTH);
                }else if(result == -3){//用户名或者密码错误
                    return ResultModel.ERROR(ResultStatus.USERNAME_OR_PASSWORD_ERROR);
                }else{
                    String errorMess = (String) json.get("errorMessage");
                    return new ResultModel(-9999, errorMess);
                }
            } else {
                String errorMess = "回本分析校验系统反馈异常";
                return new ResultModel(-9999, errorMess);
            }

        }
        return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }

    /**
     * @param recycleChannelAccount
     * @return 校验授权参数是否完整 false不完整
     */
    @Override
    public boolean checkAuthParam(RecycleChannelAccount recycleChannelAccount) {
        boolean channelAccountNotEmpty = recycleChannelAccount != null;
        boolean usernameExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getUserName());
        boolean passwordExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getPassword());
        boolean channelAliasExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getChannelAlias());
        boolean cidExist = channelAccountNotEmpty && recycleChannelAccount.getCid() > 0;

        if(usernameExist && passwordExist && channelAliasExist && cidExist){
            return  true;
        }
        return false;
    }

    /**
     * @param account
     * @param recycleChannelAccount
     * @return  添加渠道账号
     */
    @Override
    public RecycleChannelAccount addRecycleChannelAccount(Account account, RecycleChannelAccount recycleChannelAccount, RecycleChannel recycleChannel) {

        RecycleChannelAccount result = null;

        //百度的唯一账号是username
        recycleChannelAccount.setUniqueAccount(recycleChannelAccount.getUserName());

        //查询渠道账号是否添加过
        RecycleChannelAccount queryResult = recycleChannelAccountRepository.findByRootAccountIdAndUniqueAccountAndCid(account.getRootParent(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());

        boolean accountNoExist = null == queryResult;

        Date now = new Date();
        if(accountNoExist){
            queryResult = new RecycleChannelAccount();
            queryResult.setInsertDate(now);
            queryResult.setUserName(recycleChannelAccount.getUserName());
            queryResult.setUniqueAccount(recycleChannelAccount.getUniqueAccount());
            queryResult.setCid(recycleChannelAccount.getCid());
        }
        queryResult.setAccount(account.getId());
        queryResult.setRootAccount(account.getRootParent());
        queryResult.setUtime(now);
        queryResult.setStatus(1);
        queryResult.setChannelName(recycleChannel.getName());
        queryResult.setUniqueName(recycleChannel.getUniqueName());
        queryResult.setPassword(recycleChannelAccount.getPassword());
        queryResult.setChannelAlias(recycleChannelAccount.getChannelAlias());
        queryResult.setLoginCheck(recycleChannelAccount.getLoginCheck());//存储账户的校验问题
        queryResult.setValidFlag(RecycleChannelAccount.VALID_FLAG_VALID);
        result = recycleChannelAccountRepository.save(queryResult);


        return result;
    }

    /**
     * @param account
     * @return 百度没有针对渠道的个数限制(true 达到限制   false未达到限制)
     */
    @Override
    public Boolean checkChannelNumberLimit(Account account) {
        return false;
    }

    /**
     * @param recycleChannelAccount
     * @return  添加渠道的登陆凭证
     */
    @Override
    public ResultModel addLoginLicense(RecycleChannelAccount recycleChannelAccount) {
       boolean uniqueAccountExist = !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
       boolean cidExist = recycleChannelAccount.getCid() > 0;
       boolean rootAccountExist = recycleChannelAccount.getRootAccount() > 0;
       boolean loginLicenseExist = !StringUtil.isEmpty(recycleChannelAccount.getLoginLicense());
       if(uniqueAccountExist && cidExist && rootAccountExist && loginLicenseExist){
           RecycleChannelAccount channelAccount = recycleChannelAccountRepository.
                   findByRootAccountIdAndUniqueAccountAndCid(recycleChannelAccount.getRootAccount(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());
           if(channelAccount != null){
               channelAccount.setLoginLicense(recycleChannelAccount.getLoginLicense());
               channelAccount.setUtime(new Date());
               recycleChannelAccountRepository.save(channelAccount);
               return ResultModel.OK();
           }else{
               return  ResultModel.ERROR(ResultStatus.RECYCLE_CHANNELACCOUNT_NOT_EXIST);
           }
       }else{
           return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);
       }

    }

    /**
     * @param recycleChannelAccount
     * @return 获取登陆的凭证
     */
    @Override
    public ResultModel getLoginLicense(RecycleChannelAccount recycleChannelAccount) {

        boolean uniqueAccountExist = !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
        boolean cidExist = recycleChannelAccount.getCid() > 0;
        boolean rootAccountExist = recycleChannelAccount.getRootAccount() > 0;

        if(uniqueAccountExist && cidExist && rootAccountExist){
            RecycleChannelAccount channelAccount = recycleChannelAccountRepository.
                    findByRootAccountIdAndUniqueAccountAndCid(recycleChannelAccount.getRootAccount(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());
            if(channelAccount != null){
                Map<String, String> result = new HashMap<>();
                result.put("loginLicense",channelAccount.getLoginLicense());
                return ResultModel.OK(result);
            }else{
                return  ResultModel.ERROR(ResultStatus.RECYCLE_CHANNELACCOUNT_NOT_EXIST);
            }
        }else{
            return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

    }

    /**
     * @param recycleChannelAccount
     * @return 发送短信（前提username,password,phone）
     */
    @Override
    public ResultModel sendVcode(RecycleChannelAccount recycleChannelAccount) {
        boolean usernameExist = !StringUtil.isEmpty(recycleChannelAccount.getN());
        boolean passwordExist = !StringUtil.isEmpty(recycleChannelAccount.getW());
        String loginCheck = recycleChannelAccount.getLoginCheck();
        if(usernameExist && passwordExist && !StringUtil.isEmpty(loginCheck)){
            JSONObject json = JSON.parseObject(loginCheck);
            String type = (String) json.get("type");
            if(RecycleChannelAccount.LOGIN_CHECK_PHONE.equals(type)){//type = 1的时候为短信校验
                String mobile = !StringUtil.isEmpty((String)json.get("mobile")) ? (String)json.get("mobile") : "";
               if(!StringUtil.isEmpty(mobile)){
                   String url = Constant.spiderUrl + "/adt/baidu/v1/phone/req";
                   Map<String, String> params = new HashMap<>();
                   params.put("username", recycleChannelAccount.getN());
                   params.put("password", recycleChannelAccount.getW());
                   params.put("phone", mobile);
                   String responseJson = HttpClientUtil.doHttpPostRequestNew(url, "trackingio", params);

                   if (!StringUtil.isEmpty(responseJson)) {
                       JSONObject respJson = JSON.parseObject(responseJson);
                       int result = Integer.valueOf((String)respJson.get("result"));
                       if (result == 0) {//成功
                           //成功的校验信息
                           return ResultModel.OK();
                       }
                       else{
                           String errorMess = (String) respJson.get("errorMessage");
                           return new ResultModel(-9999, errorMess);
                       }
                   } else {
                       String errorMess = "回本分析校验系统反馈异常";
                       return new ResultModel(-9999, errorMess);
                   }
                }
            }
        }
        return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }

    /**
     * @param recycleChannelAccount
     * @return  手机验证码验证登陆（需要username,password,phone,code）
     */
    @Override
    public ResultModel phoneVerify(RecycleChannelAccount recycleChannelAccount, Account account,RecycleChannel recycleChannel) {
        boolean usernameExist = !StringUtil.isEmpty(recycleChannelAccount.getN());
        boolean passwordExist = !StringUtil.isEmpty(recycleChannelAccount.getW());
        String loginCheck = recycleChannelAccount.getLoginCheck();
        if(usernameExist && passwordExist && !StringUtil.isEmpty(loginCheck)){
            JSONObject json = JSON.parseObject(loginCheck);
            String type = (String) json.get("type");
            if(RecycleChannelAccount.LOGIN_CHECK_PHONE.equals(type)){//type = 1的时候为短信校验
                String mobile =  !StringUtil.isEmpty((String)json.get("mobile")) ? (String)json.get("mobile") : "";
                String vcode =  !StringUtil.isEmpty((String)json.get("vcode")) ? (String)json.get("vcode") : "";

                if(!StringUtil.isEmpty(mobile) && !StringUtil.isEmpty(vcode)){
                    String url = Constant.spiderUrl + "/adt/baidu/v1/phone/verify";
                    Map<String, String> params = new HashMap<>();
                    params.put("username", recycleChannelAccount.getN());
                    params.put("password", recycleChannelAccount.getW());
                    params.put("phone", mobile);
                    params.put("code", vcode);
                    String responseJson = HttpClientUtil.doHttpPostRequestNew(url, "trackingio", params);

                    if (!StringUtil.isEmpty(responseJson)) {
                        JSONObject respJson = JSON.parseObject(responseJson);
                        int result = Integer.valueOf((String)respJson.get("result"));
                        if (result == 0) {//成功
                            //成功的保存账号
                            String cookie = (String)json.get("cookie");//登陆凭证
                            recycleChannelAccount.setLoginLicense(cookie);
                            recycleChannelAccount = addRecycleChannelAccount(account, recycleChannelAccount, recycleChannel);
                            if (recycleChannelAccount == null) {
                                return ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_DATA_ADD_FAIL);
                            }
                            return ResultModel.OK();
                        }
                        else{
                            String errorMess = (String) respJson.get("errorMessage");
                            return new ResultModel(-9999, errorMess);
                        }
                    } else {
                        String errorMess = "回本分析校验系统反馈异常";
                        return new ResultModel(-9999, errorMess);
                    }
                }
            }
        }
        return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }

    @Override
    public ResultModel questionVerify(RecycleChannelAccount recycleChannelAccount, Account account,RecycleChannel recycleChannel) {
        boolean usernameExist = !StringUtil.isEmpty(recycleChannelAccount.getUserName());
        boolean passwordExist = !StringUtil.isEmpty(recycleChannelAccount.getPassword());
        boolean cidExist = recycleChannelAccount != null && recycleChannelAccount.getCid() > 0;

        String loginCheck = recycleChannelAccount.getLoginCheck();
        if(usernameExist && passwordExist && cidExist && !StringUtil.isEmpty(loginCheck)){
            JSONObject json = JSON.parseObject(loginCheck);
            String type = (String) json.get("type");
            if(RecycleChannelAccount.LOGIN_CHECK_QUESTION.equals(type)){//type = 2的时候为问题校验
                String id =!StringUtil.isEmpty((String)json.get("id")) ? (String)json.get("id") : "";
                String question =  !StringUtil.isEmpty((String)json.get("question")) ? (String)json.get("question") : "";
                String answer =  !StringUtil.isEmpty((String)json.get("answer")) ? (String)json.get("answer") : "";


                if(!StringUtil.isEmpty(id) && !StringUtil.isEmpty(question) && !StringUtil.isEmpty(answer)){
                    String url = Constant.spiderUrl + "/adt/baidu/v1/question/verify";
                    Map<String, String> params = new HashMap<>();
                    params.put("username", recycleChannelAccount.getN());
                    params.put("password", recycleChannelAccount.getW());
                    params.put("id", id);
                    params.put("question", question);
                    params.put("answer", answer);
                    String responseJson = HttpClientUtil.doHttpPostRequestNew(url, "trackingio", params);

                    if (!StringUtil.isEmpty(responseJson)) {
                        JSONObject respJson = JSON.parseObject(responseJson);
                        int result = Integer.valueOf((String)respJson.get("result"));
                        if (result == 0) {
                            //成功的保存账号
                            String cookie = (String)json.get("cookie");//登陆凭证
                            recycleChannelAccount.setLoginLicense(cookie);
                            recycleChannelAccount = addRecycleChannelAccount(account, recycleChannelAccount, recycleChannel);
                            if (recycleChannelAccount == null) {
                                return ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_DATA_ADD_FAIL);
                            }
                            return ResultModel.OK();
                        }
                        else{
                            String errorMess = (String) json.get("errorMessage");
                            return new ResultModel(-9999, errorMess);
                        }
                    } else {
                        String errorMess = "回本分析校验系统反馈异常";
                        return new ResultModel(-9999, errorMess);
                    }
                }
            }
        }
        return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }
}
