
package com.reyun.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.dic.RecycleChannelEnum;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AdvancedFunctionApplyService;
import com.reyun.service.RecycleAnalysisService;
import com.reyun.service.RecycleHandleService;
import com.reyun.service.ReportService;
import com.reyun.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import sun.security.action.GetPropertyAction;

import java.io.IOException;
import java.net.URLEncoder;
import java.security.AccessController;
import java.util.*;

/**
 * Created by Lzz on 18/8/27.
 * UC头条的回本分析service实现类
 */
@Service
public class RecycleUcttServiceImpl implements RecycleHandleService {

    private Logger logger = LoggerFactory.getLogger(RecycleUcttServiceImpl.class);
    @Autowired
    private RecycleChannelAccountRepository recycleChannelAccountRepository;



    /**
     *
     * 测试渠道授权
     * @return
     */
    @Override
    public ResultModel authChannel(RecycleChannelAccount recycleChannelAccount, Account account, RecycleChannel recycleChannel) {
        if (checkAuthParam(recycleChannelAccount)) {
            String url = Constant.spiderUrl + "/adt/uc/v1/login_verify";
            Map<String, String> params = new HashMap<>();
            params.put("username", recycleChannelAccount.getN());
            params.put("password", recycleChannelAccount.getW());
            String responseJson = HttpClientUtil.doHttpPostRequestNew(url, "trackingio", params);

            if (!StringUtil.isEmpty(responseJson)) {
                JSONObject json = JSON.parseObject(responseJson);
                int result = Integer.valueOf((String)json.get("result"));
                if (result == 0) {
                    //成功的保存账号
                    String cookie = (String)json.get("cookie");//登陆凭证
                    recycleChannelAccount.setLoginLicense(cookie);
                    recycleChannelAccount = addRecycleChannelAccount(account, recycleChannelAccount, recycleChannel);
                    if (recycleChannelAccount == null) {
                        return ResultModel.ERROR(ResultStatus.ECYCLE_ACCOUN_DATA_ADD_FAIL);
                    }
                    return ResultModel.OK();
                } else {
                    String errorMess = (String) json.get("errorMessage");
                    return new ResultModel(-9999, errorMess);
                }
            } else {
                String errorMess = "回本分析校验系统反馈异常";
                return new ResultModel(-9999, errorMess);
            }

        }
        return ResultModel.ERROR(ResultStatus.PARAM_INVALID);
    }

    /**
     * @param recycleChannelAccount
     * @return 校验授权参数是否完整 false不完整
     */
    @Override
    public boolean checkAuthParam(RecycleChannelAccount recycleChannelAccount) {
        boolean channelAccountNotEmpty = recycleChannelAccount != null;
        boolean usernameExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getUserName());
        boolean passwordExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getPassword());
        boolean channelAliasExist = channelAccountNotEmpty && !StringUtil.isEmpty(recycleChannelAccount.getChannelAlias());
        boolean cidExist = channelAccountNotEmpty && recycleChannelAccount.getCid() > 0;

        if(usernameExist && passwordExist && channelAliasExist && cidExist){
            return  true;
        }
        return false;
    }

    /**
     * @param account
     * @param recycleChannelAccount
     * @return  添加渠道账号
     */
    @Override
    public RecycleChannelAccount addRecycleChannelAccount(Account account, RecycleChannelAccount recycleChannelAccount, RecycleChannel recycleChannel) {

        RecycleChannelAccount result = null;

        //Uc头条的唯一账号是username
        recycleChannelAccount.setUniqueAccount(recycleChannelAccount.getUserName());

        //查询渠道账号是否添加过
        RecycleChannelAccount queryResult = recycleChannelAccountRepository.findByRootAccountIdAndUniqueAccountAndCid(account.getRootParent(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());

        boolean accountNoExist = null == queryResult;

        Date now = new Date();
        if(accountNoExist){
            queryResult = new RecycleChannelAccount();
            queryResult.setInsertDate(now);
            queryResult.setUserName(recycleChannelAccount.getUserName());
            queryResult.setUniqueAccount(recycleChannelAccount.getUniqueAccount());
            queryResult.setCid(recycleChannelAccount.getCid());
        }

        queryResult.setAccount(account.getId());
        queryResult.setRootAccount(account.getRootParent());

        queryResult.setUtime(now);
        queryResult.setStatus(1);
        queryResult.setChannelName(recycleChannel.getName());
        queryResult.setUniqueName(recycleChannel.getUniqueName());

        queryResult.setPassword(recycleChannelAccount.getPassword());
        queryResult.setChannelAlias(recycleChannelAccount.getChannelAlias());
        queryResult.setValidFlag(RecycleChannelAccount.VALID_FLAG_VALID);
        result = recycleChannelAccountRepository.save(queryResult);


        return result;
    }

    /**
     * @param account
     * @return Uc头条没有针对渠道的个数限制(true 达到限制   false未达到限制)
     */
    @Override
    public Boolean checkChannelNumberLimit(Account account) {
        return false;
    }

    /**
     * @param recycleChannelAccount
     * @return  添加渠道的登陆凭证
     */
    @Override
    public ResultModel addLoginLicense(RecycleChannelAccount recycleChannelAccount) {
       boolean uniqueAccountExist = !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
       boolean cidExist = recycleChannelAccount.getCid() > 0;
       boolean rootAccountExist = recycleChannelAccount.getRootAccount() > 0;
       boolean loginLicenseExist = !StringUtil.isEmpty(recycleChannelAccount.getLoginLicense());
       if(uniqueAccountExist && cidExist && rootAccountExist && loginLicenseExist){
           RecycleChannelAccount channelAccount = recycleChannelAccountRepository.
                   findByRootAccountIdAndUniqueAccountAndCid(recycleChannelAccount.getRootAccount(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());
           if(channelAccount != null){
               channelAccount.setLoginLicense(recycleChannelAccount.getLoginLicense());
               channelAccount.setUtime(new Date());
               recycleChannelAccountRepository.save(channelAccount);
               return ResultModel.OK();
           }else{
               return  ResultModel.ERROR(ResultStatus.RECYCLE_CHANNELACCOUNT_NOT_EXIST);
           }
       }else{
           return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);
       }

    }

    /**
     * @param recycleChannelAccount
     * @return 获取登陆的凭证
     */
    @Override
    public ResultModel getLoginLicense(RecycleChannelAccount recycleChannelAccount) {

        boolean uniqueAccountExist = !StringUtil.isEmpty(recycleChannelAccount.getUniqueAccount());
        boolean cidExist = recycleChannelAccount.getCid() > 0;
        boolean rootAccountExist = recycleChannelAccount.getRootAccount() > 0;

        if(uniqueAccountExist && cidExist && rootAccountExist){
            RecycleChannelAccount channelAccount = recycleChannelAccountRepository.
                    findByRootAccountIdAndUniqueAccountAndCid(recycleChannelAccount.getRootAccount(), recycleChannelAccount.getUniqueAccount(), recycleChannelAccount.getCid());
            if(channelAccount != null){
                Map<String, String> result = new HashMap<>();
                result.put("loginLicense",channelAccount.getLoginLicense());
                return ResultModel.OK(result);
            }else{
                return  ResultModel.ERROR(ResultStatus.RECYCLE_CHANNELACCOUNT_NOT_EXIST);
            }
        }else{
            return  ResultModel.ERROR(ResultStatus.PARAM_INVALID);
        }

    }

    @Override
    public ResultModel sendVcode(RecycleChannelAccount recycleChannelAccount) {
        return null;
    }

    @Override
    public ResultModel phoneVerify(RecycleChannelAccount recycleChannelAccount, Account account,RecycleChannel recycleChannel) {
        return null;
    }

    @Override
    public ResultModel questionVerify(RecycleChannelAccount recycleChannelAccount, Account account,RecycleChannel recycleChannel) {
        return null;
    }
}
