package com.reyun.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.*;
import com.google.common.base.Optional;
import com.google.common.collect.*;
import com.reyun.dic.*;
import com.reyun.model.*;
import com.reyun.report.*;
import com.reyun.report.parse.*;
import com.reyun.repository.*;
import com.reyun.service.*;
import com.reyun.util.*;
import com.reyun.util.http.HttpPost;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkArgument;

@Service
public class ReportServiceImp implements ReportService {

    private static final Logger logger = LoggerFactory.getLogger(ReportServiceImp.class);

    private final Predicate<Campaign> campaignPackChannelIfNull = new Predicate<Campaign>() {
        @Override
        public boolean apply(Campaign campaign) {
            return Strings.isNullOrEmpty(campaign.getPack_channelid());
        }
    };

    private final Predicate<Campaign> campaignPackChannelIfNotNull = new Predicate<Campaign>() {
        @Override
        public boolean apply(Campaign campaign) {
            return !Strings.isNullOrEmpty(campaign.getPack_channelid());
        }
    };

    private final Predicate<Channel> channelAdvertisingPredicate = new Predicate<Channel>() {
        @Override
        public boolean apply(Channel channel) {
            return ChannelCategoryEnumType.ADVERTISING.getKey().equals(channel.getCategory());
        }
    };

    private final Predicate<Channel> channelAppmarketPredicate = new Predicate<Channel>() {
        @Override
        public boolean apply(Channel channel) {
            return ChannelCategoryEnumType.APPMARKET.getKey().equals(channel.getCategory());
        }
    };

    private Function<Campaign, String> campaignStringFunction = new Function<Campaign, String>() {
        @Override
        public String apply(Campaign campaign) {
            return campaign.getCampaignid();
        }
    };

    private Function<Channel, Long> channelStringFunction = new Function<Channel, Long>() {
        @Override
        public Long apply(Channel channel) {
            return channel.getId();
        }
    };

    @Autowired
    private CampaignService campaignService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    private CampaignGroupService campaignGroupService;
    @Autowired
    private AuthService authService;
    @Autowired
    private AppRepository apprepository;
    @Autowired
    private ConfigParamService configparamService;
    @Autowired
    PackageSubCampaignRepository packageSubCampaignRepository;
    @Autowired
    ChannelRepository channelRepository;
    @Autowired
    CampaignRepository campaignRepository;
    @Autowired
    SubChannelRenameRepository subChannelRenameRepository;
    @Autowired
    SdkEventRepository sdkEventRepository;
    @Autowired
    ConfigParamRepository configParamRepository;
    @Autowired
    SceneRepository sceneRepository;
    @Autowired
    BlacklistApplicationRepository blacklistApplicationRepository;
    @Autowired
    ChannelAdvertisingPackageRepository channelAdvertisingPackageRepository;
    @Autowired
    ChannelAdvertisingRepository channelAdvertisingRepository;
    @Autowired
    DataAuthPkgsubcampaignRepository dataAuthPkgsubcampaignRepository;
    @Autowired
    AccountFunctionAuthRepository accountFunctionAuthRepository;
    @Autowired
    ServeridBackflowRepository serveridBackflowRepository;

    @Override
    public Object homeReport(Map<String, String> conditions) throws IOException {
        String url = Constant.reportUrl + "/api/trackingio/home_dau_bygeo/all";

        List<NameValuePair> valuePairs = Lists.newArrayList();
        conditions.forEach((s, s2) -> {
            valuePairs.add(new NameValuePair(s, s2));
        });

        NameValuePair[] nameValuePairs = new NameValuePair[valuePairs.size()];
        ReportChartResult reportResult = new HttpPost(url)
                .withParamers(valuePairs.toArray(nameValuePairs)).toResult(ReportChartResult.class);
        List<Object[]> val = reportResult.getVal();
        List<String> key = reportResult.getKey();

        List<Object> transList = Lists.newArrayList();
        Map prondauMap = Maps.newHashMap();
        Object[] lon = val.get(0); //经度
        Object[] lat = val.get(1); //纬度
        Object[] dau = val.get(2); //dau
        Object[] province = val.get(3); //dau
        Object[] prondau = val.get(4); //dau
        for (int i = 0; i < dau.length; i++) {
            Map<String, Object> newItem = Maps.newHashMap();
            newItem.put("name", key.get(i));
            newItem.put("value", dau[i]);
            newItem.put("geoCoord", new Object[]{lon[i], lat[i]});
            transList.add(newItem);

            if (!prondauMap.containsKey(province[i])) {
                prondauMap.put(province[i], prondau[i]);
            }
        }

        long totalDau = 0;
        if (dau != null && dau.length > 0) {
            Integer[] dauArray = Arrays.copyOf(dau, dau.length, Integer[].class);
            totalDau = plus(dauArray);
        }

        Map<String, Object> rtnMap = Maps.newHashMap();
        rtnMap.put("totalVal", totalDau);
        rtnMap.put("historyVal", new Double(totalDau * 1.3).intValue());
        rtnMap.put("val", splitVal(transList));
        rtnMap.put("prondau", prondauMap);
        return rtnMap;
    }

    @Override
    public Object reportNumber(Long appid, String reportName, Account account, Map<String, String> conditions) throws IOException {
        ReportHandler reportHandler = new ReportHandler(appid, reportName, account, conditions);

        Object demoData = reportHandler.checkDemoAppDataUrl(reportHandler);

        if (demoData != null) {
            return 0;
        }

        reportHandler.invoke();

        ReportListResult reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(reportHandler.getResponseJson(), ReportListResult.class);
        List<Map<String, Object>> value = reportListResult.getVal();
        if (value == null || value.size() == 0) {
            return 0;
        }

        Map<String, Object> result = value.get(0);
        if (result != null && result.containsKey("number")) {
            return Integer.valueOf(result.get("number").toString());
        }

        return 0;
    }

    @Override
    public Object report(Long appid, String reportName, Account account, Map<String, String> conditions) throws IOException {

        ReportHandler reportHandler = new ReportHandler(appid, reportName, account, conditions);

        Object demoData = reportHandler.checkDemoAppDataUrl(reportHandler);
        logger.info("checkDemoAppDataUrl result={}", demoData);
        if (demoData == null) {
            // 非demo app 再去构建查询参数
            reportHandler.invoke();

        } else {

            return demoData;
        }

        ReportEnumType reportEnumType = reportHandler.getReportEnumType();
        String responseJson = reportHandler.getResponseJson();

        if (conditions.get("api2sql") != null) {
            //返回sql 语句
            JSONObject data = JSON.parseObject(responseJson);
            return data.get("sql");

        }

        if (reportName.equals("bd_bycampaign_sum") || reportName.equals("bd_bycreative_sum")
                || reportName.equals("bd_bykwid_sum") || reportName.equals("subchannel_bysubchannel_sum")
                || "advertisinginfo_adname_rename".equals(reportName)) {

            if ("advertisinginfo_adname_rename".equals(reportName)) {
                ReportListResult reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, ReportListResult.class);
                return reportListResult;
            }

            return responseJson;
        }
        String realtimeReport = configparamService.getConfigParamByKey("realtime_report_name");
        List<String> realtimeReports = Arrays.asList(realtimeReport.split(","));
        if (realtimeReports.contains(reportName)) {
            String currentDateStr = DateUtil.getCurrentDateStr();
            String beforeDays = DateUtil.getBeforeDays(currentDateStr, 1);
            if (conditions.get("enddate").equals(currentDateStr) && conditions.get("startdate").equals(beforeDays)) {
                responseJson = responseJson.replace("$enddate", "今天");
                responseJson = responseJson.replace("$startdate", "昨天");
            } else {
                responseJson = responseJson.replace("$enddate", conditions.get("enddate"));
                responseJson = responseJson.replace("$startdate", conditions.get("startdate"));
            }
        }

        App app = reportHandler.getApp();
        String func = reportHandler.getFunc();
        DataViewEnumType dataViewEnumType = reportHandler.getDataViewEnumType();
        List<String> viewItemList = reportHandler.getViewItemList();
        List<String> webViewItemList = reportHandler.getWebViewItemList();
        List<String> fillCampids = reportHandler.getFillCampids();
        List<String> fillchannels = reportHandler.getFillChannelIds();
        List<String> demoAppList = reportHandler.getDemoAppList();

        Object content = null;
        String reportDesc = null;
        if (ReportEnumType.CHART.equals(reportEnumType)) {
            ReportChartResult reportChartResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, ReportChartResult.class);
            if (reportChartResult.getDesc() != null) {
                reportDesc = reportChartResult.getDesc().get(0);
            }

            if (reportName.startsWith("black_top5_list_")) {

                // 处理反作弊卫士折线图切换至列表数据的数据格式处理
                ReportChartResult result = new ChartTransNameParse().parse(reportChartResult, app.getId(), account.getId(), reportName);
                //汇总 列表数据
                content = new FakeTop5ListParse().parseByChart(result, conditions, reportName);

            } else if (reportName.startsWith("black_top5_mtti_")) {
                //反作弊卫士折线图数据处理
                content = new ChartReportResultBuilder().doChartReport(reportChartResult, app.getId(), account, reportName, conditions);
            } else if (conditions.get("func") != null && conditions.get("func").equals("duration") && conditions.get("viewtype") != null) {
                //参与度
                if (conditions.get("viewtype").equals("chart")) {
                    content = new ChartReportResultBuilder().doChartReport(reportChartResult, app.getId(), account, reportName, conditions);
                } else {
                    if (conditions.containsKey("duration_cid") && StringUtil.isEmpty(conditions.get("duration_cid"))) {
                        List<Channel> channels = authService.getAuthChannelListByAppIngnorDel(account.getId(), appid);
                        StringBuilder sb = new StringBuilder();
                        channels.stream().forEach(channel -> {
                            sb.append(channel.getId()).append(",");
                        });
                        sb.append("-1");
                        conditions.put("duration_cid", sb.toString());
                    }
                    ReportChartResult result = new ChartTransNameParse().parse(reportChartResult, app.getId(), account.getId(), reportName);
                    content = new DurationListParse().parseByChart(result, conditions, reportName);
                }
            } else if (conditions.get("func") != null && conditions.get("func").equals("terminal") && conditions.get("viewtype") != null) {
                //参与度
                if (conditions.get("viewtype").equals("chart")) {
                    content = new ChartReportResultBuilder().doChartReport(reportChartResult, app.getId(), account, reportName, conditions);
                } else if (conditions.get("viewtype").equals("list")) {
                    //ReportChartResult result = new ChartTransNameParse().parse(reportChartResult, app.getId(), account.getId(), reportName);
                    // content = new TerminalListParse().parse(reportChartResult, app.getId(), account.getId(), null, reportName, false, null, null, conditions);
                    content = new TerminalListParse().parseByList(reportChartResult, app, account, reportName, conditions);
                } else {
                    content = new ChartReportResultBuilder().doChartReport(reportChartResult, app.getId(), account, reportName);
                }
            } else {
                content = new ChartReportResultBuilder().doChartReport(reportChartResult, app.getId(), account, reportName);
            }

        } else if (ReportEnumType.LIST.equals(reportEnumType)) {

            ReportListResult listResult = null;
            ReportListResult reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, ReportListResult.class);
            if (reportListResult.getDesc() != null) {
                reportDesc = reportListResult.getDesc().get(0);
            }

            if ("campaigninfo".equals(func)) {
                List<String> searchIds = Lists.newArrayList();
                if (conditions.containsKey("searchtxt")) {
                    if (DataViewEnumType.CAMPAIGN.equals(dataViewEnumType) || DataViewEnumType.CAMPAIGN_H5.equals(dataViewEnumType)) {
                        searchIds = reportHandler.getFillCampids();
                    } else if (DataViewEnumType.CAMPAIGNGROUP.equals(dataViewEnumType)) {
                        searchIds = reportHandler.getFillCampGroupIds();
                    } else if (DataViewEnumType.CHANNEL.equals(dataViewEnumType)) {
                        searchIds = reportHandler.getFillChannelIds();
                    } else if (DataViewEnumType.SUBCHANNEL.equals(dataViewEnumType)) {
                        searchIds = reportHandler.getFillChannelIds();
                    } else if (DataViewEnumType.BDKEYWORD.equals(dataViewEnumType)) {

                    } else if (DataViewEnumType.CAMPGROUP_DATE.equals(dataViewEnumType)) {
                        searchIds = reportHandler.getFillCampGroupIds();
                    }
                }
                // 分包推广活动详情单独处理
                if ("campaigninfo_bypackagecampaign".equals(reportName)) {
                    dataViewEnumType = null;
                    String campaignid = conditions.get("campaignid");
                    if (!StringUtils.isEmpty(campaignid)) {
                        String[] split = campaignid.split(",");
                        if (conditions.containsKey("searchtxt")) {
                            searchIds = Arrays.asList(split);
                        } else {
                            fillCampids = Arrays.asList(split);
                        }
                    }
                }

                ArrayList<String> con = new ArrayList<>();
                if ("subchannel_bysubchannel".equals(reportName)) {

                    if (!demoAppList.contains(app.getAppkey())) {
                        con.add(conditions.get("cid"));
                        con.add(conditions.get("ss_keyword") != null ? URLDecoder.decode(conditions.get("ss_keyword"), StandardCharsets.UTF_8.toString()) : "null");
                    } else {
                        con = new ArrayList<>();
                        con.add("1");
                        con.addAll(viewItemList);
                        conditions.put("isdemo_source", "true");
                    }


                    if (conditions.containsKey("searchtxt") && !StringUtil.isEmpty(conditions.get("searchtxt"))) {
                        conditions.put("s_keyword", conditions.get("searchtxt"));
                    }
                }

                if (reportName.equals("bd_bycreative") || reportName.equals("bd_bykwid")) {
                    Campaign cam = campaignRepository.findCampaignByCampaignId(conditions.get("campaignid"), appid);
                    try {
                        listResult = new CampaignReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                                viewItemList, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), searchIds, cam != null ? cam.getChannel() : null, conditions);
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                } else if ("subchannel_bysubchannel".equals(reportName)) {
                    listResult = new CampaignReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                            con, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), searchIds, conditions);
                } else if ("funnel_user_event_list".equals(reportName) || "funnel_user_event_campain_list".equals(reportName)) {

                    if ("funnel_user_event_campain_list".equals(reportName)) {
                        listResult = new FunnelReportBuilder().doListReportExport(reportListResult, dataViewEnumType, appid, account,
                                con, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), searchIds, conditions);

                    } else {
                        listResult = new FunnelReportBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                                con, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), searchIds, conditions);

                    }

                } else {
                    listResult = new CampaignReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                            viewItemList, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), searchIds, conditions);
                }

                if (dataViewEnumType != null && DataViewEnumType.CAMPGROUP_DATE.getCode().equals(dataViewEnumType.getCode())) {
                    webViewItemList.add(1, "campaignid");
                }

                List<String> name = listResult.getName();
                ArrayList<String> newName = new ArrayList<>();
                //create by 2018-07-21 发现name存在null的情况
                boolean nameIsNull = name == null ? true : false;
                String sdkevent = configParamRepository.findParamsByKey("sdkevent");
                if ("campaign".equals(conditions.get("viewtype")) || "date".equals(conditions.get("viewtype")) || "channel".equals(conditions.get("viewtype"))
                        || DataViewEnumType.CHANNLE_DATE.getCode().equals(conditions.get("viewtype")) || "campgroup".equals(conditions.get("viewtype"))
                        || "account".equals(conditions.get("viewtype")) || "campaigndate".equals(conditions.get("viewtype")) || "campgroupdate".equals(conditions.get("viewtype"))) {
                    this.setEventName(nameIsNull, name, newName, appid, sdkevent);
                } else {
                    if (!nameIsNull) {
                        for (String s : name) {
                            if (!sdkevent.contains(s)) {
                                newName.add(s);
                            }
                        }
                    }
                }
                if (!"funnel_user_event_campain_list".equals(reportName)) {
                    listResult.setName(newName);
                }

                if ("campaigninfo_bysubpackagecampaign".equals(reportName) && conditions.containsKey("result_first_name_replace")) {
                    listResult.getName().remove(0);
                    listResult.getName().add(0, "分包子活动");
                }

                if ("campaign".equals(conditions.get("viewtype")) || "campgroup".equals(conditions.get("viewtype")) || "subaccount".equals(conditions.get("viewtype"))) {
                    List<Map<String, Object>> val = listResult.getVal();
                    if (val != null && val.size() > 1) {
                        Collections.sort(val, new Comparator<Map<String, Object>>() {
                            @Override
                            public int compare(Map<String, Object> o1, Map<String, Object> o2) {
                                Long num_click_total_o1 = o1.get("num_click_total") != null ? (Long) o1.get("num_click_total") : 0L;
                                Long num_click_total_o2 = o2.get("num_click_total") != null ? (Long) o2.get("num_click_total") : 0L;
                                return new Long(-(num_click_total_o1 - num_click_total_o2)).intValue();
                            }
                        });
                        listResult.setVal(val);
                    }
                }
            } else if ("roi".equals(func)) {
                List<String> searchIds = Lists.newArrayList();
                if (conditions.containsKey("searchtxt")) {
                    searchIds = reportHandler.getFillCampids();
                }
                listResult = new RoiReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                        viewItemList, reportName, conditions, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), null);
            } else if ("advertisinginfo".equals(func) || "adsenseinfo".equals(func)) {

                listResult = new ChannelAdvertisingReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                        viewItemList, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), new ArrayList<>(), conditions);


                if (!"advertisinginfo_cash_day".equals(reportName)) {
                    List<String> name = listResult.getName();
                    ArrayList<String> newName = new ArrayList<>();
                    String sdkevent = configParamRepository.findParamsByKey("sdkevent");
                    if (name != null) {
                        setEventName(name == null, name, newName, appid, sdkevent);
                    }
                    listResult.setName(newName);
                }

            } else if ("remarketing".equals(func)) {
                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                webViewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                listResult = new RemarketingReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                        viewItemList, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), new ArrayList<>(), conditions);
                List<String> name = listResult.getName();
                ArrayList<String> newName = new ArrayList<>();
                boolean nameIsNull = name == null ? true : false;
                String sdkevent = configParamRepository.findParamsByKey("sdkevent");
                this.setEventName(nameIsNull, name, newName, appid, sdkevent);
                listResult.setName(newName);
            } else if ("refluxanalysis".equals(func)) {
                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                webViewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                for (String viewitem : viewItemList) {
                    conditions.put(viewitem, "");
                }
                listResult = new RefluxanalysisReportResultBuilder().doListReport(reportListResult, dataViewEnumType, appid, account,
                        viewItemList, reportName, fillCampids, !Strings.isNullOrEmpty(conditions.get("searchtxt")), new ArrayList<>(), conditions);
                List<String> name = listResult.getName();
                ArrayList<String> newName = new ArrayList<>();
                boolean nameIsNull = name == null ? true : false;
                String sdkevent = configParamRepository.findParamsByKey("sdkevent");
                this.setEventName(nameIsNull, name, newName, appid, sdkevent);
                listResult.setName(newName);
            } else if ("user_life_cycle".equals(func)) {

                content = new UserLifeCycleResultBuilder().doListReport(reportListResult, app, account, reportName, conditions);
                return content;

            } else if ("channelinfo".equals(func)) {
                if ("channel".equals(conditions.get("viewtype"))) {
                    content = new UserLifeCycleResultBuilder().doChannelListReport(reportListResult, app, account, reportName, conditions, fillchannels);
                    return content;
                } else {
                    content = new UserLifeCycleResultBuilder().doCampaignListReport(reportListResult, app, account, reportName, conditions, fillCampids);
                    return content;
                }

            } else if ("costrecovery".equals(func)) {
                try {
                    listResult = new CostRecoveryParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            } else if ("asm".equals(func)) {
                listResult = new ASMParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);
            } else if ("asmdetail".equals(func)) {
                listResult = new ASMDetailParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);
            } else if (reportName.startsWith("black_click_")
                    && !reportName.equals("black_click_channel_total") && !reportName.equals("black_click_byappid")) {

                if (reportName.endsWith("campaign_v20")) {
                    listResult = new FakeByCampaignParse().parse(reportListResult, appid);
                } else if (reportName.endsWith("campaign")) {
                    listResult = new FakeByCampaignParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);

                } else if (reportName.endsWith("channel_v20")) {
                    listResult = new FakeByChannelParse().parse(reportListResult);
                } else if (reportName.endsWith("channel")) {
                    listResult = new FakeByChannelParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);

                } else if (reportName.endsWith("ds_v20")) {
                    listResult = new FakeByDsParse().parse(reportListResult);
                } else if (reportName.endsWith("ds")) {
                    listResult = new FakeByDsParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);
                }
            } else {
                listResult = new ListTransNameParse().parse(reportListResult, appid, account.getId(), null, reportName, false, null);
                if ("device_bymodel".equals(reportName)) {
                    if (conditions.containsKey("ismanufacturer")) {
                        listResult.getName().remove(0);
                        listResult.getName().add(0, "设备制造商");
                    }
                }
            }

            List<Object> columnKey_base = listResult.getColumnkey();
            List<Object> checkBase = new ArrayList<>();
            List<String> nameList_base = listResult.getName();
            List<String> namelistNew_Base = new ArrayList<>();
            //处理report汇总依赖指标带出来多余的指标
            List<String> funcs = new ArrayList<>();
            funcs.add("campaigninfo");
            funcs.add("remarketing");
            funcs.add("advertisinginfo");
            List<String> columnKey_bases = new ArrayList<>();
            columnKey_bases.add("ds");
            columnKey_bases.add("campaignid");
            columnKey_bases.add("cid");
            columnKey_bases.add("channelname");
            columnKey_bases.add("campaignname");
            columnKey_bases.add("subchannel");
            columnKey_bases.add("kwid");
            columnKey_bases.add("keywordname");
            columnKey_bases.add("creative");
            columnKey_bases.add("creativename");
            columnKey_bases.add("siteid");
            columnKey_bases.add("aid");
            columnKey_bases.add("csite");
            columnKey_bases.add("sp_account_number");
            columnKey_bases.add("name");
            columnKey_bases.add("advertiserid_name");
            columnKey_bases.add("adv_id");
            columnKey_bases.add("adplatform");
            columnKey_bases.add("adv_name");
            columnKey_bases.add("adgroup_id4web");
            columnKey_bases.add("adplan_id4web");
            columnKey_bases.add("AAUS0");
            columnKey_bases.add("AAUS1");
            columnKey_bases.add("AAUS2");
            columnKey_bases.add("AAUS3");
            columnKey_bases.add("AAUS4");
            columnKey_bases.add("AAUS5");
            columnKey_bases.add("AAUS6");
            columnKey_bases.add("AAUS7");
            List<String> viewType_bases = new ArrayList<>();
            viewType_bases.add("campaign_h5");
            viewType_bases.add("scenes");
            if ("adsenseinfo".equals(func)) {
                Iterator<Object> it = columnKey_base.iterator();
                while (it.hasNext()) {
                    String column = it.next().toString();
                    if (column.startsWith("num_newuser_reten_d")) it.remove();
                }
            }
            if (columnKey_base != null) {
                for (int i = 0; i < columnKey_base.size(); i++) {

                    if ("advertiserid".equals(columnKey_base.get(i))) {
                        continue;
                    }
                    if (funcs.contains(func) && !viewType_bases.contains(conditions.get("viewtype"))) {
                        if (webViewItemList.contains(columnKey_base.get(i)) || columnKey_bases.contains(columnKey_base.get(i))) {
                            checkBase.add(columnKey_base.get(i));
                            namelistNew_Base.add(nameList_base.get(i));
                        }
                    } else {
                        checkBase.add(columnKey_base.get(i));
                        namelistNew_Base.add(nameList_base.get(i));
                    }
                }
            }
            if (!"funnel_user_event_campain_list".equals(reportName)) {
                listResult.setColumnkey(checkBase);
                listResult.setName(namelistNew_Base);
            }

            //去掉只访问一个页面的次数展示（微信推广详情）
            boolean reportNameIsWxCam = "wx_campaigninfo_byds".equals(reportName);
            if (listResult != null && listResult.getColumnkey() != null && listResult.getName() != null && reportNameIsWxCam) {
                List<Object> columnKey = listResult.getColumnkey();
                List<Object> ckNew = new ArrayList<>();
                List<String> nameList = listResult.getName();
                List<String> namelistNew = new ArrayList<>();
                for (int i = 0; i < columnKey.size(); i++) {
                    if (!"num_visits_one".equals((String) columnKey.get(i))) {
                        ckNew.add(columnKey.get(i));
                        namelistNew.add(nameList.get(i));
                    }

                }
                listResult.setColumnkey(ckNew);
                listResult.setName(namelistNew);
            }


            //create by 20180720---增加针对全0数据的筛选
            List<Map<String, Object>> val = listResult.getVal();
            if (val != null && val.size() > 1) {
                Iterator<Map<String, Object>> iterator = val.iterator();
                AbstractReportParse parse = new ListCampaignDateParse();
                while (iterator.hasNext()) {
                    Map<String, Object> valPart = (Map<String, Object>) iterator.next();
                    boolean isall0 = true;//默认是全0数据(备注：因为有一些数据不存在该字段，但是作为全0数据导出)
                    boolean showall0 = true;//默认显示全0数据
                    if (valPart.containsKey("num_install_fake") && valPart.containsKey("num_install") && !reportName.startsWith("black_")) {
                        Long num_install_fake = (Long) valPart.get("num_install_fake");
                        Long num_install = (Long) valPart.get("num_install");
                        valPart.put("rate_install_fake", parse.divideRateInstallFake(num_install_fake, num_install));
                    }
                    if (valPart.containsKey("isall0")) {
                        isall0 = (boolean) valPart.get("isall0");
                    }

                    if (conditions.containsKey("showall0")) {
                        showall0 = Boolean.valueOf(conditions.get("showall0"));
                    }

                    if (!showall0 && isall0) {
                        iterator.remove();
                    }
                }
                listResult.setVal(val);
            }
            //-------------------------------------end


            //国际化处理
            List<String> names = listResult.getName();
            boolean isEn = conditions != null && LanguageTypeEnum.ENGLISH.getCode().equals(conditions.get(Constant.LOCALE_KEY));

            //处理特殊返回的列
            if (names != null && names.size() > 0 && isEn) {
                List<String> newNames = new ArrayList<>();
                List<String> newNamesPay = new ArrayList<>();
                for (String str : names) {
                    if ("付费激活周期".equals(str)) {
                        str = "paid install cycles";
                    }
                    if (str.indexOf("付费周期") > 0 && str.indexOf("第") == 0) {
//                        str = str.substring(1,2)+" install cycle";
                        newNamesPay.add(str);
                    } else {

                        if (str.indexOf("新增用户付费人数") > 0) {
                            str = str.substring(0, 11) + "New Paid Users";
                        }
                        if (str.indexOf("新增用户付费金额") > 0) {
                            str = str.substring(0, 11) + "New User Payments";
                        }
                        newNames.add(str);
                    }
                }

                for (int i = 0; i < newNamesPay.size(); i++) {
                    int index = i + 1;
                    if (index == 1) {
                        newNames.add("1st install cycle");
                    } else if (index == 2) {
                        newNames.add("2nd install cycle");
                    } else if (index == 3) {
                        newNames.add("3rd install cycle");
                    } else {
                        newNames.add(index + "th install cycle");
                    }
                }

                listResult.setName(newNames);
            }

            //处理特殊的返回值
            List<Map<String, Object>> vals = listResult.getVal();
            if (vals != null && vals.size() > 0 && isEn) {
                for (Map<String, Object> value : vals) {
                    for (String keyStr : value.keySet()) {
                        Object resultStr = value.get(keyStr);
                        if ("汇总".equals(String.valueOf(resultStr))) {
                            value.put(keyStr, "total");
                        }
                    }
                }
                listResult.setVal(vals);
            }

            //end
            content = listResult;
        }

        String desc = StringUtil.isEmpty(reportDesc) ? "" : "(" + reportDesc + ")";

        if ("selfBuilt".equals(app.getPlatform())) {
            //自建站类的app表头替换
            List<String> nameList = new ArrayList<>();
            Map<String, String> nameMap = new HashMap<>(7);
            nameMap.put("激活率", "转换率");
            nameMap.put("排重激活设备数", "排重表单设备数");
            nameMap.put("异常激活率", "异常转化率");
            nameMap.put("异常激活设备数", "异常表单设备数");
            nameMap.put("设备作弊激活数", "设备作弊表单数");
            nameMap.put("激活设备数", "表单设备数");
            nameMap.put("激活量", "表单量");
            if (content instanceof ReportListResult) {
                ReportListResult reportResult = (ReportListResult) content;
                for (String name : reportResult.getName()) {
                    if (nameMap.containsKey(name)) {
                        name = nameMap.get(name);
                    }
                    nameList.add(name);
                }
                reportResult.setName(nameList);
            } else if (content instanceof ReportChartResult) {
                ReportChartResult reportResult = (ReportChartResult) content;
                for (String name : reportResult.getName()) {
                    if (nameMap.containsKey(name)) {
                        name = nameMap.get(name);
                    }
                    nameList.add(name);
                }
                reportResult.setName(nameList);
            }


        }
        return content;
    }

    private void setEventName(boolean nameIsNull, List<String> name, List<String> newName, long appid, String sdkevent) {
        if (!nameIsNull) {
            for (String s : name) {
                SdkEvent sdkEvent = sdkEventRepository.findByEventNameAndAppid(s, appid);
                if (sdkEvent != null) {
                    newName.add(sdkEvent.getEventAlias() + "设备数");
                } else if (sdkevent.contains(s) && sdkEvent == null) {
                    if (s.startsWith("dupnum_event")) {
                        newName.add(s.substring(7, s.length()) + "设备数");
                    } else if (s.startsWith("num_event")) {
                        sdkEvent = sdkEventRepository.findByEventNameAndAppid("dup" + s, appid);
                        newName.add(sdkEvent != null ? sdkEvent.getEventAlias() + "总数" : s.substring(4, s.length()) + "总数");
                    }
                } else {
                    newName.add(s);
                }
            }
        }
    }

    private List<Object[]> splitVal(List<Object> itemList) {
        List<Object[]> rtnList = Lists.newArrayList();
        //数据区间分布: 2:4:4
        int len = itemList.size() % 5 == 0 ? itemList.size() / 5 : itemList.size() / 5 + 1;
        final List<Object> valObjects = itemList.subList(0, len);
        if (valObjects != null && valObjects.size() > 0) {
            rtnList.add(valObjects.toArray());
        } else {
            rtnList.add(new Object[0]);
        }

        List<Object> sublist = itemList.subList(len, itemList.size());
        if (sublist != null && sublist.size() > 0) {
            final List<List<Object>> partition = Lists.partition(sublist, sublist.size() % 2 == 0 ? sublist.size() / 2 : sublist.size() / 2 + 1);
            for (int i = 0; i < partition.size(); i++) {
                rtnList.add(partition.get(i).toArray());
            }
            if (rtnList.size() < 3) {
                rtnList.add(new Object[0]);
            }
        } else {
            rtnList.add(new Object[0]);
            rtnList.add(new Object[0]);
        }
        return rtnList;
    }

    private long plus(Integer[] intVal) {
        long total = 0;
        if (intVal == null || intVal.length == 0) {
            return total;
        }

        for (Integer tmp : intVal) {
            total += tmp.intValue();
        }

        return total;
    }

    /**
     * 内部数据预处理类
     */
    private class ReportHandler {
        private Long appid;
        private String reportName;
        private Account account;
        private Map<String, String> conditions;
        private App app;
        private List<String> fillCampids;
        private List<String> fillCampGroupIds;
        private List<String> fillChannelIds;
        private ReportEnumType reportEnumType;
        private DataViewEnumType dataViewEnumType;
        private String func;
        private List<String> viewItemList;
        private List<String> webViewItemList;
        private String url;
        private List<String> demoAppList;
        private String responseJson;

        public ReportHandler(Long appid, String reportName, Account account, Map<String, String> conditions) {
            this.appid = appid;
            this.reportName = reportName;
            this.account = account;
            this.conditions = conditions;
        }

        public App getApp() {
            return app;
        }

        public List<String> getFillCampids() {
            return fillCampids;
        }

        public ReportEnumType getReportEnumType() {
            return reportEnumType;
        }

        public DataViewEnumType getDataViewEnumType() {
            return dataViewEnumType;
        }

        public String getFunc() {
            return func;
        }

        public List<String> getViewItemList() {
            return viewItemList;
        }

        public List<String> getWebViewItemList() {
            return webViewItemList;
        }

        public String getUrl() {
            return url;
        }

        public List<String> getDemoAppList() {
            return demoAppList;
        }

        public String getResponseJson() {
            return responseJson;
        }

        public List<String> getFillCampGroupIds() {
            return fillCampGroupIds;
        }

        public List<String> getFillChannelIds() {
            return fillChannelIds;
        }

        public boolean isNumeric(String str) {
            for (int i = 0; i < str.length(); i++) {
                System.out.println(str.charAt(i));
                if (!Character.isDigit(str.charAt(i))) {
                    return false;
                }
            }
            return true;
        }

        private Object checkDemoAppDataUrl(ReportHandler reportHandler) {


            if (reportName.endsWith("_by")) {
                logger.info("reportName[{}]不完整的报表名称", reportName);
                return -1001;
            }


            app = apprepository.findOne(appid);
            String demoApps = configparamService.getConfigParamByKey("demo_appkey");
            demoAppList = Arrays.asList(demoApps.split(","));
            if (demoAppList.contains(app.getAppkey())) {
                //避免请求中，参数太长，导致请求失败(其实这里完全没必要传这两个参数)
                conditions.remove("campaignid");
                if (!reportName.startsWith("advertisinginfo_"))
                    conditions.remove("cid");

                reportEnumType = fetchReportEnumType(conditions);
                dataViewEnumType = fetchDataViewEnumType(conditions);
                func = conditions.get("func");

                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                if ("subchannel".equals(conditions.get("viewtype"))) {
                    p_viewcloumn = p_viewcloumn.replaceAll("dupnum_reged_mon", "dupnum_reg");
                }
                webViewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                List<String> appendlinkfieldlist = Lists.newArrayList();
                for (String viewitem : viewItemList) {
                    conditions.put(viewitem, "");
                    Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewitem);
                    if (linkfields != null && linkfields.size() > 0) {
                        for (String linkfield : linkfields) {
//                            if(!"num_visits_one".equals(linkfield)){//产品不想显示只访问一个页面的次数
                            conditions.put(linkfield, "");
                            appendlinkfieldlist.add(linkfield);
//                            }
                        }
                    }
                }

                for (String linkfield : appendlinkfieldlist) {
                    if (!viewItemList.contains(linkfield)) {
                        viewItemList.add(linkfield);
                    }
                }

                //  this.getDemoMockUrl();
                if (reportName.equals("bd_bycampaign_sum") || reportName.equals("bd_bycreative_sum")
                        || reportName.equals("bd_bykwid_sum") || reportName.equals("subchannel_bysubchannel_sum")) {
                    return reportHandler.getResponseJson();
                }

                if (reportName.endsWith("_count") || reportName.equals("asmeffectevaluate_num")) {
                    return "0";

                }


                if (reportName.equals("campaigninfo_bychannel")
                        || reportName.equals("campaigninfo_byds")
                        || reportName.equals("channelinfo_bychannel")
                        || reportName.startsWith("wx_campaigninfo_")
                        || reportName.startsWith("roi_")
                        || reportName.equals("channelinfo_order_bychannel")
                        || reportName.equals("marketinginfo_bychannel")
                        || reportName.equals("subchannel_bychannel")) {

                    this.getNewMockUrl();

                } else if (reportName.startsWith("realtime_")
                        || reportName.endsWith("_byhour")
                        || reportName.endsWith("_bychannel")
                        || reportName.startsWith("wx_dashboard_")
                        || reportName.startsWith("wx_realtime_")
                        || reportName.startsWith("wx_channel_top5_")
                        || reportName.startsWith("h5_channel_top5_")
                        || reportName.endsWith("_byds")) {

//                    this.getDemoMockUrl();
                    this.getNewDashBordMockUrl(reportName);

                } else {

                    this.getNewMockUrl();
                }


                responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);


                try {

                    if ("page_duration_ids".equals(reportName)) {
                        return new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, Object.class);
                    } else if (ReportEnumType.CHART.equals(reportEnumType)) {
                        ReportChartResult reportChartResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, ReportChartResult.class);

                        if (reportName.startsWith("userengagement_")) {
                            //处理用户参与度 demo数据
                            reportChartResult = checkKeyOfUserengagementReport(reportChartResult);
                        }

                        DemoMockCharParse demoMockCharParse = new DemoMockCharParse();
                        reportChartResult = demoMockCharParse.mockDatas(reportChartResult, conditions, reportName);
                        return reportChartResult;

                    } else if (ReportEnumType.LIST.equals(reportEnumType)) {
                        ReportListResult reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(responseJson, ReportListResult.class);
                        reportListResult = new DemoMockDateParse().parse(reportListResult, app.getId(), account.getId(), webViewItemList, reportName, false, null, conditions);
                        return reportListResult;
                    }
                } catch (Exception e) {
                    logger.error("demo mock数据解析错误[" + reportName + "]", e);
                }

            }

            return null;
        }

        public ReportChartResult checkKeyOfUserengagementReport(ReportChartResult reportChartResult) {
            if (reportName.endsWith("_bydate")) {
                List<String> dateName = new ArrayList<>();
                dateName.add(conditions.get("startdate"));
                if (!StringUtils.isEmpty(conditions.get("duration_dates"))) {
                    List<String> name = Arrays.asList(conditions.get("duration_dates").split(","));
                    dateName.addAll(name);
                }
                reportChartResult.setName(dateName);
            } else {
                if (!StringUtils.isEmpty(conditions.get("duration_cid"))) {
                    List<String> ids = Arrays.asList(conditions.get("duration_cid").split(","));
                    if (!ids.isEmpty()) {
                        List<String> cidNames = channelRepository.findNamesById(ids);
                        if (cidNames.size() != ids.size()) {
                            cidNames.add("自然量");
                        }
                        if (!cidNames.isEmpty()) {
                            reportChartResult.setKey(cidNames);
                        }
                    }
                }
            }
            return reportChartResult;
        }

        public ReportHandler invoke() {

            checkArgument(appid != null, "appid不允许为空");

            if ("campaigninfo_bypackagecampaign".equals(reportName)) {
                List<Campaign4Web> res = new ArrayList<>();
                List<PackageSubCampaign> resCampaign = new ArrayList<>();
                if (conditions.containsKey("del_flag")) {
                    String del_flag = conditions.get("del_flag");
                    Boolean delFlag;
                    delFlag = del_flag.equals("true") ? true : false;
                    Iterable<Campaign4Web> customResult = campaignService.listAuthCampaignByType(account, appid, "custom", delFlag);
                    List<Campaign4Web> customRes = campaignService.listCampaignByPackage(customResult);
                    Iterable<Campaign4Web> coopResult = campaignService.listAuthCampaignByType(account, appid, "ry_coop", delFlag);
                    List<Campaign4Web> coopRes = campaignService.listCampaignByPackage(coopResult);
                    if (ValidateUtil.isValid(customRes)) {
                        res.addAll(customRes);
                    }
                    if (ValidateUtil.isValid(coopRes)) {
                        res.addAll(coopRes);
                    }
                    for (Campaign4Web re : res) {
                        if (!delFlag) {
                            resCampaign.addAll(packageSubCampaignRepository.listSubCampaignByCampaign(re.getId()));
                        } else {
                            resCampaign.addAll(packageSubCampaignRepository.listDisableByCampaign(re.getId()));
                        }
                    }
                } else {
                    Iterable<Campaign4Web> customResult = campaignService.listAuthCampaignByType(account, appid, "custom", false);
                    List<Campaign4Web> customRes = campaignService.listCampaignByPackage(customResult);
                    Iterable<Campaign4Web> coopResult = campaignService.listAuthCampaignByType(account, appid, "ry_coop", false);
                    List<Campaign4Web> coopRes = campaignService.listCampaignByPackage(coopResult);
                    if (ValidateUtil.isValid(customRes)) {
                        res.addAll(customRes);
                    }
                    if (ValidateUtil.isValid(coopRes)) {
                        res.addAll(coopRes);
                    }
                    customResult = campaignService.listAuthCampaignByType(account, appid, "custom", true);
                    customRes = campaignService.listCampaignByPackage(customResult);
                    coopResult = campaignService.listAuthCampaignByType(account, appid, "ry_coop", true);
                    coopRes = campaignService.listCampaignByPackage(coopResult);
                    if (ValidateUtil.isValid(customRes)) {
                        res.addAll(customRes);
                    }
                    if (ValidateUtil.isValid(coopRes)) {
                        res.addAll(coopRes);
                    }
                }
                if (ValidateUtil.isValid(res)) {
                    StringBuffer sb = new StringBuffer();
                    for (int i = 0; i < res.size(); i++) {
                        sb.append(res.get(i).getCampaignid());
                        if (i != (res.size() - 1)) {
                            sb.append(",");
                        }
                    }
                    conditions.put("campaignid", sb.toString());
                } else {
                    conditions.put("campaignid", "-1");
                }
            } else if ("wx_campaigninfo_byscenes".equals(reportName) && conditions.containsKey("searchtxt")) {
                String searchtxt = conditions.get("searchtxt");
                List<String> all = sceneRepository.findLikeSceneId(searchtxt);
                StringBuilder sb = new StringBuilder();
                for (String s : all) {
                    sb.append("\'").append(s).append("\'").append(",");
                }
                String substring = sb.substring(0, sb.length() - 1);
                conditions.put("sceneid", substring);
            } else if ("black_click_channel".equals(reportName) || "black_click_ds".equals(reportName) || "black_click_ds_v20".equals(reportName) || "black_click_channel_v20".equals(reportName)) {
                List<Channel> channelIdList = authService.getAuthChannelListByApp(account.getId(), appid);
                if (channelIdList != null) {
                    StringBuilder channels = new StringBuilder();
                    for (Channel l : channelIdList) {
                        channels.append(l.getId()).append(",");
                    }
                    String substring = "";
                    if (channels.length() > 0) {
                        substring = channels.substring(0, channels.length() - 1);
                        conditions.put("channels", substring);
                    } else {
                        conditions.put("channels", "0");
                    }
                }
            } else if ("subchannel_bycampaign".equals(reportName)) {
                String subchannel = conditions.get("subchannel");
                Long cid = Long.parseLong(conditions.get("cid"));
                SubChannelRename byCidAndSubChannelChName = subChannelRenameRepository.findByCidAndSubChannelChName(subchannel, cid, appid);
                if (byCidAndSubChannelChName != null) {
                    conditions.put("subchannel", byCidAndSubChannelChName.getSubChannel());
                }
            } else if ("black_top5_mtti_ds".equals(reportName)) {
                //要求显示时间范围内 近  5  天数据,而 report api 默认补全日期 , 所以此处特殊处理
                DateTime startdate = new DateTime(conditions.get("startdate"));
                DateTime enddate = new DateTime(conditions.get("enddate"));
                if (Days.daysBetween(new DateTime(startdate), new DateTime(enddate)).getDays() >= 5) {
                    conditions.put("startdate", enddate.plusDays(-4).toString("yyyy-MM-dd"));
                }
            } else if (reportName.startsWith("black_top5_") && reportName.endsWith("_ds")) {

                //防作弊卫士曲线柱状图 最多显示近30天数据 而 report api 默认补全日期 , 所以此处特殊处理
                DateTime startdate = new DateTime(conditions.get("startdate"));
                DateTime enddate = new DateTime(conditions.get("enddate"));
                if (Days.daysBetween(new DateTime(startdate), new DateTime(enddate)).getDays() >= 30) {
                    conditions.put("startdate", enddate.plusDays(-29).toString("yyyy-MM-dd"));
                }
            } else if ("funnel_user_event_list".equals(reportName) || "funnel_user_event_campain_list".equals(reportName)) {
                String viewcolumn = conditions.get("viewcolumn");
                if (StringUtils.isEmpty(viewcolumn)) {
                    viewcolumn = "loggedin";
                }

                if (StringUtils.isEmpty(conditions.get("campaignid"))) {
                    conditions.remove("campaignid");
                }

                if (StringUtils.isEmpty(conditions.get("cid"))) {
                    conditions.remove("cid");
                }

//                String[] views = viewcolumn.replace("install,", "").split(",");
                String[] views = viewcolumn.split(",");

                StringBuffer viewbuffer = new StringBuffer();

                for (int i = 0, j = views.length; i < j; i++) {
                    if (i == 0) {
                        viewbuffer.append("'").append(views[i]).append("'");
                    } else {
                        viewbuffer.append(",'").append(views[i]).append("'");
                    }
                }
                conditions.put("funnel_types", viewbuffer.toString());

            }

            app = apprepository.findOne(appid);

            //使用权限模板配置权限时 做处理
            authService.checkRoleTempCategory(account);


            checkArgument(app != null, "应用不存在");
            List<Campaign> campaigns = Lists.newArrayList();

            if (("channel_bychannel_overview".equals(reportName) || "channel_bycampaign_overview".equals(reportName))
                    && conditions.containsKey("del_flag")) {
                // 处理 开启或关闭的活动
                List<String> campaignids = campaignRepository.findCampaignDsiabledByApp(appid);
                conditions.put("deleted_campaignid", Joiner.on(",").join(campaignids));

            } else if (conditions.containsKey("cgroupid") ||
                    "campgroup".equals(conditions.get("viewtype")) ||
                    conditions.containsKey("cid") ||
                    conditions.containsKey("del_flag") ||
                    conditions.containsKey("channelcategory") ||
                    conditions.containsKey("searchtxt") ||
                    conditions.containsKey("subscribes") ||
                    account.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey()) ||
                    account.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey()) ||
                    account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {
                campaigns = preCondition(appid, account, conditions);
                if (conditions.containsKey("del_flag") && campaigns.size() == 0) {
                    Campaign cam = new Campaign();
                    cam.setCampaignid("nothing");
                    campaigns.add(cam);
                }

                fillCampids = Lists.transform(campaigns, p -> p.getCampaignid());

            } else if (account.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(account.getRoleCategory())
                    || RoleEnumType.SUB_APP_MANAGER.getKey().equals(account.getRoleCategory())) {

                // 根据角色权限控制 留存指标
                conditions.put("isincome", "1");
                conditions.put("isretention", "1");

                //筛选活动的时候去掉自然量
                if (!conditions.containsKey("campaignid")) {
                    conditions.put("isnature", "1");
                }
            }

            //控制调起指标权限
            if (conditions.containsKey("viewcolumn")) {
                String[] viewcolumn = conditions.get("viewcolumn").split(",");
                StringBuffer viewcolums = new StringBuffer();
                Boolean wakeFake = false;
                if (viewcolumn.length > 0) {
                    for (String column : viewcolumn) {
                        if (column.equals("wake_num")) {
                            wakeFake = true;
                            break;
                        }

                    }
                }
                if (wakeFake) {
                    //查询用户是否有调起服务权限
                    BlacklistApplication arouse = blacklistApplicationRepository.findByParentAndArouse(account.getRootParent());
                    if (arouse == null || arouse.getStatus() != 2 || account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {
                        for (String column : viewcolumn) {
                            if (!column.equals("wake_num")) {
                                viewcolums.append("," + column);
                            }
                        }
                        conditions.put("viewcolumn", viewcolums.substring(1));
                    } else {
                        if (account.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey()) || account.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())) {
                            Auth auth = authService.findAuth(account.getId(), appid);
                            if (!auth.getArouseAuth()) {
                                for (String column : viewcolumn) {
                                    if (!column.equals("wake_num")) {
                                        viewcolums.append("," + column);
                                    }
                                }
                                conditions.put("viewcolumn", viewcolums.substring(1));
                            }
                        }
                    }
                }
            }

            reportEnumType = fetchReportEnumType(conditions);
            dataViewEnumType = fetchDataViewEnumType(conditions);
            checkArgument(reportEnumType != null, "报表类型只支持List、Chart");

            //1. 按功能填充检索条件
            func = conditions.get("func");
            viewItemList = Lists.newArrayList();
            if ("campaigninfo".equals(func)) {
                /**
                 * 解决子渠道中文出现乱码问题
                 */
                if (conditions.containsKey("viewtype")) {
                    String viewtype = conditions.get("viewtype");
                    if ("subchannel".equals(viewtype)) {
                        List<Campaign> subChannelCampaigns = new ArrayList<>();
                        //子渠道选渠道，查看下钻的子渠道报表时候，过滤下推广活动，只留下这个渠道下的推广活动
                        if (conditions.containsKey("cid")) {
                            Long cid = Long.parseLong(conditions.get("cid"));
                            for (Campaign camp : campaigns) {
                                if (cid.equals(camp.getChannel())) {
                                    subChannelCampaigns.add(camp);
                                }
                            }
                        }
                        if (conditions.containsKey("subscribes") && "subchannel_bychannel".equals(reportName)) {
                            for (Campaign camp : campaigns) {
                                subChannelCampaigns.add(camp);
                            }
                        }
                        List<String> fillSubChannelCampids = Lists.transform(subChannelCampaigns, campaignStringFunction);
                        String subchannel = conditions.get("subchannel");
                        if (subchannel != null && subchannel.length() != 0) {
                            try {
                                conditions.put("subchannel", URLEncoder.encode(subchannel, "utf-8"));
                            } catch (UnsupportedEncodingException e) {
                                e.printStackTrace();
                            }
                        }

                        if (fillSubChannelCampids.size() > 0) {
                            conditions.put("campaignid", Joiner.on(",").join(fillSubChannelCampids));
                        } else if (conditions.containsKey("del_flag")) {
                            conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                        }//
                    } else if (("date".equals(viewtype) || "channel".equals(viewtype)) && !conditions.containsKey("del_flag")
                            || DataViewEnumType.CHANNLE_DATE.getCode().equals(viewtype)) {
                        //查询该appid的活动所使用的渠道（去重）
                        List<Channel> channelList = channelService.findAllChannelByApp(appid);
                        List<Long> cids = new ArrayList<>();
                        for (Channel c : channelList) {
                            if ("SUBSCRIBE".equals(conditions.get("subscribes")) && "ry_coop".equals(c.getType())) {
                                cids.add(c.getId());
                            } else if (!conditions.containsKey("subscribes") || "COMMON".equals(conditions.get("subscribes"))) {
                                cids.add(c.getId());
                            }
                        }
                        cids.add(-1L);
                        if (!conditions.containsKey("cid")) {
                            conditions.put("cid", Joiner.on(",").join(cids));
                        }
                        if (conditions.containsKey("subscribes") && !reportName.equals("campaigninfo_bydatechannel") && !conditions.containsKey("campaignid")) {
                            if ((fillCampids == null || fillCampids.size() == 0) && "COMMON".equals(conditions.get("subscribes"))) {
                                fillCampids = new ArrayList<>();
                                fillCampids.add("-999");
                            }
                            conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                        }

                    } else if (((conditions.containsKey("del_flag") || conditions.containsKey("subscribes")) && !conditions.containsKey("campaignid")) && !"campaigninfo_bypackagecampaign".equals(reportName) && !"campaigninfo_bysubpackagecampaign".equals(reportName)) {
                        conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                    }
                }


                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                if ("subchannel".equals(conditions.get("viewtype"))) {
                    p_viewcloumn = p_viewcloumn.replaceAll("dupnum_reged_mon", "dupnum_reg");
                }
                webViewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));

            } else if ("roi".equals(func) && conditions.containsKey("interval")) {
                DateTime install_startdate = new DateTime(conditions.get("install_startdate"));
                DateTime install_enddate = new DateTime(conditions.get("install_enddate"));
                Integer interval = Integer.parseInt(conditions.get("interval"));

                conditions.put("pay_startdate", install_startdate.toString("yyyy-MM-dd"));
                conditions.put("pay_enddate", install_enddate.plusDays(interval).toString("yyyy-MM-dd"));
            } else if ("advertisinginfo".equals(func) || "refluxanalysis".equals(func)) {

                String p_viewcloumn = null;
                if ("advertisinginfo_cash_day".equals(reportName)) {

                    /*AdvertisingPlatform platform = advertisingPlatformRepository.findByLongBase(1);
                    p_viewcloumn = platform.getTitleKey() + ",plat_type";*/
                    p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("") + ",plat_type";
                    conditions.put("iscache", "0");
                    if (conditions.containsKey("plat_type_v")) {
                        String[] ptype = conditions.get("plat_type_v").split(",");
                        conditions.remove("plat_type_v");
                        for (int i = 0; i < ptype.length; i++) {
                            if (conditions.containsKey("plat_type_v")) {
                                conditions.put("plat_type_v", conditions.get("plat_type_v") + ",'" + ptype[i] + "'");
                            } else {
                                conditions.put("plat_type_v", "'" + ptype[i] + "'");
                            }
                        }
                    }

                } else {
                    p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                }

                webViewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                for (String viewitem : viewItemList) {
                    conditions.put(viewitem, "");
                }
                if (conditions.containsKey("del_flag")) {
                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                }

            } else if ("remarketing".equals(func)) {
                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                p_viewcloumn += ",num_install";
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                if (conditions.containsKey("del_flag")) {
                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                }

                for (String viewitem : viewItemList) {
                    conditions.put(viewitem, "");
                }
                if (conditions.containsKey("del_flag")) {
                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                }
            } else if ("adsenseinfo".equals(func)) {
                String p_viewcloumn = Optional.fromNullable(conditions.get("viewcolumn")).or("");
                viewItemList = Lists.newArrayList(Splitter.on(",").omitEmptyStrings().trimResults().split(p_viewcloumn));
                if (conditions.containsKey("del_flag")) {
                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                }

                for (String viewitem : viewItemList) {
                    conditions.put(viewitem, "");
                }
                if (conditions.containsKey("del_flag")) {
                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                }
            } else if ("user_life_cycle".equals(func)) {
                if (reportEnumType.equals(ReportEnumType.CHART)) {
                    DateTime dateTime = new DateTime();
                    DateTime startTime = DateTime.parse(conditions.get("startdate"));
                    DateTime end = Days.daysBetween(startTime, dateTime).getDays() < 15 ? dateTime : startTime.plusDays(15);
                    conditions.put("enddate", end.toString("yyyy-MM-dd"));
                    conditions.put("startdate", end.plusDays(-30).toString("yyyy-MM-dd"));

                } else {

                    DateTime start = DateTime.parse(conditions.get("startdate"));
                    DateTime end = start.plusDays(-1);
                    conditions.put("enddate", conditions.get("startdate"));
                    conditions.put("startdate", end.toString("yyyy-MM-dd"));
                }
            }

            if ("costrecovery".equals(func)) {
                conditions.put("tkio_account_id", account.getId() + "");
                conditions.put("cache", "0");
            }


            //2,搜索条件
            if (conditions.containsKey("searchtxt")) {
                if ("advertisinginfo".equals(conditions.get("func")) || "adsenseinfo".equals(conditions.get("func"))) {
                    try {
                        conditions.put("searchtxt", URLEncoder.encode(conditions.get("searchtxt"), "utf-8"));
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                }
                if ("remarketing".equals(conditions.get("func"))) {
                    try {
                        conditions.put("searchtxt", URLEncoder.encode(conditions.get("searchtxt"), "utf-8"));
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                }
                String keyword = conditions.get("searchtxt");

                if ("third_account_byspaccount".equals(reportName)) {

                    //广点通投放账号别名搜索
                    List<String> thirdAccountIds = campaignService.findThirdAccountByArails(keyword, app, "5305");

                    if (thirdAccountIds != null && thirdAccountIds.size() > 0) {

                        StringBuffer idStr = new StringBuffer();

                        for (String id : thirdAccountIds) {

                            if (idStr.length() == 0) {
                                idStr.append("'").append(id).append("'");
                            } else {
                                idStr.append(",'").append(id).append("'");
                            }
                        }

                        conditions.put("thirdaccountcode", idStr.toString());

                    } else {
                        conditions.put("thirdaccountcode", "-10");//没搜索到 则查询默认 -10
                    }

                } else if (DataViewEnumType.CAMPAIGN.equals(dataViewEnumType) || "roi_bycampaign".equals(reportName) || "subchannel_bycampaign".equals(reportName)
                        || (DataViewEnumType.BDKEYWORD.equals(dataViewEnumType) && "bd_bycampaign".equals(reportName))
                        || DataViewEnumType.CAMPAIGN_H5.equals(dataViewEnumType) || "roi_byinterval".equals(reportName)
                        || "refluxanalysis_campaign".equals(reportName)
                        || "campaigninfo_bypackagecampaign".equals(reportName) || "campaigninfo_bysubpackagecampaign".equals(reportName)) {

                    Boolean delFlag = conditions.containsKey("del_flag") ? Boolean.valueOf(conditions.get("del_flag")) : null;
                    List<Campaign> queryList;
                    if (conditions.containsKey("subscribes")) {
                        if ("SUBSCRIBE".equals(conditions.get("subscribes"))) {
                            queryList = campaignService.findByKeyword(account, appid, keyword, delFlag, true);
                        } else {
                            queryList = campaignService.findByKeyword(account, appid, keyword, delFlag, false);
                        }
                    } else {
                        queryList = campaignService.findByKeyword(account, appid, keyword, delFlag);
                    }
                    queryList = queryList != null && queryList.size() > 0 ? queryList : Lists.<Campaign>newArrayList();

                    if ("自然量".contains(keyword)) {
                        Campaign defaultCampaign = new Campaign();
                        defaultCampaign.setCampaignid("_default_");
                        queryList.add(defaultCampaign);
                        conditions.put("isnature", "1");
                    } else {
                        conditions.put("isnature", "0");
                    }


                    if (delFlag != null) {
                        conditions.put("isnature", "0");
                    }

                    fillCampids = Lists.transform(queryList, new Function<Campaign, String>() {
                        @Override
                        public String apply(Campaign campaign) {
                            return campaign.getCampaignid();
                        }
                    });
                    // 分包推广活动详情单独处理
                    if ("campaigninfo_bypackagecampaign".equals(reportName)) {
                        String campaignidStr = conditions.get("campaignid");
                        String[] campaignidArr = campaignidStr.split(",");
                        List<String> campaignidList = Arrays.asList(campaignidArr);
                        List<String> list = new ArrayList<>();
                        if (ValidateUtil.isValid(campaignidList)) {
                            for (String id : campaignidList) {
                                if (fillCampids.contains(id)) {
                                    list.add(id);
                                }
                            }
                        }
                        String filterIds = Joiner.on(",").skipNulls().join(list);
                        filterIds = !Strings.isNullOrEmpty(filterIds) ? filterIds : "-1";
                        conditions.put("campaignid", filterIds);
                        //分包活动下钻
                    } else if ("campaigninfo_bysubpackagecampaign".equals(reportName)) {
                        List<String> list = new ArrayList<>();
                        String parentCampaignid = conditions.get("parent_campaignid");
                        List<PackageSubCampaign> campaignList = packageSubCampaignRepository.findLikeName(parentCampaignid, keyword);
                        if (ValidateUtil.isValid(campaignList)) {
                            for (PackageSubCampaign sub : campaignList) {
                                if (fillCampids.contains(sub.getCampaignid())) {
                                    list.add(sub.getCampaignid());
                                }
                            }
                        }
                        String filterIds = Joiner.on(",").skipNulls().join(list);
                        filterIds = !Strings.isNullOrEmpty(filterIds) ? filterIds : "-1";
                        conditions.put("campaignid", filterIds);
                    } else {
                        String filterIds = Joiner.on(",").skipNulls().join(fillCampids);
                        filterIds = !Strings.isNullOrEmpty(filterIds) ? filterIds : "-1";
                        conditions.put("campaignid", filterIds);
                    }
                } else if (DataViewEnumType.CAMPAIGNGROUP.equals(dataViewEnumType) || DataViewEnumType.CAMPGROUP_DATE.equals(dataViewEnumType)) {
                    List<CampaignGroup> queryList;
                    if (conditions.containsKey("subscribes")) {
                        if ("SUBSCRIBE".equals(conditions.get("subscribes"))) {
                            queryList = campaignGroupService.findAllByKeyword(appid, keyword, this.account, true);
                        } else {
                            queryList = campaignGroupService.findAllByKeyword(appid, keyword, this.account, false);
                        }
                    } else {
                        queryList = campaignGroupService.findAllByKeyword(appid, keyword, this.account, null);
                    }
                    queryList = queryList != null && queryList.size() > 0 ? queryList : Lists.<CampaignGroup>newArrayList();
                    fillCampGroupIds = Lists.transform(queryList, new Function<CampaignGroup, String>() {
                        @Override
                        public String apply(CampaignGroup campaignGroup) {
                            return String.valueOf(campaignGroup.getId());
                        }
                    });

                    String filterIds = Joiner.on(",").skipNulls().join(fillCampGroupIds);
                    filterIds = !Strings.isNullOrEmpty(filterIds) ? filterIds : "-1";
                    conditions.put("cgroupid", filterIds);

                } else if (DataViewEnumType.CHANNEL.equals(dataViewEnumType) || "roi_bychannel".equals(reportName)
                        || (DataViewEnumType.SUBCHANNEL.equals(dataViewEnumType) && "subchannel_bychannel".equals(reportName))
                        || "roi_byinterval".equals(reportName)) {
                    List<Channel> queryList = channelService.findByKeyword(account, appid, keyword);
                    queryList = !CollectionUtils.isEmpty(queryList) ? queryList : Lists.<Channel>newArrayList();/*                        fillChannelIds = Lists.transform(queryList, new Function<Channel, String>() { @Override public String apply(Channel channel) { return String.valueOf(channel.getId()); } });*/
                    fillChannelIds = Lists.transform(queryList, c -> String.valueOf(c.getId()));
                    String filterIds = Joiner.on(",").skipNulls().join(fillChannelIds);
                    filterIds = !Strings.isNullOrEmpty(filterIds) ? filterIds : "-1";
                    if (("roi_bychannel".equals(reportName) || "roi_byinterval".equals(reportName)) && conditions.containsKey("cid")) {
                        String cidStr = conditions.get("cid");
                        String[] cidArr = cidStr.split(",");
                        List<String> cidList = Arrays.asList(cidArr);
                        List<String> res = new ArrayList<>();
                        if (ValidateUtil.isValid(queryList)) for (Channel channel : queryList)
                            if (cidList.contains(channel.getId() + "")) res.add(channel.getId() + "");
                        conditions.put("cid", ValidateUtil.isValid(res) ? String.join("','", res) : "-2");
                    } else conditions.put("cid", filterIds);
                } else if (DataViewEnumType.SUBCHANNEL.equals(dataViewEnumType)) {
                    if (reportName.indexOf("subchannel_bysubchannel") > -1 && conditions.containsKey("cid")) {
                        Long cid = Long.valueOf(conditions.get("cid"));
                        List<String> renameList = subChannelRenameRepository.findByCidAndLikeSubChannelChName(keyword, cid, appid);
                        if (renameList.size() > 0) {
                            keyword = String.join(",", renameList);
                            conditions.put("searchtxtin", keyword);
                        }
                    }
                    try {
                        conditions.put("searchtxt", URLEncoder.encode(keyword, "utf-8"));
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                } else if (DataViewEnumType.BDKEYWORD.equals(dataViewEnumType)) try {
                    conditions.put("searchtxt", URLEncoder.encode(keyword, "utf-8"));
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }

            //3. 分组查询过滤条件: 按日期、推广活动、推广活动组、渠道进行分类检索
            if (conditions.containsKey("channelcategory")) {
                //渠道类型
                fillChannelCategoryFilter(appid, account, conditions, campaigns, func);

            } else if (conditions.containsKey("cgroupid")) {

                //按活动组查询
                final List<Long> groupIdList = StringUtil.transString2LongList(conditions.get("cgroupid"));
                List<Long> filterGroupCampaignIdList = campaignGroupService.listCampaignIdByGroupIds(groupIdList);

                List<String> filterCampaignIdList = new ArrayList<>();

                if (!CollectionUtils.isEmpty(filterGroupCampaignIdList)) {

                    final List<String> finalFillCampaignIdList = fillCampids;
                    List<Campaign> filterCampaigns = ReportServiceImp.this.campaignService.findById(filterGroupCampaignIdList);
                    List<String> transCampidList = Lists.transform(filterCampaigns, campaignStringFunction);

                    filterCampaignIdList = Lists.newArrayList(Iterables.filter(transCampidList, new Predicate<String>() {
                        @Override
                        public boolean apply(String campaignid) {
                            return finalFillCampaignIdList.contains(campaignid);
                        }
                    }));
                }

                //注收比特殊处理
                if (("roi_bycampaign".equals(reportName) || "roi_byinterval".equals(reportName)) && conditions.containsKey("campaignid")) {
                    List<String> campaignidList = new ArrayList<>();
                    String campaignidStr = conditions.get("campaignid");
                    String[] campaignidArr = campaignidStr.split(",");

                    for (String campaignId : campaignidArr) {
                        if (filterCampaignIdList.contains(campaignId)) {
                            campaignidList.add(campaignId);
                        }
                    }
                    conditions.put("campaignid", ValidateUtil.isValid(campaignidList) ? String.join(",", campaignidList) : "-1");

                    //add by sunhao 20170818
                    fillCampids = conditions.containsKey("searchtxt") ? fillCampids : campaignidList;

                } else {
                    conditions.put("campaignid", ValidateUtil.isValid(filterCampaignIdList) ? String.join(",", filterCampaignIdList) : "-1");

                    //add by sunhao 20170818
                    fillCampids = conditions.containsKey("searchtxt") ? fillCampids : filterCampaignIdList;
                }

                //按活动组查询,去掉自然量 add by sunhao 20170531
                conditions.put("isnature", "0");

            } else if (conditions.containsKey("cid")) {

                //create by 20180724 cid = -1 来显示自然量
                if ("-1".equals(conditions.get("cid"))) {
                    conditions.put("isnature", "1");
                } else {
                    //按渠道筛选,只填充fillCampids,cid传到sql中筛选 add by sunhao 20170818
                    if (account.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey()) || account.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey()) || account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey()) || conditions.containsKey("searchtxt"))
                        fillCampids = conditions.containsKey("searchtxt") ? fillCampids : campaignService.filterChannelAndAuthCampaignId(account.getId(), app.getId(), conditions.get("cid"));
                }

            }
            if (conditions.containsKey("del_flag")) {
                conditions.put("isnature", "0");
            }


            if ((reportName.startsWith("black_click_") || reportName.startsWith("black_top5_")) &&
                    !reportName.equals("black_click_channel_total") && !reportName.equals("black_click_byappid")) {

                //控制权限

                List<Object[]> campaignIds = authService.getCampaignIdsByAuth(account, app);

                List<String> campaignIdsAuth;

                if (campaignIds != null && campaignIds.size() > 0) {
                    //权限活动存在 则需要控制权限

                    String cid = conditions.get("cid");
                    String campaignid = conditions.get("campaignid");

                    if ("all".equals(cid) || "all".equals(campaignid)) {

                        campaignIdsAuth = campaignIds.stream().map(p -> p[0].toString()).collect(Collectors.toList());

                    } else if (!StringUtils.isEmpty(cid) && StringUtils.isEmpty(campaignid)) {
                        //通过渠道筛选权限
                        List<String> cidParam = Arrays.asList(cid.split(","));
                        campaignIdsAuth = campaignIds.stream().filter(p -> cidParam.contains(p[1].toString()))
                                .map(p -> p[0].toString()).collect(Collectors.toList());
                        conditions.remove("cid");

                    } else if (StringUtils.isEmpty(cid) && !StringUtils.isEmpty(campaignid)) {
                        //通过活动筛选权限
                        List<String> campainParams = Arrays.asList(campaignid.split(","));
                        campaignIdsAuth = campaignIds.stream().filter(p -> campainParams.contains(p[0].toString()))
                                .map(p -> p[0].toString()).collect(Collectors.toList());

                        conditions.remove("cid");

                    } else {

                        campaignIdsAuth = campaignIds.stream().map(p -> p[0].toString()).collect(Collectors.toList());
                    }

                    StringBuffer campainbuffer = new StringBuffer();

                    for (int i = 0, size = campaignIdsAuth.size(); i < size; i++) {

                        if (i == 0) {
                            campainbuffer.append(campaignIdsAuth.get(i));
                        } else {
                            campainbuffer.append(",").append(campaignIdsAuth.get(i));
                        }
                    }

                    conditions.put("campaignid", campainbuffer.toString());

                } else {

                    if (conditions.containsKey("campaignid")) {

                        String campaignid = conditions.get("campaignid");
                        if (campaignid == null || "all".equals(campaignid) || campaignid.isEmpty()) {
                            conditions.remove("campaignid");
                        }
                    }

                    if (conditions.containsKey("cid")) {
                        String cid = conditions.get("cid");
                        if (cid == null || "all".equals(cid) || cid.isEmpty()) {
                            conditions.remove("cid");
                        }
                    }

                }
            } else if (conditions.containsKey("campaignid") && conditions.get("campaignid").equals("all")) {
                conditions.put("isnature", "0");
                if (CollectionUtils.isEmpty(fillCampids)) {
                    fillCampids = new ArrayList<>();
                    Iterable<Campaign> result;
                    Map jsond = new HashMap();
                    if (!account.getIsSuperUser()
                            && !account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())
                            && !account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {
                        result = authService.getAuthCampaignListLimit(account, null, appid, jsond, 500);
                    } else {
                        result = campaignService.listAllAndDisableLimit(appid, account, null, jsond, null);
                    }
                    Iterator<Campaign> iterable = result.iterator();
                    while (iterable.hasNext()) {
                        Campaign web = iterable.next();
                        if (web != null) {
                            fillCampids.add(web.getCampaignid());
                        }
                    }
                }
                conditions.put("campaignid", String.join(",", fillCampids));

            }


            if (conditions.containsKey("campaignid") && conditions.get("campaignid").equals("all")) {

                conditions.put("isnature", "0");
                conditions.put("campaignid", String.join(",", fillCampids));

            }

//            if (conditions.get("campaigninfo_appmarket_ids") != null) {
//                //分包分链接 通过cid 筛选
//                conditions.put("cid", conditions.get("campaigninfo_appmarket_ids"));
//                //conditions.remove("campaignid");
//            }
            //投放广告详情--->>>处理cid,推广活动详情也有campaigninfo_appmarket，所以分两种
            if ("advertisinginfo".equals(func) && !"advertisinginfo_cash_day".equals(reportName)) {
                if ("Android".equals(app.getPlatform())) {
                    Map<Long, ChannelAdvertisingPackage> mapCAP = new HashMap<>();
                    List<ChannelAdvertisingPackage> channelAdvertisingPackages = channelAdvertisingPackageRepository.listAll();
                    for (ChannelAdvertisingPackage channelAdvertisingPackage : channelAdvertisingPackages) {
                        mapCAP.put(channelAdvertisingPackage.getChannelAdvertisingCid(), channelAdvertisingPackage);
                    }
                    if (conditions.containsKey("cid") && !conditions.get("campaigninfo_appmarket").equals("all")) {
                        String[] cids = conditions.get("cid").split(",");
                        String cidPackage = "";
                        if (conditions.get("campaigninfo_appmarket").equals("APPMARKET")) {//分包
                            for (String cid : cids) {
                                Channel channel = channelRepository.findById(Long.valueOf(cid));
                                if ("APPMARKET".equals(channel.getCategory())) {
                                    cidPackage += "," + cid;
                                } else {
                                    ChannelAdvertisingPackage channelAdvertisingPackage = mapCAP.get(Long.valueOf(cid));
                                    if (channelAdvertisingPackage != null)
                                        cidPackage += "," + channelAdvertisingPackage.getCid();
                                }
                            }
                        } else {
                            for (String cid : cids) {
                                Channel channel = channelRepository.findById(Long.valueOf(cid));
                                if ("ADVERTISING".equals(channel.getCategory())) {
                                    cidPackage += "," + cid;
                                }
                            }
                        }
                        conditions.put("cid", cidPackage.length() > 1 ? cidPackage.substring(1) : "-1");
                    } else if (conditions.containsKey("cid") && conditions.get("campaigninfo_appmarket").equals("all")) {
                        String[] cids = conditions.get("cid").split(",");
                        String cidPackage = "";
                        for (String cid : cids) {
                            ChannelAdvertisingPackage channelAdvertisingPackage = mapCAP.get(Long.valueOf(cid));
                            if (channelAdvertisingPackage != null)
                                cidPackage += "," + channelAdvertisingPackage.getCid();
                        }
                        conditions.put("cid", conditions.get("cid") + cidPackage);

                    } else if (!conditions.containsKey("cid") && !conditions.get("campaigninfo_appmarket").equals("all")) {
                        String cap = "";
                        if (conditions.get("campaigninfo_appmarket").equals("APPMARKET")) {//分包
                            for (ChannelAdvertisingPackage channelAdvertisingPackage : channelAdvertisingPackages) {
                                cap += "," + channelAdvertisingPackage.getCid();
                            }
                        } else {//分链接
                            List<ChannelAdvertising> channelAdvertisings = channelAdvertisingRepository.listAll();
                            for (ChannelAdvertising channelAdvertising : channelAdvertisings) {
                                cap += "," + channelAdvertising.getCid();
                            }
                        }
                        conditions.put("cid", cap.length() > 1 ? cap + "-1" : "-1");

                    }//什么都不传则会直接按照appid查
                }
            } else {
                if (conditions.containsKey("campaigninfo_appmarket") && !"all".equals(conditions.get("campaigninfo_appmarket"))) {

                    // 处理分包 分链接 筛选
                    final String appmarket = "APPMARKET".equals(conditions.get("campaigninfo_appmarket")) ? "APPMARKET" : "ADVERTISING";
                    Map<String, String> conditionCids = null;
                    if (conditions.containsKey("cid")) {
                        conditionCids = new HashedMap();
                        String[] cidQuery = conditions.get("cid").split(",");
                        for (int i = 0, j = cidQuery.length; i < j; i++) {
                            conditionCids.put(cidQuery[i], "1");
                        }
                    }

                    // 推广活动详情 分包分链接筛选逻辑
                    List<Channel> channelList = channelService.findAllChannelByApp(appid);
                    StringBuffer channelIds = new StringBuffer();
                    Map<String, String> finalConditionCids = conditionCids;
                    channelList.stream().filter(channel -> {
                                if (finalConditionCids == null) {
                                    return appmarket.equals(channel.getCategory());
                                } else {
                                    return appmarket.equals(channel.getCategory())
                                            && finalConditionCids.containsKey(channel.getId().toString());
                                }
                            }
                    ).forEach(channel -> {
                        if (channelIds.length() == 0) {
                            channelIds.append(channel.getId());
                        } else {
                            channelIds.append(",").append(channel.getId());
                        }
                    });
                    if (channelIds.length() != 0 && !"advertisinginfo".equals(func)) {
                        if (appmarket.equals("ADVERTISING")) {
                            channelIds.append(",").append("-1");
                        }
                        conditions.put("cid", channelIds.toString());
                    } else {
                        // -3 表示不包含自然量 分包不包含自然量数据
                        conditions.put("cid", appmarket.equals("APPMARKET") ? "-3" : "-1");
                    }
                }

            }

            //请求报表前处理活动id（再营销活动详情），筛选deeplink活动
            if ("remarketing".equals(func)) {//增加普通账号权限时别忘了修改com.reyun.report.parse.RemarketingCampaignParse.parse(com.reyun.model.ReportListResult, java.lang.Long, java.lang.Long, java.util.List<java.lang.String>, java.lang.String, boolean, java.util.List<java.lang.String>, java.lang.Long, java.util.Map<java.lang.String,java.lang.String>)

                List<String> campaignList = new ArrayList<>();
                //活动组维度

                //分包
                if ("APPMARKET".equals(conditions.get("marketinginfo_appmarket"))) {
                    if (reportName.equals("marketinginfo_bysubpackagecampaign")) {//分包子活动下钻
                        //parent_campaignid
                        List<PackageSubCampaign> subCampaign = packageSubCampaignRepository.findFilterDeeplinkByParentCampaignid(conditions.get("parent_campaignid"), Constant.deeplinkHost);
                        for (PackageSubCampaign packageSubCampaign : subCampaign) {
                            campaignList.add(packageSubCampaign.getCampaignid());
                        }
                    } else {//一级分包

                        if (conditions.containsKey("campaignid")) {//主账号,管理员，自用用管理员筛选时走这里，不筛选则走下面，其余账号不管筛选与否都走这里
                            ArrayList<String> campaignid = Lists.newArrayList(conditions.get("campaignid").split(","));
                            campaignList = campaignRepository.findFilterDeeplinkByCampaignIdsAndChannelCategory(campaignid, "APPMARKET");
                        } else {
                            campaignList = campaignRepository.findFilterDeeplinkByAppAndChannelCategory(app.getId(), "APPMARKET");
                        }
                    }
                } else {//分链接
                    if ("ADVERTISING".equals(conditions.get("marketinginfo_appmarket"))) {
                        if (conditions.containsKey("campaignid")) {//主账号,管理员，自用用管理员筛选时走这里，不筛选则走下面，其余账号不管筛选与否都走这里
                            ArrayList<String> campaignid = Lists.newArrayList(conditions.get("campaignid").split(","));
                            campaignList = campaignRepository.findFilterDeeplinkByCampaignIdsAndChannelCategory(campaignid, "ADVERTISING");
                        } else {
                            campaignList = campaignRepository.findFilterDeeplinkByAppAndChannelCategory(app.getId(), "ADVERTISING");
                        }
                    } else {//全部
                        if (conditions.containsKey("campaignid")) {//主账号,管理员，自用用管理员筛选时走这里，不筛选则走下面，其余账号不管筛选与否都走这里
                            ArrayList<String> campaignid = Lists.newArrayList(conditions.get("campaignid").split(","));
                            campaignList = campaignRepository.findAllFilterDeeplinkByCampaignIds(campaignid, Constant.deeplinkHost);
                        } else {
                            campaignList = campaignRepository.findAllFilterDeeplinkByApp(app.getId(), Constant.deeplinkHost);
                        }
                    }
                }

                if (campaignList != null && campaignList.size() > 0) {
                    conditions.put("campaignid", StringUtils.join(campaignList, ","));
                } else {
                    conditions.put("campaignid", "-999");//无符合的活动
                }
            }

            if (conditions.containsKey("adplatform") && "advertisinginfo_adname_rename".equals(reportName)) {
                conditions.put("adplatform", conditions.get("adplatform").replace(",", "','"));
            }


            //region 4. 请求报表系统
            url = Constant.reportUrl + "/api/trackingio/" + reportName + "/" + app.getAppkey();


            if (conditions.containsKey("kwid") && !conditions.containsKey("encoded")) {
                try {
                    conditions.put("kwid", URLEncoder.encode(conditions.get("kwid"), "utf-8"));
                    conditions.put("encoded", "1");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
            if (conditions.containsKey("creative") && !conditions.containsKey("encoded")) {
                try {
                    conditions.put("creative", URLEncoder.encode(conditions.get("creative"), "utf-8"));
                    conditions.put("encoded", "1");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
            if (conditions.containsKey("adgroup_id")) {
                try {
                    conditions.put("adgroup_id", URLEncoder.encode(conditions.get("adgroup_id"), "utf-8"));
                    conditions.put("encoded", "1");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
            if (conditions.containsKey("adplan_id")) {
                try {
                    conditions.put("adplan_id", URLEncoder.encode(conditions.get("adplan_id"), "utf-8"));
                    conditions.put("encoded", "1");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
            conditions.put("iscache", Constant.iscache);
            if (conditions.containsKey("thirdAccountCode") && !StringUtil.isEmpty(conditions.get("thirdAccountCode"))) {
                String thirdaccountcode = conditions.get("thirdAccountCode");
                if (!isNumeric(thirdaccountcode)) {
                    String ta = campaignRepository.getThirdAccountByAlias(thirdaccountcode, app.getId());
                    conditions.put("thirdAccountCode", ta);
                }
            }


            //比率依赖字段处理为必须查询
            List<String> appendlinkfieldlist = Lists.newArrayList();
            for (String viewitem : viewItemList) {
                conditions.put(viewitem, "");
                Collection<String> linkfields = ViewColumnDependUtil.getRateDependFields(viewitem);
                if (linkfields != null && linkfields.size() > 0) {
                    for (String linkfield : linkfields) {
                        conditions.put(linkfield, "");
                        appendlinkfieldlist.add(linkfield);
                    }
                }
            }

            for (String linkfield : appendlinkfieldlist) {
                if (!viewItemList.contains(linkfield)) {
                    viewItemList.add(linkfield);
                }
            }

            if ("campaigninfo_bysubpackagecampaign".equals(reportName) && conditions.containsKey("result_first_name_replace")) {
                //去除部分colum(分包下钻移动到推广活动下钻时 处理该 colum)
                final String[] viewCloum = {conditions.get("viewcolumn")};
                Arrays.asList("num_impression", "rate_add_daily", "amt_arpdau_new", "amt_arppu_new", "avg_time_once_min", "avg_time_use_min", "duration", "amt_use_freq")
                        .forEach(v -> {
                            viewCloum[0] = viewCloum[0].replace(v + ",", "");
                            viewCloum[0] = viewCloum[0].replace(v, "");
                            conditions.remove(v);
                        });
                conditions.put("viewcolumn", viewCloum[0]);
            }
            if ("campaigninfo_bypackagecampaign".equals(reportName) || "campaigninfo_bysubpackagecampaign".equals(reportName)) {
                if (!(account.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(account.getRoleCategory())
                        || RoleEnumType.SUB_APP_MANAGER.getKey().equals(account.getRoleCategory()))) {
                    Boolean del_flag = null;
                    if (conditions.containsKey("del_flag")) {
                        del_flag = Boolean.valueOf(conditions.get("del_flag"));
                    }
                    List<DataAuthPkgsubcampaign> pkgsubcampaignListByApp = dataAuthPkgsubcampaignRepository.findPkgsubcampaignListByApp(account.getId(), app.getId());
                    String sub_campaignid = "";
                    for (DataAuthPkgsubcampaign dataAuthPkgsubcampaign : pkgsubcampaignListByApp) {
                        PackageSubCampaign one = packageSubCampaignRepository.findOne(dataAuthPkgsubcampaign.getPkgSubCampaign());
                        if (conditions.containsKey("del_flag") && !del_flag == one.getDelFlag()) continue;
                        if ("campaigninfo_bysubpackagecampaign".equals(reportName) && conditions.containsKey("parent_campaignid") && !conditions.get("parent_campaignid").equals(one.getParentCampaignid()))
                            continue;
                        sub_campaignid += "'" + one.getCampaignid() + "',";
                    }
                    if (sub_campaignid.length() > 0) {
                        conditions.put("sub_campaignid", sub_campaignid.substring(0, sub_campaignid.length() - 1));
                    }
                }
            }
            if (conditions.containsKey("func") && "refluxanalysis".equals(conditions.get("func")) && accountFunctionAuthRepository.getByAccountAndMenu(account.getRootParent(), "serveridbackflow") != null) {
                conditions.put("serveridbackflow", "true");
            }
            if (conditions.containsKey("cid") && conditions.get("cid").contains("5305") && ("advertisinginfo".equals(func) || "adsenseinfo".equals(func))) {
                //2155 【投放广告详情和投放广告】的【渠道筛选】中点击【广点通】时同时显示【广点通】与【新广点通】渠道的数据
                conditions.put("cid", conditions.get("cid") + ",17797");
            }


            if (account.getRoleCategory().intValue() > 2) {
                //子账号判断权限
                Auth auth = authService.findAuth(account.getId(), appid);
                if (!auth.getIsNatureOpen()) {
                    if (conditions.containsKey("cid") && conditions.get("cid") != null) {
                        String cid = conditions.get("cid").replace("-1", "").replace(",,", ",");
                        if (StringUtils.isEmpty(cid)) {
                            cid = "NULL";
                        }
                        conditions.put("cid", cid);
                    }

                    if (conditions.containsKey("campaignid") && conditions.get("campaignid") != null) {
                        String camId = conditions.get("campaignid").replace("_default_", "").replace(",,", ",");
                        if (StringUtils.isEmpty(camId)) {
                            camId = "NULL";
                        }
                        conditions.put("campaignid", camId);
                    }
                }
            }
            
            responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
            return this;
        }

        private void getNewMockUrl() {
            String urlrpname = reportName + "/" + app.getAppkey() + "/temp";
            if (reportName.equals("campaigninfo_bycampgroup")) {
                String viewType = conditions.get("viewtype");
                urlrpname += "/" + viewType;
            }

            if ("ec".equalsIgnoreCase(app.getIndustory())
                    && !reportName.startsWith("black_top5_")
                    && !reportName.startsWith("funnel_user_event_")
                    && !reportName.endsWith("_v20")
                    && !reportName.startsWith("userengagement_")
                    && !reportName.startsWith("advertisinginfo_")) {
                urlrpname += "/ec";
            } else if (reportName.startsWith("black_top5_mtti_") || reportName.startsWith("black_top5_list_mtti_")) {

                String all = conditions.get("mtti_all");
                urlrpname += "all".equals(all) ? "/all" : "/day";

            }

//            logger.info("demo url:[{}]", urlrpname);

            url = Constant.demoUtl + "/api/trackingio/" + urlrpname;

        }

        private void getNewDashBordMockUrl(String reportName) {

            String rpname = reportName;

            if (rpname.startsWith("h5_channel_top5_")
                    || rpname.startsWith("h5_dashboard_")
                    || rpname.startsWith("wx_channel_top5")
                    || rpname.startsWith("wx_dashboard_")) {
                rpname = rpname.replace("_byds", "_byhour");
            }

            url = Constant.demoUtl + "/api/trackingio/" + rpname + "/" + app.getAppkey() + "/dashboard";
        }


        private void getDemoMockUrl() {
            DateTime dateTime = new DateTime();
            if (reportName.startsWith("realtime_")) {
                // 不做处理
                url = Constant.demoUtl + "/api/trackingio/" + reportName + "/" + app.getAppkey();

            } else if ("roi".equals(func)) {
                int interval = conditions.containsKey("interval") ? Integer.parseInt(conditions.get("interval")) : 0;
                int paySize = DateUtil.getDateInterval(conditions.get("pay_startdate"), conditions.get("pay_enddate")).size();
                url = Constant.demoUtl + "/api/trackingio/" + reportName + "/" + app.getAppkey();
                int size = DateUtil.getDateInterval(conditions.get("install_startdate"), conditions.get("install_enddate")).size();
                conditions.put("install_interval", "3");
                conditions.put("pay_interval", "3");

                if (size == 7) {
                    url += "/7";
                } else if (size == 14 ||
                        getDaysMonth(dateTime, conditions.get("install_startdate"), conditions.get("install_enddate"), reportName)) {
                    url += "/14";
                } else if (size == 30 ||
                        getDaysMonth(dateTime, conditions.get("install_startdate"), conditions.get("install_enddate"), reportName)) {
                    url += "/30";
                } else if (size == 1) {
                    url += "/1";
                }
                if (paySize == 1 || interval == 1) {
                    url += "/1";
                } else if (paySize == 7 || interval == 7 ||
                        getDaysMonth(dateTime, conditions.get("pay_startdate"), conditions.get("pay_enddate"), reportName)) {
                    url += "/7";
                } else if (paySize == 14 || interval == 14) {
                    url += "/14";
                } else if (paySize == 30 || interval == 30 ||
                        getDaysMonth(dateTime, conditions.get("pay_startdate"), conditions.get("pay_enddate"), reportName)) {
                    url += "/30";
                }
                if (reportName.equals("roi_byds")) {
                    if (interval == 0) {
                        url += "/absolute";
                    } else {
                        url += "/relative";
                    }
                }
            } else {
                if (reportName.endsWith("_count")) {
                    reportName = reportName.substring(0, reportName.indexOf("_count"));
                }

                url = Constant.demoUtl + "/api/trackingio/" + reportName + "/" + app.getAppkey();


                if (conditions.containsKey("startdate")) {

                    int size = DateUtil.getDateInterval(conditions.get("startdate"), conditions.get("enddate")).size();
                    if (size == 7 || getDaysMonth(dateTime, conditions.get("startdate"), conditions.get("enddate"), reportName)) {
                        url += "/7";

                    } else if (size == 14) {
                        url += "/14";
                    } else if (size == 30 || getDaysMonth(dateTime, conditions.get("startdate"), conditions.get("enddate"), reportName)) {
                        url += "/30";
                    } else if (conditions.get("startdate").equals(DateUtil.getBeforeDays(0)) && conditions.get("enddate").equals(DateUtil.getBeforeDays(0))) {
                        url += "/0";
                    }
                }
            }

        }


        /**
         * 预处理权限数据和分页,推广活动停用、启用在conditions中以del_flag为key，del_flag为true表示停用，为false表示启用，此key为空表示全部数据
         */
        private List<Campaign> preCondition(Long appid, Account loginAccount, Map<String, String> conditions) {

            conditions.put("iscache", Constant.iscache);
            //region 1. 权限数据过滤
            List<Campaign> campaigns = Lists.newArrayList();
            List<Campaign> campaigns_new = Lists.newArrayList();
            //母账号,管理员,自应用管理员
            if (loginAccount.getIsSuperUser() || RoleEnumType.MANAGER.getKey().equals(loginAccount.getRoleCategory())
                    || RoleEnumType.SUB_APP_MANAGER.getKey().equals(loginAccount.getRoleCategory())) {
                if (conditions.containsKey("del_flag")) {
                    Boolean del_flag = Boolean.valueOf(conditions.get("del_flag"));
                    campaigns = campaignService.listAllCampaignByDelflag(appid, del_flag);
                } else {
                    // campaigns =
                    List<Object[]> objects = campaignService.listPartCampaign(appid);
                    for (Object[] obj : objects) {
                        campaigns.add(new Campaign(((BigInteger) obj[0]).longValue(), (String) obj[1], ((BigInteger) obj[2]).longValue(), (String) obj[3], (String) obj[4], (Boolean) obj[5]));
                    }
                    //campaigns = campaignService.listAllCampaign(appid, loginAccount.getId());

                }


                conditions.put("isincome", "1");
                conditions.put("isretention", "1");

                //筛选活动的时候去掉自然量
                if (!conditions.containsKey("campaignid")) {
                    conditions.put("isnature", "1");
                    //筛选预约活动
                    if (conditions.containsKey("subscribes")) {
                        for (Campaign campaign : campaigns) {
                            if (conditions.get("subscribes").equals("SUBSCRIBE") && campaign.getSubscribes() != null && campaign.getSubscribes()) {
                                campaigns_new.add(campaign);
                            } else if (conditions.get("subscribes").equals("COMMON") && (campaign.getSubscribes() == null || !campaign.getSubscribes())) {
                                campaigns_new.add(campaign);
                            }
                        }
                        campaigns = campaigns_new;
                    }
                }

            } else {
                List<Long> authCampaignList = authService.getAuthCampaignListByDelflag(loginAccount.getId(), appid, conditions.get("del_flag"));

                if (authCampaignList != null && authCampaignList.size() > 0) {
                    //campaigns = campaignService.findAllById(authCampaignList);
                    //优化 select *
                    campaigns = campaignService.findCampaignIdsAndSubscribesById(authCampaignList);
                }

                //优化 mysql in 中 campainid 过多问题（授权的活动大于100 做此优化）
                if (campaigns.size() > 100 && conditions.containsKey("startdate")) {

                    Map params = new HashMap();
                    params.put("datatype", "list");
                    params.put("startdate", conditions.get("startdate"));
                    params.put("enddate", conditions.get("enddate"));

                    String url4id = Constant.reportUrl + "/api/trackingio/campaigninfo_where_campainid/" + app.getAppkey();
                    String idItems = HttpClientUtil.doHttpPostRequest(url4id, "trackingio", conditions);
                    ReportListResult reportListResult = null;
                    try {
                        reportListResult = new ObjectMapper().configure(DeserializationFeature.USE_LONG_FOR_INTS, true).readValue(idItems, ReportListResult.class);
                    } catch (IOException e) {
                        logger.error("报表->活动ID错误", e);
                    }

                    if (reportListResult != null && reportListResult.getVal() != null && reportListResult.getVal().size() > 0) {
                        //该app时间范围内数据所包含的活动
                        Map<String, String> campainIdsInReport = reportListResult.getVal().stream().collect(
                                Collectors.toMap(v -> v.get("campaignid") + "", v -> "1", (v1, v2) -> v1));

                        int queryCampainSize = campaigns.size();
                        int sizeOfReport = campainIdsInReport.size();

                        if (sizeOfReport < queryCampainSize) {
                            // 如果数据中的活动数少于授权的活动数 再做一次过滤 （查询数据中有的活动且授权的活动）
                            List<Campaign> filterCampain = campaigns.stream().filter(v ->
                                    campainIdsInReport.containsKey(v.getCampaignid())).collect(Collectors.toList());

                            int sizeOFilter = filterCampain.size();

                            if (sizeOFilter > 0) {
                                campaigns = filterCampain;
                                if (sizeOFilter > 1000) {
                                    logger.info(" campainid in sql:{}->{}, {} of etl counts ,email: {}, app: {} ",
                                            queryCampainSize, sizeOFilter, sizeOfReport, loginAccount.getEmail(), app.getAppkey());
                                }
                            }

                        }
                    }
                }

                Auth auth = authService.findAuth(loginAccount.getId(), appid);
                conditions.put("isincome", null != auth && auth.isPayAuth() ? "1" : "0");
                conditions.put("isretention", null != auth && auth.isRetentionAuth() ? "1" : "0");
                //如果筛选则不设值
                if (!conditions.containsKey("campaignid")) {
                    //筛选预约活动
                    if (conditions.containsKey("subscribes")) {
                        for (Campaign campaign : campaigns) {
                            if (conditions.get("subscribes").equals("SUBSCRIBE") && campaign.getSubscribes() != null && campaign.getSubscribes()) {
                                campaigns_new.add(campaign);
                            } else if (conditions.get("subscribes").equals("COMMON") && (campaign.getSubscribes() == null || !campaign.getSubscribes())) {
                                campaigns_new.add(campaign);
                            }
                        }
                        campaigns = campaigns_new;
                    }
                    List<String> fillCampids = Lists.transform(campaigns, campaignStringFunction);

                    if (CollectionUtils.isEmpty(fillCampids)) {
                        fillCampids = new ArrayList<>();
                        fillCampids.add("NULL");
                    }

                    conditions.put("campaignid", Joiner.on(",").join(fillCampids));
                    conditions.put("isnature", null != auth && auth.getIsNatureOpen() ? "1" : "0");
                }


            }


            //分页
            if (conditions.containsKey("pageIndex") && conditions.containsKey("pageNumber")) {
                int pateIndex = Integer.valueOf(conditions.get("pageIndex"));
                int number = Integer.valueOf(conditions.get("pageNumber"));

                conditions.put("start", pateIndex * number + "");
                conditions.put("number", number + "");
            }

            //推广来源,日期分类,去掉自然量 --add by sunhao 20170531
/*            if (conditions.containsKey("viewtype") && conditions.get("viewtype").equals("date")) {
                conditions.put("isnature", "0");
            }*/
            //endregion
            return campaigns;
        }

        private boolean getDaysMonth(DateTime dateTime, String start, String end, String reportName) {
            if (reportName != null) {
                if (reportName.startsWith("realtime_") || reportName.startsWith("h5_realtime_")) {
                    return false;
                }
            }

            String today = dateTime.toString("yyyy-MM-dd");
            String week_first = dateTime.withDayOfWeek(1).toString("yyyy-MM-dd");
            String month_first = dateTime.withDayOfMonth(1).toString("yyyy-MM-dd");
            if (today.equals(end) && week_first.equals(start)) {
                return true;
            }
            if (today.equals(end) && month_first.equals(start)) {
                return true;
            }
            return false;
        }

        private ReportEnumType fetchReportEnumType(Map<String, String> conditions) {
            String paramVal = conditions.get("datatype");
            if (ReportEnumType.CHART.getCode().equalsIgnoreCase(paramVal)) {
                return ReportEnumType.CHART;
            } else if (ReportEnumType.LIST.getCode().equalsIgnoreCase(paramVal)) {
                return ReportEnumType.LIST;
            }
            return null;
        }

        private DataViewEnumType fetchDataViewEnumType(Map<String, String> conditions) {
            String paramVal = conditions.get("viewtype");
            for (DataViewEnumType dataViewEnumType : DataViewEnumType.values()) {
                if (dataViewEnumType.getCode().equals(paramVal)) {
                    return dataViewEnumType;
                }
            }
            return null;
        }

        private void conditionFilter(Long appid, Account loginAccount, Map<String, String> conditions, List<Campaign> campaigns, String func, Predicate<Campaign> campaignPackChannelIfNull, Function<Campaign, String> campaignStringFunction, Predicate<Channel> channelAdvertisingPredicate, Function<Channel, Long> channelStringFunction) {
            if ("campaigninfo".equals(func)) {
                fillCampaignFilter(appid, loginAccount, conditions, campaigns, campaignPackChannelIfNull, campaignStringFunction, channelAdvertisingPredicate, channelStringFunction);
            } else if ("roi".equals(func)) {
                fillRoiFilter(appid, loginAccount, conditions, campaigns, campaignPackChannelIfNull, campaignStringFunction, channelAdvertisingPredicate, channelStringFunction);
            } else if ("channeleffect".equals(func)) {
                fillChannelEffectFilter(appid, loginAccount, conditions, channelAdvertisingPredicate, channelStringFunction);
            }
        }

        private void fillChannelCategoryFilter(Long appid, Account loginAccount, Map<String, String> conditions, List<Campaign> campaigns, String func) {
            if (ChannelCategoryEnumType.ADVERTISING.getKey().equals(conditions.get("channelcategory"))) {
                conditionFilter(appid, loginAccount, conditions, campaigns, func, campaignPackChannelIfNull, campaignStringFunction, channelAdvertisingPredicate, channelStringFunction);
            } else if (ChannelCategoryEnumType.APPMARKET.getKey().equals(conditions.get("channelcategory"))) {
                conditionFilter(appid, loginAccount, conditions, campaigns, func, campaignPackChannelIfNotNull, campaignStringFunction, channelAppmarketPredicate, channelStringFunction);
            }
        }

        private void fillCampaignFilter(Long appid, Account loginAccount, Map<String, String> conditions, List<Campaign> campaigns, Predicate<Campaign> campaignPredicate, Function<Campaign, String> campaignFunction, Predicate<Channel> channelPredicate, Function<Channel, Long> channelFunction) {
            if (DataViewEnumType.DATE.getCode().equals(conditions.get("viewtype")) || DataViewEnumType.CAMPAIGNGROUP.getCode().equals(conditions.get("viewtype"))) {
                ImmutableList campids = FluentIterable.from(campaigns)
                        .filter(campaignPredicate)
                        .transform(campaignFunction)
                        .toList();
                if (conditions.containsKey("campaignid")) {
                    List<String> camidWeb = Arrays.asList(conditions.get("campaignid").split(","));
                    Collection campidsNew = org.apache.commons.collections.CollectionUtils.intersection(campids, camidWeb);
                    final String value = campidsNew != null && campidsNew.size() > 0 ? Joiner.on(",").skipNulls().join(campidsNew) : "-999";
                    conditions.put("campaignid", value);
                } else {
                    final String value = campids != null && campids.size() > 0 ? Joiner.on(",").skipNulls().join(campids) : "-999";
                    conditions.put("campaignid", value);
                }
            } else if (DataViewEnumType.CHANNEL.getCode().equals(conditions.get("viewtype"))) {
                List<Channel> qChannelList = channelService.listCampaignChannel(appid, loginAccount.getId());
                ImmutableList channelids = FluentIterable.from(qChannelList)
                        .filter(channelPredicate)
                        .transform(channelFunction)
                        .toList();
                final String value = channelids != null && channelids.size() > 0 ? Joiner.on(",").skipNulls().join(channelids) : "-999";
                conditions.put("cid", value);
            }
        }

        private void fillRoiFilter(Long appid, Account loginAccount, Map<String, String> conditions, List<Campaign> campaigns, Predicate<Campaign> campaignPredicate, Function<Campaign, String> campaignFunction, Predicate<Channel> channelPredicate, Function<Channel, Long> channelFunction) {
            if (conditions.containsKey("cid")) {
                List<Channel> qChannelList = channelService.listCampaignChannel(appid, loginAccount.getId());
                ImmutableList channelids = FluentIterable.from(qChannelList)
                        .filter(channelPredicate)
                        .transform(channelFunction)
                        .toList();
                final String value = channelids != null && channelids.size() > 0 ? Joiner.on(",").skipNulls().join(channelids) : "-999";
                conditions.put("cid", value);
            } else if (conditions.containsKey("cgroupid")) {
                ImmutableList campids = FluentIterable.from(campaigns)
                        .filter(campaignPredicate)
                        .transform(campaignFunction)
                        .toList();
                final String value = campids != null && campids.size() > 0 ? Joiner.on(",").skipNulls().join(campids) : "-999";
                conditions.put("campaignid", value);
            }
        }

        private void fillChannelEffectFilter(Long appid, Account loginAccount, Map<String, String> conditions, Predicate<Channel> channelAdvertisingPredicate, Function<Channel, Long> channelStringFunction) {
            List<Channel> qChannelList = channelService.listCampaignChannel(appid, loginAccount.getId());
            ImmutableList channelids = FluentIterable.from(qChannelList)
                    .filter(channelAdvertisingPredicate)
                    .transform(channelStringFunction)
                    .toList();
            final String value = channelids != null && channelids.size() > 0 ? Joiner.on(",").skipNulls().join(channelids) : "-999";
            conditions.put("cid", value);
        }
    }

    public static void main(String[] args) {

//        DateTime startdate = new DateTime("2019-08-01");
//        DateTime enddate = new DateTime("2019-08-30");
//        System.out.println(Days.daysBetween(new DateTime(startdate), new DateTime(enddate)).getDays());
//        System.out.println(enddate.plusDays(-29).toString("yyyy-MM-dd"));

        System.out.println(String.join("','", Arrays.asList("aa", "aac", "ccd")));


    }
}
