package com.reyun.service.impl;

import com.reyun.model.App;
import com.reyun.model.ReportToken;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ReportTokenRepository;
import com.reyun.service.ReportTokenService;
import com.reyun.util.NoSQLDBUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.UUID;

@Service
public class ReportTokenServiceImpl implements ReportTokenService {

    @Autowired
    AppRepository appRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    ReportTokenRepository reportTokenRepository;

    protected Logger logger = LoggerFactory.getLogger(ReportTokenServiceImpl.class);


    @Override
    public ReportToken create(App app) {
        reportTokenRepository.delByAppid(app.getAppkey());

        ReportToken reportToken = new ReportToken();
        reportToken.setAppid(app.getAppkey());
        reportToken.setAccessToken(UUID.randomUUID().toString().replace("-", ""));
        reportToken.setRefreshToken(UUID.randomUUID().toString().replace("-", ""));
        reportToken.setAccount(app.getRootParent());
        Date date = new Date();
        reportToken.setRefreshTime(date);
        reportToken.setAccessTokenExpires(new Date(date.getTime()+(ACCESS_TOKEN_EXPIRES_IN*1000)));
        reportToken.setRefreshTokenExpires(new Date(date.getTime()+(REFRESH_TOKEN_EXPIRES_IN*1000)));
        ReportToken save = reportTokenRepository.save(reportToken);
        NoSQLDBUtil.putReportToken(save);
        return save;
    }

    @Override
    public ReportToken getByAppidAndRefreshToken(String appid, String refresh_token) {

        return reportTokenRepository.getByAppidAndRefreshToken(appid,refresh_token);
    }

    @Override
    public ReportToken refresh(ReportToken reportToken) {
        Date date = new Date();
        reportToken.setRefreshTime(date);
        reportToken.setAccessToken(UUID.randomUUID().toString().replace("-", ""));
        reportToken.setAccessTokenExpires(new Date(date.getTime()+(ACCESS_TOKEN_EXPIRES_IN*1000)));
        reportToken.setRefreshTokenExpires(new Date(date.getTime()+(REFRESH_TOKEN_EXPIRES_IN*1000)));
        ReportToken save = reportTokenRepository.save(reportToken);
        NoSQLDBUtil.putReportToken(save);
        return save;
    }
}
