package com.reyun.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.AppTypeEnum;
import com.reyun.dic.ReportEnumType;
import com.reyun.model.*;
import com.reyun.repository.AppRepository;
import com.reyun.service.CommonParamService;
import com.reyun.service.INoSQLDBService;
import com.reyun.service.SdkDebugService;
import com.reyun.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.*;

/**
 * Created by sunhao on 17/6/24.
 * desc:SDK测试服务类
 */
@Service
public class SdkDebugServiceImpl implements SdkDebugService {

    private static final Logger logger = LoggerFactory.getLogger(SdkDebugServiceImpl.class);

    private static final String REPORT_SDK_DEBUG_LOG = "sdk_debug_log";
    private static final String REPORT_SDK_DEBUG_EVENT_COUNT = "sdk_debug_events_count";
    private static final String REPORT_SDK_DEBUG_DEVICE_COUNT = "sdk_debug_device_count";
    private static final String REPORT_CONDITION_DATA_TYPE = "datatype";
    private static final String REPORT_CONDITION_IS_CACHE = "iscache";
    private static final String REPORT_CONDITION_ID = "id";


    private static final String COMMON_ATTR_TYPE_EVENT = "event";
    private static final String COMMON_EVENT_INSTALL = "install";
    private static final String COMMON_EVENT_INSTALL_NAME = "激活";
    private static final String COMMON_EVENT_STARTUP = "startup";
    private static final String COMMON_EVENT_STARTUP_NAME = "启动";
    private static final String COMMON_EVENT_PAGEVIEW = "pageview";
    private static final String COMMON_EVENT_LAUNCH = "launch";
    private static final String COMMON_EVENT_LAUNCH_NAME = "打开小程序";
    private static final String COMMON_EVENT_PAGEVIEW_NAME = "访问页面";


    private static final String FIELD_EVENT_NAME = "xwhat";
    private static final String FIELD_NUMBER = "num";
    private static final String FIELD_OS = "os";
    private static final String FIELD_DEVICE_ID = "deviceid";
    private static final String FIELD_PACKET_CHANNEL = "campaignid";
    private static final String FIELD_LASTEST_TIME = "lasttime";

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private CommonParamService commonParamService;

    @Autowired
    private INoSQLDBService iNoSQLDBService;


    /**
     * 获取所有上报事件和数量
     *
     * @param appId app
     * @return 事件汇总列表
     */
    @Override
    public List<Event4Web> getAllEventsByApp(Long appId) {

        List<Event4Web> event4WebList = new ArrayList<>();

        App app = appRepository.findOne(appId);

        if (null != app) {

            List<Map<String, Object>> eventList = this.getSdkDebugEventCount(app.getAppkey());

            Event4Web installEvent = null;
            Event4Web startupEvent = null;
            Event4Web pageviewEvent = null;
            Event4Web launchEvent = null;

            if (!CollectionUtils.isEmpty(eventList)) {

                Map<String, CommonEvent> commonEventMap = commonParamService.findAllCommonEvent();

                for (Map<String, Object> eventMap : eventList) {

                    String eventName = eventMap.get(FIELD_EVENT_NAME).toString();

                    //是否有启动安装
                    if (COMMON_EVENT_INSTALL.equals(eventName)) {
                        installEvent = this.buildEvent4Web(eventName, commonEventMap.containsKey(eventName) ? commonEventMap.get(eventName).getAlias() : eventName,
                                Long.parseLong(eventMap.get(FIELD_NUMBER).toString()));

                    } else if (COMMON_EVENT_STARTUP.equals(eventName)) {
                        startupEvent = this.buildEvent4Web(eventName, commonEventMap.containsKey(eventName) ? commonEventMap.get(eventName).getAlias() : eventName,
                                Long.parseLong(eventMap.get(FIELD_NUMBER).toString()));

                    } else if (COMMON_EVENT_PAGEVIEW.equals(eventName)){
                        pageviewEvent = this.buildEvent4Web(eventName, commonEventMap.containsKey(eventName) ? commonEventMap.get(eventName).getAlias() : eventName,
                                Long.parseLong(eventMap.get(FIELD_NUMBER).toString()));

                    }  else if (COMMON_EVENT_LAUNCH.equals(eventName)){
                        launchEvent = this.buildEvent4Web(eventName, commonEventMap.containsKey(eventName) ? commonEventMap.get(eventName).getAlias() : eventName,
                                Long.parseLong(eventMap.get(FIELD_NUMBER).toString()));

                    }
                    else{
                        event4WebList.add(this.buildEvent4Web(eventName, commonEventMap.containsKey(eventName) ? commonEventMap.get(eventName).getAlias() : eventName,
                                Long.parseLong(eventMap.get(FIELD_NUMBER).toString())));
                    }
                }
            }

            if (!AppPlatformEnum.H5.getKey().equals(app.getPlatform()) && !AppPlatformEnum.WX.getKey().equals(app.getPlatform()) && !AppPlatformEnum.WG.getKey().equals(app.getPlatform())) {

                //install,非h5类型的app
                if (null == startupEvent) {
                    event4WebList.add(0, this.buildEvent4Web(COMMON_EVENT_STARTUP, COMMON_EVENT_STARTUP_NAME, 0));
                } else {
                    event4WebList.add(0, startupEvent);
                }

                //startup,非h5类型的app
                if (null == installEvent) {
                    event4WebList.add(0, this.buildEvent4Web(COMMON_EVENT_INSTALL, COMMON_EVENT_INSTALL_NAME, 0));
                } else {
                    event4WebList.add(0, installEvent);
                }

            } else if (AppPlatformEnum.WX.getKey().equals(app.getPlatform()) || AppPlatformEnum.WG.getKey().equals(app.getPlatform())) {

                if (null == launchEvent) {
                    event4WebList.add(0, this.buildEvent4Web(COMMON_EVENT_LAUNCH, COMMON_EVENT_LAUNCH_NAME, 0));
                } else {
                    event4WebList.add(0, launchEvent);
                }

                if (null == pageviewEvent) {
                    event4WebList.add(0, this.buildEvent4Web(COMMON_EVENT_PAGEVIEW, COMMON_EVENT_PAGEVIEW_NAME, 0));
                } else {
                    event4WebList.add(0, pageviewEvent);
                }

            } else {

                //pageview,h5类型的app
                if (null == pageviewEvent && AppPlatformEnum.H5.getKey().equals(app.getPlatform())) {
                    event4WebList.add(0, this.buildEvent4Web(COMMON_EVENT_PAGEVIEW, COMMON_EVENT_PAGEVIEW_NAME, 0));
                } else {
                    event4WebList.add(0, pageviewEvent);
                }
            }
        }

        return event4WebList;
    }

    /**
     * 构建Event4Web
     */
    private Event4Web buildEvent4Web(String eventName, String eventAlias, long eventNumber) {

        Event4Web event4Web = new Event4Web();

        event4Web.setEventName(eventName);
        event4Web.setEventNameAlias(eventAlias);
        event4Web.setEventNumber(eventNumber);

        return event4Web;
    }

    /**
     * 获取所有上报日志详情
     *
     * @param appId app
     * @return 日志列表
     */
    @Override
    public List<TkioDebugEvents> getAllEventLogByApp(Long appId) {

        App app = appRepository.findOne(appId);

        if (null != app) {

            List<TkioDebugEvents> tkioDebugEventsList = this.getSdkDebugLogList(app.getAppkey());

            if (!CollectionUtils.isEmpty(tkioDebugEventsList)) {

                Map<String, CommonEvent> commonEventMap = commonParamService.findAllCommonEvent();

                //填充通用事件名字和日期
                for (TkioDebugEvents tkioDebugEvents : tkioDebugEventsList) {

                    tkioDebugEvents.setEventName(tkioDebugEvents.getXwhat());

                    //填充通用事件名字
                    CommonEvent commonEvent = commonEventMap.get(tkioDebugEvents.getXwhat());
                    if (null != commonEvent) {

                        tkioDebugEvents.setEventName(commonEvent.getAlias());
                    }
                }

                return tkioDebugEventsList;
            }
        }

        return new ArrayList<>();
    }

    /**
     * 获取按照设备ID汇总的事件列表
     *
     * @param appId app
     * @return 汇总列表
     */
    @Override
    public List<SdkDebugDevice> getSdkDebugDeviceCount(Long appId) {

        List<SdkDebugDevice> sdkDebugDeviceList = new ArrayList<>();

        App app = appRepository.findOne(appId);

        if (null != app) {

            //从report获取数据
            String responseJson = this.getDataFromReport(REPORT_SDK_DEBUG_DEVICE_COUNT, app.getAppkey(), null);

            if (!StringUtils.isEmpty(responseJson)) {

                try {

                    ReportListResult reportListResult = new ObjectMapper().readValue(responseJson, ReportListResult.class);

                    if (null != reportListResult.getVal()) {

                        for (Map<String, Object> val : reportListResult.getVal()) {
                            //通过返回值构建数据
                            SdkDebugDevice sdkDebugDevice = new SdkDebugDevice(val.get(FIELD_OS).toString(), val.get(FIELD_DEVICE_ID).toString(),
                                    val.get(FIELD_PACKET_CHANNEL).toString(), val.get(FIELD_LASTEST_TIME).toString(), Long.parseLong(val.get(FIELD_NUMBER).toString()));

                            sdkDebugDeviceList.add(sdkDebugDevice);
                        }
                    }

                } catch (IOException e) {

                    logger.error(e.getMessage());
                }
            }
        }

        return sdkDebugDeviceList;
    }

    /**
     * 获取通用属性列表
     *
     * @return List<CommonParam>
     */
    @Override
    public List<CommonParam> getCommonEventAttr() {

        return this.commonParamService.listAllByType(COMMON_ATTR_TYPE_EVENT);
    }

    @Override
    public boolean appSdkDebugComplete(Account loginAccount, Long appId) {
        App app = appRepository.findOne(appId);

        if (null != app) {
            boolean isDynamoDbUpdated = true;

            logger.info("完成调试，修改ddb参数 : {}.", app.getAppkey());

            //同步DynamoDb
            if (iNoSQLDBService.haveAppDebugItem(app.getAppkey())) {
                isDynamoDbUpdated =  iNoSQLDBService.updateAppSdkDebugItem(app.getAppkey(),false);
            }
            logger.info("完成调试，修改ddb参数结果: {}. appkey: {}", isDynamoDbUpdated, app.getAppkey());

            if(isDynamoDbUpdated){
                //设置APP debug模式为false
                app.setIsDebug(false);
                app.setModifyTime(new Date());
                app.setModifyAccount(loginAccount.getId().toString());
                app.setDebugDate(DateUtil.getBeforeDays(0));
                appRepository.save(app);

                logger.info("完成调试，修改mysql参数结果: {}. appkey: {}", app.getIsDebug(), app.getAppkey());
                //完成调试后发送给ads

                logger.info("完成调试," + app.toString());

                //region Description: 当应用调度完成后，调用office分发任务至相关运营人员。
                if (AppTypeEnum.APP.getCode().equals(app.getTagType()) || AppTypeEnum.GAME.getCode().equals(app.getTagType())) {
                    boolean flag = doInvokeOfficeAudit(appId);
                    logger.info("调试完成, 分配任务至运营人员. app:{}, httpstatus: {},isdebug status: {}, isDynamoDbUpdated: {}", appId, flag,app.getIsDebug(), isDynamoDbUpdated);
                }

                //前面的APP顺序依次延后
                logger.info("完成调试，调顺序. appkey: {}", app.getAppkey());
                List<App> appList = appRepository.listAppsBefore(loginAccount.getRootParent(), app.getSort());
                if(ValidateUtil.isValid(appList)){
                    List<App> list = new ArrayList<>();
                    for(App a : appList){
                        int sort = a.getSort() + 1;
                        a.setSort(sort);
                        list.add(a);
                    }
                    app.setSort(1);
                    list.add(app);
                    appRepository.save(list);
                }


                //endregion
                return true;
            }
            logger.info("完成调试，ddb结果: {}. mysql结果: {}, appkey: {}", isDynamoDbUpdated, app.getIsDebug(), app.getAppkey());
        }

        return false;
    }

    private boolean doInvokeOfficeAudit(Long app){
        String url = Constant.adminUrl + "/api/app/tag/task/distribute?app=" + app;
        String result = HttpClientUtil.doHttpGetRequest(url, "");

        if (!StringUtil.isEmpty(result)) {
            Map resObj;
            try {
                resObj = new ObjectMapper().readValue(result, Map.class);
            } catch (IOException e) {
                logger.error("doInvokeOfficeAudit occur failed. app:{}", app);
                return false;
            }
            return "200".equals(resObj.get("code"));
        } else {
            return true;
        }


    }

    /**
     * 从report获取debug的数据
     *
     * @param appKey 32位appKey
     * @return List<TkioDebugEvents>
     */
    private List<TkioDebugEvents> getSdkDebugLogList(String appKey) {

        List<TkioDebugEvents> resultLogList = new ArrayList<>();

        String responseJson = this.getDataFromReport(REPORT_SDK_DEBUG_LOG, appKey, null);
        //处理请求结果
        if (!StringUtils.isEmpty(responseJson)) {
            try {

                ReportListResult reportListResult = new ObjectMapper().readValue(responseJson, ReportListResult.class);

                if (null != reportListResult.getVal()) {

                    for (Map<String, Object> debugLog : reportListResult.getVal()) {

                        TkioDebugEvents tkioDebugEvents = new ObjectMapper().convertValue(debugLog, TkioDebugEvents.class);

                        if (null != tkioDebugEvents) {
                            resultLogList.add(tkioDebugEvents);
                        }
                    }
                }

            } catch (IOException e) {

                logger.error(e.getMessage());
            }

        }

        return resultLogList;
    }

    /**
     * 获取时间统计结果
     *
     * @param appKey 32位key
     * @return List<Map>
     */
    private List<Map<String, Object>> getSdkDebugEventCount(String appKey) {

        List<Map<String, Object>> mapList = new ArrayList<>();

        String responseJson = this.getDataFromReport(REPORT_SDK_DEBUG_EVENT_COUNT, appKey, null);


        if (!StringUtils.isEmpty(responseJson)) {

            try {

                ReportListResult reportListResult = new ObjectMapper().readValue(responseJson, ReportListResult.class);

                mapList = reportListResult.getVal();

            } catch (IOException e) {
                logger.error(e.getMessage());
            }

        }

        return mapList;
    }

    /**
     * 发送获取数据请求
     *
     * @param templateName 模板名称
     * @param appKey       32KEY
     * @return response String
     */
    private String getDataFromReport(String templateName, String appKey, String id) {

        //构建条件
        Map<String, String> conditions = new HashMap<>();
        conditions.put(REPORT_CONDITION_DATA_TYPE, ReportEnumType.LIST.getCode());
        conditions.put(REPORT_CONDITION_IS_CACHE, "0");

        //按照ID查询
        if (!StringUtils.isEmpty(id)) {
            conditions.put(REPORT_CONDITION_ID, id);
        }

        String url = Constant.reportUrl + "/api/trackingio/" + templateName + "/" + appKey;

        return HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);
    }
}
