package com.reyun.service.impl;

import com.reyun.model.Channel;
import com.reyun.model.SemRename;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.repository.SemRenameRepository;
import com.reyun.service.SemRenameService;
import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.ResultModel;
import com.reyun.util.ResultStatus;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;


@Service
public class SemRenameServiceImpl implements SemRenameService {

    private static Logger logger = LoggerFactory.getLogger(SemRenameServiceImpl.class);
    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;
    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    private SemRenameRepository semRenameRepository;

    @Autowired
    private AccountRepository accountRepository;

    private static String titleFormat = "关键词id,关键词名称,创意id,创意名称,";

    @Override
    public SemRename semRenameTransfer(MultipartFile file, Long cid, Long appId, Long loginAccount) throws IOException {

        String fileName = "bdkeyword_" + appId + "_" + cid + "_" + new Date().getTime();
        final String s3key = DOWNLOAD_PATH + fileName;
        ObjectStorageUtil.putObjectInStream(S3_DOWNLOAD_BUCKET, s3key, file.getInputStream(), "xlsx", file.getSize());
        Channel one = channelRepository.findOne(cid);
        SemRename semRename = new SemRename();
        semRename.setCid(cid);
        semRename.setCreateTime(new Date());
        semRename.setStatus(0);
        semRename.setCname(one.getName());
        semRename.setFileName(fileName);
        semRename.setAppid(appId);
        semRename.setRootAccount(accountRepository.findOne(loginAccount).getRootParent());
        return semRenameRepository.save(semRename);
    }

    public ResultModel checkSemTemplateFormat(MultipartFile file) {

        InputStream stream = null;
        try {
            stream = file.getInputStream();
            if (stream == null) {
                return ResultModel.ERROR(ResultStatus.FILE_TYPE_ERROR);
            }
            Workbook workbook = WorkbookFactory.create(stream);
            Sheet sheet = workbook.getSheetAt(0);
            Row row = sheet.getRow(0);
            DataFormatter formatter = new DataFormatter();

            StringBuffer title = new StringBuffer();
            if (row != null) {
                for (int i = 0; i < 5; i++) {
                    Cell cell = row.getCell(i);
                    if (null != cell && !cell.getCellTypeEnum().equals(CellType.BLANK)) {
                        title.append(formatter.formatCellValue(cell) + ",");
                    }
                }
            }

            if (!StringUtils.equals(title.toString(), titleFormat)) {
                //标题被篡改
                return ResultModel.ERROR(ResultStatus.KEY_WORDS_TITLE_ERRO);
            }

            int rowNumber = sheet.getLastRowNum();
            Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");

            List<String> erroMessageData = new ArrayList<>();

            for (int j = 1; j <= rowNumber; j++) {
                Row row_data = sheet.getRow(j);
                if (row_data != null) {
                    if (!this.checkCellFormat(row_data.getCell(0), formatter, pattern)) {
                        erroMessageData.add("第" + (j + 1) + "行关键词id有误");
                    }
                    if (!this.checkCellFormat(row_data.getCell(2), formatter, pattern)) {
                        erroMessageData.add("第" + (j + 1) + "行创意id有误");
                    }
                }

                if (erroMessageData.size() >= 5) {
                    break;
                }
            }

            if (!erroMessageData.isEmpty()) {
                StringBuffer erroBuffer = new StringBuffer();

                for (int i = 0; i < erroMessageData.size(); i++) {
                    if (erroBuffer.length() == 0) {
                        erroBuffer.append(erroMessageData.get(i));
                    } else {
                        erroBuffer.append("、").append(erroMessageData.get(i));
                    }
                }
                erroBuffer.append(" ...... ,请检查后重新上传");
                return ResultModel.ERROR(erroBuffer.toString());
            }

            
            return null;

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("检验模板错误{}", e.getMessage());
        } finally {

            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        return null;
    }

    private boolean checkCellFormat(Cell cell, DataFormatter formatter, Pattern pattern) {
        if (null != cell && !cell.getCellTypeEnum().equals(CellType.BLANK)) {
            String id_data = formatter.formatCellValue(cell);

            if (id_data != null && id_data.indexOf("E") > 0) {
                // 对科学计数法做处理
                if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                    double id = cell.getNumericCellValue();
                    BigDecimal decimal = new BigDecimal(id);
                    id_data = decimal.toPlainString();
                }
            }

            if (!pattern.matcher(id_data).matches()) {
                // id不为数字
                return false;
            }
        }
        return true;
    }
}
