package com.reyun.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Function;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.controller.manage.MngCampaignController;
import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.dic.UploadErrorEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AppService;
import com.reyun.service.CampaignService;
import com.reyun.service.ChannelService;
import com.reyun.service.SubCampaignService;
import com.reyun.util.AwsS3Util;
import com.reyun.util.CipherUtil;
import com.reyun.util.Constant;
import com.reyun.util.ExcelUtil;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.StringUtil;
import com.reyun.util.ValidateUtil;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by sunhao on 17/5/22.
 * desc:子活动service impl
 */
@Service
public class SubCampaignServiceImpl
        implements SubCampaignService
{

    private Logger logger = LoggerFactory.getLogger(MngCampaignController.class);

    @Autowired
    AppService appService;

    @Autowired
    SubCampaignRepository subCampaignRepository;

    @Autowired
    UploadHistoryRepository uploadHistoryRepository;

    @Autowired
    SubCampaignExportRepository subCampaignExportRepository;

    @Autowired
    ChannelService channelService;

    @Autowired
    CampaignService campaignService;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    PackageSubCampaignRepository packageSubCampaignRepository;

    private final static String PARAM_KWID = "kwid";
    private final static String PARAM_CREATIVE = "creative";
    private final static String PARAM_SJUMP = "sjump";

    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;
    private static final String DOWNLOAD_PATH = "trackingio/subCampaign/";

    private static final int BUFFER_SIZE = 4096;

    @Override
    public List<SubCampaign> listSubCampaign(Long campaignId)
    {
        List<SubCampaign> subCampaignList = subCampaignRepository.listSubCampaignByCampaign(campaignId);

        return buildSubCampaignUrl(subCampaignList);
    }

    @Override
    public List<SubCampaign> listDisableCampaign(Long campaignId)
    {
        List<SubCampaign> subCampaignList = subCampaignRepository.listDisableByCampaign(campaignId);

        return buildSubCampaignUrl(subCampaignList);

    }

    @Override
    public SubCampaign findSubCampaign(Long subCampaignId)
    {
        SubCampaign subCampaign =  subCampaignRepository.findOne(subCampaignId);

        if(null != subCampaign){
            subCampaign.setSubCampaignUrl(generateShortUrlForKeyword(subCampaign.getSurl(), subCampaign.getKeyword(),
                    subCampaign.getCreative(), subCampaign.getUrl()));
        }

        return subCampaign;
    }


    /**
     * 构建子活动的显示的URL
     */
    private List<SubCampaign> buildSubCampaignUrl(List<SubCampaign> subCampaignList) {

        if (!CollectionUtils.isEmpty(subCampaignList)) {
            for (SubCampaign subCampaign : subCampaignList) {

                subCampaign.setSubCampaignUrl(generateShortUrlForKeyword(subCampaign.getSurl(), subCampaign.getKeyword(),
                        subCampaign.getCreative(), subCampaign.getUrl()));
            }
        }

        return subCampaignList;
    }

    /**
     * 保存子活动,返回值为未保存成功的子活动
     * created by sunhao 20170523
     */
    @Override
    public Map<String, List> saveSubCampaign(Account account, SubCampaign subCampaignSource)
    {

        Map<String, List> result = new HashMap<>();
        List<SubCampaign> subCampaignList = new ArrayList<>();
        List<String> resultList = new ArrayList<>();
        //生成带参数的URL
        Campaign campaign = campaignService.findOneById(subCampaignSource.getCampaignId());

        if (null != campaign) {

            Map<String, SubCampaign> subCampaignMap = this.getAllSubCampaignByCampaignId(campaign.getId());

            //分割创意
            if (!StringUtils.isEmpty(subCampaignSource.getCreative())) {

                String[] creativeArray = subCampaignSource.getCreative().split(",");

                for (String creative : creativeArray) {

                    //关键字创意重复校验
                    if (!subCampaignMap.containsKey(this.generateKey(subCampaignSource.getKeyword(), creative))) {

                        subCampaignList.add(buildOneSubCampaign(subCampaignSource, campaign, account, creative));
                    }
                    else {

                        //保存重复数据
                        resultList.add(creative);
                    }
                }
            }
            else if (!subCampaignMap.containsKey(this.generateKey(subCampaignSource.getKeyword(), subCampaignSource.getCreative()))) {

                subCampaignList.add(buildOneSubCampaign(subCampaignSource, campaign, account, null));
            }
            else {

                //保存重复数据
                resultList.add(subCampaignSource.getKeyword());
            }

            //保存在无重复的情况下保存
            if (CollectionUtils.isEmpty(resultList)) {

                subCampaignList = subCampaignRepository.save(subCampaignList);
            }
        }

        result.put("duplicate", resultList);
        result.put("success", subCampaignList);
        return result;
    }

    /**
     * 构建活动数据
     */
    private SubCampaign buildOneSubCampaign(SubCampaign subCampaignSource, Campaign campaign, Account account, String creative)
    {

        SubCampaign subCampaign = new SubCampaign(subCampaignSource);

//        String encodeUrl = generateShortUrlForKeyword(campaign.getSurl(), subCampaignSource.getKeyword(), creative, subCampaignSource.getUrl());

        subCampaign.setCreative(creative);
        subCampaign.setChannelId(campaign.getChannel());
        subCampaign.setApp(campaign.getApp());
        subCampaign.setCreateAccount(account.getId());
        subCampaign.setCreateTime(new Date());
        subCampaign.setSurl(campaign.getSurl());
        subCampaign.setType("keyword");
        subCampaign.setDelFlag(false);

        return subCampaign;
    }

    /**
     * 生成带参数的URL
     * http://uri6.com/tkio/z6vy2ex?kwid=百度&creative=搜狗
     */
    private String generateShortUrlForKeyword(String surl, String keyword, String creative, String url)
    {
//        System.out.println(surl + "," + keyword + "," + creative + "," + url);

        StringBuilder resultUrl = new StringBuilder();

        try {

            //活动短链
            resultUrl.append(Constant.trackurl).append("/").append(surl);
            //关键字
            resultUrl.append("?").append(PARAM_KWID).append("=").append(URLEncoder.encode(keyword, StandardCharsets.UTF_8.toString()));
            //创意
            if (!StringUtils.isEmpty(creative)) {
                resultUrl.append("&").append(PARAM_CREATIVE).append("=").append(URLEncoder.encode(creative, StandardCharsets.UTF_8.toString()));
            }
            //跳转
            resultUrl.append("&").append(PARAM_SJUMP).append("=").append(URLEncoder.encode(getURLParamEncode(url), StandardCharsets.UTF_8.toString()));
        }
        catch (UnsupportedEncodingException e) {

            logger.error(e.getMessage());
        }

//        System.out.println(resultUrl.toString());

        return resultUrl.toString();
    }

    /**
     * 将URL里面的中文参数ENCODE
     */
    private String getURLParamEncode(String url)
            throws UnsupportedEncodingException
    {

        StringBuilder urlBuilder = new StringBuilder();

        String[] urlArray = url.split("\\?");

        if (urlArray.length == 2) {

            urlBuilder.append(urlArray[0]).append("?");

            String[] paramArray = urlArray[1].split("&");

            for (String param : paramArray) {

                String[] params = param.split("=");
                urlBuilder.append("&").append(params[0]).append("=").append(params.length>1?URLEncoder.encode(params[1], "UTF-8"):"");
            }
        }
        else {

            urlBuilder.append(url);
        }

        return urlBuilder.toString();
    }

    /**
     * 查询子活动数量
     * Created by sunhao on 17/5/22.
     */
    @Override
    public Long findSubCampaignNumber(Long campaignId)
    {

        BigInteger number = subCampaignRepository.countSubCampaignByCampaign(campaignId);

        return (null != number) ? number.longValue() : 0L;
    }

    /**
     * 修改连接地址
     * Created by sunhao on 17/5/22.
     */
    @Override
    public int modifySubCampaignUrl(Long subCampaignId, String url, Account account)
    {

        if (StringUtil.urlMatch(url)) {

            return subCampaignRepository.updateSubCampaignUrl(subCampaignId, url, account.getId(), new Date());
        }
        else {
            return 0;
        }
    }

    /**
     * 单个或者批量停用子活动
     */
    @Override
    public int deleteSubCampaign(Account account, String ids)
    {

        List<Long> subCampaignList = StringUtil.transString2LongList(ids);

        return subCampaignRepository.deleteSubCampaign(subCampaignList, account.getId(), new Date());
    }

    /**
     * 单个或者批量启用子活动
     */
    @Override
    public int enableSubCampaign(Account account, String ids)
    {

        List<Long> subCampaignList = StringUtil.transString2LongList(ids);

        return subCampaignRepository.enableSubCampaign(subCampaignList, account.getId(), new Date());
    }

    /**
     * 导入关键字子活动
     * Created by sunhao on 17/5/22.
     */
    @Override
    public UploadHistory readKeywordFile(MultipartFile file, Account account, Long campaignId)
    {

        UploadHistory result = new UploadHistory();

        try {
            //读取EXCEL数据到uploadHistory中
            UploadHistory uploadHistory = ExcelUtil.readKeywordsExcel(file.getInputStream());

            //上传有误 return
            if (null == uploadHistory) {
                result.setErrorType(UploadErrorEnumType.READ_ERROR.getKey());
                return result;
            }

            //空表或者表头有误 return
            if (!StringUtils.isEmpty(uploadHistory.getErrorType())) {
                result.setErrorType(uploadHistory.getErrorType());
                return result;
            }

            List<Integer> uploadRepeatList = new ArrayList<>();
            List<List<Integer>> innerRepeatList = new ArrayList<>();
            StringBuilder rowDataStringBuilder = new StringBuilder();

            //校验是否重复
            Map<String, SubCampaign> subCampaignMap = getAllSubCampaignByCampaignId(campaignId);
            Map<String, Map<Integer, String>> keywordCreativeMap = uploadHistory.getKeywordCreativeMap();

            for (String keywordCreative : keywordCreativeMap.keySet()) {

                if (subCampaignMap.containsKey(keywordCreative)) {
                    //数据库重复
                    uploadRepeatList.addAll(keywordCreativeMap.get(keywordCreative).keySet());
                }
                else if (keywordCreativeMap.get(keywordCreative).size() >= 2) {
                    //表格内重复
                    innerRepeatList.add(new ArrayList<>(keywordCreativeMap.get(keywordCreative).keySet()));
                    //记录正确数据
                    String rowData = keywordCreativeMap.get(keywordCreative).entrySet().iterator().next().getValue();
                    rowDataStringBuilder.append(rowData).append("\n");
                }
                else {
                    //记录正确数据
                    String rowData = keywordCreativeMap.get(keywordCreative).entrySet().iterator().next().getValue();
                    rowDataStringBuilder.append(rowData).append("\n");
                }
            }

            //排序uploadRepeatList
            Collections.sort(uploadRepeatList, new Comparator<Integer>()
            {
                @Override
                public int compare(Integer o1, Integer o2)
                {
                    return o1 - o2;
                }
            });

            //设置返回值
            result.setUploadRepeatedList(uploadRepeatList);
            result.setInnerRepeatList(innerRepeatList);
            result.setUploadErrorList(uploadHistory.getUploadErrorList());
            result.setRowNumber(uploadHistory.getRowNumber());

            //保存记录,上传数据不为空,且错误列表为空的时候保存
            if (rowDataStringBuilder.length() > 0 && CollectionUtils.isEmpty(uploadHistory.getUploadErrorList())) {

                uploadHistory.setUploadData(rowDataStringBuilder.toString());
                uploadHistory.setAccountId(account.getId());
                uploadHistory.setCampaignId(campaignId);
                uploadHistory.setUploadTime(new Date());

                uploadHistory = uploadHistoryRepository.save(uploadHistory);
                result.setId(uploadHistory.getId());
            }
            else if (!CollectionUtils.isEmpty(uploadHistory.getUploadErrorList())) {
                //数据有错
                result.setErrorType(UploadErrorEnumType.CONTENT_ERROR.getKey());
            }
            else if (rowDataStringBuilder.length() <= 0) {
                //全部重复
                result.setErrorType(UploadErrorEnumType.REPEAT_ALL.getKey());
            }
        }
        catch (IOException e) {
            logger.info(e.getMessage());
        }

        return result;
    }

    /**
     * 保存上传的数据
     * Created by sunhao on 17/5/22.
     */
    @Override
    public List<SubCampaign> saveKeywordInfo(Long uploadHistoryId, Account account, Long campaignId)
    {

        List<Integer> saveErrorList = new ArrayList<>();
        List<SubCampaign> subCampaignList = new ArrayList<>();

        UploadHistory uploadHistory = uploadHistoryRepository.findOne(uploadHistoryId);
        Campaign campaign = campaignService.findOneById(campaignId);

        if (!StringUtils.isEmpty(uploadHistory.getUploadData()) && null != campaign) {

            //分割构建数据
            String[] dataArray = uploadHistory.getUploadData().split("\n");

            for (String data : dataArray) {

                String[] infoArray = data.split(";");

                String url = infoArray[0];
                String keyword = infoArray[1];
                String creative = (infoArray.length == 3 ? infoArray[2] : null);

                subCampaignList.add(this.buildSubCampaign(campaign, account, url, keyword, creative));
            }

            //保存
            if (!CollectionUtils.isEmpty(subCampaignList) && CollectionUtils.isEmpty(saveErrorList)) {

                subCampaignList = subCampaignRepository.save(subCampaignList);
            }
        }

        return subCampaignList;
    }

    /**
     * 构建子活动MODEL
     */
    private SubCampaign buildSubCampaign(Campaign campaign, Account account, String url, String keyword, String creative)
    {

        SubCampaign subCampaign = new SubCampaign();

        subCampaign.setChannelId(campaign.getChannel());
        subCampaign.setCampaignId(campaign.getId());
        subCampaign.setApp(campaign.getApp());
        subCampaign.setKeyword(keyword);
        subCampaign.setUrl(url);
        subCampaign.setCreative(creative);
        subCampaign.setSurl(campaign.getSurl());
        subCampaign.setCreateAccount(account.getId());
        subCampaign.setCreateTime(new Date());
        subCampaign.setDelFlag(false);
        subCampaign.setType("keyword");

        return subCampaign;
    }

    /**
     * 保存导出记录
     * create by sunhao 20170524
     */
    @Override
    public boolean exportSubCampaign(Account account, Long campaignId, List<Long> ids, String lanType)
    {

        Long recordNumber;
        Campaign campaign = campaignService.findOneById(campaignId);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");

        if (null != campaign) {

            String exportIds = StringUtil.transList2String(ids);
            Date currentDate = new Date();
            String fileName = account.getId() + "_" + campaign.getCampaignid() + "_subCampaign_" + CipherUtil.generatePassword(exportIds)
                    + "_" + simpleDateFormat.format(currentDate) + "_" + currentDate.getTime() + ".xls";

            //查询数据条数
            if (CollectionUtils.isEmpty(ids)) {
                recordNumber = subCampaignRepository.countSubCampaignByCampaign(campaignId).longValue();
            }
            else {
                recordNumber = subCampaignRepository.countSubCampaignById(ids).longValue();
            }

            //有数据导出
            if (recordNumber > 0) {

                SubCampaignExport subCampaignExport = new SubCampaignExport();
                subCampaignExport.setCampaignId(campaignId);
                subCampaignExport.setAppId(campaign.getApp());
                subCampaignExport.setCampaignKey(campaign.getCampaignid());
                subCampaignExport.setCampaignName(campaign.getName());
                subCampaignExport.setCreateAccount(account.getId());
                subCampaignExport.setCreateTime(currentDate);
                subCampaignExport.setRecordNumber(recordNumber);
                subCampaignExport.setStatus(DownloadStatusEnumType.INIT.getCode());
                subCampaignExport.setFileName(fileName);
                subCampaignExport.setExportIds(exportIds);
                subCampaignExport.setLanguageType(lanType);

                subCampaignExportRepository.save(subCampaignExport);

                return true;
            }
        }

        return false;
    }

    @Override
    public boolean exportBatchSubCampaign(Account account, Long appId,List<Long> idss) {
        StringBuffer subCampaignIds = new StringBuffer();
        ArrayList<SubCampaign> subCampaigns = new ArrayList<>();
        if(idss.size()!=0){
            String exportIds = StringUtil.transList2String(idss);
            String[] ids = exportIds.split(",");
            for(String s:ids){
                List<SubCampaign> byAppAndCampaignId = subCampaignRepository.findByAppAndCampaignId(appId, new Long(s));
                if(byAppAndCampaignId.size()!=0){
                    subCampaigns.addAll(byAppAndCampaignId);
                }
            }
        }

        if (subCampaigns.size() != 0) {
            for (SubCampaign subCampaign : subCampaigns) {
                Long id = subCampaign.getId();
                subCampaignIds = subCampaignIds.append(id + ",");
            }
            String exportIds = subCampaignIds.substring(0, subCampaignIds.length() - 1);
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
            Date currentDate = new Date();
            String fileName = account.getId() + "_" + appId + "_subCampaign_" + CipherUtil.generatePassword(exportIds)
                    + "_" + simpleDateFormat.format(currentDate) + "_" + currentDate.getTime() + ".xls";

            SubCampaignExport subCampaignExport = new SubCampaignExport();
            subCampaignExport.setAppId(appId);
            subCampaignExport.setCampaignName("子活动批量导出");
            subCampaignExport.setCreateAccount(account.getId());
            subCampaignExport.setCreateTime(currentDate);
            subCampaignExport.setRecordNumber(Long.valueOf(subCampaigns.size()));
            subCampaignExport.setStatus(DownloadStatusEnumType.INIT.getCode());
            subCampaignExport.setFileName(fileName);
            subCampaignExport.setExportIds(exportIds);

            subCampaignExportRepository.save(subCampaignExport);
            return true;
        }
        return false;
    }


    /**
     * 下载导出数据
     */
    @Override
    public HttpServletResponse downloadSubCampaign(HttpServletRequest request, HttpServletResponse response, Account account, Long exportId)
            throws IOException
    {

        SubCampaignExport subCampaignExport = subCampaignExportRepository.findOne(exportId);
        if (subCampaignExport == null) {
            return null;
        }

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        Campaign campaign=null;
        App app=null;
        String filename="";
        if(subCampaignExport.getCampaignId()!=null){
            campaign = campaignService.findOneById(subCampaignExport.getCampaignId());
            app = appService.findById(campaign.getApp());
            //产品名称_推广活动名称_渠道名称_创建导出时间_子活动短链地址列表
            filename = app.getName() + "_" + campaign.getName() + "_" + simpleDateFormat.format(subCampaignExport.getCreateTime()) + "_子活动短链地址列表.xls";
        }else {
            filename=simpleDateFormat.format(subCampaignExport.getCreateTime()) + "_全部子活动短链地址列表.xls";
        }

        String downloadIP = request.getHeader("x-forwarded-for") == null ? request.getRemoteAddr() : request.getHeader("x-forwarded-for");
        //为了解决中文名称乱码问题
        filename = new String(filename.getBytes(StandardCharsets.UTF_8.toString()));
        filename = URLEncoder.encode(filename, StandardCharsets.UTF_8.toString());
        response.setHeader("filename", filename);
        response.setHeader("content-disposition", "attachment; filename=" + filename);
        response.setHeader("Content-Type", "application/vnd.ms-excel");


        InputStream inputStream = null;
        OutputStream outputStream = response.getOutputStream();

        try {
            inputStream = ObjectStorageUtil.getObjectContentStream(Constant.bucket,
                    "trackingio/subCampaign/" + subCampaignExport.getFileName());
            //独处inputstream中的数据,然后通过output写出
            byte[] buffer = new byte[BUFFER_SIZE];
            int bytesRead = -1;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        }
        catch (IOException e) {
            return null;
        }
        finally {
            IOUtils.closeQuietly(inputStream);
            IOUtils.closeQuietly(outputStream);
        }

        //更新下载人和IP
        subCampaignExportRepository.updateDownloadInfo(account.getId(), downloadIP, new Date(), subCampaignExport.getId());

        return response;
    }


    /**
     * 获取所有子活动MAP,key为keyword-creative
     */
    private Map<String, SubCampaign> getAllSubCampaignByCampaignId(Long campaignId)
    {

        List<SubCampaign> subCampaignList = subCampaignRepository.listAllSubCampaignByCampaign(campaignId);

        return Maps.uniqueIndex(subCampaignList, new Function<SubCampaign, String>()
        {
            @Override
            public String apply(SubCampaign subCampaign)
            {
                return generateKey(subCampaign.getKeyword(), subCampaign.getCreative());
            }
        });
    }

    /**
     * 生成keyword-creative
     */
    private String generateKey(String keyword, String creative)
    {
        return keyword + "-" + (StringUtils.isEmpty(creative) ? "" : creative);
    }

    @Override
    public List<SubCampaignExport> listSubCampaignExport(Account loginAccount, Long appId)
    {
        //母账号能看到所有的导出列表,子账号只能看到自己创建的。add by sunhao 20170828
        if (loginAccount.getIsSuperUser()) {
            return subCampaignExportRepository.listByApp(appId);
        } else {
            return subCampaignExportRepository.listByAccountAndApp(appId, loginAccount.getId());
        }
    }

    @Override
    public int updateSubCampaignExportStatus(Long id, String status)
    {
        return subCampaignExportRepository.updateStatus(id, status);
    }

    @Override
    public void deleteDownloadSubCampaign(Long id)
    {
        SubCampaignExport export = subCampaignExportRepository.findOne(id);
        final String s3key = DOWNLOAD_PATH + export.getFileName();
        //删除文件
        if (ObjectStorageUtil.doesObjectExist(S3_DOWNLOAD_BUCKET, s3key)) {
            ObjectStorageUtil.deleteObject(S3_DOWNLOAD_BUCKET, s3key);
        }

        //删除记录
        subCampaignExportRepository.delete(id);
        return;
    }

    @Override
    public List<CampaignUrl4Web> findSubCampaignByUrl(Long appid, String url)
    {
        List<CampaignUrl4Web> result = new ArrayList<>();
        List<CampaignUrl4Web> temp = new ArrayList<>();
        List<Long> cidList = new ArrayList<>();
        List<Long> campaignIdList = new ArrayList<>();

        if(!StringUtils.isEmpty(url)){
            url = JSONObject.parseObject(url).get("url").toString();
        }


        List<SubCampaign> subCampaignList = subCampaignRepository.findAllByAppAndUrl(appid, url);
        if (ValidateUtil.isValid(subCampaignList)) {
            for (SubCampaign ca : subCampaignList) {
                Long cid = ca.getChannelId();
                Long campaignId = ca.getCampaignId();
                if (!cidList.contains(cid)) {
                    cidList.add(cid);
                }
                if (!campaignIdList.contains(campaignId)) {
                    campaignIdList.add(campaignId);
                }
                CampaignUrl4Web web = new CampaignUrl4Web();
                web.setCampaignId(campaignId);
                web.setSubCampaignId(ca.getId());
                web.setChannelId(cid);
                web.setSubCampaignName(ca.getKeyword());
                web.setTabType(CampaignUrl4Web.TAB_CAMP_SUN);
                temp.add(web);
            }
        }
        List<PackageSubCampaign> packageSubCampaignList = packageSubCampaignRepository.findAllByAppAndUrl(appid, url);
        if (ValidateUtil.isValid(packageSubCampaignList)) {
            for (PackageSubCampaign ca : packageSubCampaignList) {
                Long cid = ca.getChannel();
                Long campaignId = ca.getParent();
                if (!cidList.contains(cid)) {
                    cidList.add(cid);
                }
                if (!campaignIdList.contains(campaignId)) {
                    campaignIdList.add(campaignId);
                }
                CampaignUrl4Web web = new CampaignUrl4Web();
                web.setCampaignId(campaignId);
                web.setSubCampaignId(ca.getId());
                web.setChannelId(cid);
                web.setSubCampaignName(ca.getName());
                web.setTabType(CampaignUrl4Web.TAB_CAMP_SUBPK);
                temp.add(web);
            }
        }

        Map<Long, String> channelMap = new HashMap<>();
        if (ValidateUtil.isValid(cidList)){
            List<Channel> channelList = channelRepository.findById(cidList);
            if (ValidateUtil.isValid(channelList)) {
                for (Channel ch : channelList) {
                    channelMap.put(ch.getId(), ch.getName());
                }
            }
        }

        Map<Long, String> campaignMap = new HashMap<>();
        if (ValidateUtil.isValid(campaignIdList)){
            List<Campaign> campaignList = campaignRepository.findAllById(campaignIdList);
            if (ValidateUtil.isValid(campaignList)) {
                for (Campaign ca : campaignList) {
                    campaignMap.put(ca.getId(), ca.getName());
                }
            }
        }

        if (ValidateUtil.isValid(temp)) {
            for (CampaignUrl4Web web : temp) {
                Long cid = web.getChannelId();
                Long campaignId = web.getCampaignId();
                if (channelMap.containsKey(cid)) {
                    web.setChannelName(channelMap.get(cid));
                }
                if (campaignMap.containsKey(campaignId)) {
                    web.setCampaignName(campaignMap.get(campaignId));
                }
                result.add(web);
            }
        }

        return result;
    }

    @Override
    public StringBuffer exportsurls2column(Iterable<Long> ids)
    {
        StringBuffer sb = new StringBuffer("关键词,创意,url\n");
        List<SubCampaign> subCampList = subCampaignRepository.findAll(Lists.newArrayList(ids));
        if (subCampList == null || subCampList.size() == 0) {
            return sb;
        }
        for (SubCampaign subCampaign : subCampList) {
            sb.append(subCampaign.getKeyword()).append(",").append(subCampaign.getCreative()).append(",").append(generateShortUrlForKeyword(subCampaign.getSurl(), subCampaign.getKeyword(),
                    subCampaign.getCreative(), subCampaign.getUrl())).append("\n");
        }

        return sb;
    }

    @Override
    public List<Object []> findSubCampaignNumberByIds(List<Long> ids) {

        return subCampaignRepository.countSubCampaignByCampaignByIds(ids);
    }
}
