package com.reyun.service.impl;

import com.reyun.model.Channel;
import com.reyun.model.SubChannelRename;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ChannelRepository;
import com.reyun.repository.SubChannelRenameRepository;
import com.reyun.service.SubChannelService;
import com.reyun.util.Constant;
import com.reyun.util.HttpClientUtil;
import com.reyun.util.StringUtil;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * Created by song on 2018/1/23.
 */

@Service
public class SubChannelServiceImpl implements SubChannelService{

    private static final int MAX_UPLOAD_ROW = 1000;

    @Autowired
    AppRepository appRepository;

    @Autowired
    ChannelRepository channelRepository;

    @Autowired
    SubChannelRenameRepository subChannelRenameRepository;

    @Override
    public List<Channel> listChannelIncludeSubchannel(Long appId) {

        ArrayList<Channel> channels = new ArrayList<>();
        String appkey = appRepository.findAppkeyById(appId);

        Map<String, String> conditions = new HashMap<>();
        conditions.put("appid", appkey);
        String url = Constant.reportUrl + "/api/trackingio/subchannel_rename_bycampaign" + "/" + 1218L;
        String responseJson = HttpClientUtil.doHttpPostRequest(url, "trackingio", conditions);

        try {
            JSONObject jsonObject = new JSONObject(responseJson);
            JSONArray cids = jsonObject.getJSONArray("key");

            for(int i=0;i<cids.length();i++){
                long cid = cids.getLong(i);
                Channel channel = channelRepository.findById(cid);
                if(channel!=null){
                    channels.add(channel);
                }
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return channels;
    }

    @Override
    public Integer subChannelTransfer(MultipartFile file, Long cid,Long appid){
        int changeNum=0;
        Channel channel = channelRepository.findById(cid);
        try {
            Workbook workbook = WorkbookFactory.create(file.getInputStream());
            Sheet sheet = workbook.getSheetAt(0);
            DataFormatter formatter = new DataFormatter();
            int rowNumber = sheet.getLastRowNum();
            rowNumber = rowNumber > MAX_UPLOAD_ROW ? MAX_UPLOAD_ROW : rowNumber;

            String subChannel=null;
            String subChannelChina=null;

            for(int i=1;i<=rowNumber;i++){
                Row row=sheet.getRow(i);
                if(row!=null){
                    Cell cell = row.getCell(0);
                    if (null != cell && !cell.getCellTypeEnum().equals(CellType.BLANK)) {
                        subChannel = formatter.formatCellValue(cell).trim();
                    }

                    Cell cell1 = row.getCell(1);
                    if (null != cell1 && !cell1.getCellTypeEnum().equals(CellType.BLANK)) {
                        subChannelChina = formatter.formatCellValue(cell1).trim();
                    }


                    if(!StringUtil.isEmpty(subChannel) && !StringUtil.isEmpty(subChannelChina)) {
                        SubChannelRename byCidAndSubChannelChName = subChannelRenameRepository.findByCidAndSubChannelChName(subChannelChina,cid,appid);
                        if ((byCidAndSubChannelChName!=null && byCidAndSubChannelChName.getSubChannel().equals(subChannel)) || byCidAndSubChannelChName == null) {
                            SubChannelRename byCidAndSubChannel = subChannelRenameRepository.findByCidAndSubChannel(subChannel,cid,appid);
                            if (byCidAndSubChannel != null) {
                                byCidAndSubChannel.setSubChannelName(subChannelChina);
                                subChannelRenameRepository.save(byCidAndSubChannel);
                            } else {
                                SubChannelRename subChannelRename = new SubChannelRename();
                                subChannelRename.setSubChannel(subChannel);
                                subChannelRename.setSubChannelName(subChannelChina);
                                subChannelRename.setCid(cid);
                                subChannelRename.setChannelName(channel.getName());
                                subChannelRename.setDelFlag(false);
                                subChannelRename.setAppid(appid);
                                subChannelRenameRepository.save(subChannelRename);
                            }
                            changeNum++;
                        }
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InvalidFormatException e) {
            e.printStackTrace();
        }
        return changeNum;
    }


    @Override
    public List<SubChannelRename> findSubchannelList(Long appId,Long cid,String search) {
        List<SubChannelRename> allSubChannel=null;

        ArrayList<SubChannelRename> result = new ArrayList<>();
        if(cid.equals(-1L)){
            allSubChannel=subChannelRenameRepository.findAllSubChannel(appId);
        }else{
            allSubChannel=subChannelRenameRepository.findByChannelId(cid,appId);
        }

        if(search!=null){
            for(SubChannelRename sub:allSubChannel){
                if(sub.getSubChannel().contains(search) || sub.getSubChannelName().contains(search) || sub.getChannelName().contains(search)){
                    result.add(sub);
                }
            }
            return result;
        }
        return allSubChannel;
    }

    @Override
    public SubChannelRename updateName(Long id,String name) {
        SubChannelRename subChannel = subChannelRenameRepository.findById(id);
        subChannel.setSubChannelName(name);
        SubChannelRename save = subChannelRenameRepository.save(subChannel);
        return save;
    }
}
