package com.reyun.service.impl;

import com.google.common.base.Function;
import com.google.common.base.Strings;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.SynchronizeCampaignTaskService;
import com.reyun.util.StringUtil;
import com.reyun.util.TkioStreamDBUtil;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Nullable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

@Service
public class SynchronizeCampaignTaskServiceImpl implements SynchronizeCampaignTaskService {
    private static Logger logger = LoggerFactory.getLogger(SynchronizeCampaignTaskServiceImpl.class);
    @Autowired
    private ChannnelAccountLinkAppRepository channnelAccountLinkAppRepository;

    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private PackageSubCampaignRepository packageSubCampaignRepository;

    private static String isCampaign = "unknown";

    private static ExecutorService exec = Executors.newFixedThreadPool(10);

    @Override
    public void syncThirdCammpaigns(String appkey) {
        List<TkioAppThirdChannelRel> trackAppThirdChannelRels = findNeedSync(appkey);

        List<List<TkioAppThirdChannelRel>> lists = splitList(trackAppThirdChannelRels, 500);

        final List<List<TkioAppThirdChannelRel>> lists1 = Collections.synchronizedList(lists);

        int i = 0;
        for (List<TkioAppThirdChannelRel> tkioAppThirdChannelRels : lists1) {
            logger.info("{}", ++i);
            exec.execute(() -> {
                try {
                    syncThirdCammpaigns(tkioAppThirdChannelRels);
                } catch (Exception e) {
                    logger.error("实时同步活动错误", e);
                }

            });
        }

    }

    public void syncThirdCammpaigns(List<TkioAppThirdChannelRel> trackAppThirdChannelRels) {
        long l = System.currentTimeMillis();
        if (trackAppThirdChannelRels == null || trackAppThirdChannelRels.size() == 0) {
            logger.info("同步实时活动,暂无同步数据");
            return;
        }

        if (trackAppThirdChannelRels.size() > 1000) {
            logger.info("同步实时活动,共有{}条三方活动", trackAppThirdChannelRels.size());
        }

        //2. 数据字典查询
        //2.1 应用字典
        List<String> transAppkeys = Lists.transform(trackAppThirdChannelRels, new Function<TkioAppThirdChannelRel, String>() {
            @Override
            public String apply(@Nullable TkioAppThirdChannelRel tkioAppThirdChannelRel) {
                return tkioAppThirdChannelRel.getAppid();
            }
        });
        List<App> apps = this.appRepository.findByAppkeys(transAppkeys);
        final Map<String, App> dicApp = Maps.uniqueIndex(apps, new Function<App, String>() {
            @Override
            public String apply(@Nullable App app) {
                return app.getAppkey();
            }
        });

        //2.2 推广活动字典
        List<String> transCampaignIds = Lists.transform(trackAppThirdChannelRels, new Function<TkioAppThirdChannelRel, String>() {
            @Override
            public String apply(@Nullable TkioAppThirdChannelRel tkioAppThirdChannelRel) {
                return tkioAppThirdChannelRel.getCampaignid();
            }
        });
        //推广活动
        List<Campaign> campaigns = this.campaignRepository.findByCampaignidNotApp(transCampaignIds);
        final Map<String, Campaign> dicCampaign = Maps.uniqueIndex(campaigns, new Function<Campaign, String>() {
            @Override
            public String apply(@Nullable Campaign campaign) {
                return campaign.getCampaignid() + "_" + campaign.getApp();
            }
        });
        //2.2 推广父活动活动字典
        List<String> transParentCampaignIds = Lists.transform(trackAppThirdChannelRels, new Function<TkioAppThirdChannelRel, String>() {
            @Override
            public String apply(@Nullable TkioAppThirdChannelRel tkioAppThirdChannelRel) {
                return tkioAppThirdChannelRel.getParentCampaignid();
            }
        });
        //推广父活动
        List<Campaign> parnetCampaigns = this.campaignRepository.findByCampaignidNotApp(transParentCampaignIds);
        final Map<String, Campaign> dicParentCampaign = Maps.uniqueIndex(parnetCampaigns, new Function<Campaign, String>() {
            @Override
            public String apply(@Nullable Campaign campaign) {
                return campaign.getCampaignid() + "_" + campaign.getApp();
            }
        });
        //子活动
        List<PackageSubCampaign> subCampaigns = packageSubCampaignRepository.listSubCampaignByCampaignid(transCampaignIds);
        final Map<String, PackageSubCampaign> dicSubCampaign = Maps.uniqueIndex(subCampaigns, new Function<PackageSubCampaign, String>() {
            @Override
            public String apply(@Nullable PackageSubCampaign campaign) {
//                return campaign.getCampaignid() + "_" + campaign.getApp();
                return campaign.getCampaignid() + "_" + campaign.getApp() + "_" + campaign.getParentCampaignid();
            }
        });

        List<ChannelAccountLinkApp> channelAccountLinkApps = this.channnelAccountLinkAppRepository.findByAppkeyIn(transAppkeys);

       /* Map<String, ChannelAccountLinkApp> dicChannelAccoutLinkApp = Maps.uniqueIndex(channelAccountLinkApps, new Function<ChannelAccountLinkApp, String>() {
            @Override
            public String apply(@Nullable ChannelAccountLinkApp channelAccountLinkApp) {
                if (Strings.isNullOrEmpty(channelAccountLinkApp.getAccountId())) {
                    //adwords
                    return channelAccountLinkApp.getAppkey() + "-adwords-" + channelAccountLinkApp.getUserActionSetId();
                } else {
                    //gdt
                    return channelAccountLinkApp.getAppkey() + "-gdt-" + channelAccountLinkApp.getAccountId();
                }
            }
        });*/

        Map<String, ChannelAccountLinkApp> dicChannelAccoutLinkApp = channelAccountLinkApps.stream().collect(Collectors.toMap(v -> {
            if (Strings.isNullOrEmpty(v.getAccountId())) {
                //adwords
                return v.getAppkey() + "-adwords-" + v.getUserActionSetId();
            } else {
                //gdt
                return v.getAppkey() + "-gdt-" + v.getAccountId();
            }
        }, v -> v, (v1, v2) -> v1));


        //3. 数据更新(增量更新或创建推广活动)
        List<Channel> channels = this.channelRepository.findByUniqueNames(ImmutableList.of("gdt", "adwords"));
        Map<String, Channel> dicChannel = Maps.uniqueIndex(channels, Channel::getUniqueName);
        List<Campaign> uptList = Lists.newArrayList();
        List<PackageSubCampaign> uptSubList = Lists.newArrayList();
        for (TkioAppThirdChannelRel trackAppThirdChannelRel : trackAppThirdChannelRels) {
            if (!dicApp.containsKey(trackAppThirdChannelRel.getAppid())) {
                logger.warn("同步实时活动错误,应用({})不存在!", trackAppThirdChannelRel.getAppid());
                continue;
            }

            App app = dicApp.get(trackAppThirdChannelRel.getAppid());
            String prefix = "";
            if (app != null) {
                prefix = "_" + app.getId();
            }


            //同步数据为推广活动的数据处理
            if (trackAppThirdChannelRel.getParentCampaignid().equals(isCampaign)) {
                Campaign campaign = dicCampaign.get(trackAppThirdChannelRel.getCampaignid() + prefix);
                if (null == campaign) {
                    Channel channel = dicChannel.get(trackAppThirdChannelRel.getUniqueName());
                    if (channel == null) {
                        logger.warn("同步实时活动错误,渠道不存在, {}", trackAppThirdChannelRel.getUniqueName());
                        continue;
                    }

                    String chkey = spicalKey(trackAppThirdChannelRel.getAppid(), trackAppThirdChannelRel.getUniqueName(), trackAppThirdChannelRel.getAccountId());
                    final ChannelAccountLinkApp channelAccountLinkApp = dicChannelAccoutLinkApp.get(chkey);
                    if (channelAccountLinkApp == null) {
                        logger.warn("同步实时活动错误,渠道帐户配置(应用:{}({}), specialId(accountid/usersectionid):{})不存在!", app.getName(), trackAppThirdChannelRel.getAppid(), trackAppThirdChannelRel.getAccountId());
                        continue;
                    }

                    campaign = new Campaign();
                    campaign.setName(trackAppThirdChannelRel.getCampaignName());
                    campaign.setCampaignid(trackAppThirdChannelRel.getCampaignid());
                    campaign.setThirdAccount(trackAppThirdChannelRel.getAdvertiserId());
                    campaign.setAwssuccess(true);
                    campaign.setOs(app.getPlatform());
                    campaign.setApp(app.getId());
                    campaign.setChannel(channel.getId());
                    campaign.setAccount(channelAccountLinkApp.getCreateUser() == null ? -1L : channelAccountLinkApp.getCreateUser());
                    campaign.setSurl("");
                    campaign.setChannelCategory("ADVERTISING");
                    campaign.setChannelType(channel.getType());
                } else if (!campaign.getName().equals(trackAppThirdChannelRel.getCampaignName())) {
                    campaign.setName(trackAppThirdChannelRel.getCampaignName());
                    campaign.setModifyTime(new Date());
                }
                uptList.add(campaign);
                trackAppThirdChannelRel.setLastupdateTime(new Date());
            } else {//同步数据为子活动的数据处理

                PackageSubCampaign subCampaign = dicSubCampaign.get(trackAppThirdChannelRel.getCampaignid() + prefix + "_" + trackAppThirdChannelRel.getParentCampaignid());
                if (null == subCampaign) {
                    Channel channel = dicChannel.get(trackAppThirdChannelRel.getUniqueName());
                    if (channel == null) {
                        logger.warn("同步实时活动错误,渠道不存在, {}", trackAppThirdChannelRel.getUniqueName());
                        continue;
                    }

                    String chkey = spicalKey(trackAppThirdChannelRel.getAppid(), trackAppThirdChannelRel.getUniqueName(), trackAppThirdChannelRel.getAccountId());
                    final ChannelAccountLinkApp channelAccountLinkApp = dicChannelAccoutLinkApp.get(chkey);
                    if (channelAccountLinkApp == null) {
                        logger.warn("同步实时活动错误,渠道帐户配置(应用:{}({}), specialId(accountid/usersectionid):{})不存在!", app.getName(), trackAppThirdChannelRel.getAppid(), trackAppThirdChannelRel.getAccountId());
                        continue;
                    }
                    //2020-05-25 禅道1053 由于广点通分包子活动Campaignid生成规则变更,现在需要把旧分包子活动表中的Campaignid修改为新生成的Campaignid
                    String oldGdtCampaignid = getOldGdtCampaignid(trackAppThirdChannelRel.getCampaignid());
                    if (!StringUtil.isEmpty(oldGdtCampaignid)) {
                        //continue;
                        subCampaign = packageSubCampaignRepository.findSubCampaignBycampaignid(oldGdtCampaignid);
                        if (subCampaign != null) {
                            /*subCampaign.setName(trackAppThirdChannelRel.getCampaignName());
                            subCampaign.setModifyTime(new Date());
                            subCampaign.setCampaignid(trackAppThirdChannelRel.getCampaignid());
                            uptSubList.add(subCampaign);*/
                            continue;
                        }
                    }
                    subCampaign = new PackageSubCampaign();
                    subCampaign.setName(trackAppThirdChannelRel.getCampaignName());
                    subCampaign.setCampaignid(trackAppThirdChannelRel.getCampaignid());
                    subCampaign.setParentCampaignid(trackAppThirdChannelRel.getParentCampaignid());
                    subCampaign.setAwssuccess(true);
                    if (null != dicParentCampaign.get(trackAppThirdChannelRel.getParentCampaignid())) {
                        subCampaign.setParent(dicParentCampaign.get(trackAppThirdChannelRel.getParentCampaignid()).getId());
                    }
                    subCampaign.setCreateTime(trackAppThirdChannelRel.getCreateTime());
                    subCampaign.setApp(app.getId());
                    subCampaign.setChannel(channel.getId());
                    subCampaign.setAccount(channelAccountLinkApp.getCreateUser() == null ? -1L : channelAccountLinkApp.getCreateUser());
                    subCampaign.setSurl("");
                    //campaign.setChannelCategory("ADVERTISING");
                    //campaign.setChannelType(channel.getType());
                } else if (!subCampaign.getName().equals(trackAppThirdChannelRel.getCampaignName())) {
                    subCampaign.setName(trackAppThirdChannelRel.getCampaignName());
                    subCampaign.setModifyTime(new Date());
                }
                uptSubList.add(subCampaign);
                trackAppThirdChannelRel.setLastupdateTime(new Date());
            }

        }

        Map<String, Campaign> upMap = new HashMap<>();
        for (Campaign camp : uptList) {
            upMap.put(camp.getCampaignid(), camp);
        }

        Map<String, PackageSubCampaign> upSubMap = new HashMap<>();
        for (PackageSubCampaign camp : uptSubList) {
            upSubMap.put(camp.getCampaignid() + "_" + camp.getParentCampaignid(), camp);
        }

        boolean flag = false;
        if (uptList.size() > 0) {
            Iterator<Campaign> it = upMap.values().iterator();
            while (it.hasNext()) {
                Campaign cam = it.next();
                try {
                    Campaign save = campaignRepository.save(cam);
                } catch (Exception e) {
                    logger.info("同步实时活动错误{} {}", cam.getName(), cam.getCampaignid());
                    logger.error("", e);
                }
            }
            flag = true;
        }
        if (uptSubList.size() > 0) {
            Iterator<PackageSubCampaign> it = upSubMap.values().iterator();
            while (it.hasNext()) {
                PackageSubCampaign psc = it.next();
                try {
                    packageSubCampaignRepository.save(psc);
                } catch (DataIntegrityViolationException e) {
                    logger.info("同步实时活动错误 {} {}", psc.getId(), psc.getCampaignid());
                    logger.error("同步实时活动错误", e);
                }
            }
            flag = true;
        }
        if (flag) {
            logger.info("本次同步活动条数:{},用时:{}s", trackAppThirdChannelRels.size(), (System.currentTimeMillis() - l) / 1000);
            updateLastUpdateTime(trackAppThirdChannelRels);
        }
    }

    private String spicalKey(String appkey, String uniqueName, String othId) {
        return appkey + "-" + uniqueName + "-" + othId;
    }

    public void updateLastUpdateTime(List<TkioAppThirdChannelRel> tkioAppThirdChannelRels) {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        try {
            String sql = "update tkio_app_third_channel_rel set lastupdate_time = '%s'  where appid = '%s' and campaignid = '%s' and ds = '%s' and parent_campaignid = '%s'";
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            for (TkioAppThirdChannelRel tkioAppThirdChannelRel : tkioAppThirdChannelRels) {
                try {
                    pt.addBatch(String.format(sql
                            , DateTime.now().toString("yyyy-MM-dd HH:mm:ss")
                            , tkioAppThirdChannelRel.getAppid()
                            , tkioAppThirdChannelRel.getCampaignid()
                            , tkioAppThirdChannelRel.getDs()
                            , tkioAppThirdChannelRel.getParentCampaignid()));
                } catch (SQLException e) {
                    logger.error(e.getMessage(), e);
                }
            }
            pt.executeBatch();
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }
    }

    public List<TkioAppThirdChannelRel> findNeedSync(String appkey) {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        List<TkioAppThirdChannelRel> result = new ArrayList<>();
        try {
//           String sql = "select * from tkio_app_third_channel_rel where parent_campaignid ='13c7a172800e55b16fcd7352734e2d5a'";
            String sql;
            if ("-1".equals(appkey)) {
                sql = "select * from tkio_app_third_channel_rel where update_time >= (select  max(lastupdate_time) from tkio_app_third_channel_rel) or lastupdate_time is null";
            } else {
                sql = "select * from tkio_app_third_channel_rel where (update_time >= (select  max(lastupdate_time) from tkio_app_third_channel_rel) or lastupdate_time is null) and appid='" + appkey + "'";
            }
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                TkioAppThirdChannelRel tkioAppThirdChannelRel = new TkioAppThirdChannelRel();
                tkioAppThirdChannelRel.setAppid(rs.getString("appid"));
                tkioAppThirdChannelRel.setCampaignid(rs.getString("campaignid"));
                tkioAppThirdChannelRel.setCampaignName(rs.getString("campaign_name"));
                tkioAppThirdChannelRel.setAccountId(rs.getString("account_id"));
                tkioAppThirdChannelRel.setDs(rs.getString("ds"));
                tkioAppThirdChannelRel.setCreateTime(rs.getDate("create_time"));
                tkioAppThirdChannelRel.setUpdateTime(rs.getDate("update_time"));
                tkioAppThirdChannelRel.setLastupdateTime(rs.getDate("lastupdate_time"));
                tkioAppThirdChannelRel.setUniqueName(rs.getString("unique_name"));
                tkioAppThirdChannelRel.setAdvertiserId(rs.getString("advertiser_id"));
                tkioAppThirdChannelRel.setParentCampaignid(rs.getString("parent_campaignid"));
                result.add(tkioAppThirdChannelRel);
            }
        } catch (SQLException e) {
            logger.error("同步实时活动错误", e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }

        return result;
    }

    @Override
    public void synchronizeOldGdtPkgSubCampaign() {
        Map<String, Object> oldGdtCampaignid = findOldGdtCampaignid();
        List<String> oldCampaignids = (List<String>) oldGdtCampaignid.get("oldCampaignids");
        Map<String, String> campaignidMap = (Map<String, String>) oldGdtCampaignid.get("campaignidMap");
        if (!CollectionUtils.isEmpty(oldCampaignids)) {
            List<PackageSubCampaign> packageSubCampaigns = packageSubCampaignRepository.listSubCampaignByCampaignid(oldCampaignids);
            for (PackageSubCampaign packageSubCampaign : packageSubCampaigns) {
                packageSubCampaign.setCampaignid(campaignidMap.get(packageSubCampaign.getCampaignid()));
                packageSubCampaign.setModifyTime(new Date());
                PackageSubCampaign save = packageSubCampaignRepository.save(packageSubCampaign);
                updateStatus(save.getCampaignid());
            }
        }
    }

    public String getOldGdtCampaignid(String campaignid_new) {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        String campaignid_old = "";
        try {
            String sql = "select * from `tkio_stream`.`tkio_campaignid_gdt_setting_day` where campaignid_new = '" + campaignid_new + "' and isnew is false";
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                campaignid_old = rs.getString("campaignid");
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }

        return campaignid_old;
    }

    public Map<String, Object> findOldGdtCampaignid() {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        List<String> oldCampaignids = new ArrayList<>();
        Map<String, String> campaignidMap = new HashMap<>();
        Map<String, Object> allMap = new HashMap<>(2);
        try {
            String sql = "select * from `tkio_stream`.`tkio_campaignid_gdt_setting_day` where is_update is false and isnew is false";
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                campaignidMap.put(rs.getString("campaignid"), rs.getString("campaignid_new"));
                oldCampaignids.add(rs.getString("campaignid"));
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }
        allMap.put("oldCampaignids", oldCampaignids);
        allMap.put("campaignidMap", campaignidMap);

        return allMap;
    }

    public void updateStatus(String campaignid_new) {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        try {
            String sql = "update `tkio_stream`.`tkio_campaignid_gdt_setting_day` set is_update = 1 where campaignid_new = '" + campaignid_new + "'";
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            pt.execute();
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }
    }

    private void unSycCampionsCheck(String sqlcount, String tablename) {
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;

        List<String> campids = new ArrayList<>();
        try {
//            String sql = "select  campaignid from ( select count(*) a ,campaignid from tkio_app_third_channel_rel group by campaignid ) c";
            //获取所有 可同步数据
            conn = TkioStreamDBUtil.newInstance().getConn();
            pt = conn.prepareStatement(sqlcount);
            rs = pt.executeQuery();
            while (rs.next()) {
                campids.add(rs.getString("campaignid"));
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }

        logger.info("共有{}条三方活动", campids.size());
        System.out.println("共有" + campids.size() + " 条三方活动");

        Connection conn_track = null;
        PreparedStatement pt_track = null;
        ResultSet rs_track = null;
        StringBuffer ids = null;
        StringBuffer unsycids = null;
        conn_track = TkioStreamDBUtil.newInstanceTrack().getConn();
        int groupsize = 0;
        int unsyc_count = 0;

        for (String capid : campids) {

            if (ids == null) {
                ids = new StringBuffer();
                ids.append("'" + capid + "'");
            } else {
                ids.append(",'" + capid + "'");
            }

            groupsize++;


            if (groupsize >= 500) {


                String cpaidstr = ids.toString();
                String sqlcounttkio = "select count(*) coucap from " + tablename + " where campaignid in (" + cpaidstr + ")";

                try {
                    pt_track = conn_track.prepareStatement(sqlcounttkio);
                    rs_track = pt_track.executeQuery();
                    int rows = 0;
                    while (rs_track.next()) {
                        rows = rs_track.getShort("coucap");
                    }
                    if (rows != groupsize) {

                        // tkio库和实时库条数不一致时 开始对比

                        logger.info("开始比对.....");

                        String sqlcapids = "select campaignid  from " + tablename + " where campaignid in (" + cpaidstr + ")";
                        Map<String, String> nowIds = new HashMap<>();
                        pt_track = conn_track.prepareStatement(sqlcapids);
                        rs_track = pt_track.executeQuery();

                        while (rs_track.next()) {
                            nowIds.put("'" + rs_track.getString("campaignid") + "'", "ex");
                        }

                        String[] idsarry = cpaidstr.split(",");


                        for (String idsa : idsarry) {
                            String id = nowIds.get(idsa);
                            if (id == null) {
                                // 改活动在 tkio库中不存在 并记录
                                if (unsycids == null) {
                                    unsycids = new StringBuffer();
                                    unsycids.append(idsa);
                                } else {
                                    unsycids.append("," + idsa);
                                }

                                unsyc_count++;
                            }
                        }


                        logger.info("未同步数{}", unsyc_count);

                        System.out.println("当前未同步活动数：" + unsyc_count);

                    }

                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {

                }

                groupsize = 0;
                ids = null;

            }
        }

        TkioStreamDBUtil.newInstance().release(rs_track, pt_track, conn_track);

        if (unsycids == null) {
            unsycids = new StringBuffer();
        }

        logger.info(unsycids.toString());
        System.out.println("未同步的活动: " + unsycids.toString());

    }

    public static void main(String[] args) {

        SynchronizeCampaignTaskServiceImpl taskService = new SynchronizeCampaignTaskServiceImpl();

//        taskService.unSycCampionsCheck();
        //活动
        String sql1 = "select campaignid from tkio_app_third_channel_rel  where parent_campaignid ='unknown'  group by campaignid";

        taskService.unSycCampionsCheck(sql1, "campaign");

        //子活动
        String sql2 = "select campaignid from tkio_app_third_channel_rel  where parent_campaignid !='unknown'  group by campaignid";

        taskService.unSycCampionsCheck(sql2, "package_sub_campaign");
    }

    public static <T> List<List<T>> splitList(List<T> list, int pageSize) {

        int listSize = list.size();                                                           //list的大小
        int page = (listSize + (pageSize - 1)) / pageSize;                      //页数

        List<List<T>> listArray = new ArrayList<List<T>>();              //创建list数组 ,用来保存分割后的list
        for (int i = 0; i < page; i++) {                                                         //按照数组大小遍历
            List<T> subList = new ArrayList<T>();                               //数组每一位放入一个分割后的list
            for (int j = 0; j < listSize; j++) {                                                 //遍历待分割的list
                int pageIndex = ((j + 1) + (pageSize - 1)) / pageSize;   //当前记录的页码(第几页)
                if (pageIndex == (i + 1)) {                                               //当前记录的页码等于要放入的页码时
                    subList.add(list.get(j));                                               //放入list中的元素到分割后的list(subList)
                }

                if ((j + 1) == ((j + 1) * pageSize)) {                               //当放满一页时退出当前循环
                    break;
                }
            }
            listArray.add(subList);                                                         //将分割后的list放入对应的数组的位中
        }
        return listArray;
    }
}
