package com.reyun.service.impl;

import java.util.*;

import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.Account;
import com.reyun.model.LogoDic;
import com.reyun.repository.AppRepository;
import com.reyun.repository.LogoDicRepository;
import com.reyun.repository.SdkEventRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.reyun.model.Tip;
import com.reyun.repository.TipRepository;
import com.reyun.service.TipService;

@Service
public class TipServiceImpl implements TipService {

    @Autowired
    TipRepository tipRepository;
    @Autowired
    SdkEventRepository sdkEventRepository;
    @Autowired
    AppRepository appRepository;

    @Autowired
    LogoDicRepository logoDicRepository;

    /**
     * @param menu
     * @param appid
     * @param lanType 语种类型（zh 中文 en 英文）
     * @return
     */
    @Override
    public Map<String, Object> findAllByMenu(String menu, Long appid, String lanType) {
        List<Tip> tips = tipRepository.findAllByMenu(menu);
        List<String> eventName = sdkEventRepository.findAllNameByAppid(appid);
        String platform = appRepository.findPlatformByAppId(appid);
        Map<String, Object> result = new HashMap<>();
        List<Tip> tipext = new ArrayList();
        for (Tip tip : tips) {
            if (eventName.contains(tip.getKey())) {
                String eventAlias = sdkEventRepository.findByEventNameAndAppid(tip.getKey(), appid).getEventAlias();

                if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                    String newTip = tip.getEngTip().replaceAll(tip.getKey(), sdkEventRepository.findByEventNameAndAppid(tip.getKey(), appid).getEventAlias());
                    tip.setTip(newTip);
                    result.put(eventAlias, tip.getTip());
                } else {
                    String newTip = tip.getTip().replaceAll(tip.getKey(), sdkEventRepository.findByEventNameAndAppid(tip.getKey(), appid).getEventAlias());
                    tip.setTip(newTip);
                    result.put(eventAlias + "数", tip.getTip());
                }
            }

            //tip 语种处理
            if (!eventName.contains(tip.getKey())) {
                tip = tip.toTransEng(tip, lanType);
            }
            //end

            if (tip != null) {
                //if (tip.getPlatFormExt() != null && platform != null && tip.getKey().indexOf(platform) > 0) {
                if (tip.getPlatFormExt() != null && tip.getPlatFormExt().equals(platform)) {
                    tipext.add(tip);
                } else {
                    result.put(tip.getKey(), tip.getTip());
                }

                if (tip.getPageTitle() != null && !tip.getPageTitle().equals("0")) {

                    // 处理服务申请 页面描述 和 页面标题
                    Map data = new HashMap();
                    data.put("title", tip.getPageTitle());
                    data.put("desc", tip.getTip());
                    data.put("help_page", tip.getHelpPage());
                    result.put(tip.getKey(), data);

                }
            }

        }

        if (tipext.size() > 0) {
            for (Tip tip : tipext) {
                // 特殊处理 覆盖原tip
                result.put(tip.getKey(), tip.getTip());
            }
        }


        return result;
    }


    @Override
    public Map<String, String> findLogoDicData(String domain) {
        List<LogoDic> logoDics = logoDicRepository.findLogoDicData(domain);
        Map<String, String> logoMap = new HashMap<>();

        for (LogoDic dic : logoDics) {
            logoMap.put(dic.getDicKey(), dic.getDicContent());
        }

        return logoMap;
    }

    @Override
    public Map<String, String> getLogoData(Account account, String... key) {

        List<String> keys = Arrays.asList(key);

        String source = "0";

        if (account != null) {
            source = account.getRegSource() + "";
        }

        List<LogoDic> logoDics = logoDicRepository.getEailSubjectOnLogo(source + "", keys);

        Map<String, String> datas = new HashMap<>();

        for (LogoDic dic : logoDics) {
            datas.put(dic.getDicKey(), dic.getDicContent());
        }

        return datas;
    }
}
