package com.reyun.service.impl;

import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.reyun.dic.AppPlatformEnum;
import com.reyun.dic.LanguageTypeEnum;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AuthService;
import com.reyun.service.IViewColumnService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Collections;

/**
 * Created by nolan on 28/12/2016.
 * description:
 */
@Service
public class ViewColumnServiceImpl implements IViewColumnService {
    @Autowired
    private ViewColumnRepository viewColumnRepository;
    @Autowired
    private UserViewColumnRepository userViewColumnRepository;
    @Autowired
    private ConfigParamRepository configParamRepository;

    @Autowired
    private SdkEventRepository sdkEventRepository;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private AuthService authService;
    private final static String COMMON_COLUMNS = "common";

    private final static String SDK_EVENT_CONFIG = "sdkevent";

    /**
     * 根据账号APP和菜单查显示指标
     *
     * @param account 账号ID
     * @param appid     APP
     * @param menu      菜单名称
     * @param lanType   语言类型：zh(中文)/en（英文）)
     * @return
     */
    public List<ViewColumn> findUserViewColumns(Account account, Long appid, String menu, String lanType) {
        Long accountid = account.getId();

        App app = appRepository.findOne(appid);
        boolean flag = true;
        List<ViewColumn> commonList =null;
        if(account.getRoleCategory()>5){
            authService.checkRoleTempCategory(account);
        }
            if(account.getRoleCategory().longValue()==RoleEnumType.CUSTOM_AUTH.getKey().longValue()||account.getRoleCategory().longValue()==RoleEnumType.CUSTOM_ROLE.getKey().longValue()){
                Auth auth = authService.findAuth(account.getId(), appid);
                if(auth.getArouseAuth()==null||!auth.getArouseAuth()){
                    flag=false;
                }
            }else if(account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())){
                flag=false;
            }
            //调起指标权限控制
            if(flag){
                //通用指标和行业指标
                commonList = viewColumnRepository.findByIndustory(COMMON_COLUMNS, menu);
            }else{
                //通用指标和行业指标(去除调起基础调起指标字段)
                commonList = viewColumnRepository.findByIndustoryAndNotFieldCode(COMMON_COLUMNS, menu,"wake_num");
            }



        List<ViewColumn> industryList = viewColumnRepository.findByIndustory(app.getIndustory(), menu);

        industryList.addAll(commonList);

        App one = appRepository.findOne(appid);
        if("wx".equals(one.getPlatform()) || AppPlatformEnum.WG.getKey().equals(one.getPlatform())){
            commonList = viewColumnRepository.findByIndustory(COMMON_COLUMNS, menu);
            List<ViewColumn> sdk = viewColumnRepository.findByType(SDK_EVENT_CONFIG);
            industryList = viewColumnRepository.findByIndustory(app.getIndustory(), menu);
            industryList.addAll(commonList);
            industryList.addAll(sdk);
        }
        //用户指标
        Boolean firstColum = true;
        UserViewColumn userViewColumn = userViewColumnRepository.findByAccountMenu(accountid, appid, menu);
        if (null == userViewColumn) {
            firstColum = false;
            userViewColumn = userViewColumnRepository.findByAccountOne(accountid, menu);
        }

        String sdkevent = configParamRepository.findParamsByKey(SDK_EVENT_CONFIG);
        List<String> allNameByAppid = sdkEventRepository.findAllNameByAppid(appid);

        List<String> fieldCodeList = null;

        if (userViewColumn != null) {
            Iterable<String> viewcolumnList = Splitter.on(",").omitEmptyStrings().trimResults().split(userViewColumn.getViewcolumn());
            fieldCodeList = Lists.newArrayList(viewcolumnList);
        }

        for (ViewColumn viewColumn : industryList) {
            //处理推广活动详情的app创建后默认选中的指标(且用户非首次进入推广活动详情保存过指标)，可编辑且默认选中的指标先置为false(主要为了排除必选的指标)
            if(firstColum&&viewColumn.getMenu().equals("campaign")&&viewColumn.getIscheck()==true&&viewColumn.getIsedit()==true){
                viewColumn.setIscheck(false);
            }

            //处理SDK events
            if (sdkevent.contains(viewColumn.getFieldcode())) {
                SdkEvent byEventNameAndAppid = sdkEventRepository.findByEventNameAndAppid(viewColumn.getFieldcode(), appid);
                String fileCode = viewColumn.getFieldcode();

                //处理语种转化问题
                if (LanguageTypeEnum.ENGLISH.getCode().equals(lanType)) {
                    viewColumn.setFieldname(byEventNameAndAppid != null ? byEventNameAndAppid.getEventAlias() : fileCode.substring(7, fileCode.length()));
                } else {
                    if(fileCode.startsWith("dupnum_event")){
                        viewColumn.setFieldname(byEventNameAndAppid != null ? byEventNameAndAppid.getEventAlias() + "设备数" : fileCode.substring(7, fileCode.length()) + "设备数");
                    }else if(fileCode.startsWith("num_event")){
                         byEventNameAndAppid = sdkEventRepository.findByEventNameAndAppid("dup"+viewColumn.getFieldcode(), appid);
                        viewColumn.setFieldname(byEventNameAndAppid != null ? byEventNameAndAppid.getEventAlias() + "总数" : fileCode.substring(4, fileCode.length()) + "总数");
                    }
                }
                // end
            }

            if (fieldCodeList != null && fieldCodeList.contains(viewColumn.getFieldcode())) {
                viewColumn.setIscheck(true);
            }
        }


        ArrayList<ViewColumn> newItemlist = new ArrayList<>();
        //临时排序集合
        ArrayList<ViewColumn> flagItemlist = new ArrayList<>();
        ArrayList<ViewColumn> rateItemlist = new ArrayList<>();
        for (ViewColumn viewColumn : industryList) {
            if (!sdkevent.contains(viewColumn.getFieldcode()) || allNameByAppid.contains(viewColumn.getFieldcode())||allNameByAppid.contains("dup"+viewColumn.getFieldcode())) {
                if(viewColumn.getFieldname().startsWith("LTV")||viewColumn.getFieldname().startsWith("回流LTV")){

                    flagItemlist.add(viewColumn);
                    continue;
                }else if(viewColumn.getFieldname().endsWith("留存")){

                    rateItemlist.add(viewColumn);
                    continue;
                }
                //要求去掉设备作弊激活数  create by zhangyao 2018-11-30
                if(viewColumn.getId().equals(71L)){
                    continue;
                }
                //lat指标仅限ios
                if("lat".equalsIgnoreCase(viewColumn.getFieldcode())&&!"ios".equalsIgnoreCase(app.getPlatform())){
                    continue;
                }
                //处理语种转化问题
                viewColumn = viewColumn.toTransEng(viewColumn, lanType);
                // end

                newItemlist.add(viewColumn);
            }
        }
        //展示到页面的数据过于混乱,进行排序         2018-10-30  by zy
        sortIntMethod(flagItemlist);
        sortIntRateMethod(rateItemlist);

        for(ViewColumn viewColumn:flagItemlist){
            if(LanguageTypeEnum.ENGLISH.getCode().equals(lanType)){
                viewColumn.setFieldname(viewColumn.getEngFieldname());
            }
        }

        for(ViewColumn viewColumn:rateItemlist){
            if(LanguageTypeEnum.ENGLISH.getCode().equals(lanType)){
                viewColumn.setFieldname(viewColumn.getEngFieldname());
            }
        }
        newItemlist.addAll(flagItemlist);
        newItemlist.addAll(rateItemlist);

        return newItemlist;
    }

    /**
     *       2018-10-30 by zy
     *       排序
     */
    public static void sortIntMethod(List list) {
        Collections.sort(list, new Comparator() {
            @Override
            public int compare(Object o1, Object o2) {
                ViewColumn vc1 = (ViewColumn) o1;
                ViewColumn vc2 = (ViewColumn) o2;
                String fix = "LTV";
                if(vc1.getFieldname().startsWith("回流")){
                    fix = "回流"+fix;
                }
                int n1 = Integer.parseInt(vc1.getFieldname().replace(fix, ""));
                int n2 = Integer.parseInt(vc2.getFieldname().replace(fix, ""));
                if (n1 > n2) {
                    return 1;
                } else if (n1 == n2) {
                    return 0;
                } else {
                    return -1;
                }
            }
        });

    }
    public static void sortIntRateMethod(List list) {
        Collections.sort(list, new Comparator() {
            @Override
            public int compare(Object o1, Object o2) {
                ViewColumn vc1 = (ViewColumn) o1;
                ViewColumn vc2 = (ViewColumn) o2;
                String fix = "D";
                if(vc1.getFieldname().startsWith("回流")){
                    fix = "回流"+fix;
                }
                int n1 = Integer.parseInt(vc1.getFieldname().replace(fix, "").replace("留存",""));
                int n2 = Integer.parseInt(vc2.getFieldname().replace(fix, "").replace("留存",""));
                if (n1 > n2) {
                    return 1;
                } else if (n1 == n2) {
                    return 0;
                } else {
                    return -1;
                }
            }
        });

    }


        public void saveUserViewColumn(Long accountid, Long appid, String viewcolumn, String menu) {

        UserViewColumn userViewColumn = this.userViewColumnRepository.findByAccountMenu(accountid, appid, menu);

        if (userViewColumn == null) {
            userViewColumn = new UserViewColumn();
            userViewColumn.setAccount(accountid);
            userViewColumn.setApp(appid);
            userViewColumn.setViewcolumn(viewcolumn);
            userViewColumn.setMenu(menu);
            this.userViewColumnRepository.saveAndFlush(userViewColumn);
        } else {
            userViewColumn.setViewcolumn(viewcolumn);
            this.userViewColumnRepository.saveAndFlush(userViewColumn);
        }
    }
}
