package com.reyun.task;

import com.alibaba.fastjson.JSONObject;
import com.reyun.dic.RoleEnumType;
import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.service.AdsAccountService;
import com.reyun.util.*;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author: zhangyao
 * @create: 2019-07-08 15:33
 **/
@Service
public class Adsdesk {

    private static Logger logger = LoggerFactory.getLogger(Adsdesk.class);
    private static int EXPIRE_PERIOD = 500;
    private static final String ADS_S3_PATH = "campaign/";

    @Autowired
    private AdsAccountRepository adsAccountRepository;

    @Autowired
    private AccountRepository accountRepository;
    @Autowired
    private AppRepository appRepository;
    @Autowired
    private CampaignRepository campaignRepository;
    @Autowired
    private AuthRepository authRepository;
    @Autowired
    private DataAuthRepository dataAuthRepository;


    @Autowired
    private AdsAccountService adsAccountService;

    public void run() {

        if (RedisUtilNew.getInstance().tryLock("Adsdesk", EXPIRE_PERIOD) != null) {

            logger.info("Adsdesk................//开始, 执行时间: {}", DateTime.now());
            if (Constant.awsneed) {
                List<AdsAccount> all = adsAccountRepository.findAll();
                for (AdsAccount adsAccount : all) {
                    Account account = accountRepository.findOne(adsAccount.getAccount());
                    if (account != null) {
                        //0:正常；1:账户停用 ， 2:密码更新
                        if (adsAccount.getPermission() == 2 || adsAccount.getPermission() == 1) {
                            //密码更改
                            this.parse(account.getEmail(), adsAccount.getPermission(), new ArrayList<>(0));
                            continue;
                        } else if (!"true".equals(this.adsAccountService.checkAccount(account))) {
                            //流量用完或者账号停用等
                            this.adsAccountRepository.updatePermissionById(adsAccount.getId(), 1);
                            this.parse(account.getEmail(), 1, new ArrayList<>(0));
                            continue;
                        }
                        if (account.getIsSuperUser() || account.getRoleCategory().equals(RoleEnumType.MANAGER.getKey())) {

                            //主账号和管理员
                            List<Campaign> campaigns;
                            if (adsAccount.getPermission() == 0) {
                                campaigns = campaignRepository.findByRootParent(account.getRootParent());
                            } else {
                                campaigns = new ArrayList<>(0);
                            }
                            this.parse(account.getEmail(), adsAccount.getPermission(), campaigns);
                        } else if (account.getRoleCategory().equals(RoleEnumType.SUB_APP_MANAGER.getKey())) {

                            //子应用管理员按app授权
                            List<Campaign> campaigns;
                            if (adsAccount.getPermission() == 0) {
                                campaigns = campaignRepository.findByAuth(account.getId());
                            } else {
                                campaigns = new ArrayList<>(0);
                            }
                            this.parse(account.getEmail(), adsAccount.getPermission(), campaigns);
                        } else if (account.getRoleCategory().equals(RoleEnumType.CUSTOM_AUTH.getKey())
                                || account.getRoleCategory().equals(RoleEnumType.CUSTOM_ROLE.getKey())
                                || account.getRoleCategory().equals(RoleEnumType.CHANNEL_PERSON.getKey())) {

                            //子账号,渠道账号和自定义账号
                            if (adsAccount.getPermission() == 0) {
                                List<Auth> authList = authRepository.findAuthByAccountId(account.getId());
                                this.parseSubAccount(account.getEmail(), adsAccount.getPermission(), authList);
                            } else {
                                this.parseSubAccount(account.getEmail(), adsAccount.getPermission(), new ArrayList<>(0));
                            }
                        }
                    }
                }
            }
            logger.info("Adsdesk实时同步campaign...............//结束");
        } else {
            logger.info("Adsdesk另一任务正在执行 实时同步 {}", DateTime.now());
        }
    }

    private void parse(String email, int permission, List<Campaign> campaigns) {

        JSONObject jsonobj = new JSONObject();
        jsonobj.put("email", email);
        jsonobj.put("permission", permission);
        List<JSONObject> objects = new ArrayList<>(campaigns.size());
        for (Campaign campaign : campaigns) {
            JSONObject cam = new JSONObject(4);
            cam.put("campaignid", campaign.getCampaignid());
            cam.put("surl", campaign.getSurl());
            cam.put("pay_auth", true);
            cam.put("retention_auth", true);
            objects.add(cam);
        }
        jsonobj.put("campaigns", objects);
        this.UploadS3(jsonobj);
    }

    private void parseSubAccount(String email, int permission, List<Auth> auths) {
        JSONObject jsonobj = new JSONObject();
        jsonobj.put("email", email);
        jsonobj.put("permission", permission);
        List<JSONObject> campaigns = new ArrayList<>();
        for (Auth auth : auths) {
            List<DataAuth> dataAuths = dataAuthRepository.findDataAuthByAccountAndApp(auth.getAccount(), auth.getApp());
            List<Campaign> cams = new ArrayList<>();
            for (DataAuth dataAuth : dataAuths) {
                if (dataAuth.getAllCampaign()) {
                    cams.addAll(campaignRepository.findByAppAndChannel(dataAuth.getApp(), dataAuth.getChannel()));
                } else {
                    cams.add(campaignRepository.findOne(dataAuth.getCampaign()));
                }
            }
            for (Campaign cam : cams) {
                JSONObject jb = new JSONObject(4);
                if (cam == null) continue;
                jb.put("campaignid", cam.getCampaignid());
                jb.put("surl", cam.getSurl());
                jb.put("pay_auth", auth.isPayAuth());
                jb.put("retention_auth", auth.isRetentionAuth());
                campaigns.add(jb);
            }
        }
        jsonobj.put("campaigns", campaigns);
        this.UploadS3(jsonobj);
    }

    private static String[] emails = {"631268411@qq.com"};

    private static String tkioFields = "dt,adplan_id,cid,adshow_num,adclick_num,install_show,DAS0_num,adshow_num_d1,num_newuser_reten_d1,adshow_num_d2,num_newuser_reten_d2,adshow_num_d3,num_newuser_reten_d3,adshow_num_d4,num_newuser_reten_d4,adshow_num_d5,num_newuser_reten_d5,adshow_num_d6,num_newuser_reten_d6,adshow_num_d7,num_newuser_reten_d7";

    public void runAdplanByTkio() {
        if (!Constant.awsneed) {
            return;
        }
        if (RedisUtilNew.getInstance().tryLock("AdsdeskAdplanTkio", EXPIRE_PERIOD) != null) {
            Connection conn = null;
            PreparedStatement pt = null;
            ResultSet rs = null;
            conn = TKIOdbUtil.newInstance().getConn();
            try {
                for (String email : emails) {
                    Account account = accountRepository.findAccountByEmail(email);
                    List<App> apps = appRepository.findValidAppByRootParent(account.getId());
                    for (App app : apps) {
                        boolean hasdata = false;
                        StringBuffer sb = new StringBuffer();
                        String sql = "SELECT " + tkioFields + " FROM tkio.`tkio_rpt_flowreadli_day` WHERE appid= '" + app.getAppkey() + "' AND dt>='" + DateUtil.getBeforeDays(8) + "' AND dt <='" + DateUtil.getBeforeDays(1) + "' AND adplan_id!='-1'";
                        pt = conn.prepareStatement(sql);
                        rs = pt.executeQuery();
                        while (rs.next()) {
                            for (String field : tkioFields.split(",")) {
                                sb.append(rs.getString(field)).append(",");
                            }
                            hasdata = true;
                            sb.append("\n");
                        }
                        if (hasdata) {
                            String result = sb.substring(0, sb.length() - 2);
                            this.UploadS3ByAAUS(result, "tkio", email, app.getAppkey());
                        }
                    }
                }
            } catch (SQLException e) {
                logger.error(e.getMessage(), e);
            } finally {
                TkioStreamDBUtil.newInstance().release(rs, pt, conn);
            }
            logger.info("Adsdesk同步离线广告计划指标...............//结束");
        } else {
            logger.info("Adsdesk同步离线广告计划指标另一任务正在执行 实时同步 {}", DateTime.now());
        }
    }

    private static String tkioStreamFields = "dt,adplan_id,cid,adshow_num,adclick_num,install_show,DAS0_num";

    public void runAdplanByTkioStream() {
        if (!Constant.awsneed) {
            return;
        }
        if (RedisUtilNew.getInstance().tryLock("AdsdeskAdplanTkioStream", EXPIRE_PERIOD) != null) {
            Connection conn = null;
            PreparedStatement pt = null;
            ResultSet rs = null;
            conn = TKIOdbUtil.newInstance().getConn();
            try {
                for (String email : emails) {
                    Account account = accountRepository.findAccountByEmail(email);
                    List<App> apps = appRepository.findValidAppByRootParent(account.getId());
                    for (App app : apps) {
                        boolean hasdata = false;
                        StringBuffer sb = new StringBuffer();
                        String sql = "SELECT " + tkioStreamFields + " FROM tkio_stream.`tkio_rpt_flowreadli_day` WHERE appid= '" + app.getAppkey() + "' AND dt='" + DateUtil.getCurrentDateStr() + "' AND adplan_id!='-1'";
                        pt = conn.prepareStatement(sql);
                        rs = pt.executeQuery();
                        while (rs.next()) {
                            for (String field : tkioStreamFields.split(",")) {
                                sb.append(rs.getString(field)).append(",");
                            }
                            hasdata = true;
                            sb.append("\n");
                        }
                        if (hasdata) {
                            String result = sb.substring(0, sb.length() - 2);
                            this.UploadS3ByAAUS(result, "tkio_stream", email, app.getAppkey());
                        }
                    }
                }
            } catch (SQLException e) {
                logger.error(e.getMessage(), e);
            } finally {
                TkioStreamDBUtil.newInstance().release(rs, pt, conn);
            }
            logger.info("Adsdesk同步实时广告计划指标...............//结束");
        } else {
            logger.info("Adsdesk同步实时广告计划指标另一任务正在执行 实时同步 {}", DateTime.now());
        }
    }

    private void UploadS3(JSONObject jsonobj) {
        try {
            final String s3key = ADS_S3_PATH + jsonobj.getString("email") + ".txt";
            ObjectStorageUtil.putObjectInString("tkio2ads", s3key, jsonobj.toJSONString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void UploadS3ByAAUS(String result, String tbaleType, String email, String appkey) {
        try {
            final String s3key = "adplan/" + tbaleType + "/" + email + "_" + appkey + ".txt";
            ObjectStorageUtil.putObjectInString("tkio2ads", s3key, result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
