package com.reyun.task;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.model.AdvertisingBinduser;
import com.reyun.repository.AdvertisingBinduserRepository;
import com.reyun.repository.AppRepository;
import com.reyun.util.*;
import com.reyun.util.http.SSLClient;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AdvertisingApiTask {

    private static int EXPIRE_PERIOD = 60 * 4;
    private static Logger logger = LoggerFactory.getLogger(AdvertisingApiTask.class);

    @Autowired
    private AdvertisingBinduserRepository advertisingBinduserRepository;

    @Autowired
    private AppRepository appRepository;

    /*定时器调用的方法*/
    public void run() {

        if (RedisUtilNew.getInstance().tryLock("AdvertisingApiTaskTkio", EXPIRE_PERIOD) != null) {
            this.run(null);
        } else {
            logger.info("AdvertisingApiTask另一服务正在同步数据");
        }

    }

    /*可用于测试的方法*/
    public void run(AdvertisingBinduser binduser) {

        List<AdvertisingBinduser> binduserList;

        if (binduser == null) {
            Map<String, String> dateMap = new HashMap<>();
            binduserList = advertisingBinduserRepository.findByStatus();
        } else {
            binduserList = new ArrayList<>();
            binduserList.add(binduser);
        }

        String preDayTime = new DateTime().plusDays(-1).toString("yyyy-MM-dd");
        Map<String, String> csjAdidDic = null;
        if (!binduserList.isEmpty()) {
            csjAdidDic = this.getCsjAdidDic("csj", preDayTime, binduserList.get(0));
        }

        int lines = 0;
        for (AdvertisingBinduser advertisingBinduser : binduserList) {
            advertisingBinduser.setPreDayTime(preDayTime);
            advertisingBinduser.setNonce(RandomUtils.nextLong(100000, 9999999));
            advertisingBinduser.setAppKey(appRepository.findAppkeyById(advertisingBinduser.getApp()));
            if ("csj".equals(advertisingBinduser.getTypeName())) {
                //穿山甲 api 数据
                lines += this.getApiDataOfCsj(advertisingBinduser, csjAdidDic);
            }
        }

        if (lines > 0) {
            logger.info("AdvertisingApiTask同步:{}条穿山甲数据完成", lines);
        }
    }

    public int getApiDataOfCsj(AdvertisingBinduser binduser, Map<String, String> adidDic) {

        Map<String, String> params = new HashMap<>();

        params.put("user_id", binduser.getUserId());
        params.put("timestamp", System.currentTimeMillis() + "");
        params.put("nonce", binduser.getNonce() + "");
        params.put("start_date", binduser.isAuto() ? binduser.getPreDayTime() : binduser.getStartDate());
        params.put("end_date", binduser.isAuto() ? binduser.getPreDayTime() : binduser.getEndDate());
        params.put("sign", this.sign4CSJ(params, binduser.getSecureKey()));

        String[] type = new String[]{"slot", "app", "user"};
        String url = "https://partner.oceanengine.com/union/media/open/api/report/"
                + type[0] + "?" + this.getParamText(params);

        try {
            String result = HttpClientUtil.doHttpGetRequest(url, "");

            if (result == null) {
                return 0;
            }

            Map apiData = new ObjectMapper().readValue(result, Map.class);

            List<Map> dataLine = (List) apiData.get("data");

            if (dataLine == null || dataLine.isEmpty()) {
                logger.info("穿山甲api获取收入数据为空,appKey:{},code:{},message{}", binduser.getAppKey(), apiData.get("code"), apiData.get("message"));
                return 0;
            }

            StringBuffer dataSql = new StringBuffer();
            dataSql.append("replace into tkio_rpt_advertising_cash_day ( ");
            dataSql.append("ds, appid, adv_id, plat_type_name, adv_name, income_all, num_impression, up_date ) values ");
            StringBuffer sqlPart2 = new StringBuffer();
            String updateTimeValue = new DateTime().toString("yyyy-MM-dd HH:mm:ss");

            int linesTotal = 0;
            for (int i = 0; i < dataLine.size(); i++) {
                Map lineMap = dataLine.get(i);

                if (adidDic != null && !adidDic.containsKey(lineMap.get("ad_slot_id") + "")) {
                    //adid 不在trackio 报表,不处理数据
                    continue;
                }

                linesTotal++;
                sqlPart2.append("('").append(lineMap.get("stat_datetime")).append("'")
                        .append(",'").append(binduser.getAppKey()).append("'")
                        .append(",'").append(lineMap.get("ad_slot_id")).append("'")
                        .append(",'").append(binduser.getTypeName()).append("'")
                        .append(",'").append(lineMap.get("code_name")).append("'")
                        .append(",'").append(lineMap.get("cost") == null ? 0 : lineMap.get("cost")).append("'")
                        .append(",'").append(lineMap.get("show") == null ? 0 : lineMap.get("show")).append("'")
                        .append(",'").append(updateTimeValue).append("'")
                        .append("),");
            }
            if (linesTotal > 0) {
                dataSql = dataSql.append(sqlPart2);
                //插入数据
                TKIOdbUtil.newInstance().insert(dataSql.toString().substring(0, dataSql.length() - 1));
            }

            return linesTotal;

        } catch (IOException e) {
            logger.error("解析穿山甲api数据错误", e);
        }

        return 0;
    }


    private String sign4CSJ(Map<String, String> params, String secureKey) {

        String[] keys = new String[]{"secure_key", "timestamp", "nonce"};
        for (int i = 0; i < keys.length; i++) {
            if (i == 0) {
                keys[i] = secureKey;
            } else {
                keys[i] = params.get(keys[i]);
            }
        }
        Arrays.sort(keys);
        String sign = String.join("", keys);
        return DigestUtils.sha1Hex(sign);

    }

    private static Map<String, String> getCsjAdidDic(String typeName, String date, AdvertisingBinduser advertisingBinduser) {
        String sql = "select distinct adid, adplatform from tkio_rpt_flowreadli_day where adplatform in ('" +
                typeName + "', '" + typeName.toUpperCase() + "') and dt >='" + (advertisingBinduser.isAuto() ? date : advertisingBinduser.getStartDate()) + "' and dt<= '" + (advertisingBinduser.isAuto() ? date : advertisingBinduser.getEndDate()) + "'";
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        Map<String, String> dicMap = new HashMap<>();
        try {

            conn = TKIOdbUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                dicMap.put(rs.getString("adid"), rs.getString("adplatform"));
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }
        return dicMap;
    }

    private static List<String> getToponAdidDic(String appkey, AdvertisingBinduser advertisingBinduser) {
        String sql = "select distinct adid, adplatform from tkio_rpt_flowreadli_day where  dt >='" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getStartDate()) + "' and dt<= '" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getEndDate()) + "' and appid = '" + appkey + "' AND  adid REGEXP '[^0-9.]' = 0 AND LENGTH(adid)<=8";
        return getDistinctAid(sql);
    }

    private static List<String> getDistinctAid(String sql) {
        //topon的adid查全平台的adid,且必须小于int32 和纯数字类型
        Connection conn = null;
        PreparedStatement pt = null;
        ResultSet rs = null;
        List<String> adids = new ArrayList<>();
        try {

            conn = TKIOdbUtil.newInstance().getConn();
            pt = conn.prepareStatement(sql);
            rs = pt.executeQuery();
            while (rs.next()) {
                adids.add(rs.getString("adid"));
            }
        } catch (SQLException e) {
            logger.error(e.getMessage(), e);
        } finally {
            TkioStreamDBUtil.newInstance().release(rs, pt, conn);
        }
        return adids;
    }

    private String getParamText(Map<String, String> params) {
        StringBuffer txt = new StringBuffer();
        params.forEach((k, v) -> {
            if (txt.length() == 0) {
                txt.append(k).append("=").append(v);
            } else {
                txt.append("&").append(k).append("=").append(v);
            }
        });
        return txt.toString();
    }

    public void toponScheduled() {
        if (RedisUtilNew.getInstance().tryLock("AdvertisingApiTaskTkio4Topon", EXPIRE_PERIOD) != null) {
            run4Topon(null, "topon");
        } else {
            logger.info("AdvertisingApiTask另一服务正在同步数据");
        }
    }

    public void adtScheduled() {
        if (RedisUtilNew.getInstance().tryLock("AdvertisingApiTaskTkio4Adttiming", EXPIRE_PERIOD) != null) {
            run4Topon(null, "adtiming");
        } else {
            logger.info("AdvertisingApiTask另一服务正在同步数据");
        }
    }

    public void run4Topon(AdvertisingBinduser advertisingBinduser) {
        run4Topon(advertisingBinduser, advertisingBinduser.getTypeName());
    }

    //topon获取数据
    public void run4Topon(AdvertisingBinduser advertisingBinduser, String plattypename) {
        List<Object[]> byTypeAndStatus;
        if (advertisingBinduser == null) {
            advertisingBinduser = new AdvertisingBinduser();
            advertisingBinduser.setTypeName(plattypename);
            advertisingBinduser.setAuto(true);
            byTypeAndStatus = advertisingBinduserRepository.findByTypeAndStatus(plattypename);
        } else {

            byTypeAndStatus = advertisingBinduserRepository.findByTypeAndStatusAndId(advertisingBinduser.getId());
        }
        Boolean hasData = false;
        advertisingBinduser.setUpdateTimemer(new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
        advertisingBinduser.setPreDate(new DateTime().plusDays(-1).toString("yyyy-MM-dd"));
        StringBuffer dataSql = new StringBuffer("replace into tkio_rpt_advertising_cash_day ( ds, appid, adv_id, plat_type_name, plat_type, adv_name, income_all, up_date ) values ");
        for (Object[] binduser : byTypeAndStatus) {
            String appkey = binduser[0].toString();
            String[] publisherKeys = binduser[1].toString().split(",");

            List<String> adids = null;
            Map<String, String> adisMap = new HashMap<>();
            if ("topon".equals(advertisingBinduser.getTypeName())) {
                adids = getToponAdidDic(appkey, advertisingBinduser);
            } else if ("adtiming".equals(advertisingBinduser.getTypeName())) {
                String sql = "select distinct adid, adplatform from tkio_rpt_flowreadli_day where  dt >='" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getStartDate()) + "' and dt<= '" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getEndDate()) + "' and appid = '" + appkey + "'";
                adids = getDistinctAid(sql);
                adisMap = adids.stream().collect(Collectors.toMap(v -> v, v -> "1", (v1, v2) -> v1));
            }

            if (CollectionUtils.isEmpty(adids))
                continue;
            for (String publisherKey : publisherKeys) {

                if ("topon".equals(advertisingBinduser.getTypeName())) {
                    hasData = topOnSQLBuilder(advertisingBinduser, publisherKey, dataSql, appkey, adids);
                } else if ("adtiming".equals(advertisingBinduser.getTypeName())) {
                    hasData = adtSQLBuilder(advertisingBinduser, dataSql, appkey, adisMap, publisherKey);
                }
            }
        }
        if (hasData) {
            TKIOdbUtil.newInstance().insert(dataSql.toString().substring(0, dataSql.length() - 1));
        }
    }

    public Boolean adtSQLBuilder(AdvertisingBinduser advertisingBinduser, StringBuffer dataSql, String appkey, Map<String, String> adisMap, String publisherKey) {
        JSONArray arrayData = api4Adt(advertisingBinduser, publisherKey);
        boolean hasData = false;
        for (Object item : arrayData) {
            Map data = (Map) item;
            if (!adisMap.containsKey(data.get("placementId").toString())) {
                continue;
            }

            double value = data.get("revenue") == null ? 0.0
                    : Double.valueOf(data.get("revenue").toString()).doubleValue() * 6.6;
            value = BigDecimal.valueOf(value).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            dataSql.append("('").append(data.get("day"))
                    .append("','").append(appkey)
                    .append("','").append(data.get("placementId"))
                    .append("','").append(advertisingBinduser.getTypeName())
                    .append("','").append("10")
                    .append("','").append(data.get("placementName"))
                    .append("','").append(value)
                    .append("','").append(advertisingBinduser.getUpdateTimemer())
                    .append("'),");
            hasData = true;
        }
        return hasData;
    }

    private boolean topOnSQLBuilder(AdvertisingBinduser advertisingBinduser, String publisherKey, StringBuffer dataSql, String appkey, List<String> adids) {
        JSONArray records = api4topon(advertisingBinduser, publisherKey, adids);
        boolean hasData = false;
        if (records != null) {
            for (Object record : records) {
                JSONObject ad = (JSONObject) record;
                JSONObject adsource = ad.getJSONObject("adsource");
                String adsource_name = adsource.getString("adsource_name");
                String adsource_id = adsource.getString("adsource_id");
                String revenue = ad.getString("revenue");
                String ds = dateConvertion(((JSONObject) record).getString("date"));
                dataSql.append("('").append(ds).append("'")
                        .append(",'").append(appkey).append("'")
                        .append(",'").append(adsource_id).append("'")
                        .append(",'").append(advertisingBinduser.getTypeName()).append("'")
                        .append(",'").append("4").append("'")
                        .append(",'").append(adsource_name).append("'")
                        .append(",'").append(revenue).append("'")
                        .append(",'").append(advertisingBinduser.getUpdateTimemer()).append("'")
                        .append("),");
                hasData = true;
            }
        }
        return hasData;
    }

    private static String dateConvertion(String str) {
        Date parse = null;
        String dateString = "";
        try {
            parse = new SimpleDateFormat("yyyyMMdd").parse(str);
            dateString = new SimpleDateFormat("yyyy-MM-dd").format(parse);
        } catch (ParseException e) {
            dateString = null;
        }

        return dateString;
    }

    public static String signature(String httpMethod, String contentMD5,
                                   String contentType, String headerString, String resource) {

        String SignString = httpMethod + "\n" + contentMD5 + "\n" + contentType + "\n" + headerString + "\n" + resource;

        return md5Default(SignString).toUpperCase();

    }

    public static String doPost(String url, String cintent, String publisherKey, String publisherSignature,
                                String publisherTimestamp) {

        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("Content-Type", "application/json");
        headerMap.put("X-Up-Timestamp", publisherTimestamp);
        headerMap.put("X-Up-Key", publisherKey);
        headerMap.put("X-Up-Signature", publisherSignature);
        return doPost(url, cintent, headerMap);
    }


    public static String doPost(String url, String cintent, Map<String, String> headerMap) {
        HttpClient httpClient = null;
        HttpPost httpPost = null;
        String result = null;
        StringBuffer rs = new StringBuffer();
        try {
            httpClient = new SSLClient();
            httpPost = new HttpPost(url);

            for (String key : headerMap.keySet()) {
                httpPost.setHeader(key, headerMap.get(key));
            }

            httpPost.setEntity(new StringEntity(cintent));
            HttpResponse response = httpClient.execute(httpPost);

            if (response != null) {
                /*HttpEntity resEntity = response.getEntity();
                if (resEntity != null) {
                    result = EntityUtils.toString(resEntity, "utf-8");
                }*/
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    rs.append(line);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return rs.toString();
    }

    public static String doGet(String url, Map<String, String> headerMap) {
        HttpClient httpClient = null;
        HttpGet httpGet = null;
        StringBuffer rs = new StringBuffer();
        try {
            httpClient = new SSLClient();
            httpGet = new HttpGet(url);

            for (String key : headerMap.keySet()) {
                httpGet.setHeader(key, headerMap.get(key));
            }
            HttpResponse response = httpClient.execute(httpGet);
            if (response != null) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    rs.append(line);
                }
            }

        } catch (Exception e) {
            logger.error("api 请求错误", e);
        }

        return rs.toString();
    }


    public JSONArray api4Adt(AdvertisingBinduser advertisingBinduser, String publisherKey) {
        String predate = advertisingBinduser.getPreDate();
        StringBuffer urlBuffer = new StringBuffer("https://sdk.adtimingapi.com/report/v2");
        urlBuffer.append("?format=JSON")
                .append("&start=").append(advertisingBinduser.isAuto() ? predate : advertisingBinduser.getStartDate())
                .append("&end=").append(advertisingBinduser.isAuto() ? predate : advertisingBinduser.getEndDate())
                .append("&metrics=revenue")
                .append("&breakdowns=placement")
                .append("&token=").append(publisherKey)
                .append("&timeDimension=day");

        Map<String, String> header = new HashMap<>();
        //header.put("token", publisherKey);
        String result = doGet(urlBuffer.toString(), header);
        return JSON.parseArray(result);
    }

    public JSONArray api4topon(AdvertisingBinduser advertisingBinduser, String publisherKey, List<String> adids) {
        //topon api 中日期格式为 yyyyMMdd
        //String predate = date.replace("-", "");
        String cintent = "{\"startdate\":" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getStartDate()).replace("-", "") + ",\"enddate\":" + (advertisingBinduser.isAuto() ? advertisingBinduser.getPreDate() : advertisingBinduser.getEndDate()).replace("-", "") + ",\"group_by\":[\"date\",\"adsource\",\"network\"],\"adsource_id_list\":" + adids.toString() + "}";

        String contentMD5 = md5Default(cintent).toUpperCase();
        String url = "https://openapi.toponad.com/v2/fullreport";
        String publisherTimestamp = "" + System.currentTimeMillis();
        //String publisherKey = "80fd61b98a396edb16e3036d46c4079949a4297d";
        String headerString = "X-Up-Key:" + publisherKey + "\n" + "X-Up-Timestamp:" + publisherTimestamp;
        String publisherSignature = signature("POST", contentMD5, "application/json", headerString, "/v2/fullreport");

        String s = doPost(url, cintent, publisherKey, publisherSignature, publisherTimestamp);
        if (!StringUtil.isEmpty(s) && s.indexOf("records") != -1) {
            JSONObject jsonObject = JSON.parseObject(s);
            if (jsonObject.containsKey("records")) {
                JSONArray records = jsonObject.getJSONArray("records");
                if (!CollectionUtils.isEmpty(records)) {
                    return records;
                }
            }
        }
        return null;

    }

    public static String getMD5(String str) {
        try {
            return DigestUtils.md5Hex(str);
        } catch (Exception e) {
            return null;
        }
    }

    public final static String md5Default(String str) {
        try {
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(str.getBytes());
            byte b[] = md5.digest();

            StringBuffer sb = new StringBuffer("");
            for (int n = 0; n < b.length; n++) {
                int i = b[n];
                if (i < 0) i += 256;
                if (i < 16) sb.append("0");
                sb.append(Integer.toHexString(i));
            }
            return sb.toString();
        } catch (Exception e) {
            // ignore
        }
        return null;
    }


    public static void main(String[] args) {

       /* AdvertisingApiTask task = new AdvertisingApiTask();
        AdvertisingBinduser binduser = new AdvertisingBinduser();
        binduser.setAuto(true);
        binduser.setPreDate(new DateTime().plusDays(-1).toString("yyyy-MM-dd"));
        String key = "mwefwIVtzylV2L7a3uTLmoV1PQ2hU6bd";
        JSONArray array = task.api4Adt(binduser, key);
        System.out.println(array);*/

        System.out.println(new BigDecimal("0.68787").setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());

    }
}
