package com.reyun.task;

import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.Account;
import com.reyun.repository.AccountRepository;
import com.reyun.service.AccountFlowRestrictService;
import com.reyun.util.DateUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by song on 2018/2/12.
 */

@Service
public class AutoCalculateFlow {
    private static Logger logger = LoggerFactory.getLogger(AutoCalculateFlow.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE * 2 / 1000;

    @Autowired
    AccountFlowRestrictService accountFlowRestrictService;
    @Autowired
    AccountRepository accountRepository;

    public void run() {

        logger.info("计算用户流量................//Begin, 执行时间: {}", DateTime.now());
        if (RedisUtilNew.getInstance().tryLock("AutoCalculateFlowTrackingio", EXPIRE_PERIOD) != null) {
            try {
                List<Account> superAccount = accountRepository.findSuperAccount();
                logger.info("开始计算用户流量......,用户数：{}", superAccount == null ? 0 : superAccount.size());
                for (Account account : superAccount) {
                    String pastDate1 = account.getPastDate();
                    String pastDate5 = DateUtil.getBeforeDays(pastDate1, -5);
                    Date pastDate = DateUtil.parseDate(pastDate5);
                    // 过期不超过 5 天,计算流量 并 提醒用户
                    if (DateUtil.compareDate(pastDate, new Date()) != -1) {
                        try {
                            String lanType = account.getLanguageType();
                            if (StringUtils.isEmpty(lanType)) {
                                lanType = LanguageTypeEnum.CHINESE.getCode();
                            }

                            accountFlowRestrictService.RestrictFlowByAccountTask(account, lanType);

                        } catch (Exception e) {
                            logger.error("计算用户流量错误", e);
                        }
                    }
                }
            } catch (Exception e) {
                logger.error("计算用户流量错误", e);
            }
        } else {
            logger.info("其他任务正在计算用户流量......");
        }
        logger.info("计算用户流量................//End{}", DateTime.now());
    }

    public static void main(String[] args) {
//        String pastDate1 = "2018-12-3";
        String pastDate1 = "2020-01-02";
        String pastDate5 = DateUtil.getBeforeDays(pastDate1, -5);
        Date pastDate = DateUtil.parseDate(pastDate5);
        System.out.println(DateUtil.compareDate(pastDate, new Date()));
    }
}
