package com.reyun.task;

import com.alibaba.druid.sql.visitor.functions.Concat;
import com.reyun.model.BlacklistApplication;
import com.reyun.repository.BlacklistApplicationRepository;
import com.reyun.service.BlacklistApplicationCheckService;
import com.reyun.util.Constant;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 定时监测防作弊开通是否过期
 */

@Service
public class AutoCheckBlackListAppTask {
    private static Logger logger = LoggerFactory.getLogger(AutoCheckBlackListAppTask.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE / 1000;

    @Autowired
    BlacklistApplicationRepository blacklistApplicationRepository;

    @Autowired
    BlacklistApplicationCheckService blacklistApplicationCheckService;

    public void run() {

        logger.info("监测防作弊开通是否过期................//Begin, 执行时间: {}", DateTime.now());

//        boolean notProd = StringUtils.equalsIgnoreCase(Constant.mailProd,"false");
//        if (RedisUtilNew.getInstance().tryLock("AutoCheckBlackListAppTask", EXPIRE_PERIOD) != null || notProd) {
        if (RedisUtilNew.getInstance().tryLock("AutoCheckBlackListAppTask", EXPIRE_PERIOD) != null ) {
            try {
                DateTime today = new DateTime();

                List<BlacklistApplication> applications = blacklistApplicationRepository.findAllnotPast(today.plusDays(-10).toString("yyyy-MM-dd"));

                for (BlacklistApplication application : applications) {

                    DateTime past = DateTime.parse(application.getPastDate());

                    int remainDays = Days.daysBetween(today,past).getDays()+1;

                    if(remainDays <=15 && remainDays > 0){

                        blacklistApplicationCheckService.remainDaysNotify(application,remainDays);
                    }
                }

            } catch (Exception e) {
                logger.error("监测防作弊错误",e);
            }
        } else {
            logger.info("其他任务正在执行监测防作弊开通是否过期......");
        }
        logger.info("监测防作弊开通是否过期................//End{}", DateTime.now());
    }

    public static void main(String[] args) {
        DateTime today = new DateTime();

        System.out.println(today.plusDays(-5).toString("yyyy-MM-dd"));

        DateTime past= DateTime.parse("2018-12-12");

        System.out.println(Days.daysBetween(past,today).getDays());

    }
}
