package com.reyun.task;

import com.reyun.dic.LanguageTypeEnum;
import com.reyun.model.Account;
import com.reyun.model.Crowd;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.ConfigParamRepository;
import com.reyun.repository.CrowdRepository;
import com.reyun.service.CrowdService;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

@Service
public class AutoCheckCrowdTasks {

    private static org.slf4j.Logger logger = LoggerFactory.getLogger(AutoCheckCrowdTasks.class);
    private final static String mailNeed = Constant.mailNeed;

    @Autowired
    private CrowdRepository crowdRepository;

    @Autowired
    private CrowdService crowdService;

    @Autowired
    private ConfigParamRepository configParamRepository;

    @Autowired
    private AccountRepository accountRepository;

    public void run() {

        logger.info("开始查询人群分析任务是否执行完毕................//Begin, 执行时间: {}", DateTime.now());

        int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE / 1000;
        if (RedisUtilNew.getInstance().tryLock("AutoCheckCrowdTasks", EXPIRE_PERIOD) != null) {
            try {
                List<Crowd> orders = crowdRepository.findNotCompleteOrder(0);
                for (Crowd crowd : orders) {
                    HashMap<String, Object> priceDetail = crowdService.getPriceDetail(crowd);
                        double d = (Double) priceDetail.get("totalMoney");
                        BigDecimal b = new BigDecimal(d);
                        double totalMoney = b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                        if (!(0.00 == totalMoney)) {
                            crowd.setStatus(1);
                            Crowd save = crowdRepository.save(crowd);
                            sendEmailToSalesManAndCustomer(save, totalMoney);
                        }
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info(e.getMessage());
            }
        } else {
            logger.info("正在查询人群分析任务执行情况......");
        }
        logger.info("查询人群分析任务执行情况................//End");
    }

    private void sendEmailToSalesManAndCustomer(Crowd crowd, Double totalMoney) {
        boolean ismail = Boolean.parseBoolean(mailNeed);
        Account account = accountRepository.findOne(crowd.getRootAccount());
        String subject = "热云数据广告监测平台人群分析功能购买申请";
        String customerContents = "<!doctype html><html><head></head><body> " +
                "您于" + DateUtil.formatSec(crowd.getCreateTime()) + "提交的标签获取订单已处理完成，总价¥" + totalMoney +
                "，我们的销售将联系您签约付款，收到款项后您即可以下载人群标签。</br>" + "订单详细信息，" +
                "<a href=\"" + Constant.tkioUrl + "\">" + "点此查看" + "</a>" +
                "</body></html>";

        //处理国际化
        if(LanguageTypeEnum.ENGLISH.getCode().equals(account.getLanguageType())){
            subject = "Request to purchase the population analysis function of the Ad monitoring platform of Reyun";

            customerContents = "<!doctype html><html><head></head><body> " +
                    "Your tag acquisition order submitted on " + DateUtil.formatSec(crowd.getCreateTime()) + " has been processed. The total price is ¥" + totalMoney +
                    ". Our sales personnel will contact you for signing the contract and payment related matters. After the payment is received, you can download the population tag. </br>" + " Order details，" +
                    "<a href=\"" + Constant.tkioUrl + "\">" + " Point this view" + "</a>" +
                    "</body></html>";
        }
        //end

        ArrayList<String> customerEmailList = new ArrayList<>();
        Collections.addAll(customerEmailList, configParamRepository.findParamsByKey("operate_email").split(","));
        if (ismail) {
            customerEmailList.add(account.getEmail());
        }
        try {
            EmailThread emailThread1 = new EmailThread(subject, customerContents, customerEmailList);
            emailThread1.start();
        } catch (Exception e) {
            e.printStackTrace();
        }

        String larry_email = configParamRepository.findParamsByKey("larry_email");
        ArrayList<String> salesManEmailList = new ArrayList<>();
        Collections.addAll(salesManEmailList, configParamRepository.findParamsByKey("operate_email").split(","));
        if (larry_email != null && ismail) {
            salesManEmailList.add(larry_email);
        }
        String cusName = account.getName() == null ? account.getName() : "" + account.getEmail() + "(账号)于";
        String salesManContents = "<!doctype html><html><head></head><body> " +
                cusName + DateUtil.formatSec(crowd.getCreateTime()) + "提交的标签获取订单已处理完成，总价¥" + totalMoney +
                "，请联系销售进行付款跟进。</br>" + "订单详细信息，" +
                "<a href=\"" + Constant.tkioUrl + "\">" + "点此查看" + "</a>" +
                "</body></html>";

        try {
            EmailThread emailThread2 = new EmailThread(subject, salesManContents, salesManEmailList);
            emailThread2.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
