package com.reyun.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.reyun.model.App;
import com.reyun.model.ChannelAccount;
import com.reyun.repository.AppRepository;
import com.reyun.repository.ChannelAccountRepository;
import com.reyun.repository.ConfigParamRepository;
import com.reyun.service.ChannelAccountService;
import com.reyun.util.Constant;
import com.reyun.util.NoSQLDBUtil;
import com.reyun.util.RedisUtilNew;
import com.reyun.util.SendCommonPostMail;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * description:
 *
 * @author nolan
 * @date 27/11/2017
 */
public class AutoCheckTokenExpire {
    private static Logger logger = LoggerFactory.getLogger(AutoCheckTokenExpire.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE * 1 / 1000;

    @Autowired
    private ChannelAccountService channelAccountService;

    @Autowired
    private ConfigParamRepository configParamRepository;

    @Autowired
    private ChannelAccountRepository channelAccountRepository;
    @Autowired
    private AppRepository appRepository;



    public void run() {
        logger.info("检查channel_account中的acesstoken是否过期................//Begin, 执行时间: {}", DateTime.now());

        if (RedisUtilNew.getInstance().tryLock("AutoCheckTokenExpire", EXPIRE_PERIOD) != null) {

            checkDueAndEmail();
        } else {
            logger.info("正在执行中......");
        }

        logger.info("检查channel_account中的acesstoken是否过期...............//End");
    }

    public void refreshToutiaoToken() {
        logger.info("今日头条acesstoken刷新................//Begin, 执行时间: {}", DateTime.now());
        if(!"true".equals(Constant.mailProd)){
            logger.info("非AWS不刷新今日头条acesstoken......");
            return;
        }
        if (RedisUtilNew.getInstance().tryLock("refreshToutiaoToken", 1000) != null) {
            try {
                checkToutiaoChannelAccountAndDelete();//刷token之前同步头条投放账号并删除没用的投放账号
            }catch (Exception e){
                e.printStackTrace();
                logger.error("头条账号定时同步错误,checkToutiaoChannelAccountAndDelete() ERROR");
            }

            HashMap<String, ChannelAccount> channelAccountMap = new HashMap<>();
            List<ChannelAccount> allValidToutiao = channelAccountRepository.findAllValidToutiao();
            for (ChannelAccount channelAccount : allValidToutiao) {
                boolean isRefresh = false;
                if(channelAccountMap.containsKey(channelAccount.getSgin())){
                    ChannelAccount ca = channelAccountMap.get(channelAccount.getSgin());
                    channelAccount.setAccessToken(ca.getAccessToken());
                    channelAccount.setAccessTokenExpire(ca.getAccessTokenExpire());
                    channelAccount.setRefreshToken(ca.getRefreshToken());
                    channelAccount.setRefreshTokenExpire(ca.getRefreshTokenExpire());
                    channelAccount.setLastUpdateTime(new Date());
                    ChannelAccount save = channelAccountRepository.save(channelAccount);
                    List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                    //投放账号开启的时候advertiserId已经存入DDB.所以这里是false
                    NoSQLDBUtil.putMarketingapi(save,apps,false, "toutiao_");
                }else{
                    JSONObject jsonObject = refreshAccessToken(channelAccount.getRefreshToken());
                    if(jsonObject!=null&&jsonObject.containsKey("data")&&jsonObject.getInteger("code")==0){
                        com.alibaba.fastjson.JSONObject dataResult = jsonObject.getJSONObject("data");
                        if(dataResult==null){
                            logger.info("头条返回data为空,channelAccountId="+channelAccount.getId()+",data:"+jsonObject.toJSONString());
                            continue;
                        }
                        channelAccount.setAccessToken(dataResult.getString("access_token"));
                        channelAccount.setAccessTokenExpire(dataResult.getLong("expires_in"));
                        channelAccount.setRefreshToken(dataResult.getString("refresh_token"));
                        channelAccount.setRefreshTokenExpire(dataResult.getLong("refresh_token_expires_in"));
                        channelAccount.setLastUpdateTime(new Date());
                        ChannelAccount save = channelAccountRepository.save(channelAccount);
                        List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                        //投放账号开启的时候advertiserId已经存入DDB.所以这里是false
                        NoSQLDBUtil.putMarketingapi(save,apps,false, "toutiao_");
                        isRefresh = true;
                    }
                }
                if(isRefresh&&!channelAccountMap.containsKey(channelAccount.getSgin())){
                    channelAccountMap.put(channelAccount.getSgin(),channelAccount);
                }
            }
        } else {
            logger.info("今日头条acesstoken刷新正在执行中......");
        }

        logger.info("今日头条acesstoken刷新...............//End");
    }
    public void refreshKuaishouToken() {
        logger.info("快手acesstoken刷新................//Begin, 执行时间: {}", DateTime.now());
        if(!"true".equals(Constant.mailProd)){
            logger.info("非AWS不刷新快手acesstoken......");
            return;
        }
        if (RedisUtilNew.getInstance().tryLock("refreshKuaishouToken", 1000) != null) {

            List<ChannelAccount> allValidKuaishou = channelAccountRepository.findAllValidKuaishou();
            for (ChannelAccount channelAccount : allValidKuaishou) {
                String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
                // 请求参数
                Map<String, Object> data = new HashMap(3) {
                    {
                        put("app_id", channelAccount.getName());
                        put("secret", channelAccount.getPassword());
                        put("refresh_token", channelAccount.getRefreshToken());
                    }
                };
                // 构造请求
                HttpPost httpEntity = new HttpPost(url);

                CloseableHttpResponse response = null;
                CloseableHttpClient client = null;
                try {
                    client = HttpClientBuilder.create().build();
                    httpEntity.setEntity(new StringEntity(JSON.toJSONString(data), ContentType.APPLICATION_JSON));

                    response = client.execute(httpEntity);
                    if (response != null && response.getStatusLine().getStatusCode() == 200) {
                        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                        StringBuffer result = new StringBuffer();
                        String line = "";
                        while ((line = bufferedReader.readLine()) != null) {
                            result.append(line);
                        }
                        bufferedReader.close();
                        com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());
                        if("OK".equals(jsonObject.getString("message"))&&jsonObject.getJSONObject("data").size()>0){
                            com.alibaba.fastjson.JSONObject rs = jsonObject.getJSONObject("data");
                            String access_token = rs.getString("access_token");
                            channelAccount.setAccessToken(access_token);
                            String refresh_token_expires_in = rs.getString("refresh_token_expires_in");
                            channelAccount.setRefreshTokenExpire(Long.parseLong(refresh_token_expires_in));
                            String refresh_token = rs.getString("refresh_token");
                            channelAccount.setRefreshToken(refresh_token);
                            String access_token_expires_in = rs.getString("access_token_expires_in");
                            channelAccount.setAccessTokenExpire(Long.parseLong(access_token_expires_in));
                            String advertiser_id = rs.getString("advertiser_id");
                            channelAccount.setThirdAccountCode(advertiser_id);
                            channelAccount.setLastUpdateTime(new Date());
                            ChannelAccount save = channelAccountRepository.save(channelAccount);
                            List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                            NoSQLDBUtil.putMarketingapi(save,apps,false, "kuaishou_");
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (response != null) {
                            response.close();
                        }
                        client.close();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            logger.info("快手acesstoken刷新正在执行中......");
        }

        logger.info("快手acesstoken刷新...............//End");
    }
    public void refreshWeixinMPToken() {
        logger.info("微信MPtoken刷新................//Begin, 执行时间: {}", DateTime.now());
        if(!"true".equals(Constant.mailProd)){
            logger.info("非AWS不刷新微信MPtoken......");
            return;
        }
        if (RedisUtilNew.getInstance().tryLock("refreshWeixinMPToken", 1000) != null) {

            List<ChannelAccount> allValidWeixinMP = channelAccountRepository.findAllValidWeixinMP();
            for (ChannelAccount channelAccount : allValidWeixinMP) {
                String url = new StringBuffer("https://api.e.qq.com/oauth/token?")
                        .append("client_id=").append("1106237187")
                        .append("&client_secret=").append("BvITHJpxu0gufAxF")
                        .append("&grant_type=refresh_token")
                        .append("&refresh_token=").append(channelAccount.getRefreshToken()).toString();
                // 请求参数
                // 构造请求
                HttpGet httpEntity = new HttpGet(url);

                CloseableHttpResponse response = null;
                CloseableHttpClient client = null;
                try {
                    client = HttpClientBuilder.create().build();

                    response = client.execute(httpEntity);
                    if (response != null && response.getStatusLine().getStatusCode() == 200) {
                        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                        StringBuffer result = new StringBuffer();
                        String line = "";
                        while ((line = bufferedReader.readLine()) != null) {
                            result.append(line);
                        }
                        bufferedReader.close();
                        com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(result.toString());

                        if(jsonObject.getInteger("code")==0){
                            com.alibaba.fastjson.JSONObject rs = jsonObject.getJSONObject("data");
                            com.alibaba.fastjson.JSONObject authorizerInfo = rs.getJSONObject("authorizer_info");
                            String account_id = authorizerInfo.getString("account_id");
                            channelAccount.setModifyTime(new Date());

                            String access_token = rs.getString("access_token");
                            channelAccount.setAccessToken(access_token);
                            String refresh_token_expires_in = rs.getString("refresh_token_expires_in");
                            channelAccount.setRefreshTokenExpire(Long.parseLong(refresh_token_expires_in));
                            String refresh_token = rs.getString("refresh_token");
                            channelAccount.setRefreshToken(refresh_token);
                            String access_token_expires_in = rs.getString("access_token_expires_in");
                            channelAccount.setAccessTokenExpire(Long.parseLong(access_token_expires_in));

                            channelAccount.setThirdAccountCode(account_id);
                            String wechat_account_id = authorizerInfo.getString("wechat_account_id");
                            channelAccount.setName(wechat_account_id);
                            channelAccount.setLastUpdateTime(new Date());
                            channelAccountRepository.save(channelAccount);
                            List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                            NoSQLDBUtil.putMarketingapi(channelAccount,apps,false,"wxmp_");
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (response != null) {
                            response.close();
                        }
                        client.close();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            logger.info("微信MPtoken刷新正在执行中......");
        }

        logger.info("微信MPtoken刷新...............//End");
    }
    public void refreshBaiduToken() {
        logger.info("百度原生ocpc DDBappkey同步................//Begin, 执行时间: {}", DateTime.now());
        if (RedisUtilNew.getInstance().tryLock("refreshBaiduToken", 1000) != null) {

            List<ChannelAccount> allValidBaidu = channelAccountRepository.findAllValidBaidu();
            for (ChannelAccount channelAccount : allValidBaidu) {
                    channelAccount.setLastUpdateTime(new Date());
                    ChannelAccount save = channelAccountRepository.save(channelAccount);
                    List<App> apps = appRepository.findValidAppByRootParent(channelAccount.getRootAccount());
                    //投放账号开启的时候advertiserId已经存入DDB.所以这里是false
                    NoSQLDBUtil.putMarketingapi(save,apps,false, "baiduysgg_");
            }
        } else {
            logger.info("百度原生ocpc DDBappkey刷新正在执行中......");
        }

        logger.info("百度原生ocpc DDBappkey刷新...............//End");
    }

    public void checkDueAndEmail() {
        int checkDueHour = 2;
        String mails = configParamRepository.findParamsByKey("token_notify_mail");
        if(StringUtils.isEmpty(mails)){
            // 未配置收件人  不做检查
            return;
        }

        List<String> mailsUser = Arrays.asList(mails.split(","));

        Optional<List<ChannelAccount>> listOptional = channelAccountService.findByStatusTrue();
        if (listOptional.isPresent()) {
            final List<ChannelAccount> collect = listOptional.get().stream().filter(channelAccount -> {
                return isDueRange(channelAccount, checkDueHour);
            }).collect(Collectors.toList());

            if (collect != null && collect.size() > 0) {
                String mailStr = printTable("trackingio", collect);
                logger.warn(mailStr);
                try {
                    SendCommonPostMail.sendMailReuse(mailsUser,"渠道帐号TOKEN刷新错误",mailStr,null,null);
                } catch (Exception e) {
                    logger.error("发送邮件失败, content: {}, emaillist: {}", mailStr, mailsUser);
                }
            }
        }
    }

    private boolean isDueRange(ChannelAccount channelAccount, int rangeHour) {
        return channelAccount.getLastUpdateTime() != null
                && channelAccount.getAccessTokenExpire() != null
                && new DateTime(channelAccount.getLastUpdateTime()).plusSeconds(channelAccount.getAccessTokenExpire().intValue()).minusHours(rangeHour).isBeforeNow();
    }

    private String printTable(String system, List<ChannelAccount> channelAccounts) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("#").append(system).append("\n");
        final String format = "%5s %10s %10s %16s %5s %5s\n";
        stringBuilder.append("--------------------------------------------------------------------------\n");
        stringBuilder.append(String.format(format, "渠道", "别名", "渠道帐号", "最后更新日期", "TK有效期", "启用状态"));
        stringBuilder.append("--------------------------------------------------------------------------\n");
        channelAccounts.stream().forEach(channelAccount -> {
            String lastUpdateTime = "none";
            if (channelAccount.getLastUpdateTime() != null) {
                lastUpdateTime = new DateTime(channelAccount.getLastUpdateTime()).toString("yyyy-MM-dd HH:mm:ss");
            }
            stringBuilder.append(String.format(format
                    , channelAccount.getUniqueName()
                    , channelAccount.getAlias()
                    , channelAccount.getName() + "(" + channelAccount.getId() + ")"
                    , lastUpdateTime
                    , channelAccount.getAccessTokenExpire()
                    , channelAccount.getStatus()));
        });
        stringBuilder.append("\n");
        return stringBuilder.toString();
    }

    public static JSONObject refreshAccessToken(String refresh_token) {
        String appId = "1653688605754372";
        String Secret = "1e858757bdd8beaf7ca68579a8ab4d2a7b3296a3";

        // 请求地址
        String url = "https://ad.oceanengine.com/open_api/oauth2/refresh_token/";

        // 请求参数
        Map<String, Object> data = new HashMap() {
            {
                put("appid", appId);
                put("secret", Secret);
                put("grant_type", "refresh_token");
                put("refresh_token", refresh_token);
            }
        };

        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(JSONObject.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }

        } catch (Exception e) {
            logger.error("", e);
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                if (client != null) {
                    client.close();
                }
            } catch (IOException e) {
                logger.error("", e);
            }
        }
        return null;
    }

    public void checkToutiaoChannelAccountAndDelete(){
        Long appId = 1653688605754372L;
        String secret = "1e858757bdd8beaf7ca68579a8ab4d2a7b3296a3";

        List<String> accessTokens = channelAccountRepository.findToutiaoTokens();

        NoSQLDBUtil.Result advertiserIdItem = NoSQLDBUtil.queryByPrimaryKey(NoSQLDBUtil.TBL_TABLE_MARKETKNGAPI, "id",  "toutiao_advertiser_id_list");
        Set<String> advertiserIdSet = advertiserIdItem.getStringSet("datas");

        int i = 0;
        int j = 0;
        for (String accessToken : accessTokens) {
            List<ChannelAccount> channelAccounts = channelAccountRepository.findByAccessToken(accessToken);
            Map<String, ChannelAccount> collect = channelAccounts.stream().collect(Collectors.toMap(ChannelAccount::getThirdAccountCode, Function.identity(), (v1, v2) -> v1));
            HashMap<String, String> toutiaoName = channelAccountService.getToutiaoName(appId, secret, accessToken);
            logger.info("获取头条投放账号:appId={},secret={},accessToken={},result={}", appId, secret, accessToken, toutiaoName.size());
            if(toutiaoName==null) continue;
            List<App> apps = appRepository.findValidAppByRootParent(channelAccounts.get(0).getRootAccount());

            ChannelAccount ca = channelAccounts.get(0);
            String caAccessToken = ca.getAccessToken();
            Long caAccessTokenExpire =ca.getAccessTokenExpire();
            String caRefreshToken = ca.getRefreshToken();
            Long caRefreshTokenExpire = ca.getRefreshTokenExpire();
            Long createAccount = ca.getCreateAccount();
            String uniqueName = ca.getUniqueName();
            Long rootAccount = ca.getRootAccount();

            for (String advertiserid : toutiaoName.keySet()) {
                if (collect.containsKey(advertiserid)) {
                    //已有的绑定账号不变
                    collect.remove(advertiserid);
                }else{
                    //insert
                    ChannelAccount channelAccount = new ChannelAccount();
                    String name = toutiaoName.get(advertiserid);
                    channelAccount.setAlias(name);
                    channelAccount.setChannel(514L);
                    channelAccount.setAccessToken(caAccessToken);
                    channelAccount.setAccessTokenExpire(caAccessTokenExpire);
                    channelAccount.setRefreshToken(caRefreshToken);
                    channelAccount.setRefreshTokenExpire(caRefreshTokenExpire);
                    channelAccount.setCreateAccount(createAccount);
                    channelAccount.setName(name);
                    channelAccount.setThirdAccountCode(advertiserid);
                    channelAccount.setUniqueName(uniqueName);
                    channelAccount.setRootAccount(rootAccount);
                    channelAccount.setStatus(true);
                    channelAccount.setModifyTime(new Date());
                    channelAccount.setSgin(ca.getSgin());
                    channelAccount.setRole(1);
                    ChannelAccount save = channelAccountRepository.save(channelAccount);
                    NoSQLDBUtil.putMarketingapi(save,apps,true, "toutiao_");
                    j++;
                }
            }
            for (ChannelAccount channelAccount : collect.values()) {
                //剩余账号为广告主在头条取消绑定的 同时删除ddb以及mysql
                channelAccountService.delete(channelAccount.getId(),channelAccount.getCreateAccount());
                channelAccountRepository.delete(channelAccount);
                logger.info("头条取消绑定的账号，本地同步删除:channelAccount={}", JSON.toJSONString(channelAccount));
                i++;
            }
            //确保mysql DDB保持一致
                for (ChannelAccount channelAccount : channelAccounts) {
                    if(!advertiserIdSet.contains(channelAccount.getThirdAccountCode())){
                        NoSQLDBUtil.putMarketingapi(channelAccount,apps,true, "toutiao_");
                    }
                }
        }
        logger.info("头条账号定时同步,共删除"+i+"个无效账号,共添加"+j);
    }
}
