package com.reyun.task;

import com.reyun.service.CampaignService;
import com.reyun.service.CpcallbackService;
import com.reyun.util.DateUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * Created by zy on 2019-03-18
 * desc:主账号到期15天后，停止点击和cp回调
 */
public class AutoCloseExpiredAccountData {

    private static Logger logger = LoggerFactory.getLogger(AutoCloseExpiredAccountData.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_SECOND * 30 / 1000;

    @Autowired
    private CpcallbackService cpcallbackService;

    @Autowired
    private CampaignService campaignService;

    public void run() {
        if (RedisUtilNew.getInstance().tryLock("trackingioAutoCloseExpiredAccountData", EXPIRE_PERIOD) != null) {
            logger.info("关闭已经到期15天的账号的点击和cp回调......");
            //当前日期往前推15天
            String date15 = DateUtil.format(DateUtil.addDays(new Date(), -15));
            //关闭过期cp回调
            cpcallbackService.closeExpiredCpcallback(date15);
            //删除DDB和mysql的过期短链并且备份
            campaignService.deleteExpiredCampaign(date15);
        } else {
            logger.info("关闭已经到期15天的账号的点击和cp回调, 正在处理......");
        }
    }
}
