package com.reyun.task;

import com.reyun.model.KeywordsCreative;
import com.reyun.model.SemRename;
import com.reyun.repository.KeywordsCreativeRepository;
import com.reyun.repository.SemRenameRepository;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.apache.poi.ss.usermodel.*;
import org.joda.time.DateTime;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;

@Service
public class AutoCreateBdKeywords {
    private static org.slf4j.Logger logger = LoggerFactory.getLogger(AutoCheckCrowdTasks.class);
    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;
    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";
    private static final String KEYWORDS = "KEYWORDS";
    private static final String CREATIVE = "CREATIVE";

    @Autowired
    private SemRenameRepository semRenameRepository;

    @Autowired
    private KeywordsCreativeRepository keywordsCreativeRepository;

    public void run() {

        logger.info("关键词和创意id批量替换................//Begin, 执行时间: {}", DateTime.now());
        Map<String, StringBuffer> logmap = new HashMap();
        int index = 0;
        int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE / 1000;
        if (RedisUtilNew.getInstance().tryLock("AutoCreateBdKeywords", EXPIRE_PERIOD) != null) {
            try {
                List<SemRename> tasks = semRenameRepository.findNotCompleteTasks(0);

                for (SemRename semRename : tasks) {
                    Long cid = semRename.getCid();
                    String fileName = semRename.getFileName();
                    String s3key = DOWNLOAD_PATH + fileName;
                    InputStream inputStream = ObjectStorageUtil.getObjectContentStream(S3_DOWNLOAD_BUCKET, s3key);
                    int exceptin_line = 0;
                    StringBuilder errorLine = new StringBuilder();
                    Long rootAccount = semRename.getRootAccount();
                    try {
                        Workbook workbook = WorkbookFactory.create(inputStream);
                        Sheet sheet = workbook.getSheetAt(0);
                        DataFormatter formatter = new DataFormatter();
                        int rowNumber = sheet.getLastRowNum();
                        List<KeywordsCreative> updateBatchList = new ArrayList<>();

                        for (int i = 1; i <= rowNumber; i++) {
                            String keywordId = null;
                            String keywordIdChina = null;
                            String creativeId = null;
                            String creativeIdChina = null;
                            index = i;
                            exceptin_line = i;
                            Row row = sheet.getRow(i);
                            if (row != null) {
                                Cell cell = row.getCell(0);
                                if (null != cell && !cell.getCellTypeEnum().equals(CellType.BLANK)) {

                                    keywordId = dealKeyWordsActiveId(formatter, cell, index);

                                }

                                Cell cell1 = row.getCell(1);
                                if (null != cell1 && !cell1.getCellTypeEnum().equals(CellType.BLANK)) {
                                    keywordIdChina = formatter.formatCellValue(cell1);
                                }

                                Cell cell2 = row.getCell(2);
                                if (null != cell2 && !cell2.getCellTypeEnum().equals(CellType.BLANK)) {

                                    creativeId = dealKeyWordsActiveId(formatter, cell2, index);
                                }

                                Cell cell3 = row.getCell(3);
                                if (null != cell3 && !cell3.getCellTypeEnum().equals(CellType.BLANK)) {
                                    creativeIdChina = formatter.formatCellValue(cell3);
                                }

                                boolean keywordsSize = keywordId != null && keywordIdChina != null && keywordIdChina.toCharArray().length > 60;
                                boolean creativeSize = creativeId != null && creativeIdChina != null && creativeIdChina.toCharArray().length > 60;


                                if ((keywordId != null && keywordIdChina == null) || (creativeId != null && creativeIdChina == null) ||
                                        (keywordId == null && keywordIdChina != null) || (creativeId == null && creativeIdChina != null) || keywordsSize || creativeSize) {
                                    errorLine.append(i).append(",");
                                } else {
                                    if (keywordId != null && keywordIdChina != null && !StringUtils.equals(keywordId, "0")) {
                                        KeywordsCreative byKeywordsId = keywordsCreativeRepository.findByKeywordsId(rootAccount, cid, keywordId, KEYWORDS);
                                        if (byKeywordsId != null) {
                                            if (byKeywordsId.getValue() == null || (!StringUtils.equals(byKeywordsId.getValue().trim(), keywordIdChina.trim()))) {
                                                byKeywordsId.setValue(keywordIdChina);
                                                byKeywordsId.setModifyDate(new Date());
                                                updateBatchList.add(byKeywordsId);
                                            }
                                        } else {
                                            KeywordsCreative keywordsCreative = new KeywordsCreative();
                                            keywordsCreative.setThirdAccount(-1L);
                                            keywordsCreative.setChannelId(cid);
                                            keywordsCreative.setAccountId(rootAccount);
                                            keywordsCreative.setInsertDate(new Date());
                                            keywordsCreative.setKeyId(keywordId);
                                            keywordsCreative.setValue(keywordIdChina);
                                            keywordsCreative.setDataType(KEYWORDS);
                                            keywordsCreativeRepository.save(keywordsCreative);
                                        }
                                    }

                                    if (creativeId != null && creativeIdChina != null && !StringUtils.equals(creativeId, "0")) {
                                        KeywordsCreative byCreativeId = keywordsCreativeRepository.findByCreativeId(rootAccount, cid, creativeId, CREATIVE);
                                        if (byCreativeId != null) {
                                            if (byCreativeId.getValue() == null || (!StringUtils.equals(byCreativeId.getValue().trim(), creativeIdChina.trim()))) {
                                                byCreativeId.setValue(keywordIdChina);
                                                byCreativeId.setModifyDate(new Date());
                                                updateBatchList.add(byCreativeId);
                                            }
                                        } else {
                                            KeywordsCreative keywordsCreative = new KeywordsCreative();
                                            keywordsCreative.setThirdAccount(-1L);
                                            keywordsCreative.setChannelId(cid);
                                            keywordsCreative.setAccountId(rootAccount);
                                            keywordsCreative.setKeyId(creativeId);
                                            keywordsCreative.setValue(creativeIdChina);
                                            keywordsCreative.setInsertDate(new Date());
                                            keywordsCreative.setDataType(CREATIVE);
                                            keywordsCreativeRepository.save(keywordsCreative);
                                        }
                                    }
                                }
                            }
                        }

                        //批量更细
                        keywordsCreativeRepository.save(updateBatchList);

                    } catch (Exception e) {
                        logger.error("关键词和创意错误1:{}", e);

                        semRename.setStatus(2);
                        errorLine.append(exceptin_line).append(",");
                    }
                    if (errorLine.length() > 0) {
                        errorLine = new StringBuilder(errorLine.substring(0, errorLine.length() - 1));
                    }
                    if (semRename.getStatus() != 2) {
                        semRename.setStatus(1);
                    }
                    semRename.setErrorNum(errorLine.toString().split(",").length - 1);

                    semRename.setErrorLine(errorLine.toString());
                    semRenameRepository.save(semRename);
                }
            } catch (Exception e) {
                logger.error("关键词和创意错误2:{}", e);
            }
        } else {
            logger.info("另一任务正在批量替换关键词和创意id......");
        }
        logger.info("批量替换关键词和创意id..............//End {}", new Date());
    }


    private String dealKeyWordsActiveId(DataFormatter formatter, Cell cell, int linenum) {
        String creativeId = formatter.formatCellValue(cell);
        if (creativeId != null && creativeId.indexOf("E") > 0) {

            if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                double value = cell.getNumericCellValue();
                BigDecimal bd = new BigDecimal(value);
                creativeId = bd.toPlainString();

            } else {
                //包含科学技术 但单元格格式不对 无法处理时 不覆盖数据
                logger.info("LINE:{} CELL_TYPE：{},{},数据视为无效", linenum + 1, cell.getCellTypeEnum().name(), creativeId);
                creativeId = "0";

            }

        }
        return creativeId;
    }


}