package com.reyun.task;

import com.reyun.model.AdvancedFunctionApply;
import com.reyun.model.RecycleChannelAccount;
import com.reyun.repository.AdvancedFunctionApplyRepository;
import com.reyun.repository.RecycleChannelAccountRepository;
import com.reyun.service.impl.RecycleCommonServiceImpl;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.RedisUtilNew;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.Date;
import java.util.List;


/**
 * 回本分析失效账户邮件通知
 */
@Service
public class AutoRecycleAccountTask {
	private static Logger logger = LoggerFactory.getLogger(AutoRecycleAccountTask.class);
	private static int ACCOUNT_EMAIL_EXPIRE_PERIOD = 24 * 60 * 60;//此处以秒为单位
	private static int CHECK_ACCOUNTPAST_EXPIRE_PERIOD =  20 * 60;//此处以秒为单位
	private static int ADVANCEDPAST_EMAIL_EXPIRE_PERIOD = 24 * 60 * 60;//此处以秒为单位


	@Autowired
	private RecycleCommonServiceImpl recycleCommonService;
	@Autowired
	private RecycleChannelAccountRepository  recycleChannelAccountRepository;
	@Autowired
	private AdvancedFunctionApplyRepository advancedFunctionApplyRepository;



	/**
	 * 针对回本分析登陆无效的账号发送邮件重新授权
	 */
	public void autoInValidRecycleAccountEmail() {
		if (RedisUtilNew.getInstance().tryLock("AutoInValidRecycleAccountEmail", ACCOUNT_EMAIL_EXPIRE_PERIOD) != null) {

			logger.info("回本分析不可用账号邮件推送:................//Begin, 执行时间: {}", DateTime.now());
			List<RecycleChannelAccount> recycleChannelAccounts = recycleChannelAccountRepository.findDisableAccount();
			if(recycleChannelAccounts != null && recycleChannelAccounts.size() > 0){
				for(RecycleChannelAccount recycleChannelAccount : recycleChannelAccounts){
					recycleCommonService.sendDisableAcountEmail(recycleChannelAccount);
				}
			}
			logger.info("回本分析不可用账号邮件推送:................//End");
		}

	}


	/**
	 *  校验过期的账号,保留允许个数的渠道，其他置为禁用
	 */
	public void checkRecycleAccountPast() {
		if (RedisUtilNew.getInstance().tryLock("AutoCheckRecycleAccountPast", CHECK_ACCOUNTPAST_EXPIRE_PERIOD) != null) {

			logger.info("回本分析校验过期账号，停用相关渠道:................//Begin, 执行时间: {}", DateTime.now());

			//付费申请
			List<AdvancedFunctionApply> advancedFunctionApplyList = advancedFunctionApplyRepository.findInVaildData(Constant.ADVANCED_FUNCTION_MENU_RECYCLE);
			for(AdvancedFunctionApply advancedFunctionApply : advancedFunctionApplyList){
				List<BigInteger> cids = recycleChannelAccountRepository.findCidsByAccount(advancedFunctionApply.getRootParent());//去重后的渠道个数
				int channelNum = cids != null ? cids.size() : 0;
				if( channelNum > RecycleChannelAccount.CHANNEL_LIMIT){
					for(int i = RecycleChannelAccount.CHANNEL_LIMIT; i < cids.size(); i++){
						recycleChannelAccountRepository.disablePastRecycleChannelAccount(advancedFunctionApply.getRootParent(), cids.get(i).longValue());
					}
				}
			}
			logger.info("回本分析校验过期账号，停用相关渠道:................//End");
		}

	}


	/**
	 * 付费用户过期提醒
	 */
	public void advancedPastEmail() {
		if (RedisUtilNew.getInstance().tryLock("AutoAdvancedPastEmail", ADVANCEDPAST_EMAIL_EXPIRE_PERIOD) != null) {

			logger.info("回本分析付费用户过期提醒邮件推送:................//Begin, 执行时间: {}", DateTime.now());
			int expireOne = 7;
			int expireTwo = 30;
			Date sevenDay = DateUtil.addDays(new Date(), expireOne);
			Date thirtyDay = DateUtil.addDays(new Date(), expireTwo);
			String sevenStr = DateUtil.formatTime(sevenDay, "yyyy-MM-dd");
			String thirtyStr = DateUtil.formatTime(thirtyDay, "yyyy-MM-dd");

			//7天
			List<AdvancedFunctionApply> advancedFunctionApplies = advancedFunctionApplyRepository.findToExpireData(Constant.ADVANCED_FUNCTION_MENU_RECYCLE,sevenStr);
			if(advancedFunctionApplies != null){
				for(AdvancedFunctionApply advancedFunctionApply : advancedFunctionApplies){
					recycleCommonService.sendAdvancePastEmail(advancedFunctionApply, expireOne);
				}
			}

			//30天
			List<AdvancedFunctionApply> advancedFunctionApplieList = advancedFunctionApplyRepository.findToExpireData(Constant.ADVANCED_FUNCTION_MENU_RECYCLE,thirtyStr);
			if(advancedFunctionApplieList != null){
				for(AdvancedFunctionApply advancedFunctionApply : advancedFunctionApplieList){
					recycleCommonService.sendAdvancePastEmail(advancedFunctionApply, expireTwo);
				}
			}

			logger.info("回本分析付费用户过期提醒邮件推送:................//End");
		}

	}


}
