package com.reyun.task;

import com.reyun.model.WxApp;
import com.reyun.repository.WxAppRepository;
import com.reyun.service.impl.AppServiceImpl;
import com.reyun.util.DateUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;


/**
 * 处理wx_app到ddb失败的情况(多台服务器争抢资源)
 */
@Service
public class AutoSyncWxAppTask {
	private static Logger logger = LoggerFactory.getLogger(AutoSyncWxAppTask.class);
	private static int EXPIRE_PERIOD =  60 ;//此处以秒为单位
    private static int SYNC_TIME = 4;

	@Autowired
	private WxAppRepository wxAppRepository;
    @Autowired
	private AppServiceImpl appServiceImpl;

	public void run() {
		if (RedisUtilNew.getInstance().tryLock("AutoSyncWxAppTask", EXPIRE_PERIOD) != null) {

			logger.info("同步wxApp到ddb服务................//Begin, 执行时间: {}", DateTime.now());
//			String yestorday = DateUtil.getBeforeDays(1);//查询间隔一天的数据（防止创建appSecret同步失败，但是修改成功同步造成此处同步ddb错误数据的情况）
			List<WxApp> wxapps = wxAppRepository.getAllWxApps( SYNC_TIME);
			if(wxapps != null && wxapps.size() > 0){
				for (WxApp wxApp : wxapps){
					boolean pushSussess = appServiceImpl.insertOrUpdateWxAppToDDB(wxApp.getAppid(), wxApp.getWxAppid(), wxApp.getWxSecret());
					if (pushSussess){
                        wxAppRepository.updateToSuccess(wxApp.getId(), new Date());
					}else{
						wxAppRepository.updateToFailed(wxApp.getId(), new Date());
					}
				}
			}
			logger.info("同步wxApp到ddb服务................//End");
		}

	}
}
