package com.reyun.task;


import com.reyun.model.Account;
import com.reyun.repository.AccountRepository;
import com.reyun.repository.ConfigParamRepository;
import com.reyun.service.AutoTodayFlowService;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 *  计算当天用户的实时流量
 */
@Service
public class AutoTodayFlowTask {

    private static Logger logger = LoggerFactory.getLogger(AutoTodayFlowTask.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE*4 / 1000;

    @Autowired
    AutoTodayFlowService autoTodayFlowService;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    ConfigParamRepository configParamRepository;

    public void run() {

        long st = System.currentTimeMillis();
        logger.info("计算实时流量................//Begin, 执行时间: {}", DateTime.now());
        int accountNumb = 0;

        //上午七点暂停一次任务，防止和离线流量计算冲突
        DateTime dateTime = new DateTime();
        if ((dateTime.getHourOfDay() == 6 && dateTime.getMinuteOfHour() == 59) || (dateTime.getHourOfDay() == 7 && (dateTime.getMinuteOfHour() >= 0 && dateTime.getMinuteOfHour() <= 5))) {
            logger.info("计算实时流量暂停...... {}:{} 点暂停一次",dateTime.getHourOfDay(),dateTime.getMinuteOfHour() );
            return;
        }

        if (RedisUtilNew.getInstance().tryLock("AutoTodayFlowSuperAws", EXPIRE_PERIOD) != null) {
            try {
                List<Account> superAccount = accountRepository.findSuperAccount();

//               String real_time_user = configParamRepository.findParamsByKey("real_time_user");
//                if (!StringUtils.isEmpty(real_time_user)) {
//                    List<String> acc = new ArrayList<>();
//                    acc.addAll(Arrays.asList(real_time_user.split(",")));
//                    superAccount = accountRepository.findAllByEmails(acc);
//                }

                logger.info("计算实时流量......,用户数：{}",superAccount==null?0:superAccount.size());
                for (Account account : superAccount) {

                    DateTime pastDate = new DateTime(account.getPastDate());

                    if(Days.daysBetween(pastDate,new DateTime()).getDays() <= 0 ){
                        // 未过期用户 计算流量
                        try {
                            autoTodayFlowService.autoTodayFlowTask(account);
                            accountNumb++;

                        } catch (Exception e) {
                            logger.error("计算实时流量错误 "+account.getId() , e);
                        }
                    }
                }
            } catch (Exception e) {
                logger.error("计算实时流量",e);
            }
        } else {
            logger.info("其他任务正在计算实时流量......");
        }

        if(accountNumb>0){
            logger.info("计算实时流量................//End{}, 用时 {} 秒,更新{}个用户",
                    DateTime.now(),
                    (System.currentTimeMillis()-st)/1000,
                    accountNumb);
        }


    }

    public static void main(String[] args) {
//        System.out.println(Days.daysBetween(new DateTime("2019-02-25"),new DateTime()).getDays() );
        System.out.println(Days.daysBetween(new DateTime("2019-09-02"),new DateTime()).getDays());
    }
}
