package com.reyun.task;

import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.model.ExportReportInfo;
import com.reyun.repository.ExportReportInfoRepository;
import com.reyun.service.ExportReportInfoService;
import com.reyun.taskexecute.ExportKeywordReportThread;
import com.reyun.taskexecute.ExportReportThread;
import com.reyun.util.Constant;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by zxy on 17/4/27.
 * description:报表导出
 */
@Service
public class ExportReportTask {

    private static Logger logger = LoggerFactory.getLogger(ExportReportTask.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE * 1 / 1000;

    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;

    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    private static final String REPORT_KEYWORD = "bd_bykwid";

    @Autowired
    private ExportReportInfoService exportService;

    @Autowired
    private ExportReportInfoRepository exportReportInfoRepository;


//    @Resource(name = "exportThreadPoolTaskExecutor")
//    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    public void run(){

        logger.info("导出报表结果数据................//Begin, 执行时间: {}", DateTime.now());

        long exp_start = System.currentTimeMillis();
        if (RedisUtilNew.getInstance().tryLock(Constant.reportExportTask, EXPIRE_PERIOD) != null ) {

            //生成数据
            exportReport(null);

        } else {
            logger.info("ExportReportTask正在执行中......");
        }

        logger.info("导出报表结果数据................//End,耗时:{}",System.currentTimeMillis()-exp_start);
    }


    public void exportReport(Long id){

        List<ExportReportInfo> exportList;

        if(id!=null && id > 0){
           exportList = new ArrayList<>();
           exportList.add(exportReportInfoRepository.findOne(id));
        }else{
            exportList = exportService.findExport2Deal();
        }

        logger.info("报表导出,正在处理[{}条导出]",exportList.size());

        for (ExportReportInfo export : exportList) {

            //logger.info(export.getFileName());

            if(REPORT_KEYWORD.equals(export.getReportName())){
                //关键字导出特殊处理
                Thread keywordExportThread = new ExportKeywordReportThread(export);
                keywordExportThread.start();
//                threadPoolTaskExecutor.execute(new ExportKeywordReportThread(export));

            } else {
                Thread exportReportThread = new ExportReportThread(export);
                exportReportThread.start();
//                threadPoolTaskExecutor.execute(new ExportReportThread(export));

            }

        }

        //删除已过期文件
        List<ExportReportInfo> invalidExportInfoList = exportService.findInvalidExportList();
        for (ExportReportInfo invalidExport : invalidExportInfoList) {

            final String s3key = DOWNLOAD_PATH + invalidExport.getFileName();
            //删除文件
            if(ObjectStorageUtil.doesObjectExist(S3_DOWNLOAD_BUCKET, s3key)) {
                ObjectStorageUtil.deleteObject(S3_DOWNLOAD_BUCKET, s3key);
            }

            //跟新文件为失效状态
            exportService.updateStatus(invalidExport.getId(), DownloadStatusEnumType.INVALID.getCode());
        }
    }


}
