package com.reyun.task;

import com.reyun.util.AwsS3Util;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.ObjectStorageUtil;
import com.reyun.util.RedisUtilNew;
import org.apache.commons.lang.time.DateUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 * Created by mxq on 18/4/27.
 */
@Service
public class Log2S3Task {
    private static Logger logger = LoggerFactory.getLogger(Log2S3Task.class);
    private static int EXPIRE_PERIOD = (int) DateUtils.MILLIS_PER_MINUTE / 1000;

    public void run() {

        logger.info("同步日志到s3................//Begin, 执行时间: {}", DateTime.now());
        if (RedisUtilNew.getInstance().tryLock("AutoSyncLog", EXPIRE_PERIOD) != null) {

            String fileNameBefore = Constant.logPath + "/"
                    + DateUtil.getBeforeDays(7) + ".txt";
            File fileBefore = new File(fileNameBefore);
            if (fileBefore.exists()) {
                fileBefore.delete();
            }

            String fileName = Constant.logPath + "/"
                    + DateUtil.getBeforeDays(1) + ".txt";
            File file = new File(fileName);
            if (file.exists()) {
                InputStream in = null;
                try {
                    in = new FileInputStream(file);
                    ObjectStorageUtil.putObjectInStream(Constant.logBucket,
                            "ds=" + DateUtil.getBeforeDays(1) + "/" + DateUtil.getBeforeDays(1) + ".txt", in, "txt",
                            file.length());
                } catch (IOException e) {
                    logger.error(e.getMessage(), e);
                } finally {
                    if (in != null) {
                        try {
                            in.close();
                        } catch (IOException e) {
                            logger.error(e.getMessage(), e);
                        }
                    }

                }
            }
            logger.info("同步日志到s3...............//End");
            try {
                Class.forName(Constant.hiveDriver);
                try {
                    String ds = DateUtil.getBeforeDays(1);
                    Connection connection = DriverManager.getConnection(Constant.hiveUrl, Constant.hiveName, "");
                    String sql = "alter table tkiodefault.userlog add partition (ds='" + ds + "')";
                    PreparedStatement statement = connection.prepareStatement(sql);
                    statement.execute();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }

        }else {
            logger.info("正在同步......");
        }
    }
}
