package com.reyun.task;

import com.reyun.model.*;
import com.reyun.repository.*;
import com.reyun.util.Constant;
import com.reyun.util.RedisUtilNew;
import com.reyun.util.TkioStreamDBUtil;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * @author: zhangyao
 * @create: 2019-04-26 17:30
 **/
@Component
public class SyncGdtCampaignTask {

    private static Logger logger = LoggerFactory.getLogger(SyncGdtCampaignTask.class);
    private static int EXPIRE_PERIOD = 120;

    @Autowired
    private CampaignRepository campaignRepository;

    @Autowired
    private ThirdAccountAuthRepository thirdAccountAuthRepository;

    @Autowired
    private DataAuthRepository dataAuthRepository;

    @Autowired
    private AppRepository appRepository;

    @Autowired
    private AuthRepository authRepository;

    public void run() {

        if (RedisUtilNew.getInstance().tryLock("SyncGdtCampaignAuthTask", EXPIRE_PERIOD) != null) {
            logger.info("广点通账号自动授权................//开始, 执行时间: {}", DateTime.now());
            if (Constant.awsneed) {
                long l = System.currentTimeMillis();
                Connection conn = null;
                PreparedStatement pt = null;
                ResultSet rs = null;
                String sql = null;
                int i = 0;
                try {
                    List<ThirdAccountAuth > all = thirdAccountAuthRepository.findAll();
                    Map<String, List<String>> appidMap = new HashMap<>();
                    conn = TkioStreamDBUtil.newInstance().getConn();
                    for (ThirdAccountAuth thirdAccountAuth : all) {
                        List<String> appids = new ArrayList<>();
                        String third_account = thirdAccountAuth.getThird_account();
                        if(!appidMap.containsKey(third_account)){
                            sql = "select distinct appid from tkio_stream.tkio_rpt_source_analysis_day where sp_account_number = '"+ third_account +"' and cid = 5305";
                            pt = conn.prepareStatement(sql);
                            rs = pt.executeQuery();
                            while(rs.next()){
                                appids.add(rs.getString("appid"));
                            }
                            if(CollectionUtils.isEmpty(appids)){
                                sql = "select distinct appid from tkio.tkio_rlt_account_history_info where sp_account_number = '"+third_account+"'";
                                conn = TkioStreamDBUtil.newInstance().getConn();
                                pt = conn.prepareStatement(sql);
                                while(rs.next()){
                                    appids.add(rs.getString("appid"));
                                }
                            }
                            appidMap.put(third_account,appids);
                        }
                        for (String appid : appidMap.get(third_account)) {
                            //System.out.println("appsize = " + appids.size());
                            App app = this.appRepository.findByAppkey(appid);
                            Auth auth = authRepository.findAuthByAccountAndApp(thirdAccountAuth.getAccount(), app.getId());
                            if(auth!=null){
                                List<String> dataAuths = this.dataAuthRepository.getDataAuthByAppAndAllCampaign(app.getId());
                                if(!dataAuths.contains(new BigInteger(thirdAccountAuth.getAccount().toString()))){
                                    sql = "SELECT DISTINCT campaignid FROM (SELECT campaignid FROM tkio.tkio_rlt_account_history_info WHERE sp_account_number= "+third_account+" and appid = '"+app.getAppkey()+"' and cid = 5305 UNION ALL SELECT campaignid FROM tkio_stream.tkio_rpt_source_analysis_day WHERE sp_account_number="+third_account+" and appid = '"+app.getAppkey()+"' AND cid = 5305 AND sp_account_number!=-1) t";
                                    pt = conn.prepareStatement(sql);
                                    rs = pt.executeQuery();
                                    List<String> camids = new ArrayList<>();
                                    while(rs.next()){
                                        camids.add(rs.getString("campaignid"));
                                    }
                                    List<String> caids = this.campaignRepository.findIdByCam(camids);
                                    if(!CollectionUtils.isEmpty(caids)) {
                                        List<Campaign> campaigns = this.campaignRepository.findByDataAuth(app.getId(), thirdAccountAuth.getAccount(), caids);
                                        for (Campaign campaign : campaigns) {
                                            DataAuth dataAuth = new DataAuth();
                                            dataAuth.setApp(app.getId());
                                            dataAuth.setCampaign(campaign.getId());
                                            dataAuth.setChannel(campaign.getChannel());
                                            dataAuth.setAccount(thirdAccountAuth.getAccount());
                                            dataAuth.setModifyTime(new Date());
                                            dataAuth.setAllCampaign(false);
                                            dataAuth.setDelFlag(false);
                                            dataAuth.setChannelPermit(thirdAccountAuth.getChannel_permit());
                                            dataAuthRepository.save(dataAuth);
                                            i++;
                                        }
                                    }
                                }
                            }
                        }

                    }
                } catch (SQLException e) {
                    logger.error("广点通账号自动授权出错",e.getMessage());
                } finally {
                    TkioStreamDBUtil.newInstance().release(rs, pt, conn);
                }
                logger.info("广点通账号自动授权执行完毕,共授权data_auth{}条,用时{}s",i,(System.currentTimeMillis()-l)/1000);
            }
        } else {
            logger.info("广点通账号自动授权 另一任务正在执行 实时同步 {}", DateTime.now());
        }

    }
}
