package com.reyun.taskexecute;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.context.AppUtils;
import com.reyun.dic.DownloadStatusEnumType;
import com.reyun.dic.FunctionEnumType;
import com.reyun.dic.ReportEnumType;
import com.reyun.model.Account;
import com.reyun.model.ExportReportInfo;
import com.reyun.model.ReportListResult;
import com.reyun.service.AuthService;
import com.reyun.service.ExportReportInfoService;
import com.reyun.service.ReportService;
import com.reyun.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.List;
import java.util.Map;

/**
 * Created by sunhao on 17/7/7.
 * desc:导出关键字线程(专门处理关键字导出)
 */
public class ExportKeywordReportThread extends Thread {

    private static final String TEMP_FILE_DIR = "./temp";

    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;

    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    private static final String DATE_KEYWORD_EXPORT = "bd_bykwid_date";

    private static final String GAP_CONTENT = "\r\n关键字详情\r\n";

    private static final String CONDITION_START_DATE = "startdate";

    private static final String CONDITION_END_DATE = "enddate";

    private static final String CONDITION_IS_CACHE = "iscache";


    private static final Logger logger = LoggerFactory.getLogger(ExportKeywordReportThread.class);

    private ReportService reportService = AppUtils.getApplicationContext().getBean(ReportService.class);

    private ExportReportInfoService exportService = AppUtils.getApplicationContext().getBean(ExportReportInfoService.class);

    private AuthService authService = AppUtils.getApplicationContext().getBean(AuthService.class);

    private ExportReportInfo export;

    public ExportKeywordReportThread(ExportReportInfo export) {
        this.export = export;
//        this.setName("ExportKeywordReportThread");
    }


    @Override
    public void run() {

        boolean success = true;

        //更新为下载中的状态
        updateStatus(export.getId(), DownloadStatusEnumType.DOWNLOADING.getCode());

        //1,新建临时文件
        final String fileName = export.getFileName();
        File file = new File(TEMP_FILE_DIR + fileName);

        BufferedWriter bufferedWriter = null;
        FileWriter fileWriter = null;

        try {

            //目录不存在,新建目录
            if (!file.getParentFile().exists() && !file.mkdir()) {
                logger.error("文件夹创建失败");
                return;
            }

            //文件存在
            if (!file.exists() && !file.createNewFile()) {

                logger.error("文件创建失败");

            } else {
                fileWriter = new FileWriter(file);
                bufferedWriter = new BufferedWriter(fileWriter);
            }


            //2,获取基础关键字数据(原始导出),写入临时文件
            Map<String, String> conditions = ExportReportUtil.changeCondition2Map(export.getConditions());
            conditions.put(CONDITION_IS_CACHE, Constant.iscache);

            //语种类型判断
            conditions.put(Constant.LOCALE_KEY, export.getLanguageType());
            //end

            Account account = authService.getSubAccountAuthById(export.getAccount());

            if (export.getFunctionType().equals(FunctionEnumType.REPORT.getKey())) {


                Object response = reportService.report(export.getApp(), export.getReportName(), account, conditions);

                //表
                if (export.getReportType().equals(ReportEnumType.LIST.getCode())) {

                    ReportListResult reportListResult = new ObjectMapper().convertValue(response, ReportListResult.class);
                    StringBuffer sb = ExportReportUtil.getExportContent(reportListResult, export.getSubType());

                    //写入文件
                    if (null != bufferedWriter) {
                        bufferedWriter.write(sb.toString());
                        bufferedWriter.write(GAP_CONTENT);
                        bufferedWriter.flush();
                    }
                }
            }

            //3,按天获取每天的数据,append 到临时文件
            String startDate = conditions.get(CONDITION_START_DATE);
            String endDate = conditions.get(CONDITION_END_DATE);

            List<String> dateList = DateUtil.getDateInterval(startDate, endDate);

            boolean isNeedTableHead = true;

            for (String date : dateList) {

                conditions.put(CONDITION_START_DATE, date);
                conditions.put(CONDITION_END_DATE, date);


                Object response = reportService.report(export.getApp(), DATE_KEYWORD_EXPORT, account, conditions);

                ReportListResult reportListResult = new ObjectMapper().convertValue(response, ReportListResult.class);

                StringBuffer sb = ExportReportUtil.getExportContent(reportListResult, export.getSubType());

                //写入文件
                if (null != bufferedWriter) {

                    if (!isNeedTableHead) {
                        sb.delete(0, sb.indexOf("\r\n") + 2);
                    }

                    isNeedTableHead = false;
                    bufferedWriter.append(sb.toString());
                    bufferedWriter.flush();
                }
            }

        } catch (IOException e) {

            logger.error(e.getMessage());

            e.printStackTrace();

            success = false;

        } finally {

            IOUtil.close(fileWriter);
            IOUtil.close(bufferedWriter);
        }

        //4,上传临时文件到s3;
        if (success) {
            success = this.uploadFileToS3(fileName);
        }

        //更新导出记录
        if (success) {
            updateStatus(export.getId(), DownloadStatusEnumType.COMPLETE.getCode());
        } else {
            updateStatus(export.getId(), DownloadStatusEnumType.FAILED.getCode());
        }

        //删除本地临时文件
        if(file.delete()){
            System.out.println("--关键字导出"+fileName+"结束--");
        }
    }

    private void updateStatus(Long id, String status) {
        exportService.updateStatus(id, status);
    }

    /**
     * 上传数据到S3
     */
    private boolean uploadFileToS3(String fileName) {
        boolean result = true;
        final String s3key = DOWNLOAD_PATH + fileName;
        FileInputStream fileInputStream = null;
        try {
            File file = new File(TEMP_FILE_DIR + fileName);
            fileInputStream = new FileInputStream(file);
            ObjectStorageUtil.putObjectInStream(S3_DOWNLOAD_BUCKET, s3key, fileInputStream, "csv", file.length());
        } catch (IOException e) {
            IOUtil.close(fileInputStream);
            logger.error(e.getMessage());
            result = false;
        }
        return result;
    }
}
