package com.reyun.taskexecute;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.reyun.context.AppUtils;
import com.reyun.dic.*;
import com.reyun.model.*;
import com.reyun.service.*;
import com.reyun.util.*;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by zxy on 17/4/27.
 * description:生成报表导出结果文件
 */
public class ExportReportThread extends Thread {

    private static final Logger logger = LoggerFactory.getLogger(ExportReportThread.class);

    private ExportReportInfo export;

    private ExportReportInfoService exportService = AppUtils.getApplicationContext().getBean(ExportReportInfoService.class);

    private ReportService reportService = AppUtils.getApplicationContext().getBean(ReportService.class);

    private AuthService authService = AppUtils.getApplicationContext().getBean(AuthService.class);

    private CampaignService campaignService = AppUtils.getApplicationContext().getBean(CampaignService.class);

    private CampaignGroupService campaignGroupService = AppUtils.getApplicationContext().getBean(CampaignGroupService.class);

    private ChannelService channelService = AppUtils.getApplicationContext().getBean(ChannelService.class);

    private ExportGdtService exportGdtService = AppUtils.getApplicationContext().getBean(ExportGdtService.class);

    private AccountService accountService = AppUtils.getApplicationContext().getBean(AccountService.class);

    private static final String S3_DOWNLOAD_BUCKET = Constant.bucket;

    private static final String DOWNLOAD_PATH = "trackingio/reportdownload/";

    private static final String VIEW_TYPE_DAY = "day";
    private static final String VIEW_TYPE_WEEK = "week";
    private static final String VIEW_TYPE_MONTH = "month";

    private static final String REPORT_NAME_CAMPAIGN = "campaigninfo_bycampgroup";
    private static final String REPORT_NAME_CAMPAIGN_H5 = "campaigninfo_h5_bycampaign";
    private static final String SUB_TYPE_CAMPAIGN = "campaign";
    private static final String SUB_TYPE_CAMPGROUP = "campgroup";
    private static final String SUB_TYPE_CAMPAIGN_H5 = "campaign_h5";
    private static final String SPECIAL_REPORT_NAME = "campaigninfo_bydatecampaign";
    private static final String REPORT_NAME_ROI_CAMPAIGN = "roi_bycampaign";
    private static final String REPORT_NAME_ROI_CHANNEL = "roi_bychannel";
    private static final String REPORT_NAME_ROI_INTERVAL = "roi_byintervalall";
    private static final String REPORT_NAME_ROI_INTERVAL_PAY = "roi_byinterval";


    private static final String CAMPAIGN_SPECIAL_REPORT_NAME = "campaigninfo_bydatecampaign";
    private static final String CAMPAIGNGROUP_SPECIAL_REPORT_NAME = "campaigninfo_bydatecampaigngroup";
    private static final String H5_SPECIAL_REPORT_NAME = "campaigninfo_h5_bydatecampaign";

    private static final Map<String, String> channelExport = new HashMap() {
        {
            put("campaigninfo_bychannel", "campaigninfo_bydatechannel");
            put("wx_campaigninfo_bychannel", "wx_campaigninfo_bydatechannel");
        }
    };

    public ExportReportThread(ExportReportInfo export) {
        this.export = export;
        //this.setName("ExportReportThread");
    }

    @Override
    public void run() {

        boolean success = true;

        String valsize = null;

        Map<String, Long> timeflag = new HashMap<>();
        timeflag.put("start", System.currentTimeMillis());
        List<String> exportNames = new ArrayList<>();

        String export_query_start = new DateTime().toString("yyyy-MM-dd HH:mm:ss");

        try {

            Map<String, String> conditions = ExportReportUtil.changeCondition2Map(export.getConditions());
            conditions.put("iscache", Constant.iscache);

            //语种类型判断
            conditions.put(Constant.LOCALE_KEY, export.getLanguageType());
            //end

            StringBuffer sb = new StringBuffer();
            Account account = authService.getSubAccountAuthById(export.getAccount());

            timeflag.put("account", account.getId());
            exportNames.add(export.getReportName());


            if ("funnel_user_event_list".equals(export.getReportName())) {
                export.setReportName("funnel_user_event_campain_list");
                Object response = reportService.report(export.getApp(), export.getReportName(), account, conditions);
                ReportListResult reportListResult = new ObjectMapper().convertValue(response, ReportListResult.class);
                sb = ExportReportUtil.getExportContent(reportListResult, export.getSubType());

            } else if (export.getFunctionType().equals(FunctionEnumType.REPORT.getKey())) {
                logger.info("ExportReportThread查询报表：name={}", export.getReportName());
                long start = System.currentTimeMillis();
                //必传参数：reportname、startdate、enddate
                Object response = reportService.report(export.getApp(), export.getReportName(), account, conditions);
                long end = System.currentTimeMillis();
                logger.info("ExportReportThread查询报表结束:name={},app={},耗时:{}", export.getReportName(), export.getApp(), (end - start));
                printTime(timeflag, true, export.getReportName(), "get");


                //图和表
                if (export.getReportType().equals(ReportEnumType.CHART.getCode())) {

                    ReportChartResult reportChartResult = new ObjectMapper().convertValue(response, ReportChartResult.class);
                    sb = getExportContent(reportChartResult, export, conditions);

                    if (reportChartResult.getVal() != null) {
                        valsize = reportChartResult.getVal().size() + "";
                    }

                    printTime(timeflag, true, export.getReportName(), "parse");

                } else {

                    ReportListResult reportListResult;
                    boolean isgdt = "campaigninfo_bycampgroup".equals(export.getReportName()) && exportGdtService.find(export.getApp()) != null;
                    //流利说投放账号维度下载写死
                    if ("campaigninfo_bycampgroup".equals(export.getReportName()) && accountService.findOne(export.getAccount()).getRootParent() == 18017L)
                        isgdt = true;
                    if (isgdt) {
                        response = reportService.report(export.getApp(), "campaigninfo_bycampgroup_cid", account, conditions);
                        reportListResult = new ObjectMapper().convertValue(response, ReportListResult.class);
                        Iterator<Map<String, Object>> iterator = reportListResult.getVal().iterator();
                        while (iterator.hasNext()) {
                            Map<String, Object> next = iterator.next();
                            if (next.containsKey("cid")) {
                                if (next.get("cid").equals(5305L)) {
                                    iterator.remove();
                                } else {
                                    next.remove("cid");
                                }
                            }
                        }

                        conditions.put("viewtype", "account");
                        Object resp = reportService.report(export.getApp(), "third_account_byspaccount", account, conditions);
                        ReportListResult rest = new ObjectMapper().convertValue(resp, ReportListResult.class);
                        Iterator<Map<String, Object>> it = rest.getVal().iterator();
                        while (it.hasNext()) {
                            Map<String, Object> next = it.next();
                            if ("汇总".equals(next.get("sp_account_number"))) {

                                if (rest.getVal().size() == 1) {

                                    rest.setVal(new ArrayList<>());
                                } else {
                                    it.remove();
                                }

                            } else {
                                next.put("campaignid", next.get("sp_account_number"));
                                next.put("channelname", "广点通");
                                next.remove("sp_account_number");
                            }
                        }

                        reportListResult.getVal().addAll(rest.getVal());
                        reportListResult.getColumnkey().remove("cid");
                        reportListResult.getName().remove("cid");

                    } else {
                        reportListResult = new ObjectMapper().convertValue(response, ReportListResult.class);
                    }

                    sb = ExportReportUtil.getExportContent(reportListResult, export.getSubType());

                    printTime(timeflag, true, export.getReportName(), "parse");

                    if (reportListResult.getVal() != null) {
                        valsize = reportListResult.getVal().size() + "";
                    }

                    //推广活动详情特殊处理,新加H5导出(20170809)
                    boolean isNormalCampaignInfoByCampaign = REPORT_NAME_CAMPAIGN.equals(export.getReportName()) && SUB_TYPE_CAMPAIGN.equals(export.getSubType()) || SUB_TYPE_CAMPGROUP.equals(export.getSubType());
                    boolean isH5CampaignInfoByCampaign = REPORT_NAME_CAMPAIGN_H5.equals(export.getReportName()) && SUB_TYPE_CAMPAIGN_H5.equals(export.getSubType());


                    String channelExportName = channelExport.get(export.getReportName());

                    // export.getDayDeatailopne() 是否开启日期详情标识

                    if (export.getDayDeatailopne() == 1 && (isNormalCampaignInfoByCampaign || isH5CampaignInfoByCampaign || channelExportName != null)) {

                        String reportName;

                        boolean isDsCampGroup = false;

                        if (channelExportName != null) {
                            // 渠道维度报表导出 处理
                            reportName = channelExportName;
                            // 按日期解析报表
                            conditions.put("viewtype", DataViewEnumType.CHANNLE_DATE.getCode());
                            conditions.put("reportname",reportName);

                        } else {
                            // 按日期解析报表
                            //活动组按照日期,先这样，回头再优化代码
                            if (conditions.get("viewtype").equals(SUB_TYPE_CAMPGROUP)) {

                                conditions.put("viewtype", DataViewEnumType.CAMPGROUP_DATE.getCode());
                                isDsCampGroup = true;
                                reportName = CAMPAIGNGROUP_SPECIAL_REPORT_NAME;

                            } else {

                                conditions.put("viewtype", DataViewEnumType.CAMPAIGN_DATE.getCode());
                                reportName = isNormalCampaignInfoByCampaign ? CAMPAIGN_SPECIAL_REPORT_NAME : H5_SPECIAL_REPORT_NAME;
                            }
                        }
                        ReportListResult listResult;
                        if (isgdt) {

                            Object responseObject = reportService.report(export.getApp(), "campaigninfo_bydatecampaign_cid", account, conditions);

                            printTime(timeflag, true, reportName, "get");

                            listResult = new ObjectMapper().convertValue(responseObject, ReportListResult.class);
                            Iterator<Map<String, Object>> iterator = listResult.getVal().iterator();
                            while (iterator.hasNext()) {
                                Map<String, Object> next = iterator.next();
                                if (next.containsKey("cid")) {
                                    if (next.get("cid").equals(5305L)) {
                                        iterator.remove();
                                    } else {
                                        next.remove("cid");
                                    }
                                }
                            }
                            conditions.put("viewtype", "account");
                            Object report = reportService.report(export.getApp(), "campaigninfo_bydatecampaign5305", account, conditions);
                            ReportListResult vlistResult = new ObjectMapper().convertValue(report, ReportListResult.class);
                            List<Map<String, Object>> val = vlistResult.getVal();
                            Iterator<Map<String, Object>> it = val.iterator();
                            while (it.hasNext()) {
                                Map<String, Object> next = it.next();
                                if (next.get("sp_account_number").equals("汇总")) {
//                                    it.remove();
                                    if (vlistResult.getVal().size() == 1) {
                                        vlistResult.setVal(new ArrayList<>());
                                    } else {
                                        it.remove();
                                    }

                                } else {
                                    next.put("campaignid", next.get("sp_account_number"));
                                    next.put("channelname", "广点通");
                                    next.remove("sp_account_number");
                                }
                            }
                            listResult.getName().remove("cid");
                            listResult.getColumnkey().remove("cid");
                            listResult.getVal().addAll(vlistResult.getVal());

                        } else {

                            Object responseObject = reportService.report(export.getApp(), reportName, account, conditions);
                            logger.info("查询报表结果:reportName={},responseObject={}", reportName, responseObject);
                            printTime(timeflag, true, reportName, "get");

                            listResult = new ObjectMapper().convertValue(responseObject, ReportListResult.class);
                        }
                        sb.append(isEnLanguage(export) ? "\r\nCampaign Info\r\n" : "\r\n推广详情\r\n");

                        String subtype = isDsCampGroup ? "dsgroup" : export.getSubType();

                        if(conditions.containsKey("reportname")){
                            listResult.setReportname(Arrays.asList(conditions.get("reportname")));
                        }

                        sb.append(ExportReportUtil.getExportContent(listResult, subtype));

                        printTime(timeflag, true, reportName, "parse");

                        exportNames.add(reportName);

                    } else if (REPORT_NAME_ROI_CAMPAIGN.equals(export.getReportName()) && conditions.containsKey("subType")) {
                        //注收比特殊处理
                        if (isEnLanguage(export)) {
                            //英文语言
                            sb.append("\r\ninstall/payment list,").append("Billing Cycle：" + conditions.get("pay_interval")).append(",").
                                    append("Payment Start Date：" + conditions.get("pay_startdate")).append(",").
                                    append("Payment End Date：" + conditions.get("pay_enddate"));
                            sb.append("\r\nCampaign,");
                        } else {
                            sb.append("\r\n激活/付费列表,").append("付费周期：" + conditions.get("pay_interval")).append(",").
                                    append("付费起始时间：" + conditions.get("pay_startdate")).append(",").
                                    append("付费结束时间：" + conditions.get("pay_enddate"));
                            sb.append("\r\n推广活动,");
                        }

                        boolean isFirst = true;
                        List<Campaign> campaignList = new ArrayList<>();
                        if (conditions.containsKey("cgroupid")) {
                            Long cgroupid = Long.parseLong(conditions.get("cgroupid"));
                            List<Long> ids = campaignGroupService.listCamCamgroupMapByGroupid(cgroupid);
                            if (ValidateUtil.isValid(ids)) {
                                campaignList = campaignService.findById(ids);
                            }
                        } else {
                            campaignList = authService.getAuthCampaignListByAccount(export.getAccount(), export.getApp());
                            Campaign ca = new Campaign();
                            ca.setCampaignid("_default_");
//                            ca.setName("自然量");
                            ca.setName(isEnLanguage(export) ? "Organic" : "自然量");
                            campaignList.add(ca);
                        }
                        if (ValidateUtil.isValid(campaignList)) {
                            StringBuffer idSb = new StringBuffer();
                            for (int i = 0; i < campaignList.size(); i++) {
                                if (i == (campaignList.size() - 1)) {
                                    idSb.append(campaignList.get(i).getCampaignid()).append(",");
                                } else {
                                    idSb.append(campaignList.get(i).getCampaignid()).append(",");
                                }
                            }

                            List<Channel> channelList = new ArrayList<>();
                            channelList = authService.getAuthChannelListByApp(export.getAccount(), export.getApp());
                            Channel ch = new Channel();
                            ch.setId(-1l);
//                            ch.setName("自然量");
                            ch.setName(isEnLanguage(export) ? "Organic" : "自然量");
                            channelList.add(ch);

                            Map<String, String> channelMap = new HashMap<>();
                            if (ValidateUtil.isValid(channelList)) {
                                for (int i = 0; i < channelList.size(); i++) {
                                    channelMap.put(channelList.get(i).getId().toString(), channelList.get(i).getName());
                                }
                            }
                            conditions.put("campaignid", idSb.toString());
                            Object newResponse = reportService.report(export.getApp(), REPORT_NAME_ROI_INTERVAL, account, conditions);

                            printTime(timeflag, true, REPORT_NAME_ROI_INTERVAL, "get");

                            ReportListResult newReportListResult = new ObjectMapper().convertValue(newResponse, ReportListResult.class);
                            StringBuffer newSb = ExportReportUtil.getRoiExportContent(newReportListResult, conditions, channelMap);
                            sb.append(newSb);
                        }

                        printTime(timeflag, true, export.getReportName(), "parse");

                    } else if (REPORT_NAME_ROI_CHANNEL.equals(export.getReportName()) && conditions.containsKey("subType")) {

                        if (isEnLanguage(export)) {
                            sb.append("\r\ninstall/payment list,").append("Billing Cycle：" + conditions.get("pay_interval")).append(",").
                                    append("Payment Start Date：" + conditions.get("pay_startdate")).append(",").
                                    append("Payment End Date：" + conditions.get("pay_enddate"));
                            sb.append("\r\nPartner,");
                        } else {
                            sb.append("\r\n激活/付费列表,").append("付费周期：" + conditions.get("pay_interval")).append(",").
                                    append("付费起始时间：" + conditions.get("pay_startdate")).append(",").
                                    append("付费结束时间：" + conditions.get("pay_enddate"));
                            sb.append("\r\n渠道,");
                        }


                        List<Channel> channelList = new ArrayList<>();
                        if (conditions.containsKey("cid")) {
                            String cidStr = conditions.get("cid");
                            String[] cidArr = cidStr.split(",");
                            if (ValidateUtil.isValid(cidArr)) {
                                List<Long> cidList = new ArrayList<>();
                                for (String cid : cidArr) {
                                    Long id = Long.parseLong(cid);
                                    cidList.add(id);
                                }
                                channelList = channelService.listChannelByIds(cidList);
                            }
                            if (cidStr.contains("-1")) {
                                Channel ch = new Channel();
                                ch.setId(-1l);
//                                ch.setName("自然量");
                                ch.setName(isEnLanguage(export) ? "Organic" : "自然量");
                                channelList.add(ch);
                            }
                        } else {
                            channelList = authService.getAuthChannelListByApp(export.getAccount(), export.getApp());
                            Channel ch = new Channel();
                            ch.setId(-1l);
//                            ch.setName("自然量");
                            ch.setName(isEnLanguage(export) ? "Organic" : "自然量");
                            channelList.add(ch);
                        }
                        Map<String, String> channelMap = new HashMap<>();
                        if (ValidateUtil.isValid(channelList)) {
                            StringBuffer idSb = new StringBuffer();
                            for (int i = 0; i < channelList.size(); i++) {
                                channelMap.put(channelList.get(i).getId().toString(), channelList.get(i).getName());
                                if (i == (channelList.size() - 1)) {
                                    idSb.append(channelList.get(i).getId()).append(",");
                                } else {
                                    idSb.append(channelList.get(i).getId()).append(",");
                                }
                            }
                            conditions.put("cid", idSb.toString());
                            Object newResponse = reportService.report(export.getApp(), REPORT_NAME_ROI_INTERVAL, account, conditions);
                            ReportListResult newReportListResult = new ObjectMapper().convertValue(newResponse, ReportListResult.class);
                            StringBuffer newSb = ExportReportUtil.getRoiExportContent(newReportListResult, conditions, channelMap);
                            sb.append(newSb);
                        }

                        printTime(timeflag, true, export.getReportName(), "parse");
                    }
                }

            }

            printTime(timeflag, true, export.getReportName(), "parse");
            //上传到S3服务器
            uploadFileToS3(sb, export.getFileName());

            printTime(timeflag, true, export.getReportName(), "upload");

        } catch (Exception e) {
            success = false;
            logger.error("export_error", e);
            logger.error("report export_error, reportdata size:{} , exportinfo :{}", valsize, export.toString());
            logger.info("生成报表失败:", e);
        }
        logger.info("ExportReportThread导出报表操作结束:export={},success={}", JSON.toJSONString(export),success);
        //更新导出记录
        if (success) {
            updateStatus(export.getId(), DownloadStatusEnumType.COMPLETE.getCode());
        } else {
            updateStatus(export.getId(), DownloadStatusEnumType.FAILED.getCode());
        }

        printTime(timeflag, true, export.getReportName(), "end");

        String export_query_end = new DateTime().toString("yyyy-MM-dd HH:mm:ss");

        for (String rpname : exportNames) {

            long total = (timeflag.get(rpname + "_end") == null ? 0 : timeflag.get(rpname + "_end")) +
                    (timeflag.get(rpname + "_get") == null ? 0 : timeflag.get(rpname + "_get")) +
                    (timeflag.get(rpname + "_parse") == null ? 0 : timeflag.get(rpname + "_parse")) +
                    (timeflag.get(rpname + "_upload") == null ? 0 : timeflag.get(rpname + "_upload"));

            if (total > 1000 * 30) {

                logger.info("\nEXPORT_ID[{}],导出报表[{}],获取数据[{}]ms解析数据[{}]ms上传数据[{}]ms总用时[{}]ms,ACCOUNT[{}],开始时间[{}],结束时间[{}]",
                        export.getId(), export.getReportName(),
                        timeflag.get(rpname + "_get"),
                        timeflag.get(rpname + "_parse"),
                        timeflag.get(rpname + "_upload"), total,
                        timeflag.get("account"), export_query_start, export_query_end);
            }

        }
    }


    private long printTime(Map<String, Long> data, boolean startNew, String exportName, String type) {

        long tm = System.currentTimeMillis() - data.get("start");
        if (startNew) {
            data.put("start", System.currentTimeMillis());
        }
        data.put(exportName + "_" + type, tm);
        data.put(exportName, 0L);
        return tm;
    }

    /**
     * 根据日周月获取开始时间
     * created by sunhao 20170510
     */
    private String getStartDateByViewType(String startDate, String viewType) {

        String resultStartDate = startDate;

        switch (viewType) {
            case VIEW_TYPE_DAY:
                resultStartDate = startDate;
                break;
            case VIEW_TYPE_WEEK:
                resultStartDate = DateUtil.getFirstDayStrOfWeek(DateUtil.parseDate(startDate));
                break;
            case VIEW_TYPE_MONTH:
                resultStartDate = DateUtil.getFirstDayOfMonth(DateUtil.parseDate(startDate));
                break;
        }

        return resultStartDate;
    }

    /**
     * 根据日周月获取结束时间
     * created by sunhao 20170510
     */
    private String getEndDateByViewType(String endDate, String viewType) {

        String resultEndDate = endDate;

        switch (viewType) {
            case VIEW_TYPE_DAY:
                resultEndDate = endDate;
                break;
            case VIEW_TYPE_WEEK:
                resultEndDate = DateUtil.getLastDayStrOfWeek(DateUtil.parseDate(endDate));
                break;
            case VIEW_TYPE_MONTH:
                resultEndDate = DateUtil.getLastDayOfMonth(DateUtil.parseDate(endDate));
                break;
        }

        return resultEndDate;
    }

    /**
     * 根据日周月获取时间间隔
     * created by sunhao 20170510
     */
    private int getIntervalByViewType(String startDate, String endDate, String viewType) {

        int interval = 0;

        switch (viewType) {
            case VIEW_TYPE_DAY:
                interval = DateUtil.getDateInterval(startDate, endDate).size();
                break;
            case VIEW_TYPE_WEEK:
                interval = DateUtil.getEveryWeek(startDate, endDate).size();
                break;
            case VIEW_TYPE_MONTH:
                interval = DateUtil.getEveryMonth(startDate, endDate).size();
                break;
        }

        return interval;
    }

    private void dealRetentionName(Map<String, List> result) {

        List<String> names = result.get("name");
        List<String> columnKey = result.get("columnkey");
        List<String> newColumnKey = new ArrayList<>();
        List<String> newName = new ArrayList<>();
        for (int i = 0; i < columnKey.size(); i++) {
            if (i > 1) {
                newColumnKey.add(2 * i - 2, columnKey.get(i));
                newName.add(2 * i - 2, names.get(i));
                newColumnKey.add(2 * i - 1, "rate_" + columnKey.get(i));
                newName.add(2 * i - 1, names.get(i) + "留存率");
            } else {
                newColumnKey.add(i, columnKey.get(i));
                newName.add(i, names.get(i));
            }
        }
        result.put("name", newName);
        result.put("columnkey", newColumnKey);
    }

    /**
     * 处理报表中带rate的名字
     * created by sunhao 20170510
     */
    private void dealReportRateName(Map<String, List> result) {

        List<String> name = result.get("name");
        List<String> columnKey = result.get("columnkey");
        List<String> newColumnKey = new ArrayList<>();
        List<String> newName = new ArrayList<>();
        for (int i = 0; i < columnKey.size(); i++) {
            if (i != 0) {
                newColumnKey.add(2 * i - 1, columnKey.get(i));
                newName.add(2 * i - 1, name.get(i));
                newColumnKey.add(2 * i, "rate_" + columnKey.get(i));
                newName.add(2 * i, name.get(i) + "率");
            } else {
                newColumnKey.add(i, columnKey.get(i));
                newName.add(i, name.get(i));
            }
        }
        result.put("name", newName);
        result.put("columnkey", newColumnKey);
    }

    private StringBuffer getExportContent(Map<String, List> result) {
        StringBuffer sb = new StringBuffer();
        List<String> name = result.get("name");
        List<String> columnKey = result.get("columnkey");
        List<Map<String, Object>> value = result.get("val");

        if (!CollectionUtils.isEmpty(name)) {

            sb.append(String.join(",", name)).append("\r\n");
            for (Map<String, Object> v : value) {
                for (int i = 0; i < columnKey.size(); i++) {
                    sb.append(v.get(columnKey.get(i))).append(columnKey.get(i).startsWith("rate_") ? "%" : "")
                            .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                }
            }
        }

        return sb;
    }


    private StringBuffer getTotalExportContent(Map<String, List> result, String startDate, String endDate) {
        StringBuffer sb = new StringBuffer();
        List<String> name = result.get("name");
        List<String> columnKey = result.get("columnkey");
        List<Map<String, Object>> value = result.get("val");
        List<Map<String, Object>> totalValue = result.get("totalval");

        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy年MM月dd日");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd");
        Date parseStartDate = new Date();
        Date parseEndDate = new Date();
        try {
            parseStartDate = sdf2.parse(startDate);
            parseEndDate = sdf2.parse(endDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        String startDate1 = sdf1.format(parseStartDate);
        String endDate1 = sdf1.format(parseEndDate);
        String dateStr = startDate1 + "-" + endDate1;

        if (!CollectionUtils.isEmpty(name)) {

            sb.append(String.join(",", name)).append("\r\n");
            sb.append("汇总" + "\r\n");
            for (Map<String, Object> v : totalValue) {
                for (int i = 0; i < columnKey.size(); i++) {
                    if (i == 0) {
                        sb.append(dateStr + ",");
                    } else {
                        sb.append(v.get(columnKey.get(i))).append(columnKey.get(i).startsWith("rate_") ? "%" : "")
                                .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                    }
                }
            }
            sb.append("明细" + "\r\n");
            for (Map<String, Object> v : value) {
                for (int i = 0; i < columnKey.size(); i++) {
                    sb.append(v.get(columnKey.get(i))).append(columnKey.get(i).startsWith("rate_") ? "%" : "")
                            .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                }
            }
        }
        return sb;
    }


    private StringBuffer getExportContent(ReportChartResult reportChartResult, ExportReportInfo export, Map<String, String> conditions) {
        StringBuffer sb = new StringBuffer();
        List<String> column = new ArrayList<>();
        column.addAll(reportChartResult.getColumn());
        column.addAll(reportChartResult.getName());
        List<String> key = reportChartResult.getKey();
        List<Object[]> value = reportChartResult.getVal();

        Map<String, List<String>> extend_colums = matchExtendColum(export, conditions);

        boolean indexFirstAddquotes = false;

        if (!CollectionUtils.isEmpty(column)) {

            //sb.append(String.join(",", column));

            if ("手机型号".equals(column.get(0))) {
                indexFirstAddquotes = true;
            }

            Map<String, String> rateIndex = new HashMap();

            final int[] rateStart = {0};

            boolean blackTop5Report = false;

            if (export.getReportName().startsWith("black_top5_") || export.getReportName().startsWith("terminalattributes_")) {

                blackTop5Report = true;

                column.forEach(v -> {
                    if (sb.length() > 0) {
                        sb.append(",");
                    }
                    sb.append(v);
                    if (v.endsWith("占比") && rateStart[0] > 0) {
//                        sb.append(" (%)");
                        rateIndex.put(rateStart[0] - 1 + "", v);
                    }

                    rateStart[0]++;
                });

            } else {
                sb.append(String.join(",", column));
            }

            boolean hasExtends = false;

            if (extend_colums != null && extend_colums.get("title") != null && extend_colums.get("title").size() > 0) {

                extend_colums.get("title").forEach(v -> sb.append(",").append(v));
                sb.append("\r\n");
                hasExtends = true;

            } else {
                sb.append("\r\n");
            }

            for (int i = 0, vsize = value.size(); i < key.size(); i++) {

                if (indexFirstAddquotes) {
                    //处理名称中有逗号
                    sb.append("\"").append(key.get(i)).append("\"").append(",");
                } else {
                    sb.append(key.get(i)).append(",");
                }

                for (int j = 0; j < vsize; j++) {
                    if (blackTop5Report && rateIndex.get(j + "") != null) {

                        sb.append(value.get(j)[i]).append("%");
                    } else {

                        sb.append(value.get(j)[i]);
                    }

                    if (j == vsize - 1) {
                        if (hasExtends) {
                            extend_colums.get("data").forEach(v -> sb.append(",").append(v));
                            sb.append("\r\n");
                        } else {
                            sb.append("\r\n");
                        }
                    } else {
                        sb.append(",");
                    }

                }
            }
        }

        return sb;
    }


    private Map<String, List<String>> matchExtendColum(ExportReportInfo export, Map<String, String> conditions) {

        String reportname = export.getReportName();

        Map<String, List<String>> param = new HashMap<>();
        List<String> title = new ArrayList<>();
        List<String> data = new ArrayList<>();
        param.put("title", title);
        param.put("data", data);

        if (reportname.startsWith("black_top5_list_hijack_")) {
            title.add("劫持窗口期"); //劫持窗口期
            data.add(conditions.get("timeOrder"));
        } else if (reportname.startsWith("black_top5_list_device_")) {
            title.add("数据范围");
            data.add("promotion".equals(conditions.get("dataRange")) ? "推广量" : "自然量");
        } else if (reportname.startsWith("black_top5_list_corpse_")) {
            title.add("数据范围");
            data.add("promotion".equals(conditions.get("dataRange")) ? "推广量" : "自然量");
            title.add("观察周期");
            String level = conditions.get("level");
            if ("c1".equals(level)) {
                data.add("近一天");
            } else if ("c3".equals(level)) {
                data.add("近三天");
            } else if ("c7".equals(level)) {
                data.add("近七天");
            }
        }

        return param;

    }

    private void updateStatus(Long id, String status) {
        exportService.updateStatus(id, status);
    }

    /**
     * 上传文件到S3
     * created by sunhao 20170509
     */
    private void uploadFileToS3(StringBuffer fileContent, String fileName) throws IOException {
        logger.info("上传文件到S3:filename={}", fileName);
        final String s3key = DOWNLOAD_PATH + fileName;

        //删除重复文件
        if (ObjectStorageUtil.doesObjectExist(S3_DOWNLOAD_BUCKET, s3key)) {
            ObjectStorageUtil.deleteObject(S3_DOWNLOAD_BUCKET, s3key);
        }

        //上传文件
        if (fileContent != null) {
            if (fileContent.capacity() == 0) {
                fileContent.append("-无记录-");
            }
            ObjectStorageUtil.putObjectInString(S3_DOWNLOAD_BUCKET, s3key, fileContent.toString());
        }
    }


    private boolean isEnLanguage(ExportReportInfo export) {
        return StringUtils.equals(export.getLanguageType(), LanguageTypeEnum.ENGLISH.getCode());
    }


    public static void main(String[] args) {


        List<String> a = new ArrayList();

        a.add("汇总");

        Iterator<String> it = a.iterator();

        while (it.hasNext()) {

            String w = it.next();

            if (w.equals("汇总")) {

                it.remove();

            }

        }


//        DateTime start = new DateTime("2018-09-01");
//        DateTime end = new DateTime("2019-02-19");
//
//        int days = Days.daysBetween(start, end).getDays();
//
//        boolean addone = days % 10 != 0;
//
//        days = days / 10;
//
//        List dates = new ArrayList();
//
//        DateTime f = null;
//
//        for (int i = 1; i <= days; i++) {
//            Map d = new HashMap();
//            if (i == 1) {
//                d.put("start", start.toString("yyyy-MM-dd"));
//
//            } else {
//                d.put("start", f.plusDays(1).toString("yyyy-MM-dd"));
//            }
//
//            f = start.plusDays(10 * i);
//            d.put("end", f.toString("yyyy-MM-dd"));
//            dates.add(d);
//        }
//
//        if (addone) {
//            Map d = new HashMap();
//            d.put("start", f.plusDays(1).toString("yyyy-MM-dd"));
//            d.put("end", end.toString("yyyy-MM-dd"));
//            dates.add(d);
//        }
//
//        System.out.println(JSON.toJSONString(dates));
//
//
//        ExportReportParam param = new ExportReportParam();
//
//        if (param.getOpened() == 1) {
//            System.out.println("open ok");
//        }
    }


}
