package com.reyun.taskexecute;

import com.reyun.context.AppUtils;
import com.reyun.dic.LogEnumType;
import com.reyun.model.App;
import com.reyun.model.UserLog;
import com.reyun.repository.AppRepository;
import com.reyun.repository.UserLogRepository;
import com.reyun.service.UserLogService;
import com.reyun.util.Constant;
import com.reyun.util.DateUtil;
import com.reyun.util.IPAddrUtil;
import com.reyun.util.ValidateUtil;
import net.sf.json.JSONObject;
import net.sf.json.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;

import java.io.*;
import java.util.Date;
import java.util.List;

/**
 * Created by zxy on 2017/12/15.
 */
public class UserLogThread extends Thread
{
    protected Logger logger = LoggerFactory.getLogger(UserLogThread.class);

    private AppRepository appRepository = AppUtils.getApplicationContext().getBean(AppRepository.class);
    private UserLogRepository userLogRepository = AppUtils.getApplicationContext().getBean(UserLogRepository.class);
    private UserLogService userLogService = AppUtils.getApplicationContext().getBean(UserLogService.class);
    private String type;
    private String email;
    private boolean isMasterLogin;
    private String operaType;
    private String operaContent;
    private Object content;
    private List contentList;
    private String contentStr;
    private Long app;
    private String appName;
    private String objectType;
    private HttpServletRequest request;

    /**
     * 传入内容为对象
     * @param email
     * @param isMasterLogin
     * @param type
     * @param operaType
     * @param content
     * @param request
     * @param app
     * @param objectType
     */
    public UserLogThread(String email, boolean isMasterLogin, String type, String operaType, Object content, HttpServletRequest request, Long app, String objectType)
    {
        this.type = type;
        this.email = email;
        this.isMasterLogin = isMasterLogin;
        this.operaType = operaType;
        this.content = content;
        this.app = app;
        this.objectType = objectType;
        this.request = request;
    }

    /**
     * 传入内容为列表
     * @param email
     * @param isMasterLogin
     * @param type
     * @param operaType
     * @param content
     * @param request
     * @param app
     * @param objectType
     */
    public UserLogThread(String email, boolean isMasterLogin, String type, String operaType, List content, HttpServletRequest request, Long app, String objectType)
    {
        this.type = type;
        this.email = email;
        this.isMasterLogin = isMasterLogin;
        this.operaType = operaType;
        this.contentList = content;
        this.app = app;
        this.objectType = objectType;
        this.request = request;
    }

    /**
     * 传入内容为字符串
     * @param email
     * @param isMasterLogin
     * @param type
     * @param operaType
     * @param content
     * @param request
     * @param app
     * @param objectType
     */
    public UserLogThread(String email, boolean isMasterLogin, String type, String operaType, String content, HttpServletRequest request, Long app, String objectType)
    {
        this.type = type;
        this.email = email;
        this.isMasterLogin = isMasterLogin;
        this.operaType = operaType;
        this.contentStr = content;
        this.app = app;
        this.objectType = objectType;
        this.request = request;
    }

    @Override
    public void run()
    {
        try
        {
            if (this.content != null) {
                this.operaContent = JSONObject.fromObject(this.content).toString();
            } else if (this.contentList != null) {
                this.operaContent = JSONArray.fromObject(this.contentList).toString();
            } else if (this.contentStr != null) {
                this.operaContent = this.contentStr;
            }
            UserLog audit = new UserLog();
            audit.setEmail(this.email);
            audit.setMasterLogin(this.isMasterLogin);
            if (this.app != null) {
                App appObject = appRepository.findOne(this.app);
                audit.setApp(this.app);
                audit.setAppName(appObject.getName());
            }

            audit.setObjectType(this.objectType);

            audit.setPubDate(new Date());
            audit.setDs(DateUtil.format(new Date()));
            audit.setOperaContent(this.operaContent);

            if (this.request != null) {
                String ipAddr = IPAddrUtil.getIpAddrNew(request);
                audit.setIp(ipAddr);
                if(ValidateUtil.isValid(ipAddr)){
                    String locationFromIpAddr = IPAddrUtil.getLocationFromIpAddr(ipAddr);
                    audit.setLocation(locationFromIpAddr);
                }
            }

            audit.setOperaType(this.operaType);
            audit.setType(type);

            File baseFile = new File(Constant.logPath);// 存放文件的基文件夹
            if (!baseFile.exists() && !baseFile.isDirectory()) {// 如果文件夹不存在则创建
                baseFile.mkdir();
            }

            File txtFile = new File(Constant.logPath + "/"
                    + DateUtil.getBeforeDays(0) + ".txt");
            if (!txtFile.exists()) {
                try {
                    txtFile.createNewFile();// 创建文件
                } catch (IOException e) {
                    logger.error(e.getMessage(), e);
                }
            }

            StringBuffer sb = new StringBuffer();
            sb.append(audit.getType()).append("\t").append(audit.getEmail()).append("\t").append(audit.isMasterLogin()).append("\t")
                    .append(audit.getOperaType()).append("\t").append(audit.getOperaContent()).append("\t").append(audit.getPubDate()).append("\t").
                append(audit.getDs()).append("\t").append(audit.getApp()).append("\t").append(audit.getAppName()).append("\t").append(audit.getObjectType()).
                    append("\t").append(audit.getIp()).append("\t").append(audit.getLocation()).append("\n");

            //System.out.println(sb.toString());
            writeDataToFile(sb.toString(), txtFile);

            //只入库用户登录日志

            if (LogEnumType.DOWNLOAD_GDT.getCode().equals(type) ||
                    LogEnumType.DOWNLOAD_GDT_CLOSE.getCode().equals(type) ||
                    LogEnumType.BLACK_CLOSE_ALL.getCode().equals(type) ||
                    LogEnumType.BLACK_DEVICE_CLOSE.getCode().equals(type) ||
                    LogEnumType.BLACK_LIST_CLOSE.getCode().equals(type)) {
                // 记录广点通投放账号日志下载 开启日志
                // 记录防作弊关闭记录
                userLogService.saveLog(audit);
            } else {
                userLogService.saveUserLoginHistory(audit);
            }



           // userLogRepository.save(audit);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }



    // 将参数中数据写入文件中
    public void writeDataToFile(String data, File file) {
        byte bt[] = new byte[1024];
        try {
            bt = data.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            logger.error(e.getMessage(), e);
        }
        FileOutputStream in = null;
        try {
            in = new FileOutputStream(file, true);
            in.write(bt, 0, bt.length);
        } catch (FileNotFoundException e) {
            logger.error(e.getMessage(), e);
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
        } finally {
            if (in != null)
                try {
                    in.close();
                } catch (IOException e) {
                    logger.error(e.getMessage(), e);
                }
        }
    }
}
