package com.reyun.util;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.net.util.Base64;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

public class DingDingSendMsgUtil {

    protected Logger logger = LoggerFactory.getLogger(DingDingSendMsgUtil.class);
    public static final String APP_ONLINE_MSG = "SECe0246056d9266f5ffebd23eeff96dc2c6fcf80f65adc73afd741b132ab4fa222";

    public void sendMsg(String url,String content) {
        HttpClient httpclient = HttpClients.createDefault();

        HttpPost httppost = new HttpPost(url);
        httppost.addHeader("Content-Type", "application/json; charset=utf-8");

        String textMsg = "{ \"msgtype\": \"text\", \"text\": {\"content\": \""+content+"\"}}";
        StringEntity se = new StringEntity(textMsg, "utf-8");
        httppost.setEntity(se);
        try {
            HttpResponse response = httpclient.execute(httppost);
            String result= EntityUtils.toString(response.getEntity(), "utf-8");
            JSONObject jsonObject = JSONObject.parseObject(result);
            if(jsonObject.get("errcode") !=null && !"0".equals(jsonObject.get("errcode").toString())){
                logger.warn("钉钉消息发送失败：\r\nURL："+url+"\r\nContent:"+content+"\r\n失败原因："+result);
            }
        }catch (Exception e){
            logger.error("钉钉消息发送失败：\r\nURL："+url+"\r\nContent:"+content,e);
        }

    }
    /**
     * 功能描述:机器人加签所需，获取签名
     * @param secret 密钥，机器人安全设置页面，加签一栏下面显示的SEC开头的字符串
     * @param timestamp 当前时间戳，单位是毫秒，与请求调用时间误差不能超过1小时
     * @author liyin
     * @date 2019/11/26
     */
    public String getSignNew(Long timestamp,String secret){
        try {
            String stringToSign = timestamp + "\n" + secret;
            Mac mac = Mac.getInstance("HmacSHA256");
            mac.init(new SecretKeySpec(secret.getBytes(StandardCharsets.UTF_8), "HmacSHA256"));
            byte[] signData = mac.doFinal(stringToSign.getBytes(StandardCharsets.UTF_8));
            return URLEncoder.encode(new String(Base64.encodeBase64(signData)),"UTF-8");
        }catch (Exception e){
            logger.warn(e.getMessage());
            return null;
        }
    }

    public static void main(String[] args) {
        DingDingSendMsgUtil dingDingUtil = new DingDingSendMsgUtil();
        long timeMillis = System.currentTimeMillis();
        String sign = dingDingUtil.getSignNew(timeMillis, DingDingSendMsgUtil.APP_ONLINE_MSG);
        String url = "https://oapi.dingtalk.com/robot/send?access_token=d2b4b62809fe0e469d8487ffe2e8678d596fb3ed10a2fc731df274fd7c473d3a&timestamp=" + timeMillis + "&sign=" + sign;
        StringBuilder sb = new StringBuilder("Company：");
        sb
                .append("热云").append("\r")
                .append("Email：").append("243242@reyun.com").append("\r")
                .append("App Key：").append("h2j34h23hjg43g4j2h4237y89qiw").append("\r")
                .append("App Name：").append("快手").append("\r")
                .append("预计上线时间：").append("2020-07-02").append("\r")
                .append("距离上线天数：").append(1+"天");
        dingDingUtil.sendMsg(url, sb.toString());
    }
}
