package com.reyun.util;

import com.reyun.model.ReportListResult;
import net.sf.json.JSONObject;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by sunhao on 17/7/7.
 * desc:数据导出工具类
 */
public class ExportReportUtil {

    private static final String REPORT_NAME_CAMPAIGN = "campaigninfo_bycampgroup";
    private static final String SUB_TYPE_CAMPAIGN = "campaign";
    private static final String SPECIAL_REPORT_NAME = "campaigninfo_bydatecampaign";
    private static final String REPORT_NAME_CAMPAIGN_CID = "campaigninfo_bycampgroup_cid";

    private ExportReportUtil(){}

    /**
     * 获取conditionMap避免使用 new ObjectMapper().readValue(export.getConditions(), HashMap.class);转换不完全
     * created by sunhao 20150510
     */
    public static Map<String, String> changeCondition2Map(String conditions) {

        Map<String, String> resultMap = new HashMap<>();

        JSONObject conditionObject = JSONObject.fromObject(conditions);

        Iterator<String> conditionIterator = conditionObject.keys();
        while (conditionIterator.hasNext()) {
            String key = conditionIterator.next();
            resultMap.put(key, conditionObject.getString(key));
        }

        return resultMap;
    }

    public static StringBuffer getExportContent(ReportListResult reportListResult, String subType) {

        StringBuffer sb = new StringBuffer();
        List<String> name = reportListResult.getName();
        List<Object> columnKey = reportListResult.getColumnkey();
        List<String> reportName = reportListResult.getReportname();
        String report = reportName.size() > 0 ? reportName.get(0) : "";

        boolean isNeedChannelAndGroup = SUB_TYPE_CAMPAIGN.equals(subType) && REPORT_NAME_CAMPAIGN.equals(report) || SPECIAL_REPORT_NAME.equals(report)|| REPORT_NAME_CAMPAIGN_CID.equals(report);

        if("campaigninfo_bydatechannel".equals(report)){
            //修复导出报表渠道名称
            columnKey.remove("cid");
            columnKey.add(1,"channelname");
        }

        if (isNeedChannelAndGroup&&!subType.equals("dsgroup")) {
            name.add("渠道");
            name.add("活动组");
        }

        if(!columnKey.contains("campaignid")){
            if("campaigninfo_bydatecampaign".equals(reportName.get(0))){
                name.add(1,"推广活动");
                columnKey.add(1,"campaignid");
            }
        }


        List<Map<String, Object>> value = reportListResult.getVal();

        if (!CollectionUtils.isEmpty(name)) {

            sb.append(String.join(",", name)).append("\r\n");
            if (isNeedChannelAndGroup) {
                for (Map<String, Object> v : value) {
                    for (int i = 0; i < columnKey.size(); i++) {
                        /*if(columnKey.get(i).toString().startsWith("avg_time") || columnKey.get(i).equals("duration")&&v.get(columnKey.get(i)).getClass().isAssignableFrom(Long.class) ){
                            sb.append(DateUtil.formatCaseTime(Long.valueOf(v.get(columnKey.get(i)).toString())))
                                    .append(",");
                        }else{*/
                            sb.append(v.get(columnKey.get(i)))
                                    .append(columnKey.get(i).toString().startsWith("rate_") ? "%" : "")
                                    .append(",");
                        /*}*/

                    }

                    sb.append(v.containsKey("channelname") ? v.get("channelname") : " ").append(",");
                    sb.append(v.containsKey("groupname") ? v.get("groupname") : " ").append("\r\n");
                }
            } else {
                for (Map<String, Object> v : value) {
                    for (int i = 0; i < columnKey.size(); i++) {
                        if("advertisinginfo_cash_day".equals(report)){
                            sb.append(v.get(columnKey.get(i))==null?v.get(columnKey.get(i)):v.get(columnKey.get(i)).toString().replace(",",""))
                                    .append(columnKey.get(i).toString().startsWith("rate_") ? "%" : "")
                                    .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                        }else{
                            sb.append(v.get(columnKey.get(i)))
                                    .append(columnKey.get(i).toString().startsWith("rate_") ? "%" : "")
                                    .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                        }

                    }
                }
            }

        }

        return sb;
    }

    public static StringBuffer getRoiExportContent(ReportListResult result, String name, boolean isFirst) {
        StringBuffer sb = new StringBuffer();
        List<String> nameList = result.getName();
        List<Object> columnKey = result.getColumnkey();
        List<Map<String, Object>> value = result.getVal();

        if (!CollectionUtils.isEmpty(nameList)) {
            if(isFirst){
                sb.append(String.join(",", nameList)).append("\r\n");
            }
            for (Map<String, Object> v : value) {
                sb.append(name).append(",");
                for (int i = 0; i < columnKey.size(); i++) {
                    sb.append(v.get(columnKey.get(i).toString())).append(columnKey.get(i).toString().startsWith("rate_") ? "%" : "")
                            .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                }
            }
        }

        return sb;
    }

    public static StringBuffer getRoiExportContent(ReportListResult reportResult, Map<String, String> conditions, Map<String, String> channelMap) {

        List<Object> columnKey = new LinkedList<>();
        List<String> nameList = new LinkedList<>();
       Map<String, List<Map<String, Object>>> resuMap = new HashMap<>();

        List<Map<String, Object>> valListTotal = reportResult.getVal();
        if (ValidateUtil.isValid(valListTotal)){
            List<Long> cidList = new ArrayList<>();
            Map<String, List<Map<String, Object>>> mapTotal = new HashMap<>();
            for(Map<String, Object> m : valListTotal){
                if(mapTotal.containsKey(m.get("cid").toString())){
                    List<Map<String, Object>> list = mapTotal.get(m.get("cid").toString());
                    list.add(m);
                    mapTotal.put(m.get("cid").toString(), list);
                } else{
                    List<Map<String, Object>> list = new ArrayList<>();
                    list.add(m);
                    mapTotal.put(m.get("cid").toString(), list);
                }
            }

            Set<String> keySet = mapTotal.keySet();
            for(String cid : keySet){
                String install_startdate = conditions.get("install_startdate");
                String install_enddate = conditions.get("install_enddate");
                int install_interval = Integer.parseInt(conditions.get("install_interval"));
                String pay_startdate = conditions.get("pay_startdate");
                String pay_enddate = conditions.get("pay_enddate");
                int pay_interval = Integer.parseInt(conditions.get("pay_interval"));
                List<List<String>> installInterval = getDateInterval(install_startdate, install_enddate, install_interval);
                List<List<String>> payInterval = getDateInterval(pay_startdate, pay_enddate, pay_interval);
                int clumns = payInterval.size();    //列的数目
                List<Map<String, Object>> mapList = mapTotal.get(cid);

                Map<String, List<Map<String, Object>>> valMap = new HashMap<>();

                for(Map<String, Object> map : mapList){
                    String ins_ds = map.get("ins_ds").toString();
                    String pay_ds = map.get("pay_ds").toString();
                    double amt_amount = Double.parseDouble(map.get("amt_amount").toString());
                    if(valMap.containsKey(ins_ds)){
                        List<Map<String, Object>> list = valMap.get(ins_ds);
                        Map<String, Object> dataMap = new HashMap<>();
                        dataMap.put("ins_ds", ins_ds);
                        dataMap.put("pay_ds", pay_ds);
                        dataMap.put("amt_amount", amt_amount);
                        list.add(dataMap);
                        valMap.put(ins_ds, list);
                    }else{
                        List<Map<String, Object>> list = new ArrayList<>();
                        Map<String, Object> dataMap = new HashMap<>();
                        dataMap.put("ins_ds", ins_ds);
                        dataMap.put("pay_ds", pay_ds);
                        dataMap.put("amt_amount", amt_amount);
                        list.add(dataMap);
                        valMap.put(ins_ds, list);
                    }
                }


                List<Map<String, Object>> res = new ArrayList<>();
                for(List<String> list : installInterval){
                    String startDate = list.get(0);
                    String endDate = list.get(0);
                    Map<String, Object> map = new HashMap<>();

                    if(startDate.compareTo(pay_startdate) > 0){
                        pay_startdate = startDate;
                    }
                    if(pay_enddate.compareTo(pay_startdate) >= 0){
                        List<List<String>> dateInterval = getDateInterval(pay_startdate, pay_enddate, pay_interval);
                        for(int m = 0; m < dateInterval.size(); m++){// 初始化
                            map.put(m + "", 0d);
                        }
                        for(String date : list){
                            endDate = date;
                            if(valMap.containsKey(date)){
                                List<Map<String, Object>> valList = valMap.get(date);
                                for(Map<String, Object> val : valList){
                                    String pay_ds = (String)val.get("pay_ds");
                                    Double amt_amount = (Double) val.get("amt_amount");
                                    for(int j = 0; j < dateInterval.size(); j++){
                                        List<String> payList = dateInterval.get(j);
                                        if(payList.contains(pay_ds)){
                                            map.put(j+"",map.containsKey(j+"") ? ((double)map.get(j+"") + amt_amount) : amt_amount);
                                        }
                                    }
                                }
                            }
                        }
                    }

                    for(int i = 0; i < clumns; i++){
                        if(!map.containsKey(i + "")){
                            map.put(i + "","--");
                        }
                    }

                    map.put("key",startDate + "～" + endDate);
                    res.add(map);
                }
                resuMap.put( cid, res);
                if(nameList.size() == 0){
                    columnKey.add("key");
                    nameList.add("付费激活周期");
                    for(int i = 0; i < clumns; i++){
                        columnKey.add(i+"");
                        nameList.add("第" + (i + 1) + "付费周期");
                    }
                }
            }


        }

        StringBuffer sb = new StringBuffer();

        if (!CollectionUtils.isEmpty(nameList)) {
            sb.append(String.join(",", nameList)).append("\r\n");
            if(ValidateUtil.isValid(resuMap)){
                Set<String> keySet = resuMap.keySet();
                for(String key : keySet){
                    List<Map<String, Object>> value = resuMap.get(key);
                    for (Map<String, Object> v : value) {
                        sb.append(channelMap.get(key)).append(",");
                        for (int i = 0; i < columnKey.size(); i++) {
                            sb.append(v.get(columnKey.get(i).toString())).append(columnKey.get(i).toString().startsWith("rate_") ? "%" : "")
                                    .append(i == columnKey.size() - 1 ? "\r\n" : ",");
                        }
                    }
                }
            }
        }
        return sb;
    }

    public static List<List<String>> getDateInterval(String startDate, String endDate, int num){
        String date = startDate;
        List<List<String>> res = new LinkedList<>();
        while(date.compareTo(endDate) <= 0){
            List<String> list = new LinkedList<>();
            for(int i = 0; i < num; i++){
                list.add(date);
                date = getSpecifiedDayBefore(date, 1);
                if(date.compareTo(endDate) > 0){
                    break;
                }
            }
            res.add(list);
        }
        return  res;
    }

    public static String getSpecifiedDayBefore(String specifiedDay, int days) {
        Calendar c = Calendar.getInstance();
        Date date = null;
        try {
            date = new SimpleDateFormat("yyyy-MM-dd").parse(specifiedDay);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        c.setTime(date);
        int day = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day + days);

        String dayBefore = new SimpleDateFormat("yyyy-MM-dd").format(c.getTime());
        return dayBefore;
    }
}
