package com.reyun.util;



import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang.text.StrSubstitutor;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.pool.PoolStats;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import java.io.IOException;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.cert.CertificateException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static org.springframework.data.repository.init.ResourceReader.Type.JSON;

/**
 * HttpUtil
 *
 * @author liruijie@reyun.com
 * @date 2015年11月11日
 */
public class HttpUtil
{
    private static final Logger logger = LoggerFactory.getLogger(HttpUtil.class);

    private static HttpUtil instance;

    private static PoolingHttpClientConnectionManager connectionManager;
    private static HttpClientBuilder httpBuilder;

    private static PoolingHttpClientConnectionManager sslConnectionManager;
    private static HttpClientBuilder sslHttpBuilder;

    private static TrustManager truseAllManager = new X509TrustManager()
    {
        @Override
        public void checkClientTrusted(java.security.cert.X509Certificate[] arg0, String arg1)
                throws CertificateException
        {
        }

        @Override
        public void checkServerTrusted(java.security.cert.X509Certificate[] arg0, String arg1)
                throws CertificateException
        {
        }

        @Override
        public java.security.cert.X509Certificate[] getAcceptedIssuers()
        {
            return new java.security.cert.X509Certificate[0];
        }
    };

    private HttpUtil()
    {}

    public static final HttpUtil getInstance()
    {
        if (null == instance) {
            init();
        }
        return instance;
    }

    private static final synchronized void init()
    {
        if (null == instance) {
            try {
                connectionManager = new PoolingHttpClientConnectionManager();
                connectionManager.setMaxTotal(1000);
                connectionManager.setDefaultMaxPerRoute(1000);
                // update-Author:ruijie Date:2016-02-02 for:
                // 调用disableContentCompression与disableRedirectHandling --------
                httpBuilder = HttpClients.custom().disableContentCompression().disableRedirectHandling();
                httpBuilder.setConnectionManager(connectionManager);

                // ssl HttpClientBuilder
                SSLContext sslcontext = SSLContexts.custom().build();
                sslcontext.init(null, new TrustManager[] {truseAllManager}, null);
                SSLConnectionSocketFactory sslConnectionSocketFactory = new SSLConnectionSocketFactory(sslcontext,
                        new String[] {"TLSv1", "SSLv3"}, null, SSLConnectionSocketFactory.getDefaultHostnameVerifier());

                Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                        .register("http", PlainConnectionSocketFactory.INSTANCE)
                        .register("https", sslConnectionSocketFactory).build();
                sslConnectionManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);

                sslConnectionManager.setMaxTotal(1000);
                sslConnectionManager.setDefaultMaxPerRoute(1000);

                sslHttpBuilder = HttpClients.custom().disableContentCompression().disableRedirectHandling();
                sslHttpBuilder.setConnectionManager(sslConnectionManager);
//
//                ScheduledExecutorService scheduledThreadPool = Executors.newScheduledThreadPool(1);
//                scheduledThreadPool.scheduleWithFixedDelay(new Runnable()
//                {
//                    @Override
//                    public void run()
//                    {
//                        PoolStats stats = connectionManager.getTotalStats();
//                        logger.info("no ssl, msg: {}", stats.toString());
//
//                        PoolStats sslStats = sslConnectionManager.getTotalStats();
//                        logger.info("ssl, msg: {}", sslStats.toString());
//                    }
//                }, 5, 30, TimeUnit.SECONDS);
            }
            catch (Exception e) {
                logger.error(e.getMessage(), e);
            }

            instance = new HttpUtil();
        }
    }


    public String gdtAttribution(String data, String url, int timeout)
            throws IOException
    {
        CloseableHttpResponse resp = null;
        //JSON json = new JSONObject();
        String result = "";
        try {
            CloseableHttpClient httpClient = httpBuilder.build();

            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Content-Type", "application/json;charset=utf8");

            RequestConfig requestConfig = RequestConfig.custom().setSocketTimeout(timeout).setConnectTimeout(timeout).build();
            httpPost.setConfig(requestConfig);

            StringEntity se = new StringEntity(data, Charset.forName("UTF-8"));
            httpPost.setEntity(se);

            resp = httpClient.execute(httpPost);

            if (resp.getStatusLine().getStatusCode() == 200) { //如果状态码为200,就是正常返回
                result = EntityUtils.toString(resp.getEntity());
            }
        }
        catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        finally {
            if (null != resp) {
                try {
                    resp.close();
                }
                catch (IOException e) {
                    logger.error(e.getMessage(), e);
                }
            }
        }
        return result;
    }

    public static void main(String[] args)
            throws Exception
    {

        long timestamp = new Date().getTime() / 1000;
        Map<String, String> map = new HashMap<String, String>();
        map.put("access_token", "623e626cec0fce0a43c32f4f338b83f0");
        map.put("timestamp", timestamp + "");
        map.put("nonce", UUID.randomUUID().toString().replace("-", ""));

        JSONArray jsonArray = new JSONArray();

        JSONObject json = new JSONObject();
        json.put("account_id", "3504209");

        JSONObject data = new JSONObject();
        data.put("user_action_set_id", "1106248719");
        data.put("action_time", timestamp + "");
        data.put("action_type", "ACTIVATE_APP");
        data.put("action_param", new JSONObject());

        JSONObject userid = new JSONObject();
        userid.put("hash_imei", CipherUtil.generatePassword("99000830167815"));
        data.put("user_id", userid);
        jsonArray.add(data);

        json.put("actions", jsonArray);

        StrSubstitutor sub = new StrSubstitutor(map);
        String url = "https://api.e.qq.com/v1.0/user_actions/add?access_token=${access_token}&timestamp=${timestamp}&nonce=${nonce}";
        url = sub.replace(url);

        String result = HttpUtil.getInstance().gdtAttribution(json.toString(), url, 800);
        System.out.println(result);
    }
}
