package com.reyun.util;

import com.amazonaws.AmazonClientException;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.kinesis.AmazonKinesis;
import com.amazonaws.services.kinesis.AmazonKinesisClientBuilder;
import com.amazonaws.services.kinesis.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.ByteBuffer;
import java.util.UUID;

import static com.amazonaws.SDKGlobalConfiguration.ACCESS_KEY_SYSTEM_PROPERTY;
import static com.amazonaws.SDKGlobalConfiguration.SECRET_KEY_SYSTEM_PROPERTY;

public class KinesisUtil {
    private static final Logger logger = LoggerFactory.getLogger(KinesisUtil.class);
    private static KinesisUtil instance;

    private static AmazonKinesis kinesisClient ;

    public static final String TKIO_ATTRIBUTE = "attribute_stream";

    private KinesisUtil()
    {
    }

    public static KinesisUtil getInstance()
    {
        if (null == instance) {
            syncInit();
        }
        return instance;
    }

    private static synchronized void syncInit()
    {
        if (instance == null) {
            System.setProperty(ACCESS_KEY_SYSTEM_PROPERTY, Constant.accessKey);
            System.setProperty(SECRET_KEY_SYSTEM_PROPERTY, Constant.secretKey);

            AmazonKinesisClientBuilder clientBuilder = AmazonKinesisClientBuilder.standard();

            clientBuilder.setRegion(Regions.CN_NORTH_1.getName());

            kinesisClient = clientBuilder.build();

            instance = new KinesisUtil();
        }
    }

    public AmazonKinesis getKinesisClient()
    {
        return kinesisClient;
    }

    /**
     * 使用 PutRecord 添加单一记录
     * @param streamName   流名称
     * @param data   要传的数据   , \t \001 \002
     * @param partitionKey  UUID
     */

    public  void putRecord(String streamName,String data,String partitionKey){
        byte[] bytes = data.getBytes();
        if (bytes == null) {
            logger.error("putRecord data Could not be null");
            return;
        }
        PutRecordRequest putRecord = new PutRecordRequest();
        putRecord.setStreamName(streamName);
        putRecord.setPartitionKey(partitionKey);
        putRecord.setData(ByteBuffer.wrap(bytes));
        try {
            kinesisClient.putRecord(putRecord);
        } catch (AmazonClientException ex) {
           logger.error("Error sending record to Amazon Kinesis.", ex);
        }
    }

    public  void putRecord(String streamName,String data){
        String partitionKey = UUID.randomUUID().toString().replaceAll("-", "");
        putRecord(streamName,data,partitionKey);
    }
}
