package com.reyun.util;

import com.alibaba.fastjson.JSONObject;
import com.amazonaws.services.dynamodbv2.document.*;
import com.amazonaws.services.dynamodbv2.document.spec.ScanSpec;
import com.amazonaws.services.dynamodbv2.document.spec.UpdateItemSpec;
import com.amazonaws.services.dynamodbv2.document.utils.ValueMap;
import com.amazonaws.services.dynamodbv2.model.ReturnValue;
import com.amazonaws.util.BinaryUtils;
import com.mongodb.client.MongoCursor;
import com.reyun.model.App;
import com.reyun.model.ChannelAccount;
import com.reyun.model.ReportToken;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.bson.Document;
import org.bson.types.Binary;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.util.*;
import java.util.Map.Entry;

/* 以后上层应用读写nosql时(例如不同云厂商的ddb/mongodb产品)请使用本类，勿直接调用特定的产品工具类，例如DynamoDBUtil */
public class NoSQLDBUtil {
    private static final Logger LOG = LoggerFactory.getLogger(NoSQLDBUtil.class);
    public static final String TBL_TKIO_INSTALL = "tkio_install";
    public static final String TBL_TKIO_DEBUG = "tkio_debug";
    public static final String TKIO_IDFA_TOKEN = "tkio_idfa_token";
    public static final String TBL_TABLE_SURL = "tkio_surl";
    public static final String TBL_TKIO_BUNDLEID = "tkio_idfa";
    public static final String TBL_TKIO_VIVO_PARAM = "tkio_vovioSigin";
    public static final String TBL_TABLE_MARKETKNGAPI = "tkio_syncmarketingapidata";//存储头条接口返回的广告主id等信息
    public static final String TBL_TABLE_DEEPLINK = "tkio_deeplink";
    public static final String TBL_WX_APP = "tkio_wx_app";//存储微信的appid与secret
    public static final String TBL_TKIO_SETTING_NEW = "tkio_setting_new";

    /* write an item  */
    public static boolean writeItem(String tableName, Result obj) {
        if (StringUtils.isBlank(tableName) || obj == null || obj.getResult() == null) {
            LOG.warn("writeItem hits empty input param(s)");
            return false;
        }
        if (Constant.isQcloudEnv()) {
            Document document = (Document) obj.getResult();
            com.mongodb.client.result.UpdateResult ur = MongoDBUtil.getInstance().writeItem(tableName, document);
            return ur.wasAcknowledged();
        } else if (Constant.isAwsEnv()) {
            PutItemOutcome pio = DynamoDBUtil.getInstance().writeItem(tableName, (Item) obj.getResult());
            //TODO: how to check the result ?
            return pio.getPutItemResult().getSdkHttpMetadata().getHttpStatusCode() == 200;
        }
        return false;
    }

    public static boolean deleteItem(String tableName, String primaryKeyName, String primaryKeyValue) {
        if (StringUtils.isBlank(tableName) || StringUtils.isBlank(primaryKeyName)
                || StringUtils.isBlank(primaryKeyValue)) {
            LOG.warn("deleteItem hits empty input param(s)");
            return false;
        }
        if (Constant.isQcloudEnv()) {
            return MongoDBUtil.getInstance().deleteItem(tableName, primaryKeyValue).wasAcknowledged();
        } else if (Constant.isAwsEnv()) {
            DeleteItemOutcome dio = DynamoDBUtil.getInstance().deleteItem(tableName, primaryKeyName, primaryKeyValue);
            return dio.getDeleteItemResult().getSdkHttpMetadata().getHttpStatusCode() == 200;
        }
        return false;
    }

    public static boolean updateItem(String tableName, String primaryKeyName, String primaryKeyValue,
                                     UpdateResult obj) {
        if (StringUtils.isBlank(tableName) || obj == null || obj.getResult() == null) {
            LOG.warn("updateItem hits empty input param(s)");
            return false;
        }

        if (Constant.isQcloudEnv()) {
            Document document = new Document("$set", (Document) obj.getResult());
            return MongoDBUtil.getInstance().updateItem(tableName, primaryKeyName, primaryKeyValue, document)
                    .wasAcknowledged();
        } else if (Constant.isAwsEnv()) {
            UpdateItemSpec updateItemSpec = (UpdateItemSpec) obj.getResult();
            updateItemSpec.withPrimaryKey(primaryKeyName, primaryKeyValue);
            UpdateItemOutcome uio = DynamoDBUtil.getInstance().updateItem(tableName, updateItemSpec);
            return uio.getUpdateItemResult().getSdkHttpMetadata().getHttpStatusCode() == 200;
        }
        return false;
    }

    public static boolean resetDebugNum(String appid) {
        if (StringUtils.isBlank(appid)) {
            LOG.warn("resetDebugNum hits appid");
            return false;
        }
        if (Constant.isQcloudEnv()) {
            Document document = new Document("$set", new Document().append("num", 0));
            MongoDBUtil.getInstance().updateItem(TBL_TKIO_DEBUG, "appid", appid, document);
        } else if (Constant.isAwsEnv()) {
            UpdateItemSpec updateItemSpec = new UpdateItemSpec().withPrimaryKey("appid", appid)
                    .withUpdateExpression("set  num=:n").withValueMap(new ValueMap().withNumber(":n", 0))
                    .withReturnValues(ReturnValue.UPDATED_NEW);
            UpdateItemOutcome uio = DynamoDBUtil.getInstance().updateItem(TBL_TKIO_DEBUG, updateItemSpec);
            return uio.getUpdateItemResult().getSdkHttpMetadata().getHttpStatusCode() == 200;
        }
        return false;
    }

    /* point query per primary key */
    public static Result queryByPrimaryKey(String tableName, String primaryKeyName, String primaryKeyValue) {
        if (StringUtils.isBlank(tableName) || StringUtils.isBlank(primaryKeyName)
                || StringUtils.isBlank(primaryKeyValue)) {
            LOG.warn("queryByPrimaryKey hits empty input param(s)");
            return null;
        }
        if (Constant.isQcloudEnv()) {
            Document doc = MongoDBUtil.getInstance().queryDocumentByPrimaryKey(tableName, primaryKeyName,
                    primaryKeyValue);
            if (ValidateUtil.isValid(doc)) {
                doc.append(primaryKeyName, primaryKeyValue);
            }
            return new Result(doc);
        }
        Item item = DynamoDBUtil.getInstance().queryByPrimaryKey(tableName, primaryKeyName, primaryKeyValue);
        return new Result(item);
    }

    /**
     * 批量读
     */
    public static List<Result> batchQueryByPrimaryKey(String tableName, String primaryKeyName, int batchSize,
                                                      List<String> values) {
        List<Result> resultList = new ArrayList<>();
        if (StringUtils.isBlank(tableName) || StringUtils.isBlank(primaryKeyName) || CollectionUtils.isEmpty(values)) {
            LOG.warn("batchQueryByPrimaryKey hits empty input param(s)");
            return resultList;
        }
        if (Constant.isQcloudEnv()) {
            List<Document> docList = MongoDBUtil.getInstance().batchQueryByPrimaryKey(tableName, primaryKeyName,
                    batchSize, values);
            for (Document doc : docList) {
                resultList.add(new Result(doc));
            }
            return resultList;
        }
        List<Item> itemList = DynamoDBUtil.getInstance().batchQueryByPrimaryKey(tableName, primaryKeyName, batchSize,
                values);
        for (Item item : itemList) {
            resultList.add(new Result(item));
        }
        return resultList;
    }

    /**
     * 批量写
     *
     * @param tableName
     * @param items
     */
    public static void batchWrite(String tableName, List<Result> items) {
        if (Constant.isQcloudEnv()) {
            MongoDBUtil.getInstance().batchWrite(tableName, items);
        } else if (Constant.isAwsEnv()) {
            DynamoDBUtil.getInstance().batchWrite(tableName, items, 25);
        }
    }

    public static Set<String> getSetData(String tableName, String keyName, String keyVal, String setFieldName) {
        if (StringUtils.isBlank(tableName) || StringUtils.isBlank(keyName) || StringUtils.isBlank(keyVal)
                || StringUtils.isBlank(setFieldName)) {
            LOG.warn("getSetData hits empty input param(s)");
            return new HashSet<String>();
        }
        if (Constant.isQcloudEnv()) {
            return MongoDBUtil.getInstance().getStringSet(tableName, keyVal, setFieldName);
        }
        return DynamoDBUtil.getInstance().getSetFromDDB(tableName, keyName, keyVal, setFieldName);
    }

    @SuppressWarnings("unchecked")
    public static List<String> getListData(String tableName, String keyName, String keyVal, String listFieldName) {
        if (StringUtils.isBlank(tableName) || StringUtils.isBlank(keyName) || StringUtils.isBlank(keyVal)
                || StringUtils.isBlank(listFieldName)) {
            LOG.warn("getListData hits empty input param(s)");
            return new ArrayList<String>();
        }
        if (Constant.isQcloudEnv()) {
            return MongoDBUtil.getInstance().getList(tableName, keyVal, listFieldName);
        }
        return DynamoDBUtil.getInstance().getList(tableName, keyName, keyVal, listFieldName);
    }

    public static ResultIterator scanIdfaToken() {
        if (Constant.isQcloudEnv()) {
            MongoCursor<Document> documentCursor = MongoDBUtil.getInstance().getCollection(TKIO_IDFA_TOKEN).find()
                    .iterator();
            return new ResultIterator(documentCursor);
        } else if (Constant.isAwsEnv()) {
            /*
             * TODO: per
             * https://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_Scan.html
             * do we need to Paginating or Parallel Scan?
             */
            ItemCollection<ScanOutcome> items = DynamoDBUtil.getInstance().getTable(TKIO_IDFA_TOKEN)
                    .scan(new ScanSpec());
            Iterator<Item> iterator = items.iterator();
            return new ResultIterator(iterator);
        }
        return null;
    }

    public static void putMarketingapi(ChannelAccount channelAccount, List<App> apps, boolean insertAdvertiserId,
                                       String fix) {
        if (channelAccount == null || CollectionUtils.isEmpty(apps)) {
            return;
        }
        Result item = new Result();
        Result rs = queryByPrimaryKey(TBL_TABLE_MARKETKNGAPI, "id", fix + channelAccount.getThirdAccountCode());
        if(rs!=null) item = rs;
        item.withPrimaryKey("id", fix + channelAccount.getThirdAccountCode());
        item.withString("access_token", channelAccount.getAccessToken());
        if (channelAccount.getChannel().equals(6481L)) {
            item.withString("username", channelAccount.getName());
            item.withString("password", channelAccount.getPassword());
        }
        List<String> applist = new ArrayList<>(apps.size());
        for (App app : apps) {
            applist.add(app.getAppkey());
        }
        item.withList("applist", applist);
        writeItem(TBL_TABLE_MARKETKNGAPI, item);
        if (insertAdvertiserId) {
            Result advertiserIdItem = queryByPrimaryKey(TBL_TABLE_MARKETKNGAPI, "id", fix + "advertiser_id_list");
            if (!ValidateUtil.isValid(advertiserIdItem)) {
                advertiserIdItem = new Result();
                advertiserIdItem.withPrimaryKey("id", fix + "advertiser_id_list");
                Set<String> advertiserIdSet = new HashSet<>();
                advertiserIdSet.add(channelAccount.getThirdAccountCode());
                advertiserIdItem.withStringSet("datas", advertiserIdSet);
            } else {
                Set<String> advertiserIdSet = advertiserIdItem.getStringSet("datas");
                advertiserIdSet.add(channelAccount.getThirdAccountCode());
                advertiserIdItem.withStringSet("datas", advertiserIdSet);
            }
            writeItem(TBL_TABLE_MARKETKNGAPI, advertiserIdItem);
        }
    }

    public static void putReportToken(ReportToken reportToken) {
        Result rs = queryByPrimaryKey(TBL_TKIO_DEBUG, "appid", reportToken.getAppid());
        rs.withString("access_token", reportToken.getAccessToken());
        rs.withString("access_token_expires", reportToken.getAccessTokenExpires().getTime() + "");
        writeItem(TBL_TKIO_DEBUG, rs);
    }

    public static void putAppName(App app) {
        Result rs = queryByPrimaryKey(TBL_TKIO_DEBUG, "appid", app.getAppkey());
        if (rs != null) {
            rs.withString("ry_appname", app.getName());
            writeItem(TBL_TKIO_DEBUG, rs);
        }
    }

    public static boolean batchDeleteSurl(List<String> surlList) {
        if (Constant.isQcloudEnv()) {
            Document docs = new Document();
            docs.put("surl", new Document("$in", surlList));
            return MongoDBUtil.getInstance().getCollection(NoSQLDBUtil.TBL_TABLE_SURL).deleteMany(docs)
                    .wasAcknowledged();
        } else if (Constant.isAwsEnv()) {
            TableWriteItems tableWriteItems = new TableWriteItems(NoSQLDBUtil.TBL_TABLE_SURL);
            for (String s : surlList) {
                tableWriteItems.addPrimaryKeyToDelete(new PrimaryKey("surl", s));
            }
            BatchWriteItemOutcome deleteItemOutcome = DynamoDBUtil.getInstance().getDynamoDB()
                    .batchWriteItem(tableWriteItems);
            return deleteItemOutcome.getBatchWriteItemResult().getSdkHttpMetadata().getHttpStatusCode() == 200;
        }
        return false;
    }

    /* extract from aws Item.getBoolean() method */
    private static Boolean getBooleanValueFromQueryMap(Map<String, Object> map, String key) {
        if (!map.isEmpty() && map.containsKey(key)) {
            Object val = map.get(key);
            if (val instanceof Boolean) {
                return (Boolean) val;
            }
            if (val instanceof String) {
                if ("1".equals(val)) {
                    return true;
                }
                if ("0".equals(val)) {
                    return false;
                }
                return Boolean.valueOf((String) val);
            }
        }
        return false;
    }

    public static class Result {
        private Item item;
        private Document document;

        public Result() {
        }

        public Result(Item item) {
            this.item = item;
        }

        public Result(Document document) {
            this.document = document;
        }

        public String getString(String attrName) {
            if (item != null) {
                return item.getString(attrName);
            } else if (document != null) {
                return document.getString(attrName);
            }
            return null;
        }

        public Object get(String attrName) {
            if (item != null) {
                return item.get(attrName);
            } else if (document != null) {
                return document.get(attrName);
            }
            return null;
        }

        public int getInt(String attrName) {
            if (item != null) {
                return item.getInt(attrName);
            } else if (document != null) {
                return document.getInteger(attrName);
            }
            throw new NumberFormatException("value of " + attrName + " is null");
        }

        public long getLong(String attrName) {
            if (item != null) {
                return item.getLong(attrName);
            } else if (document != null) {
                return document.getLong(attrName);
            }
            throw new NumberFormatException("value of " + attrName + " is null");
        }

        public double getDouble(String attrName) {
            if (item != null) {
                return item.getDouble(attrName);
            } else if (document != null) {
                return document.getDouble(attrName);
            }
            throw new NumberFormatException("value of " + attrName + " is null");
        }

        public BigDecimal getBigDecimal(String attrName) {
            if (item != null) {
                return item.getNumber(attrName);
            } else if (document != null) {
                Object val = document.get(attrName);
                if (val == null) {
                    return null;
                }
                return val instanceof BigDecimal
                        ? (BigDecimal) val
                        : new BigDecimal(val.toString())
                        ;
            }
            throw new NumberFormatException("value of " + attrName + " is null");
        }

        public boolean getBoolean(String attrName) {
            if (item != null) {
                return item.getBoolean(attrName);
            } else if (document != null) {
                return getBooleanValueFromQueryMap(document, attrName);
            }
            return false;
        }

        public byte[] getBinary(String attrName) {
            if (item != null) {
                return item.getBinary(attrName);
            } else if (document != null) {
                // Document class does not have getBinary, so extracted from Item.getBinary
                Object val = document.get(attrName);
                if (val == null) {
                    return null;
                }
                if (val instanceof byte[]) {
                    return (byte[]) val;
                }
                if (val instanceof ByteBuffer) {
                    return BinaryUtils.copyAllBytesFrom((ByteBuffer) val);
                }
                if (val instanceof Binary) {
                    return ((Binary) val).getData();
                }
                throw new UnsupportedOperationException(val.getClass() + " cannot be converted into a byte array");
            }
            return null;
        }

        //TODO: refactoring with T class type, e.g.  <T> List<T> getList
        public List<String> getList(String attrName) {
            if (item != null) {
                return item.getList(attrName);
            } else if (document != null) {
                return document.getList(attrName, String.class);
            }
            return new ArrayList<String>();
        }

        /**
         * T must not be a subclass of <code>Number
         * if  is Number , use  getMapOfNumbers
         *
         * @param attrName
         * @param <T>
         * @return
         */
        public <T> Map<String, T> getMap(String attrName) {
            if (item != null) {
                return item.getMap(attrName);
            } else if (document != null) {
                return document.get(attrName, Map.class);
            }
            return new HashMap<>();
        }

        /**
         * Convenient method to return the specified attribute in the current item
         * as a (copy of) map of string-to-<code>T</code>'s where T must be a
         * subclass of <code>Number</code>; or null if the attribute doesn't
         * exist.
         *
         * @param attrName  the attribute name
         * @param valueType the specific number type of the value to be returned.
         *                  Currently, only<ul>
         *                  <li><code>Short</code></li>
         *                  <li><code>Integer</code></li>
         *                  <li><code>Long</code></li>
         *                  <li><code>Float</code></li>
         *                  <li><code>Double</code></li>
         *                  <li><code>Number</code></li>
         *                  <li><code>BigDecimal</code></li>
         *                  <li><code>BigInteger</code></li>
         *                  </ul>are supported.
         * @throws UnsupportedOperationException if the value type is not supported
         * @throws ClassCastException            if the attribute is not a map of string to numbers
         */
        public <T extends Number> Map<String, T> getMapOfNumbers(String attrName, Class<T> valueType) {
            if (item != null) {
                return item.getMapOfNumbers(attrName, valueType);
            } else if (document != null) {
                if (valueType == Short.class
                        || valueType == Integer.class
                        || valueType == Long.class
                        || valueType == Float.class
                        || valueType == Double.class
                        || valueType == BigDecimal.class
                        || valueType == BigInteger.class) {
                    Map<String, Object> map = document.get(attrName, Map.class);
                    Map<String, T> resultMap = new HashMap<>();
                    for (Map.Entry<String, Object> entry : map.entrySet()) {
                        if (valueType == Short.class) {
                            resultMap.put(entry.getKey(), (T) Short.valueOf(entry.getValue() + ""));
                        } else if (valueType == Integer.class) {
                            resultMap.put(entry.getKey(), (T) Integer.valueOf(entry.getValue() + ""));
                        } else if (valueType == Long.class) {
                            resultMap.put(entry.getKey(), (T) Long.valueOf(entry.getValue() + ""));
                        } else if (valueType == Float.class) {
                            resultMap.put(entry.getKey(), (T) Float.valueOf(entry.getValue() + ""));
                        } else if (valueType == Double.class) {
                            resultMap.put(entry.getKey(), (T) Double.valueOf(entry.getValue() + ""));
                        } else if (valueType == BigDecimal.class) {
                            resultMap.put(entry.getKey(), (T) new BigDecimal(entry.getValue() + ""));
                        } else if (valueType == BigInteger.class) {
                            resultMap.put(entry.getKey(), (T) new BigInteger(entry.getValue() + ""));
                        }
                    }
                    return resultMap;
                } else {
                    throw new UnsupportedOperationException("Value type " + valueType
                            + " is not currently supported");
                }
            }
            return new LinkedHashMap<>();
        }

        @SuppressWarnings("unchecked")
        public <T extends Number> Map<String, Map<String, T>> getMapOfMapNumbers(String attrName, Class<T> valueType) {
            if (item != null) {
                return item.getMap(attrName);
            } else if (document != null) {
                if (valueType == Short.class || valueType == Integer.class || valueType == Long.class
                        || valueType == Float.class || valueType == Double.class || valueType == BigDecimal.class
                        || valueType == BigInteger.class) {
                    Map<String, Map<String, T>> resultMap = new HashMap<>();
                    Map<String, Object> map = document.get(attrName, Map.class);
                    for (Map.Entry<String, Object> entry : map.entrySet()) {
                        Map<String, T> resultMapValue = new HashMap<>();
                        Map<String, Object> castMap = Map.class.cast(entry.getValue());
                        for (Entry<String, Object> subEntry : castMap.entrySet()) {
                            if (valueType == Short.class) {
                                resultMapValue.put(subEntry.getKey(), (T) Short.valueOf(subEntry.getValue() + ""));
                            } else if (valueType == Integer.class) {
                                resultMapValue.put(subEntry.getKey(), (T) Integer.valueOf(subEntry.getValue() + ""));
                            } else if (valueType == Long.class) {
                                resultMapValue.put(subEntry.getKey(), (T) Long.valueOf(subEntry.getValue() + ""));
                            } else if (valueType == Float.class) {
                                resultMapValue.put(subEntry.getKey(), (T) Float.valueOf(subEntry.getValue() + ""));
                            } else if (valueType == Double.class) {
                                resultMapValue.put(subEntry.getKey(), (T) Double.valueOf(subEntry.getValue() + ""));
                            } else if (valueType == BigDecimal.class) {
                                resultMapValue.put(subEntry.getKey(), (T) new BigDecimal(subEntry.getValue() + ""));
                            } else if (valueType == BigInteger.class) {
                                resultMapValue.put(subEntry.getKey(), (T) new BigInteger(subEntry.getValue() + ""));
                            }
                        }
                        resultMap.put(entry.getKey(), resultMapValue);
                    }
                    return resultMap;
                } else {
                    throw new UnsupportedOperationException("Value type " + valueType + " is not currently supported");
                }
            }
            return new LinkedHashMap<>();
        }

        public JSONObject getJson(String attrName) {
            if (item != null) {
                return JSONObject.parseObject(item.getJSON(attrName));
            } else if (document != null) {
//                return JSONObject.parseObject(document.getString(attrName));
                return new JSONObject(document.get(attrName, Map.class));
            }
            return null;
        }

        public String toJSONPretty() {
            if (item != null) {
                return item.toJSONPretty();
            } else if (document != null) {
                return document.toJson();
            }
            return null;
        }

        public Set<String> getStringSet(String attrName) {
            if (item != null) {
                return item.getStringSet(attrName);
            } else if (document != null) {
                return new HashSet<String>(document.getList(attrName, String.class, new ArrayList<String>()));
            }
            return new HashSet<String>();
        }

        public boolean hasAttribute(String attrName) {
            if (item != null) {
                return item.hasAttribute(attrName);
            } else if (document != null) {
                return document.containsKey(attrName);
            }
            return false;
        }

        public Result withString(String key, String val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withString(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withStringSet(String key, Set<String> val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withStringSet(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withMap(String key, Map<String, ?> val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withMap(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withInt(String key, int val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withInt(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withLong(String key, long val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withLong(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withDouble(String key, double val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withDouble(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withBoolean(String key, boolean val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withBoolean(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withBigDecimal(String key, BigDecimal val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withNumber(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withPrimaryKey(String key, Object val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withPrimaryKey(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append("_id", val);
            }
            return this;
        }

        public Result withTTL(long val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withLong("ttl", val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append("createDate", new Date());
            }
            return this;
        }

        public Result withJSON(String key, String json) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withJSON(key, json);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, Document.parse(json));
            }
            return this;
        }

        public Result withList(String key, List<?> val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withList(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result withBinary(String key, ByteBuffer val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withBinary(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                // convert ByteBuffer to bytes to keep Jackson happy
                byte[] bytes = BinaryUtils.copyBytesFrom(val);
                document.append(key, bytes);
            }
            return this;
        }

        public Result withBinary(String key, byte[] val) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.withBinary(key, val);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Result remove(String key) {
            if (Constant.isAwsEnv()) {
                item = item == null ? new Item() : item;
                item.removeAttribute(key);
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.remove(key);
            }
            return this;
        }

        public Object getResult() {
            if (Constant.isAwsEnv()) {
                return item;
            } else if (Constant.isQcloudEnv()) {
                return document;
            }

            return null;
        }

        public Map<String, Object> asMap(String primaryKeyName) {
            if (item != null) {
                return item.asMap();
            } else if (document != null) {
                Map<String, Object> map = new LinkedHashMap<String, Object>();
                map.putAll(document);
                map.put(primaryKeyName, document.get("_id"));
                map.remove("_id");
                return map;
            }
            return new LinkedHashMap<>();
        }

        public static Result fromMap(Map<String, Object> attributes, String primaryKeyName) {
            if (!ValidateUtil.isValid(attributes)) {
                LOG.warn("fromMap hits empty input param(s)");
                return null;
            }
            if (Constant.isAwsEnv()) {
                return new Result(Item.fromMap(attributes));
            } else if (Constant.isQcloudEnv()) {
                Result result = new Result(new Document(attributes));
                if (attributes.containsKey(primaryKeyName)) {
                    result.withPrimaryKey(primaryKeyName, attributes.get(primaryKeyName));
                    result.remove(primaryKeyName);
                }
                return result;
            }
            return null;
        }
    }

    public static class ResultIterator {
        private Iterator<Item> itemIterator;
        private MongoCursor<Document> documentCursor;

        public ResultIterator() {
        }

        public ResultIterator(Iterator<Item> itemIterator) {
            this.itemIterator = itemIterator;
        }

        public ResultIterator(MongoCursor<Document> documentCursor) {
            this.documentCursor = documentCursor;
        }

        public boolean hasNext() {
            if (documentCursor != null) {
                return documentCursor.hasNext();
            } else if (itemIterator != null) {
                return itemIterator.hasNext();
            }
            return false;
        }

        public Result next() {
            if (documentCursor != null) {
                return new Result(documentCursor.next());
            } else if (itemIterator != null) {
                return new Result(itemIterator.next());
            }
            return null;
        }
    }

    /* only process update operations*/
    public static class UpdateResult {
        private UpdateItemSpec updateItemSpec;
        private Document document;

        public UpdateResult() {
        }

        public UpdateResult clear() {
            if (Constant.isAwsEnv()) {
                updateItemSpec = updateItemSpec == null ? new UpdateItemSpec() : updateItemSpec;
                updateItemSpec.clearAttributeUpdate();
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.clear();
            }
            return this;
        }

        public UpdateResult withTTL(long val) {
            if (Constant.isAwsEnv()) {
                with("ttl", val);
            } else if (Constant.isQcloudEnv()) {
                with("createDate", new Date());
            }
            return this;
        }

        public UpdateResult withString(String key, String val) {
            with(key, val);
            return this;
        }

        public UpdateResult withMap(String key, Map<String, ?> val) {
            with(key, val);
            return this;
        }

        public UpdateResult withInt(String key, int val) {
            with(key, val);
            return this;
        }

        public UpdateResult withLong(String key, long val) {
            with(key, val);
            return this;
        }

        public UpdateResult withDouble(String key, double val) {
            with(key, val);
            return this;
        }

        public UpdateResult withBoolean(String key, boolean val) {
            with(key, val);
            return this;
        }

        public UpdateResult withBigDecimal(String key, BigDecimal val) {
            with(key, val);
            return this;
        }

        public UpdateResult withJSON(String key, JSONObject val) {
            with(key, val);
            return this;
        }

        public UpdateResult withList(String key, List<?> val) {
            with(key, val);
            return this;
        }

        public UpdateResult withBinary(String key, ByteBuffer val) {
            byte[] bytes = BinaryUtils.copyBytesFrom(val);
            with(key, bytes);
            return this;
        }

        public UpdateResult withBinary(String key, byte[] val) {
            with(key, val);
            return this;
        }

        public UpdateResult with(String key, Object val) {
            if (Constant.isAwsEnv()) {
                updateItemSpec = updateItemSpec == null ? new UpdateItemSpec() : updateItemSpec;
                updateItemSpec.addAttributeUpdate(new AttributeUpdate(key).put(val));
            } else if (Constant.isQcloudEnv()) {
                document = document == null ? new Document() : document;
                document.append(key, val);
            }
            return this;
        }

        public Object getResult() {
            if (Constant.isAwsEnv()) {
                return updateItemSpec;
            } else if (Constant.isQcloudEnv()) {
                return document;
            }

            return null;
        }
    }
}