package com.reyun.util;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

/* 以后上层应用请求对象存储时(例如不同云厂商的s3/cos等产品)请使用本类，勿直接调用特定的产品工具类，例如AwsS3Util */
public class ObjectStorageUtil {
    private static final Logger LOG = LoggerFactory.getLogger(ObjectStorageUtil.class);

    public static List<String> listObjects(String bucketName, String prefix) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(prefix)) {
            LOG.warn("listObjects hits empty param(s)");
            return new ArrayList<>();
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().getS3Keys(bucketName, prefix);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().listObjects(bucketName, prefix);
        }
        return null;
    }

    public static long putObjectInStream(String bucketName, String key, InputStream in, String contentType,
            long contentLength) throws IOException {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("putObjectInStream hits empty param(s)");
            return 0;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().uploadStreamToS3(bucketName, key, in, contentType, contentLength);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().putObjectInStream(bucketName, key, in, contentType, contentLength);
        }
        return 0;
    }

    public static long putObjectInString(String bucketName, String key, String str) throws IOException {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("putObjectInString hits empty param(s)");
            return 0;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().uploadStringToS3(bucketName, key, str);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().putObjectInString(bucketName, key, str);
        }
        return 0;
    }

    public static byte[] getObjectContentByteArray(String bucketName, String key) throws IOException {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("getObjectContentByteArray hits empty param(s)");
            return null;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().downloadBytesFromS3(bucketName, key);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().getObjectContentByteArray(bucketName, key);
        }
        return null;
    }

    public static InputStream getObjectContentStream(String bucketName, String key) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("getObjectContentStream hits empty param(s)");
            return null;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().downloadStreamFromS3(bucketName, key);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().getObjectContentStream(bucketName, key);
        }
        return null;
    }

    public static void deleteObject(String bucketName, String key) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("deleteObject hits empty param(s)");
            return;
        }
        if (Constant.isAwsEnv()) {
            AwsS3Util.getInstance().deleteS3Object(bucketName, key);
        } else if (Constant.isQcloudEnv()) {
            CosUtil.getInstance().deleteObject(bucketName, key);
        }
    }

    public static long getLength(String bucketName, String key) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(key)) {
            LOG.warn("getLength hits empty param(s)");
            return 0;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().length(bucketName, key);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().getLength(bucketName, key);
        }
        return 0;
    }

    public static boolean doesObjectExist(String bucketName, String objectName) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(objectName)) {
            LOG.warn("doesObjectExist hits empty param(s)");
            return false;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().doesObjectExist(bucketName, objectName);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().doesObjectExist(bucketName, objectName);
        }
        return false;
    }

    /* 注意目前默认直接针对get方法生成签名url，如果需要针对其它htt method使用本方法，请进一步改造 */
    public static URL generatePresignedUrl(String bucketName, String objectName, long expireMsFromCurrent) {
        if (StringUtils.isEmpty(bucketName) || StringUtils.isEmpty(objectName)) {
            LOG.warn("generatePresignedUrl hits empty param(s)");
            return null;
        }
        if (Constant.isAwsEnv()) {
            return AwsS3Util.getInstance().generatePresignedUrl(bucketName, objectName, expireMsFromCurrent);
        } else if (Constant.isQcloudEnv()) {
            return CosUtil.getInstance().generatePresignedUrl(bucketName, objectName, expireMsFromCurrent);
        }
        return null;
    }
}