package com.reyun.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

public class RedisUtilNew {
	protected static Logger logger = LoggerFactory.getLogger(RedisUtilNew.class);
	private static RedisUtilNew redisUtil = new RedisUtilNew();
	private JedisPool pool;

    private RedisUtilNew() {
        ResourceBundle resourceBundle = ResourceBundle.getBundle("redis");
        pool = new JedisPool(resourceBundle.getString("redis.surl.host"),
                Integer.valueOf(resourceBundle.getString("redis.surl.port")));
    }

	public static RedisUtilNew getInstance() {
		return redisUtil;
	}

	public Map<String, String> hgetAll(String key) {
		Map<String, String> result = new HashMap<String, String>();

		Jedis jedis = pool.getResource();
		result = jedis.hgetAll(key);
		pool.returnResource(jedis);

		return result;
	}

	public void hsetAll(String key, Map<String, String> map) {
		Jedis jedis = pool.getResource();
		jedis.hmset(key, map);
		pool.returnResource(jedis);
	}

	public void delete(String key) {
		Jedis jedis = pool.getResource();
		jedis.del(key);
		pool.returnResource(jedis);
	}

	public String get(String key) {
		Jedis jedis = pool.getResource();
		try {
			String result = jedis.get(key);
			if (result == null) {
				return null;
			}
			return result;
		} finally {
			pool.returnResource(jedis);
		}
	}

	public void set(String key,String value)
	{
		Jedis jedis = pool.getResource();
		jedis.set(key,value);
		pool.returnResource(jedis);
	}

	public Long getExpireTime(String key)
	{
		Jedis jedis = pool.getResource();
		Long m = jedis.ttl(key);
        pool.returnResource(jedis);
		return m;
	}


	/**
	 * 加锁.
	 *
	 * @param key
	 * @param expireTime
	 *            超时时间单位秒.
	 * @return 锁定的value,返回null值加锁失败,供释放锁时使用.
	 */
	public String tryLock(String key, int expireTime) {
		Jedis jedis = pool.getResource();
		try {
			String value = Long.toString(System.currentTimeMillis() + (expireTime * 1000));
			String result = jedis.set(key, value, "NX", "EX", expireTime);
			if (result == null) {
				return null;
			}
			return value;
		} finally {
			pool.returnResource(jedis);
		}
	}

	public String expireValue(String key, String value, int expireTime)
	{
		Jedis jedis = pool.getResource();
		try {
			String result = jedis.set(key, value, "NX", "EX", expireTime);
			if (result == null) {
				return null;
			}
			return value;
		} finally {
			pool.returnResource(jedis);
		}
	}

	public void releaseLock(String key) {
		Jedis jedis = pool.getResource();
		try {
			Set<String> set = jedis.keys(key);
			String[] kk = new String[set.size()];
			jedis.del(set.toArray(kk));
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			pool.returnResource(jedis);
		}
	}
	/**
	 * 通过key 对value进行加值+1操作,当key不存在是则value为1
	 *
	 * @param key
	 * @return
	 */
	public int intercept(String key,long expireTime) {
		Jedis jedis = pool.getResource();
        Integer value = 0;
		try {
            String s = get(key);
            s = s==null?"1":(Integer.valueOf(s)+1) + "";
            value = Integer.valueOf(s);
			if(value==1){
                jedis.set(key, "1", "NX", "EX", expireTime);
			}else{
                //value++;
                Long expireTime1 = getExpireTime(key);
                //System.out.println("expireTime1 = " + expireTime1);
                jedis.set(key, value.toString(),"XX", "EX", expireTime1);
			}

		}catch (Exception e){
		    e.printStackTrace();
        }finally {
			pool.returnResource(jedis);
		}
        return value.intValue();
	}
}
