(function() {
    var tempVersion = new Date().getTime();

    angular.module("app")
        .filter("charfilter", charfilter)
        .filter("filterList", filterList)
        .filter("filterPay", filterPay)
        .filter("numFixed", ["UtilService", numFixed])
        .filter("filterHtml", ["$sce", filterHtml])
        .filter("formatRate", ["UtilService", formatRate])
        .filter("dataRangeFilter", dataRangeFilter)
        .filter("tagsFilter", tagsFilter)
        .filter("camTrackUrlFilter", camTrackUrlFilter)
        .filter("qrCodeFilter", ["ArrayConstant", qrCodeFilter])
        .filter("checkedfilter", checkedfilter)
        .filter("campaignfilter", campaignfilter);

    //渠道首字母过滤
    function charfilter() {
        return function(input, param) {
            if (param == "") {
                return input;
            } else {
                var output = [];
                angular.forEach(input, function(v, k) {
                    if (param.indexOf(v.firstchar) > -1) {
                        output.push(v);
                    }
                });
                return output;
            }
        };
    }

    //列表多个属性过滤 
    function filterList() {
        return function(input, v, param) {
            if (!v || v == "") {
                return input;
            } else {
                var paramkeys = param.split(",");
                return input && input.filter(function(item) {
                    var has = false;
                    for (var i = paramkeys.length - 1; i >= 0; i--) {
                        if (item[paramkeys[i]] && angular.uppercase(String(item[paramkeys[i]])).indexOf(angular.uppercase(v)) > -1) {
                            has = true;
                            break;
                        }
                    }
                    return has;
                });
            }
        };
    }

    function filterPay() {
        return function(input) {
            if (input < 10000) {
                return input;
            } else {
                return (input / 10000).toFixed(1) + "w";
            }
        }
    }

    function filterHtml($sce) {
        return function(text) {
            if (text == null) {
                return "";
            } else {
                return $sce.trustAsHtml(text);
            }
        };
    }

    function numFixed(UtilService) {
        return function(num, params) {
            if (num == null || num == 'null') { return 0; }
            params = params ? params : 2;
            if (isNaN(Number(num))) {
                return num;
            } else {
                return UtilService.decimal2(Number(num), params);
            }
        }
    }

    function formatRate(UtilService) {
        return function(num, rate, formatjson, rate1) {
            if (num == null || num == 'null' || !rate || !formatjson) { return num; }
            if (isNaN(Number(num))) {
                if (num.indexOf("_") == 0) { num = num.substring(1); }
                return num;
            }
            if (rate1) { rate = rate1; }

            var format = formatjson[rate];
            if (!format) { format = 'float'; }

            if (format == 'percent') {
                return UtilService.decimal2(Number(num) * 100, 2) + "%";
            } else if (format == 'float') {
                return UtilService.decimal2(Number(num), 2);
            } else if (format == 'int') {
                return Math.round(Number(num));
            } else {
                return num;
            }
        }

    }
    //人群分析列表页面用户群转译
    function dataRangeFilter() {
        return function(val) {
            var valChi = '未知';
            if (val == 'all') {
                valChi = '历史全量';
            } else if (val == 'three') {
                valChi = '最近3个月'
            }
            return valChi;
        }

    }
    //人群分析列表页面标签
    function tagsFilter() {
        return function(info) {
            var retChi = '';
            if (info.hobbyLabel) {
                retChi += '兴趣';
            }
            if (info.hobbyLabel && info.payLabel) {
                retChi += '、';
            }
            if (info.payLabel) {
                retChi += '付费&活跃';
            }
            return retChi;
        }

    }
    //推广活动管理-分链接推广-常规渠道，广点通-新模式 短链隐藏
    function camTrackUrlFilter() {
        return function(info) {
            return ((info.pack_channelid == '' || info.pack_channelid == null) && info.surl != '' && info.surl != '-1') ? ((info.channel == 5305 && !info.surl) ? '-' : info.trackurl) : '-';
        }
    }

    function qrCodeFilter(ArrayConstant) {
        return function(type) {
            // var map = {"miniCode":"小程序码","miniQrCode":"小程序二维码","link":"链接"};
            return type ? ArrayConstant.wayPromObj[type] : '';
        }
    }

    //过滤数组中 当传值为2得情况下 某个字段是否为true
    function checkedfilter() {
        return function(input, num, name) {
            if (num == "1") {
                return input;
            } else {
                var output = [];
                angular.forEach(input, function(v, k) {
                    if (v[name] == true) {
                        output.push(v);
                    }
                });
                return output;
            }
        };
    }

    function campaignfilter() {
        return function(input, num, ischeck) {
            var list = input && input.filter(function(item){
                if(ischeck==false){
                    return item.ischeck==false||item.ischeck==undefined
                } else {
                    return item.ischeck==ischeck
                }
            })
            return list && list.filter(function(item,index) {
                return index<num
            });
        };
    }
})();